#ifndef N_SOUNDEMITTER_H
#define N_SOUNDEMITTER_H
//------------------------------------------------------------------------------
/**
    @class nSoundEmitter

    @brief attached a sound-producing speaker/emitter to the entity of your choice

    the emitter can then play passive sounds continously as the entity roams around
    it can also actively play particular sounds -> such as gun fire, explosions, or collision noise

    it works by maintain a handle to an emitter in the sound system
    it updates the position of the sound emitter in the sound world to match the posiiton of the entity this aspect is attached too
    it can be triggered to play sounds through the queue commands, or it can play sounds continously / passively

    (C)	2005	Chris Miles
*/
#ifndef N_OFFSETASPECT_H
#include "physics/noffsetaspect.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nSoundEmitter
#include "kernel/ndefdllclass.h"

//------------------------------------------------------------------------------
class nSoundEmitter : public nOffsetAspect
{
public:
    /// constructor
    nSoundEmitter();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nSoundEmitter();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    virtual void Queue( const char* soundName );

    virtual void AddPassiveSound( const char* soundName );
    virtual void SetRange( float range ) 		{ this->range = range; }
    virtual float GetRange()			{ return this->range; }
    virtual const nArray<nString>& GetPassiveSounds()	                                        { return passiveSounds; }
    virtual void SetGain( float f );
    virtual float GetGain()			{ return gain; }
    virtual void SetPitch( float f );
    virtual float GetPitch()			{ return pitch; }

    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    int emitterHandle;                                                                                                  ///reference to the actual emitter object
    bool playing;                                                                                                       ///am i playing something
    nArray<nString> passiveSounds;                                                                                      ///list of passive sounds to randomly cycle through
    float range;                                                                                                        ///how close the camera has to be to turn the sound on
    float gain;                                                                                                         ///how loud the sound is
    float pitch;                                                                                                        ///adjust the pitch (1.0) is default

    var camera;                                                                                                         ///needs to know where the camera is to determine how far it is
    var cameraPos;

    var vel;                                                                                                            ///uses speed / velocity for doppled effects
    var speed;
};
//------------------------------------------------------------------------------
#endif

