#ifndef N_ENT_H
#define N_ENT_H
//------------------------------------------------------------------------------
/**
    @class nEnt

    @brief Game Entity class

    Manages nEnvs attached to it.

    (C)	2004	Chris Miles
*/
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nEnt
#include "kernel/ndefdllclass.h"
#include "enkiduo/nvar.h"
#include "enkiduo/nvarstack.h"

#include "node/n3dnode.h"
#include "misc/noctree.h"
#include "enkiduo/nchannelholder.h"

class nEnt;
typedef nArray<nEnt*> EntArray;
class nPlannerUI;
class nOctreeElement : public nOctElement {
    public:
        nEnt* ent;
};
//------------------------------------------------------------------------------
class nEnt : public nThing
{
public:
    /// constructor
    nEnt();
    virtual void Initialize();
    /// destructor
    virtual ~nEnt();
    virtual bool Release();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    // funcs for var
    virtual 	nVar* 	CreateVar( const nString& varName );
    virtual 	nVar* 	FindVar(   const nString& varName );
    //funcs for myvar
    virtual 	nVar* 	CreateStackedVar( const nString& varName );
    virtual 	nVarStack* 	FindVarStackLocal( const nString& varName );
    //var access functions
    virtual 	void 	SetVarI	( const char* varName, int i );
    virtual 	int 	GetVarI	( const char* varName );
    virtual 	void 	SetVarF        	( const char* varName, float f );
    virtual 	void 	SetStackedVarF 	( const char* varName, float f );
    virtual 	float 	GetVarF	( const char* varName );
    virtual 	void 	SetVarV        	( const char* varName, const vector3& v );
    virtual 	void 	SetVarV        	( const char* varName, float x, float y, float z );
    virtual 	vector3 	GetVarV     	( const char* varName );
    virtual 	void 	SetVarQ        	( const char* varName, const quaternion& v );
    virtual 	void 	SetVarQ        	( const char* varName, float x, float y, float z, float w );
    virtual 	quaternion 	GetVarQ  	( const char* varName );
    virtual 	void 	SetVarS        	( const char* varName, const char* string );
    virtual 	const char* 	GetVarS	( const char* varName );
    virtual 	void 	SetVarRef      	( const char* varName, const char* pathname );
    virtual 	const char* 	GetVarRef	( const char* varName );
    virtual 	void 	SetStackedVarV 	( const char* varName, float x, float y, float z );
    virtual 	void 	SetStackedVarS 	( const char* varName, const char* s );
    virtual 	void 	SetStackedVarRef( const char* varName, const char* pathname );
    virtual 	void	DefaultStackedVarRef( const char* varName, const char* pathname );
    virtual 	bool	HasVar	( const char* varName ) { return vars->Find( varName ) != 0; }

    virtual 	void 	EditRC( const char* varName );
    virtual 	void	SaveRC( const char* varName, const char* filename );
    virtual 	void	LoadRC( const char* varName, const char* filename );
    virtual 	void 	InheritFrom( const nString& parent );
    virtual 	void 	Tick( float time, float dtime );
    virtual 	void 	PrintVars();
    virtual 	nArray<nString> GetParentPaths(){
        nArray<nString> parentPaths;
        for( int i= 0; i < varParents.Size(); i ++ )
            parentPaths.Append( varParents[i]->get()->FullNameString() );
        return parentPaths;
    }
    virtual 	void 	SetVis( const char* vis )	{ this->vis = vis;          }
    virtual 	n3DNode* 	GetVis()		{ if( vis.isvalid() ) return vis.get(); else return 0;}
    virtual 	matrix44	GetM()		{ if( vis.isvalid() ) return vis->GetM(); else return matrix44(); }
    virtual 	void 	SetNeedsVis( bool b )	{ this->needsVis = b;       }
    virtual 	bool 	GetNeedsVis()		{ return this->needsVis;    }
    virtual 	void 	SetInOctree( bool b )          	{ this->inOctree = b;       if( b == 0 ) this->RemoveFromOctree(); }
    virtual 	bool 	GetInOctree()                  	{ return this->inOctree;    }
    virtual 	void 	SetSelectable( bool b )	{ this->selectable = b;	}
    virtual 	bool 	GetSelectable()		{ return this->selectable; 	}
    virtual	void	SetHasSide( bool b )	{ this->hasSide = b; if ( b == false ) this->RemoveFromSide(); }
    virtual 	bool	GetHasSide()		{ return this->hasSide;	}

    virtual 	void	ExportChannel( const char* channelName );
    virtual	void	TryToExportChannels();

    //overrided to give aspects time to die
    virtual 	void 	Die();
    virtual 	void 	CheckDeath();
    static 	nKernelServer* kernelServer;

    virtual 	bool	Held()		{ return held; }
    virtual 	void	SetHeld( bool b ) 	{ this->held = b; }

    virtual	int	GetSideHandle();
    virtual 	void	Select();		
    virtual 	void	Deselect();	
    virtual 	bool	Selected()		{ return selected; }
protected:
    virtual 	nVar* 	FindVarLocal	( const nString& varName );
    virtual	void 	RemoveFromOctree();
    virtual                             void                                    RemoveFromSide();
    nDynAutoRef<n3DNode> 	vis; //ref to my visual node
    nRef<nRoot> 	vars;
    nArray<nDynAutoRef<nEnt>*> 	varParents;
    nVar* 		pos;
    nVar* 		rot;
    nVar*		side;
    int 		sideCounter;
    int		sideHandle;
    nVar* 		octradius;
    bool 		needsVis;
    bool 		selectable;
    bool		hasSide;
    nOctreeElement* 	octElement;
    nAutoRef<nOctree> 	octtree;
    bool 		dying;
    bool 		inOctree;
    bool 		renderContextInitialized;
    //nRef<nChannelHolder>	channelHolder;
    nArray<nString>	channelsToExport;
    nChannelHolder*	channelHolder;
    struct 	ExportedChannel {
        const char* channelName;
        nVar*	var;
        int 	handle;
    };
    nArray<ExportedChannel> exportedChannels;
    bool held;
    bool selected;
    nDynAutoRef<nPlannerUI> planner;
};
//------------------------------------------------------------------------------
#endif
