#ifndef N_EVENTSERVER_H
#define N_EVENTSERVER_H
//------------------------------------------------------------------------------
/**
    @class nEventServer

    @brief Manages events in a nice networked way.

    Anything should be able to pull an event, which is designated by a name and a single string parameter

    Aynthing can also register callback hooks to be pulled when designated events happen

    (C)	2005	Chris Miles
*/
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nEventServer
#include "kernel/ndefdllclass.h"
#include "python/npythonserver.h"
#include "enkiduo/event.h"

class nTNL;
//-----------------------------------------------------------------------------//
class nEventServer : public nThing {
public:
    /// constructor
    nEventServer();
    virtual void Initialize();
    /// destructor
    virtual ~nEventServer();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    ///manually add new events names - locks the server to throw assertions on adding events automatically though register/pull
    virtual void addEvent(const char* tag, const char* category);
    ///add a new function to the hook list
    virtual void registerPythonFunction(const char* tag, PyObject* function);

    ///trigger a particular event
    virtual void pull( void*& eventPointer, const char* tag, const char*message );

    ///for when you really have to do it the slow way
    virtual void pullInefficient( const char* tag, const char*message, bool remotePull = false );
    virtual void pullLater( const char* tag, const char*message );
    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    /// add a new event to the hash
    Event* newEvent( const char* tag, EventCategory category = EVENT_CATEGORY_LOCAL );
    EventCategory parseEventString( const char* string );
    ///list of Events
    nHashList hooks;
    bool autoCreate;
    nAutoRef<nTNL> tnl;

    struct pullLaterInfo {
        nString tag;
        nString msg;
    };
    nArray<pullLaterInfo> pullLaters;
};
//------------------------------------------------------------------------------
#endif
