#ifndef N_GAMESERVER_H
#define N_GAMESERVER_H
//------------------------------------------------------------------------------
/**
    @class nGameServer

    @brief Contains the main loop + methods for doing I/O

    triggers - displays and renders all other classes

    (C)    2004    Chris Miles
*/
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef N_KERNELSERVER_H
#include "kernel/nkernelserver.h"
#endif
#ifndef N_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nGameServer
#include "kernel/ndefdllclass.h"
#include "kernel/nkernelserver.h"
#include "kernel/ntimeserver.h"
#include "gfx/ngfxserver.h"
#include "input/ninputserver.h"
#include "script/ntclserver.h"
#include "misc/nmathserver.h"
#include "gfx/nscenegraph2.h"
#include "gfx/nchannelserver.h"
#include "misc/nconserver.h"
#include "misc/nparticleserver.h"
#include "node/n3dnode.h"
#include "misc/nspecialfxserver.h"
#include "shadow/nshadowserver.h"
#include "collide/ncollideserver.h"
#include "kernel/ndynautoref.h"
#include "misc/noctree.h"
#include "enkiduo/ntriggerserver.h"
#include "gfx/nprimitiveserver.h"
#include "enkiduo/nprofserver.h"
#include "python/npythonserver.h"

struct MouseInfo {
    int pressed[3];
    vector2 posPressed[3];
    vector2 posReleased[3];
    vector2 pos;
    vector2 corrections;
};
inline const char* str( const MouseInfo& mi ) {
    char* c = strGet();
    #ifdef __WIN32__
    _snprintf( c, STRMAX, "%s [%i,%i,%i]@[%s,%s,%s]->[%s,%s,%s]", str(mi.pos), mi.pressed[0], mi.pressed[1], mi.pressed[2], str(mi.posPressed[0]), str(mi.posPressed[1]), str(mi.posPressed[2]), str(mi.posReleased[0]), str(mi.posReleased[1]), str(mi.posReleased[2]));
    #else
    snprintf( c, STRMAX, "%s [%i,%i,%i]@[%s,%s,%s]->[%s,%s,%s]", str(mi.pos), mi.pressed[0], mi.pressed[1], mi.pressed[2], str(mi.posPressed[0]), str(mi.posPressed[1]), str(mi.posPressed[2]), str(mi.posReleased[0]), str(mi.posReleased[1]), str(mi.posReleased[2]));
    #endif 
    return c;
}
//-------------------------------------------------------------------
//------------------------------------------------------------------------------
class nEnt;
class nEventServer;
class nGameServer : public nThing {
public:
    /// constructor
    nGameServer();
    virtual void Initialize();
    /// destructor
    virtual ~nGameServer();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    /// pointer to nKernelServer
    static nKernelServer* kernelServer;

    virtual void Start();
    virtual void Stop();
    virtual bool MainLoop();
    virtual bool HandleDirectInput( nInputEvent* ie, float time );

    virtual void RenderGrid( nGfxServer* gs);
    virtual void RenderAxis( nGfxServer* gs);
    virtual void RenderScreenGrid( nGfxServer* gs);
    virtual void RenderCollideContext( nGfxServer* gs );

    nCollideShape* NewCollideShape( const char* name, const char* file );
    nCollideObject* NewCollideObject( nRoot* thing );
    void ReleaseCollideObject( nCollideObject* object );

    virtual void AddToDisplayList( nThing* t )                                  { displayList.Append( t ); }
    virtual void RemoveFromDisplayList( nThing* root )                          { displayList.Erase( displayList.Find(root) ); }
    virtual void AddToReleaseList( nThing* t )                                  { releaseList.Append( t ); }
    virtual void RemoveFromReleaseList( nThing* root )                          { releaseList.Erase( releaseList.Find(root) ); }
    
    //temp function, remove as soon as i figure the input server out
    virtual bool GetButton( const char* s );
    virtual MouseInfo GetMouseInfo();
    virtual void SetTickScript( const char* s )                                 { this->tickScript = s; }
    virtual void SetDrawGrid( bool b )                                          { drawGrid = b; }
    virtual void SetDrawAxis( bool b )                                          { drawAxis = b; }
    virtual void SetDrawScreenGrid( bool b )                                    { drawScreenGrid = b; }
    virtual bool GetDrawGrid()                                                  { return drawGrid; }
    virtual bool GetDrawAxis()                                                  { return drawAxis; }
    virtual bool GetDrawScreenGrid()                                            { return drawScreenGrid; }
    virtual void SetVisualizeCollision( bool b )                                { visualizeCollision = b; }
    virtual bool GetVisualizeCollision( )                                       { return visualizeCollision;}

    virtual nEnt* GenEnt                ( const char* Type, const char* path = 0, const char* nameWanted = 0 );
    virtual n3DNode* GenVis             ( const char* type );

    virtual void SetHoldMouse( bool b )                                         { this->holdMouse = b; }
    virtual void AddNonBusyScript( const char* s )                              { this->nonBusyScripts.Append(s); }
    virtual void SetSimple( bool b )                                            { this->simple = b; }
    virtual bool GetSimple()                                                    { return this->simple; }
    virtual void RunTCLScript( const char* fileName );
    virtual void triggerWXPython( float time, float dtime );

    virtual bool getProtectedRelease()                                          { return true; }

    virtual bool Exists( const char* path)                                      { return kernelServer->Lookup(path) != 0;                                       } 
    virtual void PushSel(const char* path)                                      { 
                                                                                    nRoot* node = kernelServer->Lookup(path);
                                                                                    if(node)
                                                                                        kernelServer->PushCwd(node);
                                                                                    else
                                                                                        n_printf("could not find path to pushsel:%s\n", path);
                                                                                }
    virtual const char* PopSel()                                                { 
                                                                                    nRoot* node = kernelServer->PopCwd();
                                                                                    return node?node->fn:"";
                                                                                }
protected:
    MouseInfo mouse;
    nAutoRef< nGfxServer >          gfx;
    nAutoRef< nInputServer >        input;
    nAutoRef< nTclServer >          script;
    nAutoRef< nPythonServer >       pythonServer;
    nAutoRef< nSceneGraph2 >        graph;
    nAutoRef< nShadowServer >       shadow;
    nAutoRef< nChannelServer >      channel;
    nAutoRef< nConServer >          console;
    nAutoRef< nMathServer >         math;
    nAutoRef< nParticleServer >     particle;
    nAutoRef< nSpecialFxServer >    fx;
    nAutoRef< nOctree >             octree;
    nAutoRef< nTriggerServer >      trigger;
    nAutoRef< nPrimitiveServer >    prim;
    nAutoRef< nCollideServer >      collide;
    nDynAutoRef< n3DNode >          camera;

    nAutoRef< nThing >              game;
    nAutoRef< nVisNode>             vis;
    nAutoRef< nThing >              ui;
    nAutoRef< nProfServer>          prof;
    nAutoRef< nEventServer>             ref_nEventServer;

    nCollideContext* collideContext;
    bool shouldQuit;
    bool drawGrid;
    bool drawAxis;
    bool drawScreenGrid;
    bool visualizeCollision;
    nArray<nThing*> displayList;
    nArray<nThing*> releaseList;
    struct KeyInfo {
        bool pressed;
        float timeDown;
        void* pressedEvent[8];
    };
    KeyInfo keys[256];
    const char* tickScript;

    nOctElement* inFrustum[1024];
    int numInFrustum;
    
    int loopCounter;
    bool holdMouse;

    float lastTime;
    nArray<nString> nonBusyScripts;
    int lastNonBusyCounter;
    bool simple;

    struct MouseCenter {
        vector2 pos;
        nArray<vector2> history;
        bool found;
    } mouseCenter;
};
//------------------------------------------------------------------------------
#endif
