#ifndef N_INCLUDES_H
#define N_INCLUDES_H
#include "kernel/nkernelserver.h"
#include "util/nstring.h"
#include "util/narray.h"
#include "kernel/ndynautoref.h"
#include "mathlib/vector.h"
#include "mathlib/quaternion.h"
#include "enkiduo/rc.h"
//#include "enkiduo/xml.h"
#include "kernel/nscriptserver.h"
#include "kernel/nref.h"
#include "kernel/nstacktrace.h"
#include "mathlib/line.h"
#include "mathlib/sphere.h"
#include "misc/nquickwatch.h"

#define fn FullNameString().Get()
#define classVar(className,varName) static nClass* varName = kernelServer->FindClass( tolower(#className) );
#define once(a) { static int ONCE_COUNTER = 0; if( ONCE_COUNTER == 0 ) { ONCE_COUNTER = 1; a; } }
//#define n_assertf(ASSERTION,ERROR,ARGS...) { if (!(ASSERTION)) { n_printf(ERROR,##ARGS); n_barf(#ASSERTION,__FILE__,__LINE__); } }
#define n_assertisa(classPtr,className) do { classVar( className, _clazz ); n_assert( classPtr->IsA( _clazz )); } while(0)

/*#ifdef __WIN32__
#include <stdarg.h>
inline void snprintf( char* string, const char* format, int n, ... ){
    va_list ap;
    va_start( ap, n );
    sprintfv( string, format, ap );
    va_end( ap );
}
#endif
*/
#define tok(STR, SEP) strtok_r( 0,SEP,&STR)
#define ifvalid(REF)			\
if( (REF).isvalid() == false ){ 		\
    static int isvalid_triggered = 0;		\
    if( isvalid_triggered == 0 ) {		\
        n_printf("%s::"#REF" not valid\n", fn);		\
        isvalid_triggered = 1;			\
    }				\
}				\
else
#define ifnonzero(ELEM)\
    if( (ELEM) == 0 ) \
        n_printf("%s %i    "#ELEM" = 0\n", __FILE__,__LINE__);\
    else
#define RUNSCRIPT(SCRIPT) {\
    RunScript(SCRIPT, kernelServer,0, __FILE__, __LINE__ );\
}
#define RUNSCRIPTD(SCRIPT) {\
    RunScript(SCRIPT, kernelServer,1, __FILE__, __LINE__);\
}
#define EVENT(EVENTTAGS)	\
    RUNSCRIPT("Event "#EVENTTAGS);
#define foreach(ITERATOR,ARRAY) 		for( int index##ARRAY = 0; index##ARRAY < ARRAY.Size() && ((ITERATOR=ARRAY[index##ARRAY]) || true ); index##ARRAY++ )
#define foreacht(TYPE,ITERATOR,ARRAY) TYPE ITERATOR; 	for( int index##ARRAY = 0; index##ARRAY < ARRAY.Size() && ((ITERATOR=ARRAY[index##ARRAY]) || true ); index##ARRAY++ )
#define foreachta(TYPE,ITERATOR,ARRAY) TYPE ITERATOR; 	for( int index##ARRAY = 0; index##ARRAY < ARRAY->Size() && ((ITERATOR=ARRAY->Get(index##ARRAY)) || true ); index##ARRAY++ )
#define foreachp(TYPE,ITERATOR,ARRAY) TYPE ITERATOR; 	for( int index##ARRAY = 0; index##ARRAY < ARRAY.Size() && ((ITERATOR=&ARRAY[index##ARRAY]) || true ); index##ARRAY++ )
//nRoot* threats = kernelServer->Lookup("/game/ents/threat");
//foreachchild( nEnt*, e, threats, nent ) {
#define foreachchild(ITER_TYPE, ITER, ROOT) \
    classVar( ITER_TYPE, FOREACHCHILD_CLASS ); \
    ITER_TYPE* ITER;\
    nRoot* FOREACHCHILD_NODE;\
    ifnonzero (ROOT) \
        FOREACHCHILD_NODE = ROOT->GetHead();\
    if (ROOT) \
        for( FOREACHCHILD_NODE = ROOT->GetHead(); (ITER = (ITER_TYPE*) FOREACHCHILD_NODE) != NULL; FOREACHCHILD_NODE = FOREACHCHILD_NODE->GetSucc() )\
            if( FOREACHCHILD_NODE->IsA(FOREACHCHILD_CLASS) == false ) {\
            } else \
	//n_printf("%s is not a "#TYPE"\n", FOREACHCHILD_NODE->GetName() );

#define profstart(PROFNAME) \
    if(prof) prof->Start(PROFNAME)
#define profstop(PROFNAME) \
    if(prof) prof->Stop(PROFNAME)
#define LOOKUP(type, path) dynamic_cast<type>(kernelServer->Lookup(path))

inline void RunScript( const char* script, nKernelServer* kernelServer, int print, const char* file, int line ){
    if( print ){
        char sect[1000];
        kernelServer->Print("RUNSCRIPT:\n");
        for( uint i = 0; i < strlen( script ); i += 1000 ){
            strncpy( sect, &script[i], 1000 );
            n_printf("%s", sect );
        }
        n_printf("\n");
    }
    const char* err= 0;
    ((nScriptServer*) kernelServer->Lookup("/sys/servers/script"))->Run( script, err );
    if( err && err[0] != '\0' ) 
        n_printf("%s %i RunScript error: |%s|\n", file, line, err );
}

inline bool contained( const vector2& point, const vector2 bounds[2] ){
    return point.x >= bounds[0].x && point.x <= bounds[1].x && point.y >= bounds[0].y && point.y <= bounds[1].y;
}
inline bool roll(float f){
    return rand() < RAND_MAX * f ;
}
inline bool roll(int n, int d){
    return rand() < RAND_MAX / d * n;
}
inline float randF( float min, float max ){
    return rand() / float(RAND_MAX) * (max-min) + min;
}
inline int randI( int min, int max ){
    return (int)((rand() / float(RAND_MAX)) * (max-min+1)) + min;
}

inline float normalF( float mean, float dev ) {
    static const double pii=3.1415927;
    static const double r_max=RAND_MAX+1;
    return (float) (dev*sqrt(-2*log((rand()+1)/r_max))*sin(2*pii*rand()/r_max)+mean);
}
template <class T>
inline T sqr(const T& v){
    return v*v;
}
template <class T>
inline void swap( T& l, T& r){
    T t = l;
    l = r;
    r = t;
}

inline char * makeMessage(const char *fmt, ...) {
    /* Guess we need no more than 100 bytes. */
    int n, size = 100;
    char *p;
    va_list ap;
    if ((p = (char*) malloc (size)) == NULL)
        return NULL;
    while (1) {
        /* Try to print in the allocated space. */
        va_start(ap, fmt);

		#ifdef __WIN32__
			n = _vsnprintf (p, size, fmt, ap);        
		#else
			n = vsnprintf (p, size, fmt, ap);
        #endif

        va_end(ap);
        /* If that worked, return the string. */
        if (n > -1 && n < size)
            return p;
        /* Else try again with more space. */
        if (n > -1)    /* glibc 2.1 */
            size = n+1; /* precisely what is needed */
        else           /* glibc 2.0 */
            size *= 2;  /* twice the old size */
        if ((p = (char*) realloc (p, size)) == NULL)
            return NULL;
    }
}

static char* makeL(){ 
    char* c = new char[64];
    c[63] = '\0';
    return c;
}

inline const char* firstLine( const char* s ) {
    static char* const l = makeL();
    strncpy( l, s, 63 );
    char* p = l;
	for( uint i = 0; i < strlen(p); i ++ )
		if( p[i] == '\n' )
			p[i] = '\0';
    //tok( p, "\n" );
    return l;
}
inline const char* tolower( const char* s ) {
	static char c[1024];
	strcpy( c, s );
	for( unsigned int i = 0; i < strlen(s); i ++ )
		c[i] = tolower(s[i]);
	return c;
}
#endif
