#ifndef N_PATHFINDER_H
#define N_PATHFINDER_H
//------------------------------------------------------------------------------
/**
  @class nPathfinder

  @brief a brief description of the class

  a detailed description of the class

  (C) 2004 Chris Miles
 */
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nPathfinder
#include "kernel/ndefdllclass.h"
#include "util/nheap.h"
#include "mathlib/sphere.h"

//------------------------------------------------------------------------------
class nPathfinder : public nThing
{
    enum {
        MAX_NUM_NODES = 2048,
    };
    public:
    /// constructor
    nPathfinder();
    virtual void Initialize();
    /// destructor
    virtual ~nPathfinder();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void  CalculateCoords(); //go through the threats and add coords around the ranges 
    virtual void  Display( nGfxServer* gfx );
    virtual float EstimateCost( int startIndex, int stopIndex );
    virtual float TraverseCost( int startIndex, int stopIndex );
    virtual void  FindPath( vector3 startPoint, vector3 stopPoint );
    virtual void  Message( nMessage* m );

    virtual void Clear()                { path.Clear(); }
    virtual int  NumWP()                { return path.Size(); }
    virtual vector3  GetWP( int index ) { return path[index]; }
    virtual void SetRC( float f )       { this->rc = f; this->CalculateCoords();}
    virtual float GetRC()               { return rc; }
    virtual void SetDrawCoords( bool b) { this->drawCoords = b;  }
    virtual bool GetDrawCoords()        { return this->drawCoords;  }
    virtual void SetDrawEdges( bool b ) { this->drawEdges = b;  }
    virtual bool GetDrawEdges()         { return this->drawEdges;  }
    virtual void SetDrawPath( bool b )  { this->drawPath = b;  }
    virtual bool GetDrawPath()          { return this->drawPath;  }
    virtual void SetDrawSpheres( bool b ) { this->drawSpheres = b;  }
    virtual bool GetDrawSpheres()       { return this->drawSpheres;  }

    virtual void SetEdgeLengthThreshold( float f )                              { this->edgeLengthThreshold = f;                                                }
    virtual float GetEdgeLengthThreshold()                                      { return this->edgeLengthThreshold;                                             }
    virtual void SetEdgeCostThreshold( float f )                                { this->edgeCostThreshold = f;                                                  }
    virtual float GetEdgeCostThreshold()                                        { return this->edgeCostThreshold;                                               }
    virtual void SetUseDetectionRange( bool b )                                 { this->useDetectionRange = b;  this->CalculateCoords();                        }
    virtual bool GetUseDetectionRange()                                         { return this->useDetectionRange;                                               }
    virtual void SetUseEffectiveRange( bool b )                                 { this->useEffectiveRange = b;  this->CalculateCoords();                        }
    virtual bool GetUseEffectiveRange()                                         { return this->useEffectiveRange;                                               }
    virtual void SetCoordDensity( float f )                                     { this->coordDensity = f;  this->CalculateCoords();                             }
    virtual float GetCoordDensity()                                             { return this->coordDensity;                                                    }

    virtual void Sum(){
        for( int i = 0; i < path.Size(); i ++ )
            n_printf("(%f,%f,%f)\n", path[i].x, path[i].y, path[i].z);
    }
    virtual void CacheAll(){
        for( int i = 0; i < coords.Size(); i ++ )
            for( int j= i+1; j < coords.Size(); j ++ )
                if( traverseCosts[i][j] <= 0.0f ) 
                    TraverseCost( i, j );
    }
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    protected:
    virtual int AddCoord( vector3 coord );
    nArray<vector3> coords;
    nArray<sphere> spheres;
    nHeap<int> open;
    float bestCostTo[MAX_NUM_NODES];
    int   parents [MAX_NUM_NODES];
    float* traverseCosts[MAX_NUM_NODES];
    float rc;
    int nodesUsed;
    float minCost, maxCost;

    float coordDensity;
    float dangerCost;
    int edgesCached;

    nArray<vector3> path;
    nArray<nArray<vector3> > paths;

    bool drawCoords;
    bool drawEdges;
    bool drawPath;
    bool drawSpheres;
    bool useDetectionRange;
    bool useEffectiveRange;
    float edgeLengthThreshold;
    float edgeCostThreshold;

    float worldSize;
};
//------------------------------------------------------------------------------
#endif

