#ifndef __WIN32__
#ifndef N_SOCKET_H
#define N_SOCKET_H
//------------------------------------------------------------------------------
/**
    @class nSocket

    @brief Socket wrapper 

    allows communications to other processes running a similar nSocket or a JNebulaSocket
    uses nonblocking calls - no threads

    a message consists of a tag and a string
    based on the tag different functions can be setup to receieve the message via the callback methods - these work well through nebula's scripting system

    connections are checked frequently, and those that die trigger connectionDied callbacks

    (C)	2004	Chris Miles
*/
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>

#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nSocket
#include "kernel/ndefdllclass.h"

//#include "enkiduo/practicalsocket.h"
//------------------------------------------------------------------------------
class nSocket : public nThing
{
public:
    /// constructor
    nSocket();
    virtual void Initialize();
    /// destructor
    virtual ~nSocket();

    virtual void SetPort( int i );
    virtual int  GetPort()				{ return this->port; }
    //open-close connections
    virtual void OpenServerSocket();			//open a socket to receive connections
    virtual void CheckConnections();			//see if anyone wants to connect to me
    virtual bool Connect( const char* hostname, const char* hostType = 0);	//connect to someone else
    virtual void Disconnect();				//close a connection
    virtual void KillConnection( int i );			//close a specified connection

    virtual void Send( const char* tag, const char* message =0 );		//send a message
    virtual bool Send( int too, const char* tag, const char* message =0 );		//send a message
    virtual void CheckMessages();			//see if i have any messages

    virtual void Receive( int	fromIndex, const char* tag, const char* message );	//I got a message,  find my callback
    virtual void AddCallback( const char* tag, const char* nodeName, const char* funcName, bool from = false );//add a callback
    virtual void AddConnectionDiedCallback( const char* nodeName, const char* funcName );//add a callback
    
    virtual void Tick( float time, float dtime  );
    virtual void PingSockets( float time );

    virtual void SetName( const char* name ) 			{ this->myName = name; }
    virtual void ReceiveIAM( 	int from, const char* message );
    virtual void ReceivePuts( 	const char* message );
    virtual void ReceiveScript( const char* message );
    virtual void ReceivePing( 	const char* message );

    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;

    virtual const char* HandleToIP( int handle ) { return sockets[handle].ip.Get(); }
protected:
    float time;

    int port;			//the port over which we communicate
    int portRange;		//the range of ports to check on server side
    int serverSocketHandle;		//server - socket on which connections are accepted
    nString myName;
    struct socketInfo {
        int handle;
        nString ip;
        nString name;
        float lastUsed;		//when this socket was last checked
    };
    float pingFreq;		//max time sockets can go without being checked
    nArray<socketInfo> sockets;		//socket handles on which I am actively communicating
    //int socketHandle;		//client - socket handle
    sockaddr_in	myAddress;
    sockaddr_in otherAddress;
    socklen_t myAddressLength;
    socklen_t otherAddressLength;
    hostent *hostInfo;		//client - info about who is connecting to us

    struct sockaddr_in localAddress;	//client - info about my location
    struct sockaddr_in serverAddress;	//client - info about the server's location

    struct Callback {
        bool from;
        bool script;
        char* tag;
        nString func;
        nRoot* node;
        nCmdProtoNative *cmd_proto;
        Callback():
            script(0),
            tag(0),
            node(0),
            cmd_proto(0),
            from(false)
        {}
    };
    nArray<Callback> callbacks;
    nArray<Callback> connectionDiedCallbacks;
    enum {
        TAG_SIZE = 16,
        MLS_SIZE = 8,
    };

    /*****************************************************************************************************************************/
    //TCPServerSocket* 	server;	//if i want to accept connections
    //TCPSocket* 		socket;	//for actual communications

    /*
    enum {
        TAG_LENGTH = 16,
    };
    struct Message {
        int index;
        char tag[16];
        nString text;
    };
    nArray<Message> messages;
    */
};
//------------------------------------------------------------------------------
#endif
#endif

