#ifndef RC_H
#define RC_H
//------------------------------------------------------------------------------
/**
    @class response curve

    @brief interpolates an input based on some buckets

    (C)	2004	Chris Miles
*/
//------------------------------------------------------------------------------
class rc
{
public:
    rc():
        bucketSize(-1),
        bucketCount(-1),
        minValue(-1.0f),
        bucketEdges(0),
        inMin(0.0f),
        inMax(0.0f),
        outMin(0.0f),
        outMax(1.0f),
        lastChecked(0.0f)
    {}
    float Response( float f ) {
        lastChecked = f;
        f = n_clamp( f, inMin, inMax );
        float index = (f - inMin)/bucketSize;
        int i = (int)index;
        float answer;
        if( i >= bucketCount-1 )
            answer = bucketEdges[bucketCount-1];
        else{
             float d = index - i;
             answer = bucketEdges[i] *(1-d) + bucketEdges[i+1]*d;
        }
        //n_assert( answer <= outMax && answer >= outMin );
        lastResponse = answer;
        return answer;
    }
    void SetInputRange( float min, float max, int samples){
        inMin = min;
        inMax = max;
        bucketCount = samples;
        bucketSize = (max-min) / (float)(samples-1);
        if( bucketEdges )
            delete[] bucketEdges;
        if ( bucketCount > 0 )
            bucketEdges = new float[bucketCount];
        else
            bucketEdges = 0;
    }
    void GetInputRange( float& min, float& max, int& samples ){
        min = inMin;
        max = inMax;
        samples = bucketCount;
    }
    void SetOutputRange( float outMin, float outMax ){
        this->outMin = outMin;
        this->outMax = outMax;
    }
    void GetOutputRange( float& min, float& max ){
        min = outMin;
        max = outMax;
    }
    void SetBucketEdge( int index, float val ){
        n_assert( val <= outMax && val >= outMin );
        n_assert(bucketEdges != 0 && "failed to initialized rc befure rc::SetVal\n");
        if( index >= bucketCount || index < 0 )
            n_error("SetVal(%i,%f) failed index is out of range(0,%i)\n", index,val, bucketCount );
        bucketEdges[index] = val;
    }
    int GetBucketCount(){ return bucketCount; }
    float& GetBucketEdge( int i ){ return bucketEdges[i]; }
    float GetInputMin(){ return inMin; }
    float GetInputMax(){ return inMax; }
    float GetOutputMin(){ return outMin; }
    float GetOutputMax(){ return outMax; }
    void Load( const char* filename ) {
        //printf("Load(%s)\n", filename );
        FILE* file = fopen( filename, "r" );
        if( file == 0 ){
            fprintf( stderr, "Could not load RC file: %s\n", filename );
            return;
        }
        int retval;
        retval = fscanf( file, "%f %f", &inMin, &inMax );			n_assert( retval == 2 );
        retval = fscanf( file, "%f %f", &outMin, &outMax );			n_assert( retval == 2 );
        retval = fscanf( file, "%f %i %f", &bucketSize, &bucketCount, &minValue );	n_assert( retval == 3 );
        this->SetInputRange( inMin, inMax, bucketCount );
        for( int i = 0; i < bucketCount; i ++ )
            retval = fscanf( file, "%f", &bucketEdges[i] );			n_assert( retval == 1 );
        fclose( file );
    }
    void Save( const char* filename ) {
        FILE* file = fopen( filename, "w" );
        if( file == 0 ){
            fprintf( stderr, "Could not save RC file: %s\n", filename );
            return;
        }
        fprintf( file, "%f %f\n", inMin, inMax );
        fprintf( file, "%f %f\n", outMin, outMax );	
        fprintf( file, "%f %i %f\n", bucketSize, bucketCount, minValue );	
        for( int i = 0; i < bucketCount; i ++ )
            fprintf( file, "%f ", bucketEdges[i] );			
        fclose( file );
    }
    float GetLastChecked(){ return lastChecked; }
    float GetLastResponse(){ return lastResponse; }
protected:
    float   bucketSize;
    int     bucketCount;
    float   minValue;
    float*  bucketEdges;

    float inMin;
    float inMax;
    float outMin;
    float outMax;
    
    float lastChecked;
    float lastResponse;
};
#endif
