#ifndef N_GA_H
#define N_GA_H
//------------------------------------------------------------------------------
/**
    @class nGA

    @brief a brief description of the class

    a detailed description of the class

    (C)	2005	Chris Miles
*/
#ifndef N_FSM_H
#include "enkiduo/nfsm.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nGA
#include "kernel/ndefdllclass.h"

//------------------------------------------------------------------------------
class nGenome;
class nEventServer;
class GAConnection;
class nGARecorder;
class nReleaseServer;
class nGA : public nFSM {
public:
    enum States {
        FRESH,                          //first created go here

        MASTER_WAITING_FOR_RUN,
        MASTER_NO_SLAVES,               //open server -> goes here
        MASTER,                         //master sits here while its running
        MASTER_START_EVALUATION,
        MASTER_CREATE_INDIVIDUAL,

        SLAVE_WAITING_FOR_MASTER,
        SLAVE_WAITING_FOR_WORLD,
        SLAVE_WAITING_FOR_INDIVIDUAL,
        SLAVE_EVALUATING,
    };
public:
    /// constructor
    nGA();
    virtual void Initialize();
    virtual void InitVars();
    virtual void InitializeWorld();

    /// destructor
    virtual ~nGA();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    virtual void Run();

    virtual void Tick( float time, float dtime );
    virtual void OpenServer( const char* ip );                                                  //setup this ga as a gamaster
    virtual void ConnectToServer( const char* ip );                                             //connect to a remote GA Master
    virtual void ConnectToLocalServer( const char* ip, GAConnection* server );                                        //connect to a local  GA Master
    virtual void AddSlaveConnection( GAConnection* slaveConnection );

    virtual void InitializeLocalSlave();
    virtual void StartEvaluation( nArray<bool> bitString );
    virtual void ClearPopulation();
    virtual bool RecordData()           { return true; }

    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    virtual void CreateGenome( nGenome* g );
    virtual void StartEvaluation( nGenome* g );
    virtual bool EvaluationFinished();
    virtual float GetFitness();
    virtual void StopEvaluation( nGenome* g );
    virtual void OnRunDone();
protected:
    virtual bool States( StateMachineEvent event, uint state, float time, float dtime );
    virtual void NormalizeFitnesses();
    virtual void Selection();
    virtual void CrossOver();
    virtual void Mutation();
    virtual void CheckPopSize();
    virtual float CalculateSimilarity( nGenome* genome );
    virtual void RecursiveSimilarityWalk( nArray<float>* similarities, nRoot* a, nRoot* b );

    virtual void InitializeSlave( nGA* slave );
    virtual void SendInitializeWorldMessage( const char* s );
    virtual void OnReceiveInitializeWorldMessage( const char* s );
    nArray<nGenome*> GetSortedGenomes();
    nGenome* SelectGenome();            
    nGenome* SelectBadGenome();            
    var popsize;
    var numgens;
    var generation;
    var minFitness;
    var maxFitness;
    var avgFitness;
    var maxEvaluations;                 //maximum number of evaluations to FINISH before stopping
    var evaluationsDone;                //the number of evaluations I have finished
    var numberToKeepUnevaluated;        //how many to keep unevaluated -> just because
    var gamestate;                      //where the xml gamestate is dumped -> to send to slaves
    nAutoRef<nEventServer> ref_event;
    float time;
    bool beginRunning;

    nString slaveClassName;
    void PingForSlaves( float time );
    var ip;
    var port;
protected:
    virtual int defaultPort()            { return 25732; }
    nGenome* genomeEvaluating;
    nGenome* slaveGenome;
    GAConnection*                       connection;
    nArray<GAConnection*>               slaveConnections;
    float lastPingTime;
    float pingFreq;
    bool localSlaveInitialized;
    nAutoRef<nReleaseServer>            ref_release;
    int maximumSimilarChildrenToCreate;
    int similarChildrenCreated;
};
//------------------------------------------------------------------------------
#endif

