#ifndef N_GAPROBLEM_H
#define N_GAPROBLEM_H
//------------------------------------------------------------------------------
/**
    @class nGAProblem

    @brief interface class - children represent different problems the GA can search

    a detailed description of the class

    (C)	2004	Chris Miles
*/
#ifndef N_ASPECT_H
#include "enkiduo/naspect.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nGAProblem
#include "kernel/ndefdllclass.h"
#include "ga/ngenome.h"
#include "ga/bittree.h"

class nGA;
//------------------------------------------------------------------------------
class nGAProblem : public nAspect
{
public:
    /// constructor
    nGAProblem();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nGAProblem();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;

    virtual nGenome* BuildEncoding();   		//constructs the encoding
    virtual nString GenomeToPlan( nGenome* genome );    	//converts a genome into an XML plan of attack
    virtual void StartEvaluation( nGenome* genome );        	//starts the evaluation of the specified individual
    virtual void StopEvaluation();
    virtual bool IsEvaluationDone();  		//checks to see if the evaluation is finished every tick
    virtual float ParseFitness();                          	//called when a game is over to parse out the fitness
    virtual void SetEvaluationDone( bool b ) 		{ this->evaluationDone = b; }
    virtual bool GetEvaluationDone() 		{ return evaluationDone; }
    virtual void DoneRunning();			//hook pulled when the ga is finished - just before the script GA::doneRunning
    virtual void GenerationDone();

protected:
    nDynAutoRef<nGA> ga;
    nAutoRef<nTimeServer> 	time;     //contains time for profiling
    bool evaluationDone;
};
//------------------------------------------------------------------------------
#endif

