#ifndef N_IMENT_H
#define N_IMENT_H
//------------------------------------------------------------------------------
/**
    @class nIMEnt

    @brief a brief description of the class

    a detailed description of the class

    (C)	2005	Chris Miles
*/
#ifndef N_IM_H
#include "im/nim.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nIMEnt
#include "kernel/ndefdllclass.h"

//-----------------------------------------------------------------------------//
enum FUNCS {
    VALUE_OVER_THREAT,
    POWER,
};
static const char* funcStrings[] = {
    "VALUE_OVER_THREAT",
    "POWER",
};
static const int numFuncStrings = 2;
//-----------------------------------------------------------------------------//
enum IPOLS {
    FLAT,
    LINEAR,
    INV_LINEAR, 
};
static const char* ipolStrings[] = {
    "FLAT",
    "LINEAR",
    "INV_LINEAR",
};
static const int numIpolStrings = 3;
//-----------------------------------------------------------------------------//
enum RADII {
    AREA,
    WEAPON,
    COLLISION_AVOIDANCE_RADIUS,
    //MY_WEAPON,
};
static const char* radiiStrings[] = {
    "AREA",
    "WEAPON",
    "COLLISION_AVOIDANCE_RADIUS",
    //"MY_WEAPON",
};
static const int numRadiiStrings = 3;
//-----------------------------------------------------------------------------//
enum WHO {
    FRIENDS,
    ENEMIES,
    NEUTRALS,
};
static const char* whoStrings[] = {
    "FRIENDS",
    "ENEMIES",
    "NEUTRALS",
};
static const int numWhoStrings = 3;
//------------------------------------------------------------------------------
class nIMEnt : public nIM {
protected:
    struct EntInfo {
        nEnt* ent;
        vector3 pos;
        vector3 coord;
        float value;
        float radius;
        float cradius;
        vector3 dimension;
        quaternion rot;
    };
public:
    /// constructor
    nIMEnt();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nIMEnt();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    virtual void SetFunc( const char* s ) { 
        for ( int i = 0; i < numFuncStrings; i ++)
            if (strcmp(funcStrings[i], s) == 0 ) 
                this->func = (FUNCS) i; 
        //n_printf("nIMEnt(%s).func = %i,%s\n", this->fn, this->func, s );
        ref_eventServer->pullLater( "updateIMs", str(this));
    }
    virtual void SetIpol( const char* s ) { 
        for ( int i = 0; i < numIpolStrings; i ++)
            if (strcmp(ipolStrings[i], s) == 0 ) 
                this->ipol = (IPOLS) i; 
        //n_printf("nIMEnt(%s).ipol = %i,%s\n", this->fn, this->ipol, s );
        ref_eventServer->pullLater( "updateIMs", str(this));
    }
    virtual void SetRadius( const char* s ) { 
        for ( int i = 0; i < numRadiiStrings; i ++)
            if (strcmp(radiiStrings[i], s) == 0 ) 
                this->radius = (RADII) i; 
        //n_printf("nIMEnt(%s).radius = %i,%s\n", this->fn, this->radius, s );
        ref_eventServer->pullLater( "updateIMs", str(this));
    }
    virtual void SetWho( const char* s ) { 
        for ( int i = 0; i < numWhoStrings; i ++)
            if (strcmp(whoStrings[i], s) == 0 ) 
                this->who = (WHO) i; 
        //n_printf("nIMEnt(%s).who = %i,%s\n", this->fn, this->who, s );
        ref_eventServer->pullLater( "updateIMs", str(this));
    }

    void SetFunc( int i )               { this->func   = (FUNCS) i;             ref_eventServer->pullLater( "updateIMs", str(this)); }
    void SetIpol( int i )               { this->ipol   = (IPOLS) i;             ref_eventServer->pullLater( "updateIMs", str(this)); }
    void SetRadius( int i )             { this->radius = (RADII) i;             ref_eventServer->pullLater( "updateIMs", str(this)); }
    void SetWho( int i )                { this->who    = (WHO)   i;             ref_eventServer->pullLater( "updateIMs", str(this)); }
    const char* GetFunc()               { return funcStrings[this->func];       }
    const char* GetIpol()               { return ipolStrings[this->ipol];       }
    const char* GetRadius()             { return radiiStrings[this->radius];    }
    const char* GetWho()                { return whoStrings[this->who];         }
    void SetRadiusCoef( float f )       { this->radiusCoef = f;                 ref_eventServer->pullLater( "updateIMs", str(this)); }
    float GetRadiusCoef()               { return this->radiusCoef;              }
    virtual void calculateValues(void** const v);
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    EntArray GetEnts ( WHO who );
    nArray<nIMEnt::EntInfo> GetEntInfo( FUNCS func, WHO who, RADII radius);
    FUNCS func;
    IPOLS ipol;
    RADII radius;
    WHO who;
    float radiusCoef;
};
//------------------------------------------------------------------------------
#endif

