#ifndef N_BOATHELMSMAN_H
#define N_BOATHELMSMAN_H
//------------------------------------------------------------------------------
/**
    @class nBoatHelmsman

    @brief translates desired heading/speeds into throttle/rudder settings

    fulfill the roll of unboard helsman,  taking desred speeds / headings from the captain and settings actual effector values
    for now just use a simple heuristics
    (C)	2005	Chris Miles
*/
#ifndef N_AI_H
#include "ai/nai.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nBoatHelmsman
#include "kernel/ndefdllclass.h"

//------------------------------------------------------------------------------
class nBoatHelmsman : public nAI {
    enum States {
        Halt,
        Forward,
        Steering,
        ReverseGlideSteering,
        ReverseTurn,
        ReverseGlideTurn,
    };
public:
    /// constructor
    nBoatHelmsman();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nBoatHelmsman();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    virtual bool Select();
    virtual bool DeSelect();

    bool States( StateMachineEvent event, uint state, float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    var speed;
    var heading;
    var desiredHeading;
    var desiredSpeed;
    var throttle;
    var rudder;
    var reverseThreshold;
    var maxSpeed;
    var inertia;

    var bbmin;
    var bbmax;
    var flcoord;
    var frcoord;
    var rlcoord;
    var rrcoord;
    var cameraDist;
    var camera;
    var cameraPos;
    var dimension;
    var side;
    var sideHandle;
};
//------------------------------------------------------------------------------
#endif

