#ifndef N_CARPHYSICS_H
#define N_CARPHYSICS_H
//------------------------------------------------------------------------------
/**
    @class nCarPhysics

    @brief a brief description of the class

    okay this holds the physics for a car
    -> 4 wheels
    -> 4 shocks
    -> square body

    (C)	2004	Chris Miles
*/
#include "ode/ode.h"
#ifndef N_ODEPHYSICS_H
#include "physics/nodephysics.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nCarPhysics
#include "kernel/ndefdllclass.h"
#include "map/nmap.h"

//------------------------------------------------------------------------------
/**
   okay so lets make the wheel stuff work first
*/
class nCarPhysics : public nODEPhysics
{
public:
    /// constructor
    nCarPhysics();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nCarPhysics();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    virtual void Halt();
protected:
    void UpdateFrame();
    void UpdateWheels();
    void UpdateSuspension();

    ///state 
    var wheelDirection;
    ///controls
    var steering;
    var throttle;
    ///frame dimensions
    var width;
    var height;
    var depth;
    /// engine parameters
    var enginePower;
    var engineResistance;
    var maxSpeed;
    /// brake parameters
    var brakePower;
    /// turning parameters
    var turningPower;
    var turningSpeed;
    /// wheel parameters
    var wheelRadiusF;
    var wheelWidthF;
    var wheelDensityF;

    var wheelRadiusR;
    var wheelWidthR;
    var wheelDensityR;
    /// suspension parameters
    var suspERP;
    var suspCFM;
    /// drive train
    enum DriveTrans {
        FRONTWHEEL,
        REARWHEEL,
        ALLWHEEL
    };
    // 0  = frontwheel
    // 1 = rearwheel
    // 2 = allwheel
    var driveTrain;

    struct Wheel {
        vector3 pos;
        quaternion rot;
        float width;
        float radius;
        float segments;
        nRef<n3DNode> ref;
        Wheel():
            width( 0.75 ),
            radius( 3 ),
            segments( 15 ) { }
        dBodyID body;
        dJointID suspension;
    };
    nArray<Wheel> wheels;
    nAutoRef<nMap> refMap;
    nAutoRef<nGameServer> gameServer;

    dBodyID 	frame;
    dGeomID	frameGeom;
};
//------------------------------------------------------------------------------
#endif

