#ifndef N_LEGIONASPECT_H
#define N_LEGIONASPECT_H
//------------------------------------------------------------------------------
/**
    @class nLegionAspect

    @brief maintains a group of units
    provides formation information / orders / targetting / upgrade information for a group of units

    (C)	2005	Chris Miles
*/
#ifndef N_ASPECT_H
#include "ai/ncontroller.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nLegionAspect
#include "kernel/ndefdllclass.h"
#include "ai/ncontroller.h"
class nPhysics;
struct UnitInfo {
    nEnt* ent;
    var pos;
    var formationPos;
    var formationRot;
    var formationPoint;
    var formationSize;
    var formationHandle;
};
//------------------------------------------------------------------------------
class nLegionAspect : public nController
{
    enum AllocationTactic {
        NTO1,
        CLOSEST,
    };

public:
    /// constructor
    nLegionAspect();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nLegionAspect();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void AddUnit( nEnt* e );
    virtual void RemoveUnit( nEnt* e );
    virtual void Tick( float time, float dtime );
    virtual void CalculateFormation();
    virtual void AssignPositions();
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    void Message( nMessage* message );
    nArray<UnitInfo>* GetUnits()		{ return &units; 	}
    nArray<vector3>* GetFormationPoints()		{ return &formationPoints; 	}
    virtual bool BlockDeath( bool )		{ DetachFromLegion(); return false; }
    virtual void BuildMyUnitsList( EntArray* myUnits );
    virtual void Upgrade( const char* varName, int level );
protected:
    virtual void AllocateAttackers( EntArray* myUnits, EntArray* enemyUnits, nMessage* message );
    virtual void SetFormationPoints();
    virtual void ApplyAlignmentForce();
    void CleanUnits();
    void DetachFromLegion();
    nArray<UnitInfo> units;
    //EntArray units;
    var pos;
    var rot;
    var formationPos;
    var formationRot;
    var formationSize;		///the x,y,z of the rectangle in formation space i occupy
    var formationHandle;		
    var parentLegion;		///reference to my parent legion
    var formation;
    var numRowsColumns;

    bool addedToLegion;
    nRef<nLegionAspect> legionAddedToo;
    nArray<vector3> formationPoints;
    nString entname;
    nDynAutoRef<nPhysics> physics;
    AllocationTactic allocationTactic;
    
    var width;
    var depth;

    var legion0;
    var l0pos;
    var l0rot;
    var l0width;
    var l0depth;
    int N;   
    nDynAutoRef<nController> myController;

    var legionLocked;

    float timeWithoutUnits;

    bool pureLegion;
    var legionRank; //rank0 -> unit, rank1-> group, rank2-> group of groups
    var minalt;
    var maxalt;

    int formationCalcNum;

    nArray<var> resources;
    var side;
};
//------------------------------------------------------------------------------
#endif

