#ifndef NTNLENT_H
#define NTNLENT_H    

#include "tnl.h" 
#include "tnlGhostConnection.h" 
#include "tnlNetInterface.h" 
#include "tnlRPC.h" 
#include "tnlSymmetricCipher.h"
#include "tnlAsymmetricKey.h"
#include "tnlCertificate.h"
#include "tnlBitStream.h"

#include "enkiduo/var.h"
#include "enkiduo/myvar.h"
#include "enkiduo/nthing.h"
#include "enkiduo/nent.h"
#include "tnl/ntnlconnection.h"
#include "enkiduo/ipolvar.h"
//------------------------------------------------------------------------------
///Representation of each entity in the game
//------------------------------------------------------------------------------
enum nTNLEntTicks {
    TNLENTTICK_SERVER,
    TNLENTTICK_CLIENT,
};
class nGameServer;
class nTNLEnt : public TNL::NetObject {
    typedef TNL::NetObject Parent;
    static const int MAXVARS = 32;
        
public:
    TNL_DECLARE_CLASS(nTNLEnt);
    enum MaskBits {
        InitialMask     = (1 << 0), ///Building's only mask bit is the initial mask, as no other states are ever set.
        VarsChangedMask = (1 << 1), ///When vars change set this
        FirstVarMask    = (1 << 2),
    };
    nTNLEnt();
    
    /// packUpdate writes the Player's ghost update from the server to the client.
    /// This function takes advantage of the fact that the InitialMask is only ever set
    /// for the first update of the object to send once-only state to the client.
    TNL::U32 packUpdate(TNL::GhostConnection *connection, TNL::U32 updateMask, TNL::BitStream *stream);
    
    /// unpackUpdate reads the data the server wrote in packUpdate from the packet.
    void unpackUpdate(TNL::GhostConnection *connection, TNL::BitStream *stream);
    
    /// onGhostAvailable is called on the server when it knows that this Player has been
    /// constructed on the specified client as a result of being "in scope".  In TNLTest
    /// this method call is used to test the per-ghost targeted RPC functionality of
    /// NetObject subclasses by calling rpcPlayerIsInScope on the Player ghost on
    /// the specified connection.
    void onGhostAvailable(TNL::GhostConnection *theConnection);

    /// onGhostAdd is called for every NetObject on the client after the ghost has
    /// been constructed and its initial unpackUpdate is called.  A return value of
    /// false from this function would indicate than an error had occured, notifying the
    /// network system that the connection should be terminated.
    bool onGhostAdd(TNL::GhostConnection *theConnection);
    void onGhostRemove();

    void tick( nTNLConnection* connection, nTNLEntTicks tickMode = TNLENTTICK_SERVER );
    void SetEnt( nEnt* e );
    nEnt* GetEnt() { return ent; }
    static nGameServer* gameServer;
    static nKernelServer* kernelServer;
protected:
    nEnt* ent;
    nRoot* varRoot;
    nRoot* varTail;
    nArray<ipolvar> vars;
    nArray<var> clientVars;             //so how do we do this...
};
#endif /* NTNLENT_H */
