import wx
import math
import nEventServer
import catBehavior

from inverseCPA import *
from common.defs import *
from utils_navigation import *
from common.log import Log
from common import EventNames
from cat.sensors.relativepositionsensor import *
from common import OptionManager
from cat import panelInfo


log = Log(tags=['all','sensors'])
DEFAULTSIZE          = panelInfo.CONTROLWRAPPERPANEL

BIGENTITYAVOIDFACTOR = 2
BIGENTITYSAFECPATIME = 10

class CatAvoidEntity(catBehavior.CatBehavior):
    def __init__(self, cPanelA, entity=None, bname = None, sensors = None, actuators = None, recorder = None, size=DEFAULTSIZE,  show=False):
        catBehavior.CatBehavior.__init__(self, cPanelA, entity=entity, bname=bname, size = size,  show=show)

        self.sensors = sensors
        
        self.relpos = RelativePositionSensor(entity = self.entity, behavior = self)
        self.sens = sensors
        pushsel(self.entityPath)

        # Set self dimensions and size
        self.width, self.height,self.length = gv('dimension')
        self.size = gi ('size')

        # Set self max speed
        self.maxSpeed = gf('maxSpeed')
        popsel()
        

    def makeBehaviorPanel(self, panel, size=None, entity = None, bname = None, behavior = None,  show=False):
        from cat.Interface import catStartPanel, catTargetPanel
	from common import behaviorNames
        from cat import panelInfo
        bsize = panelInfo.BEHAVIORPANEL
        aePanel = catStartPanel.CatStartPanel(panel, -1, color=None, size=bsize, behaviorType=bname , entity=entity, behavior = behavior)
	tsize = panelInfo.TARGETPANEL
	targetPanel = wx.Panel(panel, -1, size=tsize) 
        if(show):
            panel.addPanel(aePanel, targetPanel, bname)
        return [aePanel, targetPanel]
    

    def computeRelativeBearing(self, absHeading, thetat):
        ab1 = 2*math.pi - absHeading
        ab2 = ab1 - math.pi

        if ab2 <0:
            ab2 += 2*math.pi

        heading2 = thetat
        if heading2 < 0:
            heading2 += 2*math.pi
        heading2 = 2*math.pi - heading2

        rb2 = ab2 - heading2
        if rb2 < 0:
            rb2 += 2*math.pi

        return rb2

    def GetOwnEntityPosition(self):
        pushsel(self.entityPath)
        if call('hasvar', 'pos'):
            oy, oz, ox = gv('pos')
        else:
            oy, oz, ox = (0, 0, 0)
        popsel()
        return (oy, oz, ox)

    def GetOwnEntitySpeed(self):
        pushsel(self.entityPath)
        if call('hasvar', 'speed'):
            oSpeed = gf('speed')
        else:
            oSpeed = 0.0
        popsel()
        return oSpeed

    def GetOwnEntityHeading(self):
        pushsel(self.entityPath)
        if call('hasvar', 'heading'):
            oHeading = gf('heading')
        else:
            oHeading = 0.0
        popsel()
        return oHeading

    def GetAvoideePosition(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'pos'):
            ay, az, ax = gv('pos')
        else:
            ay, az, ax = (0, 0, 0)
        popsel()
        return (ay, az, ax)

    def GetAvoideeSpeed(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'speed'):
            aSpeed = gf('speed')
        else:
            aSpeed = 0.0
        popsel()
        return aSpeed

    def GetAvoideeHeading(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'heading'):
            aHeading = gf('heading')
        else:
            aHeading = 0.0
        popsel()
        return aHeading

    def GetAvoideeSize(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'size'):
            aSize = gi('size')
        else:
            aSize = 0.0
        popsel()
        #print "Size = ", aSize
        return aSize

    def IsAngleInInterval(self, theta, start, end):
        if end < start:
            end = end + 2*math.pi
        if theta > end:
            theta = theta - 2*math.pi
        else:
            if theta < start:
                theta = theta + 2*math.pi

        #print "start = ", math.degrees(start), " theta = ", math.degrees(theta), " end = ", math.degrees(end)
        if start <= theta and theta <= end:
            return True
        else:
            return False
    
    def CheckRightOfWay (self, rb2, thetam, thetat):
        thetam = NormalizeHeadingRad(math.radians(convertToMoboard (math.degrees(thetam))))
        thetat = NormalizeHeadingRad(math.radians(convertToMoboard (math.degrees(thetat))))
        #rb2 = NormalizeHeadingRad(math.radians(convertToMoboard (math.degrees(rb2))))
        
        #print "thetam = ", math.degrees(thetam), " thetat = ", math.degrees(thetat), " rb2 = ", math.degrees(rb2)

        if rb2 >=0 and rb2 < math.pi/2:
            # own ship in first quadrant w.r.t. target
            #print "CASE 1"
            if self.IsAngleInInterval(thetam, thetat + math.pi + rb2, thetat):
                collisionPossible = True
                rightOfWay = True
            else:
                collisionPossible = False
                rightOfWay = True
                
        if rb2 >= math.pi/2 and rb2 < math.pi:
            #print "CASE 2"
            if self.IsAngleInInterval(thetam, thetat + math.pi + rb2, thetat):
                collisionPossible = True
                rightOfWay = False
            else:
                collisionPossible = False
                rightOfWay = True

        if rb2 >= math.pi and rb2 < (3.0/2.0)* math.pi:
            # target in third quadrant
            #print "CASE 3"
            if self.IsAngleInInterval(thetam, thetat, thetat + rb2 - math.pi):
                collisionPossible = True
                rightOfWay = False
            else:
                collisionPossible = False
                rightOfWay = True

        if rb2 >= (3.0/2.0)* math.pi and rb2 < 360:
            # target in second quadrant
            #print "CASE 4"
            if self.IsAngleInInterval(thetam, thetat, thetat + rb2 - math.pi):
                collisionPossible = True
                if self.IsAngleInInterval(thetam, thetat + rb2 - math.pi - math.pi/2, thetat + rb2 - math.pi):
                    rightOfWay = False
                else:
                    rightOfWay = True
            else:
                collisionPossible = False
                rightOfWay = True

        #print "collisionPossible = ", collisionPossible, " rightOfWay = ", rightOfWay
        return collisionPossible, rightOfWay

    def getEntityDimensions(self, entityPath):
        if exists(entityPath):
            pushsel(entityPath)
            if call('hasvar', 'dimension'):
                dim = gv('dimension')
            else:
                dim = (20, 20, 100)
                
            popsel()     
            return dim
        else:
            return None

            
    def smallAvoidSingleEntity(self, x0m, y0m, vm, thetam, avoidee, entlist, fs, totalx, totaly):
    
        v_new = fs

        thetat = self.GetAvoideeHeading(avoidee) 
        y0t, tz, x0t = self.GetAvoideePosition(avoidee) 
        vt = self.GetAvoideeSpeed(avoidee)
        sizet = self.GetAvoideeSize(avoidee)

        #pushsel(avoidee)
        #print "*********Avoiding: ", getname()
        #popsel()

        #print "TARG: x0t = ", x0t, "y0t = ", y0t, "vt = ", vt, "thetat = ", math.degrees(thetat)

        absHeading = entlist[getfullname()][1]
        rb2 = self.computeRelativeBearing(absHeading, thetat)

        collisionPossible, rightOfWay = self.CheckRightOfWay(rb2, thetam, thetat)

        avoideeDim = self.getEntityDimensions(avoidee)
        if avoideeDim:
            avoideeWidth, y, avoideeLength = avoideeDim
        else:
            return totalx, totaly, v_new

        if sizet > self.size: # you are smaller
            safeCPAdist = avoideeLength * BIGENTITYAVOIDFACTOR
            safeCPAtime = BIGENTITYSAFECPATIME
        else: # you are bigger
            safeCPAdist = self.length * BIGENTITYAVOIDFACTOR
            safeCPAtime = BIGENTITYSAFECPATIME

        currentDistance = math.sqrt((x0m-x0t)**2 + (y0m-y0t)**2)

        print "++++++++========CD: %s, safeCPADist: %s, safe CPATime: %s, ROW: %s, CollPoss: %s" % (str(currentDistance),  str(safeCPAdist), str(safeCPAtime), str(rightOfWay), str(collisionPossible))

	noNeedToAvoid = self.size > sizet or (self.size == sizet and rightOfWay and currentDistance > safeCPADist )
        if noNeedToAvoid:
            print "No need to avoid"
            return totalx, totaly, v_new

        t_CPA, dist_CPA =  directCPA (x0t, y0t, vt, math.degrees(thetat), x0m, y0m, vm, math.degrees(thetam))
        print "++++++++======== CPATime: %s, CPADist: %s" % (str(t_CPA),  str(dist_CPA))

	if ( t_CPA > 0 and dist_CPA < safeCPAdist and t_CPA < safeCPAtime ):
            desiredCPAdist = safeCPAdist;
            if vt <= 0.1:
                v_new = self.GetOwnEntitySpeed()
                b_new = self.GetOwnEntityHeading() - math.pi / 4.0
                print "++++++++Target speed is low/stopped speed: %s, heading: %s" % (str(v_new),  str(math.degress(b_new)))
            
                #endif
            else:
                v_new, b_new = inverseCPA (x0t, y0t, vt, math.degrees(thetat), x0m, y0m, vm, math.degrees(thetam), desiredCPAdist, 1)
                print "++++++++Target speed is NOT low CPATime: %s, CPADist: %s" % (str(v_new),  str(math.degress(b_new)))
            xcoord = math.cos(b_new)
            ycoord = math.sin(b_new)
            totalx += xcoord
            totaly += ycoord
            if v_new < 0:
                v_new = 0
        #print "======>>totalx = ", totalx, " totaly = ", totaly, " v_new = ", v_new 
        return totalx, totaly, v_new
        
    def loop(self,time,dtime):
        pushsel()
        if exists(self.entityPath):
            
            # Initialize avoid vector to zero
            totalx = totaly = 0

            # Get own ship desired speed and heading
            vm, thetam = self.entity.actuators.getCurrentVec()

            #sel(self.entityPath)

             # Get own ship position and speed
            y0m,oz,x0m = self.GetOwnEntityPosition() #gv('pos')
            oSpeed = self.GetOwnEntitySpeed()
            oHeading = self.GetOwnEntityHeading()

            #print "\n *****************************************************************\n"
            #print "Entity: ", self.entity.getName()
            #print "\n *****************************************************************\n"
            

            # Get list of other entities
            entlist = self.relpos.getRelPos()

            # Initialize the return values
            returntuple = (0,thetam,vm)
            #v_new = self.maxSpeed
            v_new = vm                                      # Sushil --------------
            
            # Initialize the list of boats to avoid
            avoidList = []
            
            # Iterate through the list of entities
            sel('/game/ents/')
            dir = ndir()
            for subdir in dir:
                sel('/game/ents/'+subdir)
                dir2 = ndir()
                for subdir2 in dir2:
                    xcoord = ycoord = 0
                    avoidee = '/game/ents/'+subdir+'/'+subdir2
                    sel(avoidee)
                    if ((not getselectable()) or (getfullname() == self.entityPath)) :
                        #print "Fullname of entity: ", getfullname(), self.entityPath
                        pass
                    else:
                        # Compute avoid information for avoidee
                        #print "\n===============> smallAvoidSingleEntity"
                        totalx, totaly, v = self.smallAvoidSingleEntity(x0m, y0m, oSpeed, oHeading, avoidee, entlist, vm, totalx, totaly)
                        if v <= v_new:
                            v_new = v

        # Compute the vector projection from the desired heading
        fx = math.cos(thetam)
        fy = math.sin(thetam)

        #print "*******>> newSpeed = ", v_new * 1.7774514
        returntuple = (.5, math.atan2(totaly+fy,totalx+fx), v_new) #vm
        popsel()
        returntupleknots = (returntuple[0], returntuple[1], returntuple[2]*1.7774514)
        return returntupleknots

    def checkPostconditions(self):
        return False


def getBehaviorType():
    return behaviorType


