#
# @(#) mapproach.py 2005/09/27
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
"""
Implementation of Approach


"""

import wx
from primitivebehavior import *
#from cat import sensors
from common.defs import *
import math
from common.log import Log
from common import OptionManager
log = Log(tags=['approach', 'maintain'])

behaviorType = "MAPPROACH"

MAINTAIN_STATION          = 1
RAM_SHIP                  = 2
MOVE_TO                   = 3
APPROACH                  = 1
DEFAULT                   = 3

SPEED_CONST_MANEUVER      = 1
TIME_CONST_MANEUVER       = 2
BEARING_CONST_MANEUVER    = 3

# In minutes
MIN_TIME_SLIDER           = 1
MAX_TIME_SLIDER           = 10

# In degrees
MIN_BEARING_SLIDER        = 0
MAX_BEARING_SLIDER        = 360
DEFAULT_BEARING_SLIDER    = 0

#In yards
MIN_RANGE_SLIDER          = 0
if OptionManager.getOption('map') == 'sandiego':
    MAX_RANGE_SLIDER      = 5000
else:
    MAX_RANGE_SLIDER      = 20000
DEFAULT_RANGE_SLIDER      = 200

# In knots
MIN_SPEED_SLIDER          = 1
MAX_SPEED_SLIDER          = 20

# Default values for maneuver value slider
DEFAULT_SPEED_MANEUVER    = 10
DEFAULT_TIME_MANEUVER     = 1
DEFAULT_BEARING_MANEUVER  = 0


class MApproach(PrimitiveBehavior):
    """
    General Navigation Behavior
    """
    
    def __init__(self, sensors, type = behaviorType):

        """
        Creates the approach behavior.
        
        navigation_type: whether this is a maintain station, ram or move-to behavior
        
        sens (Sensors): The sensors approach should use. By default, it should
        use those from the gui.
        """
        if type == "MOVETO":
            self.navigation_type = MOVE_TO
        elif type == "MAINTAIN STATION" or type == "APPROACH":
            self.navigation_type = MAINTAIN_STATION
        elif type == "RAM SHIP":
            self.navigation_type = RAM_SHIP
        else:
            print "Error: unknown approach behavior", type
            self.navigation_type = DEFAULT
                
        PrimitiveBehavior.__init__(self, sensors, type)

        self.AddBool('maintainBearing', True)
        self.AddBool('maneuverChanged', True)
        self.AddBool('activated', False)
        self.AddBool('absoluteBearing', True)
        self.AddBool('reachedStation', True)
        self.AddBool('absoluteBearingLast', True)

        self.AddInt  ('maneuverType', SPEED_CONST_MANEUVER, BEARING_CONST_MANEUVER, SPEED_CONST_MANEUVER)

        sel(self.sens.getOwnship())
        maxSpeed = gf('maxSpeed')

        self.AddFloat('entMaxSpeed', int(maxSpeed), MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddFloat('maneuverValue',  int(maxSpeed/2), MIN_SPEED_SLIDER, int(maxSpeed))

        self.AddFloat('desiredStBearing', DEFAULT_BEARING_SLIDER, MIN_BEARING_SLIDER, MAX_BEARING_SLIDER*math.pi/180)
        self.AddFloat('desiredStRange', DEFAULT_RANGE_SLIDER, MIN_RANGE_SLIDER, MAX_RANGE_SLIDER)
        self.range = -1

        self.AddInt('desiredX', 22500, 0, 45000)
        self.AddInt('desiredY', 22500, 0, 45000)

        # Variables that represent the sliders min/max/default values; needed for the behavior panel
        self.AddInt ('maneuverValueDef', int(maxSpeed/2), MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddInt ('maneuverValueMin', MIN_SPEED_SLIDER, MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddInt ('maneuverValueMax', int(maxSpeed), MIN_SPEED_SLIDER, int(maxSpeed))

        self.desiredStBearingDef = DEFAULT_BEARING_SLIDER
        self.desiredStBearingMin = MIN_BEARING_SLIDER
        self.desiredStBearingMax = MAX_BEARING_SLIDER

        self.desiredStRangeDef = DEFAULT_RANGE_SLIDER
        self.desiredStRangeMin = MIN_RANGE_SLIDER
        self.desiredStRangeMax = MAX_RANGE_SLIDER

        self.reachedStation = False

        self.weight = 1
        self.bm_computed = 0
        self.vm_computed = 0
        self.t_computed = 0

        self.tSpeed = 0
        self.tHeading = 0
        self.timerRemaining = 0
        self.timerStart = 0

        self.tickTimerStart = 0
        self.justReset = True

##         self.AddFloat ('weight', 1, 0, 1)
##         self.AddFloat ('bm_computed', 0, 0, 2000)
##         self.AddFloat ('vm_computed', 0, 0, 2*math.pi)
##         self.AddFloat ('t_computed', 0, 0, 1000000)

##         self.AddFloat ('tSpeed', 0, 0, 22.22)
##         self.AddFloat ('tHeading', 0, 0, 2*math.pi)

##         self.AddFloat ('timerRemaining', 0, 0, 100000)
##         self.AddFloat ('timerStart', 0, 0, 100000)


        import nEventServer
        from common import EventNames
        nEventServer.register(EventNames.CAT_SELECT_TARGET, self.handleTargetEvent)
        nEventServer.register(EventNames.CAT_EXCLUSION_EVENT, self.handleExclusionEvent)
#        from cat.entity import behaviorset

        
        #print "approach: "+`self.sens.GetOwnEntity()`

    def maintainBM(self, vr, br, bm, r0, b0, r1, b1):
        # This function provides the time and speed for a maneuvering ship, given
        # a precise course for the maneuver
        # - the time value is informational
        
        #  Input:
        #  vr - reference ship speed (in knots)
        #  br - reference ship true bearing (in degrees)
        #  bm - maneuvering ship required course to perform the maneuver (in degrees)
        #  r0 - maneuvering ship initial range to the reference ship (in yards)
        #  b0 - maneuvering ship initial true bearing from reference ship (in degrees)
        #  r1 - maneuvering ship desired range to the reference ship (in yards)
        #  b1 - maneuvering ship desired true bearing from reference ship at the end of the maneuver (in degrees)
        
        #  Output:
        #  t - the amount of time in which the maneuver has to be completed (in minutes)
        #  vm - maneuvering ship required speed to perform the maneuver (in knots)
        

        #print "maintainBM:", vr, br, bm, r0, b0, r1, b1


        # Convert to radians
        br_rad = math.radians(br)
        bm_rad = math.radians(bm)
        b0_rad = math.radians(b0)
        b1_rad = math.radians(b1)

        dx = r0*math.cos(b0_rad) - r1*math.cos(b1_rad)
        dy = r0*math.sin(b0_rad) - r1*math.sin(b1_rad)

        bd_rad = math.atan2(dy, dx)
        # Compute -bd
        bd_rad = bd_rad + math.pi
        bd = bd_rad*180/math.pi
        bd_rad = math.radians(bd)
        if bd < 0:
            bd = br + 360
        if br >= 360:
            br = br - 360
        
        if bd - br <= 180:
            if bm < br and bm > bd:
                print "===>> error input range: [", br, bd, "]"
                bm_rad = bd_rad
            else:
                 print "====== input range: [", br, bd, "]"
        else:
            if bm < bd and bm > br:
                print "===>> error input range: [", bd, br, "]"
                bm_rad = bd_rad
            else:
                print "====== input range: [", bd, br, "]"

        
        # Convert speed to yards/minute
        vr = vr*2000.0/60.0

        if vr < 0.01:
            dx = -r0*math.cos(b0_rad) + r1*math.cos(b1_rad)
            dy = -r0*math.sin(b0_rad) + r1*math.sin(b1_rad)
            bm_rad = math.atan2(dy,dx)
            vm = 10
            t = -(60/2000)*dx/(math.cos(bm_rad)*vm)
            bm = bm_rad*180/math.pi
        else:
            vrx = vr*math.cos(br_rad)
            vry = vr*math.sin(br_rad)
            dx = r0*math.cos(b0_rad) - r1*math.cos(b1_rad)
            dy = r0*math.sin(b0_rad) - r1*math.sin(b1_rad)

            t = (-dy + dx*math.tan(bm_rad))/(vrx*math.tan(bm_rad) - vry)
            vm = (60.0*(dx*vry - dy*vrx))/(2000.0*(dx*math.sin(bm_rad) - dy*math.cos(bm_rad)))
            # Put in different error checking mechanism for speed
            if vm > 22:
                vm = 22
            if vm < 0:
                vm = 0
            
            bm = bm_rad*180/math.pi
        #print "maintainBM: bm = ", bm, "vm = ", vm, "t = ", t
        return (bm, vm, t)


    def maintainT (self, vr, br, t, r0, b0, r1, b1):

        # This function provides the course and speed for a maneuvering ship, given
        # a precise time for the completion of the maneuver
        
        # Input:
        # vr - reference ship speed (in knots)
        # br - reference ship true bearing (in degrees)
        # t - the amount of time in which the maneuver has to be completed (in minutes)
        # r0 - maneuvering ship initial range to the reference ship (in yards)
        # b0 - maneuvering ship initial true bearing from reference ship (in degrees)
        # r1 - maneuvering ship desired range to the reference ship (in yards)
        # b1 - maneuvering ship desired true bearing from reference ship at the end of the maneuver (in degrees)
        
        # Output:
        # bm - maneuvering ship required course to perform the maneuver (in degrees)
        # vm - maneuvering ship required speed to perform the maneuver (in knots)

        # print "maintainT:", vr, br, t, r0, b0, r1, b1

        if r0 == r1 and b0 == b1:
#            print "MaintainT: nothing to do - already at destination"
            return (0, 0, 0)

        if t == 0:
#            print "MaintainT: zero time selected: going with max speed"
            vm = 20
            bm, vm, t = self.maintainVM (vr, br, vm, r0, b0, r1, b1)
            return (bm, vm, t)
        
        # Convert to radians
        br = math.radians(br) 
        b0 = math.radians(b0) 
        b1 = math.radians(b1)
        
        # Convert speed to yards/minute
        vr = vr*2000/60
        dx = r0*math.cos(b0) - r1*math.cos(b1)
        dy = r0*math.sin(b0) - r1*math.sin(b1)

        if vr == 0:
            bm = math.atan2(-dy,-dx)
            vm = -(dx+dy)/((math.cos(bm)+math.sin(bm))*t)
        else:
            vrx = vr*math.cos(br)
            vry = vr*math.sin(br)
            vmx = (vrx*t-dx)/t
            vmy = (vry*t-dy)/t
            bm = math.atan2(vmy, vmx)
            vm = math.sqrt(vmx**2 + vmy**2)

        vm = (vm*60.0)/2000.0
        bm = bm*180/math.pi
        # print "maintainT: bm = ", bm, "vm = ", vm, "t = ", t
        return (bm, vm, t)

    def maintainVM (self, vr, br, vm, r0, b0, r1, b1):

        # This function provides the course and time required for a maneuvering ship, given
        # a precise speed for performing the maneuver
        # - the time value is informational

        # Input:
        # vr - reference ship speed (in knots)
        # br - reference ship true bearing (in degrees)
        # vm - maneuvering ship required speed to perform the maneuver (in knots)
        # r0 - maneuvering ship initial range to the reference ship (in yards)
        # b0 - maneuvering ship initial true bearing from reference ship (in degrees)
        # r1 - maneuvering ship desired range to the reference ship (in yards)
        # b1 - maneuvering ship desired true bearing from reference ship at the end of the maneuver (in degrees)
 
        # Output:
        # bm - maneuvering ship required course to perform the maneuver (in degrees)
        # t - the amount of time in which the maneuver has to be completed (in minutes)

        # print "maintainVM:", vr, br, vm, r0, b0, r1, b1

        if r0 == r1 and b0 == b1:
            print "MaintainVM: nothing to do - already at destination"
            return (0, 0, 0)
        
        # Convert to radians
        br = math.radians(br)
        b0 = math.radians(b0)
        b1 = math.radians(b1)

        # Convert speed to yards/minute
        vm_knots = vm
        vr = vr*2000/60
        vm = vm*2000/60

        vrx = vr*math.cos(br)
        vry = vr*math.sin(br)

        dx = r0*math.cos(b0) - r1*math.cos(b1)
        dy = r0*math.sin(b0) - r1*math.sin(b1)

        if vr == 0:
            if vm == 0:
                print "MaintainVM - error: must supply maneuver ship speed greater than zero"
                return (0, 0, 0)

            bm = math.atan2(-dy,-dx)
            #t = -(60.0/2000)*dx/(math.cos(bm)*vm)
            t = -(dx+dy)/(vm*(math.cos(bm)+math.sin(bm)))
        else:
            a = vrx**2 + vry**2 - vm**2
            b = -2*(vrx*dx + vry*dy)
            c = dx**2 + dy**2
            b2m4ac = b**2 - 4*a*c
            if (b2m4ac < 0):
                print "Error: maintainVM - no solution to the problem exists"
                return (0, 0, 0)                           
                
            delta = math.sqrt(b2m4ac)
            if delta < 0 or a == 0 :
                print "Error: maintainVM - no solution to the problem exists"
                return (0, 0, 0)                           
            t1 = (-b + delta)/(2*a)
            t2 = (-b - delta)/(2*a)

            if (t1 > 0):
                t = t1
            else:
                t = t2
            vmx = (vrx*t - dx)/t
            vmy = (vry*t - dy)/t
            bm = math.atan2(vmy, vmx)
            
        # Convert bm to degrees
        bm = bm*180/math.pi
        vm = vm_knots
        #print "maintainVM: bm = ", bm, "vm = ", vm, "t = ", t
        return (bm, vm, t)


    def Tick(self, time, dtime):
        """
        Processes the sensors to produce actuator output.

        Returns a tuple of the weight, heading, and speed.
        """

        if self.justReset == True:
            self.tickTimerStart = time
            self.justReset = False
        else:
            if (time - self.tickTimerStart) < 1:
                self.justReset = False
                pass
            else:
                self.justReset = True
        
        if not exists(self.sens.GetOwnEntity()):
            return (0, 0, 0)
        else:
            sel(self.sens.GetOwnEntity())

            #print "Navigation type", self.navigation_type
            # print "Maneuver type", self.GetVar ('maneuverType')
            # Get own ship heading information
            if call('hasvar', 'heading'):
                oHeading = gf('heading')
            else:
                oHeading = 0.0
            oHeading = 360 - oHeading*180/math.pi
            if oHeading > 360:
                oHeading = oHeading - 360            
#            print "approach: ownCourse = ", oHeading
            speed = gf('speed')

            ox, oz, oy = gv('pos')

            # Get target information
            if self.navigation_type == MOVE_TO:
                # Target is the origin of the coordinate system
                tx, tz, ty = (0,0,0)
                tSpeed = 0
                tHeading = 0
            elif exists(self.sens.GetTargetEntity()):
                sel(self.sens.GetTargetEntity())
                tx, tz, ty = gv('pos')
                tSpeed = gf ('speed')
                tHeading = gf ('heading')

                #Convert heading to moboard coordinate system
                tHeading = 360 - tHeading*180/math.pi
                if tHeading > 360:
                    tHeading = tHeading - 360
                elif tHeading < 0:
                    tHeading = tHeading + 360
                if tSpeed - self.tSpeed > 0.5 or tHeading - self.tHeading > 1:
                    #print "-----------------------> maneuverChanged"
                    self.SetVar('maneuverChanged', True)
		self.tSpeed = tSpeed
                self.tHeading = tHeading
            else:
                print "Not a moveto"
                return (0, 0, 0)  
            
            #print "approach tPosition: ", tx, ty, tz

            # Compute the current distance to the target
            currentStDistance = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            targetAngle = math.atan2(tx-ox, oy-ty)*180/math.pi 
            if targetAngle < 0:
                targetAngle = targetAngle + 360

            currentStBearing = targetAngle 
            if currentStBearing > 360:
                currentStBearing = currentStBearing - 360
            #print "approach currentStBearing: ", currentStBearing
#            print "approach stDistance: ", currentStDistance , "stBearing: ", currentStBearing
#            print "approach dsDistance: ", self.GetVar('desiredStRange'), "dsBearing: ", self.GetVar('desiredStBearing')

            if self.navigation_type == MAINTAIN_STATION:
                desiredStRange = self.GetVar('desiredStRange')
                if self.GetVar('absoluteBearing') == True:
                    desiredStBearing = self.GetVar('desiredStBearing')
                else:
                    desiredStBearing = self.GetVar('desiredStBearing') + tHeading;
                if desiredStBearing < 0:
                    desiredStBearing = desiredStBearing + 360
                elif desiredStBearing > 360:
                    desiredStBearing = desiredStBearing - 360
                #print "approach desiredStBearing:", desiredStBearing
                
                rangeDiff = math.sqrt((currentStDistance - desiredStRange)**2)
                bearingDiff = math.sqrt((currentStBearing - desiredStBearing)**2)
                rangeLimit = 0.1*desiredStRange

            elif self.navigation_type == RAM_SHIP:
                desiredStRange = 0
                if self.GetVar('absoluteBearing') == True:
                    desiredStBearing = self.GetVar('desiredStBearing')
                else:
                    desiredStBearing = self.GetVar('desiredStBearing') + tHeading;
                if desiredStBearing < 0:
                    desiredStBearing = desiredStBearing + 360
                elif desiredStBearing > 360:
                    desiredStBearing = desiredStBearing - 360
                #print "approach desiredStBearing:", desiredStBearing
                
                rangeDiff = math.sqrt((currentStDistance - desiredStRange)**2)
                bearingDiff = math.sqrt((currentStBearing - desiredStBearing)**2)
                rangeLimit = 20
               
            elif self.navigation_type == MOVE_TO:
##                 desiredX = self.GetVar('desiredX')
##                 desiredY = self.GetVar('desiredY')
                desiredX = 0
                desiredY = 0
                if self.sens.getMoveLocation() == None:
                    desiredX = self.GetVar('desiredX')
                    desiredY = self.GetVar('desiredY')
                    if(desiredX == 0 or desiredY == 0):
                        #print "Move To: error - no location selected to go to"
                        return (0, 0, 0)
                    #endif
                else:
                    desiredX, desiredZ, desiredY = self.sens.getMoveLocation()
                    #endelse
                desiredStRange = math.sqrt(desiredX**2 + desiredY**2)
                desiredStBearing = math.atan2(tx-desiredX, desiredY-ty)*180/math.pi
                if desiredStBearing < 0:
                    desiredStBearing = desiredStBearing + 360
                elif desiredStBearing > 360:
                    desiredStBearing = desiredStBearing - 360
                rangeDiff = math.sqrt((currentStDistance - desiredStRange)**2)
                bearingDiff = math.sqrt((currentStBearing - desiredStBearing)**2)
                rangeLimit = 20

            
            bearingLimit = 15

            #print "rangediff=", rangeDiff, "rangeLimit = ", rangeLimit, "bearingDiff = ", bearingDiff, "bearingLimit =" , bearingLimit

            if self.navigation_type == MAINTAIN_STATION:            
                if rangeDiff < rangeLimit and bearingDiff < bearingLimit:
                    self.reachedStation = True
                else:
                    self.reachedStation = False
            elif self.navigation_type == RAM_SHIP or self.navigation_type == MOVE_TO:
                if rangeDiff < rangeLimit:
                    self.reachedStation = True
                else:
                    self.reachedStation = False
                

            if self.reachedStation == True:
                #print "*******************>>>>>>>  Reached final station"
                #self.reachedStation = True
                if self.navigation_type == MAINTAIN_STATION:            
                    return (0.5, gf ('heading'), gf ('speed'))
                elif self.navigation_type == RAM_SHIP or self.navigation_type == MOVE_TO:
                    return (0.5, 0, 0)
                else:
                    return (0.5, 0, 0)
                
            if self.GetVar ('maneuverType') == BEARING_CONST_MANEUVER: #maintain bm
                bm, vm, t = self.maintainBM(tSpeed, tHeading, self.GetVar('maneuverValue'), currentStDistance, currentStBearing, desiredStRange, desiredStBearing)
                

            if self.GetVar ('maneuverType') == TIME_CONST_MANEUVER: #maintain t
                # print "Maintain time"
                if (self.GetVar('activated')):
                    # print "1: ", time, self.GetVar('maneuverValue')
                    self.timerStart = time
                    timeRemaining_min = self.GetVar('maneuverValue')
                else:
                    # print "2: ", time, self.GetVar('maneuverValue')
                    timeSpent = time - self.timerStart
                    timeRemaining_sec = 60*self.GetVar('maneuverValue') - timeSpent
                    timeRemaining_min = timeRemaining_sec/60.0
                    if timeRemaining_min < 0:
                        timeRemaining_min = 0

                # print "timerStart = ", self.timerStart, "timeRemaining = ", timeRemaining_min
                bm, vm, t = self.maintainT(tSpeed, tHeading, timeRemaining_min, currentStDistance, currentStBearing, desiredStRange, desiredStBearing)

            if self.GetVar ('maneuverType') == SPEED_CONST_MANEUVER: #maintain vm
                bm, vm, t = self.maintainVM(tSpeed, tHeading, self.GetVar('maneuverValue'), currentStDistance, currentStBearing, desiredStRange, desiredStBearing)
                
            bm = 360-bm
            if bm > 360:
                bm = bm - 360
            bm = math.radians(bm)

            # PD control: reduce the speed as approaching the final station
            if self.navigation_type == MAINTAIN_STATION:
                if rangeDiff < 60 and bearingDiff < 40:
                    reduceSpeed = True
                else:
                    reduceSpeed = False
            elif self.navigation_type == MOVE_TO: #self.navigation_type == RAM_SHIP or 
                if rangeDiff < 60:
                    reduceSpeed = True
                else:                    reduceSpeed = False
            else:
                reduceSpeed = False

            
            if reduceSpeed:
                vm_yrds_per_min = vm*2000/60
                tSpeed_yrds_per_min = tSpeed*2000/60
#                print "==>> vm_yrds_per_min = ", vm_yrds_per_min, "tSpeed_yrds_per_min = ", tSpeed_yrds_per_min

                if vm_yrds_per_min == 0:                                        #chris
                    return                                                      #chris

                timeToStation = rangeDiff/vm_yrds_per_min

#                print "==>> rangeDiff =", rangeDiff, "timeToStation [min]= ", timeToStation

                # Compute proportional term
                speedDifference_yrds_per_min = tSpeed_yrds_per_min - vm_yrds_per_min
                # Constants that work well (0.6, 4) (0.4, 4) (0.5, 6)
                Kp = 0.45
                #Kp = 0.3 + 0.07*speed
                #Kp = 0.2 + 0.08*speed + 0.01*speed**2
                #Kp = 0.1+ 0.1*speed + 0.01*speed**2
                pTerm = Kp*speedDifference_yrds_per_min
#                print "==>> speed_difference [yrds_per_min] = ", speedDifference_yrds_per_min
#                print "==>> pTerm = ", pTerm

                # Compute derivative term
                deceleration = (tSpeed_yrds_per_min - vm_yrds_per_min)/(rangeDiff/vm_yrds_per_min)
                Kd = 4
                #Last Kd = 0.6*speed
                dTerm = Kd*deceleration/600                 
#                print "==>> dTerm = ", dTerm
                
                vm_yrds_per_min = vm_yrds_per_min + pTerm + dTerm
                vm = vm_yrds_per_min*60/2000.0
                if vm < 0:
                    vm = 0
                #print "==>> new vm_yrds_per_min = ", vm_yrds_per_min, "vm[knots] = ", vm
                    

            #print "Approach : course = ", bm, "speed = ", vm, "time =", t
            self.weight = 0.7
            self.bm_computed = bm
            self.vm_computed = vm
            self.t_computed = t
            self.SetVar ('maneuverChanged', False)
            self.SetVar ('activated', False)
            self.SetVar ('absoluteBearingLast', self.GetVar('absoluteBearing'))
            return (0.7, bm, vm)
                                         

    def checkPreconditions(self):
        return True

    def handleTargetEvent(self, msg):
        self.SetVar ('maneuverChanged', True)
        #print "Target Changed to", str(msg)

    def checkPostconditions(self):
        #print self.reachedStation
        return self.reachedStation

    def handleExclusionEvent(self, msg):
        #print "Bleargh!", msg , "  Handle exclusion event!"

        entityDiBehavior=msg.split(' ',1)[0]
        #print "\n\nentity is " + entityDiBehavior
        #print "excluding " + msg.split(' ',1)[1]
        #print "my entity is " + self.sens.getOwnship()
        #print "I am " + self.type
        if entityDiBehavior == self.sens.getOwnship():
            #print "we are of the same entity!"
            bstring = msg.split(' ',1)[1]
            tmplist = bstring[1:-1].split(', ')
            typelist = []
            for i in tmplist:
                typelist.append(i.strip('\''))
            if self.type in typelist:
                self.deActivate()
                #print "Deactivating " + self.type + " for " + self.sens.getOwnship()
        #else:
            #print "I'm not deactivating anything.  I'm not of the same entity!"

#        if(self.behaviorType in msg):
#            self.handleStop(None)
#        self.statusMessage.SetLabel("Status: Stopped by exclusion")


def getBehaviorType():
        return behaviorType;
