#
# @(#) executive.py 2005/07/06
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
"""
Implementation of Approach

Approach works by first moving in a great circle around the target until the
correct bearing is met. The entity will start to move in closer. 
This is bad. It will be made better. Somehow.
"""
#TODO: Fix approach so it actually does what I say above
#TODO: Fix approach so it is actually good.
import wx
from primitivebehavior import *
#from cat import sensors
from common.defs import *
import math
from cat.entity import behaviorset
from cat.entity import EntityManager

type = "approach"

class Approach(PrimitiveBehavior):
    """
    The Approach Behavior
    """
    def __init__(self, sensors, app = True):
        """
        Creates the approach behavior.
        
        app (bool): whether approach should actually move closer or further away.
        If True, then approach should move closer. This is here because Retreat
        is identical, but with app = False.
        
        sens (Sensors): The sensors approach should use. By default, it should
        use those from the gui.
        """
        self.app = app
        PrimitiveBehavior.__init__(self, sensors)
        self.AddBool('maintainBearing', False)
        self.AddBool('setSpeed', True)
        self.AddFloat('desiredBearing', 0.0, 0, 2*math.pi)
        self.AddFloat('desiredSpeed', 0.0, 0.0, 22.22)
        #self.AddFloat('range', 0.0, 0.0, 20000.0)
        self.range = -1
        #print "approach: "+`self.sens.GetOwnEntity()`
        self.type = type

    def Tick(self, time, dtime):
        """
        Processes the sensors to produce actuator output.

        Returns a tuple of the weight, heading, and speed.
        """
        if not exists(self.sens.GetOwnEntity()) or not exists(self.sens.GetTargetEntity()):
            return (0, 0, 0)
        else:
            sel(self.sens.GetOwnEntity())
            ox, oz, oy = gv('pos')
            maxSpeed = gf('maxSpeed')
            sel(self.sens.GetTargetEntity())
            tx, tz, ty = gv('pos')
            if call('hasvar', 'speed'):
                tSpeed = gf('speed')
            else:
                tSpeed = 0.0

            dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            #bear = -math.atan2(ty-oy, ox-tx)+math.pi/2
            bear = math.atan2(tx-ox, ty-oy)

            if self.range == -1:
                self.range = dist
            
            if bear < 0:
                bear += 2*math.pi

            db = bear - self.GetVar('desiredBearing')
            moveBearing = 0
            newBearing = 0
            if self.GetVar('maintainBearing'):
                if db < math.radians(5):
                    moveBearing = 1
                    newBearing = bear - math.pi/2
                elif db > math.radians(5):
                    moveBearing = 1
                    newBearing = bear + math.pi/2

            if not self.app:
                bear = bear - math.pi
            bx = math.cos(bear) + moveBearing * math.cos(newBearing)
            by = math.sin(bear) + moveBearing * math.sin(newBearing)
            newHeading = math.atan2(by, bx)
            
            newSpeed = self.GetVar('desiredSpeed')
            return (0.5, newHeading, newSpeed)

    def checkPreconditions(self):
        return True

    def checkPostconditions(self):
        if exists(self.sens.GetOwnEntity()) and exists(self.sens.GetTargetEntity()):
            sel(self.sens.GetOwnEntity())
            ox, oz, oy = gv('pos')
            sel(self.sens.GetTargetEntity())
            tx, tz, ty = gv('pos')
            dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            if dist <= self.range:
                return True
            else:
                return False
            
behaviorset.register(type, Approach)

class ApproachPanel(PrimitiveBehaviorPanel):
    """
    Sets up the GUI for changing the parameters of approach.
    """
    def __init__(self, parent, panelID, buttonID, name, stop, parentPanel):
        """
        Creates the new panel.
        
        parentPanel (wx.Panel): Approach, Retreat, and Maintain are mutually
        exclusive. These behaviors are put under one tab and they need this
        parent tab to change its title.
        """
        PrimitiveBehaviorPanel.__init__(self, parent, panelID, buttonID, name, stop, type)
        self.parentPanel = parentPanel
        newSizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(newSizer, 1, border=10, flag=wx.ALL|wx.EXPAND)
        self.bearingCheck = wx.CheckBox(self, -1, "Bearing")
        self.bearingCheck.SetValue(False)
        self.bearingSlider = wx.Slider(self, -1, 0, 0, 360, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        self.speedSlider = wx.Slider(self, -1, 0, 0, 0, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        newSizer.Add(self.bearingCheck, border=5, flag=wx.RIGHT)
        newSizer.Add(self.bearingSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        newSizer.Add(wx.StaticText(self, -1, "Speed"))
        newSizer.Add(self.speedSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        wx.EVT_CHECKBOX(self.bearingCheck, -1, self.OnBearingCheck)

    def Tick(self, time, dtime):
        """
        Runs Approach, gets the actuator values, and passes them up to the GUI
        for fusion.
        """
        if EntityManager.currentState:
            if exists(EntityManager.currentState.sensors.GetOwnEntity()):
                sel(EntityManager.currentState.sensors.GetOwnEntity())
                max = gf('maxSpeed')
                self.speedSlider.SetRange(0, int(max*100))
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar('desiredBearing', self.bearingSlider.GetValue())
            beh.SetVar('desiredSpeed', self.speedSlider.GetValue()/100.0)
                
    def OnActivate(self, event):
        """
        Called when the activate button is pressed.
        
        Turns off other behaviors and changes the parent tab's name.
        """
        p = self.parentPanel.GetParent()
        self.panelGroup.SwapPanel(self)
        p.SetPageText(self.parentPanel.pageNum, "Movement [Approach]")
    
    def OnDeactivate(self, event):
        """
        Called when deactivate button is pressed.
        
        Because of the panel group, this panel is guaranteed to be the only one
        on and simple has to change the parent tab name.
        """
        p = self.parentPanel.GetParent()
        p.SetPageText(self.parentPanel.pageNum, "Movement [off]")

    def OnBearingCheck(self, event):
        """
        Called when the bearing checkbox is pressed.
        
        Toggles whether bearing should be considered when approaching.
        """
        if EntityManager.currentState:
            EntityManager.currentState.bset.getList(type)[0].SetVar('maintainBearing', event.IsChecked())

    def refresh(self):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            self.bearingCheck.SetValue(beh.GetVar('maintainBearing'))
            self.bearingSlider.SetValue(beh.GetVar('desiredBearing'))
            self.speedSlider.SetValue(beh.GetVar('desiredSpeed')*100)

