import wx
import math
from cat.Interface import sensors
from common.defs import *
from common.log import Log
log = Log(tags=['all','sensors'])
from cat.entity import behaviorset
from cat.entity import EntityManager
import nEventServer
from common import EventNames

from common import OptionManager
if OptionManager.getOption('map') == 'sandiego':
    MAX_RANGE_SLIDER      = 5000
else:
    MAX_RANGE_SLIDER      = 20000
         
from primitivebehavior import *

behaviorType = "AVOID LAND"

class AvoidLand(PrimitiveBehavior):
    def __init__(self,sens):
        Behavior.__init__(self,sens)
        self.AddFloat("range",1000.0,0.0,10000.0)
        self.AddFloat("speed",0.0,0.0,28.0)
        self.type = behaviorType
        self.sens = sens
        import nEventServer
        from common import EventNames
        nEventServer.register(EventNames.CAT_EXCLUSION_EVENT,self.handleExclusionEvent)

    def Tick(self,time,dtime):
        #print "in avoidland tick"
        if exists(self.sens.GetOwnEntity()):
            weight = .5
            totalx = totaly = 0;
            ranges = []
            sel(self.sens.GetOwnEntity())
            speed = self.GetVar('speed')
            heading = gf("heading")
            ox,oz,oy = gv('pos')
            panic = False
            inEffect = False
            
            for i in range(181):
                tx = ox + self.GetVar('range')*math.sin(heading - math.pi/2 + float(i)*(math.pi/180.0))
                ty = oy + self.GetVar('range')*math.cos(heading - math.pi/2 + float(i)*(math.pi/180.0))
                #pass point (tx,ty) to lineIntersect function
                dist = call('/map.getintersect',ox,0,oy,tx,-30,ty)
                #print ' intersect: ',dist[0],dist[2],-90 + i,time
                #
                #ranges.append(dist)

                """
                #if lineIntersection function returns a distance:
                xcoord = dist * math.cos(i * (math.pi/180.0))
                ycoord = dist * math.sin(i * (math.pi/180.0))
                """

                
                #else if lineintersection function returns a point of intersection
                if(dist[0] != 0 or dist[2] != 0 or dist[1] != 0):
                    xcoord = dist[0]-ox
                    ycoord = dist[2]-oy
                    
                    mag = math.sqrt(xcoord**2 + ycoord**2)
                    theta = (heading + (float(i)-90) * math.pi/180.0) - math.pi
                    
                    xcoord = math.cos(theta) * mag
                    ycoord = math.sin(theta) * mag

                    inEffect = True
                    
                    if mag != 0 and not panic:
                        mag = self.GetVar('range')/mag - 1.0
                        totalx = totalx + xcoord
                        totaly = totaly + ycoord
                    elif mag == 0:
                        totalx = xcoord
                        totaly = ycoord
                        panic = True
                


                    #print totalx,totaly
            if inEffect == False:
                newHeading = gf('desiredHeading')
            else:
                newHeading = math.atan2(totaly,totalx)
            if newHeading < 0:
                newHeading += 2*math.pi
            return (weight,newHeading,speed)
        
    def handleExclusionEvent(self, msg):
        #print "Bleargh!", msg , "  Handle exclusion event!"

        entityDiBehavior=msg.split(' ',1)[0]
        #print "\n\nentity is " + entityDiBehavior
        #print "excluding " + msg.split(' ',1)[1]
        #print "my entity is " + self.sens.getOwnship()
        #print "I am " + self.type
        if entityDiBehavior == self.sens.getOwnship():
            #print "we are of the same entity!"
            bstring = msg.split(' ',1)[1]
            tmplist = bstring[1:-1].split(', ')
            typelist = []
            for i in tmplist:
                typelist.append(i.strip('\''))
            if self.type in typelist:
                self.deActivate()
                #print "Deactivating " + self.type + " for " + self.sens.getOwnship()
        #else:
            #print "I'm not deactivating anything.  I'm not of the same entity!"

            
behaviorset.register(behaviorType,AvoidLand)

def getBehaviorType():
    return behaviorType


class AvoidLandPanel(wx.Panel):
    def __init__(self, parent, panelID, color, defaultsize, behaviorType = 'avoidland'):

        wx.Panel.__init__(self,parent,panelID,size=defaultsize,style = wx.RAISED_BORDER| wx.DOUBLE_BORDER)
        
        nEventServer.register(EventNames.CAT_SELECT_ENT,      self.handleEntSelection)
        
        self.behaviorType = behaviorType
	self.SetBackgroundColour(color)
        self.parentPanel = parent
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.statusMessage = wx.StaticText(self,-1,self.behaviorType)
        self.sizer.Add(self.statusMessage,0,wx.ALIGN_LEFT)
        self.sizer.Add(wx.StaticText(self,-1,'Avoidance radius:'))
        labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.rangeslider = wx.Slider(self,-1,1000,0,MAX_RANGE_SLIDER,style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        wx.EVT_SLIDER(self.rangeslider,-1,self.OnRangeChange)
        labelSizer.Add(self.rangeslider,1,border = 5,flag = wx.RIGHT|wx.EXPAND)
        labelSizer.Add(wx.StaticText(self, -1, 'yards'), flag=wx.ALIGN_BOTTOM)
        self.sizer.Add(labelSizer, 0, border=10, flag=wx.EXPAND|wx.BOTTOM)
        self.sizer.Add(wx.StaticText(self,-1,'Speed'))

        labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.speedSlider = wx.Slider(self,-1,0,0,20,style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        wx.EVT_SLIDER(self.speedSlider,-1,self.OnSpeedChange)
        labelSizer.Add(self.speedSlider,1,border = 5,flag = wx.RIGHT|wx.EXPAND)
        labelSizer.Add(wx.StaticText(self, -1, 'knots'), flag=wx.ALIGN_BOTTOM)
        self.sizer.Add(labelSizer, 0, border=10, flag=wx.EXPAND|wx.BOTTOM)
        self.startStopButton = wx.Button(self,-1,'Start', style=wx.BU_EXACTFIT)
        self.startStopButtonState = 'start'
        wx.EVT_BUTTON(self.startStopButton,-1,self.OnClick)
        emptysizer = wx.BoxSizer()
        self.sizer.Add(emptysizer, 1, flag=wx.EXPAND)
        self.sizer.Add(self.startStopButton, 0, wx.ALIGN_RIGHT|wx.ALIGN_BOTTOM)
        self.SetSizer(self.sizer)
        self.EXCLUSION_LIST = ()

        
        from cat.entity import EntityManager
        if(EntityManager.currentState):
            entity = EntityManager.currentState.entity
            sel(entity)
            speedSlider.SetValue(gf('maxspeed')/2.0)
            

        from common import tickmanager
        tickmanager.register('AvoidLand', self.Tick)

    def OnSpeedChange(self,event):
        behavior = self.getBehavior()
        if(behavior):
            behavior.SetVar('speed',self.speedSlider.GetValue())
        else:
            print "no attached avoidland behavior to maneuver on"
            
    def OnRangeChange(self,event):
        behavior = self.getBehavior()
        if(behavior):
            behavior.SetVar('range',self.rangeslider.GetValue())
        else:
            print "no attached avoidland behavior to maneuver on"

            
    def OnClick(self,event):
        if(self.startStopButtonState == 'start'):
            self.handleStart(event)
        else:
            self.handleStop(event)

    def handleStart(self,event):
        self.startStopButtonState = "stop"
        self.startStopButton.SetLabel("Stop")
        behavior = self.getBehavior()
        if(behavior):
            import nEventServer
            from common import EventNames
            nEventServer.pull(EventNames.CAT_EXCLUSION_EVENT, self.getSensors().GetOwnEntity() + ' '+str(self.EXCLUSION_LIST))
            behavior.SetVar('speed',self.speedSlider.GetValue())
            behavior.SetVar('range',self.rangeslider.GetValue())
            behavior.activate()
        else:
            print "No current behavior selected"
            
    def getBehavior(self):
        from cat.entity import EntityManager
        if(EntityManager.currentState):
            entity = EntityManager.currentState.entity
            entityState = EntityManager.currentState
            behaviorList = entityState.bset.getList(self.behaviorType)
            #kludge because we do not yet handle multiple behaviors running simultaneously
            behavior = behaviorList[0]
            return behavior
        else:
            return None

    def handleStop(self,event):
        self.startStopButtonState = "start"
        self.startStopButton.SetLabel("Start")
        behavior = self.getBehavior()
        if(behavior):
            behavior.deActivate()
        else:
            print "No behavior to DeActivate"

    def Tick(self,time,dtime):
        if EntityManager.currentState:
            if exists(EntityManager.currentState.sensors.GetOwnEntity()):
                try:
                    sel(EntityManager.currentState.sensors.GetOwnEntity())
                    max = gf('maxSpeed')
                    self.speedSlider.SetRange(0,max)
                except:
                    printExcept('cat:AvoidLandPanel')
                        
        #print "in avoidlandpanel tick"
        #if EntityManager.currentState:
            #if exists(EntityManager.currentState.sensors.GetOwnEntity()):
            #sel(EntityManager.currentState.sensors.GetOwnEntity())
         #   beh = EntityManager.currentState.bset.getList(behaviorType)[0]
          #  beh.SetVar('range',self.rangeslider.GetValue())

    def getSensors(self):
        from cat.entity import EntityManager
        if(EntityManager.currentState):
            sensors = EntityManager.currentState.getSensors()
            return sensors
        else:
            return None

    def handleEntSelection(self, msg):
        if EntityManager.currentState:
            if exists(EntityManager.currentState.sensors.GetOwnEntity()):
                try:
                    sel(EntityManager.currentState.sensors.GetOwnEntity())
                    max = gf('maxSpeed')
                    self.speedSlider.SetRange(0,max)
                except:
                    printExcept('cat:AvoidEntityPanel')
            beh = EntityManager.currentState.bset.getList(behaviorType)[0]
            self.speedSlider.SetValue(beh.GetVar('speed'))
            self.rangeslider.SetValue(beh.GetVar('range'))
            if beh.isActive():
                self.startStopButtonState = 'stop'
                self.startStopButton.SetLabel('Stop')
            else:
                self.startStopButtonState = 'start'
                self.startStopButton.SetLabel('Start')
