#
# @(#) executive.py 2005/07/06
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
#TODO: Finish comments
import wx
from primitivebehavior import *
#from cat import sensors
from common.defs import *
import math
from cat.entity import behaviorset
from cat.entity import EntityManager

type = "maintain"

class Maintain(PrimitiveBehavior):
    """The Maintain Station Behavior"""
    def __init__(self, sensors):
        PrimitiveBehavior.__init__(self, sensors)
        self.AddBool("maintainBearing", True)
        self.AddBool("maintainRange", True)
        self.AddInt("rangeTolerance", 100, 0, 1000)
        self.AddFloat("bearingTolerance", math.degrees(5), 0, math.pi)
        self.AddFloat("range",  0.0, 0.0, 10000.0)
        self.AddFloat("bearing", 0.0, 0.0, 360.0)
        self.isPoint = False
        self.point = (0,0,0)
        self.maintaining = False
        self.centered = False
        self.sens = sensors
        self.type = type
        #print "maintain: "+`self.sens.GetOwnEntity()`
    
    def Tick(self, time, dtime):
        if exists(self.sens.GetOwnEntity()) and exists(self.sens.GetTargetEntity()):
            self.maintaining = True
            sel(self.sens.GetOwnEntity())
            ox, oz, oy = gv("pos")
            maxSpeed = gf("maxSpeed")

            if self.isPoint:
                tx, tz, ty = self.point
                self.SetVar('maintainBearing', False)
                tSpeed = maxSpeed
            else:
                sel(self.sens.GetTargetEntity())
                tx, tz, ty = gv("pos")
                tSpeed = gf("speed")

            
            d = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            b = math.atan2(tx-ox, ty-oy)

            if self.isPoint:
                newHeading = b
                newSpeed = tSpeed
                weight = 0.5
            else:
                #if not self.GetVar('range') or not self.GetVar('bearing'):
                #    self.SetVar('range', d)
                #    self.SetVar('bearing', b)

                #if self.GetVar('bearing') < 0:
                #    self.SetVar('bearing', self.GetVar('bearing')+ 2*math.pi)

                if b < 0:
                    b += 2*math.pi

                moveRange = moveBearing = 0
                newBearing1 = newBearing2 = 0

                bWindow = self.bearingWindow()
                if bWindow < 0:
                    newBearing1 = b - math.pi/2
                    moveBearing = 1
                elif bWindow > 0:
                    newBearing1 = b + math.pi/2
                    moveBearing = 1

                rWindow = self.rangeWindow()
                if not self.centered and not rWindow:
                    self.centered = True
                if rWindow < 0:
                    newBearing2 = math.pi - b
                    moveRange = 1
                elif rWindow > 0:
                    newBearing2 = b
                    moveRange = 1

                #print bWindow, rWindow

                bx = moveBearing * math.cos(newBearing1) + moveRange * math.cos(newBearing2)
                by = moveBearing * math.sin(newBearing1) + moveRange * math.sin(newBearing2)
                newHeading = math.atan2(by, bx)

                newSpeed = maxSpeed / 2.0

                midSpeed = (maxSpeed + tSpeed) / 2.0

                if maxSpeed < tSpeed:
                    newSpeed = maxSpeed
                elif midSpeed > newSpeed:
                    newSpeed = midSpeed

                #print not rWindow and not bWindow
                if not rWindow and not bWindow:
                    newSpeed = 0
                
                #if moveBearing or moveRange:
                weight = 0.5
                #else:
                #    weight = 0
            return (weight, newHeading, newSpeed)
        else:
            return (0, 0, 0)

    def bearingWindow(self):
        if exists(self.sens.GetOwnEntity()) and exists(self.sens.GetTargetEntity()):
            inWindow = 0
            if self.GetVar("maintainBearing"):
                sel(self.sens.GetOwnEntity())
                ox, oz, oy = gv("pos")
                sel(self.sens.GetTargetEntity())
                hd = gf('heading')
                tx, tz, ty = gv("pos")
                b = math.atan2(ox-tx, oy-ty) - hd
                #print "^^bearing window",math.degrees(b)
                if b > math.pi:
                    b -= math.pi * 2
                if b < -math.pi:
                    b += math.pi * 2
                db = b - self.GetVar('bearing')
                #print db
                if db < -self.GetVar("bearingTolerance"):
                    inWindow = -1
                elif db > self.GetVar("bearingTolerance"):
                    inWindow = 1
            return inWindow
        else:
            return 0

    def rangeWindow(self):
        if exists(self.sens.GetOwnEntity()) and exists(self.sens.GetTargetEntity()):
            inWindow = 0
            if self.GetVar("maintainRange"):
                sel(self.sens.GetOwnEntity())
                ox, oz, oy = gv("pos")
                sel(self.sens.GetTargetEntity())
                tx, tz, ty = gv("pos")
                d = math.sqrt((ox-tx)**2 + (oy-ty)**2)
                rangeTol = self.GetVar("rangeTolerance")
                dr = d - self.GetVar('range')
                if dr < -rangeTol:
                    inWindow = -1
                elif dr > rangeTol:
                    inWindow = 1
            return inWindow
        else:
            return 0
        

    def checkPreconditions(self):
##         if self.rangeWindow() or self.bearingWindow():
##             return True
##         else:
##             return False
        rw = self.rangeWindow()
        bw = self.bearingWindow()
        if (rw == 0 and bw == 0) \
           or self.maintaining:
            return True
        else:
            return False

    def checkPostconditions(self):
        #return not self.checkPreconditions()
        return self.maintaining
        
behaviorset.register(type, Maintain)

class MaintainPanel(PrimitiveBehaviorPanel):
    def __init__(self, parent, panelID, buttonID, name, stop, parentPanel, beh = None):
        PrimitiveBehaviorPanel.__init__(self, parent, panelID, buttonID, name, stop, type)
        self.parentPanel = parentPanel
        
        newSizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(newSizer, 1, border=10, flag=wx.ALL|wx.EXPAND)
        self.bearingCheck = wx.CheckBox(self, -1, "Bearing")
        self.bearingCheck.SetValue(True)
        self.bearingSlider = wx.Slider(self, -1, 5, 0, 180, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        newSizer.Add(self.bearingCheck, border=5, flag=wx.RIGHT)
        newSizer.Add(self.bearingSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        wx.EVT_CHECKBOX(self.bearingCheck, -1, self.OnBearingCheck)
        

        newSizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(newSizer, 1, border=10, flag=wx.ALL|wx.EXPAND)
        self.rangeCheck = wx.CheckBox(self, -1, "Range")
        self.rangeCheck.SetValue(True)
        self.rangeSlider = wx.Slider(self, -1, 100, 0, 1000, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        newSizer.Add(self.rangeCheck, border=5, flag=wx.RIGHT)
        newSizer.Add(self.rangeSlider,  proportion=1, border=5, flag=wx.RIGHT|wx.EXPAND)
        wx.EVT_CHECKBOX(self.rangeCheck, -1, self.OnRangeCheck)

    def Tick(self, time, dtime):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar("rangeTolerance", self.rangeSlider.GetValue())
            beh.SetVar("bearingTolerance", self.bearingSlider.GetValue())
        
    def OnActivate(self, event):
        p = self.parentPanel.GetParent() #Gui Frame
        self.panelGroup.SwapPanel(self)
        p.SetPageText(self.parentPanel.pageNum, "Movement [Maintain]")

        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            if exists(beh.sens.GetOwnEntity()) and exists(beh.sens.GetOwnEntity()):
                sel(beh.sens.GetOwnEntity())
                ox, oz, oy = gv("pos")
                sel(beh.sens.GetTargetEntity())
                tx, tz, ty = gv("pos")
                hd = gf('heading')

                d = math.sqrt((ox-tx)**2 + (oy-ty)**2)
                b = math.atan2(ox-tx, oy-ty) - hd
                #print "^^^^^ Bearing", math.degrees(b), "^^^^^"
                beh.SetVar('range', d)
                beh.SetVar('bearing', b)
    
    def OnDeactivate(self, event):
        p = self.parentPanel.GetParent()
        p.SetPageText(self.parentPanel.pageNum, "Movement [off]")

    def OnRangeCheck(self, event):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar("maintainRange", event.IsChecked())

    def OnBearingCheck(self, event):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar('maintainBearing', event.IsChecked())

    def refresh(self):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            self.bearingCheck.SetValue(beh.GetVar('maintainBearing'))
            self.rangeCheck.SetValue(beh.GetVar('maintainRange'))
            self.rangeSlider.SetValue(beh.GetVar('rangeTolerance'))
            self.bearingSlider.SetValue(beh.GetVar('bearingTolerance'))
