#
# @(#) executive.py 2005/07/06
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
#TODO: Finish comments
import wx
from primitivebehavior import *
#from cat import sensors
from common.defs import *
import math
from wx.lib.intctrl import *
import random

from cat.entity import behaviorset
from cat.entity import EntityManager

type = "moveto"

class MoveTo(PrimitiveBehavior):
    """The Move To Behavior"""
    def __init__(self, sensors):
        PrimitiveBehavior.__init__(self, sensors)
        self.AddInt("distTolerance", 100, 0, 1000)
        self.AddVector("point", (0,0,0), 0, 50000)
        self.AddBool('random', False)
        self.rand = random.Random()
        self.firstRun = True
        self.gotClose = False
        self.type = type
    
    def Tick(self, time, dtime):
        if exists(self.sens.GetOwnEntity()):
            sel(self.sens.GetOwnEntity())
            
            ox, oz, oy = gv("pos")
            maxSpeed = gf("maxSpeed")
            
            tx, tz, ty = self.GetVar('point')
            
            d = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            b = math.atan2(tx-ox, ty-oy)
            
            newHeading = b
            if d < self.GetVar('distTolerance'):
                newSpeed = 0
            else:
                newSpeed = maxSpeed
            weight = 0.5
            
            return (weight, newHeading, newSpeed)
        else:
            return (0, 0, 0)

    def checkRange(self, dist):
        if exists(self.sens.GetOwnEntity()):
            sel(self.sens.GetOwnEntity())
            ox, oz, oy = gv("pos")
            tx, tz, ty = self.GetVar('point')
            
            d = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            if d <= dist:
                return True
        return False

    def makePoint(self):
        if self.GetVar('random'):
            self.SetVar('point', (self.rand.randint(20000, 25000), 0,\
                                  self.rand.randint(20000, 25000)))
        elif self.GetVar('point') == (0,0,0):
            self.SetVar('point', gv('pos'))

    def checkPreconditions(self):
        if self.firstRun:
            self.makePoint()
            self.firstRun = False
        if self.checkRange(self.GetVar('distTolerance')/2.0):
            self.gotClose = True
        return not self.gotClose

    def checkPostconditions(self):
        return self.checkRange(self.GetVar('distTolerance'))
            
behaviorset.register(type, MoveTo)

class MoveToPanel(PrimitiveBehaviorPanel):
    def __init__(self, parent, panelID, buttonID, name, stop, parentPanel):
        PrimitiveBehaviorPanel.__init__(self, parent, panelID, buttonID, name, stop, type)
        self.rand = random.Random()
        self.parentPanel = parentPanel

        newSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(newSizer, 0, border=10, flag=wx.ALL)
        newSizer.Add(wx.StaticText(self, -1, 'x:'), border=10, flag=wx.RIGHT)
        self.xpos = IntCtrl(self, value=0, size=(60,-1), min=0, limited=True)
        newSizer.Add(self.xpos, 0, border=10, flag=wx.RIGHT)
        newSizer.Add(wx.StaticText(self, -1, 'y:'), border=10, flag=wx.RIGHT)
        self.ypos = IntCtrl(self, value=0, size=(60,-1), min=0, limited=True)
        newSizer.Add(self.ypos, 0)

        newSizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(newSizer, 0, border=10, flag=wx.ALL|wx.EXPAND)
        self.distSlider = wx.Slider(self, -1, 100, 0, 1000, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        newSizer.Add(self.distSlider,  proportion=1, border=5, flag=wx.RIGHT|wx.EXPAND)

        newSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.randomCheck = wx.CheckBox(self, 10, "Random")
        self.randomButton = wx.Button(self, 20, "Random Point")
        wx.EVT_BUTTON(self.randomCheck, 10, self.OnRandomCheck)
        wx.EVT_BUTTON(self.randomButton, 20, self.OnRandomButton)
        newSizer.Add(self.randomCheck)
        newSizer.Add(self.randomButton)
        self.sizer.Add(newSizer, border=10, flag=wx.ALL)

    def Tick(self, time, dtime):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar("distTolerance", self.distSlider.GetValue())
            beh.SetVar('point', (self.xpos.GetValue(), 0, self.ypos.GetValue()))
            beh.SetVar('random', self.randomCheck.IsChecked())
        
    def OnActivate(self, event):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            if beh.GetVar('point') == (0,0,0) and exists(beh.sens.GetOwnEntity()) and \
                   self.xpos.GetValue() == 0 and self.ypos.GetValue() == 0:
                sel(beh.sens.GetOwnEntity())
                point = gv('pos')
                beh.SetVar('point', gv('pos'))
                self.xpos.SetValue(int(point[0]))
                self.ypos.SetValue(int(point[2]))
            
        p = self.parentPanel.GetParent() #Gui Frame
        self.panelGroup.SwapPanel(self)
        p.SetPageText(self.parentPanel.pageNum, "Movement [MoveTo]")
    
    def OnDeactivate(self, event):
        p = self.parentPanel.GetParent()
        p.SetPageText(self.parentPanel.pageNum, "Movement [off]")

    def OnRandomCheck(self, event):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar('random', self.randomCheck.IsChecked())

    def OnRandomButton(self, event):
        x = self.rand.randint(15000, 25000)
        y = self.rand.randint(15000, 25000)
        self.xpos.SetValue(x)
        self.ypos.SetValue(y)

    def refresh(self):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar('random', self.randomCheck.IsChecked())
            self.distSlider.SetValue(beh.GetVar('distTolerance'))
            x, z, y = beh.GetVar('point')
            self.xpos.SetValue(x)
            self.ypos.SetValue(y)
            self.randomCheck.SetValue(beh.GetVar('random'))
