#
# @(#) executive.py 2005/07/06
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
"""
Implementation of Retreat.

Seeing that Retreat is almost identical to Approach, everything is pretty much
either and direct inherit or a cut-and-paste of what is in there.
"""
import wx
from approach import *

from cat.entity import behaviorset
from cat.entity import EntityManager

type = "retreat"

class Retreat(Approach):
    """
    The Retreat Behavior.
    
    Just inherits from Approach and flips the flag for approaching to retreating.
    """
    def __init__(self, sensors):
        Approach.__init__(self, sensors, False)
        self.type = type
        #self.SetVar('range', 20000.0)

    def checkPreconditions(self):
        return True

    def checkPostconditions(self):
        if exists(self.sens.GetOwnEntity()) and exists(self.sens.GetTargetEntity()):
            sel(self.sens.GetOwnEntity())
            ox, oz, oy = gv('pos')
            sel(self.sens.GetTargetEntity())
            tx, tz, ty = gv('pos')
            dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            if dist >= self.range:
                return True
            else:
                return False
        
behaviorset.register(type, Retreat)

class RetreatPanel(PrimitiveBehaviorPanel):
    """
    The retreat GUI panel
    
    See ApproachPanel for comments. This is just a cut and paste job.
    """
    def __init__(self, parent, panelID, buttonID, name, stop, parentPanel):
        PrimitiveBehaviorPanel.__init__(self, parent, panelID, buttonID, name, stop, type)
        self.parentPanel = parentPanel

        newSizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(newSizer, 1, border=10, flag=wx.ALL|wx.EXPAND)
        self.bearingCheck = wx.CheckBox(self, -1, "Bearing")
        self.bearingCheck.SetValue(False)
        self.bearingSlider = wx.Slider(self, -1, 0, 0, 360, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        self.speedSlider = wx.Slider(self, -1, 0, 0, 0, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        newSizer.Add(self.bearingCheck, border=5, flag=wx.RIGHT)
        newSizer.Add(self.bearingSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        newSizer.Add(wx.StaticText(self, -1, "Speed"))
        newSizer.Add(self.speedSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        wx.EVT_CHECKBOX(self.bearingCheck, -1, self.OnBearingCheck)

    def Tick(self, time, dtime):
        if EntityManager.currentState:
            if exists(EntityManager.currentState.sensors.GetOwnEntity()):
                sel(EntityManager.currentState.sensors.GetOwnEntity())
                max = gf('maxSpeed')
                self.speedSlider.SetRange(0, int(max*100))
            beh = EntityManager.currentState.bset.getList(type)[0]
            beh.SetVar('desiredBearing', self.bearingSlider.GetValue())
            beh.SetVar('desiredSpeed', self.speedSlider.GetValue()/100.0)
                
    def OnActivate(self, event):
        p = self.parentPanel.GetParent()
        self.panelGroup.SwapPanel(self)
        p.SetPageText(self.parentPanel.pageNum, "Movement [Retreat]")
    
    def OnDeactivate(self, event):
        p = self.parentPanel.GetParent()
        p.SetPageText(self.parentPanel.pageNum, "Movement [off]")

    def OnBearingCheck(self, event):
        if EntityManager.currentState:
            EntityManager.currentState.bset.getList(type)[0].SetVar('maintainBearing', event.IsChecked())

    def refresh(self):
        if EntityManager.currentState:
            beh = EntityManager.currentState.bset.getList(type)[0]
            self.bearingCheck.SetValue(beh.GetVar('maintainBearing'))
            self.bearingSlider.SetValue(beh.GetVar('desiredBearing'))
            self.speedSlider.SetValue(beh.GetVar('desiredSpeed')*100)
