from cat.Interface import sensors
from cat.Interface.flagsensor import FlagSensor
from cat.Interface.entitydistancesensor import EntityDistanceSensor
from cat.BehaviorNetwork import triggers
import cat.Behaviors as Behaviors

import xml.sax
from xml.sax.handler import ContentHandler
import math
from common.defs import *
NETWORK = 0
BEHAVIOR = 1
PARAMETERS = 2
SENSORS = 3
LINKS = 4

class NetworkController(ContentHandler):
    
    def __init__(self, sens=sensors.guiSensors, extvars = {}):
        self.running = False
        self.network = {}
        self.behaviors = []
        self.links = []
        self.inParam = False
        self.inLinks = False
        self.sens = sens
        self.extvars = extvars
        self.triggers = []
        self.slots = {}
        self.customTriggers = []
        self.triggerMonitor = None

    def Tick(self, time, dtime):
        theta = 0.0
        r = 0.0
        self.triggerMonitor.Tick(time, dtime)
        #ignore custom panel
        #compute custom triggers
        #append to list
        self.INIT.sendActivation()
        if not self.INIT.checkPreconditions():
            actuators = []
            weightsum = 0
            dx = 0.0
            dy = 0.0
            firstcut = []
            for id, node in self.network.iteritems():
                if node.checkPreconditions():
                    firstcut.append(node)
                else:
                    node.reset()
                    
            for node in firstcut:
                if not node.inhibited:
                    #print node.id
                    self.avoidEntity.active = node.child.GetVar('avoidentity')
                    self.avoidEntity.SetVar('panicRange', \
                                            node.child.GetVar('epPanicRange'))
                    self.avoidEntity.SetVar('effectiveRange', \
                                            node.child.GetVar('epEffectiveRange'))
                    self.avoidEntity.SetVar('desiredSpeed', \
                                            node.child.GetVar('epDesiredSpeed'))
                    action = node.Tick(time, dtime)
                    weightsum += action[0]
                    actuators.append(action)
                else:
                    node.reset()
            if self.avoidEntity.active:
                action = self.avoidEntity.Tick(time, dtime)
                weightsum += action[0]
                actuators.append(action)
            #print "done tick"
            #print weightsum
            if weightsum:
                for action in actuators:
                    dx += float(action[0])/float(weightsum) * math.cos(action[1])
                    dy += float(action[0])/float(weightsum) * math.sin(action[1])
                    r += float(action[0])/float(weightsum) * action[2]
                theta = math.atan2(dy, dx)
        else:
            print "Got here!!!!"
            self.running = False
        self.INIT.reset()
        if exists(self.sens.GetOwnEntity()):
            sel(self.sens.GetOwnEntity())
            sf('desiredHeading', theta)
            sf('desiredSpeed', r)

    def RunNetwork(self, filepath):
        file = open(filepath)
        xml.sax.parse(file, self)
        file.close()
         
        customTriggers = {}
        self.avoidEntity = Behaviors.AvoidEntity(sens=self.sens)
        for trig in self.triggers:
            params = trig.parameters
            trigState = {'sensorType':trig.name, 'value':trig.value, 'type':trig.type}
            #check for parameter here
            tmpName = 'triggerParam_state_'+trig.id
            if self.extvars.has_key(tmpName):
                trigState['value'] = self.extvars[tmpName]
            fromPath = ''
            toPath = ''
            #print "((((((((((", trig.slots
            #print '********', self.slots
            for id, name in trig.slots:
               # print "@@@@@", "id:", id, ", name:", name,", extvars:", self.extvars 
                ent = {'name':name, 'value':self.slots[id], 'type':'ent'}
                if self.extvars.has_key(id):
                    ent['value'] = self.extvars[id]
                    if name == 'From':
                        fromPath = ent['value']
                    if name == 'To':
                        toPath = ent['value']
                else:
                    if name == 'From':
                        fromPath = self.slots[id]
                    if name == 'To':
                        toPath = self.slots[id]
                params.append(ent)
            fingerprint = (trigState, params)
            sensor = None
            if trigState['sensorType'] == 'Distance Between Entities':
                #print '^^^^^^^^^', fromPath, toPath
                sensor = EntityDistanceSensor(fromPath, toPath, calculateDistance=True)
            elif trigState['sensorType'] == 'Entities In Range':
                pass
            elif trigState['sensorType'] == 'startScenario':
                sensor = FlagSensor('startScenario', sensors.IsStartScenario)
            elif trigState['sensorType'] == 'targetQuery':
                sensor = FlagSensor('targetQuery', sensors.IsTargetQuery)
            customTriggers[trig.id] = (fingerprint, sensor)
        self.triggerMonitor = triggers.CustomTriggerMonitor(customTriggers)
        
        self.INIT = Behaviors.NetworkBehavior(self.triggerMonitor)
        self.INIT.id = "INIT"
        for beh in self.behaviors:
            node = Behaviors.NetworkBehavior(self.triggerMonitor)
            node.id = beh.id
            node.groupID = beh.group
            if beh.name == 'cat.Behaviors.approach':
                node.child = Behaviors.Approach(sens=self.sens)
            elif beh.name == 'cat.Behaviors.maintain':
                node.child = Behaviors.Maintain(sens=self.sens)
            elif beh.name == 'cat.Behaviors.retreat':
                node.child = Behaviors.Retreat(sens=self.sens)
            elif beh.name == 'cat.Behaviors.formate':
                node.child = Behaviors.Formate()
            elif beh.name == 'cat.Behaviors.fire':
                node.child = Behaviors.Fire(sens=self.sens)
            elif beh.name == 'cat.Behaviors.moveto':
                node.child = Behaviors.MoveTo(sens=self.sens)
            elif beh.name == 'cat.Behaviors.stop':
                node.child = Behaviors.Stop(sens=self.sens)

            for param in beh.parameters:
                if param.type == 'bool':
                    node.child.SetVar(param.name, param.val == 'True')
                elif param.type == 'float':
                    node.child.SetVar(param.name, float(param.val))
                elif param.type == 'int':
                    node.child.SetVar(param.name, int(param.val))
                elif param.type == 'vector':
                    #print param.val
                    if param.val.__class__ == tuple:
                        node.child.SetVar(param.name, param.val)
                    else:
                        node.child.SetVar(param.name, vector3(param.val[1:-1]))

            for sens in beh.sensors:
                if sens.type == 'pre':
                    node.child.preTriggers.append(sens.id)
                elif sens.type == 'post':
                    node.child.postTriggers.append(sens.id)

            self.INIT.addPrecondition(node, Behaviors.NetworkBehavior.ORDERING)
            self.network[node.id] = node

        for link in self.links:
            self.network[link.dest].addPrecondition(self.network[link.orig], link.type)
        self.running = True
       

    def startElement(self, name, attrs):
        if name in ['to', 'from']: #links
            self.inLinks = True
            self.lnk = self.link()
            self.lnk.dir = name
            type = attrs.get('type', '')
            if type == 'ordering':
                self.lnk.type = self.link.ORDERING
            elif type == 'enabling':
                self.lnk.type = self.link.ENABLING
            elif type == 'permanent':
                self.lnk.type = self.link.PERMANENT
            if name == 'to':
                self.lnk.orig = self.beh.id
            else:
                self.lnk.dest = self.beh.id
        elif name == 'behavior':
            self.beh = self.behavior()
            self.beh.name = attrs.get('name', '')
            self.beh.group = attrs.get('group', '')
            self.beh.id = attrs.get('id', '')
        elif name in ['float', 'int', 'bool', 'vector']: #parameters
            self.inParam = True
            self.param = self.parameter()
            self.param.type = name
            self.param.name = attrs.get('name', '')
        elif name == 'post':
            self.xmlsensor = self.sensor()
            self.xmlsensor.id = attrs.get('id', '')
            self.xmlsensor.type = 'post'
        elif name == 'pre':
            self.xmlsensor = self.sensor()
            self.xmlsensor.id = attrs.get('id', '')
            self.xmlsensor.type = 'pre'
        elif name == 'trigger':
            self.tmpTrigger = self.trigger()
            self.tmpTrigger.name = attrs.get('name', '')
            self.tmpTrigger.id = attrs.get('id', '')
        elif name == 'parameter':
            n = attrs.get('name', '')
            t = attrs.get('type', '')
            v = attrs.get('value', '')
            if t == 'float':
                v = float(v)
            elif t == 'int':
                v = int(v)
            elif t == 'bool':
                v = (v == 'True')
            if n == 'state':
                self.tmpTrigger.value = v
                self.tmpTrigger.type = t
            else:
                self.tmpTrigger.paramters.append({'name':n,'type':t,'value':v})
        elif name == 'slot':
            id = attrs.get('id', '')
            name = attrs.get('name','')
            self.tmpTrigger.slots.append((id, name))
        elif name == 'entity':
            #s = self.slot()
            path = attrs.get('name', '')
            binding = attrs.get('id', '')
            self.slots[binding] = path

    def endElement(self, name):
        if name == 'behavior':
            self.behaviors.append(self.beh)
        elif name in ['float', 'int', 'bool', 'vector']: #parameters
            extname = self.beh.id + '_' + self.param.name
            if self.extvars.has_key(extname):
                self.param.val = self.extvars[extname]
            self.inParam = False
            self.beh.parameters.append(self.param)
        elif name in ['to', 'from']: #links
            self.inLinks = False
            self.links.append(self.lnk)
        elif name == 'trigger':
            self.triggers.append(self.tmpTrigger)
        elif name in ['post', 'pre']:
            self.beh.sensors.append(self.xmlsensor)
        
    def characters(self, buf):
        if self.inParam:
            self.param.val += buf.strip()
        elif self.inLinks:
            if self.lnk.dir == 'to':
                self.lnk.dest += buf.strip()
            else:
                self.lnk.orig += buf.strip()

    class behavior:
        def __init__(self):
            self.name = ''
            self.id = ''
            self.group = ''
            self.sensors = []
            self.parameters = []
            self.links = []

    class parameter:
        def __init__(self):
            self.name = ''
            self.type = ''
            self.val = ''

    class sensor:
        def __init__(self):
            self.id = ''
            self.type = ''

    class link:
        ORDERING = Behaviors.NetworkBehavior.ORDERING
        ENABLING = Behaviors.NetworkBehavior.ENABLING
        PERMANENT = Behaviors.NetworkBehavior.PERMANENT
        def __init__(self):
            self.dir = ''
            self.type = ''
            self.dest = ''
            self.orig = ''
    
    class trigger:
        def __init__(self):
            self.name = ''
            self.id = ''
            self.value = ''
            self.type = ''
            self.parameters = []
            self.slots = []
