#
# @(#) catElementaryPanel.py 2005/12/6
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
"""
Button panel to switch between authoring and execution mode.
"""
import wx
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'cat'])

import nEventServer
from common import EventNames
from cat import panelInfo


AUTHORING = 1
EXECUTION = 2

AUTHACTIVELABEL = " Authoring "
AUTHLABEL = "<<<"
EXECACTIVELABEL = " Execution "
EXECLABEL = " >>> "

EXECCOLOR = "#f6f3a2"
AUTHCOLOR = "#71b4e8"

ACTIVECOLOR = "#5380ff"
INACTIVECOLOR = "#f0f904"
BUTTONSIZE = (107,30)


class CatButtonPanel(wx.Panel):
	
    ID_AUTHBUTTON = wx.NewId()
    ID_EXECBUTTON = wx.NewId()
    def __init__(self, parent, color="CYAN", size=None, entity = None):
	    
        wx.Panel.__init__(self, parent, -1, size = size, style = wx.RAISED_BORDER| wx.DOUBLE_BORDER)
        
	self.entity  = entity
        x, y = size

        self.SetBackgroundColour(color)
        self.parentPanel = parent
	
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
	
        # Buttons to switch between authoring and execution modes
        self.authButton = wx.Button(self, self.ID_AUTHBUTTON, AUTHACTIVELABEL, size = BUTTONSIZE)
	self.execButton = wx.Button(self, self.ID_EXECBUTTON, EXECLABEL, size = BUTTONSIZE )

        # Set the event handler for the two buttons that switch panel modes
        wx.EVT_BUTTON(self.authButton, self.ID_AUTHBUTTON, self.SwitchToAuthoring)
        wx.EVT_BUTTON(self.execButton, self.ID_EXECBUTTON, self.SwitchToExecution)

        self.authButton.SetBackgroundColour(AUTHCOLOR)
        # Disable button for active panel 
        self.execButton.SetBackgroundColour(EXECCOLOR)
        self.activePanel = AUTHORING 

        # Layout buttons horizontally
        self.buttonSizer.Add(self.authButton, 1, wx.EXPAND)
        self.buttonSizer.Add(self.execButton, 1, wx.EXPAND)

        # Layout button group
        self.sizer.Add(self.buttonSizer, 0, wx.EXPAND)
	
	self.SetSizer(self.sizer)


    def SetRecorderPanels(self, _recWrapper):
        '''
        Maintain a pointer to the recording panels in order to switch
        panels when buttons are clicked.
        Sushil: Could these be passed in during contstruction?
        '''
        self.recWrapper = _recWrapper


    def ShowRecordPanels(self):
        '''
        Show recording panel which was last active.
        Called by methods which want to show all panels associated with
        an entity.
        '''
        if self.activePanel == AUTHORING:
            self.recWrapper.show("Auth")
        else:
            self.recWrapper.show("Exec")

    def HideRecordPanels(self):
        '''
        Hide both recording panels.  Called by methods which want to hide
        all panels associated with an entity.
        '''
        self.recWrapper.hideAll()


    # Switches the panel to exec mode
    def SwitchToExecution(self,event):
        '''
        Switches panel mode to execution.
        '''
        self.activePanel = EXECUTION

        self.recWrapper.show("Exec")


        # Change label and color of active button
        self.authButton.SetLabel(AUTHLABEL)
        self.execButton.SetLabel(EXECACTIVELABEL)

        # Set entity mode to exec
        self.entity.setMode(panelInfo.EXECUTION_MODE)



    # Switches the panel to authoring mode
    def SwitchToAuthoring(self,event):
        '''
        Switches panel mode to authoring.
        '''
        self.activePanel = AUTHORING

        self.recWrapper.show("Auth")

        # Change label and color of active button
        self.authButton.SetLabel(AUTHACTIVELABEL)
        self.execButton.SetLabel(EXECLABEL)

        # Set entity mode to authoring
        self.entity.setMode(panelInfo.AUTHORING_MODE)
	
