#
# @(#) catEntityPanel.py 2005/10/02
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
'''
Defines the entity information display

'''
import wx
import wx.lib.fancytext as fancytext
import common.WindowManager as WindowManager
from common.defs import *
import xml.sax
from xml.sax.handler import ContentHandler
import knob
import math

from cat.Behaviors import catPrimitiveApproach as mapp
from common import tickmanager
from cat import behaviorInfo

import nEventServer


HEADING                    = "Heading"
CURRENTSPEED               = "Current"
DESIREDSPEED               = "Desired"

PANELSIZE                  = (215, 300)
TIME_STEP                  = 1
HEADINGSPEEDCONTROLSIZE   = (30, 20)
CURRENTSPEEDBARSIZE   = (30, 20)
SLIDERSIZE   = (15, 50)

class EntityPanel(wx.Panel):
	
    '''
    Entity panel class.  Contains the heading knob, the current speed display
    bar, and the current speed slider.
    '''
    
    def __init__(self, parent, size = None, color = "LIGHT BLUE", entity = None):
        if(size):
            self.size = size
        else:
            self.size = PANELSIZE
        
        wx.Panel.__init__(self, parent, -1, size=size, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)

        self.entity = entity;
	self.bgcolor = parent.GetBackgroundColour()
        self.SetBackgroundColour(self.bgcolor)

        self.funcname = 'EntityPanel:'+str(color)
        #tickmanager.register( self.funcname , self.Tick ) #

	spacer0 = wx.StaticText(self, -1, "", size= (  8, 110), style = wx.ALIGN_RIGHT) 
	spacer1 = wx.StaticText(self, -1, "", size= ( 32, 110), style = wx.ALIGN_BOTTOM)
	spacer2 = wx.StaticText(self, -1, "", size= (  5, 110), style = wx.ALIGN_BOTTOM)
	spacer3 = wx.StaticText(self, -1, "", size= (215,   2), style = wx.ALIGN_RIGHT)
	spacer4 = wx.StaticText(self, -1, "", size= (215,   2), style = wx.ALIGN_RIGHT)
	spacer5 = wx.StaticText(self, -1, "", size= (215,   2), style = wx.ALIGN_RIGHT)
	
	vert1  = wx.BoxSizer(wx.VERTICAL)
       	vert2  = wx.BoxSizer(wx.VERTICAL)
      	vert3  = wx.BoxSizer(wx.VERTICAL)
	
	gap0   = wx.BoxSizer(wx.VERTICAL)
      	gap1   = wx.BoxSizer(wx.VERTICAL)
      	gap2   = wx.BoxSizer(wx.VERTICAL)
	
	horz1  = wx.BoxSizer(wx.HORIZONTAL)
   	
        #################### info: current entity and position #################################

        if(self.entity):
            text = "ENTITY:          "

	l1 	= wx.StaticText (self, -1,text)
	self.t1 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l2 	= wx.StaticText (self, -1," Position (X,Y):   ")
	self.t2 = wx.TextCtrl (self, -1, "00000, 00000", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	space 	= 2
	sizer 	= wx.FlexGridSizer (cols = 2, hgap = space, vgap = space)
	sizer.AddMany ([l1, self.t1, l2, self.t2])
	
	####################### heading knob ################################################### 
	
        # useless
	color1 	       = wx.Colour(21, 86, 21)
	self.headingKnob    	= knob.Knob(self, -1)

	vert1.Add(self.headingKnob, 1,flag=wx.EXPAND|wx.CENTER)
	vert1.Add(wx.StaticText(self, -1, HEADING), flag=wx.CENTER)
        # Event handler for heading change
	self.Bind(knob.EVT_KNOB, self.OnHeadingKnobChange, id=self.headingKnob.GetId())
       
	####################### current speed gauge ###########################################
	
	self.currentSpeedBarText   = wx.StaticText(self, -1, "max", CURRENTSPEEDBARSIZE)
        self.currentSpeedBar       = wx.Gauge(self, -1, 10, (-1, -1), (7, 50), style=wx.GA_VERTICAL|wx.GA_SMOOTH)

	vert2.Add(self.currentSpeedBarText, 0, border = 0,    flag = wx.CENTER)
	vert2.Add(self.currentSpeedBar,     1, border = 0,    flag = wx.CENTER)
	vert2.Add(wx.StaticText(self, -1, CURRENTSPEED),      flag = wx.CENTER)
	
	
	####################### desired speed gauge ###########################################
	
	self.desiredSpeedCtrl 	= wx.StaticText(self, -1, "max", size=HEADINGSPEEDCONTROLSIZE)
        # The inverse flag makes the bottom, instead of the top, of the slider the lowest value
        self.desiredSpeedSlider = wx.Slider(self, -1, 20, 0, size=SLIDERSIZE, style=wx.SL_VERTICAL|wx.SL_AUTOTICKS|wx.SL_RIGHT|wx.SL_INVERSE)
	wx.EVT_SCROLL(self.desiredSpeedSlider, self.OnSpeedSliderChange)

	vert3.Add(self.desiredSpeedCtrl, 0, border = 0,    flag = wx.CENTER)
	vert3.Add(self.desiredSpeedSlider,     1, border = 0,    flag = wx.CENTER)
	vert3.Add(wx.StaticText(self, -1, DESIREDSPEED),      flag = wx.CENTER)

        ########## set max speed and range for current speed slider
        entPath = self.entity.getEVAPath()
        pushsel(entPath)

        self.maxSpeed = gf('maxSpeed') * behaviorInfo.METERS_PER_SEC_TO_KNOTS *100
        self.desiredSpeedSlider.SetRange(0, int(self.maxSpeed))
        self.desiredSpeedSlider.SetValue(0)
        self.desiredSpeed = gf('desiredSpeed')
        sp = "%5.1f" % self.desiredSpeed
        self.desiredSpeedCtrl.SetLabel('0.0')

        popsel()
	
	####################### the three controls together ##################################################
	
	gap0.Add(spacer0)
	gap1.Add(spacer1)	
	gap2.Add(spacer2)
	
	horz1.Add(gap0, 0, flag=wx.EXPAND)
	horz1.Add(vert1, 1, flag=wx.EXPAND)
        horz1.Add(gap1,  0, flag=wx.EXPAND)
	horz1.Add(vert2, 0, flag=wx.EXPAND)
	horz1.Add(gap2,  0, flag=wx.EXPAND)
	horz1.Add(vert3, 0, flag=wx.EXPAND)
	
	sizerEntity = wx.BoxSizer(wx.VERTICAL)
	
	sizerEntity.Add(spacer3)
	sizerEntity.Add(sizer)
	sizerEntity.Add(spacer4)
        sizerEntity.Add(horz1,1, flag=wx.EXPAND)
	sizerEntity.Add(spacer5, 0)
	self.SetSizer(sizerEntity)
        self.oldTime = 0
	
    def getPanel(self):
        return self

    def Show(self):
        wx.Panel.Show(self)
        tickmanager.register( self.funcname , self.Tick ) #

    def Hide(self):
        wx.Panel.Hide(self)
        tim = tickmanager.unregister( self.funcname , self.Tick ) #


    def Tick(self, time, dtime):
        '''
        Each time tick update current speed display bar.
        '''
        if((not (time - self.oldTime > TIME_STEP)) or (not self.IsShown())):
            return
        self.oldTime = time
        evaPath = self.entity.getEVAPath()

        pushsel(evaPath) #-----------------------------------------
        entName = evaPath.split('/')[-1]
        self.t1.SetValue(entName)
        ox, oz, oy = gv('pos')
        self.t2.SetValue(`int(ox)`+', '+`int(oy)`)
        # get speed only once! Sushil
        cs 	= gf("speed") * behaviorInfo.METERS_PER_SEC_TO_KNOTS * 100
        rad 	= gf("heading")
        drad    = gf("desiredHeading")
        val 	= 360 - math.degrees(rad)
        if val < 0:
            val += 360.0
        elif val >= 360.0:
            val -= 360.0
        h = '%.1f' % val
            
        ds	= gf("desiredSpeed")
        popsel()        #-----------------------------------------

        self.currentSpeedBar.SetRange(int(self.maxSpeed))
        self.currentSpeedBar.SetValue(int(cs))

        # If the ship's desired speed was changed by an outside function,
        # such as a behavior, then update the current speed slider to show
        # actual desired speed
        if abs(self.desiredSpeed - ds) > 0.1 :
            # get actual desired speed
            
            ds 	= ds * behaviorInfo.METERS_PER_SEC_TO_KNOTS * 100
            # update slider
            self.desiredSpeedSlider.SetRange(0, int(self.maxSpeed))
            self.desiredSpeedSlider.SetValue(int(ds))
            ds /= 100.0
            sp = "%5.1f" % ds
            # update label
            self.desiredSpeedCtrl.SetLabel(sp)
            self.desiredSpeed = ds * behaviorInfo.KNOTS_TO_METERS_PER_SEC

    
        # Current speed bar update
        sc = '%5.1f' % (cs / 100)
        self.currentSpeedBarText.SetLabel(sc)
        self.headingKnob.SetValue(drad)
        self.headingKnob.SetRealAngle(rad)



###########################################################
# Juan
    def OnHeadingKnobChange(self, event):
        '''
        Handles updating the wxTextCtrl to display the desired heading when set by
        knob.

        Don\'t call this function. It is for internal use only.
        '''
        if(self.entity.isAnyBehaviorActive()): # if any behavior active, do not function
            self.desiredHeadingKnob.SetRealAngle(0)
            event.SetAngle(0)
            return
        self.entity.actuators.setDCHeading(event.GetAngle())

    def OnSpeedSliderChange(self, event):
        """
        Handles updating the wxTextCtrl to display the desired speed when set by the
        slider.

        Don't call this fuction. It is for internal use only.
        """
        if(self.entity.isAnyBehaviorActive()): # If any behavior active, do not function
            self.desiredSpeedCtrl.SetLabel("0")
            self.desiredSpeedSlider.SetValue(int(self.maxSpeed))            
            return
        # Get speed from slider and set entity's speed to the desired speed
        speed = (self.desiredSpeedSlider.GetValue())/100.0
        sp = "%5.1f" % speed
        self.desiredSpeed  = speed * behaviorInfo.KNOTS_TO_METERS_PER_SEC
        self.desiredSpeedCtrl.SetLabel(sp)
        self.entity.actuators.setDCSpeed(self.desiredSpeed)
        # Keep desired speed as variable to check if behaviors change desired speed


###########################################################
