#
# @(#) catSelectorPanel 2005/12/7
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

"""
Cat Selector Panel

Authoring and Execution UI for approach
"""
import wx
import catElementaryPanel
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'cat'])

import nEventServer
from common import EventNames
from cat.Behaviors import catPrimitiveApproach as mapp

COMBOBOXSIZE=(125,25)
TEXTSIZE    =(80, 30)

class CatSelectorPanel(catElementaryPanel.CatElementaryPanel):
    def __init__(self, parent, panelID, color="CYAN", size=None, behaviorType = None, entity = None, behavior = None):
        catElementaryPanel.CatElementaryPanel.__init__(self, parent, panelID, color, size, behaviorType, entity, behavior)
        
	self.maneuvers = ['Maintain speed', 'Maintain time']
        self.maneuverTypeSelected = False                
        self.maneuverTypeList = wx.ComboBox(self,-1,value=self.maneuvers[0],size=COMBOBOXSIZE,choices=self.maneuvers,style=wx.CB_DROPDOWN|wx.CB_READONLY|wx.CENTER)
        self.maneuverTypeList.SetStringSelection(self.maneuvers[0])
        wx.EVT_COMBOBOX(self.maneuverTypeList,-1,self.OnManeuverTypeSelect)
        spacer =wx.StaticText(self, -1, "Beh", size=(185,2), style=wx.ALIGN_RIGHT)
        self.hsizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hsizer.Add(wx.StaticText(self, -1, self.behaviorType[0:8] + "  ", size=TEXTSIZE, style=wx.EXPAND|wx.CENTER))
        self.hsizer.Add(self.maneuverTypeList)
        #self.sizer.Add(self.maneuverTypeList)
        self.sizer.Add(self.hsizer)
        self.sizer.Add(spacer)
        self.timeVal = mapp.DEFAULT_TIME_MANEUVER
        self.speedVal = mapp.MIN_SPEED_SLIDER

        # Initialize maneuver slider
	
	self.selector = wx.StaticText(self, -1, "Selector")
	self.sizer.Add(self.selector)

	self.labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        #pushsel(self.entity.getEVAPath())
        from cat import behaviorInfo
        ms = self.entity.getMaxSpeed() * behaviorInfo.METERS_PER_SEC_TO_KNOTS
        self.speedVal = ms/2
        #popsel()
#	self.maneuverValueSlider = wx.Slider(self, -1, mapp.DEFAULT_SPEED_MANEUVER, mapp.MIN_SPEED_SLIDER, mapp.MAX_SPEED_SLIDER, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
	self.maneuverValueSlider = wx.Slider(self, -1, ms/2, mapp.MIN_SPEED_SLIDER, ms, style=wx.SL_LABELS|wx.SL_HORIZONTAL)        
        self.labelSizer.Add(self.maneuverValueSlider, 1, border=3, flag=wx.RIGHT|wx.EXPAND)
        self.unitsLabel = wx.StaticText(self, -1, 'knots')
        self.labelSizer.Add(self.unitsLabel, flag=wx.ALIGN_CENTER)
        wx.EVT_SLIDER(self.maneuverValueSlider, -1, self.OnManeuverValueChange)
        self.sizer.Add(self.labelSizer, 1, flag=wx.EXPAND|wx.RIGHT)
		
    def stop(self):
        print "Stopping panel"
	self.WriteNotifMsg ("Stopping panel")
	    
    def OnManeuverTypeSelect(self, event):
        #print "Maneuver Type selection: ", event
        behavior = self.behavior
        if (not self.maneuverTypeSelected):
            if self.maneuverTypeList.GetValue() != 'Select Type...':
                select = self.maneuverTypeList.GetStringSelection()
                if select == 'Maintain speed':
                    print('Maintain speed maneuver selected')
		    self.selector.SetLabel("Speed")
                    if behavior:
                        behavior.SetVar('maneuverType', mapp.SPEED_CONST_MANEUVER)
                        print "CatApproachPanel: Max Speed of %s is %s" % (self.entity.getEVAPath(), behavior.GetVar('entMaxSpeed'))
                        self.maneuverValueSlider.SetRange(mapp.MIN_SPEED_SLIDER, behavior.GetVar('entMaxSpeed'))
                        #self.maneuverValueSlider.SetValue(int(behavior.GetVar('entMaxSpeed')/2))
                        self.maneuverValueSlider.SetValue(self.speedVal)
                    self.unitsLabel.SetLabel('knots')

                elif select == 'Maintain time':
                    print('Maintain time maneuver selected')
		    self.selector.SetLabel("Time")
                    if behavior:
                        #print "OnManeuverTypeSelect: Time"
                        behavior.SetVar('maneuverType', mapp.TIME_CONST_MANEUVER)
                        self.maneuverValueSlider.SetRange(mapp.MIN_TIME_SLIDER, mapp.MAX_TIME_SLIDER)
                        #self.maneuverValueSlider.SetValue(mapp.DEFAULT_TIME_MANEUVER)
                        self.maneuverValueSlider.SetValue(self.timeVal)
                    self.unitsLabel.SetLabel('min')
                self.OnManeuverValueChange(event)
                self.maneuverTypeSelected = True
        else:
            self.maneuverTypeSelected = False
	    self.selector.SetLabel("Selector")
            self.OnManeuverTypeSelect(wx.EVT_COMBOBOX)
	    
    def OnManeuverValueChange(self, event):
        """
        Called when the maneuver value (time, speed, course) change
        """
        behavior = self.behavior
        if(behavior):
            behavior.SetVar('maneuverChanged', True)
            behavior.SetVar('reachedStation', False)
            behavior.SetVar('maneuverValue', int(self.maneuverValueSlider.GetValue()))
            #if not self.maneuverTypeSelected:
            #   if self.maneuverTypeList.GetValue() != 'Select Type...':
            select = self.maneuverTypeList.GetStringSelection()
            if select == 'Maintain speed':
                self.speedVal = int(self.maneuverValueSlider.GetValue())
            elif select == 'Maintain time':
                self.timeVal = int(self.maneuverValueSlider.GetValue())
#            print "Maneuver value", str(self.maneuverValueSlider.GetValue())

        else:
            print "No attached mapproach behavior to maneuver on"
            self.WriteNotifMsg ("no att mappr to maneuver")

