#
# @(#) executive.py 2005/07/06
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
"""
An incompletely implemented knob widget for rotary control.

This widget is missing many of the functions neccessary to be a complete
widget. High on the list is a way of disabling the widget. Also, it would look
a lot better.

There are two things in this file. There is a wx event to bind functions to
knob changes. And then there is the actual knob.
"""
import wx
import math

wxEVT_COMMAND_KNOB = wx.NewEventType()

class KnobEvent(wx.PyCommandEvent):
    "Event sent from the generic buttons when the button is activated."
    def __init__(self, id, angle):
        "Creates the KnobEvent"
        wx.PyCommandEvent.__init__(self, id = id)
        self.SetEventType(wxEVT_COMMAND_KNOB)
        self.angle = angle

    def SetAngle(self, ang):
        "Sets the angle of this event. Probably should override SetValue"
        self.angle = ang
        
    def GetAngle(self):
        "Gets the angle of this event. Probably should override GetValue"
        return self.angle

EVT_KNOB = wx.PyEventBinder(wxEVT_COMMAND_KNOB, 1)

class Knob(wx.PyControl):
    """
    A simple Knob Control.
    """
    def __init__(self, parent, ID,
                 pos = wx.DefaultPosition, size = wx.DefaultSize,
                 style = 0, validator = wx.DefaultValidator,
                 name = "knob"):
        "Creates the knob."
        cstyle = style
        if cstyle == 0:
            cstyle = wx.BORDER_NONE
        wx.Control.__init__(self, parent, ID, pos, size, cstyle, validator, name)
        self.angle = 0
        self.realangle = 0
        self.up = False
        self.hasFocus = False    
        self.da = True
        self.dd = True
        self.style = style
        self.circlecolor = self.GetParent().GetBackgroundColour()
        self.bgcolor= self.GetParent().GetBackgroundColour()

        self.rapen = wx.Pen(wx.BLACK, 3, wx.SOLID)
        self.dapen = wx.Pen(wx.RED, 1, wx.SOLID)
        self.ratext = wx.BLACK
        self.datext = wx.RED
        self.cirpen = wx.Pen(wx.BLACK, 1, wx.SOLID)
        self.bgbrush = wx.Brush(self.bgcolor, wx.SOLID)
        self.cirbrush = wx.Brush(self.circlecolor, wx.SOLID)
        
        self.Bind(wx.EVT_PAINT,            self.OnPaint)
        self.Bind(wx.EVT_LEFT_DOWN,        self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP,          self.OnLeftUp)
        self.Bind(wx.EVT_MOTION,           self.OnMotion)
    
    def Notify(self):
        """
        Sends a new event to the wx event.

        This is activated whenever a new value is found.
        """  
        #evt = KnobEvent(self.angle)
        #self.GetEventHandler().ProcessEvent(evt)
        #self.GetEventHandler().ProcessEvent(evt)
        wx.PostEvent(self, KnobEvent(self.GetId(), self.angle))
    
    def OnPaint(self, event):
        """
        Repaints the knob.
        """
        #print "OnPaint called"
        
        dc = wx.BufferedPaintDC(self)

        dc.SetBackground(self.bgbrush)
        dc.Clear()
        
        (width, height) = self.GetClientSizeTuple()

        x1 = y1 = 0
        x2 = width-1
        y2 = height-1
        cx = (x1+x2)/2
        cy = (y1+y2)/2

        if(cy < cx):
            y2 = height-1-20
            cy = (y1+y2)/2
            r = cy
        else:
            r = cx

        cy = height - r - 1

        hx = cx + r * math.sin(self.angle + math.pi)
        hy = cy + r * math.cos(self.angle + math.pi)
        ax = cx + r * math.sin(self.realangle + math.pi)
        ay = cy + r * math.cos(self.realangle + math.pi)

        rr = math.degrees(self.realangle)
        if rr < 0:
            rr += 360
        dr = math.degrees(self.angle)
        if dr < 0:
            dr += 360
        
        realstr = '%d' % rr
        desstr =  '%d' % dr
        
        dc.SetBrush(self.cirbrush)
        dc.SetPen(self.cirpen)
        dc.DrawCircle(cx, cy, r)
        if self.da:
            dc.SetPen(self.rapen)
            dc.DrawLine(cx, cy, ax, ay)
            dc.SetTextForeground(self.ratext)
            dc.DrawText(realstr, 1, -5)
        if self.dd:
            dc.SetPen(self.dapen)
            dc.DrawLine(cx, cy, hx, hy)
            dc.SetTextForeground(self.datext)
            dc.DrawText(desstr, width-30, -5)

        
    def FindAngle(self, event):
        """
        Uses the center point of the knob and the point clicked and finds the angle
        between the two.
        """
        (width, height) = self.GetClientSizeTuple()
        x1 = y1 = 0
        x2 = width-1
        y2 = height-1
        cx = (x1+x2)/2
        cy = (y1+y2)/2
        (hx, hy) = event.GetPositionTuple()
        self.angle = math.atan2(hx-cx, hy-cy) - math.pi
        
    def OnLeftDown(self, event):
        """
        Sets up the knob to receive input from the mouse.

        I'm not completely sure what this all does. I copied most of it
        from and example control and it works.
        """
        if self.IsEnabled():
            self.FindAngle(event)
            self.up = False
            self.CaptureMouse()
            self.SetFocus()
            self.Refresh()
            event.Skip()

    def OnLeftUp(self, event):
        """
        Called when the left mouse button is released.

        Only if the mouse is released inside of the control is the
        angle updated. The mouse is also released.
        """
        if not self.IsEnabled() or not self.HasCapture():
            return
        if self.HasCapture():
            self.ReleaseMouse()
            if not self.up:    # if the button was down when the mouse was released...
                self.Notify()
            self.up = True
            x,y = event.GetPositionTuple()
            w,h = self.GetClientSizeTuple()
            if x<w and x>=0 and y<h and y>=0:
                self.FindAngle(event)
            self.Refresh()
            event.Skip()

    def OnMotion(self, event):
        """
        Updates the angle of the knob when the mouse moves in the control.

        The angle is only updated if the left button was pressed and held
        inside of the control and the motion is within the bounds of the control.
        """
        if not self.IsEnabled() or not self.HasCapture():
            return
        if event.LeftIsDown() and self.HasCapture():
            x,y = event.GetPositionTuple()
            w,h = self.GetClientSizeTuple()
            if not self.up and x<w and x>=0 and y<h and y>=0:
                self.FindAngle(event)
                #self.up = False
                self.Refresh()
                self.Notify()
                return
            if self.up and (x<0 or y<0 or x>=w or y>=h):
                #self.GetAngle(event)
                #self.up = True
                self.Refresh()
                return
        event.Skip()
    
    def GetValue(self):
        """
        Gets the value of the angle in radians.
        """
        return self.angle

    def SetValue(self, angle):
        self.angle = angle
        self.Refresh()

    def SetRealAngle(self, angle):
        self.realangle = angle
        self.Refresh()

    def SetDrawRealAngle(self, da):
        """
        Draws the real angle with a black needle.
        """
        self.da = da
        self.Refresh()

    def SetDrawDesiredAngle(self, dd):
        self.dd = dd
        self.Refresh()

    def SetRealAngleColor(self, color):
        self.rapen = wx.Pen(color, 3, wx.SOLID)

    def SetDesiredAngleColor(self, color):
        self.dapen = wx.Pen(color, 1, wx.SOLID)

    def SetRealAngleTextColor(self, color):
        self.ratext = color

    def SetDesiredAngleTextColor(self, color):
        self.datext = color

    def SetCirlceColor(self, color):
        self.cirbrushx = wx.Brush(color, wx.SOLID)
