#
# @(#) catApproachPanel 2005/10/17
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

"""
Cat Approach panel

Authoring and Execution UI for approach
"""
import wx

from common.defs import *
from common.log import Log
log = Log(tags=['all', 'cat'])

import nEventServer
from common import EventNames
from cat.Behaviors import catPrimitiveApproach as mapp

class CatApproachPanel(wx.Panel):
    """
    Sets up the GUI for changing the parameters of approach.
    """
    def __init__(self, parent, panelID, color="CYAN", size=None, behaviorType = None, entity = None, behavior = None):
        """
        Creates the new panel.
        parentPanel (wx.Panel): Approach, Retreat, and Maintain are mutually
        exclusive. These behaviors are put under one tab and they need this
        parent tab to change its title.
        """
        wx.Panel.__init__(self, parent, panelID, size = size, style = wx.RAISED_BORDER| wx.DOUBLE_BORDER)

#        print "A/M Panel Being Created for " + entity.getName()
        self.bearingSlider = None
        self.rangeSlider   = None
        self.rb            = None

        self.entity = entity
        self.behavior = behavior
        self.bname = behavior.getName()
#        self.SetBackgroundColour(parent.GetBackgroundColour())
        self.parentPanel = parent
        self.sizer = wx.BoxSizer(wx.VERTICAL)

        self.statusMessage = wx.StaticText(self, -1, self.bname, size = (200, -1))
        self.sizer.Add(self.statusMessage, 0, wx.ALIGN_LEFT)

        # Initialize maneuver selection combobox
        self.maneuvers = ['Maintain speed', 'Maintain time']
        self.maneuverTypeSelected = False                
        self.maneuverTypeList = wx.ComboBox(self,-1,value=self.maneuvers[0],size=(175,25),choices=self.maneuvers,style=wx.CB_DROPDOWN|wx.CB_READONLY)
        self.maneuverTypeList.SetStringSelection(self.maneuvers[0])
        wx.EVT_COMBOBOX(self.maneuverTypeList,-1,self.OnManeuverTypeSelect)
        spacer =wx.StaticText(self, -1, "", size=(195,5), style=wx.ALIGN_RIGHT)
        self.sizer.Add(self.maneuverTypeList)
        self.sizer.Add(spacer)
        self.timeVal = mapp.DEFAULT_TIME_MANEUVER
        self.speedVal = mapp.MIN_SPEED_SLIDER

        # Initialize maneuver slider
	

	self.selector = wx.StaticText(self, -1, "Selector")
	self.sizer.Add(self.selector)

	self.labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        #pushsel(self.entity.getEVAPath())
        ms = self.entity.getMaxSpeed() * mapp.YDS_PER_SEC_TO_KNOTS
        self.speedVal = ms/2
        #popsel()
#	self.maneuverValueSlider = wx.Slider(self, -1, mapp.DEFAULT_SPEED_MANEUVER, mapp.MIN_SPEED_SLIDER, mapp.MAX_SPEED_SLIDER, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
	self.maneuverValueSlider = wx.Slider(self, -1, ms/2, mapp.MIN_SPEED_SLIDER, ms, style=wx.SL_LABELS|wx.SL_HORIZONTAL)        
        self.labelSizer.Add(self.maneuverValueSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        self.unitsLabel = wx.StaticText(self, -1, 'knots')
        self.labelSizer.Add(self.unitsLabel, flag=wx.ALIGN_CENTER)
        wx.EVT_SLIDER(self.maneuverValueSlider, -1, self.OnManeuverValueChange)
        self.sizer.Add(self.labelSizer, 1, flag=wx.EXPAND|wx.RIGHT)
        #self.sizer.Add(wx.StaticText(self, -1, "New station"))

        # Initialize range slider
        bearingSizer = wx.BoxSizer(wx.HORIZONTAL)

        from common import behaviorNames
        if(not (self.bname.startswith(behaviorNames.RAM) or self.bname.startswith(behaviorNames.MOVETO))):
            self.sizer.Add(wx.StaticText(self, -1, "New station range"))
            
            self.labelSizer = wx.BoxSizer(wx.HORIZONTAL)
            self.rangeSlider = wx.Slider(self, -1, mapp.DEFAULT_RANGE_SLIDER, mapp.MIN_RANGE_SLIDER, mapp.MAX_RANGE_SLIDER, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
            self.labelSizer.Add(self.rangeSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
            self.labelSizer.Add(wx.StaticText(self, -1, 'yards'))
            self.sizer.Add(self.labelSizer, 1, flag=wx.EXPAND)
            wx.EVT_SLIDER(self.rangeSlider, -1, self.OnRangeChange)


        # Initialize bearing slider
            self.sizer.Add(wx.StaticText(self, -1, "New station bearing"))
            self.labelSizer = wx.BoxSizer(wx.HORIZONTAL)
            self.bearingSlider = wx.Slider(self, -1, mapp.DEFAULT_BEARING_SLIDER, mapp.MIN_BEARING_SLIDER, mapp.MAX_BEARING_SLIDER, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
            self.labelSizer.Add(self.bearingSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
            self.labelSizer.Add(wx.StaticText(self, -1, 'degs '))
            self.sizer.Add(self.labelSizer, 1, flag=wx.EXPAND)
            wx.EVT_SLIDER(self.bearingSlider, -1, self.OnBearingChange)


        #if (not (self.bname.startswith(behaviorNames.RAM) or self.bname.startswith(behaviorNames.MOVETO))):
            bearingOptions = ['Absolute', 'Relative']
            
            self.rb = wx.RadioBox(
                self, -1, "", wx.DefaultPosition, wx.DefaultSize,
                bearingOptions, 1, wx.RA_SPECIFY_COLS
                )
            
            self.Bind(wx.EVT_RADIOBOX, self.OnEvtRadioBox, self.rb)
            bearingSizer.Add(self.rb, 0)#, wx.ALL, 20)

        if(self.bname.startswith(behaviorNames.MOVETO)):
            desiredXLabel = wx.StaticText (self, -1," Move Coord X:    ")
            self.desiredX = wx.TextCtrl (self, -1, " ", size = (70, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
            desiredYLabel = wx.StaticText (self, -1," Move Coord Y:    ")
            self.desiredY = wx.TextCtrl (self, -1, " ", size = (70, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
            spacerE = wx.StaticText(self, -1, "", size= (215,   15), style = wx.ALIGN_RIGHT)
            self.sizer.Add(spacerE)
	
            space = 2
            desiredCoordSizer = wx.FlexGridSizer (rows = 2, cols = 2, hgap = space, vgap = space)
            desiredCoordSizer.AddMany ([desiredXLabel, self.desiredX, desiredYLabel, self.desiredY])
            self.sizer.Add(desiredCoordSizer)
           
	
	self.startStopButton = wx.Button(self, -1, "Start", style=wx.BU_EXACTFIT)
	#self.startStopButton.SetSize(wx.Size(self.startStopButton.GetDefaultSize().GetWidth()-5, self.startStopButton.GetDefaultSize().GetHeight()-5))
	self.startStopButtonState = "start"
        wx.EVT_BUTTON(self.startStopButton, -1, self.OnClick)
	
	
	spacerY =wx.StaticText(self, -1, "", size=(65, 50), style=wx.ALIGN_BOTTOM)
	bearingSizer.Add(spacerY)
	
	bearingSizer.Add(self.startStopButton, 0, wx.ALIGN_RIGHT|wx.ALIGN_BOTTOM)
        self.sizer.Add(bearingSizer)
	
	spacerX =wx.StaticText(self, -1, "", size=(195,5), style=wx.ALIGN_RIGHT)
	self.sizer.Add(spacerX)
	
	notifLabel = wx.StaticText (self, -1," Status: ")
	self.notifMsg = wx.StaticText (self, -1, "", size = (100, 20), style = wx.TE_READONLY | wx.TE_LEFT)
	space = 2
	notifSizer = wx.FlexGridSizer (cols = 2, hgap = space, vgap = space)
	notifSizer.AddMany ([notifLabel, self.notifMsg])
	self.sizer.Add(notifSizer)

	self.SetSizer(self.sizer)
        wx.EVT_SET_FOCUS(self, self.handleEvent)
        

    def WriteNotifMsg(self, msg):
        #self.notifMsg.SetLabel (msg)
        print str(msg)
	
    def OnEvtRadioBox(self, event):
	behavior = self.behavior
        if(behavior):
	    selection = self.rb.GetSelection()
	    if selection == 0:
	        behavior.SetVar('absoluteBearing', True)
	    else:
		behavior.SetVar('absoluteBearing', False)
		
	
    def stop(self):
        print "Stopping panel"
	self.WriteNotifMsg ("Stopping panel")
	    

    def OnManeuverTypeSelect(self, event):
        #print "Maneuver Type selection: ", event
        behavior = self.behavior
        if (not self.maneuverTypeSelected):
            if self.maneuverTypeList.GetValue() != 'Select Type...':
                select = self.maneuverTypeList.GetStringSelection()
                if select == 'Maintain speed':
                    print('Maintain speed maneuver selected')
		    self.selector.SetLabel("Speed")
                    if behavior:
                        behavior.SetVar('maneuverType', mapp.SPEED_CONST_MANEUVER)
                        print "CatApproachPanel: Max Speed of %s is %s" % (self.entity.getEVAPath(), behavior.GetVar('entMaxSpeed'))
                        self.maneuverValueSlider.SetRange(mapp.MIN_SPEED_SLIDER, behavior.GetVar('entMaxSpeed'))
                        #self.maneuverValueSlider.SetValue(int(behavior.GetVar('entMaxSpeed')/2))
                        self.maneuverValueSlider.SetValue(self.speedVal)
                    self.unitsLabel.SetLabel('knots')
                    
                #elif select == 'Maintain course':
                #    log.write('Maintain course maneuver selected')
		#    self.selector.SetLabel("Course")
                #    if behavior:
                #        behavior.SetVar('maneuverType', mapp.BEARING_CONST_MANEUVER)
                #        self.maneuverValueSlider.SetRange(mapp.MIN_BEARING_SLIDER, mapp.MAX_BEARING_SLIDER)
                #        self.maneuverValueSlider.SetValue(mapp.DEFAULT_BEARING_MANEUVER)
                #    self.unitsLabel.SetLabel('')
                elif select == 'Maintain time':
                    print('Maintain time maneuver selected')
		    self.selector.SetLabel("Time")
                    if behavior:
                        #print "OnManeuverTypeSelect: Time"
                        behavior.SetVar('maneuverType', mapp.TIME_CONST_MANEUVER)
                        self.maneuverValueSlider.SetRange(mapp.MIN_TIME_SLIDER, mapp.MAX_TIME_SLIDER)
                        #self.maneuverValueSlider.SetValue(mapp.DEFAULT_TIME_MANEUVER)
                        self.maneuverValueSlider.SetValue(self.timeVal)
                    self.unitsLabel.SetLabel('min')
                self.OnManeuverValueChange(event)
                self.maneuverTypeSelected = True
        else:
            self.maneuverTypeSelected = False
	    self.selector.SetLabel("Selector")
            self.OnManeuverTypeSelect(wx.EVT_COMBOBOX)
        

        
    def OnManeuverValueChange(self, event):
        """
        Called when the maneuver value (time, speed, course) change
        """
        behavior = self.behavior
        if(behavior):
            behavior.SetVar('maneuverChanged', True)
            behavior.SetVar('reachedStation', False)
            behavior.SetVar('maneuverValue', int(self.maneuverValueSlider.GetValue()))
            #if not self.maneuverTypeSelected:
            #   if self.maneuverTypeList.GetValue() != 'Select Type...':
            select = self.maneuverTypeList.GetStringSelection()
            if select == 'Maintain speed':
                self.speedVal = int(self.maneuverValueSlider.GetValue())
            elif select == 'Maintain time':
                self.timeVal = int(self.maneuverValueSlider.GetValue())
#            print "Maneuver value", str(self.maneuverValueSlider.GetValue())

        else:
            print "No attached mapproach behavior to maneuver on"
            self.WriteNotifMsg ("no att mappr to maneuver")
        
        
    def OnRangeChange(self, event):
         """
         Called on station range change
         """
         # print "On Range Change: ", event
         behavior = self.behavior
         if(behavior):
             behavior.SetVar('maneuverChanged', True)
             behavior.SetVar('reachedStation', False)
             behavior.SetVar('desiredStRange', self.rangeSlider.GetValue())            
             # print "Range value: ", str(self.rangeSlider.GetValue())
         else:
             print "No attached behavior to range change"
             self.WriteNotifMsg ("no att b to range chg")
             
    def OnBearingChange(self, event):
         """
         Called on station bearing change
         """
#         print "On station bearing Change: ", event
         behavior = self.behavior
         if(behavior):
             behavior.SetVar('maneuverChanged', True)
             behavior.SetVar('reachedStation', False)
             behavior.SetVar('desiredStBearing', self.bearingSlider.GetValue())
         else:
             print "No attached behavior to bearing change"
	     self.WriteNotifMsg ("no att b to bearing chg")
             
    def OnClick(self, event):
        """
        Called when the activate button is pressed.
        Turns off other behaviors and changes the parent tab's name.
        """
        if(self.behavior):
            self.behavior.handleStateSwitch()
            self.startStopButton.SetLabel(self.behavior.startStop)
        else:
            print "ApproachType: I am on a time out due to bad behavior"

    def SetStartStop(self):
        self.startStopButton.SetLabel(self.behavior.startStop)
#         if(self.maneuverValueSlider):
#             val = self.behavior.GetVar('maneuverValue')
#             print "Maneuver Val: " + str(val)
#             self.maneuverValueSlider.SetValue(val)
        

    def updatePanels(self):
 #       print "Handling Page change in " + self.behavior.getName()
        self.SetStartStop()
        if(self.maneuverTypeList):
            mtype = self.behavior.GetVar('maneuverType')
            if(mtype == mapp.SPEED_CONST_MANEUVER):
                self.maneuverTypeList.SetValue('Maintain speed')
            elif(mtype == mapp.TIME_CONST_MANEUVER):
                self.maneuverTypeList.SetValue('Maintain time')

        if(self.maneuverValueSlider):
            val = self.behavior.GetVar('maneuverValue')
#            print "Page Changed: Maneuver Val: " + str(val)
            self.maneuverValueSlider.SetValue(val)
        if(self.bearingSlider):
#            print "Changeing bearing slider"
            self.bearingSlider.SetValue(self.behavior.GetVar('desiredStBearing'))
        if(self.rangeSlider):
#            print "Changeing range slider"            
            self.rangeSlider.SetValue(self.behavior.GetVar('desiredStRange'))
        if(self.rb):
            if(self.behavior.GetVar('absoluteBearing')):
                val = 0
            else:
                val = 1
            self.rb.SetSelection(val)
        self.Layout()
        self.Validate()
        self.Show()

    def handleEvent(self, event):
#        print "Handling window focus Event"
        self.updatePanels()

    def handlePageChange(self, msg):
        self.updatePanels()
