#
# @(#) controlPanel.py 2005/09/27
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

'''
Defines the Control Panel in GUI CAT.

'''
import wx
import wx.lib.fancytext as fancytext
import common.WindowManager as WindowManager
from common.defs import *
import xml.sax
from xml.sax.handler import ContentHandler
import knob
import math
from cat.merge import xmlmerger
import manager
from important import *
import nEventServer
from sensors import *
import cat.Behaviors as Behaviors
from cat.BehaviorNetwork.recorder import Recorder

from cat.entity import EntityManager

from common import EventNames
import nEventServer
from controlwrapper import ControlWrapper
from cat.Behaviors import sapproach
from cat.Behaviors import mmaintain
from cat.Behaviors import mram
from cat.Behaviors import mmoveto
from cat.Behaviors import avoidland
from cat.Behaviors import avoidentity
from cat.Behaviors import fire

HITPOINTS      = "Damage: "
POSITION       = "Pos: "
HEADING        = "Head: "
SPEED          = "Speed: "
DESIREDHEADING = " DesiredHeading"
DESIREDSPEED   = " DesiredSpeed"
SELECTTARGET   = "Sel"
TARGET         = "Target"
BEARING        = "Bearing: "
RANGE          = " Range: "
MACHINEGUN     = "MG"
SELECTWEAPON   = "Fire"
WEAPON         = "Weapon"
FORMATE        = "Formate"
FIRE           = "Fire"
MAINTAIN       = "Maintain"
APPROACH       = "Approach"
RETREAT        = "Retreat"
HEADINGSPEEDCONTROLSIZE   = (50, 20)
BUTTON_PAD 	= 8

class ControlPanel(wx.Panel):
    
    '''Control panel class in CAT GUI'''
    
    def __init__(self, parent, size, color = "CYAN"):
        
        wx.Panel.__init__(self, parent, -1, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)
        
        nEventServer.register(EventNames.CAT_SELECT_ENT, self.refreshControls)
        nEventServer.register(EventNames.RMENU_BEHAVIOR_CHANGE, self.handleBehaviorChange)
        nEventServer.register(EventNames.CAT_ALL_BEHAVIORS_STOP, self.refreshControls)
        nEventServer.register(EventNames.CAT_UNSELECT_ENT, self.refreshControls)
   
	self.bgcolor = color
        self.SetBackgroundColour(self.bgcolor)
	
	labelControl  = wx.StaticText(self, -1, "CONTROL", size=(200,-1))
	
	spacer1 = wx.StaticText(self, -1, "", size= (10, 10), style = wx.ALIGN_RIGHT)
	spacer2 = wx.StaticText(self, -1, "", size= (195, 5), style = wx.ALIGN_RIGHT)
	spacer3 = wx.StaticText(self, -1, "", size= (215, 1), style = wx.ALIGN_RIGHT)
	spacer4 = wx.StaticText(self, -1, "", size= (215, 1), style = wx.ALIGN_RIGHT)
	spacer5 = wx.StaticText(self, -1, "", size= (195, 5), style = wx.ALIGN_RIGHT)
	
	self.controlWrapper = ControlWrapper(self, -1, self.GetBackgroundColour())
	
	import os	
        prefix = os.path.dirname(__file__)+"/"
	
	####################### create buttons to select behaviors ################## 
	
	#self.play_bmp = wx.Bitmap(prefix+"play.gif", wx.BITMAP_TYPE_GIF)
	
	bmpApproach 	= wx.Bitmap(prefix+"approach.gif", wx.BITMAP_TYPE_GIF)
        b0 		= wx.BitmapButton(self, 30, bmpApproach, (20, 20),
                       (bmpApproach.GetWidth()+BUTTON_PAD, bmpApproach.GetHeight()+BUTTON_PAD), name=sapproach.getBehaviorType())
        b0.SetToolTipString("Approach")
	wx.EVT_BUTTON(b0, -1, self.controlWrapper.showEvent)
	
        
	bmpMaintain 	= wx.Bitmap(prefix+"maintain.gif", wx.BITMAP_TYPE_GIF)
	#mask1 		= wx.Mask(bmpMaintain, "WHITE")
        #bmpMaintain.SetMask(mask1)

	b1 		= wx.BitmapButton(self, 30, bmpMaintain, (20, 20),
                       (bmpMaintain.GetWidth()+BUTTON_PAD, bmpMaintain.GetHeight()+BUTTON_PAD), name=mmaintain.getBehaviorType())
        b1.SetToolTipString("Maintain Station")
	wx.EVT_BUTTON(b1, -1, self.controlWrapper.showEvent)
	
	bmpRam 		= wx.Bitmap(prefix+"ram.gif", wx.BITMAP_TYPE_GIF)
	#mask2 		= wx.Mask(bmpRam, "GREEN")
        #bmpRam.SetMask(mask2)
	b2 		= wx.BitmapButton(self, 30, bmpRam, (20, 20),
                       (bmpRam.GetWidth()+BUTTON_PAD, bmpRam.GetHeight()+BUTTON_PAD), name=mram.getBehaviorType())
        b2.SetToolTipString("Ram")
     	wx.EVT_BUTTON(b2, -1, self.controlWrapper.showEvent)
	
	bmpMoveTo 	= wx.Bitmap(prefix+"moveto.gif", wx.BITMAP_TYPE_GIF)
	#mask3 		= wx.Mask(bmpMoveTo, "GREEN")
        #bmpMoveTo.SetMask(mask3)
        b3 		= wx.BitmapButton(self, 30, bmpMoveTo, (20, 20),
                       (bmpMoveTo.GetWidth()+BUTTON_PAD, bmpMoveTo.GetHeight()+BUTTON_PAD), name=mmoveto.getBehaviorType())
        b3.SetToolTipString("Move To")
     	wx.EVT_BUTTON(b3, -1, self.controlWrapper.showEvent)
	
	bmpAvoidLand 	= wx.Bitmap(prefix+"gray.gif", wx.BITMAP_TYPE_GIF)
	#mask4 = wx.Mask(bmpAvoidLand, "CADET BLUE")
        #bmpMoveTo.SetMask(mask4)
        b4 = wx.BitmapButton(self, 30, bmpAvoidLand, (20, 20),
                       (bmpAvoidLand.GetWidth()+BUTTON_PAD, bmpAvoidLand.GetHeight()+BUTTON_PAD), name=avoidland.behaviorType)                                     #crap hack - chris - go back to bmpAvoidLaned1
        b4.SetToolTipString("Avoid Land")
     	wx.EVT_BUTTON(b4, -1, self.controlWrapper.showEvent)

        bmpAvoidEntity  = wx.Bitmap(prefix+"gray.gif", wx.BITMAP_TYPE_GIF)
        #mask6           = wx.Mask(bmpAvoidEntity,"MAGENTA")
        #bmpAvoidEntity.SetMask(mask6)
        b6              = wx.BitmapButton(self,30, bmpAvoidEntity, (20,20),
                       (bmpAvoidEntity.GetWidth()+BUTTON_PAD, bmpAvoidEntity.GetHeight()+BUTTON_PAD),name=avoidentity.behaviorType)
        b6.SetToolTipString("Avoid Entities")
        wx.EVT_BUTTON(b6, -1, self.controlWrapper.showEvent)

        bmpFire         = wx.Bitmap(prefix+"fire.gif", wx.wx.BITMAP_TYPE_GIF)
        #mask7           = wx.Mask(bmpFire, "MAGENTA")
        #bmpAvoidEntity.SetMask(mask7)
        b7              = wx.BitmapButton(self,30, bmpFire, (20,20),
                       (bmpFire.GetWidth()+BUTTON_PAD, bmpFire.GetHeight()+BUTTON_PAD),name=fire.behaviorType)
        b7.SetToolTipString("Fire")
        wx.EVT_BUTTON(b7, -1, self.controlWrapper.showEvent)
	
	bmpStop 	= wx.Bitmap(prefix+"stopall.gif", wx.wx.BITMAP_TYPE_GIF)
	#mask5 		= wx.Mask(bmpStop, "RED")
        #bmpStop.SetMask(mask5)
	b5 		= wx.BitmapButton(self, 30, bmpStop, (20, 20),
                       (bmpStop.GetWidth()+BUTTON_PAD-3, bmpStop.GetHeight()+BUTTON_PAD-3), name="stop_all")
        b5.SetToolTipString("Stop all behaviors")
     	wx.EVT_BUTTON(b5, -1, self.controlWrapper.showEvent)
		
	############### place behavior control buttons on a bar #######################
		
	box 		= wx.StaticBox(self, -1, "", style=wx.SUNKEN_BORDER)
        bsizer 		= wx.StaticBoxSizer(box, wx.HORIZONTAL)
        bsizer.Add	(b0, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b1, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b2, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b3, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b4, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add      (b6, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add      (b7, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
	bsizer.Add	(spacer1, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b5, 0, wx.CENTER|wx.ALIGN_RIGHT, 1)
	
	border 		= wx.BoxSizer()
        border.Add	(bsizer, 1, wx.EXPAND|wx.ALL, 2)
 
    	############## add control wrapper panel to show selected behavior) ###########
	
	box1 		= wx.StaticBox(self, -1, "", style=wx.DOUBLE_BORDER|wx.SUNKEN_BORDER)
        bsizer1 	= wx.StaticBoxSizer(box, wx.VERTICAL)	
	bsizer1.Add	(self.controlWrapper, 1, wx.EXPAND)
	
        self.behaviorType = mmaintain.getBehaviorType()
	
	import 		approachPanelA
        import 		approachPanelB
	import 		ramPanel
	import 		movetoPanel
        import          firePanel
	
        defaultSize 	= (-1, -1) #(210, 350) # 
        
	######### 0. Approach #################


	approachPanel 	= approachPanelB.ApproachPanel(self.controlWrapper, -1, self.GetParent().GetBackgroundColour(), \
                                                      defaultSize, sapproach.getBehaviorType())
	self.controlWrapper.addPanel(approachPanel, sapproach.getBehaviorType())
	
	######### 1. Maintain #################
	
	maintainPanel 	= approachPanelA.ApproachPanelA(self.controlWrapper, -1, self.GetParent().GetBackgroundColour(), \
                                                      defaultSize, mmaintain.getBehaviorType())
	self.controlWrapper.addPanel(maintainPanel, mmaintain.getBehaviorType())
	
	######### 2. Ram ######################

        mramPanel 	= ramPanel.RamPanel(self.controlWrapper, -1, self.GetParent().GetBackgroundColour(), \
                                                      defaultSize, mram.getBehaviorType())
	self.controlWrapper.addPanel(mramPanel, mram.getBehaviorType())

       
	######### 3. MoveTo ###################

	mmovetoPanel 	= movetoPanel.MovetoPanel(self.controlWrapper, -1, self.GetParent().GetBackgroundColour(), \
                                                      defaultSize, mmoveto.getBehaviorType())
	self.controlWrapper.addPanel(mmovetoPanel, mmoveto.getBehaviorType())

         ######### 4. Avoid Land ###############
        #from cat.Behaviors import avoidland
        #self.behaviorType = avoidland.getBehaviorType()
 	avoidLandPanel = avoidland.AvoidLandPanel(self.controlWrapper,-1,self.GetParent().GetBackgroundColour(), \
                                                      defaultSize,avoidland.behaviorType)	
 	self.controlWrapper.addPanel(avoidLandPanel, avoidland.behaviorType)

        ######### 5. Avoid Entites ############
        avoidEntityPanel = avoidentity.AvoidEntityPanel(self.controlWrapper,-1,self.GetParent().GetBackgroundColour(), \
                                                      defaultSize,avoidentity.behaviorType)
        self.controlWrapper.addPanel(avoidEntityPanel,avoidentity.behaviorType)


        ######### 5. Fire Panel ############
        firePanel = firePanel.FirePanel(self.controlWrapper,-1,self.GetParent().GetBackgroundColour(), \
                                                      defaultSize, fire.behaviorType)
        self.controlWrapper.addPanel(firePanel, fire.behaviorType)


	######## add knob and slider to select desired heading and speed ##############
	
	vert1 = wx.BoxSizer(wx.VERTICAL)	### knob
       	vert2 = wx.BoxSizer(wx.VERTICAL)	### spacer
      	vert3 = wx.BoxSizer(wx.VERTICAL)	### slider
	
	
	horz1 = wx.BoxSizer(wx.HORIZONTAL)
   	
	######### knob
	
        #self.desiredHeadingCtrl = wx.StaticText(self, -1, "0.0", size=HEADINGSPEEDCONTROLSIZE,\
        #                                       style=wx.ALIGN_CENTER)
	#vert1.Add(self.desiredHeadingCtrl, border = 0, flag=wx.CENTER)
	self.desiredHeadingKnob = knob.Knob(self, -1)
        self.desiredHeadingKnob.SetDrawRealAngle(False)
	
	vert1.Add(self.desiredHeadingKnob, 1, flag=wx.EXPAND|wx.CENTER)
	vert1.Add(wx.StaticText(self, -1, DESIREDHEADING), border =2, flag=wx.CENTER)
	self.Bind(knob.EVT_KNOB, self.OnHeadingKnobChange, id=self.desiredHeadingKnob.GetId())
      
        ######### separator
	
	spacerS = wx.StaticText(self, -1, "", size= (15, 110), style = wx.ALIGN_BOTTOM)
	vert2.Add(spacerS)
	
	######### slider 
	
	self.desiredSpeedCtrl 	= wx.StaticText(self, -1, "max", size=HEADINGSPEEDCONTROLSIZE,\
                                              style = wx.ALIGN_CENTER)				      
	self.desiredSpeedSlider = wx.Slider(self, -1, 20, 0, 20, style = wx.SL_VERTICAL|wx.SL_AUTOTICKS|wx.SL_RIGHT)
	wx.EVT_SCROLL(self.desiredSpeedSlider, self.OnSpeedSliderChange)
	
	
	vert3.Add(self.desiredSpeedCtrl, 0, border=0, flag=wx.CENTER)
	vert3.Add(self.desiredSpeedSlider, 1, border=2, flag=wx.EXPAND|wx.CENTER)
	vert3.Add(wx.StaticText(self, -1, DESIREDSPEED), 0, border =0, flag=wx.CENTER)
	
	horz1.Add(vert1, 1, flag = wx.EXPAND)
        horz1.Add(vert2, 0, flag = wx.EXPAND)
	horz1.Add(vert3, 0, flag = wx.EXPAND)
        
	#########  putting all together ######################################################

	bsizer1.Add(spacer2)
	border1 = wx.BoxSizer()
        border1.Add(bsizer1, 1, wx.EXPAND, 5)
	
	sizerControl = wx.BoxSizer(wx.VERTICAL)
	sizerControl.Add(labelControl, 0, wx.ALIGN_LEFT)
	sizerControl.Add(spacer3)
	sizerControl.Add(border)
	sizerControl.Add(border1)
	sizerControl.Add(spacer4)
	sizerControl.Add(horz1)
	sizerControl.Add(spacer5)
	self.SetSizer(sizerControl)
	
	
    def handleBehaviorChange(self, msg):
        from common import defs
        from common import behaviorNames
        from cat.entity import EntityManager
        tmp = defs.parseRMenuMsg(msg)
        text = tmp.text.strip()
        #print "Handling behavior change: ", str(msg), text
        
	if (text.startswith(behaviorNames.MOVETO)):
            loc = tmp.mousePos
            EntityManager.currentState.sensors.setMoveLocation(loc)
            self.controlWrapper.show(mmoveto.getBehaviorType())
            return
        target = tmp.objectsOver[0]
        EntityManager.currentState.sensors.setTarget(target)
        if(text.startswith(behaviorNames.MAINTAIN)):
            self.controlWrapper.show(mmaintain.getBehaviorType())
        elif(text.startswith(behaviorNames.APPROACH)):
            self.controlWrapper.show(mmaintain.getBehaviorType())
        elif (text.startswith(behaviorNames.RAM)):
            self.controlWrapper.show(mram.getBehaviorType())            
        elif (text.startswith(behaviorNames.FIRE)):
            self.controlWrapper.show(mfire.getBehaviorType())            
        else:
            #print "No behavior panel found for:", target
            self.controlWrapper.hide()

    def getPanel(self):
        return self

    def refreshControls(self, msg):
        #print msg
        if EntityManager.currentState:
            #sel(EntityManager.currentState.sensors.GetOwnEntity())
            max = EntityManager.currentState.actuators.getMaxSpeed()*100#gf("maxSpeed") * 100
            ds  = EntityManager.currentState.actuators.getDesiredSpeed()*100#gf("desiredSpeed") * 100
            rad = EntityManager.currentState.actuators.getDesiredHeading()#gf('desiredHeading')
            #val = math.degrees(rad)
            #if val < 0:
            #    val += 360.0
            #sval = "%5.1f" % val
            #self.desiredHeadingCtrl.SetLabel(sval)
            self.desiredHeadingKnob.SetValue(rad)
            #if self.desiredSpeedSlider.GetMax() == 0:
            self.desiredSpeedSlider.SetRange(0, int(max))
            self.desiredSpeedSlider.SetValue(max-int(ds))
            sx = "%5.1f" % (ds/100)
            self.desiredSpeedCtrl.SetLabel(sx)
        else:
            #comes here when nothing is selected - should set default values for things
            #print "refresh controls error"
            pass
        if msg != 'behaviorStop':
            self.controlWrapper.refresh()
    
    def OnHeadingKnobChange(self, event):
        '''
        Handles updating the wxTextCtrl to display the desired heading when set by
        knob.

        Don\'t call this function. It is for internal use only.
        '''
        #angle = math.degrees(event.GetAngle())
        #if angle < 0:
        #    angle += 360
        #angle = int(angle)
        #angle = 360 - angle #Sushil
        #sangle = "%5.1f" % angle
	#self.desiredHeadingCtrl.SetLabel(sangle)

        if EntityManager.currentState and not EntityManager.currentState.actuators.areLocked():
            EntityManager.currentState.actuators.setDesiredHeading(event.GetAngle())
	#own = GetOwnEntity()
	#if exists(own):
	#    sel(own)
	#    sf('desiredHeading', event.GetAngle())
        
    def OnSpeedSliderChange(self, event):
        """
        Handles updating the wxTextCtrl to display the desired speed when set by the
        slider.

        Don't call this fuction. It is for internal use only.
        """
        if EntityManager.currentState and not EntityManager.currentState.actuators.areLocked():
            sel(EntityManager.currentState.sensors.GetOwnEntity())
            max = gf("maxSpeed")*100
            speed = (max - self.desiredSpeedSlider.GetValue())/100.0
	    sp = "%5.1f" % speed
            self.desiredSpeedCtrl.SetLabel(sp)
            EntityManager.currentState.actuators.setDesiredSpeed(speed)
        
        #if exists(GetOwnEntity()):
        #    sel(GetOwnEntity())
        #    max = gf('maxSpeed')*100
        #    speed = (max - self.desiredSpeedSlider.GetValue())/100.0
        #    self.desiredSpeedCtrl.SetLabel("%.1f"%speed)
	#    
	#    sf('desiredSpeed', speed)

    def tick(self, time, dtime):
        if EntityManager.currentState:
            if EntityManager.currentState.recorder.isActive() or \
               EntityManager.currentState.controller.isActive() or \
               EntityManager.currentState.actuators.areLocked():
                own = EntityManager.currentState.sensors.GetOwnEntity()
                sel(own)

                max = gf("maxSpeed") * 100
                ds = gf("desiredSpeed") * 100
                rad = gf('desiredHeading')
                #val = math.degrees(rad)
                #if val < 0:
                #    val += 360.0
                #s = "%5.1f"% val
                #self.desiredHeadingCtrl.SetLabel(s)
                self.desiredHeadingKnob.SetValue(rad)
                #if self.desiredSpeedSlider.GetMax() == 0:
                self.desiredSpeedSlider.SetRange(0, int(max))
                self.desiredSpeedSlider.SetValue(max-int(ds))
                sx = "%5.1f" % (ds/100)
                self.desiredSpeedCtrl.SetLabel(sx)
        else:
            #self.desiredHeadingCtrl.SetLabel ("0.0")
            self.desiredHeadingKnob.SetValue (0.0)
            self.desiredSpeedSlider.SetRange (0,10)
            self.desiredSpeedSlider.SetValue (10)
            self.desiredSpeedCtrl.SetLabel   ("0.0")

        
    
