#
# @(#) executive.py 2005/07/06
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
#TODO: Look into horizontal layout
#TODO: Set up recorder
#TODO: Tighten interface
'''
Defines the GUI and its operations for cat.

'''
import wx
import wx.lib.fancytext as fancytext
import common.WindowManager as WindowManager
from common.defs import *
import xml.sax
from xml.sax.handler import ContentHandler
import knob
import math
from cat.merge import xmlmerger
import manager
from important import *
import nEventServer
from sensors import *
import cat.Behaviors as Behaviors
from cat.BehaviorNetwork.recorder import Recorder
from cat.entity import EntityManager

HITPOINTS      = "Damage: "
POSITION       = "Pos: "
HEADING        = "Head: "
SPEED          = "Speed: "
HEADING        = " Heading"
CURRENTSPEED   = " CSpeed"
DESIREDSPEED   = " DSpeed"
SELECTTARGET   = "Sel"
TARGET         = "Target"
BEARING        = "Bearing: "
RANGE          = " Range: "
MACHINEGUN     = "MG"
SELECTWEAPON   = "Fire"
WEAPON         = "Weapon"
FORMATE        = "Formate"
FIRE           = "Fire"
MAINTAIN       = "Maintain"
APPROACH       = "Approach"
RETREAT        = "Retreat"
HEADINGSPEEDCONTROLSIZE    = (50, 20)

class EntityPanel(wx.Panel):
	
    '''Entity panel class'''
    
    def __init__(self, parent, size, color = "LIGHT BLUE"):
        
        wx.Panel.__init__(self, parent, -1, (215,500), style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)
        
	self.bgcolor = color
        self.SetBackgroundColour(self.bgcolor)

#	labelEntity = wx.StaticText(self, -1, "ENTITY")
	
	spacer0 = wx.StaticText(self, -1, "", size= (  8, 110), style = wx.ALIGN_RIGHT)
	spacer1 = wx.StaticText(self, -1, "", size= ( 32, 110), style = wx.ALIGN_BOTTOM)
	spacer2 = wx.StaticText(self, -1, "", size= (  5, 110), style = wx.ALIGN_BOTTOM)
	spacer3 = wx.StaticText(self, -1, "", size= (215,   2), style = wx.ALIGN_RIGHT)
	spacer4 = wx.StaticText(self, -1, "", size= (215,   2), style = wx.ALIGN_RIGHT)
	spacer5 = wx.StaticText(self, -1, "", size= (215,   2), style = wx.ALIGN_RIGHT)
	
	vert1  = wx.BoxSizer(wx.VERTICAL)
       	vert2  = wx.BoxSizer(wx.VERTICAL)
      	vert3  = wx.BoxSizer(wx.VERTICAL)
	
	gap0   = wx.BoxSizer(wx.VERTICAL)
      	gap1   = wx.BoxSizer(wx.VERTICAL)
      	gap2   = wx.BoxSizer(wx.VERTICAL)
	
	horz1  = wx.BoxSizer(wx.HORIZONTAL)
   	
        #################### info: current entity and position #################################
	 
	l1 	= wx.StaticText (self, -1,"ENTITY:          ")
	self.t1 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l2 	= wx.StaticText (self, -1," Position (X,Y):   ")
	self.t2 = wx.TextCtrl (self, -1, "00000, 00000", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	space 	= 2
	sizer 	= wx.FlexGridSizer (cols = 2, hgap = space, vgap = space)
	sizer.AddMany ([l1, self.t1, l2, self.t2])
	
	####################### heading knob ################################################### 
	
	color1 	       = wx.Colour(21, 86, 21)
	
	#self.headingText 	= wx.StaticText(self, -1, "0.0", size=HEADINGSPEEDCONTROLSIZE, style=wx.ALIGN_CENTER)
	self.headingKnob    	= knob.Knob(self, -1)
        self.headingKnob.Disable()
	#self.headingKnob.SetRealAngleColor("DARK GREEN")
        #self.headingKnob.SetDesiredAngleColor(color1)
	#self.headingKnob.SetRealAngleTextColor("DARK GREEN")
        #self.headingKnob.SetDesiredAngleTextColor(color1)
        #self.headingKnob.SetCircleColor("SIENNA")
	
	#vert1.Add(self.headingText, border = 4, flag=wx.CENTER)
	vert1.Add(self.headingKnob, 1,flag=wx.EXPAND|wx.CENTER)
	vert1.Add(wx.StaticText(self, -1, HEADING), flag=wx.CENTER)

	#self.Bind(knob.EVT_KNOB, self.OnHeadingKnobChange, id=self.currentHeadingKnob.GetId())
       
	####################### current speed gauge ###########################################
	
	self.currentSpeedBarText   = wx.StaticText(self, -1, "max", (30, 20))
        self.currentSpeedBar       = wx.Gauge(self, -1, 10, (-1, -1), (7, 50), style=wx.GA_VERTICAL|wx.GA_SMOOTH)

	vert2.Add(self.currentSpeedBarText, 0, border = 0,    flag = wx.CENTER)
	vert2.Add(self.currentSpeedBar,     1, border = 0,    flag = wx.CENTER)
	vert2.Add(wx.StaticText(self, -1, CURRENTSPEED),      flag = wx.CENTER)
	
	
	####################### desired speed gauge ###########################################
	
	self.desiredSpeedBarText   = wx.StaticText(self, -1, "max", (30, 20))
        self.desiredSpeedBar       = wx.Gauge(self, -1, 10, (-1, -1), (7, 50), style=wx.GA_VERTICAL|wx.GA_SMOOTH)

	vert3.Add(self.desiredSpeedBarText, 0, border = 0,    flag = wx.CENTER)
	vert3.Add(self.desiredSpeedBar,     1, border = 0,    flag = wx.CENTER)
	vert3.Add(wx.StaticText(self, -1, DESIREDSPEED),      flag = wx.CENTER)
	
# 	####################### desired speed slider ##########################################
# 
# 	self.desiredSpeedSliderText   = wx.StaticText(self, -1, "max", size=HEADINGSPEEDCONTROLSIZE, style=wx.ALIGN_CENTER)
# 	self.desiredSpeedSlider    = wx.Slider(self, -1, 100, 0, 100, style=wx.SL_VERTICAL|wx.SL_AUTOTICKS|wx.SL_RIGHT)
# 
# 	#wx.EVT_SCROLL(self.currentSpeedSlider, self.OnSpeedSliderChange)
# 
# 	vert3.Add(self.desiredSpeedSliderText, 0, border=0, flag=wx.CENTER)
# 	vert3.Add(self.desiredSpeedSlider, 1, border=2, flag=wx.EXPAND|wx.CENTER|wx.TOP|wx.BOTTOM)
# 	vert3.Add(wx.StaticText(self, -1, DESIREDSPEED), flag=wx.CENTER)
# 	
	####################### the three controls together ##################################################
	
	gap0.Add(spacer0)
	gap1.Add(spacer1)	
	gap2.Add(spacer2)
	
	horz1.Add(gap0, 0, flag=wx.EXPAND)
	horz1.Add(vert1, 1, flag=wx.EXPAND)
        horz1.Add(gap1,  0, flag=wx.EXPAND)
	horz1.Add(vert2, 0, flag=wx.EXPAND)
	horz1.Add(gap2,  0, flag=wx.EXPAND)
	horz1.Add(vert3, 0, flag=wx.EXPAND)
	
	sizerEntity = wx.BoxSizer(wx.VERTICAL)
	
	#sizerEntity.Add(labelEntity)
	sizerEntity.Add(spacer3)
	sizerEntity.Add(sizer)
	sizerEntity.Add(spacer4)
        sizerEntity.Add(horz1,1, flag=wx.EXPAND)
	sizerEntity.Add(spacer5, 0)
	self.SetSizer(sizerEntity)
	
    def getPanel(self):
        return self
    
    def tick(self, time, dtime):
        if EntityManager.currentState:
            
	    own = EntityManager.currentState.sensors.GetOwnEntity()
#            if exists(own):
            sel(own)
	    
	    # current entity name  
            self.t1.SetValue(getname())
	    
	    # current position 
            ox, oz, oy = gv('pos')
            self.t2.SetValue(`int(ox)`+', '+`int(oy)`)
            
	    max 	= gf("maxSpeed") * 100
            cs 		= gf("speed") * 100
            ds		= gf("desiredSpeed") * 100
	    rad 	= gf("heading")
            drad        = gf("desiredHeading")
            val 	= 360 - math.degrees(rad)
    
	    if val < 0:
                val += 360.0
            elif val >= 360.0:
                val -= 360.0
            h = '%.1f' % val
            
	    #if self.currentSpeedSlider.GetMax() == 0:
            
	    #self.desiredSpeedSlider.SetRange(0, int(max))
	    #self.desiredSpeedSlider.SetValue(max-int(ds))
            
	    self.currentSpeedBar.SetRange(int(max))
	    self.currentSpeedBar.SetValue(int(cs))
    
    	    self.desiredSpeedBar.SetRange(int(max))
	    self.desiredSpeedBar.SetValue(int(ds))
    
	    sd = '%5.1f' % (ds / 100)
            sc = '%5.1f' % (cs / 100)
	    
	    #self.desiredSpeedSliderText.SetLabel(sd)
	    
	    self.currentSpeedBarText.SetLabel(sc)
	    self.desiredSpeedBarText.SetLabel(sd)
	    #self.headingText.SetLabel(h)
            self.headingKnob.SetValue(drad)
            self.headingKnob.SetRealAngle(rad)
        
	else:
            self.t1.SetValue('none')
            self.t2.SetValue('00000, 00000')
            
	    #self.headingText.SetLabel("0.0")
            self.headingKnob.SetValue(0)
            self.headingKnob.SetRealAngle(0)
           
	    self.currentSpeedBarText.SetLabel("max")
	    self.currentSpeedBar.SetRange(1)
            self.currentSpeedBar.SetValue(0)
	    
	    self.desiredSpeedBarText.SetLabel("max")
	    self.desiredSpeedBar.SetRange(1)
            self.desiredSpeedBar.SetValue(0)


	    #self.desiredSpeedSlider.SetRange(0, 1)
            #self.desiredSpeedSlider.SetValue(1)
	    #self.desiredSpeedSliderText.SetLabel("max")
            



