# @(#) executive.py 2005/07/06 @author Copyright (c) 2005 The
#University of Nevada, Reno. All rights reserved.  This software is
#provided "AS IS," without a warranty of any kind. ALL EXPRESS OR
#IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
#IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
#OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. THE UNIVERSITY OF NEVADA,
#RENO AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
#BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
#SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL THE UNIVERSITY OF
#NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT
#OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL
#OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
#LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
#EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGES.  


"""
Provides outside information to CAT

Sensors has two parts. First there is the class that actually hold all the
information about the world and soon will actually do processing to figure it
out. Second there are global functions that work through a single Sensor
object. Through this multiple modules can all share the same sensor
information. Primarily, changes to global sensors will notify CAT so it can make
its own changes.

If something needs to maintain its own sensor state, then it should instantiate
a new Sensor object.
"""

startScenario = False
targetQuery = False
startScenarioChange = False
targetQueryChange = False

class Sensors:
    """
    Contains info on the world and methods to get it.

    This will expand to provide more information like ranges to
    surrounding land and entities.
    """
    def __init__(self, own=None, target=None):
        """
        Creates a new Sensors object.
        """
        self.OwnEntity = own
        self.TargetEntity = target

    def SetTargetEntity(self, name):
        """
        Set the target entity. 'name' is a path string in EVA.
        """
        self.TargetEntity = name
    
    def GetTargetEntity(self):
        """
        Get the target entity EVA path.
        """
        return self.TargetEntity

    def SetOwnEntity(self, name):
        """
        Set the entity to be controlled by cat. 'name' is a path string
        in EVA.
        """
        self.OwnEntity = name

    def GetOwnEntity(self):
        """
        Get the controlled entity EVA path.
        """
        return self.OwnEntity

    def SetTargetQuery(self, flag):
        SetTargetQuery(flag)

    def IsTargetQuery(self):
        return IsTargetQuery()
    
    def SetStartScenario(self, flag):
        SetStartScenario(flag)

    def IsStartScenario(self):
        return IsStartScenario()
      
guiSensors = Sensors()
  
def SetTargetEntity(name):
    """
    A global function that will set the Target in the GUI.
    """
    guiSensors.TargetEntity = name
    import gui
    gui.panel.OnSetTargetEntity()
    del gui
    
def GetTargetEntity():
    """
    A global function that will get the Target that the GUI is using.
    """
    return guiSensors.TargetEntity

def SetOwnEntity(name):
    """
    A global function that will set the entity the GUI will control.
    """
    guiSensors.OwnEntity = name
    import gui
    gui.panel.OnSetOwnEntity()
    del gui

def GetOwnEntity():
    """
    A global function that will get the Target that the GUI is controlling.
    """
    return guiSensors.OwnEntity

def SetTargetQuery(flag):
    global targetQuery, targetQueryChange
    if targetQuery != flag:
        targetQueryChange = True
    targetQuery = flag

def IsTargetQuery():
    return targetQuery
    
def HasTargetQueryChanged():
    global targetQueryChange
    if targetQueryChange:
        targetQueryChange = False
        return True
    return False

def SetStartScenario(flag):
    global startScenario, startScenarioChange
    if startScenario != flag:
        startScenarioChange = True
    startScenario = flag
        
def HasStartScenarioChanged():
    global startScenarioChange
    if startScenarioChange:
        startScenarioChange = False
        return True
    return False

def IsStartScenario():
    return startScenario
