#
# @(#) actuator.py 2005/10/19
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
from common.log import Log
log = Log(tags=['newcat'])
from common.defs import *
import nEventServer
import math
from common import behaviorNames

TIME_STEP         = 2   # how often (seconds) should I change speed/heading

class Actuators:
    """Actuators for units"""
    def __init__(self, entity):
        self.entity  = entity
        self.entPath = entity.getEVAPath()
        self.oldTime = 0
        self.weights = {}
        self.speeds  = {}
        self.headings= {}
        self.speed   = 0.0
        self.heading = 0.0

#         for bname in behaviorNames.FUSABLE_NAMES:
#             self.weights [bname] = 0.0
#             self.speeds  [bname] = 0.0
#             self.headings[bname] = 0.0
#             #        from common import tickmanager
            #        tickmanager.register("Actuator"+self.entPath,  self.Tick )
         
    def setWeight(self, bname, weight):
        if(self.weights.has_key(bname)):
            self.weights[bname] = weight

    def setDesiredSpeed(self, bname, speed):
        if(self.speeds.has_key(bname)):
            self.speeds[bname] = speed

    def setDesiredHeading(self, bname, heading):
        if(self.headings.has_key(bname)):
            self.headings[bname] = heading

    def setDCSpeed(self, speed):
        self.speed = speed
        pushsel(self.entPath)
        sf('desiredSpeed',   float(speed))
        popsel()

    def setDCHeading(self, heading):
        self.heading = heading
        pushsel(self.entPath)        
        sf('desiredHeading',   float(heading))        
        popsel()

            
    def update(self):
        if self.entity.isAvoidActive():
            self.speed,self.heading = self.computeWeightedSpeedHeading(justAvoid=True)
        else:
            self.speed, self.heading  = self.computeWeightedSpeedHeading(justAvoid=False)
        maxSpeed = self.entity.getMaxSpeed()
        if (self.speed > maxSpeed):
            self.speed = maxSpeed
        pushsel(self.entPath)
        sf('desiredSpeed',   float(self.speed))
        sf('desiredHeading', float(self.heading))
        popsel()


    def computeWeightedSpeedHeading(self, justAvoid=False):
        """Returns the [weighted average speed, highest weighted heading]"""
        speed      = 0.0
        heading    = 0.0
        sumWeights = 0.0
        totalx = 0.0
        totaly = 0.0
        avoidsActivated = 0
        bnameList = []
        if justAvoid:
            for bname in self.weights.keys():
                if bname.startswith("AVOID"):
                    if self.weights[bname]:
                        avoidsActivated += 1
                        bnameList.append(bname)
            #print "*******ComputeWSH: justAvoid=True,aa, blist", str(avoidsActivated), str(bnameList)
                        
        else:
            for bname in self.weights.keys():
                if not bname.startswith('AVOID'):
                    bnameList.append(bname)
            #print "*******ComputeWSH: justAvoid=FALSE,aa, blist", str(avoidsActivated), str(bnameList)                    
                    
        if justAvoid and not avoidsActivated:
                bnameList = []
                for bname in self.weights.keys():
                    if not bname.startswith('AVOID'):
                        bnameList.append(bname)
                #print "*******ComputeWSH: justAvoid=TRUE, avoidsActivated=0,aa, blist", str(avoidsActivated), str(bnameList)                                            
            
        for bname in bnameList:
            #print "Beh: %s, bnameList = %s, \nweights: %s \nSpeeds: %s \nHeadings: %s" % (bname, str(bnameList), str(self.weights), str(self.speeds), str(self.headings))
            speed   = speed   + (self.weights[bname] * self.speeds[bname])
            totalx += self.weights[bname] * math.cos(self.headings[bname])
            totaly += self.weights[bname] * math.sin(self.headings[bname])
            #print "Entity: %s, bname: %s, weight: %s, speed: %s, heading: %s" % (self.entity.getName(), bname, self.weights[bname], self.speeds[bname], self.headings[bname])
            sumWeights = self.weights[bname] + sumWeights
            #end For

            
        if sumWeights > 0.001:
            speed   = speed/sumWeights
            heading = math.atan2(totaly,totalx)
        else:
            speed = 0.0
            pushsel(self.entPath)
            heading = gf('heading')
            popsel()
            
        
        return [speed, heading]

    def getCurrentVec(self):

        self.speed,self.heading = self.computeWeightedSpeedHeading(justAvoid=False)
        
        return [self.speed, self.heading]
    
    def addBehavior(self, bname):
        self.weights[bname] = 0.0
        self.speeds[bname] = 0.0
        self.headings[bname] = 0.0

    def deleteBehavior(self, bname):
        if(self.weights.has_key(bname)):
            del self.weights[bname]
            del self.speeds[bname]
            del self.headings[bname]
