#
# @(#) entity.py 2005/09/29
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
import wx
import wx.lib.fancytext as fancytext
from common.log import Log
log = Log(tags=['entity'])
from common.defs import *
from common import behaviorNames
#from common import modeNames
from cat.entity import actuator
from cat.sensors import sensor
from cat import panelInfo

from cat.Interface import catButtonPanel
from cat.Interface import catSwitchRecordWrapper
from cat.Interface import catTriggerPanel

from cat.xml import xmlReader

from cat.Controllers import controller
from cat.Interface import catEntityPanel
class Entity:
    """An entity or unit in the system"""
##Z
    def __init__(self, entPath, panelA, panelT, panelV, size=None, netSpecList = None):
        self.authorPanel  = panelA 	# main panel (will change name to command")
        ####Y self.execPanel    = wx.Panel(self.authorPanel) #needed locally for second recorder panel (playback)
        self.triggerPanel = panelT
        self.netVisPanel  = panelV

        self.netSpecList  = netSpecList
        self.entPath = entPath
        #        self.netPanels   = netPanels
        self.EVAPath     = entPath
        self.name        = entPath.split('/')[-1]
        self.actuators   = actuator.Actuators(self)
        self.sensors     = sensor.Sensors(self)
#        self.maxSpeed    = self.computeMaxSpeed()

        self.currentControllerName = None
        self.playback              = False
        self.controllerDict        = {}
        self.netSpecDict           = {}
        self.filenames             = {}
        self.timeToCompletion      = 3601 # one hour = 3600 secs
        #self.bnet        = {}

        from cat import panelInfo
        x, y = size
        eSize = panelInfo.ENTITYPANEL
        cSize = panelInfo.BEHAVIORCONTROLPANEL
        wSize = panelInfo.CONTROLWRAPPERPANEL
        rSize = panelInfo.RECORDPANEL

        ## was here

        ## panels
        self.entityPanelA = self.initEntityPanel(self.authorPanel, size=eSize)

        # target panel is with primitive behavior
        self.sbcDict = {}

        self.triggerDict = {}

        # print "Created Entity Controller"
        from cat.Interface import catControlPanel
        self.contAuthPanel = catControlPanel.ControlPanel(self.authorPanel, size=cSize, color=panelInfo.CONTROL_AUTHOR_COLOR, entity=self, wsize=wSize)
        #        print "Created control Panels with size: ", str(self.contAuthPanel.GetSize())        

	
        # Create recorder wrapper to switch between auth/exec
        self.switchRecWrapper = catSwitchRecordWrapper.CatSwitchRecordWrapper(self.authorPanel, size=size, color=panelInfo.CONTROL_AUTHOR_COLOR, entity=self)
        import recorder
        self.recorder = recorder.Recorder(self.switchRecWrapper, size=rSize, entity=self)
        self.authRecordPanel, self.execRecordPanel = self.recorder.getPanels()
        
        # Add record panels to wrapper
        self.switchRecWrapper.add(self.authRecordPanel, "Auth")
        self.switchRecWrapper.add(self.execRecordPanel, "Exec")

        ## upto here
	
        ## controllers
        self.initControllers()
        self.recorder.updateNets()
	

        # sbc = single behavior controller
        #XXZ
	self.sbcDict     = self.initPrimitiveBehaviorControllers(self.authorPanel, size=wSize)
        # print "Created Primitive Behaviors"
        self.currentPrimitiveBehaviorName = behaviorNames.EMPTY
        self.mode = panelInfo.AUTHORING_MODE
        
        self.aSizer = wx.BoxSizer(wx.VERTICAL)

     	################################# NEXT 2 lines added by SD
        # Create button panel for exec/auth switching and add it to command panel
        self.buttonPanel = catButtonPanel.CatButtonPanel(self.authorPanel, color="CYAN", size=eSize, entity=self)
        self.aSizer.Add(self.buttonPanel, 0)
	########################################

        self.aSizer.Add(self.entityPanelA, 0)
        self.aSizer.Add(self.contAuthPanel, 0, flag=wx.FIXED_MINSIZE)

        # Triggers dummy panel
        self.triggerPanel = catTriggerPanel.CatTriggerPanel(self.authorPanel, color=panelInfo.TRIGGERPANELCOLOR, size=panelInfo.TRIGGERPANEL, entity=self)

        self.aSizer.Add(self.triggerPanel)

        
        # Make auth recorder active
        self.switchRecWrapper.show("Auth")
        # Keep a reference of recorder wrapper in the button panel class
        self.buttonPanel.SetRecorderPanels(self.switchRecWrapper)
	self.aSizer.Add(self.switchRecWrapper,0)
	
        self.authorPanel.SetSizer(self.aSizer)
        self.authorPanel.Layout()

        bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
	
        if(bc):
            bc.Show()
        self.authorPanel.Show()

        #from cat import netvis
        #self.visualizer = netvis.initialize(self.netVisPanel,self)

        print "Finished Making Entity", self.getName()
        call('/sys/servers/audio.play', 'entityCreated')
        #-----------------------------------------------------------------
	

    def getMaxSpeed(self):
        pushsel(self.getEVAPath())
        if (varexists(self.getEVAPath(), 'maxSpeed', 'float')):
            mspeed = float(gf('maxSpeed'))
        else:
            print "Cannot calculate maxSpeed at this point in time"
            mspeed = 5.0
        #print "Max speed = ", mspeed
        popsel()
        return mspeed

    def setTTC(self, val):
        self.timeToCompletion = val

    def getTTC(self):
        return self.timeToCompletion

#-------------------------------------------------------------------------------------------
    def getMode(self):
        return self.mode
#-------------------------------------------------------------------------------------------
    def isAuthorMode(self):
        #print self.entPath
        return (self.mode == panelInfo.AUTHORING_MODE)
#-------------------------------------------------------------------------------------------
    def setMode(self, mode):
        self.mode = mode
        #self.updatePanels(mode)
        print "Entity: Handling page change, setting mode to: ", str(mode)

#-------------------------------------------------------------------------------------------
    def isAvoidActive(self):
        if (self.playback):
            cntlr = self.getController(self.currentControllerName)
            if (cntlr):
                if(cntlr.hasAvoidActive()):
                    return True
            return False
        else:
            avoidLand = self.getPrimitiveBehaviorController(behaviorNames.AVOIDLAND)
            avoidEntity = self.getPrimitiveBehaviorController(behaviorNames.AVOIDENTITY)
            return (avoidEntity.getBehavior().runnable or avoidLand.getBehavior().runnable )
#-------------------------------------------------------------------------------------------
    def handleAvoidTarget(self, tp):
        if(self.currentControllerName):
            cntlr = self.getController(self.currentControllerName)
            if (cntlr):
                for id, behavior in cntlr.getBehaviorNet().iteritems():
                    if(id.startswith("AVOID")):
                        behavior.setTarget(tp)
#-------------------------------------------------------------------------------------------
    def initControllers(self):
        if(self.netSpecList):
            for netSpec, filepath in self.netSpecList:
                # self.makeController(netSpec)
                self.makeNetSpecDict(netSpec, filepath)                
        else:
            import glob
            cpath = 'cat/networks/'
            files = glob.glob(cpath+'*.bn')
            #print "Controllers:",  files
            for pathname in files:
                #self.loadController(pathname)
                self.loadNetSpecDict(pathname)                
        return


    def makeNetSpecDict(self, netSpec, filename):
        self.netSpecDict[netSpec.name] = netSpec
        self.filenames[netSpec.name] = filename

    def loadNetSpecDict(self, pathname):
        try:
            fname = pathname.split('/')[-1]
            netSpec = xmlReader.slurp(pathname)
            self.makeNetSpecDict(netSpec, pathname)
        except:
            printExcept("LoadNetSpecDict: Cannot load file: %s" % pathname)
        #print "Done loading :", fname

    def getNetSpecDict(self):
        return self.netSpecDict
#-------------------------------------------------------------------------------------------
    def makeController(self, netSpec = None, pathname = None):
        try:
            controlNet = controller.Controller(self.authorPanel, entity=self, filename = pathname, spec = netSpec)
            self.controllerDict[controlNet.getNetName()] = controlNet
        except:
            printExcept("makeController: Cannot make Controller: %s" % netSpec.name)
        print "Done making network : ", netSpec.name

    def loadController(self, pathname):
        try:
            fname = pathname.split('/')[-1]
            netSpec    = xmlReader.slurp(pathname)
            self.makeController(netSpec = netSpec, filename = pathname)
        except:
            printExcept("Cannot load file: %s" % pathname)
        #print "Done loading :", fname       
        

    def getNetList(self):
        return [self.controllerDict, {}]

    def getFilenames(self):
        return self.filenames

#     def OnAuthorMode(self, event):
#         self.setMode(panelInfo.AUTHORING_MODE)
    
#     def OnExecMode(self, event):
#         self.setMode(panelInfo.EXECUTION_MODE)
    

    def getControllerPanels(self):
        return self.contAuthPanel


    def getName(self):
        return self.name

    def getActuators(self):
        return self.actuators

    def getSensors(self):
        return self.sensors

    def setBehaviorSizer(self, sizer):
        self.behaviorSizer = sizer

    def handleTrigger(self, triggerSpec):
        for key, val in self.sbcDict.iteritems():
            val.triggerStop(triggerSpec)

    def addTrigger(self, trigger):
        self.triggerDict[trigger.getName()] = trigger

    def addTriggers(self, triggerlist):
        for trigger in triggerlist:
            self.addTrigger(trigger)

    def deleteTrigger(self, triggerName):
        if self.triggerDict.has_key(triggerName):
            self.triggerDict.pop(triggerName)

    def clearTriggers(self):
        self.triggerDict = {}

    def getTriggers(self):
        return self.triggerDict.values()

    def getTrigger(self, triggerName):
        return self.triggerDict.get(triggerName)

    def delete(self):
        self.entityPanelA.Destroy()
        ##Z self.entityPanelE.Destroy()        
        self.contAuthPanel.Destroy()
        ##Z self.contExecPanel.Destroy()        

    def Show(self):
        """Show all panels associated with Entity"""
        #print "Showing panels for: "  + self.getEVAPath()        
        self.authorPanel.Show()
        ##Zself.execPanel.Show()        
        self.entityPanelA.Show()
        ##Z self.entityPanelE.Show()        
        self.contAuthPanel.Show()
        ##Z self.contExecPanel.Show()

        #self.visualizer.Show()
        bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
        if(bc):
            bc.Show()
        # if(self.playback):
        self.displayController(self.currentControllerName)

        self.switchRecWrapper.Show()
        self.buttonPanel.Show()


    def Hide(self):
        """Hide all information associated with Entity"""
#        print "Hiding panels for: " + self.getEVAPath()
        self.authorPanel.Hide()
        ##Z self.execPanel.Hide()        
        self.entityPanelA.Hide()
        ##Z self.entityPanelE.Hide()
        self.contAuthPanel.Hide()
        ##Z self.contExecPanel.Hide()

        #self.visualizer.Hide()
        self.hideAllPBCs()
        self.hideControllers()

        self.buttonPanel.Hide()
        self.switchRecWrapper.Hide()

        
    # Stop all primitive behaviors
    def stopAll(self, event):
        #print "Controller Dict: ", str(self.controllerDict)
        for key, val in self.sbcDict.iteritems():
            #print "StopAll SBCs: Stopping: ", val.getBehaviorName()
            val.stop()
            #val.Hide()

        for key, val in self.controllerDict.iteritems():
            #print "StopAll Controllers: Stopping: ", val.getName()            
            val.stop()


    def stopRelated(self, bname):
        # fix for networks
        if (not behaviorNames.RELATED.has_key(bname)):
            return
        related = behaviorNames.RELATED[bname]
        for name in related:
            bc = self.getPrimitiveBehaviorController(name)
            if(bc):
                print "StopRelated:Stopping ", bc.getBehaviorName()
                bc.stop()


    # creates primitive behavior controllers for this entity.
    # Allows controlling an entity from CAT's primitive behavior panels
    ##Z
    def initPrimitiveBehaviorControllers(self, panelA, size = None):
        from cat.Controllers import primitiveBehaviorController
        from common import behaviorNames
        sbcDict = {}
        for sbc in behaviorNames.BEHAVIOR_NAMES:
            sbcDict[sbc] = primitiveBehaviorController.PrimitiveBehaviorController(panelA, entity=self, bname=sbc, size=size, show = True)
        return sbcDict

    def getPrimitiveBehaviorController(self, bname):
        if (self.sbcDict.has_key(bname)):
#            print "Entity: Getting behavior controller: ", bname
            return self.sbcDict[bname]
        else:
            #print "No such controller exists: ", bname
            return None

        #     def setPrimitiveBehaviorController(self, behaviorName):
        #         self.sbcDict[name] = singleBehaviorController(self.topPanel, entity=self, bn=behaviorName)
        
    # Show the panel associated with the behavior with name bname,
    # event handler for controlPanel behavior switching buttons
    def showBehavior(self, event):
        bname = event.GetEventObject().GetName()
        #        print "BehaviorNames Selected: ", bname        
        self.displayBehavior(bname)

    def displayBehavior(self, bname):
        # print "Entity: Showing primitive Behavior: " + bname
        self.currentPrimitiveBehaviorName = bname
        bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)

        self.contAuthPanel.Hide()
        ##Z self.contExecPanel.Hide()
        self.contAuthPanel.Show()
        ##Z self.contExecPanel.Show()

        if (bc):
            bc.Show()
            bc.getBehavior().updatePanels()

    def isAnyBehaviorActive(self):
        for key, val in self.sbcDict.iteritems():
            if(val.behavior.runnable):
                return True
        return False


    def hideAllPBCs(self):
        from common import behaviorNames
        for key, val in self.sbcDict.iteritems():
            val.Hide()
#        print "Hid all %s behaviors" % self.getEVAPath()

    def hideControllers(self):
        cntl = self.getController(self.currentControllerName)
        if(cntl):
            cntl.hide()

    def initEntityPanel(self, panel, size = None):
        """entityPanel designed by Sergiu"""
	
        panel1 = catEntityPanel.EntityPanel(panel, size=size, entity=self)
	
        return panel1

    def getRecorder(self):
        return self.recorder

    def getEntityPanel(self):
        return self.entityPanel

    def setEntityPanel(self, ePanel):
        self.entityPanel = ePanel

    def getEVAPath(self):
        return self.EVAPath

    def setEVAPath(self, newPath):
        self.EVAPath = newPath
        
 
    def setTarget(self, targetPath):
        if(self.currentPrimitiveBehaviorName):
            #print "Setting %s as target  for %s" % (targetPath, self.currentPrimitiveBehaviorName)
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            bc.setTarget(targetPath)
        else:
            print "No current behavior selected for target: ", targetPath
    
    def getTarget(self):
        if(self.currentPrimitiveBehaviorName):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            return bc.getTarget()
        else:
#            print "No current behavior selected for target: ", targetPath
            return None
        
    def handleTargetSelection(self, targetPath, msg):
        if(self.playback):
            self.handleControllerTargetSelection(targetPath, msg)
            return
        if(self.currentPrimitiveBehaviorName):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            bc.getBehavior().handleTargetSelection(targetPath, msg)
        else:
            print "handleTargetSelection: No current behavior selected for target: ", targetPath

    def handleBehaviorChange(self, msg):
        if (self.playback):
            print "%%%%%%%%%%%%%%%%%%%%%CANNOT play behavior while controller is running", msg
            return
        args     = parseRMenuMsg(msg)
        moveLoc  = args.mousePos
        menuItem = args.menuItem
        text     = args.text
        bname    = str(menuItem)
        # self.displayBehavior(bname)
        #print "BehaviorName Picked: ", bname
        bc = self.getPrimitiveBehaviorController(bname)
        behavior = bc.getBehavior()
        from common import behaviorNames
        if(bname.startswith(behaviorNames.MOVETO)):
            behavior.setMoveLocation(moveLoc)
        elif (bname in behaviorNames.FUSABLE_NAMES):
            targetPath = args.objectsOver[0]
            behavior.handleTargetSelection(targetPath, msg)
        behavior.start()
        self.displayBehavior(bname)
        self.sayBehavior(bname)

    def sayBehavior(self, bname):
        if(bname == behaviorNames.MOVETO):
            call('/sys/servers/audio.play', 'moveTo')
        elif (bname == behaviorNames.MAINTAIN):
            call('/sys/servers/audio.play', 'maintain')            
        elif (bname == behaviorNames.APPROACH):
            call('/sys/servers/audio.play', 'approach')            
        elif (bname == behaviorNames.RAM):
            call('/sys/servers/audio.play', 'ram')
        elif (bname == behaviorNames.FIRE):
            call('/sys/servers/audio.play', 'fire')
            
    def updatePanels(self, mode=None):
        # print "Entity: handling page change or other update"
        return
        # not needed with only command panels
        from common import behaviorNames        
        if(not self.currentPrimitiveBehaviorName.startswith(behaviorNames.EMPTY)):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            bc.getBehavior().handlePageChange("Tryng hard to GOOOO")
        elif (self.playback):
            cntrllr = self.getController(self.currentControllerName)
            if (cntrllr):
                cntrllr.handlePageChange(mode)
        else:
            print "Update Panels: handlePageChange: No current behavior: "

    def stopRecording(self):
        for key, val in self.sbcDict.iteritems():
            if(val.behavior.runnable):            
                val.behavior.stopRecord()

    ##################Controller related methods######################                

    def displayController(self, cname):
        print "Entity: Showing controller: " + str(cname)
        self.currentControllerName = cname
        c = self.getController(cname)
        if (c):
            c.show()
            #self.authRecordPanel.updatePanels(cname)
            self.execRecordPanel.updatePanels(cname)            
        else:
            print "No controller exists"

    def getController(self, cname):
        if(self.controllerDict.has_key(cname)):
            cntrllr = self.controllerDict[cname]
            return cntrllr
        elif(self.netSpecDict.has_key(cname)):
            self.makeController(netSpec = self.netSpecDict[cname])
            cntrllr = self.controllerDict[cname]
            return cntrllr
        else:
            return None
        
    def handleInitRequest(self, netName, args):
        cntl = self.getController(netName)
        print "Entity: Initializing controller: ", netName
        if (cntl):
            if (args):
                cntl.setSPArgs(args)
            self.currentControllerName = netName
            self.playback = True
            cntl.init()

    def handlePlayRequest(self, netName, args):
        self.stopAll(None)
        cntl = self.getController(netName)
        print "Entity: Playing controller:", netName
        if (cntl):
            if (args):
                cntl.setSPArgs(args)
            self.currentControllerName = netName
            cntl.init()
            self.playback = True
            self.buttonPanel.SwitchToExecution(None)
            cntl.start()

    def handleStopRequest(self, netName):
        cntl = self.getController(netName)
        if (cntl):
            self.currentControllerName = None
            cntl.stop()
            self.playback = False

    def handlePauseRequest(self, netName):
        cntl = self.getController(netName)
        if cntl:
            cntl.pause()
            self.playback = True
            
    def handleControllerTargetSelection(self, targetPath, msg):
        #print "*********************", self.currentControllerName
        cntl = self.getController(self.currentControllerName)
        if (cntl):
            cntl.handleTargetSelection(targetPath, msg)
        else:
            print "Cannot select target, no current controller selected"
