#
# @(#) entitySet.py 2005/09/29
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
from common.log import Log
log = Log(tags=['newcat'])
from common.defs import *
import nEventServer
from common import EventNames

class EntitySet:
    """A set of unique entities in the system"""
    ##Z
    def __init__(self, panelA, panelT, panelV, size = None):
        self.authorPanel      = panelA
        self.triggerPanel     = panelT
        self.netVisPanel      = panelV
        #        self.netPanels  = netPanels
        self.size = size
        self.netSpecList = []
        self.entityList  = []
        self.entityDict  = {}
        pushsel("/game/ents/")
        for entType in ndir():
            sel(entType)
            for ent in ndir():
                entPath = "/game/ents/" + entType + "/" + ent
                self.entityList.append(entPath)
            sel("..")
        print("EntityList: " + str(self.entityList))
        self.currentEntPath = None
        popsel()

        nEventServer.register(EventNames.SELECTION_CHANGED,       self.leftSelectEnt)
        nEventServer.register(EventNames.RMENU_DELETE_ENT,        self.handleDeletion)
        #nEventServer.register(EventNames.RMENU_CREATE_ENT,        self.pullLater)
        #nEventServer.register(EventNames.SP_CREATE_ENT,           self.SPpullLater)
        nEventServer.register(EventNames.ENTITY_CREATED,          self.handleEntCreation)
        nEventServer.register(EventNames.SP_CREATE_ENT,           self.handleSPEntCreation)
        nEventServer.register(EventNames.RMENU_CAT_SELECT_TARGET, self.handleTargetSelection)
        nEventServer.register(EventNames.RMENU_BEHAVIOR_CHANGE,   self.handleBehaviorChange)
       #nEventServer.register(EventNames.CAT_PAGE_CHANGED,        self.handlePageChange)                 
        nEventServer.register(EventNames.RMENU_ASSIGN_OBJECTIVE,  self.handleObjectiveChange)           
        nEventServer.register(EventNames.CAT_DELETE_NETWORK,      self.onDelete)
        self.pulledStrings = []

    def onDelete(self, msg):
        for ns, fn in self.netSpecList:
            if ns.name == unicode(msg):
                self.netSpecList.remove((ns, fn))
                break
        
    def setNetSpecList(self, nsl):
        self.netSpecList = nsl

    def handlePageChange(self, event):
        print "EntitySet: Handling page change"
        mode = int(event.GetEventObject().GetSelection())
        if(self.currentEntPath):
            self.entityDict[self.currentEntPath].setMode(mode)
        else:
            print "No own ship selected"

    def handleTargetSelection(self, msg):
        args    = parseRMenuMsg(msg)
        targetPath  = args.objectsOver[0]
#        print "Setting %s -- target to: %s" % (self.currentEntPath.split('/')[-1], targetPath.split('/')[-1])
        if(self.currentEntPath):
#            self.entityDict[self.currentEntPath].setTarget(targetPath)
            self.entityDict[self.currentEntPath].handleTargetSelection(targetPath, msg)
            call('/sys/servers/audio.play', 'targetSelected')
        else:
            print "No ownship for target: ", target
        
    def handleObjectiveChange(self, msg):
        print "Handling Objective Change: ", str(msg)

    def handleBehaviorChange(self, msg):
        '''
        Behavior handler.
        '''
        if(self.currentEntPath):
            self.entityDict[self.currentEntPath].handleBehaviorChange(msg)
        else:
            print "No Entity for handling behavior change: ", str(msg)
        
    def handleDeletion(self, msg):
        args = parseRMenuMsg(msg)
        who = args.objectsSelected[0]
        if(self.currentEntPath.startswith(who)):
            self.currentEntPath = None
        #endifx
        delete (who)

    def delete(self, entPath):
        if(self.entDict.has_key(entPath)):
            ent = self.entDict[entPath]
            ent.Hide()
            ent.delete()
        
    def handleEntCreation(self, path):
	 ##Z
         self.makeEntity(path, self.authorPanel, self.triggerPanel, self.netVisPanel)
            # self.entityDict[entPath].Show()

    def handleSPEntCreation(self, msg):
        #for entPath in self.pulledStrings:
        entPath = str(msg)
        if(not self.entityDict.has_key(entPath)):
		##Z
            self.makeEntity(entPath, self.authorPanel, self.triggerPanel, self.netVisPanel)
        #self.pulledStrings = []
        
    def leftSelectEnt(self, msg):
        """link into the standard selection mechanism"""
        print 'leftSelectEnt', msg
        #from cat.entity import EntityManager
        ent = call('/ui/planner.getobjectsselected')
        if ent:
            from sp.playermanager import playerManager                          #chris
            if playerManager.canControlEnt(ent):                                #chris -> cant cat select entities on another team
                #print 'Selected entities are: ', ent
                ents = ent.split(' ')
                print 'Selected entities are :', str(ents)
                if(len(ents) >= 1):
                    entPath = ents[0].strip() 
                else:
                    entPath = ent.strip()
                #print 'ent', ent
                call('/ui/planner.clearcatselection')
                call('/ui/planner.addcatselectedent', entPath)
                #print "Selected entity is: " + str(entPath)
                if (self.currentEntPath):
                    self.entityDict[self.currentEntPath].Hide()
                self.currentEntPath = entPath
                            
                #Have to hide the panel before you go and show it again... for some reason.
		#print self.entityDict
                if(self.entityDict.has_key(entPath)):
                    self.entityDict[entPath].Hide()
                    self.entityDict[entPath].Show()
                else:
                    #print "Cat information on Entity does not exist: " + str(ent)
		    ##Z
                    self.makeEntity(entPath, self.authorPanel, self.triggerPanel, self.netVisPanel)
                    self.entityDict[entPath].Show()
                    from cat import panelInfo
                    #self.entityDict[entPath].setMode(panelInfo.AUTHORING_MODE)
                    self.entityDict[entPath].setMode(self.entityDict[entPath].mode)
            else:
                print "Sides: Cannot control ent: ", str(ent)
        else:
            print "Entity does not exist: " + str(ent)
        #endif
        
	##Z
    def makeEntity(self, entPath, authorPanel, trigPanel, netVisPanel):
        import entity
        log.write("Making entity: " + str(entPath))
        print "Making entity: " + str(entPath)
	##Z
        
        ent = entity.Entity(entPath, authorPanel, trigPanel, netVisPanel, size = self.size, netSpecList = self.netSpecList)
        #if entPath == "/game/ents/cigarette/cigarette1":
        #    try:
        #        from cat.triggers.trigger import Trigger
        #        from cat import catInfo
        #        print "Created trigger"
        #    
        #        trigger = Trigger(self.entityDict["/game/ents/cigarette/cigarette0"],
        #                          "Test Trigger", catInfo.DAMAGE_SENSOR, toPath="/game/ents/cigarette/cigarette1")
        #    except:
        #        printExcept("creating trigger")
        #    self.tmpTrigger = trigger
        #if entPath == "/game/ents/cigarette/cigarette2":
        #    print "*********** firing trigger"
        #    try:
        #        self.tmpTrigger.fire()
        #    except:
        #        printExcept("firing trigger")
        self.entityDict[entPath] = ent

    def makeCatEntities(self):
        import entity
        for entPath in self.entityList:
            self.makeEntity(entPath, self.authorPanel, self.triggerPanel, self.netVisPanel)
        return self.entityDict

    def getEntities(self):
        return self.entityDict

    def setEntities(self, eDict):
        self.entityDict = eDict

    def getEntity(self, entityPath):
        if(self.entityDict.has_key(entityPath)):
            return self.entityDict[entityPath]
        else:
            log.write("No entity of that name: " + entityPath)
            print("No entity of that name: " + entityPath)
            return None

    def setSelectedEntity(self, entityPath):
        if (self.entityDict.has_key(entityPath)):
            self.currentEntPath = entityPath
            self.entityDict[entityPath].Show()
        
#     def setEntity(self, entPath):
#         import entity
#         ##Z
# 	self.entityDict[entPath] = entity.Entity(entPath, self.authorPanel)

    def AttachController(self, entPath, task, args):
        # print "In entity Set: Gettings entity"
        if(not self.entityDict.has_key(entPath)):
		##Z
            self.makeEntity(entPath, self.authorPanel, self.triggerPanel, self.netVisPanel)
            ## creates side effect by adding entry to self.entityDict
            #print "EntitySet: Made entity: ", entPath
        ent = self.getEntity(entPath)
        if(ent):
            #ent.handleInitRequest(task, args)
            print "EntitySet Handling network play request:", task, entPath, args
            # should we not change entity's mode to execution?
            from cat import panelInfo
            ent.setMode(panelInfo.EXECUTION_MODE)
            ent.handlePlayRequest(task, args)
#         
