import wx
import wx.lib.ogl as ogl

from cat import *
from common import *

class BehaviorLink(ogl.LineShape):
    """holds information about a link between two behaviorShapes"""
    def __init__(self, fromShape, toShape, type, vertices, canvas, onLinkLeftClick, onLinkRightClick):
        self.fromShape = fromShape
        self.toShape = toShape
        self.type = type
        self.canvas = canvas
        self.onLinkLeftClick = onLinkLeftClick
        self.onLinkRightClick = onLinkRightClick

        ogl.LineShape.__init__(self)
        self.MakeLineControlPoints(2)

        if self.type == "enabling":
            self.SetPen(wx.Pen(wx.Colour(255,0,0),1,wx.SOLID))
            self.SetBrush(wx.Brush(wx.Colour(255,0,0),wx.SOLID))
        elif self.type == "permanent":
            self.SetPen(wx.Pen(wx.Colour(0,255,0),1,wx.SOLID))
            self.SetBrush(wx.Brush(wx.Colour(0,255,0),wx.SOLID))
        else:
            self.SetPen(wx.Pen(wx.Colour(255,255,255),1,wx.SOLID))
            self.SetBrush(wx.Brush(wx.Colour(255,255,255),wx.SOLID))
        self.AddArrow(ogl.ARROW_ARROW)

        startx, starty = vertices[0]
        endx, endy = vertices[-1]
        self.SetEnds(startx,starty,endx,endy)

        for i in range(len(vertices)-2):
            x,y = vertices[i]
            self.InsertLineControlPoint(point=(x,y))

#        if drawBoundingBox:
#            self.drawBox()
#        else:
#            self.box = None

        self.canvas.AddShape(self)

    def drawBox(self):
        controlPoints = self.GetLineControlPoints()
        minx,miny = maxx,maxy = controlPoints[0]
        for (x,y) in controlPoints:
            minx = min(x,minx)
            miny = min(y,miny)
            maxx = max(x,maxx)
            maxy = max(y,maxy)
        gutter = 5
        self.box = ogl.LineShape()
        self.box.MakeLineControlPoints(2)
        self.box.SetEnds(minx-gutter,miny-gutter,minx-gutter,miny-gutter)
        self.box.InsertLineControlPoint(point=(minx-gutter,maxy+gutter))
        self.box.InsertLineControlPoint(point=(maxx+gutter,maxy+gutter))
        self.box.InsertLineControlPoint(point=(maxx+gutter,miny-gutter))
        self.box.SetPen(wx.Pen(wx.Colour(255,255,255),1,wx.SOLID))
        self.box.SetDraggable(False)
        self.canvas.AddShape(self.box)

    def OnLeftClick(self, x, y, keys, attachment):
        self.onLinkLeftClick(self)

    def OnRightClick(self, x, y, keys, attachment):
        self.onLinkRightClick(self)

    def makeUpArrow(self, linex, liney, lineStep):
        tox = self.toShape.GetX()
        toy = self.toShape.GetY()
        tow = self.toShape.GetWidth()
        toh = self.toShape.GetHeight()

        fromx = self.fromShape.GetX()
        fromy = self.fromShape.GetY()
        fromw = self.fromShape.GetWidth()
        fromh = self.fromShape.GetHeight()

        fromBottomx = self.makeBottomx(self.fromShape,lineStep) + fromx
        toTopx = self.makeTopx(self.toShape,lineStep) + tox

        self.SetEnds(fromBottomx,fromy+fromh/2,toTopx,toy-toh/2)

        bottomy = self.make(fromy+fromh*2,liney,lineStep) 
        topy = self.make(toy-toh*2,liney,lineStep)
        leftx = self.unsignedMake(lineStep,linex,lineStep)    

        self.InsertLineControlPoint(point = (fromBottomx,bottomy))
        self.InsertLineControlPoint(point = (leftx,bottomy))
        self.InsertLineControlPoint(point = (leftx,topy))
        self.InsertLineControlPoint(point = (toTopx,topy))

    def makeDownArrow(self, linex, liney, lineStep):
        tox = self.toShape.GetX()
        toy = self.toShape.GetY()
        tow = self.toShape.GetWidth()
        toh = self.toShape.GetHeight()

        fromx = self.fromShape.GetX()
        fromy = self.fromShape.GetY()
        fromw = self.fromShape.GetWidth()
        fromh = self.fromShape.GetHeight()

        fromBottomx = self.makeBottomx(self.fromShape,lineStep) + fromx
        toTopx = self.makeTopx(self.toShape,lineStep) + tox

        self.SetEnds(fromBottomx,fromy+fromh/2,toTopx,toy-toh/2)

        midy = self.make(fromy + (toy - fromy) / 2,liney,lineStep)
        self.InsertLineControlPoint(point = (fromBottomx,midy))
        self.InsertLineControlPoint(point = (toTopx,midy))

    def makeSideArrow(self, linex, liney, lineStep):
        tox = self.toShape.GetX()
        toy = self.toShape.GetY()
        tow = self.toShape.GetWidth()
        toh = self.toShape.GetHeight()

        fromx = self.fromShape.GetX()
        fromy = self.fromShape.GetY()
        fromw = self.fromShape.GetWidth()
        fromh = self.fromShape.GetHeight()

        fromTopx = self.makeTopx(self.fromShape,lineStep) + fromx
        toTopx = self.makeTopx(self.toShape,lineStep) + tox

        self.SetEnds(fromTopx,fromy-fromh/2,toTopx,toy-toh/2)

        topy = self.make(fromy-(fromh+toh),liney,lineStep)
        self.InsertLineControlPoint(point = (fromTopx,topy))
        self.InsertLineControlPoint(point = (toTopx,topy))

    def inList(self, num, list, tol):
        for i in list:
            if i >= num - tol and i <= num + tol:
                return True
        return False

    def unsignedMake(self, x, linex, lineStep):
        ret = x
        dist = 0
        ok = True
        while self.inList(ret,linex,lineStep):
            dist = dist + lineStep
            ret = x + dist
        linex.append(ret)
        return ret

    def make(self, x, linex, lineStep):
        ret = x
        sign = 1
        dist = 0
        ok = True
        while self.inList(ret,linex,lineStep):
            sign = sign * -1
            if sign < 0:
                dist = dist + lineStep
            ret = x + dist * sign
        linex.append(ret)
        return ret
    
    def makeTopx(self, shape, lineStep):
        ret = shape.topx
        shape.topx = shape.topx * -1
        if shape.topx <= 0:
            shape.topx = shape.topx - lineStep
        return ret

    def makeBottomx(self, shape, lineStep):
        ret = shape.bottomx
        shape.bottomx = shape.bottomx * -1
        if shape.bottomx <= 0:
            shape.bottomx = shape.bottomx - lineStep
        return ret

