import wx
import wx.lib.ogl as ogl

from cat import *
from common import *

class BehaviorShape(ogl.EllipseShape):
    """holds information about one behavior"""
    def __init__(self, gridx, gridy, behavior, canvas, onBehaviorLeftClick, onBehaviorRightClick):
        self.behavior = behavior
        self.canvas = canvas
        self.onBehaviorLeftClick = onBehaviorLeftClick
        self.onBehaviorRightClick = onBehaviorRightClick
        self.active = False
        self.lastx = 0
        self.lasty = 0
        self.box = None

        nameDict = {}
        nameDict[behaviorNames.MAINTAIN] = "MA"
        nameDict[behaviorNames.APPROACH] = "AP"
        nameDict[behaviorNames.MOVETO] = "MO"
        nameDict[behaviorNames.RAM] = "RA"
        nameDict[behaviorNames.FIRE] = "FI"
        nameDict[behaviorNames.AVOIDLAND] = "AL"
        nameDict[behaviorNames.AVOIDENTITY] = "AE"
        nameDict[behaviorNames.STOPALL] = "ST"
        nameDict[behaviorNames.NONE] = "NO"
        nameDict[behaviorNames.EMPTY] = "EM"

        w = 35
        h = 25
        thList = []

        ogl.EllipseShape.__init__(self,w,h)
        if self.getName() in nameDict.keys():
            text = nameDict[self.getName()]
        else:
            text = self.getName()[0:2]
        self.AddText(text)
#        if not directLocation:
#        self.SetX(gridx * xStep + xGutter)
#        self.SetY(gridy * yStep + yGutter)
#        else:
        self.SetX(gridx)
        self.SetY(gridy)
        self.SetBrush(wx.Brush(wx.Colour(100,100,255),wx.SOLID))
        self.SetPen(wx.Pen(wx.Colour(255,255,255),0,wx.SOLID))

        self.canvas.AddShape(self)

    def makeBox(self):
        print "making box"
        self.box = ogl.RectangleShape(self.GetWidth(),self.GetHeight())
        self.box.SetX(self.GetX())
        self.box.SetY(self.GetY())
        self.box.SetBrush(wx.Brush(wx.Colour(0,0,0),wx.TRANSPARENT))
        self.box.SetPen(wx.Pen(wx.Colour(255,255,255),1,wx.SOLID))
        self.box.SetDraggable(False)
        self.canvas.RemoveShape(self)
        self.canvas.AddShape(self.box)
        self.canvas.AddShape(self)

    def removeBox(self):
        print "removing box"
        self.canvas.RemoveShape(self.box)
        self.box = None

    def updateBox(self):
        print "updating box"
        if self.box:
            print "really updating box"
            self.box.SetX(self.GetX())
            self.box.SetY(self.GetY())
            self.box.SetWidth(self.GetWidth())
            self.box.SetHeight(self.GetHeight())

    def collides(self, testx, testy):
        w = self.GetWidth()
        h = self.GetHeight()
        x = self.GetX() - w/2
        y = self.GetY() - h/2
        return testx >= x and testx <= x + w and testy >= y and testy <= y + h

    def getName(self):
        return self.behavior.bname

    def OnRightClick(self, x, y, keys, attachment):
        self.onBehaviorRightClick(self)

    def OnLeftClick(self, x, y, keys, attachment):
        self.onBehaviorLeftClick(self)

