import wx
import wx.lib.ogl as ogl

from cat import *
from common import *

class LinkMenu(wx.Panel):
    def __init__(self, parent, size, fromBehavior, toBehavior, type, behaviorShapes, behaviorNet, netSpec, reloadNetSpec):
        self.fromBehaviorID = fromBehavior.bname
        self.toBehaviorID = toBehavior.bname
        self.firstType = type
        self.behaviorShapes = behaviorShapes
        self.behaviorNet = behaviorNet
        self.netSpec = netSpec
        self.reloadNetSpec = reloadNetSpec

        self.oldFromBehaviorID = self.fromBehaviorID
        self.oldToBehaviorID = self.fromBehaviorID
        self.firstFromBehaviorID = self.fromBehaviorID
        self.firstToBehaviorID = self.toBehaviorID
        
        wx.Panel.__init__(self,parent,size=size)
        self.SetBackgroundColour(parent.GetBackgroundColour())

        self.behaviorNames = []
        for behavior, shape in behaviorShapes.items():
            self.behaviorNames.append(behavior.bname)
        self.typeNames = ["ordering","enabling","permanent"]

        self.vsizer = wx.BoxSizer(wx.VERTICAL)

        self.vsizer.Add(wx.StaticText(self,-1,"Editing Link"),1,wx.EXPAND)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(wx.StaticText(self,-1,"From:"),1,wx.EXPAND)
        self.fromBox = wx.ComboBox(self,-1,choices=self.behaviorNames,style=wx.CB_DROPDOWN|wx.CB_READONLY)
        wx.EVT_TEXT(self.fromBox,-1,self.onBoxChange)
        self.fromBox.SetStringSelection(self.fromBehaviorID)
        hsizer.Add(self.fromBox,2,wx.EXPAND)
        self.vsizer.Add(hsizer,1,wx.EXPAND)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(wx.StaticText(self,-1,"To:"),1,wx.EXPAND)
        self.toBox = wx.ComboBox(self,-1,choices=self.behaviorNames,style=wx.CB_DROPDOWN|wx.CB_READONLY)
        wx.EVT_TEXT(self.toBox,-1,self.onBoxChange)
        self.toBox.SetStringSelection(self.toBehaviorID)
        hsizer.Add(self.toBox,2,wx.EXPAND)
        self.vsizer.Add(hsizer,1,wx.EXPAND)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(wx.StaticText(self,-1,"Type:"),1,wx.EXPAND)
        self.typeBox = wx.ComboBox(self,-1,choices=self.typeNames,style=wx.CB_DROPDOWN|wx.CB_READONLY)
        wx.EVT_TEXT(self.typeBox,-1,self.onBoxChange)
        self.typeBox.SetStringSelection(self.firstType)
        hsizer.Add(self.typeBox,2,wx.EXPAND)
        self.vsizer.Add(hsizer,1,wx.EXPAND)

        okID = 200
        deleteID = 201

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(wx.Button(self,okID,"OK"),1,wx.EXPAND)
        wx.EVT_BUTTON(self,okID,self.onOK)
        hsizer.Add(wx.Button(self,deleteID,"DELETE"),1,wx.EXPAND)
        wx.EVT_BUTTON(self,deleteID,self.onDelete)
        self.vsizer.Add(hsizer,1,wx.EXPAND)

        self.vsizer.Add(wx.StaticText(self,-1,""),10,wx.EXPAND)

        self.SetPosition((0,0))
        (w,h) = size
        self.vsizer.SetDimension(0,0,w,h)
        self.SetSize(size)
        self.SetSizer(self.vsizer)
        self.SetAutoLayout(1)
        self.vsizer.Fit(self)
        self.Layout()
        self.Show()

    def cleanup(self):
        return None

    def getBehavior(self, bid):
        ret = None
        for testBid, behavior in self.behaviorNet.items():
            if bid == testBid:
                ret = behavior
        return ret

    def getBspec(self, bid):
        ret = None
        for bspec in self.netSpec.behaviors:
            if bspec.id == bid:
                ret = bspec
        return ret

    def onBoxChange(self, e):
        self.oldFromBehaviorID = self.fromBehaviorID
        self.oldToBehaviorID = self.toBehaviorID

        self.fromBehaviorID = self.fromBox.GetValue()
        self.toBehaviorID = self.toBox.GetValue()
        type = self.typeBox.GetValue()

        oldFromBspec = self.getBspec(self.oldFromBehaviorID)
        for link in oldFromBspec.links:
            if link.dest == self.oldToBehaviorID:
                oldFromBspec.links.remove(link)

        for link in oldFromBspec.links:
            if link.dest == self.oldToBehaviorID:
                oldFromBspec.links.remove(link)

        fromBspec = self.getBspec(self.fromBehaviorID)
        fromBspec.links.append(xml.behaviorSpec.Link("to",type,self.toBehaviorID))

        self.reloadNetSpec(self.netSpec)

    def onOK(self, e):
        self.onBoxChange(None)
        self.Destroy()

    def onDelete(self, e):
        self.oldFromBehaviorID = self.fromBehaviorID
        self.oldToBehaviorID = self.toBehaviorID

        self.fromBehaviorID = self.firstFromBehaviorID
        self.toBehaviorID = self.firstToBehaviorID
        type = self.firstType

        oldFromBspec = self.getBspec(self.oldFromBehaviorID)
        for link in oldFromBspec.links:
            if link.dest == self.oldToBehaviorID:
                oldFromBspec.links.remove(link)

        for link in oldFromBspec.links:
            if link.dest == self.oldToBehaviorID:
                oldFromBspec.links.remove(link)

        fromBspec = self.getBspec(self.fromBehaviorID)
        fromBspec.links.append(xml.behaviorSpec.Link("to",type,self.toBehaviorID))

        self.reloadNetSpec(self.netSpec)
        self.Destroy()







