import string
import os
import wx
import wx.lib.ogl as ogl

from cat import *
from common import *
import nEventServer

import behaviorLink
import behaviorShape
import behaviorMenu
import linkMenu
import mainMenu
import lineLayout
import loadMenu
import saveMenu

class NetForm(wx.Panel):
    def __init__(self, parent, id, entity):
        #init cat interface
        self.entity = entity
        self.controller = self.entity.getController(self.entity.currentControllerName)
        self.netSpec = None
        self.lastControllerName = "none"

        #init shape dicts
        self.behaviorShapes = {}
        self.linkShapes = {}

        #init activity pointers
        self.activeShape = None
        self.selectedShape = None
        self.selectedLink = None
        self.targetPath = None
        self.behaviorTarget = {}

        #init this window
        wx.Panel.__init__(self,parent)
        self.vsizer = wx.BoxSizer(wx.VERTICAL)

        #init the sizes
        size = (220,940)
        w,h = size
        self.SetSize(size)
        self.topSize = (w,h/3)
        self.bottomSize = (w,h*2/3)
        topw, toph = self.topSize
        bottomw, bottomh = self.bottomSize

        #init the ogl canvas
        self.canvas = ogl.ShapeCanvas(self)
        self.diagram = ogl.Diagram()
        self.canvas.SetDiagram(self.diagram)
        self.diagram.SetCanvas(self.canvas)
        self.canvas.SetSize(self.topSize)

        #make the top sizer and put the canvas and buttons in it
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        self.topSizer.Add(self.canvas,9,wx.EXPAND)
        self.topSizer.Add(self.makeBehaviorBar(),1,wx.EXPAND)

        addLinkID = 120
        self.topSizer.Add(wx.Button(self,addLinkID,"Add Link"),1,wx.EXPAND)
        wx.EVT_BUTTON(self,addLinkID,self.onAddLink)

        startID = 121
        stopID = 122
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        startButton = wx.Button(self,startID,"Start")
        startButton.SetToolTipString("Run This Network")
        wx.EVT_BUTTON(self,startID,self.onStartButton)
        hsizer.Add(startButton,1,wx.EXPAND)
        stopButton = wx.Button(self,stopID,"Stop")
        stopButton.SetToolTipString("Stop Any Running Network")
        wx.EVT_BUTTON(self,stopID,self.onStopButton)
        hsizer.Add(stopButton,1,wx.EXPAND)
        self.topSizer.Add(hsizer,1,wx.EXPAND)

        saveID = 123
        loadID = 124
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        saveButton = wx.Button(self,saveID,"Save")
        saveButton.SetToolTipString("Save This Network")
        wx.EVT_BUTTON(self,saveID,self.onSaveButton)
        hsizer.Add(saveButton,1,wx.EXPAND)
        loadButton = wx.Button(self,loadID,"Load")
        loadButton.SetToolTipString("Load a Network")
        wx.EVT_BUTTON(self,loadID,self.onLoadButton)
        hsizer.Add(loadButton,1,wx.EXPAND)
        self.topSizer.Add(hsizer,1,wx.EXPAND)

        self.vsizer.Add(self.topSizer,1,wx.EXPAND)

        #init the bottom window
        self.bottomWindow = wx.Panel(self,size=self.bottomSize)
        self.bottomWindow.SetBackgroundColour(wx.Colour(100,150,255))
        self.bottomWindowChild = None
        self.vsizer.Add(self.bottomWindow,2,wx.EXPAND)
        
        #reload the default controller
        #self.reloadNetSpec(self.netSpec)

        #finish window setup
        self.diagram.ShowAll(1)
        self.SetSizer(self.vsizer)
        self.SetAutoLayout(1)
        self.vsizer.Fit(self)
        self.Show(1)

        #register the target callback
        nEventServer.register(EventNames.RMENU_CAT_SELECT_TARGET,self.handleTargetSelection)

    #makes the bar of bitmap buttons used to make behaviors
    #called by __init__
    def makeBehaviorBar(self):
        prefix = os.path.dirname(__file__) + "/gifs/"
        buttonPad = 8

        approachID = 110
        maintainID = 111
        ramID = 112
        moveToID = 113
        avoidLandID = 114
        avoidEntityID = 115
        fireID = 116

        bmp = wx.Bitmap(prefix+"approach.gif",wx.BITMAP_TYPE_GIF)
        b0 = wx.BitmapButton(self,approachID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.APPROACH)
        b0.SetToolTipString("Approach")
        wx.EVT_BUTTON(self,approachID,self.makeApproach)

        bmp = wx.Bitmap(prefix+"maintain.gif",wx.BITMAP_TYPE_GIF)
        b1 = wx.BitmapButton(self,maintainID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.MAINTAIN)
        b1.SetToolTipString("Maintain")
        wx.EVT_BUTTON(self,maintainID,self.makeMaintain)

        bmp = wx.Bitmap(prefix+"ram.gif",wx.BITMAP_TYPE_GIF)
        b2 = wx.BitmapButton(self,ramID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.RAM)
        b2.SetToolTipString("Ram")
        wx.EVT_BUTTON(self,ramID,self.makeRam)

        bmp = wx.Bitmap(prefix+"moveto.gif",wx.BITMAP_TYPE_GIF)
        b3 = wx.BitmapButton(self,moveToID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.MOVETO)
        b3.SetToolTipString("Move To")
        wx.EVT_BUTTON(self,moveToID,self.makeMoveTo)

        bmp = wx.Bitmap(prefix+"gray.gif",wx.BITMAP_TYPE_GIF)
        b4 = wx.BitmapButton(self,avoidLandID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.AVOIDLAND)
        b4.SetToolTipString("Avoid Land")
        wx.EVT_BUTTON(self,avoidLandID,self.makeAvoidLand)

        bmp = wx.Bitmap(prefix+"gray.gif",wx.BITMAP_TYPE_GIF)
        b5 = wx.BitmapButton(self,avoidEntityID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.AVOIDENTITY)
        b5.SetToolTipString("Avoid Entities")
        wx.EVT_BUTTON(self,avoidEntityID,self.makeAvoidEntity)

        bmp = wx.Bitmap(prefix+"fire.gif",wx.BITMAP_TYPE_GIF)
        b6 = wx.BitmapButton(self,fireID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.FIRE)
        b6.SetToolTipString("Fire")
        wx.EVT_BUTTON(self,fireID,self.makeFire)

    	box = wx.StaticBox(self, -1, "", style=wx.SUNKEN_BORDER)
        bsizer 		= wx.StaticBoxSizer(box, wx.HORIZONTAL)
        bsizer.Add	(b0, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b1, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b2, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b3, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b4, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add  (b5, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add  (b6, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        return bsizer

    #loads netSpec into the controller and reloads the display
    def reloadNetSpec(self, netSpec):
        if self.selectedShape:
            selectedShapeBehavior = self.selectedShape.behavior

        #make a netSpec if needed
        if not netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])
        else:
            self.netSpec = netSpec

        #hide the authoring and exec panels to keep things straight
        self.entity.stopAll(None)
        for behavior, shape in self.behaviorShapes.items():
            behavior.Hide()

        #make a controller if needed and load it into entity
        if not self.controller:
            panela = self.entity.authorPanel
            self.controller = Controllers.controller.Controller(panela,self.entity,"",[],self.netSpec)
            self.controller.init()
            self.entity.playback = False
            self.entity.controllerDict[self.controller.getNetName()] = self.controller
            #self.entity.makeController(self.netSpec)
        #else load the netSpec into the existing controller
        else:
            self.controller.makeNetwork(self.netSpec)
            self.stopController()

        self.controller.hide()

        #reset the activity pointers
        self.activeShape = None
        #self.selectedShape = None
        #self.selectedLink = None

        #clear out any open editing window
        #self.clearBottomWindow()

        #reload the ogl display
        self.reloadShapes()

        if self.selectedShape and selectedShapeBehavior in self.behaviorShapes.keys():
            self.selectedShape = self.behaviorShapes[selectedShapeBehavior]
            self.selectedShape.makeBox()
            self.diagram.ShowAll(1)
            self.Refresh()

        #reset the target of all behaviors to targetPath
        #TODO: change this to set individually set targets back right
        if self.targetPath:
            print "reload netspec resetting targets"
            for behavior, shape in self.behaviorShapes.items():
                targetPath = self.targetPath
                if behavior.bname in self.behaviorTarget.keys():
                    targetPath = self.behaviorTarget[behavior.bname]
                self.setBehaviorTarget(behavior,targetPath)

    #reset the display with a controller
    def reloadController(self, controller):
        #copy the controller and get its netSpec
        self.controller = controller
        if self.controller:
            self.netSpec = self.controller.netSpec
        else:
            self.netSpec = None

        #reset the activity pointers
        self.activeShape = None
        self.selectedShape = None
        self.selectedLink = None

        #clear out any open editing window
        self.clearBottomWindow()

        #reload the ogl display
        self.reloadShapes()

    #returns the behavior that matches bspec
    #called by reloadShapes() and reloadLinks() and onAddLink()
    def getBehaviorByBspec(self, bspec):
        ret = None
        if self.controller:
            for bid, behavior in self.controller.behaviorNet.items():
                if bid == bspec.id:
                    ret = behavior
        return ret
    
    #sets the background of the canvas to blue
    #called by reloadShapes()
    def initCanvasBackground(self):
        bgRect = ogl.RectangleShape(1000,1000)
        bgRect.SetX(0)
        bgRect.SetY(0)
        bgRect.SetPen(wx.Pen(wx.Colour(0,0,0),0,wx.TRANSPARENT))
        bgRect.SetBrush(wx.Brush(wx.Colour(0,0,100),wx.SOLID))
        bgRect.SetDraggable(False)
        self.canvas.AddShape(bgRect)

    #makes a behaviorShape
    #called by reloadShapes()
    def makeBehaviorShape(self, bspec, x, y):
        behavior = self.getBehaviorByBspec(bspec)
        shape = behaviorShape.BehaviorShape(x,y,behavior,self.canvas,self.onBehaviorLeftClick,self.onBehaviorRightClick)
        self.behaviorShapes[behavior] = shape
        return shape

    #reloads the ogl display
    def reloadShapes(self):
        #don't try to reload a null or empty netSpec
        if not self.netSpec or len(self.netSpec.behaviors) == 0:
            self.initCanvasBackground()
            self.diagram.ShowAll(1)
            self.Refresh()
            return

        #make a list of behaviors with shapes and behaviors without shapes
        oldBspecs = []
        newBspecs = []
        for bspec in self.netSpec.behaviors:
            found = False
            for behavior, shape in self.behaviorShapes.items():
                if behavior.bname == bspec.id:
                    found = True
            if found:
                x = -1
                y = -1
                for behavior, shape in self.behaviorShapes.items():
                    if behavior.bname == bspec.id:
                        x = shape.GetX()
                        y = shape.GetY()
                if x != -1 and y != -1:
                    oldBspecs.append((bspec,x,y))
                else:
                    newBspecs.append(bspec)
            else:
                newBspecs.append(bspec)

        #clear out the canvas
        self.behaviorShapes = {}
        self.diagram.DeleteAllShapes()
        self.initCanvasBackground()

        #put old shapes back where they were
        for bspec, x, y in oldBspecs:
            self.makeBehaviorShape(bspec,x,y)

        #search for places to put new shapes
        #w = self.canvas.GetWidth()
        #h = self.canvas.GetHeight()
        xStep = 50
        yStep = 50
        xGutter = 50
        yGutter = 50
        w,h = self.canvas.GetSize()
        w = w - xGutter
        h = h - yGutter
        for bspec in newBspecs:
            x = xGutter
            y = yGutter
            placed = False
            while y < h and not placed:
                while x < w and not placed:
                    collides = False
                    for behavior, shape in self.behaviorShapes.items():
                        if shape.collides(x,y):
                            collides = True
                    if not collides:
                        self.makeBehaviorShape(bspec,x,y)
                        placed = True
                    x = x + xStep
                x = xGutter
                y = y + yStep
            if y >= h:
                print "unable to find a good place to put behavior %s" % bspec.id
                self.makeBehaviorShape(bspec,10,10)

        #reload links
        self.reloadLinks()

    #reload the arrows in the ogl display
    def reloadLinks(self):
        #don't try to reload from a null netspec
        if not self.netSpec:
            return

        #init top and bottom x values for all shapes
        for behavior, shape in self.behaviorShapes.items():
            shape.thList = []

        #clear old lines off the canvas
        for key, linkShape in self.linkShapes.items():
            self.canvas.RemoveShape(linkShape)

        #build a dict of shapes by bid
        shapeDict = {}
        for behavior, shape in self.behaviorShapes.items():
            shapeDict[behavior.bname] = shape

        #get shape pairs and save types
        shapePairList = []
        typeDict = {}
        for bspec in self.netSpec.behaviors:
            fromBehavior = self.getBehaviorByBspec(bspec)
            if fromBehavior in self.behaviorShapes.keys():
                fromShape = self.behaviorShapes[fromBehavior]
                for link in bspec.links:
                    if link.dest in shapeDict.keys():
                        toShape = shapeDict[link.dest]
                        shapePairList.append((fromShape,toShape))
                        typeDict[(fromShape,toShape)] = link.type
        
        #make lines using the A* layout function
        lineList = lineLayout.makeLineLayout(self.behaviorShapes,shapePairList,self.canvas.GetSize())
        for line in lineList:
            fromShape = line.fromShape
            toShape = line.toShape
            type = typeDict[(fromShape,toShape)]
            lc = self.onLinkLeftClick
            rc = self.onLinkRightClick
            link = behaviorLink.BehaviorLink(fromShape,toShape,type,line.vertices,self.canvas,lc,rc)
            self.linkShapes[(fromShape.behavior,toShape.behavior)] = link

        #refresh the display
        self.diagram.ShowAll(1)
        self.Refresh()

    #adds a bspec matching behavior to self.netSpec and reloads it
    def addBehavior(self, behavior):
        if self.netSpec:
            sensors = xml.behaviorSpec.BehaviorSensorSpec([],[])
            i = string.find(behavior.bname,'_')
            name = behavior.bname[0:i]
            bspec = xml.behaviorSpec.BehaviorSpec(name,behavior.bname,"0",[],sensors,[],[],[])
            self.netSpec.behaviors.append(bspec)
            self.reloadNetSpec(self.netSpec)

    #removes any bspec matching behavior from self.netSpec and reloads it
    def removeBehavior(self, behavior):
        if self.netSpec:
            if self.selectedShape and self.selectedShape.behavior == behavior:
                self.selectedShape = None
            self.clearBottomWindow()
            for bspec in self.netSpec.behaviors:
                if bspec.id == behavior.bname:
                    self.netSpec.behaviors.remove(bspec)
            self.reloadNetSpec(self.netSpec)

    #adds a link to netSpec and reloads it
    def addLink(self, fromBehavior, toBehavior, type = "ordering"):
        if self.netSpec:
            for bspec in self.netSpec.behaviors:
                if bspec.id == fromBehavior.bname:
                    link = xml.behaviorSpec.Link("to",type,toBehavior.bname)
                    bspec.links.append(link)
            self.reloadNetSpec(self.netSpec)

    #removes a link from netSpec and reloads it
    def removeLink(self, fromBehavior, toBehavior):
        if self.netSpec:
            for frombspec in self.netSpec.behaviors:
                if frombspec.id == fromBehavior.bname:
                    for link in frombspec.links:
                        if link.dest == toBehavior.bname:
                            frombspec.links.remove(link)
            self.reloadNetSpec(self.netSpec)

    #called when a linkShape is left clicked
    def onLinkLeftClick(self, linkShape):
        fromName = linkShape.fromShape.getName()
        toName = linkShape.toShape.getName()
        shapeList = []
        for behavior, shape in self.behaviorShapes.items():
            shapeList.append(shape)
        self.selectedLink = linkShape
        parent = self.bottomWindow
        size = self.bottomSize
        fromBehavior = linkShape.fromShape.behavior
        toBehavior = linkShape.toShape.behavior
        type = linkShape.type
        if self.controller:
            behaviorNet = self.controller.behaviorNet
        else:
            behaviorNet = None
        window = linkMenu.LinkMenu(parent,size,fromBehavior,toBehavior,type,self.behaviorShapes,behaviorNet,self.netSpec,self.reloadNetSpec)
        self.addBottomWindow(window)

    #called when a linkShape is right clicked
    def onLinkRightClick(self, linkShape):
        fromName = linkShape.fromShape.getName()
        toName = linkShape.toShape.getName()
        self.removeLink(linkShape.fromShape.behavior,linkShape.toShape.behavior)

    #called when a behaviorShape is left clicked
    def onBehaviorLeftClick(self, behaviorShape):
        if self.selectedShape == behaviorShape:
            self.selectedShape.removeBox()
            self.Refresh()
            self.selectedShape = None
            self.clearBottomWindow()
        else:
            if self.selectedShape:
                self.selectedShape.removeBox()
            self.selectedShape = behaviorShape
            self.selectedShape.makeBox()
            self.diagram.ShowAll(1)
            self.Refresh()
            window = behaviorMenu.BehaviorMenu(self,self.bottomSize,behaviorShape,self.entity,self.controller,self.reloadShapes)
            self.addBottomWindow(window)

    #called when a behaviorShape is right clicked
    def onBehaviorRightClick(self, behaviorShape):
        self.removeBehavior(behaviorShape.behavior)

    #called each time interval
    def tick(self, time, dtime):
        self.checkControllerChanged()
        self.checkShapesMoved()
        self.updateActiveShape()

    #if the controller changed reload it
    #called by tick
    def checkControllerChanged(self):
        if self.entity.currentControllerName != self.lastControllerName:
            self.lastControllerName = self.entity.currentControllerName
            self.reloadController(self.entity.getController(self.entity.currentControllerName))

    #if any shapes moved reload links
    #called by tick
    def checkShapesMoved(self):
        moved = False
        for behaivor, shape in self.behaviorShapes.items():
            if shape.GetX() != shape.lastx or shape.GetY() != shape.lasty:
                moved = True
                shape.lastx = shape.GetX()
                shape.lasty = shape.GetY()
                if shape == self.selectedShape:
                    shape.updateBox()
        if moved:
            self.reloadLinks()

    #activeShape is rendered in red
    #called by tick
    def updateActiveShape(self):
        self.lastActiveShape = self.activeShape
        for behavior, shape in self.behaviorShapes.items():
            if shape.behavior.runnable and shape.behavior.checkPrePorts() and shape.behavior != self.lastActiveShape:
                self.activeShape = shape
        if self.activeShape != self.lastActiveShape:
            print "active shape set to %s" % self.activeShape.behavior.bname
            for behavior, shape in self.behaviorShapes.items():
                shape.SetBrush(wx.Brush(wx.Colour(100,100,255),wx.SOLID))
                shape.active = False
            self.activeShape.SetBrush(wx.Brush(wx.Colour(255,0,0),wx.SOLID))
            self.activeShape.active = True
            self.Refresh()

    #callback for add link button, adds link
    def onAddLink(self, e):
        #make a netSpec if needed, this should've already happened in the constructor
        if not self.netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])

        #find two unlinked shapes using netspec
        fromBehavior = None
        toBehavior = None


        #don't search an empty 
        if len(self.netSpec.behaviors) > 0:

            #calculate traffic in and out of each node
            numIn = {}  #dict of number of connections going in by behavior
            numOut = {} #same but going out
            for bspec in self.netSpec.behaviors:
                behavior = self.getBehaviorByBspec(bspec)
                numOut[behavior] = len(bspec.links)
                numIn[behavior] = 0
                for outBspec in self.netSpec.behaviors:
                    for link in outBspec.links:
                        if link.dest == bspec.id:
                            numIn[behavior] = numIn[behavior] + 1
    
            numBehaviors = len(self.netSpec.behaviors)

            #search the sorted traffic dicts for a pair that isn't very connected
            from operator import itemgetter
            for bestToBehavior, bestNumIn in sorted(numIn.items(),key=itemgetter(1)):
                for bestFromBehavior, bestNumOut in sorted(numOut.items(),key=itemgetter(1)):
                    if (not fromBehavior or not toBehavior) and (bestFromBehavior, bestToBehavior) not in self.linkShapes.keys() and bestFromBehavior != bestToBehavior:
                        fromBehavior = bestFromBehavior
                        toBehavior = bestToBehavior

        #if found add link
        if fromBehavior and toBehavior:
            self.addLink(fromBehavior,toBehavior,"ordering")
        else:
            print "error adding link: network already fully connected"

    #these functions are callbacks for the behavior buttons
    #they just call makeBehavior with the right bname out of behaviorNames
    def makeApproach(self, e):
        self.makeBehavior(behaviorNames.APPROACH)

    def makeMaintain(self, e):
        self.makeBehavior(behaviorNames.MAINTAIN)

    def makeRam(self, e):
        self.makeBehavior(behaviorNames.RAM)

    def makeMoveTo(self, e):
        self.makeBehavior(behaviorNames.MOVETO)

    def makeAvoidLand(self, e):
        self.makeBehavior(behaviorNames.AVOIDLAND)

    def makeAvoidEntity(self, e):
        self.makeBehavior(behaviorNames.AVOIDENTITY)

    def makeFire(self, e):
        self.makeBehavior(behaviorNames.FIRE)

    #makes a behavior id based on a behavior name
    #called by makeBehavior
    def makeBid(self, bname):
        i = 1
        id = bname + "_1"
        ok = False
        while not ok and self.controller:
            id = bname + "_" + str(i)
            ok = True
            for bid, behavior in self.controller.behaviorNet.items():
                if id == bid:
                    ok = False
            i = i + 1
        return id

    #makes a behavior, adds it to the controller, and clicks it
    #called by the make(some bname) functions
    def makeBehavior(self, bname):
        #make a bid
        bid = self.makeBid(bname)

        #make a pbc to get default params out of
        panelA = self.entity.authorPanel
        pbc = Controllers.primitiveBehaviorController.PrimitiveBehaviorController(panelA,self.entity,bname=bname,bid=bid)
        behavior = pbc.behavior

        #init some behaviorSpec attributes
        sensors = xml.behaviorSpec.BehaviorSensorSpec([],[])
        params = []

        #copy pbc params
        for paramName in behavior.params.keys():
            val, typ, min, max = behavior.params[paramName]
            params.append(xml.behaviorSpec.Parameter(paramName,type,val,min,max))

        #make behaviorSpec
        behaviorSpec = xml.behaviorSpec.BehaviorSpec(bname,bid,"1",[],sensors,params,[],[])

        #make a netSpec if needed, this should've already happened in the constructor
        if not self.netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])

        #add the behaviorSpec to the netSpec and reload it
        self.netSpec.behaviors.append(behaviorSpec)
        self.reloadNetSpec(self.netSpec)

        #find the shape we added using its bid and simulate a click on it to open the edit window
        #also set its target if one has been specified
        shape = None
        for testBid, testShape in self.behaviorShapes.items():
            if testBid == bid:
                shape = testShape
        if shape:
            self.onBehaviorLeftClick(shape)

    #starts controller
    def startController(self):
        if self.controller:
            for behavior, shape in self.behaviorShapes.items():
                behavior.stop()
                behavior.Hide()
            self.entity.stopAll(None)
            self.entity.currentControllerName = self.controller.name
            self.entity.playback = True
            self.controller.start()
            #self.entity.handlePlayRequest(self.controller.name,None)

    #stops controller
    def stopController(self):
        if self.controller:
            for behavior, shape in self.behaviorShapes.items():
                behavior.stop()
                behavior.Hide()
            self.entity.stopAll(None)
            self.entity.hideAllPBCs()
            #self.controller.stop()
            self.entity.handlePauseRequest(self.controller.name)
            self.entity.playback = False

            #reset the active shape
            self.activeShape = None
            for behavior, shape in self.behaviorShapes.items():
                shape.SetBrush(wx.Brush(wx.Colour(100,100,255),wx.SOLID))
                shape.active = False
            self.Refresh()

    #restarts controller
    def restartController(self):
        self.stopController()
        self.startController()

    #button callback for start button
    def onStartButton(self, e):
        self.startController()

    #button callback for stop button
    def onStopButton(self, e):
        self.stopController()

    #handles target selection
    #if a behavior menu is open, it assigns the target to that behavior
    #else it just assigns it to the whole network
    def handleTargetSelection(self, msg):
        args = defs.parseRMenuMsg(msg)
        self.targetPath = args.objectsOver[0]
        print "netform setting target path to %s" % self.targetPath
        if self.IsShown():
            if self.selectedShape:
                print "setting selected shape target path to %s" % self.targetPath
                self.setBehaviorTarget(self.selectedShape.behavior,self.targetPath)
            else:
                for behavior, behaviorShape in self.behaviorShapes.items():
                    print "%s setting target to %s" % (behavior.bname,self.targetPath.split('/')[-1])
                    self.setBehaviorTarget(behaviorShape.behavior,self.targetPath)

    def setBehaviorTarget(self, behavior, targetPath):
        if behavior:
            behavior.setTarget(targetPath)
            self.behaviorTarget[behavior.bname] = targetPath

    #handles page change
    def handlePageChange(self, lastMode, mode):
        print "netForm.handlePageChange(): lastMode = %s, mode = %s" % (lastMode,mode)
        if mode == 3 and lastMode == 0:
            print "going from authoring to netvis, stopping controller"
            self.entity.stopAll(None)

    #callback for save button
    def onSaveButton(self, e):
        #make a netSpec if needed, this should've already happened in the constructor
        if not self.netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])
        self.addBottomWindow(saveMenu.SaveMenu(self.bottomWindow,self.bottomSize,self.netSpec.name,self.onSave,self.clearBottomWindow))

    #saves netspec to filename
    def onSave(self, filename):
        if self.netSpec:
            xml.xmlWriter.generateNetSpecXML(self.netSpec,filename)

    #callback for load button
    def onLoadButton(self, e):
        netSpecName = ""
        if self.netSpec:
            netSpecName = self.netSpec.name
        self.addBottomWindow(loadMenu.LoadMenu(self.bottomWindow,self.bottomSize,netSpecName,self.onLoad,self.clearBottomWindow))

    #tries to load netspec from filename
    def onLoad(self, filename):
        netSpec = None
        try:
            netSpec = xml.xmlReader.slurp(filename)
        except:
            print "unable to load netSpec from %s" % filename
        if netSpec:
            self.reloadNetSpec(netSpec)

    #kill any open editing window
    def clearBottomWindow(self):
        if self.bottomWindowChild:
            self.bottomWindowRet = self.bottomWindowChild.cleanup()
            #self.bottomWindow.RemoveChild(self.bottomWindowChild) #lol segfault
            self.bottomWindowChild.Hide()
            #del self.bottomWindowChild
            self.bottomWindowChild = None
            self.Refresh()
            self.bottomWindow.Refresh()

    #show window in the bottom section of the overall form
    def addBottomWindow(self, window):
        if self.bottomWindowChild:
            self.clearBottomWindow()
        self.bottomWindowChild = window
        window.Reparent(self.bottomWindow)
        #window.SetSize(self.bottomSize)
        window.SetPosition((0,0))
        window.Show(1)
        window.Refresh()






