import xml.sax
from xml.sax.handler import ContentHandler
from xml.sax.saxutils import XMLGenerator
from xml.sax.xmlreader import AttributesImpl as Att
import sys

class NetworkHandler(ContentHandler):
    start = False
    end = False
    parameter = False
    def __init__(self):
        self.behaviors = []

    def startElement(self, name, attrs):
        if name == 'behavior':
            self.behavior = BehaviorNode()
            self.behavior.behaviorName = attrs.get('name', "")
        elif name == 'pretrigger':
            id = attrs.get('id', "")
            self.behavior.pretriggers.append(id)
        elif name == 'posttrigger':
            id = attrs.get('id', "")
            self.behavior.posttriggers.append(id)
        elif name == 'start':
            self.start = True
            self.behavior.startTime = ""
        elif name == 'end':
            self.end = True
            self.behavior.endTime = ""
        elif name == 'parameter':
            self.pVal = ""
            self.pName = attrs.get('name', "")
            type = attrs.get('type', "")
            min = attrs.get('min', "")
            max = attrs.get('max', "")
            val = None
            self.behavior.parameters[self.pName] = (type, min, max, val)
            self.parameter = True

    def endElement(self, name):
        if name == 'start':
            self.start = False
        if name == 'end':
            self.end = False
        if name == 'parameter':
            self.parameter = False
            type, min, max, val = self.behavior.parameters[self.pName]
            self.behavior.parameters[self.pName] = (type, min, max, self.pVal)
        if name == 'behavior':
            self.behaviors.append(self.behavior)

    def characters(self, buf):
        if self.start:
            self.behavior.startTime += buf
        if self.end:
            self.behavior.endTime += buf
        if self.parameter:
            self.pVal += buf

nextId = 0

class BehaviorNode:
    def __init__(self):
        global nextId
        self.id = nextId
        nextId += 1
        self.behaviorName = None
        self.pretriggers = []
        self.posttriggers = []
        self.startTime = None
        self.endTime = None
        self.parameters = {}

    def equals(self, other):
        if self.__class__ == other.__class__:
            if self.behaviorName == other.behaviorName:
                #for id in self.pretriggers:
                #    if id not in other.pretriggers:
                #        return False
                #for id in self.posttriggers:
                #    if id in other.posttriggers:
                #        return False
                for key1, val1 in self.parameters.iteritems():
                    for key2, val2 in other.parameters.iteritems():
                        if key1 == key2 and val1 != val2:
                            return False
                return True
        return False

class xml_logger:
    currentTabs = 0
    def __init__(self, output):
        logger = XMLGenerator(output)
        logger.startDocument()
        self._logger = logger
        self._output = output
        self.output = output
        
    def WriteEntry(self, name, atts, buf):
        attrs = Att(atts)
        print >> self._output, '\t' * (self.currentTabs),
        self._logger.startElement(name, atts)
        self._logger.characters(buf)
        self._logger.endElement(name)
        print >> self._output, '\n',

    def OpenTag(self, name, atts, nl):
        attrs = Att(atts)
        print >> self._output, '\t' * (self.currentTabs),
        self.currentTabs += 1
        self._logger.startElement(name, atts)
        if nl:   
            print >> self._output, '\n',

    def CloseTag(self, name):
        self.currentTabs -= 1
        print >> self._output, '\t' * self.currentTabs,
        self._logger.endElement(name)
        print >> self._output, '\n',
        
    def Close(self):
        pass
        self._logger.endDocument()

def ConstructNetwork(infile, outfile, netname, triggerString=''):
    def condense(behaviors):
        change = True
        while(change):
            change = False
            pss = behaviors[:]
            for n1 in pss:
                for n2 in behaviors:
                    s1 = int(n1.startTime)
                    s2 = int(n2.startTime)
                    e1 = int(n1.endTime)
                    e2 = int(n2.endTime)
                    if (s1 != s2 or e1 != e2) and \
                       n1.equals(n2) and \
                       (s2 - e1 <= 20 and s2 - e1 >= 0):
                        n1.endTime = n2.endTime
                        if len(n2.pretriggers) != 0 and behaviors.index(n2)+1 < len(behaviors):
                            behaviors[behaviors.index(n2)+1].pretriggers = n2.pretriggers
                        behaviors.remove(n2)
                        change = True
                        
    def filter(behaviors):
        for n in behaviors:
            s = int(n.startTime)
            e = int(n.endTime)
            if e - s < 10:
                behaviors.remove(n)
                
    def build(behaviors, outfile):
        #f = open("cat/networks/"+outfile, "w")
        logger = xml_logger(outfile)
        #do stuff
        id = 0
        for n in behaviors:
            n.id = id
            id += 1
        logger.OpenTag('network', {'name':netname}, True)
        print >> outfile, triggerString
        for n in behaviors:
            logger.OpenTag('behavior', {'name':n.behaviorName, 'id':`n.id`, 'group':'0'}, True)
            logger.OpenTag('parameters', {}, True)
            for name, tup in n.parameters.iteritems():
                type, min, max, val = tup 
                logger.WriteEntry(type, {'name':name, 'min':min, 'max':max}, val)
            logger.CloseTag('parameters')
            logger.OpenTag('sensors', {}, True)
            for id in n.pretriggers:
                logger.WriteEntry('pre', {'id':id}, '')
            for id in n.posttriggers:
                logger.WriteEntry('post', {'id':id}, '')
            logger.CloseTag('sensors')
            logger.OpenTag('links', {}, True)
            for m in behaviors:
                if n.id < m.id:
                    s1 = int(n.startTime)
                    s2 = int(m.startTime)
                    e1 = int(n.endTime)
                    e2 = int(m.endTime)
                    if e1 >= e2:
                        logger.WriteEntry('to', {'type':'permanent'}, str(m.id))
                    elif e1 < e2 and e1 >= s2:
                        logger.WriteEntry('to', {'type':'enabling'}, str(m.id))
                    elif e1 < s2:
                        logger.WriteEntry('to', {'type':'ordering'}, str(m.id))
            logger.CloseTag('links')
            logger.CloseTag('behavior')
        logger.CloseTag('network')
        logger.Close()
        #stop
        #f.close()

    def partition(behaviors, lo, hi):
        pivot = int(behaviors[lo].startTime)
        while True:
            while int(behaviors[hi].startTime) >= pivot and lo < hi:
                hi -= 1
            while int(behaviors[lo].startTime) < pivot and lo < hi:
                lo += 1
            if lo < hi:
                tmp = behaviors[lo]
                behaviors[lo] = behaviors[hi]
                behaviors[hi] = tmp
            else: return hi

    def quicksort(behaviors, p, r):
        if p < r:
            q = partition(behaviors, p, r)
            if q == r:
                q -= 1
            quicksort(behaviors, p, q)
            quicksort(behaviors, q+1, r)
    
    parser = xml.sax.make_parser()
    handler = NetworkHandler()

    tmpfile = open("cat/networks/recorderoutput.xml", "w")
    print >> tmpfile, triggerString, infile
    tmpfile.close()
    xml.sax.parseString(infile, handler)
    behaviors = handler.behaviors
    quicksort(behaviors, 0, len(behaviors)-1)
    condense(behaviors)
    filter(behaviors)
    build(behaviors, outfile)
