#from cat import gui
import cat
import triggers
import constructor
import wx
from cat.Interface import sensors

from StringIO import *

class Recorder:

    def __init__(self):
        self.out = ''
        self.hasMaterial = False
        self.recording = False
        self.behaviors = []
        self.startTime = {}
        self.endTime = {}
        self.runningTime = 0
        self.usedTriggers = {}
        self.trigid = 0
        self.slots = {}
        self.triggerString = ''

    def Tick(self, time, dtime):
        if self.recording:
            triggers.Tick(time, dtime)
            change = triggers.stateChanged
            triggerList = triggers.triggerList
            for beh in self.behaviors:
                if (beh.IsActive()) and not change:
                    if self.startTime[beh] == -1:
                        self.startTime[beh] = self.runningTime
                else:
                    if not self.startTime[beh] == -1:
                        self.endTime[beh] = self.runningTime

                        idlist = []            
                        if len(triggerList) > 0:
                            for trig in triggerList:
                                self.usedTriggers[self.trigid] = trig
                                idlist.append(self.trigid)
                                self.trigid += 1
                            triggerList = []

                        if change:
                            beh.postTriggers.extend(idlist)

                        self.out = self.out + '\t<behavior name="'+str(beh.__module__)+'">\n'
                        for id in beh.preTriggers:
                            self.out += '\t\t<pretrigger id="'+str(id)+'" />\n'
                        for id in beh.postTriggers:
                            self.out += '\t\t<posttrigger id="'+str(id)+'" />\n'

                        if change:
                            for id in idlist:
                                beh.postTriggers.remove(id)

                        self.out = self.out + '\t\t<start>'+str(self.startTime[beh])+'</start>\n'
                        self.out = self.out + '\t\t<end>'+str(self.endTime[beh])+'</end>\n'

                        for key, value in beh.vars.iteritems():
                            self.out = self.out + '\t\t<parameter name="'+key+\
                                  '" type="'+str(value[1])+ \
                                  '" min="'+str(value[2])+ \
                                  '" max="'+str(value[3])+ \
                                  '">'+str(value[0])+'</parameter>\n'
                        self.out = self.out + '\t</behavior>\n'
                        self.startTime[beh] = -1

                        if change:
                            beh.preTriggers.extend(idlist)

                        self.slots[sensors.GetTargetEntity()] = 'target'
                        self.slots[sensors.GetOwnEntity()] = 'trainee'

            self.runningTime += 1

    def SetBehaviors(self, behList):
        self.behaviors = behList
        for beh in self.behaviors:
            self.startTime[beh] = -1
            self.endTime[beh] = -1

    def SetRecording(self, recording):
        self.recording = recording
        if recording:
            self.out = "<run>\n"
            self.hasMaterial = True
        else:
            self.hasMaterial = False
            for beh in self.behaviors:
                if not self.startTime[beh] == -1:
                    self.endTime[beh] = self.runningTime
                    self.out = self.out + '\t<behavior name="'+str(beh.__module__)+'">\n'

                    for id in beh.preTriggers:
                        self.out += '\t\t<pretrigger id="'+str(id)+'" />\n'
                    for id in beh.postTriggers:
                        self.out += '\t\t<posttrigger id="'+str(id)+'" />\n'

                    self.out = self.out + '\t\t<start>'+str(self.startTime[beh])+'</start>\n'
                    self.out = self.out + '\t\t<end>'+str(self.endTime[beh])+'</end>\n'

                    for key, value in beh.vars.iteritems():
                        self.out = self.out + '\t\t<parameter name="'+key+\
                              '" type="'+str(value[1])+ \
                              '" min="'+str(value[2])+ \
                              '" max="'+str(value[3])+ \
                              '">'+str(value[0])+'</parameter>\n'
                    self.out = self.out + '\t</behavior>\n'

                    self.slots[sensors.GetTargetEntity()] = 'target'
                    self.slots[sensors.GetOwnEntity()] = 'trainee'
            self.PrintTriggers()
    
    def PrintTriggers(self):
        i = 65
        self.triggerString += '\t<triggers>\n'
        for id, val in self.usedTriggers.iteritems():
            sensor, parameters = val
            self.triggerString += '\t\t<trigger name="'+sensor['sensorType']+'" id="'+`id`+'">\n'
            if sensor['type'] != 'entlist':
                self.triggerString += '\t\t\t<parameter name="state" type="'+sensor['type']+'" value="'+`sensor['value']`+'" />\n'
            for param in parameters:
                if param['type'] == 'ent':
                    if self.slots.has_key(param['value']):
                        param['value'] = self.slots[param['value']]
                    else:
                        self.slots[param['value']] = '%c'%i
                        param['value'] = self.slots[param['value']]
                        i += 1
                    self.triggerString += '\t\t\t<slot name="'+param['name']+'" id="'+param['value']+'" />\n'
                else:
                    self.triggerString += '\t\t\t<parameter name="'+param['name']+'" type="'+param['type']+'" value="'+`param['value']`+'" />\n'
            self.triggerString += '\t\t</trigger>\n'
        self.triggerString += '\t</triggers>\n\t<slots>\n'
        for path, id in self.slots.iteritems():
            self.triggerString += '\t\t<entity name="'+path+'" id="'+id+'" />\n'
        self.triggerString += '\t</slots>\n'
        #print self.triggerString
            
    def Save(self, name, netname, savetofile = True):
        if not self.recording:
            outfilename = "cat/networks/"+name
            if savetofile:
                outfile = open(outfilename, "w")
            else:
                outfile = StringIO()
            self.out = self.out + "</run>\n"
            infile = self.out
            constructor.ConstructNetwork(infile, outfile, netname, self.triggerString)

            if not savetofile:
                outstr = outfile.getvalue()
                outfile.close()
                return outstr
            else:
                outfile.close()
            self.out = ''
        else:
            self.SetRecording(False)
            return self.Save(name, netname, savetofile)

    def IsRecording(self):
        return self.recording
