from sensor import Sensor
from cat import catInfo
from common.defs import *
import math

class DistanceSensor(Sensor):
    def __init__(self, toPath=None, fromPath=None):
        Sensor.__init__(self)
        self.toPath = toPath
        self.fromPath = fromPath

    def tick(self, time, dtime):
        Sensor.tick(self, time, dtime)
        self.sensorVal = self.getDistance()

    def getDistance(self):
        try:
            if not exists(self.toPath) or not exists(self.fromPath):
                return -1
            pushsel()
            sel(self.toPath)
            ox, oz, oy = gv('pos')
            sel(self.fromPath)
            tx, tz, ty = gv('pos')
            popsel()

            dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            return dist
        except:
            printExcept('DistanceSensor')
            return 0
    
    def handleBindingChange(self, oldPath, newPath):
        if self.toPath == oldPath:
            self.toPath = newPath
        if self.fromPath == oldPath:
            self.fromPath = newPath

class DistanceThreshold:
    def __init__(self, type, value=None):
        self.type = type
        self.value = value
        if value == None:
            self.value = 0
            self.set = False
        else:
            self.value = value
            self.set = True

    def isSet(self):
        return self.set

    def fire(self, sensor=None):
        if sensor:
            self.value = sensor.getValue()
            self.set = True

    def check(self, sensor):
        val = sensor.getValue()
        if type == catInfo.LESS_THAN:
            return val < self.value
        elif type == catInfo.GREATER_THAN:
            return val > self.value
        elif type == catInfo.APPROX:
            return val < value*1.05 and val > value * 0.95

    def getValue(self):
        return self.value
