from cat import catInfo
from distancesensor import *
from alertsensor import *
from damagesensor import *
from cat.xml.behaviorSpec import TriggerSpec
from common import tickmanager

class Trigger:
    def __init__(self, entity, name, type, subtype=None, threshold=None, toPath=None, fromPath=None):
        self.entity = entity
        self.name = name
        self.type = type
        self.subtype = subtype
        self.fired = False

        if self.entity.mode == 0:
            tickmanager.register(self.name, self.tick)

        if type == catInfo.DISTANCE_SENSOR:
            self.sensor = DistanceSensor(toPath, fromPath)
            self.threshold = DistanceThreshold(subtype, threshold)
        elif type == catInfo.ALERT_SENSOR:
            self.sensor = AlertSensor()
            self.threshold = AlertThreshold()
        if type == catInfo.DAMAGE_SENSOR:
            self.sensor = DamageSensor(toPath)
            self.threshold = DamageThreshold(subtype, threshold)

    def tick(self, time, dtime):
        self.sensor.tick(time, dtime)

    def getName(self):
        return self.name

    def getType(self):
        return self.type

    def getSubtype(self):
        return self.subtype

    def fire(self):
        if self.fired:
            return
        self.fired = True
        if self.entity.mode == 0: #authoring mode
            self.threshold.fire(self.sensor)
            #print "************* threshold:", self.getThreshold(), self.entity.entPath
            triggerSpec = TriggerSpec(self.name, self.type, self.subtype,
                                      self.getThreshold(), self.getTo(), self.getFrom())
            self.entity.handleTrigger(triggerSpec)
        elif self.entity.mode == 1: #execution mode
            self.threshold.fire()
    
    def hasFired(self):
        return self.fired

    def hasThreshold(self):
        return self.threshold.isSet()

    def getThreshold(self):
        return self.threshold.getValue()

    def checkThreshold(self):
        if self.threshold.isSet():
            self.fired = self.fired or self.threshold.check(self.sensor)
        
    def setTo(self, path):
        self.sensor.setTo(path)

    def setFrom(self, path):
        self.sensor.setFrom(path)

    def getTo(self):
        return self.sensor.getTo()

    def getFrom(self):
        return self.sensor.getFrom()

    def getValue(self):
        return self.sensor.getValue()
