#
# @(#) xmlReader.py 2005/11/13
# @author SJL Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#


#from common import behaviorNames

import StringIO
import behaviorSpec
from behaviorSpec import *

import xml.dom.minidom

def slurp(pathname):
    dom = xml.dom.minidom.parse(pathname)
    nlist = handleDom(dom)
    print "done parsing"
    dom.unlink()
    return nlist[0] # There should only be one net per file
    
def handleDom(dom):
    nlist = []
    networkList = dom.getElementsByTagName('network')
    for network in networkList:
        print "network %s: " % network.getAttribute("name")
        nlist.append(handleNetwork(network))
    return nlist

def handleNetwork(network):
    name = network.getAttribute('name')
    ownship = network.getAttribute('ownship')
    #triggerLists = network.getElementsByTagName('triggers')
    #slotsLists = network.getElementsByTagName('slots')
    blist = network.getElementsByTagName('behavior')
    #tlists = []
    #for triggerList in triggerLists:
    #    #print 'handling trigger: ', str(triggerList)
    #    tl = handleTriggerList(triggerList)
    #    #print 'appending trigger: ', str(tl)
    #    tlists.append(tl)
    #slists = []
    #for slotList in slotsLists:
    #    #print 'handling slot: ', str(slotList)
    #    sl = handleSlotList(slotList)
    #    #print 'Appending slot: ', str(sl)
    #    slists.append(sl)
    behaviorList = []
    for behavior in blist:
        #print 'Handling behavior: ', str(behavior)
        b = handleBehavior(behavior)
        #print 'Appending behavir: ', str(b)
        behaviorList.append(b)
    ns = behaviorSpec.NetworkSpec(name = name, ownship = ownship, behaviors = behaviorList)
    return ns

def handleBehavior(behavior):
    name  = str(behavior.getAttribute("name"))
    group = str(behavior.getAttribute("group"))
    id    = str(behavior.getAttribute("id"))
    #print 'HandleBehavior: name: ', name
    targetList    = behavior.getElementsByTagName('target')
    parameterList = behavior.getElementsByTagName("parameters")
    #sensorsList   = behavior.getElementsByTagName("sensors")
    bindingsList  = behavior.getElementsByTagName("bindings")    
    linksList     = behavior.getElementsByTagName("links")
    triggerList   = behavior.getElementsByTagName('triggers')
    tlist = []
    bindings = None
    for target in targetList:
        t = getText(target)
        tlist.append(t)
    for parameters in parameterList: 
        params = handleParameters(parameters)
    #for sensors in sensorsList:
    #    senses = handleSensors(sensors)
    for trigger in triggerList:
        triggers = handleTriggers(trigger)
    for bind in bindingsList:
        bindings = handleBindings(bind, id)
    for links in linksList:
        lnks = handleLinks(links)
    
    bs = behaviorSpec.BehaviorSpec(name = name, group=group, id = id, targets = tlist,
                                   parameters = params, links = lnks,
                                   bindings = bindings, triggers = triggers)
    return bs

def handleBindings(bindings, bid):
    binList = []
    #    print "Bindings are ", bindings
    blist = bindings.getElementsByTagName('binding')
    if (not bindings):
        return []
    for bind in blist:
        name = str(bind.getAttribute('name'))
        val  = str(bind.getAttribute('path'))
        # print "Binding: %s to %s " % (name, val)
        binSpec = Binding(name = name, val = val, humanReadable = bid)
        binList.append(binSpec)
        # end for
    return binList


def handleParameters(parameters):
    paramList = []
    boolList   = parameters.getElementsByTagName('bool')
    intList    = parameters.getElementsByTagName('int')
    floatList  = parameters.getElementsByTagName('float')
    vecList    = parameters.getElementsByTagName('vec')    
    for bv in boolList:
        paramList.append(handleParam(bv, 'bool'))
    for iv in intList:
        paramList.append(handleParam(iv, 'int'))        
    for fv in floatList:
        paramList.append(handleParam(fv, 'float'))        
    for vv in vecList:
        paramList.append(handleParam(vv, 'vec'))
    return paramList


def handleParam(bv, type):
    name = str(bv.getAttribute('name'))
    #print "Handling Parameter: ", name, str(bv)
    min  = str(bv.getAttribute('min'))
    max  = str(bv.getAttribute('max'))
    vv   = getText(bv)
    v  = convert(vv, type)
    mn = convert(min, type)
    mx = convert(max, type)
    ps = behaviorSpec.Parameter(name=name, type = type, min=mn, max = mx, val=v) 
    return ps 

def convert(val, type):
    if(type.startswith('int')):
        v = int(float(val))
    elif (type.startswith('bool')):
        # print "val = ", str(val)
        if(val.lower().startswith("tr")):
            v = True
        else:
            v = False
    elif (type.startswith('float')):
        v = float(val)
    elif (type.startswith('vec')):
        v = vector3(val[1:-1])
    return v

def handleTriggers(triggers):
    triggerList = triggers.getElementsByTagName('trigger')
    trigs = []
    for trigger in triggerList:
        name = str(trigger.getAttribute('name'))
        type = str(trigger.getAttribute('type'))
        id = str(trigger.getAttribute('id'))
        subtype = topath = frompath = theshold = None
        subtypes = trigger.getElementsByTagName('subtype')
        if subtypes:
            subtype = getText(subtypes[0])
        topaths = trigger.getElementsByTagName('to')
        if topaths:
            topath = getText(topaths[0])
        frompaths = trigger.getElementsByTagName('from')
        if frompaths:
            frompath = getText(frompaths[0])
        thresholds = trigger.getElementsByTagName('threshold')
        if thresholds:
            threshold = getText(thresholds[0])
        tspec = TriggerSpec(name, type, subtype, threshold, topath, frompath, id)
        trigs.append(tspec)
    return trigs

#def handleSensors(sensors):
#    #print "Handling sensors", str(sensors)
#    preList = sensors.getElementsByTagName("pre")
#    prel = []
#    for pre in preList:
#        sSpec = handleSensorList(pre)
#        prel.append(sSpec)
#        # end for
#    postList = sensors.getElementsByTagName('post')
#    posl = []
#    for post in postList:
#        sSpec = handleSensorList(post)
#        posl.append(sSpec)
#        #end for
#    s = BehaviorSensorSpec(pre = prel, post = posl)
#    return s

#def handleSensorList(sensor):
#    id   = str(sensor.getAttribute('id'))
#    type = str(sensor.getAttribute('type'))
#    name = str(sensor.getAttribute('name'))
#    paramList = sensor.getElementsByTagName('parameter')
#    paramSpecList = handleSensorParams(paramList)
#    sSpec = SensorSpec(name=name, id = id, type = type, params = paramSpecList)
#    return sSpec

#def handleSensorParams(paramList):
#    plist = []
#    for param in paramList:
#        name  = str(param.getAttribute('name'))
#        type  = str(param.getAttribute('type'))
#        value = str(param.getAttribute('value'))
#        # print "Name: %s, type: %s, value: %s" % (name, type, value)
#        pSpec = behaviorSpec.SensorParamSpec(name = name, type = type, value = value)
#        plist.append(pSpec)
#        # end for
#    return plist



def handleLinks(links):
    #print "Handling links", str(links)
    toList = links.getElementsByTagName("to")
    tl = []
    for to in toList:
        type = str(to.getAttribute("type"))
        id = str(to.getAttribute('trigid'))
        toWho = getText(to)
        l = Link(dir = 'to', type = type, dest = toWho, triggerid=id)
        tl.append(l)
    return tl


#def handleSlotList(slotList):
#    #print "Handling slotList:", str(slotList)
#    entities = slotList.getElementsByTagName("entity")
#    #print "\tslots:"
#    elist = []
#    for entity in entities:
#        name = entity.getAttribute("name")
#        id   = entity.getAttribute("id")
#        ss = SlotSpec(name = name, id = id)
#        elist.append(ss)
#    return elist

def getText(nodelist):
    tag = nodelist.tagName
    rc = ""
    for node in nodelist.childNodes:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return str(rc).strip()



#def handleTriggerList(triggerList):
#    #print "Handling triggerList:", str(triggerList)
#    triggers = triggerList.getElementsByTagName('trigger')
#    tl = []
#    for trigger in triggers:
#        ts = handleTrigger(trigger)
#        tl.append(ts)
#    return tl

#def handleTrigger(trigger):
#    name = str(trigger.getAttribute('name'))
#    id   = str(trigger.getAttribute('id'))
#    triggerParamList = trigger.getElementsByTagName('parameter')
#    triggerSlotList  = trigger.getElementsByTagName('slot')
#    tpl = []
#    for parameter in triggerParamList:
#        pname = str(parameter.getAttribute('name'))
#        ptype = str(parameter.getAttribute('type'))
#        pval = str(parameter.getAttribute('value'))
#        pv   = convert(pval, ptype)
#        tps = TriggerParameterSpec(name = pname, type = ptype, val = pv)
#        tpl.append(tps)
#    tsl = []
#    for slot in triggerSlotList:
#        sname = str(slot.getAttribute('name'))
#        sid = str(slot.getAttribute('id'))
#        tss = TriggerSlotSpec(name = sname, id = sid)
#        tsl.append(tss)
#    triggerSpec = TriggerSpec(name = name, id = id, triggers=tpl, slots=tsl)
#    return triggerSpec

#slurp('sc1.bn')
