from npython import *
from common.defs import *
"""
    loads entity definition xml files - creating them in the /lib/ent directory
"""
import sys
import inspect
from xml.dom import minidom
from xml.dom.minidom import getDOMImplementation

from common.log import Log
log = Log(tags=['all','EntityDefiner'])
from common.defs import struct
#/-----------------------------------------------------------------------------//
def parse(filename):
    """parses an xml file - creating the entity definition"""
    from common.defs import *
    pushsel()
    log.open('entityParser.parse(%s)' % filename)
    try:
        file = open( filename )
        document = minidom.parse( file )
    except:
        printExcept('parsing %s' % file)
        file.close()
        return
    file.close()

    for ed in document.getElementsByTagName('entityDefinition'): #foreach entitydefinition node
        entType = getText( ed )
        log.open('ent', entType)
        if exists('/lib/ents/%s' % entType):
            print 'redefining', entType
            ndel('/lib/ents/%s' % entType)
        if exists('/lib/vis/%s' % entType):
            ndel('/lib/vis/%s'  % entType)
        if exists('/lib/node/%s' % entType):
            ndel('/lib/node/%s' % entType)

        sn('nent', '/lib/ents/' + entType)
        cs('type', entType)

        cs('type', entType)
        #check attributes on the ent itself
        #is it attached to a side
        side = ed.getAttribute('side')
        if type(side) == str:           call('.sethasside', True)
        elif int(str(side)):            call('.sethasside', True)
        else:                           call('.sethasside', False)
        #is it selectable
        selectable = ed.getAttribute('selectable')
        if type(selectable) == str:     call('.setselectable', True)
        elif int(str(selectable)):      call('.setselectable', True)
        else:                           call('.setselectable', False);          
        #should it be ghosted
        serverEnt = ed.getAttribute('server')
        if serverEnt:
            new('nserverent', 'serverent')

        #foreach variable
        for var in ed.getElementsByTagName('var'):
            name                        = var.getAttribute('name')
            varType                        = var.getAttribute('type')
            value                       = var.getAttribute('value')
            name, varType                  = str(name), str(varType)
            log.write('var', name, varType, value)
            try:
                if   varType == 'int':           ci(  name, int    (value))
                elif varType == 'float':         cf(  name, float  (value))
                elif varType == 'vector3':       cv(  name, vector3(value))
                elif varType == 'string':        cs(  name, str    (value))
                elif varType == 'ref':           cref(name, str    (value))

                #check if its defined server or client
                serverVar = var.getAttribute('server')
                if serverVar:
                    call('vars/%s.setcategory' % name, 'server')

                clientVar = var.getAttribute('client')
                if clientVar:
                    call('vars/%s.setcategory' % name, 'client')

            except:
                printExcept('Exception in parsing vartypes' + name + "," + varType)
        # end -> for each var 

        #foreach aspect 
        for aspect in ed.getElementsByTagName('aspect'):
            aspectType = getText( aspect )
            role = aspect.getAttribute('role')
            if not role: role = aspectType
            log.write('aspect', aspectType)
            sn(aspectType, role)

            offset = aspect.getAttribute('offset')
            if offset:
                v = vector3(offset)
                #print('.setoffset', v[0], v[1], v[2])
                call('.setoffset', v[0], v[1], v[2])
            sel('..')

        #build a standard vis node - just a link
        buildStandardVis(entType)

        #/-----------------------------------------------------------------------------//
        #build a standard vis node with mesh/textures of same name
        shader = None
        standardVisNode = ed.getElementsByTagName('standardVis')
        if standardVisNode:
            shader = str(standardVisNode[0].getAttribute('shader'))
            colored = str(standardVisNode[0].getAttribute('colored'))
            buildStandardNode( entType, shader, colored )
        #/-----------------------------------------------------------------------------//
        customNodes = ed.getElementsByTagName('customNode')
        if customNodes:
            for node in customNodes:
                txt = getText(node)
                buildCustomNode(entType, txt)
        #/-----------------------------------------------------------------------------//
        #attach a particle emitter to this entity
        particles = ed.getElementsByTagName('particles')
        for part in particles:
            attachParticles(part, entType)
        #/-----------------------------------------------------------------------------//
        #attach child vis nodes to this entity
        childViss = ed.getElementsByTagName('childVis')
        for childVis in childViss:
            attachChildVis(childVis, entType)

        #exportVarToChannels
        varsToExport = ed.getElementsByTagName('exportVarToChannel')
        if varsToExport:
            for var in varsToExport:
                varname                 = str(var.getAttribute('var'))
                #print 'exporting', varname
                call('exportchannel', varname)

        #create any fragments
        fragments = ed.getElementsByTagName('fragment')
        for fragment in fragments:
            fragType = str(fragment.getAttribute('type'))
            #print 'fragment', fragType
            if not fragType:
                print 'warning ent: %s has fragment without type' % fragType
                continue
            frag = newUnique('nfragment')
            call('%s.initvars' % frag)
            call('.sss', 'fragment', fragType)

        #create passive sound emitters
        passivesoundslist = ed.getElementsByTagName('passivesounds')
        for passivesounds in passivesoundslist:
            range                       = float(passivesounds.getAttribute('range'))
            gain                        = float(passivesounds.getAttribute('gain'))
            pitch                       = float(passivesounds.getAttribute('pitch'))

            sn('nsoundemitter', 'passivesounds')
            if range:                   call('.setrange', range)
            if gain:                    call('.setgain', gain)
            if pitch:                   call('.setpitch', pitch)
            for passivesound in passivesounds.getElementsByTagName('passivesound'):
                name = str(passivesound.getAttribute('name'))
                call('.addpassivesound', name)
            sel('..')

        #create any hardpoints, and setup any child ents which attach to them
        hardpoints = ed.getElementsByTagName('hardpoint')
        sn('nthing', 'hardpoints')
        for hardpoint in hardpoints:
            snUnique('nhardpoint')                                              

            #set its offset
            pos = hardpoint.getAttribute('pos')                                 
            if pos:
                v = vector3(pos)
                call('setoffset', v[0], v[1], v[2])
            ent = hardpoint.getAttribute('ent')
            sel('..')

            #create a corresponding child ent
            log.write('hardpoint', hardpoint, 'at', pos, 'ent', ent)
            if ent:
                snUnique('nchildent')
                call('.initvars')
                sel('..')
                call('../.sss', 'child', str(ent))
                
        sel('..')
        log.close()
    log.close()
    try:
        call('/lib/ents.sort')
    except:
        pass
    popsel()
#/-----------------------------------------------------------------------------//
def buildStandardVis( type ):
    """build the standard visnode tree for an entity
    use its type names for textures / meshes
    """
    from common.defs import sn
    from common.defs import genLink
    pushsel()
    sn('n3dnode', '/lib/vis/' + type)
    genLink('/lib/node/' + type)
    popsel()
#/-----------------------------------------------------------------------------//
def buildStandardNode( type, shader= 'standard', colored=True):
    from common.defs import sn
    from common.defs import genMesh
    from common.defs import genTex
    from common.defs import genShader
    if not shader: shader = 'standard'

    pushsel()
    log.write('standardvis', 'shader', shader)
    sn('n3dnode', '/lib/node/' + type)
    genMesh(type)
    if not colored:
        genTex(type)
    else:
        if sn('nflipflop', 'colorflipflop'):
            call('.setchannel', 'sideHandle')
            for i, color in enumerate(['neutral', 'blue', 'yellow', 'green', 'red']):
                genTex(color)
                call('tex.setname', color)
                call('.addkey', i, color)
            call('.addkey', i+1, color)
        sel('..')
    genShader(shader)
    popsel()
#/-----------------------------------------------------------------------------//
def buildCustomNode( type, txt ) :
    #print 'buildCustomNode', type, 
    #print txt
    pushsel()

    sn('n3dnode', '/lib/node/' + type)
    #print 'evaling in', psel()
    for line in txt.split('\n'):
        #print 'line', line
        eval(line)
    popsel()
#/-----------------------------------------------------------------------------//
def attachParticles(ed,type):
    from common.defs import pushsel
    from common.defs import popsel
    import FileManager
    import re
    """attach a particle system to my visual node"""
    pushsel('/lib/vis/' + type)
    part                                = ed.getAttribute('type')
    #print 'attaching particle', type, part, psel()
    filepath                            = FileManager.find('particle', part)

    log.write('particles', part)
    if not filepath:
        print 'failed to find particle type', part
        return

    get(filepath)
    pos = ed.getAttribute('pos') 
    if pos:
        v = vector3(pos)
        call('%s.txyz' % ndir()[-1], v[0], v[1], v[2] )
    size = ed.getAttribute('size') 
    if size:
        v = vector3(size)
        call('%s.sxyz' % ndir()[-1], v[0], v[1], v[2] )
    popsel()
    #print 'done attaching particle', type, part, psel()
#/-----------------------------------------------------------------------------//
def attachChildVis(ed, type):
    """attach a child visual tree to the visual tree for this entity
    the child has been created in /lib/vis/childvis
    """
    #print 'EntityDefiner.attachChildVis', ed, type
    node = ed.getAttribute('name')
    node = str(node)
    size = ed.getAttribute('size')
    colored = str(ed.getAttribute('colored'))
    if size:
        size = vector3(size)
    #print 'node', node
    if exists('/lib/node/%s' % node) == False:
        #print 'creating standard vis node for non-found childvis', node
        buildStandardNode(node, colored)
    if node:
        pushsel()
        sn('n3dnode', '/lib/node/%s/%s' %(type, node))
        if size:
            call('.sxyz', size[0], size[1], size[2])
        genLink('/lib/node/%s' % node)
        #from common.defs import genLink
        popsel()
#/-----------------------------------------------------------------------------//
