import os
from npython import *
from defs import *
from common import FileManager
mapInfo = None
#/-----------------------------------------------------------------------------//
class MapInfo:
    def __init__(self,dict):
        new('nent', '/map')
        self.cloudSize                  = 500
        self.name                       = 'unnamed'
        self.description                = 'no description'
        self.terrain                    = 'white.bmp'
        self.heightmap                  = 'flat.bmp'
        self.detail                     = 'detail.bmp'
        self.skybox                     = None
        self.detailsize                 = 64
        self.error                      = 0
        self.minheight                  = 0
        self.maxheight                  = 100
        self.worldsize                  = 5000
        self.lattop                     = 40
        self.latbottom                  = 35
        self.lngleft                    = 115
        self.lngright                   = 120
        self.camerapos                  = '2500 100 2500'
        self.camerarot                  = '0 0 0 1'
        self.blocksize                  = 17
        self.envmap                     = 0
        self.cloudHeight                = 0
        self.cloudTexture               = None

        if dict.has_key('text'):                  self.name                     = dict['text'] 
        if dict.has_key('description'):           self.description              = dict['description'] 
        if dict.has_key('terrain'):               self.terrain                  = dict['terrain']
        if dict.has_key('heightmap'):             self.heightmap                = dict['heightmap']
        if dict.has_key('detail'):                self.detail                   = dict['detail']
        if dict.has_key('skybox'):                self.skybox                   = dict['skybox']
        if dict.has_key('detailsize'):            self.detailsize               = int(dict['detailsize'])
        if dict.has_key('error'):                 self.error                    = int(dict['error'])
        if dict.has_key('minheight'):             self.minheight                = float(dict['minheight'])
        if dict.has_key('maxheight'):             self.maxheight                = float(dict['maxheight'])
        if dict.has_key('worldsize'):             self.worldsize                = float(dict['worldsize'])
        if dict.has_key('lattop'):                self.lattop                   = float(dict['lattop'])
        if dict.has_key('latbottom'):             self.latbottom                = float(dict['latbottom'])
        if dict.has_key('lngleft'):               self.lngleft                  = float(dict['lngleft'])
        if dict.has_key('lngright'):              self.lngright                 = float(dict['lngright'])
        if dict.has_key('camerapos'):             self.camerapos                = dict['camerapos']
        if dict.has_key('camerarot'):             self.camerarot                = dict['camerarot']
        if dict.has_key('blocksize'):             self.blocksize                = int(dict['blocksize'])
        if dict.has_key('envmap'):                self.envmap                   = 1
        if dict.has_key('cloudtexture'):          self.cloudTexture             = dict['cloudtexture']
        if dict.has_key('cloudheight'):           
            self.cloudHeights = dict['cloudheight'].split(' ')
            self.cloudHeights = map(float, self.cloudHeights)                   #convert all of them to floats
        if dict.has_key('cloudsize'):             
            self.cloudSizes = dict['cloudsize'].split(' ')
            self.cloudSizes = map(float, self.cloudHeights)                    
        self.gridinterval = self.worldsize / 369.0

    def __str__(self):
        return "name:%s desc:%s terrain:%s heightmap:%s detail:%s skybox:%s detailsize:%i error:%i maxheight:%f worldsize:%f lattop:%f latbottom:%f lngleft:%f lngright:%f camerapos:%s gridinterval:%f" % (self.name, self.description, self.terrain, self.heightmap, self.detail, self.skybox, self.detailsize, self.error, self.maxheight, self.worldsize, self.lattop, self.latbottom, self.lngleft, self.lngright, self.camerapos, self.gridinterval)
#/-----------------------------------------------------------------------------//
def load(mapname):
    """load the map from an xml file"""
    global mapInfo
    filepath = FileManager.find('xml', mapname)
    if not filepath:
        print 'could not find xml file to open -- defaulting to openwater'
        from common import OptionManager
        OptionManager.setOption('map', 'openwater')
        load(OptionManager.getOption('map'))
        return

    document = parseXML( filepath )

    print 'MapManager.load(%s,%s)' % (mapname, filepath)
    counter = 0
    #foreach map node 
    for ed in document.getElementsByTagName('MAP'): 
        mapInfo = MapInfo( xmlToDict(ed) )
        assert counter == 0 #make sure only one map node in the xml file
        counter += 1
        for cloudplane in ed.getElementsByTagName('CLOUDPLANE'):
            createCloudPlane(str(cloudplane.getAttribute('texture')), float(cloudplane.getAttribute('height')), float(cloudplane.getAttribute('size')), cloudplane.getAttribute('speed'))
        for waterPlane in ed.getElementsByTagName('WATERPLANE'):
            createWaterPlane()

    for ent in document.getElementsByTagName('ENT'):
        import MissionManager
        MissionManager.loadEntity(ent)

    createMapNode()
    createTerrainNode()
    createSkyBox()
    setEVAVars()
    adjustCamera()
    from minimap import minimap 
    minimap.setBackgroundImage(mapname + 'minimap')

    for wave in document.getElementsByTagName('WAVE'):
        createWave( wave )
#/-----------------------------------------------------------------------------//
def setHeightRange( minheight, maxheight ):
    print 'MapManager.setHeightRange', minheight, maxheight
    global mapInfo
    mapInfo.minheight = minheight
    mapInfo.maxheight = maxheight
#/-----------------------------------------------------------------------------//
def createMapNode():
    """create the map node - the storage class containing heightmap information"""
    ndel('/map/map')
    sn('nmap', '/map/map')
    heightmap                           = FileManager.find( 'tex', mapInfo.heightmap )
    print 'MapManager.createMapNode(%s)' % heightmap
    call('setheightmap',                heightmap )
    call('setgridinterval',             mapInfo.gridinterval )
    if( mapInfo.maxheight < mapInfo.minheight):
        set('setheightrange', mapInfo.maxheight, mapInfo.minheight )
    else:
        set('setheightrange', mapInfo.minheight, mapInfo.maxheight )
#/-----------------------------------------------------------------------------//
def createTerrainNode():
    """create the visual part of the map - what is drawn to the screen    """
    ndel('/game/vis/terrain')
    sn('n3dnode', '/game/vis/terrain')
    #sn('nipol', 'ty')
    #call('.connect', 'ty')
    #call('.setchannel', 'camera_alt')
    #call('.addkey1f', 0, -1)
    #call('.addkey1f', 1000, -1)
    #call('.addkey1f', 100000, -10)
    #sel('..')

    sn('nmapnode', '/game/vis/terrain/map')
    set('setmap', '/map/map')
    set('setblocksize',     mapInfo.blocksize )
    set('setdetailsize',     mapInfo.detailsize )
    set('seterror',         mapInfo.error )
    sel('..')

    if not mapInfo.envmap:
        genShader( 'terrain' )
        genTex( mapInfo.terrain, mapInfo.detail )
    else:
        genShader( 'terrain' )
        #genShader( 'envmappedterrain' )
        #genShader( 'translucentterrain' )
        period = 2
        #genTex( mapInfo.terrain, mapInfo.detail, 'envmap' )
        sn('nflipflop','tex')
        for i in range(1,64):
            genTex( mapInfo.terrain, 'waves_%02i'%i );   
            call('tex.setname', 'frame%i'%i)
            call('.addkey', period * (i-1) / 32.0, 'frame%i'%i)
        call('.setchannel', 'time')

#/-----------------------------------------------------------------------------//
def changeHeightmap(heightmap):
    """replace the existing heightmap with the specified one"""
    print 'MapManager.changeHeightmap', heightmap
    global mapInfo
    mapInfo.heightmap = heightmap
    createMapNode()
    createTerrainNode()
#/-----------------------------------------------------------------------------//
def createSkyBox():
    """Create the skybox"""
    from common import OptionManager
    omskybox = OptionManager.getOption('skybox')
    if omskybox:
        mapInfo.skybox = omskybox
    if not mapInfo.skybox:
        return None

    def genSide(name, tx, ty, tz, rx, ry, rz):
        sn('n3dnode', name)
        set('txyz', tx, ty, tz)
        set('rxyz', rx, ry, rz)
        genMesh('plane')
        genTex('%s_%s' % (mapInfo.skybox, name) )
        genShader('skybox')
        sel('..')

    sn('n3dnode', '/game/vis/sky')
    set('sxyz', 5, 5, 5)
    set('setlockviewer', 1)
    genSide('xp',    0,    0,    1,    90,    0,    0)
    genSide('zp',    1,    0,    0,    90,    90,    0)
    genSide('xn',    0,    0,    -1,    90,    180,0)
    genSide('zn',  -1,    0,    0,    90,    270,0)
    genShader('cloud')
    genSide('yp',    0,    1,    0,    0,    0,    0)
    genSide('yn',    0,    -1,    0,    180,0,    0)

cloudCounter = 0
#/-----------------------------------------------------------------------------//
def createCloudPlane(texture, height, size, speed):
    """create a plane (future:number of planes) with clouds"""
    speed = map(float, speed.split(','))
    new('n3dnode', '/game/vis/clouds')
    global cloudCounter
    new('n3dnode', '/game/vis/clouds/clouds%i' % cloudCounter)
#/-----------------------------------------------------------------------------//
    pushsel('/game/vis/clouds/clouds%i' % cloudCounter)
    cloudCounter += 1

    call('ty', height )
    call('sxyz', mapInfo.worldsize, 1, mapInfo.worldsize )
    genMesh('plane')
    genTex(texture)
    genShader('cloud')

    sel('shader')
    call('setrenderpri', -10 - cloudCounter)
    call('.begintunit', 0)
    call('.sxyz', mapInfo.worldsize/size, mapInfo.worldsize/size, mapInfo.worldsize/size)
    sn('nipol', 'txyz')
    call('.connect', 'txyz0')
    call('addkey3f', 0, 0, 0, 0)
    call('addkey3f', 60, speed[0], speed[1], 0 )
    sel('..')
    call('.endtunit')
    popsel()
#/-----------------------------------------------------------------------------//
def createWaterPlane():
    """create the visual part of the map - what is drawn to the screen    """
    #print 'createWaterPlane()'
    sn('n3dnode', '/game/vis/water')
    #call('.sxyz', mapInfo.worldsize, mapInfo.worldsize, mapInfo.worldsize)
    call('.sxyz', 1, 1, 1)
    if sn('nwaver', 'waver'):
        genMesh('waveplane')
        call('mesh.setreadonly', True)
        call('.beginsources', 1)
        call('.setsource', 0, 'mesh', 'time')
        call('.endsources')
        call('.setnormalize', True)
        call('.setupdatecoord', True)
        call('.setupdatenorm', False)
        call('.setupdatecolor', False)
        call('.setupdateuv0', True)
        call('.setupdateuv1', True)
        call('.setupdateuv2', True)
        call('.setupdateuv3', True)
    sel('..')
    
    genShader( 'water' )
    #call('.rx', 90)
    call('shader.setcullmode', 'none')
    period = 2
    #genTex( mapInfo.terrain, mapInfo.detail, 'envmap' )
    sn('nflipflop','tex')
    for i in range(1,64):
        #genTex( mapInfo.terrain, 'waves_%02i'%i );   
        genTex( 'waves_%02i'%i, 'waves_%02i'%i );   
        call('tex.setname', 'frame%i'%i)
        call('.addkey', period * (i-1) / 32.0, 'frame%i'%i)
    call('.setchannel', 'time')
    sel('..')
    sn('nipol', 'ty')
    call('.connect', 'ty')
    call('.setchannel', 'camera_alt')
    call('.addkey1f', 0,  0)
    call('.addkey1f', 20,  0)
    call('.addkey1f', 50,  -5)
    call('.addkey1f', 100000, -1000)
    sel('..')
#/-----------------------------------------------------------------------------//
wavenum = 0
def createWave(wave):
    """every xml wave setting should correspond to an entity in /map/waves which is used by the waves to draw themselves"""
    #print 'createWave', wave
    global wavenum
    pushsel()
    type                                = wave.getAttribute('type')
    period                              = wave.getAttribute('period')
    direction                           = wave.getAttribute('direction')
    amplitude                           = wave.getAttribute('amplitude')
    wavelength                          = wave.getAttribute('wavelength')
    type                                = str(type)
    period                              = float(period)
    direction                           = float(direction)
    amplitude                           = float(amplitude)
    wavelength                          = float(wavelength)

    #print 'creating wave type:%s period:%f direction:%f amplitude:%f' % (type, period, direction, amplitude)
    if not exists('/map/waves'):
        new('nent', '/map/waves')
    if sn('nent', '/map/waves/wave%i' % wavenum):
        wavenum += 1
        cs('type', type)
        cf('period', period)
        cf('direction', direction)
        cf('amplitude', amplitude)
        cf('wavelength', wavelength)
    popsel()
#/-----------------------------------------------------------------------------//
def adjustCamera():
    global mapInfo
    pieces = mapInfo.camerapos.split()
    set('/game/camera.sv', 'pos', float(pieces[0]), float(pieces[1]), float(pieces[2]))

    pieces = mapInfo.camerarot.split()
    print 'cameraRot', pieces
    set('/game/camera.sq', 'rot', float(pieces[0]), float(pieces[1]), float(pieces[2]), float(pieces[3]))
#/-----------------------------------------------------------------------------//
def setEVAVars():
    pushsel('/map')
    cs('name',                          mapInfo.name)
    cs('description',                   mapInfo.description)
    cs('terrain',                       mapInfo.terrain)
    cs('heightmap',                     mapInfo.heightmap)
    cs('detail',                        mapInfo.detail)
    cs('skybox',                        mapInfo.skybox)
    ci('detailsize',                    mapInfo.detailsize)
    ci('error',                         mapInfo.error)
    cf('maxheight',                     mapInfo.maxheight)
    cf('worldsize',                     mapInfo.worldsize)
    cf('lattop',                        mapInfo.lattop)
    cf('latbottom',                     mapInfo.latbottom)
    cf('lngleft',                       mapInfo.lngleft)
    cf('lngright',                      mapInfo.lngright)
    cs('camerapos',                     mapInfo.camerapos)
    ci('blocksize',                     mapInfo.blocksize)
    ci('envmap',                        mapInfo.envmap)
    popsel()
#/-----------------------------------------------------------------------------//
