from npython import *
from defs import *
import re
import xml.dom.minidom
filepath = 'common/xml/runoptions.xml'
"""
current command line options:
sp                                      enable/disable sp
cat                                     enable/disable cat
executive                               enable/disable the executive
"""
#/-----------------------------------------------------------------------------//
class Option:
    """
    an option represents one of the things that can be set through the command line, or through the upcoming optionGui
    each option can have a limited set of values, a default, and an actual value to which it is set
    """
    def __init__(self, name, possibilities, defaultValue, description=''):
        self.name                       = name
        self.possibilities              = possibilities
        self.defaultValue               = defaultValue
        self.value                      = defaultValue
        self.description                = str(description)
        self.importance                 = 0
        global category
        self.category                   = category
    #/-----------------------------------------------------------------------------//
    def default(self, importanceThreshold = 1):
        if self.importance <= importanceThreshold:
            self.value = self.defaultValue
        else:
            print 'Option', self.name, 'not default im too important'
    #/-----------------------------------------------------------------------------//
    def __repr__(self):
        return '%s:%s' % (self.name, self.value)
    #/-----------------------------------------------------------------------------//
    def getValue(self):
        if not self.value:
            return 0
        else:
            try:
                return int(self.value)
            except:
                return str(self.value)
    #/-----------------------------------------------------------------------------//
    def setValue(self, value):
        if not value:
            self.value = 0
        else:
            try:
                self.value = int(value)
            except:
                self.value = str(value)
#/-----------------------------------------------------------------------------//
category = ''
def createCategory(newCategory):
    global category
    category = newCategory
#/-----------------------------------------------------------------------------//
def createOption(name, possibilities, default, descr=None):
    """
    creates a new option which can be set / modified
    """
    global options, autosave
    #print 'OptionManager.createOption(%s,%s,%s,%s)' % (name, possibilities, default, descr)
    assert name not in options
    options[name] = Option(name=name, possibilities=possibilities, defaultValue=default, description=descr)
    orderedOptions.append( name )
#/-----------------------------------------------------------------------------//
def setOption(optionname, value, importance=1):
    """set a particular options value"""
    #print 'OptionManager.setOption(%s,%s)' % (optionname, value)
    global options, autosave
    if optionname not in options:
        print 'warning setting an option which was not explicited created before hand'
        createOption( name=optionname, possibilities=[], default=None )
    options[optionname].setValue(value)
    options[optionname].importance = importance
    if autosave:
        saveOptions()
#/-----------------------------------------------------------------------------//
def processArgs():
    """process args passed to go, see when we can do
    it accepts 3 kinids of args +ex which means set flag ex to True
    -ex which means set flag ex to false
    ex=hello which is a variable set
    """
    #print 'OptionManager.processArgs'
    global options, autosave
    autosave = False

    args = call('/sys/share/args.gets')
    #print 'args', args, 

    #check for +ex options
    reg = re.compile(r'\+\S+')
    plusoptions = reg.findall(args)
    #print 'plusoptions', plusoptions
    for optionname in plusoptions:
        optionname = optionname[1:]
        setOption( optionname, 1, 2 )

    #check for -ex options
    reg = re.compile(r'\-\S+')
    args = call('/sys/share/args.gets')
    minusoptions = reg.findall(args)
    #print 'minusoptions', minusoptions
    for optionname in minusoptions:
        optionname = optionname[1:]
        setOption( optionname, 0, 2 )

    #check for ex=hello options
    reg = re.compile(r'\S+=\S+')
    args = call('/sys/share/args.gets')
    equaloptions = reg.findall(args)
    #print 'equaloptions', equaloptions
    reg = re.compile(r'(\S+)=(\S+)')
    for arg in equaloptions:
        optionname, val = reg.search(arg).groups()
        try:
            val = int(val)
        except:
            val = str(val)
        setOption(optionname, val, 2)
        #options[optionname] = val

    autosave = True
    saveOptions()
    print 'options=', options
#/-----------------------------------------------------------------------------//
def loadOptions():
    """load previous options from the xml file"""
    print 'OptionManager.loadOptions'
    global options, autosave
    try:
        file                                = open(filepath)
        doc                                 = minidom.parse(file)
        file.close()
    except IOError:
        print 'could not read run options file', filepath
    else:
        autosave = False
        for option in doc.getElementsByTagName('option'):
            val = option.getAttribute('value')
            name = str(option.getAttribute('name'))
            assert name
            setOption(name, val, 1)
            #options[name] = val
        autosave = True
        saveOptions()
#/-----------------------------------------------------------------------------//
def useDefaultOptions( importanceThreshold = 1 ):
    print 'OptionManager.useDefaultOptions'
    for option in options.values():
        option.default( importanceThreshold )
    saveOptions()
#/-----------------------------------------------------------------------------//
def saveOptions():
    """save options back into the eva tree and the xml file, including any changes"""
    #print 'OptionManager.saveOptions'
    exportToEVA                         ()
    doc                                 = xml.dom.minidom.Document()
    root                                = doc.createElement('runoptions')
    doc.appendChild                     (root)
    for option in options.values():
        node                            = doc.createElement('option')
        root.appendChild                (node)
        node.setAttribute               ('name', option.name)
        node.setAttribute               ('value', str(option.getValue()))
    text = doc.toprettyxml('    ')
    try:
        file = open(filepath, 'w')
        file.write(text)
        file.close()
    except IOError:
        print 'could not save run options file', filepath
        printExcept()
#/-----------------------------------------------------------------------------//
def getOption(optionname):
    """return the value for an option"""
    #print 'OptionManager.getOption(%s)' % optionname
    try:
        return options[optionname].getValue()
    except:
        print 'option %s not found' % optionname
        return None
#/-----------------------------------------------------------------------------//
#def defaultOption(optionname, value):
    #"""set a default option value (what will be set if its not already in the file)"""
    #global options
    #if not optionname in options:
        #try:                            value = int(value)
        #except:                         pass
        #options[optionname] = value
        #saveOptions()
#/-----------------------------------------------------------------------------//
new('nent', '/options')
sel('/options')
def exportToEVA():
    #pushsel('/options')
    for option in options.values():
        val = option.getValue()
        #if type(option.value) == None:
            #call('/options.ci', option.name, 0)
        if type(val) == int:
            call('/options.ci', option.name, val)
        else:
            call('/options.cs', option.name, val)
            
        #elif type(option.value) == int:
            #call('/options.ci', option.name, option.value)
        #else:
            #call('/options.cs', option.name, str(option.value))
    try:                       
        popsel()
    except:
        pass
#/-----------------------------------------------------------------------------//
options                                 = {}
orderedOptions                          = []
autosave                                = True
def load():
    loadOptions()
    #print 'finished loading options', options
    processArgs()
    #print 'finished processingArgs options', options
    saveOptions()
    if getOption('nomem'):
        useDefaultOptions()
#/-----------------------------------------------------------------------------//
#def openOptionExplorer(event):
    #print 'openOptionExplorer'
    #from common import optionexplorer
    #optionexplorer.open()
#/-----------------------------------------------------------------------------//
#def createMenu(panel):
    #import wx
    #menu = wx.Menu()
    #openOptionExplorerItem = menu.Append( 1534, 'Edit', 'Display / Change Options')
    #wx.EVT_MENU(panel, 1534, openOptionExplorer)
    #return menu
#/-----------------------------------------------------------------------------//
