from common.defs import *
import wx
class Log:
    def __init__(self, tags):
        self.tags = tags
    def write(self, *args):
        logManager.write(self, args)
    def open(self, *args):
        logManager.open(self, args)
    def close(self):
        logManager.close(self)
#else:
#    print 'disabling logging'
#    class Log:
#        def __init__(self, tags):
#            pass
#        def write(self, *args):
#            pass
#        def open(self, *args):
#            pass
#        def close(self):
#            pass
    

class LogManager(wx.Frame):
    """handles basic logging communications,
        different modules can write to different windows, and they can all be nicely viewed in a treeview
        for big messages they can be contained in a block, which can then be collapsed in the tree
        should also dump of these information to a file
        as well as provide options to send it to stdout
    """
    #/-----------------------------------------------------------------------------//
    tagsToPrint = []
    #/-----------------------------------------------------------------------------//
    def __init__(self,parent,id):
        import WindowManager
        windowName                      = 'Log'
        WindowManager.register          (windowName, self)
        position,size                   = WindowManager.get(windowName)
        wx.Frame.__init__(self,parent,id,windowName, pos=position, size=size)
        self.notebook = wx.Notebook(self,-1)
        self.trees = {}
        self.files = {}
        global logManager
        logManager = self
    #/-----------------------------------------------------------------------------//
    def write(self, writer, *args):
        """Write a message into the tree"""
        tags, txt = self._process(writer, args)
        for tag in tags:
            self._checkTag(tag)
            self._printToFile(tag, txt)
            self._printToStdOut(tag, txt)
            self.trees[tag].AppendItem(self.trees[tag].nodes[-1], txt)
            for node in self.trees[tag].nodes:
                self.trees[tag].Expand(node)
    #/-----------------------------------------------------------------------------//
    def open(self,writer, *args):
        """open a subtree"""
        tags, txt = self._process(writer, args)
        for tag in tags:
            self._checkTag(tag)
            self._printToFile(tag, txt)
            self._printToStdOut(tag, txt)
            node = self.trees[tag].AppendItem(self.trees[tag].nodes[-1], txt)
            self.trees[tag].nodes.append(node)
    #/-----------------------------------------------------------------------------//
    def close(self,writer):
        """close a subtree"""
        tags, txt = self._process(writer)
        for tag in tags:
            self._checkTag(tag)
            self.trees[tag].nodes = self.trees[tag].nodes[:-1]
    #/-----------------------------------------------------------------------------//
    def _checkTag(self,tag):
        if not self.trees.has_key(tag):
            self.trees[tag] = wx.TreeCtrl(self.notebook,-1)
            self.notebook.AddPage(self.trees[tag], tag)
            self.trees[tag].nodes = [self.trees[tag].AddRoot(tag)]
            self.files[tag] = open('logs/%s'%tag, 'w')
    #/-----------------------------------------------------------------------------//
    def _printToFile(self, tag, txt):
        if txt:
            for i in range(1, len(self.trees[tag].nodes)):
                self.files[tag].write('  ')
            self.files[tag].write(txt)
            self.files[tag].write('\n')
    #/-----------------------------------------------------------------------------//
    def _printToStdOut(self, tag, txt):
        if txt:
            if tag in self.tagsToPrint:
                for i in range(1, len(self.trees[tag].nodes)):
                    print '  ',
                print txt
    #/-----------------------------------------------------------------------------//
    def _process(self,writer,args=None):
        try:
            tags = writer.tags
        except:
            printExcept()
            tags = ['all']
        if args:
            txt = str(args[0])
            for arg in args[1:]:
                txt += ' ' + str(arg)
            if txt[-1] == '\n':
                txt = txt[:-1]
        else:
            txt = None
        #print 'tags', tags, 'txt', txt
        return tags, txt
