#
# @(#) commandclient.py 2005/07/19
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

import threading
import socket
from common.log import Log
log = Log(tags=['all', 'comms'])
import nEventServer
from common import EventNames
import sys
import time
import nEventServer
from common.defs import *
import math


csleepTime = 0.05

class CommandThread(threading.Thread):
    """Obtains new speed, heading and sends it to COVE"""
    def __init__(self, host, port, comm):
        threading.Thread.__init__(self)        
        log.write("Command Thread being initialized")
        nEventServer.register(EventNames.COVE_COMMAND_DISCONNECT, self.handleCommandDisconnect)
        self.host = host
        self.port = port
        self.comm = comm


    def run(self):
        log.write("Command Thread starting running")
        self.commandSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.commandSocket.connect((self.host, self.port))
        log.write("Opened command socket")        
        log.write("Command Thread connected to cove at:"
                  + `self.host` + ":" + `self.port`)
        self.cdie = 0
        while self.cdie == 0:
            self.sendLagoonCommands(self.commandSocket)            
            time.sleep(csleepTime)
        self.sendQuitCommand(self.commandSocket)
        self.commandSocket.close()
        log.write("Command Thread received disconnect signal")

    def handleCommandDisconnect(self, msg):
        #print 'Handling command disconnect'
        self.cdie = 1

####################################################################

    def sendLagoonCommands(self, con):
#        log.write("Getting lagoon state")
        d = self.comm.getCommandState()
        #print "Command state: ", str(d)
        for k, info in d.iteritems():
            ds, dh = info
            #dsInKnots = 0.5626 * ds
            try:
                #print "LagoonToCoveMap: ", str(self.comm.lagoonCache)
                tmp = k.strip()
                if(self.comm.hasCachedLagoonEntity(tmp)):
                    ent = self.comm.getCachedLagoonEntity(tmp)
                    #spdata = ' speed_' + ent + '_' + str(dsInKnots) + ' | '
                    spdata = ' speed_' + ent + '_' + str(ds) + ' | '
                    dh = self.comm.unrotate(dh)
                    hddata = ' heading_' + ent + '_' + str(dh) + ' | '
                    data = spdata + hddata
                    #print "Sending command: " +  data
                    con.send(data)
            except:
                printExcept("sendLagoonCommands: Error")
        
#        log.write("")


####################################################################

####################################################################

    def sendQuitCommand(self, con):
        quitString = ' quit |'
        con.send(quitString)
        
#        log.write("")


####################################################################

