#
# @(#) covecomms.py 2005/07/16
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
import threading
from npython import *
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'comms'])
import wx
import sys
import nEventServer
from common import EventNames
from common import OptionManager
import cat
import nEventServer
from wxPython.wx import *
import socket
import string
from const import *
import sp
from stringnames import *
import math

ID_HARBOR         = 2323000
ID_CONNECT        = 2323001
ID_DISCONNECT     = 2323002
COVE_HOST         = '192.168.1.4'
COVE_COMMAND_PORT = 4001
COVE_STATE_PORT   = 4000

twoPi = 2 * math.pi

#def getGUI(parent, size, log):
#    return CovePanel(parent, size, log)

class CovePanel:
    """The cove comm puts items on the menu bar"""

    def __init__(self, parent, s):
        """constuctor - initialize things, add connect/disconnet to menu-bar
        and set up a tick method for two way communications"""
        self.entDict                    = {}
        from common import tickmanager
        tickmanager.register            ( 'CovePanel', self.tick ) # should receive and send info every second

        menubar                         = parent.GetMenuBar()
        menu                            = wx.Menu()
        #menu.Append(ID_HARBOR, '&GetHarborInfo', 'Get Depth Information on a harbor from cove');

        if OptionManager.getOption('harborclient'):
            from harborclient import harborClient
            harborMenu                  = harborClient.createMenu(self, parent)
            menu.AppendMenu             (ID_HARBOR, "D&ownload Harbor From Cove", harborMenu) 
        menu.Append                     (ID_CONNECT, '&Connect', 'Connect to Cove');
        menubar.Append                  (menu, 'VShipComm')

        EVT_MENU                        (parent, ID_HARBOR, self.onHarbor)
        EVT_MENU                        (parent, ID_CONNECT, self.onConnect)
        EVT_MENU                        (parent, ID_DISCONNECT, self.onDisconnect)

        import translate
        self.translator                 = translate.Translator(getCameraPos())
        nEventServer.register           (EventNames.COVE_CREATE_ENTITY, self.handleCreateEntity)
        nEventServer.register           (EventNames.COVE_POSITION_ENTITY, self.handlePositionEntity)        
        self.createEntityFlag           = 0
        self.posEntityFlag              = 0
        self.coveCache                  = {}
        self.lagoonCache                = {}
        self.posUpdateQ                 = []
        self.caughtUp                   = True

    def handleCreateEntity(self, msg):
#        print "Creating Entity with message:" +  str(msg)
        self.createEntityFlag = 1

    def handlePositionEntity(self, msg):
#        print "Positioning Entity with message:" +  str(msg)
        self.posEntityFlag    = 1


    def translate(self, pos):
        import translate
        tp = self.translator.CoveToLagoon((pos[0], pos[1], pos[2]))
#        tp = self.translator.CoveToLagoon((-pos[1], pos[2], -pos[0]))        
        return (tp)

    def onHarbor(self, event):
        print "Getting harbor information"

    def onConnect(self, event):
        """Set up two threads for sending and receiving from cove"""
        log.write('Connecting to Cove')
        self.coveHost = OptionManager.getOption('covehost')
        if self.coveHost == None:
            self.coveHost = COVE_HOST
        self.coveStatePort = OptionManager.getOption('coveport')
        if self.coveStatePort == None:
            self.coveStatePort = COVE_STATE_PORT
        else:
            self.coveStatePort = string.atoi(self.coveStatePort)
        self.coveCommandPort = self.coveStatePort + 1

        log.write("Opening Sockets to Cove Host: ", self.coveHost)
        log.write("Creating Threads...")
        import stateclient
        self.stateThreadHandle   = stateclient.StateThread(self.coveHost, self.coveStatePort, self)        
        import commandclient
        self.commandThreadHandle = commandclient.CommandThread(self.coveHost, self.coveCommandPort, self)
        
        log.write("Starting Threads...")
        self.stateThreadHandle.start()
        self.commandThreadHandle.start()
        log.write("Threads Started")

        pushsel('/lib/ents')
        entlist = ndir()
        global RELEVANT_LIST
        RELEVANT_LIST = []
        for ent in entlist:
            if getselectable(ent):
                RELEVANT_LIST.append(ent)
        popsel()
        print "RELEVANT LIST", str(RELEVANT_LIST)

    def onDisconnect(self, event):
        #log.write('Disconnecting from Cove')
        nEventServer.pull(EventNames.COVE_STATE_DISCONNECT)
        nEventServer.pull(EventNames.COVE_COMMAND_DISCONNECT)
        #log.write('Sockets closed, bye')        
        tmp = waitThread(self.stateThreadHandle, self.commandThreadHandle)
        tmp.start() # Threading is better than the whole app freezing
        log.write("Threads dead")

    oldtime = 0
    def tick(self, time, dtime):
#        if time - self.oldtime < 2:
#            return
        self.oldtime = time
        pushsel()

        sel('/game/ents/')
        entTypes = ndir()
#        log.write(entTypes)
        entInfoTuple = []
        global RELEVANT_LIST
        for entType in entTypes:
            if (entType.strip() not in RELEVANT_LIST):
                #print "EntType: ", entType, " not in RL"
                continue
            #print "EntType: ", entType, " in RL:", str(RELEVANT_LIST)
            sel(entType)
            ents = ndir()
            #print 'Ent Info: ', ents
            self.getEntityStates(ents, entInfoTuple)
            sel('..')

        popsel()
        self.entDict = dict(entInfoTuple)
#        log.write("Entity Information: " + `self.entDict`)

        if (self.createEntityFlag == 1):
#            log.write ("I am creating an ent")
            self.createEntityFlag = 0
            self.createEntity()

        if (self.posEntityFlag == 1):
#            log.write ("Repositioning Entity")
            self.posEntityFlag = 0
            self.reposition()

    def createEntity(self):
        label = self.newEntityData[COVELABEL]
        if (self.coveCache.has_key(label)):
            log.write(str(label) + " already created")
            return

#        from sp.groupmanager import groupManager
#        groupManager.createGroup(self.newEntityData[LAGOONGROUP], self.newEntityData[LAGOONSIDE])
        log.write("creatingEnt:" + self.newEntityData[LAGOONTYPE] + "," + self.newEntityData[LAGOONGROUP] + ", at:  " + str(self.newEntityData[LAGOONPOS]) + " with heading: " + str(self.newEntityData[LAGOONHEADING]))
        from sp.entmanager import entManager
#        entPath = entManager.createEnt(self.newEntityData[LAGOONTYPE], self.newEntityData[LAGOONGROUP], self.newEntityData[LAGOONPOS])
        entPath = entManager.createEnt(self.newEntityData[LAGOONTYPE], position=self.newEntityData[LAGOONPOS])        

        if (exists(entPath)):
            pushsel()
            sel(entPath)
            heading = self.newEntityData[LAGOONHEADING]
            deg = self.radiansToDegrees(heading)+ 180
            srot('rot', 0, deg, 0)
            sf('desiredSpeed', 0.0)
            sf('desiredHeading', heading)
            vel = self.newEntityData[LAGOONVELOCITY]
            sv('vel', vel[0], vel[1], vel[2])

            popsel()
        
        self.coveCache[label] =  entPath
        entId = self.getEntPart(entPath)
        if(entId != ''):
            print "EntId: |" + entId + "| being cached with: |" + label + "| "
            self.lagoonCache[entId] = label
        print "Check: EntId: *" + entId + "* should be being cached with: *" + label + "* "
        return
            
    def setNewEntityData(self, data):
        self.newEntityData = {}
        self.newEntityData[COVELABEL]       = data[COVELABEL]
        self.newEntityData[LAGOONGROUP]     = data[LAGOONGROUP]
        self.newEntityData[LAGOONSIDE]      = data[LAGOONSIDE]
        self.newEntityData[LAGOONTYPE]      = data[LAGOONTYPE]        
        self.newEntityData[LAGOONPOS]       = data[LAGOONPOS]
        self.newEntityData[LAGOONHEADING]   = data[LAGOONHEADING]
        self.newEntityData[LAGOONSPEED]     = data[LAGOONSPEED]
        self.newEntityData[LAGOONVELOCITY]  = data[LAGOONVELOCITY]                

    def reposition(self):
        for newPosData in self.posUpdateQ:
            epath    = newPosData[ENTPATH]
            epos     = newPosData[LAGOONPOS]
            heading  = newPosData[LAGOONHEADING]
            vel      = newPosData[LAGOONVELOCITY]
 #           log.write("Repositioning entity" + str(epath) + " at " + str(epos) + " heading: " + str(heading))
   
            if(exists(epath)):
                pushsel()

                sel(epath)
                cpos = gv('pos')
                diff = vecDiff(epos, cpos)
#                cv('posError', diff[0], diff[1], diff[2])
                call('cv', 'posError', diff[0], diff[1], diff[2])
                npos = interp(cpos, epos, 2,3)
#                sv('pos', epos[0], epos[1], epos[2])
#                sv('pos', npos[0], npos[1], npos[2])
                chead = gf('heading')
                coveHeading = math.atan2(vel[0], vel[2])
                ihead = coveHeading #interpHeading(chead, heading, 1.0)
                ihead = heading - math.pi
                if(ihead < 0):
                    ihead = twoPi + ihead
                
                deg = math.degrees(ihead) #+ 180
#                if(not zeroVec(vel)):
                srot('rot', 0, deg, 0)
#                sf('desiredHeading', heading)
                cv = gv('vel')
                nvel = interp(cv, vel, 2, 3)
                #sv('vel', nvel[0], nvel[1], nvel[2])
                #print "Vel: ", str(vel)
                #sv('vel', vel[0], vel[1], vel[2])                

                popsel()
#                log.write("Finished repositioning entity" + str(epath) + " at " + str(epos) + " with heading " + str(heading) + " or degrees:" + str(deg))
        self.posUpdateQ = []

    def setNewPosData(self, posData):
        newPosData = dict([])
        newPosData[ENTPATH]   = posData[ENTPATH]
        newPosData[LAGOONPOS] = posData[LAGOONPOS]
        newPosData[LAGOONHEADING] = posData[LAGOONHEADING]
        newPosData[LAGOONVELOCITY] = posData[LAGOONVELOCITY]
        newPosData[LAGOONSPEED] = posData[LAGOONSPEED]                
        self.posUpdateQ.append(newPosData)
#        log.write("PosUpdateQ: " + str(self.posUpdateQ))

    def hasCachedCoveEntity(self, label):
        return self.coveCache.has_key(label)

    def getCachedCoveEntityPath(self, label):
        return self.coveCache[label]

    def rotate(self, heading):
        global twoPi        
        nh = twoPi - heading 
        if(nh < 0):
            return twoPi - nh
        else:
            return  nh

    def unrotate(self, heading):
        global twoPi
        nh = twoPi - heading #- math.pi
        if (nh < 0 ):
            return  twoPi - nh
        elif(nh > twoPi):
            return nh%twoPi
        else:
            return nh

    def hasCachedLagoonEntity(self, entId):
        return self.lagoonCache.has_key(entId)

    def getCachedLagoonEntity(self, entId):
        return self.lagoonCache[entId]

    def getEntPart(self, path):
        index = path.rfind('/')
        if(index != -1):
            return path[index+1:].strip()
        else:
            return ''

    def radiansToDegrees(self, radians):
        return (180.0/math.pi) * radians

    def getCommandState(self):
        return self.entDict
        
    def getEntityStates(self, ents, entInfoTuple):
        for ent in ents:
            if(not exists(ent)):
                continue
            sel(ent)
#            entInfo = ndir()
#            log.write("EntInfo: ", entInfo)
            try:
                ds = gf('desiredSpeed')
                dh = gf('desiredHeading')
#               log.write('Desired Speed, Heading of ' + `ent`
#                          + `ds` + ', ' + `dh`)
                info = ds, dh
                nt = ent, info
                entInfoTuple.append(nt)
            except:
                log.write(`ent` + ' does not have speed/heading')

#             sel('vars')

#             evars = ndir()
#             log.write("Ent variables: ", evars)

#            sel ('..')

            sel('..')

        
#     def findEnt(self, name):

#         pushsel()

#         sel('/game/ents/')
#         entTypes = ndir()
# #        log.write(entTypes)
#         entInfoTuple = []
#         for entType in entTypes:
#             if (entType not in RELEVANT_LIST):
#                 continue
#             sel(entType)
#             ents = ndir()
#             for ent in ents:
#                 if (not exists(ent)):
#                     continue
#                 if (ent.lower().startswith(name.lower())):
#                     return '/game/ents/' + str(entType) + '/' + str(ent)
                
#                 sel('..')
#             sel('..')

#         popsel()
#         return ''


class waitThread(threading.Thread):
    """Waits for threads to die"""
    def __init__(self, thread1, thread2):
        threading.Thread.__init__(self)
        self.t1 = thread1
        self.t2 = thread2

    def run(self):
        log.write("Waiting for threads to die")
        self.t1.join()
        self.t2.join()
        log.write("Threads died...")

def interp(vec1, vec2, weight1, weight2):
    vec = [vec1[0], vec1[1], vec1[2]]
    for i in range(len(vec1)):
        vec[i] = ((vec1[i] * weight1) + (vec2[i] * weight2))/(weight1 + weight2)
    return vec

def vecDiff(vec1, vec2):
    vec = [vec1[0], vec1[1], vec1[2]]
    for i in range(len(vec1)):
        vec[i] = (vec1[i] - vec2[i])
    return vec

        
def getCameraPos():
    pushsel()
    sel ('/game/camera')
    cpos = gv('pos')
    pos = (cpos[0], 0.0, cpos[2])
    heading = gf('heading')
    log.write('Camera Heading before SROT: ' + str(heading))
    srot('rot', 0, 180, 0)
    log.write('Camera Position: ' + str(cpos) + " flat: " + str(pos))
    heading = gf('heading')
    log.write('Camera Heading after SROT: ' + str(heading))
    popsel()
    return pos

def interpHeading(current, head2, currentHeadingWeight):
    return ((currentHeadingWeight * current) + head2)/(currentHeadingWeight + 1.0)

def zeroVec(vec):
    for i in range(len(vec)):
        if(closeTo(vec[i],0)):
            return True
    return False

def closeTo(n, target):
    if (math.fabs(float(n - target)) < 3):
        return True
    else:
        return False

