#
# @(#) stateclient.py 2005/07/25
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


import threading
import socket
import time
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'comms'])
import nEventServer
from common import EventNames
import sys
import nEventServer
import splitter
from stringnames import *
from translate import *
import sp
import const # contains COVE_LAGOON_MAP
import math

sleepTime = 0.5

class StateThread(threading.Thread):
    """Obtains, parses, and stores cove's entities' state"""
    def __init__(self, host, port, comm):
        threading.Thread.__init__(self)
        log.write(self, "State Thread being initialized")
        nEventServer.register(EventNames.COVE_STATE_DISCONNECT, self.handleStateDisconnect)
        self.host = host
        self.port = port
        self.comm = comm
        self.pathCache = {}
        self.isFirstEnt = True

    def run(self):
        log.write("State Thread starting")
        self.stateSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.stateSocket.connect((self.host, self.port))
        log.write("Opened state socket")
        log.write("State Thread: Cove connecting from: " + `self.host` + ':' +
                  `self.port`)
        self.sdie = 0
        data = ''
        while self.sdie == 0:
            data = data + self.getState(self.stateSocket)
            leftover = self.updateEnts(data)
            data = leftover
#            log.write("Leftover Data in run: " + data)
#            time.sleep(sleepTime)
        self.stateSocket.close()
        log.write("State Thread received disconnect signal")

    def handleStateDisconnect(self, msg):
        print 'Handling state disconnect'
        self.sdie = 1

    def getState(self, con):
#        log.write("Getting state information from " + `con`)
        try:
            dta = self.stateSocket.recv(8192)
        except:
            nEventServer.pull(EventNames.COVE_COMMAND_DISCONNECT, '')
            self.sdie = 1
            dta = ''
            printExcept('Cove seems to have died...')
            
#        log.write("Received: " + dta)
        return dta
        
    def updateEnts(self, data):
#        log.write("Updating entity state with cove data: \n" + `data`)
        res, left = splitter.coveDataParse(data)
        structuredInfo = splitter.extractInfo(res)
        for info in structuredInfo: # a list of dictionaries
            self.updateLagoonEnt(info)
#        log.write(structuredInfo)
#        log.write("")
        return left

    def updateLagoonEnt(self, info):
        try:
            tmpEnt = info[LABELSTRING]
            pos = info[LOCATIONSTRING]
            vel = parsePos(info[VELOCITYSTRING])
            heading = float(info[HEADINGSTRING])
            speed = float(info[SPEEDSTRING])
            ent = tmpEnt.strip() #tmpEnt.strip().lower()
            if(self.comm.hasCachedCoveEntity(ent)):
                tpos = parsePos(pos)
                self.setNewPosData(ent, tpos, heading, speed, vel)
                nEventServer.pull(EventNames.COVE_POSITION_ENTITY, '')
#                log.write("Update Existing Entity: Cove:" + str(ent) + "(" + str(parsePos(pos)) + ")     Lagoon: Pos (" + str(tpos) + ")" )
            else:
                if(self.isFirstEnt):
                    self.comm.translator.setCovePoint(parsePos(pos))
                    self.isFirstEnt = False
                for label, mapping in const.COVE_LAGOON_MAP.iteritems():
                    if ent.startswith(label):
                        entType, entSide = mapping
                        #entType, entSide = const.COVE_LAGOON_MAP[ent]
                        entGroup = genNewGroup()
                        tpos = parsePos(pos)
                        self.setNewCoveData(ent, entGroup, entSide, entType, tpos, heading, speed, vel)
                        nEventServer.pull(EventNames.COVE_CREATE_ENTITY, '')
                        log.write("Create New Entity: Cove:" + str(ent) + "("  + str(parsePos(pos)) + ")     Lagoon type: " + str(entType) + "(" + str(tpos) + ")" )
                        break
        except:
            printExcept("updateLagoonEnt")

    def setNewCoveData(self, label, group, side, type, pos, heading, speed, vel):
        data = {}
        data[COVELABEL]          = str(label)
        data[LAGOONGROUP]        = str(group)
        data[LAGOONSIDE]         = str(side)
        data[LAGOONTYPE]         = str(type)
        data[LAGOONPOS]          = self.comm.translate(pos)
        data[LAGOONHEADING]      = self.comm.rotate(heading)
        data[LAGOONSPEED]        = speed
        data[LAGOONVELOCITY]     = vel
        print "Cove heading: " + str(heading) + " Lagoon heading: " + str(data[LAGOONHEADING])
        self.comm.setNewEntityData(data)
        time.sleep(1)

    def setNewPosData(self, label, pos, heading, speed, vel):
        posData = {}
        posData[ENTPATH]        = str(self.comm.getCachedCoveEntityPath(label))
        posData[LAGOONPOS]      = self.comm.translate(pos)
        posData[LAGOONHEADING]  = self.comm.rotate(heading)
        posData[LAGOONSPEED]    = speed
        posData[LAGOONVELOCITY] = vel
#        print "Cove heading: " + str(heading) + " Lagoon heading: " + str(posData[LAGOONHEADING])
        cd = self.comm.radiansToDegrees(heading)
        ld = self.comm.radiansToDegrees(posData[LAGOONHEADING])
#        print "Cove degrees: " + str(cd) + " Lagoon degrees: " + str(ld)
        self.comm.setNewPosData(posData)
        
#        pass
#        log.write("Setting " + str(label) + " new position to: " + str(pos))

def parsePos(posString):
#    seq = posString.split(')')
#    item = seq[0]
#    posV = item.strip('()')
#    print "Strip (), Position: " + posV
#    tmp = posV.replace('(', '')
#    print "Position: " + str(posString)
    posSeq = posString.split()
    x = float(posSeq[1])
    y = float(posSeq[2])
    z = -1.0 * float(posSeq[0])
#    print "Position: " + str(x) + ", " + str(y) + ", " + str(z)
    return (x, y, z)

def parseVec(vecString):
#    seq = posString.split(')')
#    item = seq[0]
#    posV = item.strip('()')
#    print "Strip (), Position: " + posV
#    tmp = posV.replace('(', '')
#    print "Position: " + str(posString)
    posSeq = posString.split()
    x = float(posSeq[1])
    y = float(posSeq[2])
    z = -1.0 * float(posSeq[0])
#    print "Position: " + str(x) + ", " + str(y) + ", " + str(z)
    return (x, y, z)

counter = 0
def genNewGroup():
    global counter
    gn = "Group" + str(counter)
    counter += 1
#    log.write("=============group Name: " + str(gn))
    return gn

