from npython import *
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'evaexplorer'])
from evatree import EVATree
import wx
import panels
import sys
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
selection = None
def GetSelection():
    return selection
class EVAExplorer(wx.Frame):
    """show 2 panels - on the left a tree containing all objects inside the eva tree
    on the right a window showing options for whatever object is selected on the left
    user should be able to change / modify everything for that object

    so how do we do this?
    welll the eva tree is fairly straightforward - all we need to do now is bind selection events into opening the right window
    what we need is a different window class for each kind of object(nent, naspect, nshadernode, etc)
    then get those windows to open, similar to what we did before with tcl, but much nicer...

    now we should create a default selected object whatever =) 

    okay so when an object is selected it pulls this function, which is awesome, and then we can get the full path, easy
    now we need to get the class of the object, and from there pull the inheritence heirarchy
    we can do this with just a list of classes, walk through them and do is isa
    """
    #/-----------------------------------------------------------------------------//
    def __init__(self,parent,id):
        import panels
        #FIXME-try _import or find some other better way to do this
        import panels.unselectedpanel
        import panels.nrootpanel
        import panels.nthingpanel
        import panels.nentpanel
        import panels.nhardpointpanel
        from common import WindowManager 
        #print 'EVAExplorer.init'

        windowName                      = 'EVAExplorer'
        WindowManager.register          (windowName, self)
        position,size                   = WindowManager.get(windowName)

        wx.Frame.__init__(self,parent,id,"EVA Explorer", pos=position, size=size)

        menu = wx.Menu()
        menu.Append(1001,'&Refresh\tAlt-R','Refresh the Eva tree' )
        menubar = wx.MenuBar()
        menubar.Append( menu, 'Tree')
        self.SetMenuBar( menubar )
        self.CreateStatusBar()
        self.splitter = Splitter( self )
        sty = wx.BORDER_SUNKEN
        
        self.evaTree                    = EVATree(self.splitter, -1, self.OnSelChanged)
        self.Bind                       (wx.EVT_MENU, self.evaTree.refresh, id = 1001)
        self.notebook                   = ObjectNotebook(self.splitter, -1)
        self.splitter.SetMinimumPaneSize(20)
        self.splitter.SplitVertically   (self.evaTree, self.notebook, 300)
        self.Show()
    #/-----------------------------------------------------------------------------//
    def OnSelChanged( self, event ):
        """a new part of the tree has been selected 
        find its class and all parent classes, then display the appropriate panels
        """
        import panels
        item = event.GetItem()
        if not item:
            assert 0, 'managed to unselect'
        path = self.evaTree.tree.GetPyData( item )
        log.write('Sel Changed: ', path )
        clazz = call('%s.getclass' % path)
        global selection
        selection = path

        parentClasses = []
        from evatree import NebulaClasses
        for cl in NebulaClasses:
            if call('%s.isa' % path, cl ):
                parentClasses.append(cl)

        self.notebook.DeleteAllPages()
        lastPage = 0
        for i, cl in enumerate(parentClasses):
            panel = wx.Panel(self.notebook, -1)
            sizer = wx.GridBagSizer(1, 1)
            sizer.AddGrowableRow(0)
            sizer.AddGrowableCol(0)
            panel.SetSizer(sizer)
            self.notebook.AddPage(panel, cl)
            cl += 'Panel'
            try:
                panelModule             = getattr(panels, cl.lower())
                panelClass              = getattr(panelModule, cl)
                window                  = panelClass(panel, -1, path)
                sizer.Add               (window, (0,0), flag=wx.EXPAND)
                lastPage                = i
            except AttributeError:
                printExcept()
                st                      = wx.StaticText(panel, -1, 'No panel for %s' % cl)
                sizer.Add               (st, (0,0), flag=wx.EXPAND)
        self.notebook.SetSelection(i)                        #select the last panel (exact class being used)
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
class Splitter(wx.SplitterWindow):
    #/-----------------------------------------------------------------------------//
    def __init__(self,parent):
        #print 'MainPanel.init'
        wx.SplitterWindow.__init__(self,parent,-1,style = wx.SP_LIVE_UPDATE )
        self.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGED, self.OnSashChanged )
        self.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGING, self.OnSashChanging )
    #/-----------------------------------------------------------------------------//
    def OnSashChanged( self, evt ):
        log.write("sash changed to %s\n" % str(evt.GetSashPosition()))
    #/-----------------------------------------------------------------------------//
    def OnSashChanging( self, evt ):
        log.write("sash changing to %s\n" % str(evt.GetSashPosition())) 
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
class ObjectNotebook(wx.Notebook):
    def __init__(self,parent, id):
        wx.Notebook.__init__(self,parent,id)
        panel1 = wx.Panel(self, -1)
        panel2 = wx.Panel(self, -1)
        st1 = wx.StaticText(panel1, -1, 'Panel 1')
        st2 = wx.StaticText(panel2, -1, 'Panel 2')
        self.AddPage( panel1, 'Panel 1')
        self.AddPage( panel2, 'Panel 2')
        self.DeleteAllPages()

