import sys
import wx
from wx import ListCtrl
from npython import *
from common.log import Log
log = Log(tags=['all', 'evaexplorer'])
class nentPanel(wx.Panel):
    def __init__(self, parent, id, path):
        """
        firstly - display all the relevant information about this entity
        pos          22400.0000 -0.0001 22400.0000          l:31678.3848
        rot          0.0000 1.0000 0.0000 0.0000       p:0.0000 r:0.0000 y:0.0000
        side         /game/sides/blue
        octradius    1.000000
        priority     5
        hp           100.000000
        mass         100.000000
        inertia      50.000000
        friction     0.100000
        rfriction    0.500000
        enginePower  1.000000
        rudderTorque 0.003000
        pythonmodule sp
        pythonclass  Wander
        vel          0.0000 -0.0100 0.0000          l:0.0100
        force        0.0000 0.0000 0.0000          l:0.0000
        rvel         0.0000 0.0000 0.0000 0.0000       p:0.0000 r:0.0000 y:0.0000
        torque       0.0000 0.0000 0.0000          l:0.0000
        alignedtorque 0.0000 0.0000 0.0000          l:0.0000
        label        Le Magnifique
        minalt       0.000000
        maxalt       -1.000000
        constantforce 0.0000 -1.0000 0.0000          l:1.0000
        throttle     0.000000
        rudder       0.000000
        engineRudderDissipation 0.000000
        keelforce    1.000000
        ********************************************************
        ////////////////////////////////////////////////////////
        vars physics engine gravity altlimit ai

        ////////////////////////////////////////////////////////
        """
        from npython import call
        log.write('nentpanel().init')
        wx.Panel.__init__(self,parent, id, style = wx.EXPAND | wx.SUNKEN_BORDER | wx.TAB_TRAVERSAL | wx.CLIP_CHILDREN | wx.FULL_REPAINT_ON_RESIZE)

        gbs = self.gbs = wx.GridBagSizer(4, 3)
        self.gbs.SetVGap(1)
        self.gbs.SetHGap(10)

        #display the basics - name/path/type
        sel(path)
        name = call('%s.getname' % path)
        type = call('../.getname')

        gbs.Add                         (wx.StaticText(self, -1, 'Name'), (0, 0), (1, 1), wx.ALIGN_CENTER)
        gbs.Add                         (wx.StaticText(self, -1, 'Path'), (1, 0), (1, 1), wx.ALIGN_CENTER)
        gbs.Add                         (wx.StaticText(self, -1, 'Type'), (2, 0), (1, 1), wx.ALIGN_CENTER)

        gbs.Add                         (wx.TextCtrl(self, -1, name), (0,1), flag=wx.EXPAND ) 
        gbs.Add                         (wx.TextCtrl(self, -1, path), (1,1), flag=wx.EXPAND ) 
        gbs.Add                         (wx.TextCtrl(self, -1, type), (2,1), flag=wx.EXPAND )


        #okay lets make an aspect list
        self.aspectlist                 = ListCtrl(self, -1, style=wx.LC_REPORT)
        self.aspectlist.InsertColumn    (0, 'Role')
        self.aspectlist.InsertColumn    (0, 'Class')

        sel(path)
        aspects = ndir()
        for aspect in aspects:
            if aspect == 'vars':                                                continue
            if not call('%s.isa' % aspect, 'naspect'):                          continue
            log.write('aspect', aspect)
            index = self.aspectlist.InsertStringItem                            (sys.maxint, aspect)
            type = call                                                         ('%s.getclass' % aspect)
            self.aspectlist.SetStringItem                                       (index, 1, type)
            log.write('index', index)
        #print 'adding to grid'
        gbs.Add                         (self.aspectlist, (3,0), (1,3), flag=wx.EXPAND)
        numaspects = len(aspects)
        if numaspects == 0:
            numaspects = 1
        gbs.AddGrowableRow              (3, numaspects)

        #now what - well lets get a list of aspects and a list of vars
        self.varlist                    = ListCtrl(self, -1, style=wx.LC_REPORT)
        self.varlist.InsertColumn       (0, 'Var Name')
        self.varlist.InsertColumn       (1, 'Var Type')
        self.varlist.InsertColumn       (2, 'x')
        self.varlist.InsertColumn       (3, 'y')
        self.varlist.InsertColumn       (4, 'z')
        self.varlist.InsertColumn       (5, 'w')

        sel('%s/vars' % path)
        vars = ndir()
        sel('..')
        for var in vars:
            index = self.varlist.InsertStringItem(sys.maxint, var)
            try:
                type = call('vars/%s.gettype' % var)
            except:
                type = 'void'

            self.varlist.SetStringItem  (index, 1, type)
            x = y = z = w = None
            if type == 'int':           x                                       = call('gi'   , var)
            if type == 'float':         x                                       = call('gf'   , var)
            if type == 'string':        x                                       = call('gs'   , var)
            if type == 'ref':           x                                       = call('gref' , var)
            if type == 'vector3':       x, y, z                                 = call('gv'   , var)
            if type == 'quaternion':    x, y, z, w                              = call('gq'   , var)
            
            if x:                       self.varlist.SetStringItem(index, 2, str(x))
            if y:                       self.varlist.SetStringItem(index, 3, str(y))
            if z:                       self.varlist.SetStringItem(index, 4, str(z))
            if w:                       self.varlist.SetStringItem(index, 5, str(w))

        gbs.Add                         (self.varlist, (4, 0), (1,3), flag=wx.EXPAND)
        numvars = len(vars)
        if numvars == 0:
            numvars = 1
        gbs.AddGrowableRow              (4, numvars)
        gbs.AddGrowableCol              (1)
        self.SetSizerAndFit             (gbs)
