from npython import *
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'ga'])
import wx
import wx.lib.mixins.listctrl
import nEventServer
from common import EventNames
import sys
#/-----------------------------------------------------------------------------//
class GAList(wx.ListCtrl, wx.lib.mixins.listctrl.ListCtrlAutoWidthMixin):
    #/-----------------------------------------------------------------------------//
    def __init__(self, parent, id, path, size=wx.DefaultSize):
        wx.ListCtrl.__init__(self, parent, id, style=wx.LC_REPORT | wx.BORDER_SUNKEN, size=size) #LC_EDIT_LABELS
        self.path = path
        wx.lib.mixins.listctrl.ListCtrlAutoWidthMixin.__init__(self)
        nEventServer.register( EventNames.GA_GENERATION_FINISHED, self.update)
        nEventServer.register( EventNames.GA_GENOME_CHANGED, self.updateIndividual)
        self.InsertColumn(0, 'Genome')
        self.InsertColumn(1, 'State')
        self.InsertColumn(2, 'Rk')
        self.InsertColumn(3, 'Gen')
        self.InsertColumn(4, 'Fitness')
        self.InsertColumn(5, 'Shared')
        self.InsertColumn(6, 'Sim')
        self.InsertColumn(7, '%tile')
        self.InsertColumn(8, 'Norm')
        self.InsertColumn(9, 'Bits')

        self.SetColumnWidth(0, 80)
        self.SetColumnWidth(1, 120)
        self.SetColumnWidth(2, 30)
        self.SetColumnWidth(3, 35)
        self.SetColumnWidth(4, 60)
        self.SetColumnWidth(5, 60)
        self.SetColumnWidth(6, 60)
        self.SetColumnWidth(7, 40)
        self.SetColumnWidth(8, 40)
        self.SetColumnWidth(9, 1200)
        self.genomesToIndexes = {}

        self.itemDataMap = {}
    #/-----------------------------------------------------------------------------//
    def update(self, msg):
        for genome in ndirin(msg):
            path = '%s/%s' % (msg, genome)
            #print 'GAList.update', path
            self.updateIndividual( path )
    #/-----------------------------------------------------------------------------//
    def updateIndividual(self, path):
        try:
            parent = call('%s.getparentpath' % path)
            if parent != self.path:
                #print '%s ignoring %s->%s' % ( self.path, parent, path)
                return
            if path not in self.genomesToIndexes:
                index = self.InsertStringItem(sys.maxint, getname(path))
                self.SetStringItem(index, 0, getname(path))
                self.genomesToIndexes[path] = index
                self.SetItemData(index, index)
                self.itemDataMap[self.genomesToIndexes[path]] = range(10)

            self.itemDataMap[self.genomesToIndexes[path]][1] = call('%s.getstate' % path)
            self.itemDataMap[self.genomesToIndexes[path]][2] = call('%s.getrank' % path)
            self.itemDataMap[self.genomesToIndexes[path]][3] = call('%s.getgeneration' % path)
            self.itemDataMap[self.genomesToIndexes[path]][4] = call('%s.getfitness' % path)
            self.itemDataMap[self.genomesToIndexes[path]][5] = call('%s.getsharedfitness' % path)
            self.itemDataMap[self.genomesToIndexes[path]][6] = call('%s.getsimilarity' % path)
            self.itemDataMap[self.genomesToIndexes[path]][7] = call('%s.getfitnesspercentile' % path)
            self.itemDataMap[self.genomesToIndexes[path]][8] = call('%s.getnormalizedfitness' % path)
            self.itemDataMap[self.genomesToIndexes[path]][9] = str(call('%s.getbits' % path))

            self.SetStringItem( self.genomesToIndexes[path], 1, call('%s.getstate' % path))
            self.SetStringItem( self.genomesToIndexes[path], 2, '%i' % call('%s.getrank' % path))
            self.SetStringItem( self.genomesToIndexes[path], 3, '%i' % call('%s.getgeneration' % path))
            self.SetStringItem( self.genomesToIndexes[path], 4, '%.2f' % call('%s.getfitness' % path))
            self.SetStringItem( self.genomesToIndexes[path], 5, '%.2f' % call('%s.getsharedfitness' % path))
            self.SetStringItem( self.genomesToIndexes[path], 6, '%.2f' % call('%s.getsimilarity' % path))
            self.SetStringItem( self.genomesToIndexes[path], 7, '%.2f' % call('%s.getfitnesspercentile' % path))
            self.SetStringItem( self.genomesToIndexes[path], 8, '%.2f' % call('%s.getnormalizedfitness' % path))
            self.SetStringItem( self.genomesToIndexes[path], 9, str(call('%s.getbits' % path)))
        except:
            printExcept('GAExplorer.updateIndividual')
#/-----------------------------------------------------------------------------//
class GAExplorer(wx.Panel, wx.lib.mixins.listctrl.ColumnSorterMixin):
    #/-----------------------------------------------------------------------------//
    def __init__(self,parent,id, path):
        wx.Panel.__init__(self,parent,id)
        self.path = path
        sizer = wx.GridBagSizer(0,0)
        t = wx.StaticText(self, -1,  'Evaluation Time Remaining', size=(222,30))
        sizer.Add( t, (0,0), flag=wx.EXPAND)
        t = self.timeRemainingCtrl = wx.Gauge(self, -1, range=int(call('/game/ga.gf', 'evaluationTime')), size=(200,30))
        sizer.Add( t, (0,1), flag=wx.EXPAND)

        t = wx.StaticText(self, -1,  'Num Evaluations:', size=(222,30))
        sizer.Add( t, (1,0), flag=wx.EXPAND)
        t = self.numEvalsCtrl = wx.TextCtrl(self, -1, size=(200,30))
        sizer.Add( t, (1,1), flag=wx.EXPAND)

        t = self.gaList = GAList( self, -1, path)
        sizer.Add( t, (2,0), span=(1,2), flag=wx.EXPAND)
        sizer.AddGrowableRow(2)
        sizer.AddGrowableCol(1)
        self.SetSizer( sizer )
        nEventServer.register( EventNames.GA_GENERATION_FINISHED, self.generationFinished)
        from common import tickmanager
        tickmanager.register( 'GAExplorer', self.tick )
        self.itemDataMap = self.gaList.itemDataMap
        wx.lib.mixins.listctrl.ColumnSorterMixin.__init__(self,10)
    #/-----------------------------------------------------------------------------//
    def GetListCtrl( self ):
        return self.gaList
    #/-----------------------------------------------------------------------------//
    def generationFinished(self, msg):
        #print 'GAExplorer.ganeration finished', msg
        pass
    #/-----------------------------------------------------------------------------//
    def tick(self, time, dtime):
        timeRemaining = call('/game/ga.gf', 'evaluationTimeRemaining')
        if ( timeRemaining < 0 ):
            timeRemaining = 0
        self.timeRemainingCtrl.SetValue ( timeRemaining )
        self.numEvalsCtrl.SetValue      ( '%i' % call('/game/ga.gi', 'evaluationsDone'))
#/-----------------------------------------------------------------------------//
class GANotebook( wx.Notebook ):
    def __init__(self, parent, id):
        wx.Notebook.__init__(self, parent, id)
        self.gas = {}
        nEventServer.register( EventNames.GA_RUNNING, self.onGaRunning)
    def onGaRunning(self, gaPath):
        #print 'GANotebook.onGaRunning', gaPath
        if gaPath not in self.gas:
            print 'new ga', gaPath
            self.gas[gaPath] = GAExplorer(self, -1, gaPath )
            self.AddPage( self.gas[gaPath], gaPath )
#/-----------------------------------------------------------------------------//

from common import OptionManager
frame = wx.Frame(None,-1, title='GAExplorer', pos=(1250,0), size=(422,501))
#frame = wx.Frame(None,-1, title='GAExplorer', pos=(1250,0), size=(669,660))
gaNotebook = GANotebook( frame, -1 )
#gaExplorer = GAExplorer( gaNotebook,-1)
#gaNotebook.AddPage( gaExplorer, 'GA' )
#sizer = wx.GridBagSizer(0,0)
#sizer.Add(GAExplorer, (0,0), flag=wx.EXPAND)
#frame.SetSizer( sizer )
if OptionManager.getOption('gaexplorer'):
    frame.Show()
import garecorder
#/-----------------------------------------------------------------------------//
