from npython import *
from common.defs import *
import random
import nEventServer
from common                             import EventNames
from common import OptionManager
MAX_PLAYERS = 64
#/-----------------------------------------------------------------------------//
class PlayerManager:
    """manages the players and their info"""
    #/-----------------------------------------------------------------------------//
    def __init__(self):
        new( 'nthing', '/game/players')
        nEventServer.register( EventNames.UPDATE_PLAYER,                         self.updatePlayer )
        nEventServer.register( EventNames.PLAYERS_CHANGED,                       self.playersChanged )
    #/-----------------------------------------------------------------------------//
    def createLocalPlayer(self):
        print 'PlayerManager.createLocalPlayer'
        playerName = self.createPlayerName()
        print 'playerName', playerName
        pushsel('/game/players')
        sn('nbasicent', playerName)
        ci('local', 1)
        playername = OptionManager.getOption('name')
        if not playername:
            playername = 'localplayer'
        cs('playername', playername)

        playerside = OptionManager.getOption('side')
        if not playerside:
            playerside = 'neutral'
        cs('playerside', '/game/sides/%s' % playerside)
        
        playerfaction = OptionManager.getOption('faction')
        if not playerfaction:
            playerfaction = 'Space Marines'
        cs('playerfaction', playerfaction)

        nEventServer.pullLater(EventNames.PLAYERS_CHANGED, 'createLocalPlayer')
    #/-----------------------------------------------------------------------------//
    def findLocalPlayer(self):
        for player in ndirin('/game/players'):
            playerPath = '/game/players/%s' % player
            if call('%s.gi' % playerPath , 'local') == 1:
                return playerPath
    #/-----------------------------------------------------------------------------//
    def findLocalSide(self):
        return call('%s.gs' % self.findLocalPlayer(), 'playerside')
    #/-----------------------------------------------------------------------------//
    def findLocalSideType(self):
         return call('%s.gs' % self.findLocalSide(), 'sidetype')
    #/-----------------------------------------------------------------------------//
    def createPlayerName(self):
        for i in range(10):
            name = 'Player%i' % random.randrange(0,10000)
            if not exists('/game/players/%s' % name):
                break
        assert i < 10, 'somehow failed to create a player name'
        return name
    #/-----------------------------------------------------------------------------//
    def setLocalPlayerName(self, playerName):
        playerPath = self.findLocalPlayer()
        print 'setting player %s.name = %s' % (playerPath, playerName)
        call('%s.ss' % playerPath, 'playername', playerName)
        OptionManager.setOption('name', playerName)
        nEventServer.pullLater(EventNames.PLAYERS_CHANGED, 'setLocalPlayerName')
    #/-----------------------------------------------------------------------------//
    def setLocalPlayerSide(self, sideName):
        print 'setLocalPlayerSide', sideName
        sideName = '/game/sides/%s' % sideName
        playerPath = self.findLocalPlayer()
        print 'setting player %s.side = %s' % (playerPath, sideName)
        call('%s.ss' % playerPath, 'playerside', sideName)
        popsel()
        nEventServer.pullLater(EventNames.SIDES_CHANGED, 'local:%s' % sideName)
        nEventServer.pullLater(EventNames.PLAYERS_CHANGED, 'setLocalPlayerSide')
        #/-----------------------------------------------------------------------------//
    def setLocalPlayerFaction(self, factionName):
        print 'setLocalPlayerFaction', factionName
        playerPath = self.findLocalPlayer()
        print 'setting player %s.name = %s' % (playerPath, factionName)
        call('%s.ss' % playerPath, 'playerfaction', factionName)
        nEventServer.pullLater(EventNames.PLAYERS_CHANGED, 'setLocalPlayerFaction')
    #/-----------------------------------------------------------------------------//
    def canControlGroup(self, groupName):
        """check to see if i can legally control this group"""
        playerPath = self.findLocalPlayer()
        sidetype = self.findLocalSideType()
        #check if i can legally control this unit
        if sidetype == 'bm':            #bm can control everything
            return True
        elif sidetype == 'observer':    #observer cant control anything
            return False
        elif sidetype == 'combat' or sidetype == 'neutral':                           #other types (neutral/combat) can control their own units
            playerSide = call('%s.gs' % playerPath, 'playerside')
            canControl = True
            for entName in ndirin('/comm/groups/%s' % groupName):
                path = call('/comm/groups/%s/%s.gs' % (groupName, entName), 'path')
                side = call('%s.gref' % path, 'side')
                #print 'entName', entName, 'path', path, 'side', side, 'playerPath', playerPath, 'playerSide', playerSide
                canControl = (canControl and side == playerSide)
            return canControl
        else:
            print 'unknown sidetype %s' % sidetype
            return False
    #/-----------------------------------------------------------------------------//
    def canControlEnt(self, entPath):
        if not exists(entPath):
            print 'calling canControlEnt on a non existent ent:', entPath
            return False

        playerPath = self.findLocalPlayer()
        sidetype = self.findLocalSideType()
        if sidetype == 'bm':            #bm can control everything
            return True
        elif sidetype == 'observer':    #observer cant control anything
            return False
        elif sidetype == 'combat' or sidetype == 'neutral':                           #other types (neutral/combat) can control their own units
            playerSide = call('%s.gs' % playerPath, 'playerside')
            side = call('%s.gref' % entPath, 'side')
            return side == playerSide
    #/-----------------------------------------------------------------------------//
    def updatePlayer(self, msg):
        print 'updatePlayer', msg
        id, playerName, playerSide, playerFaction = msg.split(':')
        id                              = 'Player%i' % int(id)
        playerName                      = str(playerName)
        playerSide                      = str(playerSide)
        playerFaction                   = str(playerFaction)
        pushsel('/game/players')

        changed = False
        #player doesn't exist locally - create
        if ( exists(id) == False):
            print 'creating new remote player', id
            sn('nbasicent',             id)
            ci('local',                 0)
            cs('playername',            playerName)
            cs('side',                  playerSide)
            cs('faction',               playerFaction)
            changed = True
        #check to see if any of the data has changed
        else:
            sel(id)
            if gs('playername') != playerName:
                print 'updating player %s name from %s to %s' % ( id, gs('playername'), playerName)
                ss('playername', playerName)
                changed = True
            if gs('playerside') != playerSide:
                print 'updating player %s side from %s to %s' % ( id, gs('playerside'), playerSide)
                ss('playerside', playerSide)
                changed = True
            if gs('playerFaction') != playerFaction:
                print 'updating player %s faction from %s to %s' % ( id, gs('playerFaction'), playerFaction)
                ss('playerFaction', playerFaction)
                changed = True
        if changed:
            nEventServer.pullLater(EventNames.PLAYERS_CHANGED, 'updatePlayer')
        popsel()
    #/-----------------------------------------------------------------------------//
    def playersChanged(self, msg):
        print 'playersChanged', msg
        call('/net/tnl.playersChanged')
        
#/-----------------------------------------------------------------------------//
playerManager = PlayerManager()
#print 'done loading playermanager'
