from npython import *
from common.defs import *
import common.tickmanager
from sp import sp
from im.im import imManager
from common.log import Log
log = Log(tags=['all', 'sp', 'PlanReplayer'])
import nEventServer
from common import EventNames
class PlanReplayer:
    """replays a plan saved to a file, supported commands:
        rightclick
        createGroup
        assignEntToGroup
    """
    #/-----------------------------------------------------------------------------//
    def __init__(self, filename, instant=False):
        log.write( 'Replaying plan', filename, instant)
        common.tickmanager.register('PlanReplayer', self.tick)
        doc = parseXML(filename)
        #print 'doc', doc.toxml()
        self.commands = []
        try:
            for command in doc.getElementsByTagName('command'):
                func = str(command.getAttribute('func'))
                if not instant:
                    time = float(command.getAttribute('time'))
                else:
                    time = 0.0          #run them all right away
                self.commands.append(struct(func=func, time=time, node=command))
        except:
            printExcept('parsing plan %s' % filename)
    #/-----------------------------------------------------------------------------//
    def tick(self, time, dtime):
        #/-----------------------------------------------------------------------------//
        while (len(self.commands) > 0 and self.commands[0].time < time):
            command                     = self.commands.pop(0)
            #print                       'replayer execing command', command
            if command.func == 'createEnt':
                entType                 = str(command.node.getAttribute('entType'))
                groupName               = str(command.node.getAttribute('groupName'))
                if groupName == 'None': groupName = None
                position                = vector3(command.node.getAttribute('position'))
                side                    = str(command.node.getAttribute('side'))
                if side == 'None':      side = None
                #print 'replaying createEnt', entType, groupName, position, side

                from entmanager import entManager
                entManager.createEnt(entType=entType, groupName=groupName, position=position, side=side)
            #/-----------------------------------------------------------------------------//
            if command.func == 'setSide':
                entPath                 = str(command.node.getAttribute('entPath'))
                side                    = str(command.node.getAttribute('side'))

                from entmanager import entManager
                entManager.setSide(entPath = entPath, side=side)

            #/-----------------------------------------------------------------------------//
            if command.func == 'assignEntToGroup':
                path                    = str(command.node.getAttribute('path'))
                groupName               = str(command.node.getAttribute('groupName'))
                if groupName == 'None': groupName = None
                index                   = str(command.node.getAttribute('index'))
                if index == 'None':     index = None
                else:                   index = int(index)
                log.write( 'Replayer.assignEntToGroup', path, groupName, index)
                from groupmanager import groupManager
                groupManager.assignEnt( path, groupName, index)
            #/-----------------------------------------------------------------------------//
            if command.func == 'assignObjectiveToGroup':
                objectiveName           = str(command.node.getAttribute('objectiveName'))
                groupName               = str(command.node.getAttribute('groupName'))
                log.write( 'Replayer.assignObjectiveToGroup', objectiveName, groupName)
                sp.assignObjectiveToGroup( objectiveName, groupName )
            #/-----------------------------------------------------------------------------//
            if command.func == 'commandGroup':
                groupName               = str(command.node.getAttribute('groupName'))
                point                   = str(command.node.getAttribute('point'))
                point                   = vector3(point)
                targets                 = str(command.node.getAttribute('targets'))
                log.write( 'Replayer.commandGroup', groupName, point, targets )
                if targets == '[]':
                    targets = None
                sp.commandGroup( groupName, point, targets)
            #/-----------------------------------------------------------------------------//
            if command.func == 'moveCamera':
                pos                     = str(command.node.getAttribute('pos'))
                pos                     = vector3(pos)
                rot                     = str(command.node.getAttribute('rot'))
                rot                     = quaternion(rot)
                call('/game/camera.sv', 'pos', pos[0], pos[1], pos[2])
                call('/game/camera.sq', 'rot', rot[0], rot[1], rot[2], rot[3])
            #/-----------------------------------------------------------------------------//
            #if command.func == 'assignIMToGroup':
                #groupName               = str(command.node.getAttribute('groupName'))
                #im                      = str(command.node.getAttribute('im'))
                #log.write('Replayer.assignIMToGroup', groupName, im)
                #imManager.assignIMToGroup( im=im, groupName=groupName )
            #/-----------------------------------------------------------------------------//
            if len(self.commands) == 0:
                nEventServer.pull(EventNames.PLAN_LOADED, 'SP Replayer')
                
        else:
            #print 'next command time', self.commands[0].time
            pass

from common import OptionManager
instant = True
plan = OptionManager.getOption('loadplan')
if not plan:
    plan = OptionManager.getOption('loadplantimed')
    instant = False
if plan:
    shortfilename                   = plan
    filename                        = 'sp/xml/plans/%s.xml' % shortfilename
    try:
        replayer = PlanReplayer(filename, instant)
    except:
        printExcept('auto loading plan')
        del replayer
