#define N_IMPLEMENTS nCarAI
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ai/ncarai.h"
#include "map/nmap.h"

nNebulaScriptClass(nCarAI, "nai");

//------------------------------------------------------------------------------
/**
*/
nCarAI::nCarAI():
map( kernelServer, this )
{
    map = "/map/map";
    stateNames.Append("drive");
    stateNames.Append("backup");
}

void nCarAI::Initialize(){
    nAI::Initialize();
    this->AddToDisplayList();
}

void nCarAI::InitVars(){
    nAI::InitVars();
    destination.Connect	( ent, "destination");
    throttle.Connect	( ent, "throttle" );
    steering.Connect	( ent, "steering" );
    pos.Connect		( ent, "pos" );
    rot.Connect		( ent, "rot" );
    desiredHeading.Connect	( ent, "desiredheading" );
    desiredHeading.DefaultF	( 0.0f );
    wheelDirection.Connect	( ent, "wheeldirection" );
    wheelDirection.DefaultF	( 0.0f );
    destination.DefaultV( 0,0,0 );
    sensorRange.Connect	( ent, "sensorrange");
    sensorRange.DefaultF(	100.0f );
    leveledSensors.Connect	( ent, "leveledsensors");
    leveledSensors.DefaultI	( 0 );

    rc_dWheelHeadingToTorque.Connect( ent, "dwheelheadingtotorque" );
    rc_dWheelHeadingToTorque.DefaultInputRange( -N_PI, N_PI, 21 );
    rc_dWheelHeadingToTorque.DefaultOutputRange( -1.0f, 1.0f );
    rc_dWheelHeadingToTorque.DefaultRC( RC_LINE, -1.0f, 1.0f );

    rc_dWheelHeadingToForce.Connect( ent, "dwheelheadingtoforce" );
    rc_dWheelHeadingToForce.DefaultInputRange( -N_PI, N_PI, 21 );
    rc_dWheelHeadingToForce.DefaultOutputRange( -1.0f, 1.0f );
    rc_dWheelHeadingToForce.DefaultRC( RC_LINE, 1.0f, 1.0f );
    rangeSensors.SetFixedSize( 8 );
    averageSteps.Connect	( ent, "averagesteps");
    averageSteps.DefaultI	( 10 );

    avoidanceBias.Connect	( ent, "avoidancebias" );
    avoidanceBias.DefaultF	( 1.0f );

    nString name;
    for( int i = 0; i < 8; i ++ ) {
        rangeSensors[i] = 1.0f;
        name.PrintF("rangetoheading%i", i);
        rc_rangeToHeading[i].Connect( ent, name.Get() );
        rc_rangeToHeading[i].DefaultInputRange( 0.0f, 1.0f, 7 );
        rc_rangeToHeading[i].DefaultOutputRange( -N_PI, N_PI );
        rc_rangeToHeading[i].DefaultRC( RC_LINE, 0.0f, 0.0f );
    }
}

nCarAI::~nCarAI(){
}
//------------------------------------------------------------------------------
/**
*/

/**
  right now we find angles forward - to the side
  take the dot product
  and based on those we turn the appropriate amount
  what we need to do is..umm
  figure out where are wheels are headed
  where we want them to be
  and steer towards that

  now we need to add some projections...so start projecting rays. and draw some lines
  okay we have the ranges..now what
*/
void nCarAI::Tick( float time, float dtime ){
    nAI::Tick( time, dtime );
}
bool nCarAI::States( StateMachineEvent event, uint state, float time, float dtime ){
    BeginStateMachine
        State(Drive)
            OnTick
                vector3 p = pos.GetV();
                quaternion q = rot.GetQ();
                vector3 d = destination.GetV();
                vector3 x = d - p;
                float l = x.len();

                //at the goal -> stop
                if( l < 8 ) {
                    throttle.SetF( -1.0f );
                    steering.SetF( 0.0f );
                } else if(l < 10 ) {
                    throttle.SetF( 0.0f );
                    steering.SetF( 0.0f );
                } else {
                    if( avoidanceBias.GetF() > 0.0f )
                            Sense();
                    if( rangeSensors[4] < .1 )
                        SetState( Backup );
                    x.norm();	//vector to dest
                    this->DrawLine( p, p + x, blue );

                    vector3 right(1,0,0);
                    right = q.rotate( right );

                    vector3 forward( 0,0,-1 );
                    forward = q.rotate( forward );

                    //this->DrawLine( p, p + right * sidewaseness, red );
                    float heading			= (float) atan2( forward.x, forward.z );
                    float desiredHeading	= (float) atan2( x.x, x.z );
                    float avoidanceOffset	= 0.0f;
                    for( int i = 0; i < rangeSensors; i ++ ) {
                        float offset = rc_rangeToHeading[i].Response( rangeSensors[i] );
                        avoidanceOffset += offset;
                        this->DrawAngle( p, desiredHeading, blue );
                    }
                    desiredHeading += avoidanceOffset * avoidanceBias.GetF();
                    this->DrawAngle( p, desiredHeading, red );
                    this->DrawAngle( p, heading, white );
                    //this->DrawAngle( p, desiredHeading, red );
                    float wd = wheelDirection.GetF();
                    this->DrawAngle( p, heading + wd, yellow );

                    float dh = desiredHeading - heading;
                    this->DrawAngle( p, heading + dh, green );

                    //float desiredWheelDirection = n_clamp( dh, -1.0f, 1.0f );
                    float desiredWheelDirection = dh;
                    this->DrawAngle( p, heading + desiredWheelDirection, red );

                    float dWheelDirection = -desiredWheelDirection + wd;
                    //float st = n_clamp( dWheelDirection, -1.0f, 1.0f );
                    float st2 = rc_dWheelHeadingToTorque.Response( dWheelDirection );
                    steering.SetF( st2 );

                    float th = rc_dWheelHeadingToForce.Response( dWheelDirection );
                    throttle.SetF( th );
                }
        State(Backup)
            OnEnter
                time0 = time;
                    SetState( Drive );
            OnTick
                throttle.SetF( -1.0f );
                steering.SetF( 0.0f );
                if( time - time0 > 5.0 )
                    SetState( Drive );
    EndStateMachine
}

/*
   do a 360 sweep in say umm..i dunno...8 pieces
   */
void nCarAI::Sense(){
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();
    float range = sensorRange.GetF();
    int steps = averageSteps.GetI();
    for( int i = 0; i < rangeSensors; i ++ ) {
        //just need to get the vectorI
        float t = 2 * N_PI * i / (float) rangeSensors.Size();
        if ( leveledSensors.GetI() == 1 ) {
            vector3 forward(0,0,-1);
            forward = q.rotate( forward);
            t -= (float) atan2( forward.x, forward.z );
        }
        vector3 p1((float) (range * sin(t)), 0, (float)(-range * cos(t)));
        if ( leveledSensors.GetI() == 0 ) {
            p1 = q.rotate( p1 );
        }
        p1 += p;
        line3 line(p,p1);
        vector3 intersect;
        bool b = map->GetIntersect( line, intersect );
        if( b == false ) {
            intersect = p1;
            this->DrawLine( p, intersect, green );
        }
        else 
            this->DrawLine( p, intersect, red );
        rangeSensors[i] = (float)( rangeSensors[i] * ((steps - 1) / (float) steps) + ((intersect - p).len() / sensorRange.GetF()) * (1.0 / (float) steps) );
        vector3 d = intersect - p;
        d.norm();
        float ratio = 1.0f;
        for( int j = 0; j <= i; j ++ ) {
            vector3 rp = p + d * rangeSensors[i] * sensorRange.GetF() * ratio;
            ratio -= .025f;
            this->DrawLine( rp - vector3( 0,1, 0), rp + vector3(0,1,0), white );
        }
    }
}
