#define N_IMPLEMENTS nThreatAI
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ai/nthreatai.h"
#include "util/nheap.h"

nNebulaScriptClass(nThreatAI, "ncontroller");

//------------------------------------------------------------------------------
/**
*/
nThreatAI::nThreatAI():
    platformRoot( kernelServer, this ),
    targettedNode( kernelServer, this ),
    lastFired( -1.0f )
{
}

void nThreatAI::Initialize(){
    nController::Initialize();
    platformRoot = "/game/ents/platform";
    if( Simple() == false )
        targettedNode = (this->CreateVis("/vis/targetted", "ui")).Get();
}

void nThreatAI::InitVars(){
    nController::InitVars();
    pos                 	.Connect( ent, "pos" );
    target              	.Connect( ent, "target" );
    target              	.DefaultRef("none");
    targetPos           	.Connect( target, "pos" );
    detectionRange      	.Connect( ent, "detectionrange");
    detectionRange      	.DefaultF( 50 );
    effectiveRange      	.Connect( ent, "effectiverange");
    effectiveRange      	.DefaultF( 20 );
    fires               	.Connect( ent, "type" );
    fires               	.DefaultS("sam");
    active		.Connect( ent, "active" );
    active		.DefaultF( 1 );
    cooldown		.Connect( ent, "cooldown" );
    cooldown		.DefaultF( 0.1f );
    const float ringDensity = 12.0f;
    nString name;
    n3DNode* ring;
    float r;
    int numRings;
    char ringName[1024];
    nVisNode* vis = (nVisNode*) ent->GetVis();
    if( vis ) {
        ring = (n3DNode*) vis->Find( "effectiverange" );
        r = effectiveRange.GetF();
        ifnonzero( ring ){
            ring->Sxyz(r,1,r);
            ring->GetFullName( ringName, 1024 );
            numRings = int(r / ringDensity);
            for( int i = 1; i < numRings; i ++ ){
	char name[1024];
	sprintf( name, "%s%i", ringName, i );
	n3DNode* ring2 = (n3DNode*) ring->Clone( name );
	float t = 360.0f * i / (float)numRings / 25.0f;
	ring2->Ry( t );
            }
        }
        ring = (n3DNode*) vis->Find( "detectionrange" );

        r = detectionRange.GetF();
        ifnonzero( ring ){
            ring->Sxyz(r,1,r);
            ring->GetFullName( ringName, 1024 );
            numRings = int(r / ringDensity);
            for( int i = 1; i < numRings; i ++ ){
	char name[1024];
	sprintf( name, "%s%i", ringName, i );
	n3DNode* ring2 = (n3DNode*) ring->Clone( name );
	float t = 360.0f * i / (float)numRings / 25.0f;
	ring2->Ry( t );
            }
        }
    }
    entname = ent->fn;
}

nThreatAI::~nThreatAI(){
    if( targettedNode.isvalid() )
        targettedNode->Release();
}
//------------------------------------------------------------------------------
/**
 */

/**
  looks at platforms to see who is in range,  if in range then it attacks them
 **/
void nThreatAI::Tick( float time, float dtime ){
    nController::Tick( time, dtime );
    if( ent->Held() ) return;
    bool act = ( active.GetF() > .5f );
    if( act ) {
        if ( time - lastFired > cooldown.GetF() ) {  //we can fire
            vector3 p = pos.GetV();
            if( (target.isvalid() == false /*&& roll(.001)*/) || roll( .5 ) ) { //retarget
	nHeap<nEnt*> heap;
	heap.initialize( 32 );
	nRoot* pr = platformRoot.get();
	foreachchild( nEnt, platform, pr ) {
	//ifvalid( platformRoot ) for( nEnt* platform = (nEnt*) platformRoot->GetHead(); platform; platform = (nEnt*) platform->GetSucc() ){
	    vector3 tp = platform->GetVarV("pos");
	    float dist = (tp-p).len();
	    if( dist < detectionRange.GetF() ) //acquire target
	        heap.add( dist, platform) ;
	        //target.SetRef( platform->fn );
	}
	if ( heap.length() > 0 )
	    target.SetRef (heap.pop() );
            }
            if( target.isvalid() ){
	vector3 tp = targetPos.GetV();
	float dist = (tp - p).len();
	ifvalid( targettedNode ){
	    char fullname[1024];
	    targettedNode->GetFullName( fullname, 1024 );
	    targettedNode->SetActive(true);
	    targettedNode->Txyz(tp.x, tp.y, tp.z);
	}
	if( dist > detectionRange.GetF() ){ //lose sight of it
	    target.SetRef("none");
	}
	else if( dist < effectiveRange.GetF() ){ //attack it
	    nMessage* m = new nMessage();
	    m->time = time;
	    m->from = FullNameString();
	    m->to = target.GetRef()->FullNameString();
	    m->title = "attacking";
	    m->data = fires.GetS();
	    Send( m );
	    lastFired = time;
	}
	else { //can see but cant do anything
	}
            }
            else ifvalid( targettedNode )
	targettedNode->SetActive( false );
        }
    } 
}

void nThreatAI::Message( nMessage* message ){
    if( message->to == entname ){
        if( message->title == "command" ){
            ent->SetVarV("pos", message->v );
        }
    }
}
