#define N_IMPLEMENTS nThreatAllocator
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ai/nthreatallocator.h"
#include "map/ninfluencemap.h"

nNebulaScriptClass(nThreatAllocator, "nai");

//------------------------------------------------------------------------------
/**
*/
nThreatAllocator::nThreatAllocator():
    defensability 	( kernelServer, this ),
    threatRedundance 	( kernelServer, this ),
    threatViability 	( kernelServer, this ),
    extendedCoverage 	( kernelServer, this ),
    popupViability 	( kernelServer, this ),
    closeness		( kernelServer, this ),
    formwall		( kernelServer, this ),
    startPosFound	( false )
{
    nString imdir = "/game/im/";
    nString d = imdir + "defensability";

    defensability 	= imdir + "defensability";
    threatRedundance 	= imdir + "threatRedundance";
    threatViability 	= imdir + "threatViability";
    extendedCoverage 	= imdir + "extendedCoverage";
    popupViability 	= imdir + "popupViability";
    closeness		= imdir + "closeness";
    formwall		= imdir + "formwall";
    this->AddLogTag("nThreatAllocator");
    this->AddToTriggerList();
}

void nThreatAllocator::Initialize(){
    nAI::Initialize();
}

void nThreatAllocator::InitVars(){
    nAI::InitVars();
    offset.Connect( ent, "offset");
    offset.DefaultF( 20.0f );
}

void nThreatAllocator::FindStartPos() { 
    nRoot* r = kernelServer->Lookup("/game/ents/platform");
    if( r ) { 
        nEnt* plat = (nEnt*) (r->GetHead());
        startPos = plat->GetVarV("startpos");
        closeness->SetA( startPos.x );
        closeness->SetB( startPos.z );
        log("startpos:(%f,%f,%f)", startPos.x, startPos.y, startPos.z );
        n_printf("startpos:(%f,%f,%f)", startPos.x, startPos.y, startPos.z );
        startPosFound = true;
    }
}
nThreatAllocator::~nThreatAllocator(){
}
//------------------------------------------------------------------------------
/**
 */

void nThreatAllocator::Tick( float time, float dtime ){
    nAI::Tick( time, dtime );
    if( startPosFound == false )
        FindStartPos();
}

/**
  okay the idea here is to place all the threats
  to do this..
  we can use greedy algorithms combined with influence maps
  so we create a map to figure out where the best place to place our threats is, and then we place a threat there..and repeat..
  not terribly fast, but thats okay
 **/

void nThreatAllocator::PlaceThreats(){
    this->InitVars();
    if( startPosFound == false )
        FindStartPos();
    log ("PlaceThreats");
    float range;
    nEnt* threat;
    this->UnallocateThreats();
    while( this->FindBiggestThreat( threat, range ) ) { 		//foreach threat
        this->CalcDefensability( range );
        this->CalcThreatRedundance( range );

        threatViability->Set( *defensability );
        threatViability->Add( *threatRedundance );
        ifvalid( closeness ) {
            closeness->Calculate();
            threatViability->Add( *closeness );
        }
        ifvalid( formwall ) {
            formwall->Calculate();
            threatViability->Add( *formwall );
        }

        vector2 bestThreatPlacement = threatViability->GetMaxPoint(); 	//get the most valuable point on this map
        log("bestThreatPlacement:%f,%f", bestThreatPlacement.x, bestThreatPlacement.y );
        float threatViabilityValue = threatViability->GetNormVal( bestThreatPlacement );
        log("threatViabilityValue:%f", threatViabilityValue );

        /*
           this->CalcExtendedCoverage( range );
           this->CalcPopupViability( range );
           vector2 bestPopupPlacement = popupViability->GetMaxPoint();
           log("bestPopupPlacement:%f,%f", bestPopupPlacement.x, bestPopupPlacement.y );
           float popupViabilityValue = popupViability->GetVal( bestPopupPlacement );
           log("popupViabilityValue:%f", popupViabilityValue );

           float scaledPopupValue = popupViabilityValue * threatVsPopupCoef;
           log("scaledPopupValue:%f", scaledPopupValue );

           if( scaledPopupValue > threatViabilityValue ) 		//place it as a popup
           this->Allocate( threat, bestPopupPlacement );
           else
         */
        this->Allocate( threat, bestThreatPlacement ); 		//place it as a threat
    }
    log("/PlaceThreats");
}
void nThreatAllocator::UnallocateThreats(){
    nRoot* threats = kernelServer->Lookup("/game/ents/threat");
    foreachchild( nEnt, e, threats ) {
        e->CreateVar("allocated")->SetI( 0 );
        e->SetVarV("pos", vector3(-1000.0f,0.0f,-1000.0f) );
        e->SetVarF("active", 1.0f );
    }
}
void nThreatAllocator::Allocate( nEnt* threat, vector2 location ) {
    float t = randF( 0, N_PI * 2.0f );
    float r = randF( 0, offset.GetF() ) ;
    vector2 o ( r * cos( t ), r * sin( t ) );
    location += o;
    threat->SetVarV("pos", vector3( location.x, 0, location.y ) );
    threat->SetVarI("allocated", 1 );
}
bool nThreatAllocator::FindBiggestThreat( nEnt*& threat, float& range ){
    nRoot* threats = kernelServer->Lookup("/game/ents/threat");
    bool found = false;
    foreachchild( nEnt, e, threats ) {
        if( e->GetVarI("allocated") == 1 )  //skip threats that have already been allocated
            continue;
        float rad = e->GetVarF("effectiverange");
        if( found == false || rad > range ) {
            range = rad;
            threat = e;
            found = true;
        }
    }
    return found;
}
//sum (target.value * closeness (target, myrange))
void nThreatAllocator::CalcDefensability( float range ){	
    defensability->SetA( range );
    defensability->Calculate();
}
//sum (closeness (threat, myrange + threat.range))
void nThreatAllocator::CalcThreatRedundance( float range ){	
    threatRedundance->SetA( range );
    threatRedundance->Calculate( );
}
//sum (closeness (threat, myrange + threat.range))
void nThreatAllocator::CalcExtendedCoverage ( float range ){	
    nString cmd;
    cmd.PrintF("InfluenceMap::calc extendedCoverage {[sum within threat {#effectiverange + %f}] - [sum within threat #effectiverange]}", range);
    RUNSCRIPTD( cmd.Get() );

}
//sum (myrange, extendedCoverage)
void nThreatAllocator::CalcPopupViability ( float range ){	
    nString cmd;
    cmd.PrintF("InfluenceMap::calc popupViability {[sumRadiusIM %f extendedCoverage]}", range );
    RUNSCRIPTD( cmd.Get() );
}
