#define N_IMPLEMENTS nAutoSelector
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "aspect/nautoselector.h"

nNebulaScriptClass(nAutoSelector, "naspect");

//------------------------------------------------------------------------------
/**
*/
nAutoSelector::nAutoSelector():
    potentials( kernelServer, this )
{
    potentials = "/game/ents/cockroach";
}

void nAutoSelector::Initialize(){
    nAspect::Initialize();
    this->AddToDisplayList();
}

void nAutoSelector::InitVars(){
    nAspect::InitVars();
    target.Connect( ent, "target" );
    target.DefaultRef("");
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    selectionCoeffs.Connect( ent, "selectioncoeffs" );
    selectionCoeffs.DefaultV( 
            10.0f, //points lost for being too side
            10.0f, //how do we deal with differing distance measures?? well we can normalize according to distance..
            0.01f); //points lost for distance
}

nAutoSelector::~nAutoSelector(){
}
//------------------------------------------------------------------------------
/**
*/

/**
  Automatically updates your selected target
  goes through a list of possible targets
  the one selected is chosen based upon its location relative to the orientation of the ent
  crossValues contains the relative offsets - .z = distance in front of target
  distance.x / y are relative measures of how far to the side the target is
  coeffs contains the weights given to each of these in determining who too select
  the ent with lowest value is selected
*/
void nAutoSelector::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    nRoot* node;
    classVar( nent, entClass );

    vector3 p = pos.GetV();
    quaternion r = rot.GetQ();
    vector3 right	( 1,0,0 );
    vector3 up		( 0,1,0 );
    vector3 forward	( 0,0,-1 );
    right 	= r.rotate( right );
    up 	= r.rotate( up );
    forward 	= r.rotate( forward );
    //this->DrawLine( p, p + right * 100, green );
    //this->DrawLine( p, p + up * 100, red );
    this->DrawLine( p, p + forward * 100, blue );
    
    vector3 crossValues;
    vector3 coeffs = selectionCoeffs.GetV();
    //i think that might be good
    nEnt* bestFound = 0;

    float minValue = -1;
    for( node = potentials->GetHead(); node; node = node->GetSucc() ) {
        if( node->IsA( entClass ) ) {
            nEnt* e = ( nEnt* ) node;
            vector3 tp = e->GetVarV("pos");
            crossValues.x = (tp-p) % right;
            crossValues.y = (tp-p) % up;
            crossValues.z = (tp-p) % forward;
            if( crossValues.z < 0 ) continue;

            float l = (tp - p).len();
            crossValues.x /= l;
            crossValues.y /= l;

            float x = crossValues.x;
            float y = crossValues.y;
            float z = crossValues.z;

            //this->DrawLine( p, p + right * 	crossValues.x, 		white );
            //this->DrawLine( p, p + up *  	crossValues.y * 100, 	white );
            //this->DrawLine( p, p + forward * 	crossValues.z * 100, 	white );

            vector3 vals(crossValues.x * coeffs.x, crossValues.y * coeffs.y, crossValues.z * coeffs.z );
            float value = vals.len();
            //n_printf("value(%s) = %f\n", e->fn, value );
            if( minValue < 0 || value < minValue ) {
	minValue = value;
	bestFound = e;
            }
        }
    }
    if( bestFound ) {
        target.SetRef( bestFound->fn );
        this->DrawLine( p, bestFound->GetVarV("pos"), white );
    }
}
