#define N_IMPLEMENTS nCameraController
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "aspect/ncameracontroller.h"
#include "physics/nphysics.h"
#include "physics/nstabilizer.h"
#include "ui/ninterfaceserver.h"
#include "enkiduo/ntriggerserver.h"

nNebulaScriptClass(nCameraController, "nfsm");

//------------------------------------------------------------------------------
nCameraController::nCameraController():
    physics                             ( kernelServer, this ),
    ref_input                           ( kernelServer, this ),
    ref_game                            ( kernelServer, this ),
    ref_stabilizer                      ( kernelServer, this ),
    ref_interface                       ( kernelServer, this ),
    ref_trigger                         ( kernelServer, this )
{
    actualTime0 = 0;
}
//------------------------------------------------------------------------------
void nCameraController::Initialize(){
    nFSM::Initialize();
    ref_input                           = "/sys/servers/input";
    ref_game                            = "/sys/servers/game";
    ref_interface                       = "/ui/interface";
    ref_trigger                         = "/sys/servers/trigger";

    stateNames.Append("Free");
    stateNames.Append("FirstPerson");
    stateNames.Append("ThirdPerson");
    stateNames.Append("Homeworld");
    stateNames.Append("TetheredWatch");
    stateNames.Append("Pan");
}
//------------------------------------------------------------------------------
void nCameraController::InitVars(){
    nFSM::InitVars();
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    ptightness.Connect( ent, "ptightness" );
    ptightness.DefaultF(	1.0f );
    rtightness.Connect( ent, "rtightness" );
    rtightness.DefaultF(	1.0f );
    vel.Connect( ent, "vel" );
    rvel.Connect( ent, "rvel" );

    attachedTo.Connect                  ( ent, "attachedto" );
    attachedTo.DefaultRef               ("");
    poffset.Connect                     ( attachedTo, "cameraoffset" );
    roffset.Connect                     ( attachedTo, "cameraroffset" );
    attachedPos.Connect                 ( attachedTo, "pos" );
    attachedRot.Connect                 ( attachedTo, "rot" );

    force.Connect                       ( ent, "force" );
    torque.Connect                      ( ent, "alignedtorque" );

    realignDelay.Connect                ( ent, "realignDelay" );
    realignDelay.DefaultF               (1.0f);
    lastMovementF = 0.0f;
    lastMovementT = 0.0f;

    watching.Connect                    ( ent, "watching");
    watching.DefaultRef                 ("");
    watchingPos.Connect                 ( watching, "pos" );

    mass.Connect                        ( ent, "mass" );
    inertia.Connect                     ( ent, "inertia" );
    relativeMovementMassCoef.Connect    ( ent, "relativeMovementMassCoef" );
    relativeMovementMassCoef.DefaultF   ( 0.1f ); //need to reduce the mass since there is infinite friction (we are only moving for 1 frame)
    relativeMovementInertiaCoef.Connect ( ent, "relativeMovementInertiaCoef" );
    relativeMovementInertiaCoef.DefaultF( 0.2f ); //need to reduce the inertia since there is infinite friction (we are only moving for 1 frame)

    ref_stabilizer                      = ent->ChildPath("stabilizer");
    physics                             = ent->ChildPath("physics");

    panTime.Connect                     ( ent, "panTime");
    panTime.DefaultF                    ( 2.0f );
    panTimeRemaining.Connect            ( ent, "panTimeRemaining" );
    panTimeRemaining.DefaultF           ( 0.0f );
    panDestination.Connect              ( ent, "panDestination");
    panDestination.DefaultV             ( 0,0,0 );

    windowSettings.Connect( ent, "windowSettings");
    windowSettings.DefaultRef("/sys/share/windowsettings");
    aspectRatio.Connect( windowSettings, "aspectRatio" );
}
//------------------------------------------------------------------------------
nCameraController::~nCameraController(){
}
//------------------------------------------------------------------------------
bool nCameraController::States( StateMachineEvent event, uint state, float time, float dtime ){
    nAspect::Tick( time, dtime );

    float actualTime = ref_trigger->GetActualTime();
    dtime = actualTime - this->actualTime0;
    if ( this->actualTime0 == 0 ) {
        this->actualTime0 = actualTime; 
        return true;
    } else
        this->actualTime0 = actualTime; 

    if(force.isdirty())                 lastMovementF = time;
    if(torque.isdirty())                lastMovementT = time;

    //-----------------------------------------------------------------------------/
    float rtime = (float) kernelServer->ts->GetFrameTime();
    n_assert( physics.isvalid() && "camera has no physics");

    BeginStateMachine
        State(Free)
            OnTick
                if( ref_game->GetButton("forward") )      physics->ApplyAlignedForce(  vector3( 0.000, 0.000,-dtime) );
                if( ref_game->GetButton("backward") )     physics->ApplyAlignedForce(  vector3( 0.000, 0.000, dtime) );
                if( ref_game->GetButton("up") )           physics->ApplyAlignedForce(  vector3( 0.000, dtime, 0.000) );
                if( ref_game->GetButton("down") )         physics->ApplyAlignedForce(  vector3( 0.000,-dtime, 0.000) );
                if( ref_game->GetButton("left") )         physics->ApplyAlignedForce(  vector3(-dtime, 0.000, 0.000) );
                if( ref_game->GetButton("right") )        physics->ApplyAlignedForce(  vector3( dtime, 0.000, 0.000) );

                if( ref_game->GetButton("rollleft") )     physics->ApplyAlignedTorque( vector3( 0.000, 0.000, dtime) );
                if( ref_game->GetButton("rollright") )    physics->ApplyAlignedTorque( vector3( 0.000, 0.000,-dtime) );
                if( ref_game->GetButton("yawleft") )      physics->ApplyAlignedTorque( vector3( 0.000, dtime, 0.000) );
                if( ref_game->GetButton("yawright") )     physics->ApplyAlignedTorque( vector3( 0.000,-dtime, 0.000) );
                if( ref_game->GetButton("pitchup") )      physics->ApplyAlignedTorque( vector3(-dtime, 0.000, 0.000) );
                if( ref_game->GetButton("pitchdown") )    physics->ApplyAlignedTorque( vector3( dtime, 0.000, 0.000) );
        //-----------------------------------------------------------------------------//
        State(FirstPerson)              
            OnTick
                if( ref_game->GetButton("rollleft") )     physics->ApplyAlignedTorque( vector3( 0.000, 0.000, dtime) );
                if( ref_game->GetButton("rollright") )    physics->ApplyAlignedTorque( vector3( 0.000, 0.000,-dtime) );
                if( ref_game->GetButton("yawleft") )      physics->ApplyAlignedTorque( vector3( 0.000, dtime, 0.000) );
                if( ref_game->GetButton("yawright") )     physics->ApplyAlignedTorque( vector3( 0.000,-dtime, 0.000) );
                if( ref_game->GetButton("pitchup") )      physics->ApplyAlignedTorque( vector3(-dtime, 0.000, 0.000) );
                if( ref_game->GetButton("pitchdown") )    physics->ApplyAlignedTorque( vector3( dtime, 0.000, 0.000) );
        //-----------------------------------------------------------------------------//
        State(ThirdPerson)              //my super smart idea is this -> just apply the force like normal, manually tick the physics, then check the diff in positions / rotations
            OnTick
                if( attachedTo.isvalid() == false ) {
                    this->SetState(Free);
                    return true;
                }
                vector3 force;
                vector3 torque;
                if( ref_game->GetButton("forward") )      force += vector3( 0.000, 0.000,-dtime);
                if( ref_game->GetButton("backward") )     force += vector3( 0.000, 0.000, dtime);
                if( ref_game->GetButton("up") )           force += vector3( 0.000, dtime, 0.000);
                if( ref_game->GetButton("down") )         force += vector3( 0.000,-dtime, 0.000);
                if( ref_game->GetButton("left") )         force += vector3(-dtime, 0.000, 0.000);
                if( ref_game->GetButton("right") )        force += vector3( dtime, 0.000, 0.000);

                //if( ref_game->GetButton("rollleft") )     t += vector3( 0.000, 0.000, dtime); //dont save these
                //if( ref_game->GetButton("rollright") )    t += vector3( 0.000, 0.000,-dtime);
                if( ref_game->GetButton("rollleft") )     physics->ApplyAlignedTorque( vector3( 0.000, 0.000, dtime) );
                if( ref_game->GetButton("rollright") )    physics->ApplyAlignedTorque( vector3( 0.000, 0.000,-dtime) );
                if( ref_game->GetButton("yawleft") )      torque += vector3( 0.000, dtime, 0.000);
                if( ref_game->GetButton("yawright") )     torque += vector3( 0.000,-dtime, 0.000);
                if( ref_game->GetButton("pitchup") )      torque += vector3(-dtime, 0.000, 0.000);
                if( ref_game->GetButton("pitchdown") )    torque += vector3( dtime, 0.000, 0.000);
                if( force.len() > 0 || torque.len() > 0 ) {
                    //n_printf("f:%s t:%s\n", str(force), str(torque));
                    force = rot.GetQ().rotate(force);                                    //align the force along our direction
                    //physics->Tick(0,dtime);
                    vector3 t           = pos.GetV();
                    vector3 t0          = t;
                    vector3 dt          (0,0,0);
                    vector3 ddt         = force;
                    quaternion q        = rot.GetQ();
                    quaternion q0       = q;
                    quaternion dq       (0,0,0,0);
                    vector3 ddq         (0,0,0);
                    vector3 addq        = torque;
                    //n_printf("t:%s dt:%s ddt:%s\n", str(t), str(dt), str(ddt));
                    //n_printf("q:%s dq:%s ddq:%s addq:%s\n", str(q), str(dq), str(ddq), str(addq));
                    float friction      = 0;
                    float rfriction     = 0;
                    //n_printf("m:%f i:%f\n", relativeMovementMass.GetF(), relativeMovementInertia.GetF() );

                    physics->DoMath( t, dt, ddt, q, dq, ddq, addq, mass.GetF() * relativeMovementMassCoef.GetF(), inertia.GetF() * relativeMovementInertiaCoef.GetF(), friction, rfriction, dtime );
                    //n_printf("pos[%s->%s] rot[%s->%s]\n", str(t0), str(t), str(q0), str(q));

                    quaternion aq       = attachedRot.GetQ();
                    vector3 oldforward(0,0,-1);
                    oldforward = q0.rotate(oldforward);
                    aq.invert();

                    vector3 forward(0,0,-1);
                    forward = q.rotate(forward);
                    //n_printf("len:%f,%f\n", oldforward.len(), forward.len());

                    dt                  = t - t0;
                    vector3 doffset     = aq.rotate( dt );                                       //rotate our dp by the orientation of the entity we are watching
                    //n_printf("doffset:%s\n", str(doffset));

                    quaternion ro       = aq * q;
                    vector3 v(0,0,-1);
                    v = ro.rotate(v);
                    matrix44 m;
                    m.lookat(v, vector3(0,1,0));
                    ro = m.get_quaternion();

                    poffset.SetV        ( poffset.GetV() + doffset );
                    roffset.SetQ        ( ro );
                }
                float delay                         = realignDelay.GetF();
                float sinceLastCoefF                = ((time - lastMovementF) - delay ) / delay;
                float sinceLastCoefT                = ((time - lastMovementT) - delay ) / delay;
                //sinceLastCoefF                      = n_clamp(sinceLastCoefF, 0.0f, 1.0f);
                //sinceLastCoefT                      = n_clamp(sinceLastCoefT, 0.0f, 1.0f);
                sinceLastCoefF                      = 1.0f;
                sinceLastCoefT                      = 1.0f;
                //n_printf("%f %f -> %f,%f\n", lastMovementF, lastMovementT, sinceLastCoefF, sinceLastCoefT);
                poffset.DefaultV                    ( 0.0f, 0.0f, -1.0f );
                roffset.DefaultQ                    ( 0.0f, 0.0f, 0.0f, 1.0f );
                vector3 p                       = pos.GetV();
                quaternion r                    = rot.GetQ();
                vector3 po                      = poffset.GetV();
                quaternion ro                   = roffset.GetQ();
                float pt                        = ptightness.GetF();
                float rt                        = rtightness.GetF();
                pt                             *= sinceLastCoefF;
                rt                             *= sinceLastCoefT;
                vector3 ap                      = attachedPos.GetV();
                quaternion ar                   = attachedRot.GetQ();
                vector3 np                      = ((ap + ar.rotate(po)) - p) * pt + p;
                pos.SetV( np );

                //ipol my rotation towards its standard orientation
                quaternion nr;
                if ( watching.isvalid() == false ) {
                    //n_printf("slerping to : %s (%s*%s)\n", str(ar*ro), str(ar), str(ro));
                    nr.slerp( r, ar * ro , rt ); 
                } else {
                    quaternion q;
                    vector3 d = watchingPos.GetV() - p;
                    d.norm();
                    q.set_from_axes(vector3(0,0,-1), d);
                    matrix44 m;
                    m.lookat(d, vector3(0,1,0));
                    q = m.get_quaternion();
                    nr.slerp( r, q, rt ); 
                }
                rot.SetQ( nr );
        State(Pan)
            OnEnter
                panTimeRemaining.SetF( panTime.GetF() );
            OnTick
                if( ref_game->GetButton("forward") )      this->SetState(Free);
                if( ref_game->GetButton("backward") )     this->SetState(Free);
                if( ref_game->GetButton("up") )           this->SetState(Free);
                if( ref_game->GetButton("down") )         this->SetState(Free);
                if( ref_game->GetButton("left") )         this->SetState(Free);
                if( ref_game->GetButton("right") )        this->SetState(Free);

                if( ref_game->GetButton("rollleft") )     this->SetState(Free);
                if( ref_game->GetButton("rollright") )    this->SetState(Free);
                if( ref_game->GetButton("yawleft") )      this->SetState(Free);
                if( ref_game->GetButton("yawright") )     this->SetState(Free);
                if( ref_game->GetButton("pitchup") )      this->SetState(Free);
                if( ref_game->GetButton("pitchdown") )    this->SetState(Free);

                float timeRemaining = panTimeRemaining.GetF();
                float percentageElapsed = dtime / timeRemaining;
                percentageElapsed = n_clamp(percentageElapsed, 0.0f, 1.0f);
                float percentageRemaining = 1.0f - percentageElapsed;
                vector3 p = pos.GetV();
                vector3 dp = panDestination.GetV();
                //n_printf("percentageRemaining:%f, percentageElapsed:%f\n", percentageRemaining, percentageElapsed);
                pos.SetV( p * percentageRemaining + dp * percentageElapsed);
                timeRemaining -= dtime;
                panTimeRemaining.SetF(timeRemaining);
                if ( timeRemaining <= 0.0f ) {
                    this->SetState(Free);
                }
    EndStateMachine
}
/**
 * okay so we want to move the camera to a particular point in space
 */
void nCameraController::PanToPoint(vector3 point) {
    //n_printf("nCameraController::PanToPoint(%s)\n", str(point));
    panDestination.SetV( point );
    this->SetState(Pan);
}
/**
 * pan the camera to view a group of entities
 */
void nCameraController::PanToEnts( EntArray ents ) {
    //n_printf("nCameraController::PanToGroup()\n");
    nEnt* e;
    vector3 p;
    quaternion q = rot.GetQ();
    nArray<vector3> positions;
    foreach(e, ents){
        //n_printf("ent:%s\n", e->fn);
        positions.Append(e->GetVarV("pos"));
    }

    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();

    //find the distance from the unit
    vector3 centerForward = ref_interface->ScreenToWorld(vector2(.5f,.5f));
    //n_printf("centerForward:%s\n", str(centerForward) );
    float d = (centerForward - pos.GetV()).len();
    //n_printf("d:%f\n", d );
    if (d > 1000)
        d = 25.0f;

    p = positions[0] - q.rotate(vector3(0,0,-d));
    matrix44 m;
    for(int i = 0; i < 1000; i ++){
        m.set(q);
        m.set_translation(p);
        m.invert_simple();
        int left, right, above, below, behind;
        left = right = above = below = behind = 0;
        for( int i = 0; i < positions; i ++ ) {
            //n_printf("positions:%s\n", str(positions[i]));
            vector3 screenPos = m * positions[i];
            if (screenPos.z > 0) behind++;
            else {
                screenPos.norm();
                //n_printf("screenpos:%s\n", str(screenPos));
                screenPos.x *= 3.0f; //want them in the center
                screenPos.y *= 3.0f; //want them in the center
                if ( screenPos.x > aspect ) right++;
                if ( screenPos.x < -aspect ) left++;
                if ( screenPos.y > 1.0 )  above++;
                if ( screenPos.y <-1.0 )  below++;
            }
        }
        //n_printf("left:%i right:%i above:%i below:%i behind:%i\n", left, right, above, below, behind);
        if ( left + right + above + below + behind == 0 )
            break;
        else {
            while( left > 0 && right > 0 ) {
                left--;
                right--;
                behind++;
            }
            while( above > 0 && below > 0 ) {
                above--;
                below--;
                behind++;
            }
            vector3 v(right-left, above-below, behind);
            const float moveDist = 2.0f;
            v *= moveDist;
            //n_printf("v:%s\n", str(v));
            p += q.rotate(v);
        }
    }
    this->PanToPoint( p );
}
void nCameraController::PanToEnts( const char* entPaths ){
    //n_printf("nCameraController::PanToGroup\n");
    char* entPathsDup = n_strdup(entPaths);
    const char* path;
    EntArray paths;
    while( path = tok(entPathsDup, " ")){
        //n_printf("path:%s\n");
        nRoot* r = kernelServer->Lookup(path);
        classVar(nent, entClass);
        if (r->IsA(entClass)){
            paths.Append((nEnt*) r);
        } else {
            n_printf("trying to pan to non ent:%s\n", path);
        }
    }
    this->PanToEnts(paths);
}
