#define N_IMPLEMENTS nLineAspect
//------------------------------------------------------------------------------
//  (C) 2005	Juan
//------------------------------------------------------------------------------
#include "aspect/nlineaspect.h"
#include "ui/ntextdisplayer.h"

nNebulaScriptClass(nLineAspect, "nfsm");

//------------------------------------------------------------------------------
//nLineAspect::nLineAspect() {
nLineAspect::nLineAspect():ref_text(kernelServer,this) {
    ref_text="/lib/node/text/text";
}
//------------------------------------------------------------------------------
void nLineAspect::Initialize(){
    nFSM::Initialize();

    this->createVisNode(this->fn);

    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );

    // creates a variable on the entity
    pointA.Connect( ent, "pointA");
    pointA.DefaultV(0,0,0);
    pointB.Connect( ent, "pointB");
    pointB.DefaultV(50000,0,50000);
    targetA.Connect( ent, "targetA");
    targetB.Connect( ent, "targetB");
    // Set default references to blank
    targetA.DefaultRef("");
    targetB.DefaultRef("");
    targetAPos.Connect( targetA, "pos");
    targetBPos.Connect( targetB, "pos");

    range.Connect( ent, "range" );
    range.DefaultF( 0.0f );

    // Initialize camera
    camera.Connect( ent, "camera" );
    camera.DefaultRef("/game/camera");
    cameraPos.Connect( camera, "pos" );
    cameraRot.Connect( camera, "rot" );


}
//------------------------------------------------------------------------------
void nLineAspect::InitVars(){
    nFSM::InitVars();
}
//------------------------------------------------------------------------------
nLineAspect::~nLineAspect(){
}
//------------------------------------------------------------------------------
///called all the time
void nLineAspect::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    vector3 pos0, pos1;
    if( targetA.isvalid() )     pos0 = targetAPos.GetV();
    else                        pos0 = pointA.GetV();
    if( targetB.isvalid() )     pos1 = targetBPos.GetV();
    else                        pos1 = pointB.GetV();

    vector3 d = pos1-pos0;
    d.y+=1.;
    float dist = d.len();
    d.norm();

    ///calculate the angle needed to point me towards target
    quaternion q;
    q.set_from_axes(vector3(0,0,-1),-d);

    pos.SetV(pos0);
    rot.SetQ(q);
    range.SetF(dist);
}

bool nLineAspect::Attach(nSceneGraph2* sceneGraph)
{
    float textSize=20;
    bool draw=false;

    bool compute=false;
    // Do the camera math to determine screen position
    matrix44 vrw                = camera.GetRef()->GetM();
    matrix44 vrw_inverted       = vrw;
    vrw_inverted.invert_simple();

    vector3 pos0;
    vector3 pos1;
    // Draw text only if targets are valid
    if( targetA.isvalid() && targetB.isvalid())
    {
        pos0 = targetAPos.GetV();
        pos1 = targetBPos.GetV();
        compute = true;
    }
    else
    {
        pos0 = pointA.GetV();
        pos1 = pointB.GetV();
    }

    vector3 textPos = vrw_inverted*pos0;

    // Normalize text coordinates
    textPos.norm();

    // Only draw text if line exists
    if ( compute )
    {
        // Draw text if line is in viewing window
        vector2 v(textPos.x, textPos.y);

        if ( v.len() < -textPos.z )
            draw=true;

        if ( draw )
        {
            vector2 v2(textPos.x, textPos.z);

            if ( v2.len() > -textPos.y )
                draw=true;
            else
                draw=false;
        }
    }


    if ( textPos.y <= -0.9 )
        textSize+=4;
    else if ( textPos.y < -0.6 )
        textSize+=2;

    if (textPos.z <= -0.9)
        textSize+=4;
    else if (textPos.z <= -0.6)
        textSize+=2;


    nString txt = "";
    txt.PrintF("%.1f", (pos1-pos0).len());

    // Draw only if target is valid and the object is in the window
    if ( draw )
    {
        if(ref_text.isvalid())
        {
            ref_text->SetString(txt.get(), ALIGN_CENTER);
            ref_text->SetOffset(vector3(textPos.x*textSize, textPos.y*textSize+1, -textSize));
            ref_text->Attach(sceneGraph);
        } else {
            n_printf("nLineAspect(%s).ref_text is invalid\n", this->fn);
        }
    }

    this->sceneGraph = sceneGraph;
    this->Process(EVENT_ATTACH, 0, 0);

    return true;
}
