#define N_IMPLEMENTS nPythonAspect
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
//#include "python/npythonserver.h"
#include "aspect/npythonaspect.h"

nNebulaScriptClass(nPythonAspect, "naspect");

PyObject* pythoncmd_AttachPythonAspect( PyObject * /*self*/, PyObject *args);
void createnPythonAspectModule(nPythonServer* pythonServer);

nPythonAspect* pythonAspectWaiting = 0;
//------------------------------------------------------------------------------
nPythonAspect::nPythonAspect():
    pythonServer( kernelServer, this )
{
}
//-----------------------------------------------------------------------------//
void nPythonAspect::Initialize(){
    nAspect::Initialize();
    pythonServer = "/sys/servers/python";
    tickFunction = 0;
}
//-----------------------------------------------------------------------------//
void nPythonAspect::InitVars(){
    nAspect::InitVars();
    pythonModule.Connect( ent, "pythonmodule");
    pythonModule.DefaultS("none");
    pythonClass.Connect ( ent, "pythonclass");
    pythonClass.DefaultS("none");

    static bool moduleInitialized = false;
    if ( moduleInitialized == false ) {
        createnPythonAspectModule(pythonServer.get());
        moduleInitialized = true;
    }
}
//-----------------------------------------------------------------------------//
nPythonAspect::~nPythonAspect(){
}
//-----------------------------------------------------------------------------//
void nPythonAspect::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    if( pythonClass.isdirty() || pythonModule.isdirty() ) {
        pythonModule.isdirty();                                                 //avoid short circuiting whe - make sure both are set as undirty
        n_assert( pythonAspectWaiting == 0 );
        pythonAspectWaiting = this;
        //n_printf("nPythonAspect attaching to %s.%s\n", pythonClass.GetS().Get(), pythonModule.GetS().Get() );
        nString cmd;
        cmd.PrintF("attachPythonAspect('%s','%s')", pythonModule.GetS().Get(), pythonClass.GetS().Get() );
        //n_printf("nPythonAspect calling: %s\n", cmd.Get() );
        const char* err;
        pythonServer->Run(cmd.Get(), err );
        if( err && err[0] != '\0' )
            n_printf("Error in npythonaspect: %s\n", err );
    }
    if( tickFunction ) {
        kernelServer->PushCwd( ent );
        PyObject* arglist = Py_BuildValue("(f,f)", time, dtime);
        PyObject* result = PyEval_CallObject(this->tickFunction, arglist);
        Py_XDECREF(arglist);
        Py_XDECREF(result);
        kernelServer->PopCwd();
    } else
        n_printf("warning nPythonAspect(%s) has no tick function\n", this->fn );
}

void nPythonAspect::setTickFunction( PyObject* inst, PyObject* func ) {
    n_assert( this && inst && func );
    //n_printf("nPythonAspect::setTickFunction(%p,%p)\n", inst,func );
    this->pyInst                        = inst;
    this->tickFunction                  = func;
}
//-----------------------------------------------------------------------------//
static PyMethodDef nPythonAspectMethods[] = {
    {"attachFunc",pythoncmd_AttachPythonAspect,METH_VARARGS,NULL},
    {NULL,NULL,0,NULL}    /* Sentinel */
};
//-----------------------------------------------------------------------------//
/**
 * we need to call this to create our module so the other can import it
**/
//-----------------------------------------------------------------------------//
void createnPythonAspectModule(nPythonServer* pythonServer){
    printf("createnPythonAspectModule\n");
    Py_InitModule("npythonaspect",  nPythonAspectMethods);
    const char* err;
    nString cmd;
    cmd.PrintF("from common.pythonaspect import *");
    pythonServer->Run(cmd.Get(), err );
    if( err && err[0] != '\0' )
        n_printf("Error in importing pythonaspect: %s\n", err );
}
//-----------------------------------------------------------------------------//
/**
 * okay we will use a static function to get our pointer to the python object what we need
 * to do this we register a global variable with a pointer to the c class who wants the python object
 * then we call a python function to instantiate the python object, and call this static function
 **/
//-----------------------------------------------------------------------------//
PyObject* pythoncmd_AttachPythonAspect( PyObject * self, PyObject *args) {
    PyObject* newInst;
    PyObject* newTickFunction;
    //printf("pythoncmd_AttachPythonAspect(%p,%p)\n", self, args);
    // Extract two strings from the Python args object
    // Also, ":<desc>" for a descriptive error if an exception is raised.
    if(PyArg_ParseTuple(args, "OO:attachPythonAspect", &newInst, &newTickFunction)) {
        //printf("nPythonAspect::pythoncmd_AttachPythonAspect(%p)\n", newTickFunction);
        if (!PyCallable_Check(newTickFunction)) {
            //printf("error not callable\n");
        }
        else {
            n_assert(pythonAspectWaiting && "Trying to set a pythonAspect but no one is waiting to receive it");
            Py_INCREF(newTickFunction);
            pythonAspectWaiting->setTickFunction( newInst, newTickFunction );
            pythonAspectWaiting = 0;
        }
    }
    Py_INCREF(Py_None);
    return Py_None;
}
