#define N_IMPLEMENTS nSide
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "misc/nquickwatch.h"
#include "aspect/nside.h"
#include "ui/ntextdisplayer.h"

nNebulaScriptClass(nSide, "naspect");

//------------------------------------------------------------------------------
/**
*/
nSide::nSide(){
}

void nSide::Initialize(){
    nAspect::Initialize();
    this->AddToDisplayList();
}

void nSide::InitVars(){
    nAspect::InitVars();
    playerSide.Connect( ent, "playerside");
    playerSide.DefaultI( 0 );
    dtimesum = 0;
}

nSide::~nSide(){
}
//------------------------------------------------------------------------------
/**
*/

void nSide::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    foreachp( Resource*, r, resources ) {
        //printf("side(%s) checking resource %s\n", str(this), r->name.Get() );
        if( r->amount0 > 0 && dtime > 0 ){ 
            r->damount.SetF( (r->amount.GetF() - r->amount0) / (dtime / 10.0f) );
        }
        r->amount0 = r->amount.GetF();
        this->time = time;
        this->dtimesum += dtime;
    }
}

void nSide::AddUnit( nEnt* e ) {
    //n_printf("side:%s adding %s to its ent list\n", this->fn, e->fn );
    units.Push() = e;
}
void nSide::RemoveUnit( nEnt* e ) {
    //n_printf("side:%s removing %s from its ent list\n", this->fn, e->fn );
    for( int i = 0; i < units; i ++ ) {
        if( units[i] == e ) {
            //n_printf("found in slot: %i\n", i );
            units[i] = 0;
            units.Erase( i );
            //n_printf("done erasing\n");
        }
    }
}
////////////////////////////////////////////////////////////////////////////////
void nSide::SetRelationship( const char* side, const char* relationship) {
    n_printf("%s->SetRelationship(%s,%s)\n", this->fn, side, relationship );
    nString sidePath;
    sidePath.PrintF("/game/sides/%s/side", side );
    nSide* s = (nSide*) kernelServer->Lookup(sidePath.Get() );
    if( s == 0 ) {
        n_printf("trying to set relationship (%s,%s) to a non existant side\n", side, relationship );
        return;
    }
    if( strcmp( relationship, "hostile" ) == 0 ) {
        hostiles.Append( s );
    } 
    else
        n_printf("unknown relationship:%s\n", relationship );
}
////////////////////////////////////////////////////////////////////////////////
void nSide::Display( nGfxServer* gfx ){
    nAspect::Display( gfx );
    //ifvalid( playerSide ) {
        //if( playerSide.GetI() == 1 )  {
            foreachp( Resource*, r, resources ) {
                nString string;
                //string.PrintF("%3.0f(%+.1f)", r->amount.GetF(), r->damount.GetF() );
                ifvalid( r->td ) {
                    string.PrintF("%i", (int) r->amount.GetF() );
                    r->td->SetString( string.Get() );
                }
                ifvalid( r->tdDelta ) {
                    string.PrintF("%+i", (int) r->damount.GetF() );
                    r->tdDelta->SetString( string.Get() );
                }
            }
        //}
    //}
}
//////////////////////////////////////////////////////////////////////////////// 
void nSide::AddResource( const char* name ) {
    //n_printf("nSide(%s).AddResource(%s)\n", str(this), name );
    Resource* r                         = &resources.Push();
    r->name                             = name;
    r->amount.Connect                   ( ent, name );
    r->amount.DefaultF                  ( 0.0f );
    r->amount0                          = -1.0f;
    r->damount.Connect                  ( ent, (nString(name) + "d").Get() );
    r->damount.DefaultF                 ( 0.0f );

    nString path;
    r->tdName.initialize                ( kernelServer, this );
    path.PrintF                         ("/game/vis/ui/%s/%sname", this->GetParent()->GetName(), name );
    r->tdName                           = path.Get();

    r->td.initialize                    ( kernelServer, this );
    path.PrintF                         ("/game/vis/ui/%s/%s", this->GetParent()->GetName(), name );
    r->td                               = path.Get();

    r->tdDelta.initialize               ( kernelServer, this );
    path.PrintF                         ("/game/vis/ui/%s/d%s", this->GetParent()->GetName(), name );
    r->tdDelta                          = path.Get();
}
//-----------------------------------------------------------------------------//
const EntArray*  nSide::GetHostileUnits(){
    nSide* hostileSide;
    hostileUnits.Clear();
    nEnt* hostile;
    foreach( hostileSide, hostiles ) {
        const EntArray& hostiles = *hostileSide->GetUnits();
        foreach( hostile, hostiles )
            hostileUnits.Append( hostile );
    }
    return &hostileUnits;
}
//-----------------------------------------------------------------------------//
const SideArray* nSide::GetNeutrals(){
    nRoot* sideRoot = this->GetParent()->GetParent();
    foreachchild( nEnt, sideEnt, sideRoot ) {
        nSide* side = dynamic_cast<nSide*>(sideEnt->Find("side"));
        if (side) {
            if ( side != this && this->hostiles.Find(side) == 0 ){
                neutrals.Append( side );
                //n_printf("%s considers %s a neutral\n", str(this), str(side));
            }
        } else {
            n_printf("%s.GetNeutrals detecting possible malformed sideEnt at %s\n", str(this), str(sideEnt));
        }
    }
    return &neutrals;
}
//-----------------------------------------------------------------------------//
const EntArray* nSide::GetNeutralUnits(){
    this->neutralUnits.Clear();
    this->GetNeutrals();
    foreacht( nSide*, neutralSide, neutrals){
        const EntArray& neutrals = *neutralSide->GetUnits();
        foreacht( nEnt*, neutral, neutrals ) {
            this->neutralUnits.Append( neutral );
            //n_printf("\t%s\n", str(neutral) );
        }
    }
    return &this->neutralUnits;
}
//-----------------------------------------------------------------------------//
