#define N_IMPLEMENTS nEnt
//------------------------------------------------------------------------------
//  (C) 2004    Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nent.h"
#include "enkiduo/nrcserver.h"
#include "enkiduo/naspect.h"
#include "aspect/nside.h"
#include "ui/nplannerui.h"

nNebulaScriptClass(nEnt, "nthing");

//------------------------------------------------------------------------------
/**
*/
nEnt::nEnt():
    vis                                 ( kernelServer, this ),
    octtree                             ( kernelServer, this ),
    planner                             ( kernelServer, this ),
    renderContextInitialized            ( false ),
    channelHolder                       ( 0 ),
    selectable                          ( true ),
    hasSide                             ( true ),
    inOctree                            ( true ),
    held                                ( false ),
    vars                                ( this ),
    sideCounter                         ( -1 ),
    sideHandle                          ( -1 )
{
}

/**
  Creates the vars child node
  Connects variables pos,rot,octradius
  */
void nEnt::Initialize()
{
    nThing::Initialize();

    kernelServer->PushCwd( this );
    vars = (nRoot*) kernelServer->New("nroot", "vars" );
    n_assert( vars.isvalid() );
    kernelServer->PopCwd();

    pos = CreateVar("pos");
    rot = CreateVar("rot");
    side = CreateVar("side");
    octradius = CreateVar("octradius");

    pos->SetV(0,0,0);
    rot->SetQ(0,1,0,0);
    if( side->GetCounter() == 0 ) {
        //n_printf( "%s.side->SetRef(\"/game/sides/neutral\"); c: %i\n", this->fn, side->GetCounter());
        side->SetRef("/game/sides/neutral");
    }
    octradius->SetF(1.0f);
    needsVis = true;

    octtree = "/sys/servers/octree";
    octElement = 0;

    this->inOctree = true;
    dying = false;

    planner = "/ui/planner";
    selected = false;
}
//------------------------------------------------------------------------------
/**
  Release my vis child
  Removes me from the octree
  */
nEnt::~nEnt() {
    //n_printf("nEnt::~nEnt(%s)\n", this->GetName() );
}
bool nEnt::Release(){
    //n_printf("%s being released %i,%i\n", this->fn, hasSide, side->isvalid() );
    if( vis.isvalid() )
        vis->Release();
    this->RemoveFromOctree();
    this->InvalidateAllRefs();
    for( int i = 0; i < varParents; i ++ )
        delete varParents[i];
    //n_printf("%s being destroyed %i,%i\n", this->fn, hasSide, side->isvalid() );
    return nThing::Release();
}
/**
  Removes me from the octree
  */
void nEnt::RemoveFromOctree(){
    if( octElement ) {
        n_assert( octtree.isvalid() );
        octtree->RemElement( this->octElement );
        //n_printf("Removing %s from octree\n", this->fn );
    }
    this->octElement = 0;
}
/**
  Finds existing / creates a new variable with the appropriate name
  @param varName name of the variable to create
  @return pointer to the created variable
  */
nVar* nEnt::CreateVar( const nString& varName ){
    nVar* v = FindVar( varName );
    if( v == 0 ){
        //n_printf("%s(nEnt) var not found creating %s\n", fn, varName.Get() );
        nString name = FullNameString() + nString("/vars/") + varName;
        v = (nVar*) kernelServer->New("nvar", name.Get() );
    }
    return v;
}
/**
  Finds an existing var
  @param varName name of the variable being created
  @return pointer to that variable
  */
nVar* nEnt::FindVar( const nString& varName ){
    nVar* v = FindVarLocal( varName );
    for( int i = 0; v == 0 && i < varParents.Size(); i ++ ){
        if( varParents[i]->isvalid() ){
            //n_printf("checking parent %s for var %s\n", varParents[i]->get()->GetName(), varName.Get() );
            nVar* parentVar = varParents[i]->get()->FindVar( varName ); 
            if( parentVar != 0 ){
                //    n_printf("var found\n");
                nString name = FullNameString() + nString("/vars/") + varName;
                v = (nVar*) kernelServer->New("nvar", name.Get() );
                v->InheritFrom( parentVar );
            }
        }
        else {
            n_printf("varParents[%i] == %s not valid\n", i, varParents[i]->getname());
        }
    }
    return v;
}
/**
  Searches for a variable attached to me - no recursive inheritence serach
  */
nVar* nEnt::FindVarLocal( const nString& varName ){
    return (nVar*) vars->Find( varName.Get());
}
/**
  Creates a stacked variable
  */
nVar* nEnt::CreateStackedVar( const nString& varName ){
    //n_printf("nEnt(%s)::CreateStackedVar(%s)\n", fn, varName.Get() );
    nVar* v = FindVar( varName );
    classVar(nvarstack, varstackClass );
    if( v && (v->IsA( varstackClass ) == false) )  ///we have a local copy of that variable - link to it
        return v;
    nVarStack* vs = FindVarStackLocal( varName );
    /*if( vs == 0 ){
      nString name = FullNameString() + "/vars/" + varName;
      vs = (nVarStack*) kernelServer->New("nvarstack", name.get() );
      }*/
    return vs->CreateVar( varName );
}
/**
  Finds a local stacked variable
  */
nVarStack* nEnt::FindVarStackLocal( const nString& varName ){
    //n_printf("nEnt::FindVarStackLocal(%s)\n", varName.Get() );
    nVarStack* vs = (nVarStack*) vars->Find( varName.Get() );

    if( vs == 0 ){
        nString name = FullNameString() + "/vars/" + varName;
        vs = (nVarStack*) kernelServer->New("nvarstack", name.get() );
    }
    return vs;
}

/**
  Inherits variables from another ent
  @param parentName path to the parent I am inheriting from
  */
void nEnt::InheritFrom( const nString& parentName ){
    nDynAutoRef<nEnt>* ref = new nDynAutoRef<nEnt>( kernelServer, this );
    *ref = parentName.Get();
    //n_printf("%s inheriting from %s\n", fn, parentName.Get() );
    varParents.Append( ref );
}

/**
  Updates my vis sibling
  Updates octree information
  Exports any appropriate channel information to my channelholder
  Checks death if dying
  */
void nEnt::Tick( float , float ){
    //check if my side has changed
    if( this->hasSide ) {
        int c = side->GetCounter();
        if( c > sideCounter ) {
            //n_printf("%s.side dirty\n", this->fn);
            if( side->isvalid() == false )
                n_printf("warning %s has invalid side\n", this->fn);
            else {
                n_assert( side->isvalid() );
                nEnt* e = (nEnt*) side->GetRef();
                if( e == 0 )
                    n_error("%s being set to a non existing side\n", this->fn );
                //n_printf("WARNING: %s has an invalid side ref\n", this->fn );
                else {
                    this->RemoveFromSide();
                    sideCounter = c;
                    nSide* s = (nSide*) e->Find("side");
                    //if( s == 0 ) 
                    if( s == 0 ) {
                        sideHandle = -1;
                        //n_error("side;%s does not have an nside\n");
                    }
                    else {
                        //n_printf("%s->AddUnit(%s)\n", s->fn, this->fn );
                        s->AddUnit( this );
                        sideHandle = e->GetVarI("handle");
                    }
                }
            }
        }
    } //else if(side->isvalid() == false)
        //side->SetRef("/game/sides/neutral");
    if( needsVis ) { 
        ifvalid( octtree )
            if( inOctree )
                if( octElement == 0 ) {
                    octElement = new nOctreeElement();
                    octElement->ent = this;
                    octtree->AddElement( this->octElement );
                    //n_printf("Adding %s to octree\n", this->fn );
                }
        if( vis.isvalid() ){
            if( renderContextInitialized == false ) {
                static int renderContextCounter = 0;
                vis->RenderContextCreated( renderContextCounter++ );
                renderContextInitialized = true;
            }
            vector3 p = pos->GetV();
            quaternion q = rot->GetQ();
            vis->Txyz( p.x, p.y, p.z );
            vis->Qxyzw( q.x, q.y, q.z, q.w );
            float r = octradius->GetF();
            if( octElement && octtree.isvalid() ) {
                ifvalid( octtree )
                    octtree->UpdateElement( octElement, p, r );
            }
            //else
            //FIXME once( n_printf("octree not valid()\n"));
            if( channelHolder ){
                for( int i = 0; i < exportedChannels.Size(); i ++ ){
                    int h = exportedChannels[i].handle;
                    float f;
                    switch( exportedChannels[i].var->GetType() ){
                        case VARTYPE_INT:
                            f = (float) exportedChannels[i].var->GetI();
                            break;
                        case VARTYPE_FLOAT:
                            f = exportedChannels[i].var->GetF();
                            break;
                        default:
                            n_error("trying to export channel of unsupported type:%i\n", exportedChannels[i].var->GetType() );
                    }
                    //float f = exportedChannels[i].var->GetF();
                    channelHolder->Set( h, &f, 1 );
                }
            }
        } else {
            once( n_printf("%s.vis.notvalid()\n",fn) );
        }
    }
    if( channelsToExport.Size() > 0 )
        this->TryToExportChannels();
    if( dying )
        CheckDeath();
}

/**
  Prints out all my information nicely to the screen
  */
void nEnt::PrintVars(){
    static nClass* nVarClass        = kernelServer->FindClass("nvar");
    static nClass* nVarStackClass   = kernelServer->FindClass("nvarstack");
    char string[128];
    for ( nRoot* r = vars->GetHead();
            r != 0;
            r = r->GetSucc() ){
        if( r->IsA( nVarClass )){
            ((nVar*)r)->ToString( string );
            n_printf("%s\n", string );
        }
        else if ( r->IsA( nVarStackClass ) ){
            ((nVarStack*)r)->Sum();
        }
        else
            n_printf("non var class %s in %s/vars\n", r->GetName(), fn );


    }
}
void nEnt::SetVarI( const char* varName, int i ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetI( i );
    else
        n_printf("%s(nEnt)::SetVarI(%s,%f) failing as var is not found\n", fn, varName, i );
}

int nEnt::GetVarI( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    int i = 0;
    if( nvar )
        i = nvar->GetI();
    else
        n_printf("%s(nEnt)::GetVarI(%s) failing as var is not found\n", fn, varName );
    return i;
}

void nEnt::SetVarF( const char* varName, float f ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetF( f );
    else
        n_printf("%s(nEnt)::SetVarF(%s,%f) failing as var is not found\n", fn, varName, f );
}

void nEnt::SetStackedVarF( const char* varName, float f ){
    nVarStack* vs = this->FindVarStackLocal( varName );
    ((nVar*)vs->GetTail())->SetF( f );
}
float nEnt::GetVarF( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    float f = 0.0f;
    if( nvar )
        f = nvar->GetF();
    else {
        n_printf("%s(nEnt)::GetVarF(%s) failing as var is not found\n", fn, varName );
    }
    return f;
}

void nEnt::SetVarV( const char* varName, const vector3& v ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetV( v );
    else
        n_printf("%s(nEnt)::SetVarV(%s,(%f,%f,%f)) failing as var is not found\n", fn, varName, v.x, v.y, v.z );
}

void nEnt::SetVarV( const char* varName, const float x, const float y, const float z ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetV( x,y,z );
    else
        n_printf("%s(nEnt)::SetVarV(%s,%f,%f,%f) failing as var is not found\n", fn, varName, x,y,z );
}

vector3 nEnt::GetVarV( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    vector3 v;
    if( nvar )
        v = nvar->GetV();
    else
        n_printf("%s(nEnt)::GetVarV(%s) failing as var is not found\n", fn, varName );
    return v;
}
void nEnt::SetVarQ( const char* varName, const quaternion& q ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetQ( q );
    else
        n_printf("%s(nEnt)::SetVarQ(%s,(%f,%f,%f,%f)) failing as var is not found\n", fn, varName, q.x, q.y, q.z, q.w );
}

void nEnt::SetVarQ( const char* varName, const float x, const float y, const float z, const float w ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetQ( x,y,z,w );
    else
        n_printf("%s(nEnt)::SetVarQ(%s,%f,%f,%f,%f) failing as var is not found\n", fn, varName, x,y,z,w );
}

quaternion nEnt::GetVarQ( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    quaternion q;
    if( nvar )
        q = nvar->GetQ();
    else
        n_printf("%s(nEnt)::GetVarQ(%s) failing as var is not found\n", fn, varName );
    return q;
}


void nEnt::SetVarS( const char* varName, const char* string ){
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetS( string );
    else
        n_printf("%s(nEnt)::SetVarS(%s, %s) failing as var is not found\n", fn, varName );
}
const char* nEnt::GetVarS( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    const char* s = 0;
    if( nvar )
        s = nvar->GetS().Get();
    else
        n_printf("%s(nEnt)::GetVarS(%s) failing as var is not found\n", fn, varName );
    return s;
}

void nEnt::SetVarRef( const char* varName, const char* pathName ){
    n_printf("SetVarRef(%s,%s)\n", varName, pathName);
    nVar* nvar = this->FindVar( varName );
    if( nvar )
        nvar->SetRef(pathName);
    else
        n_printf("%s(nEnt)::SetVarRef(%s, %s) failing as var is not found\n", fn, varName );
}
char EntGetVarRefString[1024];
const char* nEnt::GetVarRef( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    const char* s = 0;
    if( nvar ){
        nThing* t = nvar->GetRef();
        if( t )
            s = t->GetFullName( EntGetVarRefString, 1024 );
        else
            s = "none";
    }
    else
        n_printf("%s(nEnt)::GetVarRef(%s) failing as var is not found\n", fn, varName );
    return s;
}

void nEnt::SetStackedVarV( const char* varName, float x, float y, float z ){
    nVarStack* vs = this->FindVarStackLocal( varName );
    ((nVar*)vs->GetTail())->SetV( x, y, z );
}
void nEnt::SetStackedVarS( const char* varName, const char* s ){
    nVarStack* vs = this->FindVarStackLocal( varName );
    ((nVar*)vs->GetTail())->SetS( s );
}
void nEnt::SetStackedVarRef( const char* varName, const char* ref ){
    nVarStack* vs = this->FindVarStackLocal( varName );
    if( vs )
        ((nVar*)vs->GetTail())->SetRef( ref );
    else
        n_printf("%s could not set varstack:%s to :%s because varstack wasn't found\n", this->fn, varName, ref );
}
void nEnt::DefaultStackedVarRef( const char* varName, const char* pathname ) {
    nVarStack* vs = this->FindVarStackLocal( varName );
    nVar* v = vs->PushVar( varName );
    v->SetRef( pathname );
    n_printf("v->SetRef( %s )\n", pathname );
}

/**
  Opens a tk-gui window to edit the appropriate rc value
  poorly supported
  */
void nEnt::EditRC( const char* varName ){
    nVar* nvar = this->FindVar( varName );
    if( nvar ){
        nRCServer* rc = (nRCServer*) kernelServer->Lookup("/sys/servers/rc");
        char fullname[1024];
        nvar->GetFullName( fullname, 1024 );
        rc->SetRC( fullname, nvar->GetRC() );
    }
    else
        n_printf("%s(nEnt)::EditRef(%s) failing as var is not found\n", fn );
}
void nEnt::SaveRC( const char* varName, const char* filename ) {
    CreateVar(varName)->SaveRC(filename);
}
void nEnt::LoadRC( const char* varName, const char* filename ) {
    CreateVar(varName)->LoadRC(filename);
}
/**
  Begins the death cycle
  */
void nEnt::Die(){
    dying = true;
    if( selected && planner.isvalid() )
        planner->RemoveSelectedEnt( this );
    if( hasSide && side->isvalid() ) {
        nEnt* e = (nEnt*)side->GetRef();
        //n_printf("%s being removed from side %s\n", this->fn, e->fn );
        n_assert( e );
        nSide* s = (nSide*) e->Find("side");
        if( s )
            s->RemoveUnit( this );
    }
    CheckDeath();
    nThing::Die();
}
void nEnt::Select() { 
    //n_printf("%s selected\n", this->fn );
    n_assert( selected == false );
    this->selected = true; 
    foreachchild(nAspect, aspect, this){
        aspect->Select();
    }
    planner->AddSelectedEnt( this );
}
void nEnt::Deselect() { 
    //n_printf("%s unselected\n", this->fn );
    n_assert( selected == true );
    this->selected = false; 
    foreachchild(nAspect, aspect, this){
        aspect->DeSelect();
    }
    planner->RemoveSelectedEnt( this );
}
/**
  Alerts child nodes of dying status,  checks if any blocking
  If not - Die
  */
void nEnt::CheckDeath(){
    classVar(naspect,aspectClass);
    bool deathBlocked = false;
    for ( nRoot* c = GetHead(); c; c = c->GetSucc() ){
        if( c->IsA( aspectClass ) )
            deathBlocked = deathBlocked || ((nAspect*)c)->BlockDeath( false );
    }
    if( deathBlocked == false )
        nThing::Die();
}
/**
  Designates a var to be exported as a channel in the vis tree
  Currently supports only f variables
  */
void nEnt::ExportChannel( const char* channelName ){
    char* s = new char[strlen( channelName)+1];
    strcpy( s, channelName );
    channelsToExport.Append( s );
    delete[] s;
}

/**
  Builds the channel holder object 
  Links any information needed to export channels
  */
void nEnt::TryToExportChannels(){
    if( channelHolder == 0 ) {
        if( vis.isvalid() ) {
            kernelServer->PushCwd( vis.get() );
            nChannelHolder* ch = (nChannelHolder*) kernelServer->New("nchannelholder", "channelholder" );
            kernelServer->PopCwd();
            n_assert( ch && "failed to create channel server\n");
            channelHolder = ch;
        } else {
            //n_printf("%s cant create channel holder / vis is not valid\n", this->GetName() );
        }
    }
    if( channelHolder ){
        for( int i = 0; i < channelsToExport.Size(); i ++ ) {
            nVar* v = this->FindVar( channelsToExport[i].Get() );
            if( v ) {
                ExportedChannel e;
                e.channelName = channelsToExport[i].Get();
                e.var = v;
                n_assert( v );
                float f;

                switch( e.var->GetType() ){
                    case VARTYPE_INT:
                        f = (float) e.var->GetI();
                        break;
                    case VARTYPE_FLOAT:
                        f = e.var->GetF();
                        break;
                    default:
                        n_error("trying to export channel of unsupported type:%i\n", e.var->GetType() );
                }

                e.handle = channelHolder->Add( channelsToExport[i].Get(), &f, 1 );
                //n_printf("e:%i %i\n", e.var, e.handle );
                exportedChannels.Append( e ); 
                channelsToExport.Erase( i );
                //n_printf("%s exported %s\n", fn, e.channelName );
            } /*else {
                n_printf("cannot find var:%s to export\n", channelsToExport[i] );
                }*/
        }
    }
}

int nEnt::GetSideHandle(){
    return sideHandle;
}
void nEnt::RemoveFromSide(){
    if( sideHandle != -1 ){ //changing sides mid game
        //have to search for our old side - bad bad chris
        nRoot* sides = kernelServer->Lookup("/game/sides");
        foreachchild( nEnt, e, sides ) {
            if( e->GetVarI("handle") == sideHandle ) {
                nSide* s = (nSide*) e->Find("side");
                if( s == 0 ) n_error("side;%s does not have an nside\n");
                n_printf("%s->RemoveUnit(%s)\n", str(s), str(this));
                s->RemoveUnit( this );
            }
        }
    }
}
