#define N_IMPLEMENTS nFSM
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nfsm.h"
#include "enkiduo/nfsmlink.h"
#include "kernel/nstacktrace.h"

nNebulaScriptClass(nFSM, "naspect");

//------------------------------------------------------------------------------
/**
*/
nFSM::nFSM():
    nAspect()
{
    this->complainAboutDoubleStateSets = true;
}

void nFSM::Initialize()
{
    nAspect::Initialize();
    changeState = false;
    currState.local = true;
    currState.localState = -1;

    //check if you are a child fsm -> a state, in which case wait for a parent to trigger you.
    classVar(nfsm,fsmclass);
    if( parent->IsA( fsmclass ) )
        this->triggerSelf = false; //if your a child state dont call self, let it call you.
    else
        this->triggerSelf = true; 

    this->sceneGraph = 0;
}

nFSM::~nFSM()
{
}

void nFSM::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    if( this->triggerSelf )
        Process( EVENT_UPDATE, time, dtime );
}

bool nFSM::SetState( uint localState ){
    //n_assert (changeState == false );
    if( changeState == true && this->complainAboutDoubleStateSets ) {
        n_printf("warning nFSM(%s) is in state %s and has been told to change state to both [%s|%s] simulatenously\n", this->fn, this->stateNames[currState.localState], this->stateNames[nextState.localState], this->stateNames[localState]);
        n_print_stacktrace("----");
        //n_warn("trying to change states twice");
    }
    changeState = true;
    nextState.local = true;
    nextState.localState = localState;
    return true;
}
bool nFSM::SetState( nString stateName ) {
    if( this->SetStateLocal( stateName ) == true )
        return true;
    else 
        return this->SetStateExternal( stateName );
}
bool nFSM::SetStateExternal( nString externalStateName ){
    n_assert( changeState == false && "trying to change states twice\n");
    nRoot* r = this->Find(externalStateName.Get() );
    nRoot* p = r;
    int counter = 0;
    while( p ){
        counter--;
        p = p->GetHead();
    }
    nextState.localState = counter;
    if (r != 0 ) {
        classVar( nfsm, fsmClass );
        if( r->IsA( fsmClass ) ){
            changeState = true;
            nextState.local = false;
            nextState.externalState = (nFSM*) r;
        }
        else
            once( n_printf("%s trying to link to non FSM child %s\n", str(this), externalStateName.Get() ) );
        return true;
    }
    else
        once( n_printf("trying to link to non existent child fsm %s\n", externalStateName.Get() ) );
    return true;
}
bool nFSM::SetStateLocal( nString stateName ){
    if( changeState == true ) {
        n_printf("trying to change state twice\n");
    }
    for( int i = 0; i < stateNames.Size(); i++ )
        if( stateName == stateNames[i] )
            return this->SetState( i );
    return false;
}
void nFSM::Process( StateMachineEvent event, float time, float dtime ){
    Event( event, time, dtime );
    for( int i = 0; this->changeState == true && i < 10; i ++ ){
        Event( EVENT_EXIT, time, dtime );
        currState = nextState;
        changeState = false;
        Event( EVENT_ENTER, time, dtime );
    }
}
void nFSM::Event( StateMachineEvent event, float time, float dtime ){
    if( currState.local ){
        if( currState.localState < 0 ){
            currState.localState = 0;
            Process( EVENT_ENTER, time, dtime );
        }
        States( event, currState.localState, time, dtime );
    }
    else{
        currState.externalState->Process( event, time, dtime );
    }
}

/*
enum STATES{
    DefaultState,
    Roam,
};
*/

bool nFSM::States( StateMachineEvent , uint , float , float ){
    return false;
}

void nFSM::AddChildState( const char* className, const char* childName ){
    //n_printf(" kernelServer->New( %s, %s )\n", className, (FullNameString() + "/" + childName).Get() );
    kernelServer->PushCwd( this );
    kernelServer->New( className, childName );
    kernelServer->PopCwd();
}

/**
 * create my vis node brother who will pass his attach to me
 **/
void nFSM::createVisNode( const char* path ) {
    //n_printf("nFSM::createVisNode(%s)\n", path);
    int i = randI(0,32000);
    char name[1024];
    int j = snprintf( name, 1024, path, i);
    //n_printf("j: %i built name: %s\n", j, name);
    char name2[1024];
    snprintf(name2, 1024, "/game/vis/%s", name);
    //n_printf("name2:%s\n", name2);
    n3DNode* node = (n3DNode*) kernelServer->New("n3dnode", name2, "nvisnode");
    kernelServer->PushCwd(node);
    nFSMLink* link = (nFSMLink*) kernelServer->New("nfsmlink", "link", "nvisnode");
    kernelServer->PopCwd();
    //n_printf("link:%p\n", link);
    link->SetFSM( this );
    //((nFSMLink*)this->visNode.get())->SetFSM( this );
    this->visNode = link;
}

bool nFSM::Attach(nSceneGraph2* sceneGraph){
    //n_printf("nFSM::Attach(%p)\n", sceneGraph);
    this->sceneGraph = sceneGraph;
    this->Process(EVENT_ATTACH, 0, 0);
    return true;
}
