#define N_IMPLEMENTS nMPIServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nmpiserver.h"

nNebulaScriptClass(nMPIServer, "nthing");

//------------------------------------------------------------------------------
/**
*/
nMPIServer::nMPIServer()
{
}

void nMPIServer::Initialize(){
    nThing::Initialize();
    MPI_Init(0,0);
    MPI_Comm_rank( MPI_COMM_WORLD, &rank );
    MPI_Comm_size( MPI_COMM_WORLD, &numprocs);
    n_assert( rank >= 0 && rank < numprocs );
    n_assert( numprocs >= 1 );
    char command[1024];
    sprintf( command, "global rank\n set rank %i", rank );
    RUNSCRIPT( command );
    sprintf( command, "global numprocs\n set numprocs %i", numprocs );
    RUNSCRIPT( command );

    this->AddToTriggerList();
    this->AddToReleaseList();

    numCallbacks = 0;
}

nMPIServer::~nMPIServer(){
    MPI_Finalize();
}

void nMPIServer::AddCallback( int tag, void* clazz, callbackFunc func, int proc ){
    //n_printf("nMPIServer::AddCallback( %i, %i, %i, %i )>\n", tag, clazz, func, proc );
    n_assert( numCallbacks < MAX_CALLBACKS );
    tags                    [numCallbacks] = tag;
    clazzes                 [numCallbacks] = clazz;
    funcs                   [numCallbacks] = func;
    recievingFrom           [numCallbacks] = proc;
    InitRecv( numCallbacks++ );
}
//tick should check recieves, if any activated then activate the callback
void nMPIServer::Tick( float time, float dtime ){
    //n_printf("<nMPIServer::Tick(%f,%f)>\n", time, dtime );
    nThing::Tick( time, dtime );
    for( int i = 0; i < numCallbacks; i ++ ){
        int b;
        MPI_Status status;
        MPI_Test( requests + i, &b, &status );
        if( b ){ //recieved that message
            //n_printf("recieved message %i\n", i );
            funcs[i](clazzes[i], recievingFrom[i]);
            InitRecv( i );
        }
    }
    //n_printf("</nMPIServer::Tick(%f,%f)>\n", time, dtime );
}

void nMPIServer::InitRecv( int index /*callback index*/ ){
    n_printf("<nMPIServer::InitRecv( %i )>\n", index );
    MPI_Irecv( 0, 0, MPI_INT, recievingFrom[index], tags[index], MPI_COMM_WORLD, requests+index );
}

void nMPIServer::Send( int tag, int proc ){
    n_printf("<nMPIServer::Send( %i , %i)>\n", tag, proc );
    MPI_Send( 0, 0, MPI_INT, proc, tag, MPI_COMM_WORLD );
}
