#define N_IMPLEMENTS nODEServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nodeserver.h"
#include "node/n3dnode.h"


nNebulaScriptClass(nODEServer, "nthing");

//------------------------------------------------------------------------------
/**
*/
nODEServer::nODEServer():
prof (kernelServer, this )
{
    prof = "/sys/servers/prof";
}

void nODEServer::AddNode( nODEPhysics* node ){
    nodes.At( nodes.Size()) = node;
}
/**
  ODE OVERVIEW
   1. create dynamics world
   2. create bodies in the dynamics world
   3. set the state of all bodies
   4. create joints in the dynamics world
   5. attach the joints to the bodies
   6. set the parameters of all joints
   7. create a collision world and collision geometry objects as neccessary
   8. Loop:
        8.1 apply forces to the bodies as neccessary
        8.2 adjust the joint parameters as neccessary
        8.3 call collision detection
        8.4 create a contact point for every collision point and put it in a contact joint group
        8.5 take a simulation step
        8.6 remove all joints in the contact joint group
   9 destroy the dynamics and collision worlds
*/

dBodyID nODEServer::CreateBody() {
    return dBodyCreate( world );
}

void nODEServer::RemBody( dBodyID id ){
    dBodyDestroy(id);
}
/**
  Initialize things
  creates the world
  creates the space
  adds the empty contact group for collisions

  sets default values for the universe
*/
void nODEServer::Initialize(){
    nThing::Initialize();
    //1 - create dynamics world
    world = dWorldCreate();
    space = dHashSpaceCreate(0);
    contactJoints = dJointGroupCreate( 0 );

    dWorldSetGravity 		(world,0,-500.0,0);
    dWorldSetERP 		(world,.2);	//error correction
    dWorldSetCFM 		(world,1e-5);	//constraint force mixing
    dWorldSetAutoDisableFlag 		(world,0);
    dWorldSetContactMaxCorrectingVel 	(world,0.1);
    dWorldSetContactSurfaceLayer 	(world,0.001);
    dCreatePlane		(space,  0, 1,  0, 0 );
    dCreatePlane		(space,  0, -1,  0, -10 );
    dCreatePlane		(space,  1, 0,  0, 0 );
    dCreatePlane		(space,  0, 0, +1, 0 );
    dCreatePlane		(space, -1, 0,  0, -50 );
    dCreatePlane		(space,  0, 0, -1, -50 );

    this->AddToTriggerList();
    this->AddToDisplayList();
}

nODEServer::~nODEServer(){
    dWorldDestroy( world );
    dSpaceDestroy( space );
}
//------------------------------------------------------------------------------
/**
 */
static void CallbackWrapper ( void* data, dGeomID o1, dGeomID o2 ) {
    ((nODEServer*)data)->Callback( data, o1, o2 );
}
void nODEServer::Callback( void *data, dGeomID o1, dGeomID o2 ) {
    int i;
    dBodyID b1 = dGeomGetBody(o1);
    dBodyID b2 = dGeomGetBody(o2);
    if (b1 && b2 && dAreConnectedExcluding (b1,b2,dJointTypeContact)) return;

    dContact contact[MAX_CONTACTS];
    if (int numc = dCollide ( o1, o2, nODEServer::MAX_CONTACTS, &contact[0].geom, sizeof(dContact))) {
        for (i=0; i<numc; i++) {
            /*
            contact[i].surface.mode = dContactBounce;
            contact[i].surface.mu = 100.0; //dInfinity;
            contact[i].surface.mu2 = 0;
            contact[i].surface.bounce = 0.2;
            contact[i].surface.bounce_vel = 0.1;
            contact[i].surface.soft_cfm = 0.001;
            */

            contact[i].surface.mode = dContactSlip1 | dContactSlip2 | dContactSoftERP | dContactApprox1;
            //contact[i].surface.mu = dInfinity;
            contact[i].surface.mu = 10.0;
            contact[i].surface.slip1 = 0.91;
            contact[i].surface.slip2 = 0.91;
            contact[i].surface.soft_erp = 10.0;
            contact[i].surface.soft_cfm = 0.03;
            dJointID c = dJointCreateContact ( world, contactJoints, contact+i );
            dJointAttach (c,b1,b2);
            //this->DrawPoint( vector3(contact[i].geom.pos[0], contact[i].geom.pos[1], contact[i].geom.pos[2]), white );
        }
    }
}

void nODEServer::AddContact( dBodyID b1, dBodyID b2, vector3 pos, vector3 normal, float depth, vector3 fdir ) {
    dContact& contact = contacts.Push();
    //contact.surface.mode = dContactSlip1 | dContactSlip2 | dContactApprox1 | dContactSoftERP | dContactSoftCFM;
    //contact.surface.mode = dContactSlip1 | dContactSlip2 | dContactApprox1;
    contact.surface.mode = dContactBounce;
    contact.surface.mu = dInfinity;
    //contact.surface.mu = 100.0;
    contact.surface.slip1 = 0.00;
    contact.surface.slip2 = 0.00;
    contact.surface.soft_erp = 0.5;
    contact.surface.soft_cfm = 0.0;
    contact.geom.pos[0] = pos.x;
    contact.geom.pos[1] = pos.y;
    contact.geom.pos[2] = pos.z;
    contact.geom.normal[0] = normal.x;
    contact.geom.normal[1] = normal.y;
    contact.geom.normal[2] = normal.z;
    contact.geom.depth = depth;
    contact.fdir1[0] = fdir.x;
    contact.fdir1[1] = fdir.y;
    contact.fdir1[2] = fdir.z;

    dJointID c = dJointCreateContact ( world, contactJoints, &contact );
    dJointAttach (c,b1,b2);
    this->DrawPoint( vector3(contact.geom.pos[0], contact.geom.pos[1], contact.geom.pos[2]), green, .1);
}

void nODEServer::Tick( float time, float dtime ){
    //static int iii = 0;
    //if( iii++ % 100 != 0 ) return;
    nThing::Tick( time, dtime );
    //check for collisions inside the root space
    prof->Begin("collision");
    dSpaceCollide( space, this, CallbackWrapper );
    //update the world
    dWorldQuickStep( world, dtime );
    //clear any contact joings
    dJointGroupEmpty ( contactJoints );
    prof->End("collision");
}
