#define N_IMPLEMENTS nPathfinder
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/npathfinder.h"
#include "enkiduo/ntriggerserver.h"
#include "enkiduo/nent.h"
nNebulaScriptClass(nPathfinder, "nthing");
//------------------------------------------------------------------------------
bool Intersection( const line3 & line, const sphere& sphere, float& mu0, float& mu1 ){
    float a, b, c, i;
    vector3 ls = line.start();
    vector3 le = line.end();
    vector3 sp = sphere.p;

    a = sqr(le.x-ls.x)+sqr(le.y-ls.y)+sqr(le.z-ls.z);
    b = 2 * ( (le.x-ls.x)*(ls.x-sp.x)
            + (le.y-ls.y)*(ls.x-sp.y)
            + (le.z-ls.z)*(ls.z-sp.z));
    c = sqr(sp.x)+sqr(sp.y)+sqr(sp.z)
        + sqr(ls.x)+sqr(ls.y)+sqr(ls.z)
        - 2 * (sp.x*ls.x + sp.y*ls.y + sp.z*ls.z) - sqr(sphere.r);
    i = sqr(b) - 4 * a * c;
    if( i < 0.0 ){
        return false;
    }
    else if ( i == 0 ){
        mu0 = -b / (2*a);
        mu1 = -b / (2*a);
        return false;
    }
    else {
        i = (float) sqrt(i);
        mu0 = (-b + i) / (2*a);
        mu1 = (-b - i ) / (2*a);
        return true;
    }
}

nPathfinder::nPathfinder():
    coords(1000,100),
    coordDensity(15.0f),
    dangerCost(1000.0f),
    rc(1.0f),
    drawCoords	(false),
    drawEdges	(false),
    drawPath	(false),
    drawSpheres	(false),
    edgeCostThreshold(-1.0f),
    edgeLengthThreshold(250.0f),
    useDetectionRange(true),
    useEffectiveRange(true)
{
    for( int i = 0; i < MAX_NUM_NODES; i ++ ){
        traverseCosts[i] = new float[MAX_NUM_NODES];
        for( int j = 0; j < MAX_NUM_NODES; j ++ )
            traverseCosts[i][j] = -1.0f;
    }
    //worldSize = 1000;
}


void nPathfinder::Initialize(){
    nThing::Initialize();
    this->AddToDisplayList();
}

nPathfinder::~nPathfinder(){
}

void nPathfinder::Display( nGfxServer* gfx ){
    int i;
    //DRAW COORDS
    if( drawCoords ) {
        gfx->Begin( N_PTYPE_LINE_LIST );
        gfx->Rgba( 1,1,1,1 );	
        for( i = 0; i < coords.Size(); i ++ ){
            //      n_printv( coords[i] );
            //      n_printv( coords[i] + vector3(0,5,0) );
            gfx->Coord( coords[i] );
            gfx->Coord( coords[i] + vector3(0,5,0) );
        }
        gfx->End();
    }
    //DRAW EDGES
    if( drawEdges ) {
        gfx->Begin( N_PTYPE_LINE_LIST );
        gfx->Rgba( 1,0,1,1 );
        int j;
        float maxCost = -1.0f;
        for( i = 0; i < coords.Size(); i ++ ) {
            for( j =  i+1; j < coords.Size(); j ++ ) {
                if( traverseCosts[i][j] > 0.0f && (edgeCostThreshold>0 ? traverseCosts[i][j] < edgeCostThreshold:true )) {
                    if( edgeLengthThreshold == 0 || (coords[i] - coords[j]).len() < edgeLengthThreshold ){
                        if( traverseCosts[i][j] > maxCost )
                            maxCost = traverseCosts[i][j];
                    }
                }
            }
        }
        for( i = 0; i < coords.Size(); i ++ ) {
            for( j = i+1; j < coords.Size(); j ++ ){
                if( traverseCosts[i][j] > 0.0f && (edgeCostThreshold>0 ? traverseCosts[i][j] < edgeCostThreshold:true )) {
                    if( edgeLengthThreshold == 0 || (coords[i] - coords[j]).len() < edgeLengthThreshold ){
                        float c = traverseCosts[i][j] / maxCost;
                        c = n_clamp( c, 0.0f, 1.0f );
                        gfx->Rgba( c, 0.0f, 1.0f-c, 1.0f );   
                        gfx->Coord( coords[i]+vector3(0,c,0) );
                        gfx->Coord( coords[j]+vector3(0,c,0) );
                    }
                }						
            }
        }
        gfx->End();
    }
    //DRAW SPHERES (CIRCLES)
    if( drawSpheres ) {    
        gfx->Rgba( 1,.5,0,1 );
        for(  i = 0; i < spheres.Size(); i ++ ){
            gfx->Begin( N_PTYPE_LINE_STRIP );
            float theta;
            for( theta = 0.0f; theta < 2.0f * N_PI; theta += .15f )
                gfx->Coord( spheres[i].p + vector3( n_cos(theta) * spheres[i].r, 0, n_sin(theta) * spheres[i].r ));
            gfx->Coord( spheres[i].p + vector3( n_cos(theta) * spheres[i].r, 0, n_sin(theta) * spheres[i].r ));
            gfx->End();
        }
    }
    //DRAW PATH
    if( drawPath ) {
        gfx->Rgba( 0,1,0,1 );
        for( int i = 0; i < paths; i ++ ) {
            gfx->Begin( N_PTYPE_LINE_STRIP );	
            for( int j = 0; j < paths[i]; j ++ ) {
                vector3 v = path[i];
                v.y += 2.0f;
                gfx->Coord( v );
                gfx->End();
            }
        }
    }
}

//-----------------------------------------------------------------------------//
///Parses out the locations of all threats and places routing coordinates around their perimeter
//-----------------------------------------------------------------------------//
void nPathfinder::CalculateCoords(){
    nScriptServer* script = (nScriptServer*) kernelServer->Lookup("/sys/servers/script"); 
    n_assert( script );
    const char* s;
    script->Run("return $Map::worldsize", s);
    sscanf( s, "%f", &this->worldSize );
    n_printf("worldsize---%s->%f\n", s, this->worldSize );

    nRoot* threatRoot = kernelServer->Lookup("/game/ents/threat");
    n_assert( threatRoot );
    vector3 coord;
    sphere sph;
    spheres.Clear();
    coords.Clear();
    //edges.Clear();
    int i;
    for( i = 0; i < MAX_NUM_NODES; i ++ ){
        for( int j = 0; j < MAX_NUM_NODES; j ++ ){
            traverseCosts[i][j] = -1.0f;
        }
    }
    for( i = 0; i < MAX_NUM_NODES; i ++ )
        parents[i] = -1;
    edgesCached = 0;
    ifnonzero( threatRoot ){
        for (nEnt* threat = (nEnt*)threatRoot->GetHead(); threat; threat = (nEnt*)threat->GetSucc() ){
            sph.p = threat->GetVarV("pos");			
            if( this->useDetectionRange ) {
                sph.r = rc * threat->GetVarF("detectionrange");
                spheres.Append( sph );
            }
            if( this->useEffectiveRange ) {
                sph.r= rc * threat->GetVarF("effectiverange");
                spheres.Append( sph );
            }
        }
    }
    for( i = 0; i < spheres.Size(); i ++ ){
        float r = spheres[i].r;
        float thetaStep = this->coordDensity / r;
        thetaStep = 2.0f * N_PI / this->coordDensity;
        //some trig to determine how far to extend the radius so that paths between coords do not intersect the circle
        float radius = n_sqrt( sqr(r) / (sqr(n_cos(thetaStep))-sqr(n_sin(thetaStep))) );
        for( float theta =0; theta <= N_PI* 2.0f; theta += thetaStep ) {
            coord.x = n_cos( theta ) * radius + spheres[i].p.x;
            coord.y = spheres[i].p.y;
            coord.z = n_sin( theta ) * radius + spheres[i].p.z;
            //coords.Append( coord );
            AddCoord( coord );
        }
    }
    //now that we know #coords
    //reset travel costs
    open.initialize( MAX_NUM_NODES * 10 );
    if( path.Size() > 1 ) 
        FindPath( path[0], path[path.Size()-1] );
}

float nPathfinder::EstimateCost( int startIndex, int goalIndex ){
    n_assert( startIndex >= 0 && startIndex < MAX_NUM_NODES );
    n_assert( goalIndex >= 0 && goalIndex < MAX_NUM_NODES );
    if( startIndex == goalIndex ) return 0;
    return ( coords[goalIndex] - coords[startIndex] ).len();
}

float nPathfinder::TraverseCost( int startIndex, int goalIndex ){
    n_assert( startIndex >= 0 && startIndex < MAX_NUM_NODES );
    n_assert( goalIndex >= 0 && goalIndex < MAX_NUM_NODES );
    if( traverseCosts[startIndex][goalIndex] > 0.0f )
        return traverseCosts[startIndex][goalIndex]; //try to fetch from cache
    line3 line( coords[startIndex],coords[goalIndex] );
    float distInDanger = 0.0f;
    float mu0, mu1;
    for( int i = 0; i < spheres.Size(); i ++ ){
        if( Intersection( line, spheres[i], mu0, mu1 ) ){
            if( mu0 > mu1 )
                swap( mu0, mu1 );
            if( mu0 >= 1.0f || mu1 <= 0.0f) //collision is outside segment
                continue;
            if( mu0 < 0.0f )
                mu0 = 0.0f;
            if( mu1 > 1.0f )
                mu1 = 1.0f;
            distInDanger += ( mu1 - mu0 ) * line.len();
        }
    }
    float cost = this->dangerCost * distInDanger + line.len();
    //n_assert( cost > 0 );
    if( cost <= 0.0 ){
        once( n_printf("warning cost : 0\n") );
    }
    n_assert( startIndex >= 0 && startIndex < MAX_NUM_NODES );
    n_assert( goalIndex >= 0 && goalIndex < MAX_NUM_NODES );
    traverseCosts[startIndex][goalIndex] = cost; //save to cache
    traverseCosts[goalIndex][startIndex] = cost; //save to cache
    edgesCached++;
    //if( edgesCached %1000 == 0 )
    //    n_printf("edgesCached: %i\n", edgesCached );
    return cost;
}
void nPathfinder::FindPath( vector3 startPoint, vector3 goalPoint ){
    //printf("<nPathFinder::FindPath()>\n");
    if( coords.Size() <= 0 )
        this->CalculateCoords();
    int i;
    path.Clear();
    if( startPoint.isequal(goalPoint,.001f) ){
        path.Append( startPoint );
        return;
    }
    int startIndex = -1, goalIndex = -1;
    startIndex = AddCoord( startPoint );
    goalIndex = AddCoord( goalPoint );
    if (startIndex < 0 || goalIndex < 0 ) {
        n_printf("WARNING: trying to pathfind to an area outside the world\n");
        return;
    }
    open.clear(); //empty heap

    n_assert( startIndex >= 0 && startIndex < coords.Size() );
    n_assert( goalIndex >= 0 && goalIndex < coords.Size() );
    n_assert( coords.Size() <= MAX_NUM_NODES );

    for( i = 0; i < coords.Size(); i++ )
        bestCostTo[i] = -1.0f;
    bestCostTo[startIndex] = 0.0f; 

    open.add( EstimateCost( startIndex, goalIndex), startIndex);
    int current;
    while( open.length() > 0 ){
        float f;
        open.pop( f, current );
        float d =  f - ( bestCostTo[current] + EstimateCost( current, goalIndex ) );
        if( fabs(d) > .1f ) //repeat node (not best way here).
            continue;
        //n_printf("f:%f best:%f estimate:%f d:%f\n", f, bestCostTo[current], EstimateCost( current, goalIndex ), d);
        n_assert( current >= 0 && current < coords.Size() );
        if( current == goalIndex ){ //if at goal
            nArray<vector3> reversedPath;
            while( current != startIndex ){
                reversedPath.Append( coords[current] );
                current = parents[current];
            }
            n_assert( current == startIndex );
            reversedPath.Append( coords[startIndex] ); //append the start pos
            for( i = reversedPath.Size()-1; i >= 0; i-- )
                path.Append( reversedPath[i] );
            //n_printf("ending findpath\nPath:\n");
            //this->Sum();
            //n_printf("returning n: \n", path.Size());
            return;
        }
        for( int neighbor = 0; neighbor < coords.Size(); neighbor ++ ){ //foreach neighbor
            if( neighbor != current ){
                float costTo = TraverseCost( current , neighbor ) + bestCostTo[current];
                n_assert( costTo > 0.0 );
                if( bestCostTo[neighbor] < 0.0f || costTo < bestCostTo[neighbor] ){ //found new way there
                    float costFrom = EstimateCost( neighbor, goalIndex );
                    if( neighbor!= goalIndex )
                        //n_assert( costFrom > 0.0 );
                        if( costFrom <= 0.0 ){
                            once( n_printf("warning cost : 0\n") );
                        }
                    float totalCost = costTo + costFrom;
                    bestCostTo[neighbor] = costTo;
                    parents[neighbor] = current;
                    open.add( totalCost, neighbor );
                }
            }
        }
    }
    n_error("failed to find path == not good\n");
    //printf("<\nPathFinder::FindPath()>\n");
    //paths.Append(path);
    nArray<vector3>& v = paths.Push();
    for ( int i= 0; i< path; i ++ )
        v.Append( path[i] );
    return;
}

void nPathfinder::Message( nMessage* m ){
    if( m->to == "unselected" ){
        //if( (m->v - vector3(0,0,0)).len() > 100.0f )
        //    n_printf("nPathFinder::message( %f,%f,%f )\n", m->v.x, m->v.y, m->v.z );
        if( fabs(m->v.y) < 10.0f )
            this->FindPath( vector3(0,0,0), m->v );
        else
            n_printf("nPathFinder::message( %f,%f,%f )\n", m->v.x, m->v.y, m->v.z );
    }
}
int nPathfinder::AddCoord( vector3 coord ) {
    if( coord.x > 0 && coord.x < worldSize && coord.z > 0 && coord.z < worldSize ) {
        for( int i = 0; i < coords; i ++ )
            if( coord.isequal( coords[i], .1f ) )
                return i;
        coords.Append( coord );
        return coords.Size()-1;
    } else {
        return -1;
    }
}
