#define N_IMPLEMENTS nProfServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nprofserver.h"
#include "util/nheap.h"

nNebulaScriptClass(nProfServer, "nthing");

//------------------------------------------------------------------------------
/**
*/
nProfServer::nProfServer():
    timeServer( kernelServer, this )
{
    if( prof == 0 )
        prof = this;
}

void nProfServer::Initialize(){
    nThing::Initialize();
    timeServer = "/sys/servers/time";
    current = root = 0;
    this->AddToTriggerList();
    this->AddToDisplayList();
    //this->Tick(0,0);
}
//------------------------------------------------------------------------------
/**
*/
nProfServer::~nProfServer()
{
    if( root ) 
        delete root;
}

void nProfServer::BeginFrame(){
    if( root == 0 )
        current = root = new ProfNode("root");
    //n_assert( root == current );
    root->BeginFrame();
    root->Start( timeServer->GetTime());
}

void nProfServer::EndFrame(){
    if( root != current ) {
        n_printf("nProfServer::EndFrame current(%s) != root(%s)\n", current->fn, root->fn);
        while( current != root ) {
            this->End("whatever");
        }
        //for (ProfNode* node = current; node && node != root; node = node->parent ) {
            //node->Stop( timeServer->GetTime() );
        //}
    }
    root->Stop( timeServer->GetTime() );
    root->EndFrame();
    this->CalculateStats();
}
    
void nProfServer::Begin( nString name ){
    //n_printf("nProfServer(%s)\n", name.get() );
    bool found = false;
    for( int i = 0; i < current->children; i ++ ) {
        if( current->children[i]->name == name ) {
            current = current->children[i];
            found = true;
            break;
        }
    }
    if( !found ){
        ProfNode* node = new ProfNode( name );
        node->parent = current;
        current->children.Append( node );
        current = node;
    }
    current->Start( timeServer->GetTime());
}
void nProfServer::End( nString name ){
    current->Stop( timeServer->GetTime() );
    n_assert( current->parent );
    current = current->parent;
}
void nProfServer::CalculateStats(){
    if( root )
        root->CalculateStats();
}
void nProfServer::ClearStats(){
    if( root )
        root->ClearStats();
}
void nProfServer::Display( nGfxServer* gfx ){
    gfx->BeginText();
    gfx->TextPos( .1, -1.0 );
    if( root )
        root->Display( gfx, 0 );
    gfx->EndText();
}

void ProfNode::CalculateStats( float parentFrameTime, float totalFrameTime ){
    int i;
    totalSampleTimes += sampleTime;
    if( parent == 0 ) {
        totalFrameTime = this->frameTime;
        parentFrameTime = this->frameTime;
    }
    float adjustedFrameTime = this->frameTime;
    for( i = 0; i < children; i ++ )
        adjustedFrameTime -= children[i]->frameTime;
    totalPercentage 	= frameTime / totalFrameTime;
    childPercentage                     = adjustedFrameTime / totalFrameTime;
    subtreePercentage 	= frameTime / parentFrameTime;
    if ( frameTime > maxFrameTime )
        maxFrameTime = frameTime;
    if ( totalPercentage > maxTotalPercentage )
        maxTotalPercentage = totalPercentage;
    if ( childPercentage > maxChildPercentage )
        maxChildPercentage = childPercentage;
    if ( subtreePercentage > maxSubtreePercentage ) 
        maxSubtreePercentage = subtreePercentage;

    for( i = 0; i < children; i ++ )
        children[i]->CalculateStats( this->frameTime, totalFrameTime );
}
void ProfNode::ClearStats(){
    this->sampleStart = 0;
    this->sampleStop = 0;
    this->sampleTime = 0;
    this->frameTime = 0;
    this->totalSampleTimes = 0;
    this->totalPercentage = 0;
    this->childPercentage = 0;
    this->subtreePercentage = 0;
    this->maxFrameTime = 0;
    this->maxTotalPercentage = 0;
    this->maxChildPercentage = 0;
    this->maxSubtreePercentage = 0;
    for( int i = 0; i < children; i ++ )
        children[i]->ClearStats();
}

void ProfNode::Display( nGfxServer* gfx, int indent ) {
    int i;
    char name[1024];
    name[0] = '\0';
    for( i = 0; i< indent; i ++ )
        strcat( name, "  " );
    strcat( name, this->name.Get() );
    char text[1024];
    sprintf( text, "%-20s ft:%.6f tt:%6.2f s:%.2f a:%.2f t:%.2f\n", name, this->frameTime, this->totalSampleTimes, this->subtreePercentage, this->childPercentage, this->totalPercentage );
    //sprintf( text, "%-20s ft:%.6f tt:%6.2f s:%.2f a:%.2f t:%.2f\n", name, this->maxFrameTime, this->totalSampleTimes, this->maxSubtreePercentage, this->maxChildPercentage, this->maxTotalPercentage );
    gfx->Text( text );
    for( i = 0; i < children; i ++ )
        children[i]->Display( gfx, indent +1 );
}
void ProfNode::BeginFrame(){
    this->sampleTime = 0.0f;
    for( int i = 0; i < children; i ++ )
        children[i]->BeginFrame();
}
void ProfNode::Start( float time ) { 
    this->sampleStart = time; 
}
void ProfNode::Stop( float time ) { 
    this->sampleStop = time; 
    this->sampleTime += sampleStop - sampleStart;
}
void ProfNode::EndFrame(){
    const float avgSteps = 1.0f;
    const float numerator = (avgSteps-1.0f)/avgSteps;
    const float denom = 1.0f - numerator;
    this->frameTime = this->frameTime * numerator + denom * sampleTime;
    for( int i= 0; i < children; i ++ )
        children[i]->EndFrame();
}

nString ProfNode::FullNameString(){
    nString answer;
    for( ProfNode* node = this; node; node = node->parent ) {
        if ( node == this )
            answer = this->name;
        else
            answer = node->name + "/" + answer;
    }
    return answer;
}
