#define N_IMPLEMENTS nReleaseServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nreleaseserver.h"
#include "enkiduo/naspect.h"
nNebulaScriptClass(nReleaseServer, "nthing");

//------------------------------------------------------------------------------
nReleaseServer::nReleaseServer():
    nodes( 1024, 1024 )
{
    this->AddLogTag("releaseserver");
    heap.initialize( 1024 );
    downleveledHeap.initialize( 1024 );

}
//-----------------------------------------------------------------------------//
void nReleaseServer::Initialize(){
    nThing::Initialize();
}
//-----------------------------------------------------------------------------//
nReleaseServer::~nReleaseServer(){
}
//-----------------------------------------------------------------------------//
void nReleaseServer::Tick( float time, float dtime ){
    nThing::Tick( time, dtime );
}
//-----------------------------------------------------------------------------//
void nReleaseServer::ReleaseNow( const char* pathname, bool quite ) { 
    log("nReleaseServer::ReleaseNow(%s)\n", pathname );
    nRoot* r = kernelServer->Lookup(pathname);
    if( r ) 
        this->ReleaseNow( r );
    else if( quite == false )
        n_printf("ReleaseServer::ReleaseNow Failed to find ent:%s\n", pathname );
}
//-----------------------------------------------------------------------------//
void nReleaseServer::ReleaseNow( nRoot* root ){
    //printf("nReleaseServer::ReleaseNow(%s)\n", str(root));
    if( root == 0 ) {
        n_printf("Trying to erase non existing entity:%p\n", root );
        return;
    }
    char* name = new char[128];
    root->GetFullName( name, 128 );
    log("<nReleaseServer::ReleaseNow(%s)>\n", name );
    if( nodes.Size() != 0 )
        log("warning: release now running with a non empty node list s:%i", nodes.Size() );
    nodes.Clear();
    if( root == 0 )
        return;
    bool releaseMyself = false;
    nodes.Append( root );
    heap.add( root->ReleasePriority(), name );
    //make a big list of all the nodes i am to release
    int depth = 0;
    int b0 = 1, b1 = 1;
    for( int i = 0; i < nodes.Size(); i ++ ){
        if( i >= b0 ) {
            depth ++;
            b0 = b1;
        }
        for( nRoot* child = nodes[i]->GetHead(); child; child = child->GetSucc() ) {
            if( child == this )  {
                releaseMyself = true;
                log("child == this\n");
            }
            else {
                nodes.Append( child );
                b1++;
                name = new char[128];
                n_assert( name );
                child->GetFullName( name, 128 );
                if( child->getProtectedRelease() == false )
                    heap.add( -1.0f * child->ReleasePriority(), name );
                else
                    log("%s is protected\n", name );
                log("node:%s at depth:%i\n", name, depth );
            }
        }
    }
    nodes.Clear();
    log("releasing: %i nodes\n", heap.length() );
    bool cont;
    bool first;
    int maxPriority;
    int maxIndex;
    const char* c;
    float f;
    int numErased;
    float minPriority;
    //while we are erasing nodes
    do {
        log("heaplength:%i\n", heap.length() );
        numErased = 0;
        //figure out our next layer to remove
        if( heap.length() > 0 ) {
            minPriority = heap.topValue();
            log("minPriority:%f\n", minPriority ); 
        } else {
            log("heap.length == 0\n");
        }
        while( heap.length() > 0 ) {
            if( numErased > 0 && fabs(heap.topValue() - minPriority) > .5 ) {
                log("f:%f, min:%f\n", minPriority );
                break;
            }
            heap.pop( f, c ); //stupid windows
            log("c:%s\n", c);
            nRoot* r = kernelServer->Lookup(c);
            if( r ) {
                classVar(naspect, aspectclass);
                if( r->IsA( aspectclass ) ){
                    bool blocked = ((nAspect*)r)->BlockDeath( true );
                    if( blocked ) {
                        log("%s blocked", r->GetName() );
                    }
                }
                if( r->GetHead() != 0 ) { //it has children - dont erase it now
                    downleveledHeap.add( f, c );
                } else {
                    //printf("%s->Release(%f)\n", c, f );
                    r->Release();
                    //printf("/%s->Release(%f)\n", c, f );
                    minPriority = f;
                    numErased++;
                    delete[] c;
                } 
            }
        }

        while( downleveledHeap.length() > 0 ) {
            downleveledHeap.pop( f, c ); //stupid windows
            log("moving :%s from downleveled to current heap\n", c);
            heap.add( f, c );
        }
        log("numErased:%i\n", numErased );
    } while( numErased > 0 );
    if( heap.length() > 0 ) {
        log("didn't erase\n");
    }
    while( heap.length() > 0 ) {
        const char* c = heap.pop();
        log("%s\n", c );
    }
}
