#define N_IMPLEMENTS nSocket
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
//#include <asm/ioctls.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include "enkiduo/nsocket.h"
#include "kernel/ncmdprotonative.h"

nNebulaScriptClass(nSocket, "nthing");

//------------------------------------------------------------------------------
/**
 */
nSocket::nSocket():
    port( 5347 ),
    portRange( 20 ),
    serverSocketHandle(0),
    pingFreq( 1.0 )
{
    this->AddLogTag("socket");
    this->myAddressLength = sizeof( myAddress );
    this->otherAddressLength = sizeof( otherAddress );
}

void nSocket::Initialize(){
    nThing::Initialize();
    this->AddToTriggerList();
    this->AddCallback( "iam", 		this->fn, 	"receiveiam", true );
    this->AddCallback( "ping", 		this->fn, 	"receiveping" );
    this->AddCallback( "puts", 		this->fn, 	"receiveputs" );
    this->AddCallback( "script", 	this->fn, 	"receivescript" );
}

nSocket::~nSocket(){
    Disconnect();
    if( serverSocketHandle )
        shutdown( serverSocketHandle, SHUT_RDWR );
}
//------------------------------------------------------------------------------
/**
  Set the port on which I operate
*/
void nSocket::SetPort( int i ) {
    log("SetPort(%i)", i );
    this->port = i;
}
/**
    Initializes me to accept connections
 */
void nSocket::OpenServerSocket(){
    int ret;
    log("OpenServerSocket()" );
    serverSocketHandle = socket(AF_INET, SOCK_STREAM, 0 );                      n_assert( serverSocketHandle >= 0 && "failed to open serverSocket");
    ret = fcntl(serverSocketHandle, F_SETFL, O_NONBLOCK);                       n_assert( ret >= 0 && "failed to set serverSocket to non blocking");
    int bind_return = -1;
    myAddress.sin_family                        = AF_INET;
    myAddress.sin_addr.s_addr                   = htonl(INADDR_ANY);
    int tport;
    for( int i = 0; bind_return && i < portRange; i ++ ) {
        tport = port + i;
        log("trying to bind to port:%i", tport );
        myAddress.sin_port                      = htons(tport); //change to big endian
        bind_return                             = bind(serverSocketHandle,(struct sockaddr *) &myAddress,sizeof(myAddress));
    }
    if( bind_return )
        log("cannot bind serverSocket");
    else {
        ret = listen( serverSocketHandle, 10 );                                 n_assert( ret >= 0 && "failed to listen onServerSocketHandle (another serverlistening)");
        log("listening on port : %i", tport);
    }
}
/**
  Accept any pending connections
 */
void nSocket::CheckConnections(){			//see if I have anyone trying to connect
    int socketHandle = accept( serverSocketHandle,  (sockaddr*)&otherAddress, &otherAddressLength);
    if( socketHandle == -1 /*should be EAGAIN*/ )
        //log("no connections pending");
        ;
    else {
        socketInfo& i = sockets.Push();
        i.handle = socketHandle;
        i.ip = inet_ntoa( otherAddress.sin_addr ); 	//add its ip to my info
        log("received connection from : %s on port: %i", inet_ntoa(otherAddress.sin_addr), ntohs(otherAddress.sin_port) );
    }
}
/**
  Connects to another socket
  @param hostname address to connect too
 */
bool nSocket::Connect(const char* hostname, const char* hostType ){
    log("Connect( %s, %s)", hostname, hostType? hostType: "any" );

    hostInfo = gethostbyname(hostname); //get ip
    if( hostInfo == NULL ) log("unknown host: %s", hostname );

    //open socket
    int socketHandle		 = socket(AF_INET, SOCK_STREAM, 0);	
    n_assert( socketHandle >= 0 && "cannot create socket");

    //bind
    myAddress.sin_family 		= AF_INET;
    myAddress.sin_addr.s_addr 		= htonl(INADDR_ANY);
    myAddress.sin_port 		= htons(0);
    int bind_return 		= bind(socketHandle, (struct sockaddr *) &myAddress, sizeof(myAddress));	
    n_assert( bind_return >= 0 && "cannot bind socket");

    //setup serverAddress
    otherAddress.sin_family 		= hostInfo->h_addrtype;
    memcpy( (char *) &otherAddress.sin_addr.s_addr, hostInfo->h_addr_list[0], hostInfo->h_length);
    int connect_return;
    for( int i = 0; i < portRange; i++ ) {
        int tport = port + i;
        //log("trying to connect to port : %i", port+portOffset);
        otherAddress.sin_port 		= htons( tport );
        connect_return 		= connect( socketHandle , (struct sockaddr *) &otherAddress, sizeof(otherAddress) );
        if( connect_return == 0 ) {
            socketInfo& i 	= sockets.Push();
            i.handle 	= socketHandle;
            i.ip 	= hostname;
            log("Connected to %s on port: %i", hostname, tport );

            Send("iam", myName.Get() );
            return true;
        }
    } 
    log("Could not connect to server : %s", hostname );
    return false;

}

/************************************************************************
  Disconnects any active connections
 ************************************************************************/
void nSocket::Disconnect(){
    log("Disconnect()");
    for( int i = 0; i < sockets.Size(); i ++ )
        if( sockets[i].handle >= 0 )
            KillConnection( i );
}				
/************************************************************************
  Erases a dead connection
 ************************************************************************/
void nSocket::KillConnection( int handle ) {
    n_assert( sockets.Size() > 0 && handle < sockets.Size() && handle >= 0 );
    log("KillConnection(%i)", handle );
    shutdown( sockets[handle].handle, SHUT_RDWR );
    for( int i = 0; i < connectionDiedCallbacks.Size(); i++ ){
        Callback& c = connectionDiedCallbacks[i];
        if( c.script ) {
            nString script;
            script.PrintF("%s %i", c.func.Get(), handle );
            RUNSCRIPT( script.Get() );
        } else {
            //log("Dispatch: %i %s %s", fromIndex, tag, message );
            //create a new cmd object to send to it
            nCmd *cmd = c.cmd_proto->NewCmd();		n_assert( cmd->GetNumInArgs() == 1 && "connection died callback function must take exactly 1 parameter (int)");
            cmd->Rewind();
            cmd->In()->SetI( handle ); 	//attach the message to it (func should accept a string)
            c.node->Dispatch( cmd );	//dispatch the command
            c.cmd_proto->RelCmd( cmd );	//release the command
        }
    }
    sockets[handle].handle = -1;
}
/************************************************************************
  Sends a message 
  @param tag title of the message being sent, strlen < 16
  @param message string to send
 ************************************************************************/
void nSocket::Send( const char* t, const char* message ){
    bool sent = false;
    for( int i = 0; i < sockets.Size(); i ++ ){
        if( sockets[i].handle >= 0 ) {
            if( this->Send( i, t, message ) )
	sent = true;
        }
    }
    if (!sent )
        if( strcmp( t, "ping") && strcmp( t, "varupdates" ) ) //stop logging these useless messages
            log("failed to send message");
}
bool nSocket::Send( int i, const char* t, const char* message ) {
    if( strcmp( t, "ping" ) && strcmp( t, "varupdates") )
        log("S: %s[%i]---<%s>%s", sockets[i].name.Get(), i, t, firstLine(message) );
    if( sockets[i].handle == -1 ) {
        log("WARNING: trying to send to a closed connection");
        return false;
    }
    char *tag = new char[TAG_SIZE+1];
    char *mls = new char[MLS_SIZE+1];
    n_assert( tag && mls );
    int l = strlen( message );
    if( l == 0 ) {
        l = 1;
        message = "0";
    }
    strncpy( tag, t, TAG_SIZE );
    char format[10];
    int retval;
    retval = snprintf( format, 10, "%%0%ii", MLS_SIZE );
    //log("format: %s %i", format, retval );
    retval = snprintf( mls, MLS_SIZE+1, format, l );
    //log("mls: %s %i", mls, retval);

    //log("sending tag:*%s*", tag );
    int socketHandle = sockets[i].handle;
    int retvalTag = send( socketHandle, tag, 		TAG_SIZE, 	MSG_NOSIGNAL );
    int retvalMls = send( socketHandle, mls, 		MLS_SIZE, 	MSG_NOSIGNAL );
    int retvalMsg = send( socketHandle, message, 	l, 	MSG_NOSIGNAL );
    if( retvalTag != TAG_SIZE ) 
        log("WARNING: tag sent was wrong size %i != %i", retvalTag, TAG_SIZE );
    if( retvalMls != MLS_SIZE )
        log("WARNING: mls sent was wrong size %i != %i", retvalMls, MLS_SIZE );
    if( retvalMsg != l )
        log("WARNING: message sent was wrong size %i != %i", retvalMsg, l);

    delete[] tag;
    delete[] mls;

    if( retvalTag == -1 || retvalMls == -1 || retvalMsg == -1 ) {
        KillConnection( i );
        return false; 
    }
    else {
        sockets[i].lastUsed = time;
        return true;
    }
    //log("finished sending");
}
/************************************************************************
  Receives any pending messages
 ************************************************************************/
void nSocket::CheckMessages(){
    int retval;
    char tag[TAG_SIZE+1];
    tag [TAG_SIZE] = '\0';

    for( int i = 0; i < sockets.Size(); i++ ){
        int socketHandle = sockets[i].handle;
        if( socketHandle < 0 ) { //dont check closed connections
            //log("not checking socket: %i its closed", i );
            continue;
        }
        retval = recv( socketHandle, tag, TAG_SIZE, MSG_DONTWAIT );
        if( retval > 0 ) { //They have sent us a message
            //log("Received tag: %s", tag );
            if( retval != TAG_SIZE )
	log ("WARNING: tag sent was wrong size" );

            char mls[MLS_SIZE+1];
            mls[MLS_SIZE] = '\0';
            retval = recv( socketHandle, mls, MLS_SIZE, MSG_WAITALL );
            //log("received mls: %s", mls );
            if( retval != MLS_SIZE )
	log( "WARNING: mls sent was wrong size" );
            int l= atoi( mls );

            char message[l+1];
            message[l] = '\0';
            retval = recv( socketHandle, message, l, MSG_WAITALL );
            //log("Received message: %s", message);
            if( retval != l )
	log("WARNING: message sent was wrong size" );

            char trimmedTag[16];
            sscanf( tag, "%s", trimmedTag );
            this->Receive( i, trimmedTag, message );

            sockets[i].lastUsed = time;
        }
    }
}

/************************************************************************
  Called when a message is received
  searches through library of active callbacks - making the appropriate callback
 ************************************************************************/
void nSocket::Receive( int fromIndex, const char* tag, const char* message ) {
    //find matching callback
    if( strcmp( tag, "ping" ) ) {
        log	("R: %s[%i]---<%s>%s", sockets[fromIndex].name.Get(), fromIndex, tag, firstLine(message));
    }
    int found = 0;
    for( int i = 0; i < callbacks.Size(); i ++ ){
        if( strcmp( callbacks[i].tag, tag ) == 0 ){
            //if( found > 0 )
	//log("found multiple callbacks for %s", tag );
            found++;
            Callback& c = callbacks[i];
            if( c.script ) {
	nString script;
	if( c.from == false )
	    script.PrintF("%s {%s}", c.func.Get(), message );
	else
	    script.PrintF("%s %i {%s}", c.func.Get(), fromIndex, message );
	RUNSCRIPT( script.Get() );
            } else {
	//if( strcmp( tag, "ping" ) )
	//log("Dispatch: %i %s %s", fromIndex, tag, firstLine(message) );
	//create a new cmd object to send to it
	nCmd *cmd = c.cmd_proto->NewCmd();
	cmd->Rewind();
	if( c.from == false ) 
	    n_assert( cmd->GetNumInArgs() == 1 && "callback function must take exactly 1 parameter (string)");
	if( c.from == true ) 
	    n_assert( cmd->GetNumInArgs() == 2 && "callback function must take exactly 2 parameter (int, string)");
	if( c.from )
	    cmd->In()->SetI( fromIndex );
	cmd->In()->SetS( message ); 	//attach the message to it (func should accept a string)
	c.node->Dispatch( cmd );	//dispatch the command
	c.cmd_proto->RelCmd( cmd );	//release the command
            }
        }
    }
    if( found == 0 ){
        log("Failed to find callback for message : <%s>%s", tag, firstLine(message) );
    }
}
/************************************************************************
  Sets up a new callback
  @param tag tag that triggers the callback
  @param nodeName path to the node to call
  @param funcName name of the function to call - must take a single string
 ************************************************************************/
void nSocket::AddCallback( const char* tag, const char* nodeName, const char* funcName, bool from ){
    log("nSocket::AddCallback(%s,%s,%s)", tag, nodeName, funcName );
    if( strcmp( nodeName, "script") == 0 ) {
        Callback c;
        c.script = true;
        //c.tag = tag;
        c.tag = new char[strlen(tag)+1];
        strcpy( c.tag, tag );
        c.func = funcName;
        c.from = from;
        callbacks.Append( c );
        log("Added script callback %s->%s", c.tag, c.func.Get());
    } else {
        nRoot* node = kernelServer->Lookup( nodeName );
        if( node ) {
            nClass *cl = node->GetClass();
            nCmdProtoNative *cproto = cl->FindNativeCmdByName(funcName); //returns the prototype
            //the cmd_proto is an interface into that function
            if( cproto ) {
	Callback c;
	c.tag = new char[strlen(tag)+1];
	strcpy( c.tag, tag );
	//c.tag = tag;
	c.func = funcName;
	c.node = node;
	c.cmd_proto = cproto;
	c.from = from;
	callbacks.Append( c );
	log("Added callback %s->%s.%s %i", tag, nodeName, funcName, from );
            }
            else {
	log("cannot find function: %s(%s)::%s", node->GetName(), node->GetClass()->GetName(), funcName );
            }
        }
        else {
            log("cannot find node: %s", node );
        }
    }
}

/************************************************************************
  Sets ups a callback to receieve a connection died event
  @param tag tag that triggers the callback
  @param nodeName path to the node to call
  @param funcName name of the function to call - must take a single string
 ************************************************************************/
void nSocket::AddConnectionDiedCallback( const char* nodeName, const char* funcName ){
    log("nSocket::AddConnectionDiedCallback(%s,%s)", nodeName, funcName );
    if( strcmp( nodeName, "script") == 0 ) {
        Callback c;
        c.script = true;
        c.func = funcName;
        connectionDiedCallbacks.Append( c );
        log("Added script connection died callback %s", c.func.Get());
    } else {
        nRoot* node = kernelServer->Lookup( nodeName );
        if( node ) {
            nClass *cl = node->GetClass();
            nCmdProtoNative *cproto = cl->FindNativeCmdByName(funcName); //returns the prototype
            //the cmd_proto is an interface into that function
            if( cproto ) {
	Callback c;
	c.func = funcName;
	c.node = node;
	c.cmd_proto = cproto;
	connectionDiedCallbacks.Append( c );
            }
            else {
	log("cannot find function: %s(%s)::%s", node->GetName(), node->GetClass()->GetName(), funcName );
            }
        }
        else {
            log("cannot find node: %s", node );
        }
    }
}

/************************************************************************
  checks for pending connections / messages
 ************************************************************************/
void nSocket::Tick( float time, float dtime ) {
    nThing::Tick( time, dtime );
    this->time = time;
    if( this->serverSocketHandle ) {
        this->CheckConnections();
        this->PingSockets( time );
    }
    if( sockets.Size() > 0 ) {
        this->CheckMessages();
    }
}
/************************************************************************
  Ping all sockets that haven't seen recent activity
 ************************************************************************/
void nSocket::PingSockets( float time ) {
    for( int i = 0; i < sockets.Size(); i ++ ){
        if( sockets[i].handle >= 0 ) {
            if( sockets[i].lastUsed + pingFreq < time ){
	nString s;
	s.PrintF( "%f", time );
	Send( i, "ping", s.Get() );
            }
        }
    }
}
/************************************************************************
  Basic callbacks
 ************************************************************************/
void nSocket::ReceiveIAM( int from, const char* message ) {
    log("ReceiveIAM( %i %s )", from, message );
    if ( sockets[from].name == "" ) { //first time i got their name
        sockets[from].name = message;
        Send( from, "iam", myName.Get() );
    }
}
void nSocket::ReceivePing( const char* message ) {
}
void nSocket::ReceivePuts( const char* message ) {
    log("%s", message );
}
/************************************************************************
  Messages calls for script command
 ************************************************************************/
void nSocket::ReceiveScript( const char* message ) {
    const char* output = 0;
    bool success = ((nScriptServer*) (kernelServer->Lookup("/sys/servers/script")))->Run( message, output );
    char* outputLocal = 0;
    if( output ){
        outputLocal = new char[strlen(output)+1];
        strcpy( outputLocal, output );
    }
    //log("Output: %s", outputLocal );
    if( success )
        this->Send("puts", outputLocal );
    else
        this->Send("puts", "script call failed");
    if( outputLocal )
        delete[] outputLocal;
}
