#define N_IMPLEMENTS nVar
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nvar.h"
#include "kernel/npersistserver.h"
#include "enkiduo/nent.h"
#define command(NAME) static void NAME( void* slf, nCmd* cmd )
#define self ((nVar*)slf)
#define ins cmd->In()->GetS
#define inf cmd->In()->GetF
#define ini cmd->In()->GetI
#define inb cmd->In()->GetB
#define outs cmd->Out()->SetS
#define outf cmd->Out()->SetF
#define outi cmd->Out()->SetI
#define outb cmd->Out()->SetB

command( gettype ) {
const char* VarTypeStrings[9] = { 
    "void",
    "int",
    "float",
    "vector3",
    "quaternion",
    "string",
    "ref",
    "rc",
    "object"
};
    outs( VarTypeStrings[self->GetType()] );
}
command( seti ){    
    self->SetI( ini() ); 
}
command( setf ){    
    self->SetF( inf() ); 
}
command( setv ){
    float x = inf();
    float y = inf();
    float z = inf();
    self->SetV( x,y,z );
}
command( getv ) {
    vector3 v = self->GetV();
    outf( v.x );
    outf( v.y );
    outf( v.z );
}
command( setq ){
    float x = inf();
    float y = inf();
    float z = inf();
    float w = inf();
    self->SetQ( x,y,z,w );
}
command( sets ){
    self->SetS( ins() );
}
command( gets ){
    outs( self->GetS().Get() );
}
command( setref ){
    self->SetRef( ins() );
}
command( getref ) {
    nEnt* t = self->GetRef();
    if( t ) 
        outs( t->fn );
    else
        outs( "none" );
}
command( setbucketedge ){
    int i = ini();
    float f = inf();
    self->SetBucketEdge( i, f );
}
command( setinputrange ){
    float min = inf();
    float max = inf();
    int samples = ini();
    self->SetInputRange( min,max,samples );
}
command( setoutputrange ){
    float min = inf();
    float max = inf();
    self->SetOutputRange( min, max );
}
command( getbucketedge ){
    outf( self->GetBucketEdge( ini() ) );
}
command( getinputrange ){
    float min, max;
    int samples;
    self->GetInputRange( min, max, samples );
    outf( min );
    outf( max );
    outi( samples );
}
command( getoutputrange ){
    float min, max;
    self->GetOutputRange( min, max );
    outf( min );
    outf( max );
}
command( setcategoryi ) {
    self->SetCategory( (VARCAT) ini() );
}
command( getcategoryi ) {
    outi( self->GetCategory() );
}
command( getcounter ) {
    outi( self->GetCounter() );
}
command( setcategory ) {
    const char* s = ins();
    if( strcmp( s, "server" ) == 0 )
        self->SetCategory( VARCAT_SERVER );
    else if( strcmp( s, "client" ) == 0 )
        self->SetCategory( VARCAT_CLIENT );
    else
        n_printf("unknown category: %s\n", s );
}
command( getcategory ) {
    int i = self->GetCategory();
    switch( i ) {
        case VARCAT_VOID:
            outs( "void" );
            break;
        case VARCAT_SERVER:
            outs( "server" );
            break;
        case VARCAT_CLIENT:
            outs( "client" );
            break;
        default:
            outs( "unknown" );
            break;
    }
}
command( getlastchecked) { outf( self->GetLastChecked()); }
    
void
n_initcmds(nClass* clazz)
{
    clazz->BeginCmds();
    clazz->AddCmd( "s_gettype_v",           'GT',   gettype );
    clazz->AddCmd( "v_seti_i",              'STI',  seti );
    clazz->AddCmd( "v_setf_f",              'STF',  setf );
    clazz->AddCmd( "v_setv_fff",            'STV3', setv );
    clazz->AddCmd( "fff_getv_v",            'GTV3', getv );
    clazz->AddCmd( "v_setq_ffff",           'STQ',  setq );
    clazz->AddCmd( "v_sets_s",              'STS',  sets );
    clazz->AddCmd( "s_gets_v",              'GTS',  gets );
    clazz->AddCmd( "v_setref_s",            'STRF', setref );
    clazz->AddCmd( "s_getref_v",            'GTRF', getref );
    clazz->AddCmd( "v_setbucketedge_if",    'SBEG', setbucketedge );
    clazz->AddCmd( "v_setinputrange_ffi",   'SIRG', setinputrange );
    clazz->AddCmd( "v_setoutputrange_ff",   'SORG', setoutputrange );
    clazz->AddCmd( "i_getcounter_v",        'GCNT', getcounter );

    clazz->AddCmd( "f_getbucketedge_i",     'GBEG', getbucketedge );
    clazz->AddCmd( "ffi_getinputrange_v",   'GIRG', getinputrange );
    clazz->AddCmd( "ff_getoutputrange_v",   'GORG', getoutputrange );

    clazz->AddCmd( "v_setcategoryi_i", 	'SCI', setcategoryi );
    clazz->AddCmd( "i_getcategoryi_v", 	'GCI', getcategoryi );
    clazz->AddCmd( "v_setcategory_s", 	'SC', setcategory );
    clazz->AddCmd( "s_getcategory_v", 	'GC', getcategory );
    clazz->AddCmd( "f_getlastchecked_v", 	'GLC', getlastchecked );
    clazz->EndCmds();
}

bool
nVar::SaveCmds(nPersistServer* ps)
{
	int i;
    if (nRoot::SaveCmds(ps))
    {
        nCmd* cmd;
        float min, max;
        int samples;
        switch( this->type ){
            case VARTYPE_INT:
                cmd = ps->GetCmd( this, 'STI' );
                cmd->In()->SetI( this->intVal );
                ps->PutCmd( cmd );
                break;
            case VARTYPE_FLOAT:
                cmd = ps->GetCmd( this, 'STF' );
                cmd->In()->SetF( this->floatVal );
                ps->PutCmd( cmd );
                break;
            case VARTYPE_VECTOR3:
                cmd = ps->GetCmd( this, 'STV3' );
                cmd->In()->SetF( this->vectorVal->x );
                cmd->In()->SetF( this->vectorVal->y );
                cmd->In()->SetF( this->vectorVal->z );
                ps->PutCmd( cmd );
                break;
            case VARTYPE_QUATERNION:
                cmd = ps->GetCmd( this, 'STQ' );
                cmd->In()->SetF( this->quatVal->x );
                cmd->In()->SetF( this->quatVal->y );
                cmd->In()->SetF( this->quatVal->z );
                cmd->In()->SetF( this->quatVal->w );
                ps->PutCmd( cmd );
                break;
            case VARTYPE_STRING:
                cmd = ps->GetCmd( this, 'STS' );
                cmd->In()->SetS( this->stringVal->Get() );
                ps->PutCmd( cmd );
                break;
            case VARTYPE_REF:
                //FIXME-make sure i dont need this if( this->isvalid() ) {
                    cmd = ps->GetCmd( this, 'STRF' );
                    cmd->In()->SetS( this->GetRefPath() );
                    ps->PutCmd( cmd );
                //}
                break;
            case VARTYPE_RC:
                this->GetInputRange( min, max, samples );
                cmd = ps->GetCmd( this, 'SIRG' );
                cmd->In()->SetF( min );
                cmd->In()->SetF( max );
                cmd->In()->SetI( samples );
                ps->PutCmd( cmd );

                this->GetOutputRange( min, max );
                cmd = ps->GetCmd( this, 'SORG' );
                cmd->In()->SetF( min );
                cmd->In()->SetF( max );
                ps->PutCmd( cmd );

                for( i = 0; i < this->GetBucketCount(); i ++ ){
                    cmd = ps->GetCmd( this, 'SBEG' );
                    cmd->In()->SetI( i );
                    cmd->In()->SetF( this->GetBucketEdge(i) );
                    ps->PutCmd( cmd );
                }
                break;
            default:
                char name[1024];
                this->GetFullName( name, 1024 );
                //n_printf("cloning / saving crappy nvar %s with type : %i\n", name, this->type );
                break;
        }
        cmd = ps->GetCmd( this, 'SCI' );
        cmd->In()->SetI( this->category );
        ps->PutCmd( cmd );
        return true;
    }
    return false;
}

