#define N_IMPLEMENTS nCaseInjector
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ga/ncaseinjector.h"
#include "ga/ngenome.h"

nNebulaScriptClass(nCaseInjector, "naspect");

//------------------------------------------------------------------------------
/**
*/
nCaseInjector::nCaseInjector()
{
}

void nCaseInjector::Initialize(){
    nAspect::Initialize();
    //this->AddToTriggerList();
    this->AddLogTag("ga caseinjector");
}

void nCaseInjector::InitVars(){
    nAspect::InitVars();
    generation.Connect	( ent, "generation" );
    injectionFrequency.Connect	( ent, "injectionfrequency" );
    injectionFrequency.DefaultI	( 1 );
    injectionAmount.Connect	( ent, "injectionamount" );
    injectionAmount.DefaultI	( 1 );
    generationLastInjected.Connect( ent, "generationlastinjected" );
    generationLastInjected.DefaultI( -1 );
    popsize.Connect	( ent, "popsize" );
    numgens.Connect	( ent, "numgens" );
    numInjected.Connect ( ent, "numinjected" );
    numInjected.DefaultI ( 0 );
}

nCaseInjector::~nCaseInjector(){
}
//------------------------------------------------------------------------------

void nCaseInjector::Inject( const char* s ){ 
    classVar( ngenome, genomeClass );
    log("nCaseInjector::Inject(%s) gen=%i", s, generation.GetI());
    int i;
    int li = generationLastInjected.GetI();
    int gen = generation.GetI();
    if( gen == numgens.GetI() ) {
        log("GA has finished - not injecting");
        return;
    }
    if( li < 0 || gen - li > injectionFrequency.GetI() ) {
        log("Injecting");
        //set1 -> identify who to inject
        float maxFit;
        //go through population - find highest fitness individual
        nGenome* bestGenome, *g;
        nRoot* root = ent->Find("ga");
        float fitness; 
        bool first = true;
        nRoot* r;
        for( r = root->GetHead(); r; r = r->GetSucc() ) {
            n_assert( r->IsA( genomeClass ) && "non-ngenome classes as children of /ga/ga");
            g = (nGenome*) r;
            fitness = g->GetFitness();
            log("Genome:%s has fitness: %f", g->fn, fitness );
            if( first || fitness > maxFit ) {
	first = false;
	maxFit = fitness;
	bestGenome = g;
            }
        }
        log("Genome: %s has highest fitness: %f", bestGenome->fn, fitness );
        /**
          Okay now we have the best individual..now we need to figure out who to inject
          okay so go through our casebase population...set similarity metrics equal to hamming distance something or another? then do a roulette wheel selection thing on the entire thing
        */
        nArray<Case> casebase;
        root = kernelServer->Lookup("/casebase/genomes");
        if( root == 0 ){
            log("no casebase found");
            return;
        }
        for( r = root->GetHead(); r; r = r->GetSucc() ){
            n_assert( r->IsA( genomeClass ) );
            Case& c = casebase.Push();
            c.similarity = -1;
            c.genome = (nGenome*) r;

            if( c.genome->IsCast() == false ) {
				const char* castTo = "/ga/ga/genome0";
				log("casting %s to %s", c.genome->fn, castTo );
				c.genome->CastTo( castTo ); //case all the casebase-genoms into the current encoding
            }
        }
        if( casebase.Size() == 0 ) return;
        //compare hamming ditsances between the casebase and the best individual
        int min, max;
        first = true;
        for( i = 0; i < casebase; i ++ ) {
            casebase[i].similarity = bestGenome->HammingDistanceTo( casebase[i].genome );
            int v = casebase[i].similarity;
            if( first ) {
	min = v;
	max = v;
	first = false;
            } else {
	if ( v < min )
	    min = v;
	if( v > max )
	    max = v;
            }
            log("casebase[%i].similarity = %i", i, casebase[i].similarity );
        }
        //build fitness values for everything in the casebase
        float svSum = 0.0f;
        int range = max - min;
        if( range == 0 ) range = 1;
        for( i = 0; i < casebase; i ++ ) {
            int v = casebase[i].similarity;
            float sv = (v - min) / range;
            sv = 1.0f - sv;
            sv = sv * .9f + .1f;
            casebase[i].selectionValue = sv;
            casebase[i].selectionValuePosition = svSum;
            svSum += sv;
            log("casebase[%i].selectionValue = %f", i, casebase[i].selectionValue );
        }
        //now...determine who to inject
        //to do this..simple roulette wheel selection
        nArray<int> toInject;
        for( i = 0; i < injectionAmount.GetI() ; i++ ) {
            float choice = randF( 0, svSum );
            int j;
            for( j = 0; j + 1 < casebase; j ++ )
	if( casebase[j+1].selectionValuePosition > choice )
	    break;
            log( "choose : %i", j );
            toInject.Append( j );
        }

        //okay now we have to actually replace them - go through the genome list...replace thos eon the bottom of the list
        //go through population - find highest fitness individual
        root = ent->Find("ga");
        int rank;
        first = true;
        for( r = root->GetHead(); r && i < injectionAmount.GetI(); r = r->GetSucc() ) {
            n_assert( r->IsA( genomeClass ) && "non-ngenome classes as children of /ga/ga");
            g = (nGenome*) r;
            rank = g->GetRank();
            log("Genome:%s has rank: %i", g->fn, rank);
            if( rank != -1 ) { //uneavluated / bad individual
	if( rank >= popsize.GetI() - injectionAmount.GetI() ) {
	    log("Replacing genome:%s with casebase[%i]", g->fn, toInject[i] );
	    numInjected.SetI( numInjected.GetI() + 1 );
	    *g = *casebase[toInject[i]].genome;
	    g->SetInjected();
	    i++;
	}
            }
            //now what? check to see who has the highest rank..replace them =)
        }
        if( i != toInject.Size() )
            log("ERROR: only Injected %i individuals instead of %i - likely had bad individuals at time of nextgen event", i, toInject.Size() );
    }
}
