#define N_IMPLEMENTS nGA
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "ga/nga.h"
#include "ga/ngenome.h"
#include "enkiduo/neventserver.h"
#include "tnl/gaconnection.h"
#include "tnl/ganetinterface.h"
#include "ga/ngeneint.h"
#include "ga/ngenefloat.h"
#include "enkiduo/nreleaseserver.h"
//-----------------------------------------------------------------------------//
nNebulaScriptClass(nGA, "nfsm");
//------------------------------------------------------------------------------
nGA::nGA():
ref_event( kernelServer, this ),
ref_release( kernelServer, this ){
    stateNames.Append("FRESH");

    stateNames.Append("MASTER_WAITING_FOR_RUN");
    stateNames.Append("MASTER_NO_SLAVES");
    stateNames.Append("MASTER");
    stateNames.Append("MASTER_START_EVALUATION");
    stateNames.Append("MASTER_CREATE_INDIVIDUAL");

    stateNames.Append("SLAVE_WAITING_FOR_MASTER");
    stateNames.Append("SLAVE_WAITING_FOR_WORLD");
    stateNames.Append("SLAVE_WAITING_FOR_INDIVIDUAL");
    stateNames.Append("SLAVE_EVALUATING");

    ref_event                           = "/sys/servers/event";
    ref_release                         = "/sys/servers/release";
    this->beginRunning = false;
    this->connection = 0;

    genomeEvaluating = 0;
    slaveGenome = 0;
    slaveClassName = "nga";
    lastPingTime = -1.0f;
    pingFreq = 1.0f;
    localSlaveInitialized = false;
    maximumSimilarChildrenToCreate = 10;
}
//------------------------------------------------------------------------------
void nGA::Initialize(){
    nFSM::Initialize();
}
//------------------------------------------------------------------------------
void nGA::InitVars(){
    nFSM::InitVars();
    popsize.Connect( ent, "popsize" );
    popsize.DefaultI( 20 );
    numgens.Connect( ent, "numgens" );
    numgens.DefaultI( 100 );
    generation.Connect( ent, "generation" );
    generation.DefaultI( -1 );
    minFitness.Connect( ent, "minFitness" );
    minFitness.DefaultF( 0.0f );
    maxFitness.Connect( ent, "maxFitness" );
    maxFitness.DefaultF( 0.0f );
    avgFitness.Connect( ent, "avgFitness" );
    avgFitness.DefaultF( 0.0f );
    maxEvaluations.Connect( ent, "maxEvaluations" );
    maxEvaluations.DefaultI( 20 );
    evaluationsDone.Connect( ent, "evaluationsDone" );
    evaluationsDone.DefaultI( 0 );
    numberToKeepUnevaluated.Connect( ent, "numberToKeepUnevaluated" );
    numberToKeepUnevaluated.DefaultI( 1 );
    //generational.Connect( ent, "generational" );
    //generational.DefaultI( 1 );
    //numEvaluations.Connect( ent, "numEvaluations" );
    //numEvaluations.DefaultI( 0 );
    ip.Connect( ent, "ip" );
    ip.DefaultS( "127.0.0.1" );
    port.Connect( ent, "port" );
    port.DefaultI( this->defaultPort() );

    nEnt* gameStateRoot = dynamic_cast<nEnt*>( kernelServer->Lookup("/game/state"));
    n_assert( gameStateRoot && "/game/state is not found or is not an nent");
    gamestate.Connect( gameStateRoot, "xml" );
    gamestate.DefaultS("");

}
//------------------------------------------------------------------------------
nGA::~nGA(){
}

//-----------------------------------------------------------------------------//
/**
 * i am becoming a gamaster
 */
void nGA::OpenServer( const char* ip ){                                        //setup this ga as a gamaster
    n_printf("nGA(%s).OpenServer(%s)\n", this->fn, ip);
    if( this->connection == 0 ) {
        this->connection = new GAConnection();
        this->connection->SetGA( this );
    }
    this->connection->openServer( ip );
    //this->SetState(MASTER_NO_SLAVES);
}
//-----------------------------------------------------------------------------//
void nGA::ConnectToServer( const char* ip ){                                   //connect to a remote GA Master
    n_printf("nGA(%s).ConnectToServer(%s)\n", this->fn, ip);
    if( this->connection == 0 ) {
        this->connection = new GAConnection();
        this->connection->SetGA( this );
    }
    this->connection->connectToServer( ip );
    this->SetState( SLAVE_WAITING_FOR_WORLD );
}
//-----------------------------------------------------------------------------//
void nGA::ConnectToLocalServer( const char* ip, GAConnection* connection ){    //connect to a local  GA Master
    n_printf("nGA(%s).ConnectToLocalServer(%s,%p)\n", this->fn, ip, connection);
    if( this->connection == 0 ) {
        this->connection = new GAConnection();
        this->connection->SetGA( this );
    }
    this->connection->connectToLocalServer( ip, connection );
    this->SetState( SLAVE_WAITING_FOR_WORLD );
}
//-----------------------------------------------------------------------------//
void nGA::InitializeLocalSlave(){
    if( localSlaveInitialized == true ) return;
    localSlaveInitialized = true;
    //n_printf("nGA(%s)::InitializeLocalSlave()\n", this->fn);
    //n_printf("ip:%s port:%i\n", ip.GetS().Get(), port.GetI() );
    nString address;
    address.PrintF("%s:%i", ip.GetS().Get(), port.GetI() );
    //n_printf("my ip:%s\n", address.Get());
    this->OpenServer(address.Get());                                   //first off i should become a server
    nString slaveName = nString(this->GetName()) + "local";
    nThing* parent = dynamic_cast<nThing*>( this->GetParent());
    n_assert( parent && "nGA has bad parent");
    nGA* slave = dynamic_cast<nGA*>(parent->CreateChild(slaveClassName.Get(), slaveName.Get())); //spawn off a lsave
    n_assert( slave && "failed to create my local slave for some reason"); 
    slave->ConnectToLocalServer( address.Get(), this->connection ); 
    this->InitializeSlave( slave );
}
//------------------------------------------------------------------------------
void nGA::Tick( float time, float dtime ){
    this->time = time;
    nFSM::Tick( time, dtime );

    if ( connection )
        connection->Tick(time, dtime ); 
    /*for( int i = 0; i < slaveConnections; i ++ ) {
        slaveConnections[i]->Tick( time, dtime );
    }*/
}
//-----------------------------------------------------------------------------//
bool nGA::States( StateMachineEvent event, uint state, float time, float dtime ){
    //n_printf("nGA::States(%i,%f,%f)\n", state, time, dtime);
    /*if (this->connection && event == EVENT_ENTER) {
        n_printf("cascading setstate:%i\n", state);
        this->connection->rpcSetState( state );
    }*/
    BeginStateMachine
        //-----------------------------------------------------------------------------//
        State(FRESH)
            OnEnter
                //n_printf("nGA(%s).FRESH\n", this->fn);
            OnTick
                if( connection && connection->state == SLAVE_WAITING_FOR_WORLD )
                    this->SetState( SLAVE_WAITING_FOR_WORLD );
        //-----------------------------------------------------------------------------//
        State(MASTER_WAITING_FOR_RUN)
            OnEnter
                //n_printf("nGA(%s).MASTER_WAITING_FOR_RUN\n", this->fn);
            OnTick
                this->PingForSlaves(time);
                if ( time > 1.0 && this->beginRunning ) {
                    this->beginRunning = false;
                    this->SetState(MASTER_NO_SLAVES);
                }
        //-----------------------------------------------------------------------------//
        State(MASTER_NO_SLAVES)
            OnEnter
                //n_printf("nGA(%s).MASTER_NO_SLAVES\n", this->fn);
            OnTick
                this->PingForSlaves(time);
                if ( this->slaveConnections.Size() > 0 )
                    this->SetState( MASTER );
        //-----------------------------------------------------------------------------//
        State(MASTER)
            OnEnter
                //n_printf("nGA(%s).MASTER\n", this->fn);
            OnTick
                this->CheckPopSize();
                //n_printf("nGA(%s).MASTER\n", str(this));
                if( this->GetHead() == 0 )
                    n_printf("no children()?\n");
                this->PingForSlaves(time);
                int numberUnevaluated = 0;
                int totalGenomes = 0;
                foreachchild( nGenome, genome, this ) {
                    totalGenomes++;
                    if ( genome->GetStateI() == UNEVALUATED )
                        numberUnevaluated++;
                }
                //n_printf("totalGenomes:%i numberUnevaluated:%i\n", totalGenomes, numberUnevaluated);
                GAConnection* slaveConnection = 0;
                for( int i = 0; slaveConnection == 0 && i < this->slaveConnections; i ++ ) {
                    //n_printf("slaveConnection[%i](%i).state = %s\n", i, slaveConnections[i]->getId(), stateNames[slaveConnections[i]->state] );
                    if ( slaveConnections[i]->state == SLAVE_WAITING_FOR_INDIVIDUAL ){
                        slaveConnection = slaveConnections[i];
                    }
                }
                if( (similarChildrenCreated > maximumSimilarChildrenToCreate) || (numberUnevaluated == 0 && evaluationsDone.GetI() > maxEvaluations.GetI() )) {
                    this->OnRunDone();
                    evaluationsDone.SetI(0);
                    this->SetState(MASTER_WAITING_FOR_RUN);
                }
                else if( numberUnevaluated >= numberToKeepUnevaluated.GetI() && slaveConnection ) 
                    this->SetState(MASTER_START_EVALUATION);
                else if( numberUnevaluated < numberToKeepUnevaluated.GetI() && evaluationsDone.GetI() < maxEvaluations.GetI() )  {
                    //n_printf("numberUnevaluated:%i\n", numberUnevaluated);
                    this->SetState(MASTER_CREATE_INDIVIDUAL);
                }
                else
                    ; //just waiting for evaluations to finish
                
        //-----------------------------------------------------------------------------//
        State(MASTER_START_EVALUATION)
            OnEnter
                //n_printf("nGA(%s).MASTER_START_EVALUATION\n", this->fn);
            OnTick
                //if ( evaluationsDone.GetI() % 10 == 0 )
                this->NormalizeFitnesses();
                GAConnection* slaveConnection = 0;
                for( int i = 0; slaveConnection == 0 && i < this->slaveConnections; i ++ ) {
                    if ( slaveConnections[i]->state == SLAVE_WAITING_FOR_INDIVIDUAL ){
                        slaveConnection = slaveConnections[i];
                    }
                }
                if ( slaveConnection != 0 )  { //if we have someone to assign to this individual
                    foreachchild( nGenome, genome, this ) {
                        if ( genome->GetStateI() == UNEVALUATED ) {
                            similarChildrenCreated = 0;
                            genome->SetState(BEING_EVALUATED);
                            //this->genomeEvaluating = genome;
                            TNL::Vector<bool> bitString = genome->GetTNLBitString();
                            //n_printf("bitString:");
                            //for( int i = 0; i < bitString.size(); i ++ )
                                //n_printf("%i", bitString[i]);
                            //n_printf("\n");
                            slaveConnection->genomeEvaluating = genome;
                            slaveConnection->rpcStartEvaluation( genome->GetTNLBitString() );
                            slaveConnection->state = SLAVE_EVALUATING;
                            //this->StartEvaluation( genome );
                            //this->SetState(EVALUATING);
                            break;
                        }
                    }
                }
                this->SetState(MASTER);
        //-----------------------------------------------------------------------------//
        State(MASTER_CREATE_INDIVIDUAL)
            OnEnter
                //n_printf("nGA(%s).MASTER_CREATE_INDIVIDUAL\n", this->fn);
            OnTick
                //nArray<nGenome*> genomes                                        = this->GetSortedGenomes();
                similarChildrenCreated += 1;
                nGenome* lowestFitnessGenome;
                nGenome* secondLowestFitnessGenome;
                //nGenome* lowestFitnessGenome                                    = genomes[-1];
                //nGenome* secondLowestFitnessGenome                              = genomes[-2];
                lowestFitnessGenome                                             = this->SelectBadGenome();
                if( lowestFitnessGenome ) {
                    int i = 0;
                    do {
                        secondLowestFitnessGenome                                   = this->SelectBadGenome();
                        //n_printf("lowestFitnessGenome:%p secondLowestFitnessGenome:%p\n", lowestFitnessGenome, secondLowestFitnessGenome);
                        i++;
                        if ( i > 10 )  {
                            secondLowestFitnessGenome = 0;
                            break;
                        }
                    } while ( lowestFitnessGenome == secondLowestFitnessGenome );

                    if( secondLowestFitnessGenome ) {
                        //n_printf("lowestFitnessGenome: %s secondLowestFitnessGenome: %s\n", str(lowestFitnessGenome), str(secondLowestFitnessGenome) );
                        lowestFitnessGenome->                                           SetState(UNEVALUATED);
                        lowestFitnessGenome->                                           Tick(0,0);
                        secondLowestFitnessGenome->                                     SetState(UNEVALUATED);
                        secondLowestFitnessGenome->                                     Tick(0,0);
                        nGenome* parent1                                                = this->SelectGenome();
                        nGenome* parent2                                                = this->SelectGenome();
                        if ( parent1 == 0 || parent2 == 0 )  {
                            n_printf("warning create individual failed -> could not select both parents: %p %p\n", parent1, parent2);
                        } else {
                            //n_printf("parent1:%s parent2:%s\n", parent1->fn, parent2->fn);
                            lowestFitnessGenome->                                   CopyFrom( parent1 );
                            secondLowestFitnessGenome->                             CopyFrom( parent2 );
                            lowestFitnessGenome->                                   CrossOverWith( secondLowestFitnessGenome );
                            //evaluationsDone.SetI( evaluationsDone.GetI() + 2 );
                            lowestFitnessGenome->Tick(0,0);
                            secondLowestFitnessGenome->Tick(0,0);
                        }
                    }
                } else
                    ; //some stupid something that sends it here right away
                this->SetState(MASTER);
        //-----------------------------------------------------------------------------//
        State(SLAVE_WAITING_FOR_MASTER)
            OnEnter
                //n_printf("nGA(%s).SLAVE_WAITING_FOR_MASTER\n", this->fn);
                if( this->connection == 0 ) {
                    nString address;
                    address.PrintF("IP:%s:%i", ip.GetS().Get(), port.GetI() );
                    this->OpenServer(address.Get());                                   //first off i should become a server
                }
                //this->connection->rpcSetState( SLAVE_WAITING_FOR_MASTER );
        //-----------------------------------------------------------------------------//
        State(SLAVE_WAITING_FOR_WORLD)
            OnEnter
                //n_printf("nGA(%s).SLAVE_WAITING_FOR_WORLD\n", this->fn);
                this->connection->rpcSetState( SLAVE_WAITING_FOR_WORLD );
                this->SetState( SLAVE_WAITING_FOR_INDIVIDUAL );
        //-----------------------------------------------------------------------------//
        State(SLAVE_WAITING_FOR_INDIVIDUAL)
            OnEnter
                //n_printf("nGA(%s).SLAVE_WAITING_FOR_INDIVIDUAL\n", this->fn);
                this->connection->rpcSetState( SLAVE_WAITING_FOR_INDIVIDUAL );
        //-----------------------------------------------------------------------------//
        State(SLAVE_EVALUATING)
            OnEnter
                //n_printf("nGA(%s).EVALUATING\n", this->fn);
            OnTick
                if( this->EvaluationFinished() ) {
                    //save its fitness 
                    float fitness = this->GetFitness();
                    this->connection->rpcSetFitness( fitness );
                    //this->slaveGenome->EvaluationFinished( fitness );
                    //pull the stop evaluation hook
                    this->StopEvaluation( this->slaveGenome );
                    //this->SetState(IDLE);
                    //n_printf("nGA(%s).slave has finished evaluating fitness:%f\n", str(this), fitness );
                    this->SetState(SLAVE_WAITING_FOR_INDIVIDUAL);
                } else {
                    //n_printf("slave(%s) not done evaluating\n", str(this));
                }
        //-----------------------------------------------------------------------------//
    EndStateMachine
}
//-----------------------------------------------------------------------------//
void nGA::InitializeWorld(){
    //n_printf("nGA(%s).InitializeWorld\n", this->fn);
}
//-----------------------------------------------------------------------------//
///start the ga evaluation
//-----------------------------------------------------------------------------//
void nGA::Run(){
    //n_printf("nGA(%s).Run() state=%s\n", this->fn, this->GetState());
    nAspect::Tick(0,0);
    if( this->GetStateI() == FRESH )
        this->SetState(MASTER_WAITING_FOR_RUN);
    //if( this->GetStateI() == MASTER_WAITING_FOR_RUN ) {
        //this->SetState(MASTER_NO_SLAVES);
    this->InitializeLocalSlave();
    //}
    this->InitializeWorld();
    this->genomeEvaluating = 0;
    generation.SetI(0);
    foreachchild( nGenome, genome, this ) {
        genome->SetState(UNEVALUATED);
        genome->Randomize();
        genome->SetGeneration( generation.GetI() );
    }
    //this->SetState(IDLE);
    this->beginRunning = true;
    static void* eventPtr = 0;
    ref_event->pull(eventPtr, "gaRunning", str(this));
}
//-----------------------------------------------------------------------------//
void nGA::CheckPopSize(){
    //if (popsize.isdirty() ){                                                    //first off check to see if our population is the right size
    int numGenomes = 0;
    foreachchild(nGenome, genome, this){
        numGenomes++;
    }
    for( int i = numGenomes; i < popsize.GetI(); i ++ ) {
        for( int j = 0; j < 10000; j ++ ) {
            char name[64];
            snprintf(name, 64, "genome%i", j);
            if ( this->Find(name) == 0)  { //create a new genome child with this name
                //n_printf("new ngenome %s\n", name);
                nGenome* g = dynamic_cast<nGenome*>(this->CreateChild("ngenome", name));
                n_assert(g && "failed to create ngenome");
                this->CreateGenome(g);
                g->SetState(UNEVALUATED);
                g->Randomize();
                g->SetGeneration(generation.GetI() );
                //n_printf("created genome:%s with bits:%s\n", str(g), g->GetBits());
                break;
            }
        }
    }
    //}
}
//-----------------------------------------------------------------------------//
void nGA::ClearPopulation(){
    //n_printf("nGA(%s).ClearPopulation()\n", str(this));
    nArray<nGenome*> pop;
    nArray<nString> popNames;
    foreachchild( nGenome, child, this ){ 
        pop.Append( child );
        popNames.Append( str(child));
    }
    foreach( child, pop ) {
        ref_release->ReleaseNow( child );
        //child->Release();
    }
    for( int i = 0; i < popNames; i ++ )
        n_assert( LOOKUP(nGenome*, popNames[i].Get()) == 0 );
    slaveGenome = 0; //will be released with the others above
    for( int i = 0; i < slaveConnections; i ++ )
        slaveConnections[i]->rpcClearPopulation(i);
}
//-----------------------------------------------------------------------------//
///create the genome
//-----------------------------------------------------------------------------//
void nGA::CreateGenome( nGenome* g ) {
    n_printf("nGA::CreateGenome(%p,%s)\n", g, g?g->fn:"");
}
//-----------------------------------------------------------------------------//
void nGA::StartEvaluation( nArray<bool> bitString ){
    n_printf("nGA(%s.%s).StartEvaluation(", str(this), str(this->slaveGenome) );
    bool b;
    foreach( b, bitString ) n_printf("%i", b );
    n_printf(")\n");

    if ( this->slaveGenome == 0 ) {
        this->slaveGenome = dynamic_cast<nGenome*>(this->CreateChild("ngenome", "slaveGenome"));
        n_assert(this->slaveGenome && "failed to create ngenome");
        this->CreateGenome(this->slaveGenome);
        this->slaveGenome->SetState(UNEVALUATED);
        this->slaveGenome->Randomize();
    }
    if( this->slaveGenome ) {
        this->slaveGenome->SetBitString( bitString );
        this->slaveGenome->SetGeneration(generation.GetI() );
        this->StartEvaluation( this->slaveGenome );
        this->SetState(SLAVE_EVALUATING);
    }
}
//-----------------------------------------------------------------------------//
//hook pulled to start the evalution of an individual
void nGA::StartEvaluation( nGenome* g ){
    n_printf("nGA::StartEvaluation(%s)\n", g?g->fn:"0x0");
}
//-----------------------------------------------------------------------------//
//check to see if the evaluation has finished
bool nGA::EvaluationFinished(){
    n_printf("nGA::EvaluationFinsihed()\n");
    return true;
}
//-----------------------------------------------------------------------------//
float nGA::GetFitness(){
    n_printf("nGA::GetFitness()\n");
    return randF(0,100.0f);
}
//-----------------------------------------------------------------------------//
void nGA::StopEvaluation( nGenome* g ) {
    n_printf("nGA::StopEvaluation(%s)\n", g?g->fn:"0x0");
}
//-----------------------------------------------------------------------------//
/// go through all the members of the population - find the min/max/avg and find there normalized fitnesses -> which scale linearly from 0->1->4 based on position
//-----------------------------------------------------------------------------//
void nGA::NormalizeFitnesses(){
    nArray<nGenome*> genomes = this->GetSortedGenomes();
    for( int i = 0; i < genomes; i ++ )
        genomes[i]->SetRank(i);

    float minFitness;
    float maxFitness;
    float avgFitness;
    int genomeIndex = 0;
    //find the max/min fitness
    foreachchild(nGenome, genome, this){
        float fitness = genome->GetFitness();
        if ( genomeIndex == 0 )
            avgFitness = minFitness = maxFitness = fitness;
        else {
            if ( fitness < minFitness )
                minFitness = fitness;
            if ( fitness > maxFitness )
                maxFitness = fitness;
            avgFitness = avgFitness * genomeIndex / float(genomeIndex + 1) + fitness * 1.0f / genomeIndex;
        }
        genomeIndex ++;
    }
    this->minFitness.SetF( minFitness );
    this->maxFitness.SetF( maxFitness );
    this->avgFitness.SetF( avgFitness );
    float fitnessRange = maxFitness - minFitness;
    if ( fitnessRange == 0.0f ) fitnessRange = 1.0f;
    //n_printf("minFitness:%f maxFitness:%f avgFitness:%f\n", minFitness, maxFitness, avgFitness );
    {
        foreachchild(nGenome, genome, this){
            float fitness                = genome->GetFitness();
            float populationSimilarity  = this->CalculateSimilarity( genome );
            genome->SetSimilarity( populationSimilarity );
            //float sharedFitness = fitness * (1.0f - populationSimilarity);
            float sharedFitness = fitness / populationSimilarity;
            //n_printf("genome:%s fit:%f populationSimilarity:%f sharedFitness:%f \n", str(genome), fitness, populationSimilarity, sharedFitness );
            genome->SetSharedFitness( sharedFitness );
        }
    }
    {
        foreachchild(nGenome, genome, this){
            float fitness               = genome->GetSharedFitness();
            float fitnessPercentile     = (fitness - minFitness) / fitnessRange;
            genome->SetFitnessPercentile(fitnessPercentile);
            float normalizedFitness     = fitnessPercentile * fitnessPercentile * 4.0f + 1.0f; //exponential curve from 0->4
            //n_printf("fitness: %f normalizedFitness: %f\n", fitness, normalizedFitness );
            genome->SetNormalizedFitness( normalizedFitness );
        }
    }
    static void* eventPtr = 0;
    if( this->RecordData())
        ref_event->pull( eventPtr, "gaGenerationFinished", this->fn );
}
//-----------------------------------------------------------------------------//
/**
 * go through the population -> everyone whos fitness percentile is less then .5 (less then half) should be replaced via selection
 **/
//-----------------------------------------------------------------------------//
void nGA::Selection(){
    //local function to do my qsorting
    //n_printf("nGA(%s).Selection() gen:%i\n", this->fn, generation.GetI() );
    //n_printf("sorted genomes:\n");
    //foreach( genome, genomes ) {
    //n_printf("    %f:%s\n", genome->GetFitness(), genome->fn );
    //}
    nArray<nGenome*> genomes = this->GetSortedGenomes();
    for( int i = 0; i < genomes; i ++ )
        genomes[i]->SetRank(i);

    //okay so now what do we do -> alright for each individual in the bottom half -> we need to select a parent
    //for now just copy the top half
    int startIndex = genomes.Size() / 2;
    for( int i = startIndex; i < genomes; i ++ ) {
        genomes[i]->CopyFrom( genomes[i-startIndex] );
        genomes[i]->SetGeneration( generation.GetI() );
    }
}
//-----------------------------------------------------------------------------//
/*
 * call john edwards and have him do some cross over
 * pretty much just walk the list of newbies and have them crossover with each other
 **/
//-----------------------------------------------------------------------------//
void nGA::CrossOver(){
    nArray<nGenome*> genomesToCrossOver;
    foreachchild( nGenome, genome, this ) {
        if ( genome->GetGeneration() == generation.GetI() )
            genomesToCrossOver.Append( genome );
    }
    for( int i = 1; i < genomesToCrossOver; i ++ )
        genomesToCrossOver[i-1]->CrossOverWith( genomesToCrossOver[i] );
}
//-----------------------------------------------------------------------------//
/**
 * walk my genomes and call there mutation function
 **/
//-----------------------------------------------------------------------------//
void nGA::Mutation(){
    foreachchild( nGenome, genome, this ) {
        if ( genome->GetGeneration() == generation.GetI() )
            genome->Mutation();
    }
}
//-----------------------------------------------------------------------------//
///Choose a genome to parent offspring
//-----------------------------------------------------------------------------//
nGenome* nGA::SelectGenome(){
    //n_printf("nGA(%s).SelectGenome()\n", this->fn);
    nArray<nGenome*> genomes;

    float totalFitness = 0.0f;
    int numberOfEligableGenomes = 0;
    foreachchild( nGenome, genome, this ) {
        if ( genome->GetStateI() == EVALUATED ) {
            //numberOfEligableGenomes++;
            genomes.Append( genome );
            totalFitness += genome->GetNormalizedFitness();
        }
    }
    //n_printf("totalFitness:%f\n", totalFitness);
    //int selection = randI( 0, numberOfEligableGenomes-1 );
    float selection = randF( 0, totalFitness );
    foreach( genome, genomes ) {
        if( genome->GetNormalizedFitness() >= selection )
            return genome;
        selection -= genome->GetNormalizedFitness();
    }
    return 0;
}
//-----------------------------------------------------------------------------//
///Choose a genome to be replaced
//-----------------------------------------------------------------------------//
inline float convertGoodFitnessToBadFitness( float f ) { return 6.0f - f; }
nGenome* nGA::SelectBadGenome(){
    //n_printf("nGA(%s).SelectBadGenome()\n", this->fn);
    nArray<nGenome*> genomes;

    float totalFitness = 0.0f;
    int numberOfEligableGenomes = 0;
    foreachchild( nGenome, genome, this ) {
        if ( genome->GetStateI() == EVALUATED ) {
            //numberOfEligableGenomes++;
            //        n_printf("genome:%s, gen:%i\n", str(genome), genome->GetGeneration() );
            genomes.Append( genome );
            totalFitness += convertGoodFitnessToBadFitness( genome->GetNormalizedFitness() );
        }
    }
    //n_printf("totalFitness:%f\n", totalFitness);
    //int selection = randI( 0, numberOfEligableGenomes-1 );
    float selection = randF( 0, totalFitness );
    foreach( genome, genomes ) {
        float fit = convertGoodFitnessToBadFitness( genome->GetNormalizedFitness() );
        if( fit >= selection )
            return genome;
        selection -= fit;
    }
    return 0;
}

//-----------------------------------------------------------------------------//
///Create an array with a sorted list of genomes, by fitness
//-----------------------------------------------------------------------------//
nArray<nGenome*> nGA::GetSortedGenomes(){
    struct comparator {
        static int compar( const void* a, const void* b ){
            //cast it into a pointer to the genome (qsort calls me with the addresses of the members in the array -> so they are really doubl epointers
            const nGenome* x = *static_cast<nGenome**>((void*)a);
            const nGenome* y = *static_cast<nGenome**>((void*)b);
            n_assert( x && y );
            return *x < *y;
        }
    };
    nArray<nGenome*> genomes;
    foreachchild(nGenome, genome, this){
        genomes.Append( genome );
    }
    qsort( genomes, genomes.Size(), sizeof( nGenome* ), comparator::compar ); //sort the population
    return genomes;
}
//-----------------------------------------------------------------------------//
void nGA::AddSlaveConnection( GAConnection* slaveConnection ){
    n_assert( slaveConnection && "nGA trying to addslaveconnection on a null pointer" );
    //n_printf("nGA(%s).AddSlaveConnection(%i)\n", this->fn, slaveConnection->getId() );
    this->slaveConnections.Append( slaveConnection );
    //n_printf("slaveConnection:%p\n", slaveConnection);
    slaveConnection->rpcSetState((TNL::U32) SLAVE_WAITING_FOR_WORLD);
}
//-----------------------------------------------------------------------------//
void nGA::SendInitializeWorldMessage( const char* s ){
    //n_printf("nGA::SendInitializeWorldMessage(%s)\n", s);
}
//-----------------------------------------------------------------------------//
void nGA::OnReceiveInitializeWorldMessage( const char* s ){
    //n_printf("nGA::OnReceiveInitialzeWorldMessage(%s)\n", s );
}
//-----------------------------------------------------------------------------//
void nGA::PingForSlaves( float time ){
    if ( lastPingTime < 0 || time - lastPingTime > pingFreq ) {
        TNL::Address broadcastAddress(TNL::IPProtocol, TNL::Address::Broadcast, port.GetI());
        nString address;
        //address.PrintF("IP:%s:%i", "134.197.40.215", port.GetI());
        //address.PrintF("IP:%s:%i", "broadcast", port.GetI());
        //TNL::Address broadcastAddress(address.Get());
        //TNL::Address broadcastAddress(address.Get());
        TNL::Nonce nonce;
        nonce.getRandom();
        if( this->connection ) {
            GANetInterface* interface = this->connection->getInterface();
            if( interface ) {
                interface->sendPing(broadcastAddress, nonce);
            }
        }
        this->lastPingTime = time;
    }
}
//-----------------------------------------------------------------------------//
void nGA::InitializeSlave( nGA* slave ){
    n_printf("nGA(%s).InitializeSlave(%s)\n", str(this), str(slave));
}
//-----------------------------------------------------------------------------//
///Determine how similar this genome is to the rest of population
///We can do phenotypic since we have all the nice information
//-----------------------------------------------------------------------------//
float nGA::CalculateSimilarity( nGenome* genome ){
    float speciationThreshold = .9f;
    float similarityAverage = 0.0f;
    nArray<float> subSimilarities;
    if( genome->GetStateI() != EVALUATED ) {
        //return 0.0f;
        return 1.0f;
    }
    foreachchild( nGenome, other, this ) {
        if( other == genome || other->GetStateI() != EVALUATED) continue;
        //n_printf("comparing %s to %s ", str(genome), str(other));
        //walk both trees and compare phenomes
        float subSimilarityAverage;
        nArray<float> similarities;
        this->RecursiveSimilarityWalk( &similarities, genome, other );
        subSimilarityAverage = calculateAverage( similarities );
        //n_printf("subSimilarityAverage:%f\n", subSimilarityAverage );
        subSimilarities.Append( subSimilarityAverage );
    }
    int othersInSpecies = 1;
    for( int i = 0; i < subSimilarities; i ++ )
        if( subSimilarities[i] > speciationThreshold )
            othersInSpecies++;
    return (float)othersInSpecies;
    //similarityAverage = calculateAverage( subSimilarities );
    //return similarityAverage;
}
//-----------------------------------------------------------------------------//
void nGA::RecursiveSimilarityWalk( nArray<float>* similarities, nRoot* nodeA, nRoot* nodeB ) {
    //n_printf("RecursiveSimilarityWalk(%s,%s)\n", str(nodeA), str(nodeB) );
    n_assert( nodeA && nodeB );
    classVar( ngeneint,   geneintClass );
    classVar( ngenefloat, genefloatClass );
    if( nodeA->IsA( geneintClass ) ){
        n_assert( nodeB->IsA( geneintClass ) && "tree mismatch in similarity check");
        nGeneInt* a = (nGeneInt*) nodeA;
        nGeneInt* b = (nGeneInt*) nodeB;
        int aval = a->GetI();
        int bval = b->GetI();
        int min, max;
        a->GetRange( &min, &max );
        int range = max - min;
        int diff = fabs(aval-bval);
        float similarity = diff / (float) range;
        similarity = 1.0f - similarity;
        //n_printf("a:%s b:%s aval:%i bval:%i diff:%i range:%i similarity:%f\n", str(a), str(b), aval, bval, diff, range, similarity );
        similarities->Append( similarity );
    }
    if( nodeA->IsA( genefloatClass ) ){
        n_assert( nodeB->IsA( genefloatClass ) && "tree mismatch in similarity check");
        nGeneFloat* a = (nGeneFloat*) nodeA;
        nGeneFloat* b = (nGeneFloat*) nodeB;
        float aval = a->GetF();
        float bval = b->GetF();
        float min, max;
        a->GetRange( &min, &max );
        float range = max - min;
        float diff = fabs(aval-bval);
        float similarity = diff / (float) range;
        similarity = 1.0f - similarity;
        //n_printf("a:%s b:%s aval:%f bval:%f diff:%f range:%f similarity:%f\n", str(a), str(b), aval, bval, diff, range, similarity );
        similarities->Append( similarity );
    }
    nRoot* nodeBChild = nodeB->GetHead();
    foreachchild( nRoot, nodeAChild, nodeA ) {
        this->RecursiveSimilarityWalk( similarities, nodeAChild, nodeBChild );
        nodeBChild = nodeBChild->GetSucc();
    }
}
//-----------------------------------------------------------------------------//
void nGA::OnRunDone(){
    n_printf("nGA(%s).RunDone\n", str(this));
    foreachchild( nGenome, genome, this ) {
        SetState( EVALUATED );
    }
}
