#define N_IMPLEMENTS nGAMaster
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ga/ngamaster.h"

nNebulaScriptClass(nGAMaster, "nga");

//------------------------------------------------------------------------------
/**
*/
nGAMaster::nGAMaster():
    socket( kernelServer, this ),
    strikeServerHandle( -1 )
{
    this->AddLogTag("gamaster");
}

void nGAMaster::Initialize(){
    nGA::Initialize();
    socket = "/sys/servers/socket";
}

void nGAMaster::InitVars(){
    nGA::InitVars();
    maxEvalTime.Connect( ent, "maxevaltime" );
    maxEvalTime.DefaultF( 5.0f );
    maxBoots.Connect( ent, "maxboots" );
    maxBoots.DefaultI( 0 );
    numBoots.Connect( ent, "numboots" );
    numBoots.DefaultI( 0 );
}

void nGAMaster::InitGA(){
    log("<nGAMaster::Master()>");
    this->BuildEncoding();                  //determines what the individuals are like
    int bitStringLength = sampleEncoding->GetBitString().Size();
    n_printf("*********************************\n");
    n_printf("setting bit tree depth to : %i\n", bitStringLength );
    n_printf("*********************************\n");
    bittree.SetDepth( bitStringLength );
    this->InitPopulation();                 //initialize the population
    this->nextIndividualToEvaluate = 0;     //should start evaluating at the first individual
    generation.SetI( 0 );                   //you are at generation 0 
    log("</nGAMaster::Master()>");
}

nGAMaster::~nGAMaster(){
}
//------------------------------------------------------------------------------
void nGAMaster::Tick( float time, float ){
    //nGA::Tick( time, dtime );
    float realTime = (float) kernelServer->ts->GetFrameTime();
    timeTaken.SetF( realTime - startTime );
    this->time = realTime;
    if( running ) { 
        this->Step();
        this->CheckIfDone();
    }
}
//------------------------------------------------------------------------------
//------------------------------------MASTER------------------------------------
//------------------------------------------------------------------------------
/**
  The main tick function
  checks if any individuals are unevaluated,  if so sends them out to the appropriate slave
  if the generation is finished, it goes to the next,  finishing if done
 */
void nGAMaster::Step(){
    //log("<nGAMaster::Step()>");
    if ( !running ) return;
    if ( this->slaves.Size() == 0 ) //no slaves = nothing to do
        return;
    bool allEvaluated = true;
    //See if we can delegate any individuals
    for( int i = 0; i < pop.Size(); i ++ ){
        if( pop[i]->IsEvaluated() == false ) {
            float fit;
            int gen;
            if( HasEvaluated( pop[i].get(), fit, gen ) ) {
	pop[i]->StartEvaluation();
	pop[i]->FinishEvaluation( fit, gen );
	cacheHits.SetI( cacheHits.GetI() + 1 );
            }
            allEvaluated = false;
            if( pop[i]->GetEvaluationStarted() == false ) {
	for( int j = 0; j < slaves.Size(); j ++ ){
	    if( slaves[j].ready && slaves[j].busy == false ){
	        this->Delegate( i, slaves[j].socketHandle );
	        slaves[j].busy = true;
	        slaves[j].individualSent = i;
	        break;                          
	    }
	}
            } else if( time - pop[i]->GetTimeStarted() > maxEvalTime.GetF() ) {
	log("pop[%i] is taking longer then it should,  sending to another", i);
	for( int j = 0; j < slaves.Size(); j ++ ) {
	    if( slaves[j].individualSent == i ) {
	        log("slave : %i is the one who is being slow, telling him to abort");
	        socket->Send(slaves[j].socketHandle, "abortevaluation", "too slow" );
	    }
	    if( slaves[j].ready && slaves[j].busy == false ) {
	        this->Delegate( i, slaves[j].socketHandle );
	        slaves[j].busy = true;
	        slaves[j].individualSent = i;
	        break;                          
	    }
	}
            }
        }
    }
    //generation is all evaluated
    if( allEvaluated ){ //generation is done
        this->NextGeneration();
    }
}

/**
  Send an individual to a slave to be evaluated
  @param individual the individual to be evaluated
  @param socketHandle the slave to evaluate it
 */
void nGAMaster::Delegate( int individual, int socketHandle ){                                      //outsource an evaluation
    log ("nGAMaster::Delegate( %i %i )", individual, socketHandle );
    pop[individual]->StartEvaluation( time );
    int allocSize;
    nGenome* alloc = pop[individual]->GetChild(0);
    allocSize = alloc->NumChildren();
    int* allocs = new int[allocSize];
    for ( int i = 0; i < allocSize; i++){ 
        allocs[i] = alloc->GetChild(i)->GetEI();
        //log("allocs[%i]=%i", i, allocs[i] );
    }
    float rc = pop[individual]->GetChild(1)->GetF();
    char msg[1024];
    memset( msg, 0, 1024 );
    sprintf( msg, "%f %i ", rc, allocSize );
    char piece[16];
    for( int i = 0; i < allocSize; i ++ ) {
        sprintf( piece, "%i ", allocs[i] );
        strcat( msg, piece );
    }
    n_assert( strlen( msg ) < 1024 );

    socket->Send( socketHandle, "evaluate", msg );

    delete[] allocs;
}
/** The strikeserver has connected to me */
void nGAMaster::ReceiveStrikeServer( int from, const char* ){
    log("StrikeServer on connection: %i", from );
    this->strikeServerHandle = from;
}
/** A slave has connected to me */
void nGAMaster::ReceiveSlave( int from, const char* ){
    log("Slave on connection: %i", from );
    //slaveIndexes.Append( from );
    SlaveInfo& info = slaves.Push();
    slaveTimes.Push();
    info.ip = socket->HandleToIP( from );
    info.socketHandle = from;
    info.busy = false;
    info.individualSent = -1;
    info.ready = false;

    if( situation != "none" )
        socket->Send( from, "situation", situation.Get() );
}
/// The slave is finished parsing the situation and is ready to go
void nGAMaster::ReceiveSlaveReady( int handle, const char* msg ) {
    int from = -1;
    for( int i = 0; from == -1 && i < slaves; i ++ )
        if( slaves[i].socketHandle == handle ) {
            from = i;
        }
    if( from == -1 )
        log("ERROR Receieved  slave ready from an unknown slave on socketHandle: %i", from );
    else 
        slaves[from].ready = true;
}
/** The StrikeServer has sent me a situation to plan for */
void nGAMaster::ReceiveSituation ( int from, const char* s ){
    if( from != strikeServerHandle )
        log("warning from != strikeserver %i != %i", from, strikeServerHandle);

    log( "nGAMaster::ReceiveSituation" );
    log( "nGAMaster::Sending situation to slaves: %i", slaves.Size() );
    for( int i = 0; i < slaves.Size(); i ++ ) {
        slaves[i].ready = false;
        log("send situation on socket:%i", i );
        socket->Send( slaves[i].socketHandle, "situation", s );
    }
    this->SetSituation( s );
}
/** A slave has returned a fitness */
void nGAMaster::ReceiveFitness( int handle , const char* msg ) {
    log("nGAMaster::ReceiveFitness( %i %s )", handle, msg );
    if( running == false ) {
        log("slave %i evaluated an individual when the ga was not running - likely a straggler - ignoring");
        return;
    }
    for( int i = 0; i < slaves.Size(); i ++ ){
        if( slaves[i].socketHandle == handle){ 		//message is from this slave
            int slaveFrom = i;
            log("slaveFrom: %i", slaveFrom );
            int evaluatedIndex = slaves[slaveFrom].individualSent;
            log("evaluatedIndex: %i", evaluatedIndex );
            float fitness;
            sscanf( msg, "%f", &fitness );
            n_assert( slaveFrom >= 0 && slaveFrom < slaves.Size() );
            if( evaluatedIndex < 0 ) {
	log("slave: %i finished an individual which had been reassigned - ignoring");
	break;
            }
            log("fitness: %f", fitness );
            pop[evaluatedIndex]->FinishEvaluation( fitness, generation.GetI() );
            CacheIndividual( pop[evaluatedIndex].get() );
            slaves[slaveFrom].busy = false;
            slaves[slaveFrom].numEvaluated++;

            this->numEvaluated++;
            evals.SetI( evals.GetI() + 1 );
            break;
        }
    }
    this->Step();
}
void nGAMaster::ReceiveSlaveAborted( int handle, const char* msg ) {
    for( int i = 0; i < slaves; i ++ ) {
        if( slaves[i].socketHandle == handle ) {
            log("slave : %i has aborted", i );
            slaves[i].busy = false;
            slaves[i].evaluationsFailed++;
        }
    }
}
///Recieved profiling data - display it
void nGAMaster::ReceiveProfiling( int handle, const char* msg ){
    log("nGAMaster::ReceiveProfiling( %i %s )", handle, msg );
    if( running == false ) {
        log("Received profiling while not running - likely a straggler finishing after the run was done - ignoring");
        return;
    }
    int slaveFrom = -1;
    for( int i = 0; i < slaves.Size(); i ++ )
        if( slaves[i].socketHandle == handle ) 		//message is from this slave
            slaveFrom = i;
    if( slaveFrom == -1 ) {
        log("cannot find slave message was from");
        return;
    }
    Times& t = slaveTimes[slaveFrom];
    int retval = sscanf( msg, "%f %f %f %f %f %f %f %f %f %f %f %f", 
            &t.routing, &t.loadingMission, &t.loadingPlan, &t.genomeToPlan, &t.parsingFitness, &t.loading, &t.evaluation, &t.running,
            &t.evaluationStarted, &t.evaluationStopped, &t.runningStarted, &t.runningStopped );
    if( retval != 12 )
        log("received bad profile %s, only parsed %i out", msg, retval );
}

void nGAMaster::ReceiveRun( int from, const char* msg ) {
    if( from != strikeServerHandle )
        log("WARNING: from != strikeServerHandle %i != %i", from, strikeServerHandle );
    this->Run();
}

/**
  Okay our connection died what now
  1. figure out which slave it was
  2. remove that slave from my list of slaves
  3. reassign that evaluation
  4. reboot the slave
 */
void nGAMaster::ConnectionDied( int handle ){
    log ( "nGAMaster::ConnectionDied( %i )", handle );
    const char* ip = socket->HandleToIP( handle );
    log( "ip: %s", ip );
    //Figure out who it was
    int slaveNum = -1;
    for( int i = 0; i < slaves.Size(); i ++ ){ 
        if( slaves[i].socketHandle == handle ) {
            slaveNum = i;
            break;
        }
    }
    log("slaveNum: %i", slaveNum );
    if( slaveNum == -1 ){
        log("connection %i died, but i cant find it as one of my slaves", handle );
        return;
    } 

    //reassign the evaluation
    if( slaves[slaveNum].busy ) { // it was actively doing something when it died
        int wasEvaluating = slaves[slaveNum].individualSent;
        log("wasEvaluating: %i", wasEvaluating );
        slaves[slaveNum].individualSent = -1;
        pop[wasEvaluating]->AbortEvaluation();
    }
    //remove it from my list of slaves
    slaves.Erase( slaveNum );

    int numBooted = numBoots.GetI();
    if( numBooted < maxBoots.GetI() ) {
        numBoots.SetI( numBooted+1 );
        nString script;
        script.PrintF("Socket::remoteBoot %s gaslave", ip );
        log("script:%s", script.Get() );
        RUNSCRIPT( script.Get() );
    }
}

/**
  Ga is done running
  send your best plan back to the strikeserver
 */
void nGAMaster::DoneRunning(){
    log("<DoneRunning>");
    nGA::DoneRunning();
    socket->Send( strikeServerHandle, "blueplan", this->bestPlan.Get() );
    for( int i = 0; i < slaves; i ++ ) {
        slaves[i].busy = false;
        slaves[i].individualSent = -1;
    }
    log("</DoneRunning>");
}

nString nGAMaster::GetSlaveInfo(){
    nString ans;
    nString line;
    for( int i = 0; i < slaves; i ++ ) {
        SlaveInfo& s = slaves[i];
        Times& t = slaveTimes[i];
        line.PrintF("%i %s %i %i %i %i %f %f %f %f %f %f %f %f %f %f %f %f\n", i, s.ip.Get(), s.ready, s.busy, s.individualSent, s.numEvaluated, t.routing, t.loadingMission, t.loadingPlan, t.genomeToPlan, t.parsingFitness, t.loading, t.evaluation, t.running, t.evaluationStarted, t.evaluationStopped, t.runningStarted, t.runningStopped );
        ans += line;
    }
    return ans;
}

void nGAMaster::NextGeneration(){
    nGA::NextGeneration();
    //void any pending evaluations - keep them from screwing up the next generation
    for( int i = 0; i < slaves; i ++ ) {
        slaves[i].individualSent = -1;
        if( slaves[i].busy )
            socket->Send(slaves[i].socketHandle, "abortevaluation", "gen is over" );
    }
    /*for( int i = 0; i < pop.Size(); i ++ ) {
        pop[i]->AbortEvaluation();
    }*/
}
