#define N_IMPLEMENTS nGARed
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ga/ngared.h"
#include "ga/nga.h"
#include "enkiduo/nent.h"
#include "map/ninfluencemap.h"
#include "ai/nthreatallocator.h"

nNebulaScriptClass(nGARed, "ngaproblem");

//------------------------------------------------------------------------------
/**
*/
nGARed::nGARed():
    platformEntRoot 	( kernelServer, this ),
    targetEntRoot   	( kernelServer, this ),
    redAI		( kernelServer, this ),
    defensability	( kernelServer, this ),
    threatRedundance	( kernelServer, this ),
    closeness		( kernelServer, this ),
    ta		( kernelServer, this )
{
    platformEntRoot         	= "/game/ents/platform";
    targetEntRoot           	= "/game/ents/target";
    redAI 		= "/game/sides/red";
    defensability	= "/game/im/defensability";
    threatRedundance	= "/game/im/threatRedundance";
    closeness 		= "/game/im/closeness";
    ta 		= "/game/sides/red/threatallocator";
}

void nGARed::Initialize(){
    nGAProblem::Initialize();
}

void nGARed::InitVars(){
    nGAProblem::InitVars();
}

nGARed::~nGARed(){
}
//------------------------------------------------------------------------------
/**
*/

void nGARed::Tick( float time, float dtime ){
    nGAProblem::Tick( time, dtime );
}

///construct the default encoding -> the structure will be copied to all others
nGenome* nGARed::BuildEncoding(){
    log("<nGARed::BuildEncoding()>");
    nGenome* sampleEncoding = (nGenome*) kernelServer->New("ngenome", "/ga/sampleencoding");
    sampleEncoding->AddF("defensability", 	0.00f, 	0.01f, .00005 );
    sampleEncoding->AddF("threatRedundance", 	-2.0f, 	1.0f, .01f ); 
    sampleEncoding->AddF("closeness", 	-1.0f, 	1.0f, .01f );
    sampleEncoding->AddF("offset", 	0.0f, 	100.0f );
    n_printf("bitlength: %i\n", sampleEncoding->GetBitString().Size() );
    sampleEncoding->Randomize();
    log("</nGARed::BuildEncoding()>");
    return sampleEncoding;
}
/// Load / Reset the mission
void nGARed::LoadMission(){
    log("<nGARed::LoadMission()>");
    double missionLoadingStarted = time->GetTime();
    
    nString script;
    script.PrintF("Plan::load {%s} yes", ga->GetSituation() );
    RUNSCRIPT( script.get() );

    double missionLoadingFinished = time->GetTime();
    ga->times.loadingMission = missionLoadingFinished - missionLoadingStarted;
    log("</nGARed::LoadMission()>");
}
void nGARed::LoadPlayerParameters( nGenome* g ) {
    float defensabilityCoef 	= g->GetChild(0)->GetF();
    float threatRedundanceCoef 	= g->GetChild(1)->GetF();
    float closenessCoef 	= g->GetChild(2)->GetF();
    float offsetCoef 	= g->GetChild(3)->GetF();
    ifvalid( defensability )
        defensability->SetCoef( defensabilityCoef );
    ifvalid( threatRedundance )
        threatRedundance->SetCoef( threatRedundanceCoef );
    ifvalid( closeness )
        closeness->SetCoef( closenessCoef );
    ifvalid( redAI )
        redAI->SetVarF("offset", offsetCoef );
    RUNSCRIPT("InfluenceMap::importVars");
}
void nGARed::TriggerPlayer(){
    log("nGARed::TriggerPlayer");
    ta->PlaceThreats();
    RUNSCRIPT("set StrikeServer::info(red.plan) [Plan::buildPlan red]");
    log("/nGARed::TriggerPlayer");
}
void nGARed::TriggerBlueGA(){
    log("nGARed::TriggerBlueGA");
    RUNSCRIPT("StrikeServer::replan now");
    //RUNSCRIPT("StrikeServer::Play");
    log("/nGARed::TriggerBlueGA");
}
/// Is the evaluation finished?
bool nGARed::IsEvaluationDone(){
    nRoot *platform, *aiRoot, *ai;
    int numRunning = 0;
    int numChecked = 0;
    ifvalid( platformEntRoot ) 
        for ( platform = platformEntRoot->GetHead(); platform; platform = platform->GetSucc() ){
            if ( ((nEnt*) platform)->Held() )
	numRunning++;
            aiRoot = platform->Find("ai");
            if( aiRoot ) for ( ai = aiRoot->GetHead(); ai; ai = ai->GetSucc() ) {
	numChecked++;
	if( ((nThing*)ai)->IsAlive() )
	    numRunning++;
            }
        }
    if( numRunning == 0 ) {
        return true;
    }
    return false;
}
///called to stop an evaluation -> clear things up
void nGARed::StopEvaluation(){
    log("nGARed::StopEvaluation()>");
    RUNSCRIPT("pause;GA::clearMission");
    log("/nGARed::StopEvaluation()>");
}
//begin an evaluation
void nGARed::StartEvaluation( nGenome* g ){
    log("<nGARed::StartEvaluation()>");
    this->LoadMission();			//load the situation
    this->LoadPlayerParameters( g );
    this->TriggerPlayer();
    this->TriggerBlueGA();
    log("</nGARed::StartEvaluation()>");
}
float nGARed::ParseFitness() {
    log ("<nGARed::DoneEvaluation>" );
    //return plan.toString();
    ga->times.runningStopped = time->GetTime();
    ga->times.running = ga->times.runningStopped - ga->times.runningStarted;
    double fitnessParsingStart = time->GetTime();
    float fitness = 0;
    float damageBlue = 0.0f;
    float damageRed = 0.0f;

    float totalDistanceTraveled = 0;
    ifvalid( targetEntRoot ) for ( nEnt* target = (nEnt*) targetEntRoot->GetHead(); target; target = (nEnt*) target->GetSucc() ){
        float value = target->GetVarF("value");
        float hp = target->GetVarF("hp");
        float damageDone = (1.0f - hp) * value;
        log("target  : %20s v:%f h:%f d:%f", target->fn, value, hp, damageDone );
        damageRed += damageDone;
    }
    ifvalid( platformEntRoot) for ( nEnt* platform= (nEnt*) platformEntRoot->GetHead(); platform; platform = (nEnt*) platform->GetSucc() ){
        float value = platform->GetVarF("value");
        float hp = platform->GetVarF("hp");
        float distanceTraveled= platform->GetVarF("distancetraveled");
        totalDistanceTraveled += distanceTraveled;
        float damageDone = (1.0f - hp) * value;
        log("platform: %20s v:%f h:%f d:%f dist:%f", platform->fn, value, hp, damageDone, distanceTraveled );
        damageBlue += damageDone;
    }
    double fitnessParsingStop = time->GetTime();
    ga->times.parsingFitness = fitnessParsingStop - fitnessParsingStart;

    ga->times.evaluationStopped = time->GetTime();
    ga->times.evaluation = ga->times.evaluationStopped - ga->times.evaluationStarted;
    float distancePenalty = totalDistanceTraveled * .01f;
    fitness = damageBlue + distancePenalty - damageRed ;
    log("fitness: %f damageBlue: %f damageRed: %f distancePenalty: %f\n", fitness, damageBlue, damageRed, distancePenalty );
    log ("</nGARed::DoneEvaluation>" );
    return fitness;
}
///convert this genome into an xml string
nString nGARed::GenomeToPlan( nGenome* g ) {
    XMLString plan;
    this->LoadPlayerParameters( g );
    ta->PlaceThreats();
    nRoot* threatRoot = kernelServer->Lookup("/game/ents/threat");
    foreachchild(nEnt, e, threatRoot ) {
        plan.open("THREAT", e->GetName() );
        nString moveCoords;
        vector3 p = e->GetVarV("pos");
        moveCoords.PrintF("%f %f %f", p.x, p.y, p.z );
        plan.wrap("POS", moveCoords.Get() );
        plan.close("THREAT");
    }
    return plan;
}

/*
   okay we need to sort out all the cases
   and put them into the case base appropriately
*/
void nGARed::GenerationDone() {
    log("nGARed::GenerationDone");
    int gen = ent->GetVarI("generation");
    nRoot* genomes = kernelServer->Lookup("/ga/ga");
    XMLString cases;
    FILE* f = fopen("xml/casebase/red/cases.xml", "a");
    cases.open("GEN", gen );
    foreachchild( nGenome, g, genomes ) {
        cases.open("CASEGROUP", g->GetName() );
        nString s;
        s.PrintF("%f", g->GetFitness() );
        cases.wrap("CASEFITNESS", s.Get() );
        s.PrintF("%i", g->GetRank() );
        cases.wrap("CASERANK", s.Get() );
        cases += g->GetMisc();
        cases.close("CASEGROUP");
        fprintf( f, "%s", cases.Get() ); //have to dump it in pieces cuz these are way big
        cases = "";
    }
    cases.close("GEN");
    fprintf( f, "%s", cases.Get() );
    fclose( f );
    log("/nGARed::GenerationDone");
}
