#define N_IMPLEMENTS nIM
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "im/nim.h"
#include "im/nimmanager.h"
#include "im/nallocator.h"

nNebulaScriptClass(nIM, "naspect");

const float maxLineHeight = 100.0f;

//------------------------------------------------------------------------------
nIM::nIM():map(kernelServer, this),
           ref_eventServer( kernelServer, this ){
           }
//------------------------------------------------------------------------------
void nIM::Initialize(){
    nAspect::Initialize();
    this->vals                          = 0;
    this->normVals                      = 0;
    this->points                        = 0;
    this->res                           = 0;
    this->valsDirty                     = false;
    map = "/map/map";
    nRoot* manager = 0;
    nRoot* p = this->GetParent();
    while( p && manager == 0 ) {
        manager = p->Find("manager");
        p = p->GetParent();
    }
    //n_printf("%s found manager at %s\n", this->fn, manager->fn );

    p = this->GetParent();
    while( p ){
        if ( dynamic_cast<nAllocator*>(p) != 0 ) {
            allocator = (nAllocator*) p;
            break;
        }
        else
            p = p->GetParent();
    }
    if( allocator == 0 )
        n_printf("%s failed to find allocator\n", this->fn );
    //else
    //n_printf("%s found allocator at %s\n", this->fn, allocator->fn );

    //nIMManager* im                      = (nIMManager*) kernelServer->Lookup("/im/manager");
    //n_assert( im && "failed to find immanager");
    ((nIMManager*)manager)->AddIM(this);
    this->AddToTriggerList();
    this->coef                          = 1.0f;
    behavior = "none";
    ref_eventServer                     = "/sys/servers/event";
}
//------------------------------------------------------------------------------
void nIM::InitVars(){
    nAspect::InitVars();
    worldMin.Connect                    (ent, "worldMin");
    worldMax.Connect                    (ent, "worldMax");
    nEnt* mapEnt                        = LOOKUP(nEnt*, "/map");
    worldSize.Connect                   (mapEnt, "worldsize");
    resolution.Connect                  (ent, "resolution");
    lineBrightness.Connect              (ent, "linebrightness");
    minLocation.Connect                 (ent, "minlocation");
    maxLocation.Connect                 (ent, "maxlocation");
    worldMin.DefaultV                   (0,0,0);
    worldMax.DefaultV                   (1000,0,1000);
    worldMin.SetV(0,0,0);
    worldMax.SetV( worldSize.GetF(), 0, worldSize.GetF() );
    resolution.DefaultI                 (256);
    lineBrightness.DefaultF             (.5f);
    minLocation.DefaultV                (0,0,0);
    maxLocation.DefaultV                (0,0,0);

    //myGroup.Connect                     (ent, "myGroup");
    //myGroup.DefaultRef                  ("");

    steps.Connect( ent, "steps" );
    steps.DefaultI( 1 );
}
//------------------------------------------------------------------------------
nIM::~nIM(){
    this->deallocArrays();
}
//------------------------------------------------------------------------------
void nIM::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    //if( this->valsDirty )
    //this->calcVals();
    if(resolution.isdirty()){
        //n_printf("nIM(%s).resizing to %i\n", this->fn, resolution.GetI() );
        this->deallocArrays();
        this->res = resolution.GetI();
        this->allocArrays();
        this->calcPoints();
        //this->valsDirty = true;
    }
    if( worldMin.isdirty() || worldMax.isdirty() ) {
        //this->calcPoints();
        //this->valsDirty = true;
    }
}
//-----------------------------------------------------------------------------//
template <class T>
    bool deallocArray( T**& arr, int width, int height){
        if ( arr == 0 ) 
            return false;
        n_assert( width > 0 && height > 0 );
        for( int i = 0; i < width; i ++ )
            delete[] arr[i];
        delete[] arr;
        arr = 0;
        return true;
    }
//-----------------------------------------------------------------------------//
template <class T>
bool allocArray( T**& arr, int width, int height, T* defaultValue = NULL){
    n_assert( arr == 0 ); //right now have to dealloc before reallocing
    n_assert( width > 0 && height > 0 );
    arr = new T*[width];
    n_assert( arr && "cannot allocate array");
    for( int i = 0; i < width; i ++ ) {
        arr[i] = new T[height];
        n_assert( arr[i] && "cannot allocate subarray" );
    }
    if( defaultValue )
        for( int i = 0; i < width; i ++ )
            for( int j = 0; j < height; j ++ )
                arr[i][j] = *defaultValue;
    return true;
}
//-----------------------------------------------------------------------------//
void nIM::deallocArrays(){
    deallocArray(vals,     res,     res );
    deallocArray(normVals, res,     res );
    deallocArray(points,   res+1, res+1 );
}
//-----------------------------------------------------------------------------//
void nIM::allocArrays(){
    static float defaultVal = 0.0f;
    static vector3 defaultPoint(0,0,0);
    allocArray(vals,     res,   res,   &defaultVal);
    allocArray(normVals, res,   res,   &defaultVal);
    allocArray(points,   res+1, res+1, &defaultPoint);
}
//-----------------------------------------------------------------------------//
    void nIM::GetColor( float val, nGfxServer* gfx ) {
        if( val >= 0.0f )
            gfx->Rgba( .3f * val, val, 0.2, .15f ); //goes to yellow
        else if( val < 0.0f && val > -1.0f)
            gfx->Rgba( -val, -val * .5, .2, .15f );
        else
            gfx->Rgba( 1.0, 0, 0, .2f );
    }
//-----------------------------------------------------------------------------//
void nIM::Display( nGfxServer* gfx ){
    //n_printf("nIM(%s).Display(%p)\n", this->fn, gfx );
    if( points == 0 || vals == 0 ) {
        //n_printf("nIM(%s).Skipping display:%p,%p\n", this->fn, points, vals );
        return;
    }
    int x, y;
    float lb = lineBrightness.GetF();
    float** vals = this->normVals;      //comment this line to draw the actual vals instead of normalized
    for( y = 0; y < res-1; y ++ ) {
        gfx->Begin( N_PTYPE_TRIANGLE_STRIP );
        for( x = 0; x < res; x ++ ) {
            this->GetColor( vals[x][y], gfx );
            gfx->Coord( points[x][y] );
            this->GetColor( vals[x][y+1], gfx );
            gfx->Coord( points[x][y+1] );
        }
        gfx->End();
    }
    for( x = 0; x < res; x ++ ) {
        gfx->Begin(N_PTYPE_LINE_STRIP);
        for( y = 0; y < res; y ++ ) {
            gfx->Rgba( vals[x][y], vals[x][y], 0.2 + lb, 0.25f );
            gfx->Coord( points[x][y] + vector3(0,.1,0) );
        }
        gfx->End();
    }
    for( y = 0; y < res; y ++ ) {
        gfx->Begin(N_PTYPE_LINE_STRIP);
        for( x = 0; x < res; x ++ ) {
            gfx->Rgba( vals[x][y], vals[x][y], 0.2 + lb, 0.25f );
            gfx->Coord( points[x][y] + vector3(0,.1,0) );
        }
        gfx->End();
    }
    gfx->Begin(N_PTYPE_LINE_LIST);
    for( x = 0; x < res; x ++ ) {
        for( y = 0; y < res; y++ ) {
            this->GetColor( vals[x][y], gfx );
            gfx->Coord( points[x][y] );
            gfx->Coord( points[x][y] + vector3(0,vals[x][y] * maxLineHeight,0) );
        }
    }
    vector3 minLoc = minLocation.GetV();
    gfx->Rgba(1,0,0,1);
    //vertical bar
    gfx->Coord( minLoc.x,               minLoc.y,                               minLoc.z );
    gfx->Coord( minLoc.x,               minLoc.y+maxLineHeight + 20,            minLoc.z );
    //bottom x
    gfx->Coord( minLoc.x-20,            minLoc.y+1,                             minLoc.z-20 );
    gfx->Coord( minLoc.x+20,            minLoc.y+1,                             minLoc.z+20 );
    gfx->Coord( minLoc.x-20,            minLoc.y+1,                             minLoc.z+20 );
    gfx->Coord( minLoc.x+20,            minLoc.y+1,                             minLoc.z-20 );
    //top x
    gfx->Coord( minLoc.x-20,            minLoc.y+maxLineHeight,                 minLoc.z-20 );
    gfx->Coord( minLoc.x+20,            minLoc.y+maxLineHeight,                 minLoc.z+20 );
    gfx->Coord( minLoc.x-20,            minLoc.y+maxLineHeight,                 minLoc.z+20 );
    gfx->Coord( minLoc.x+20,            minLoc.y+maxLineHeight,                 minLoc.z-20 );

    vector3 maxLoc = maxLocation.GetV();
    gfx->Rgba(1,1,1,1);
    gfx->Coord( maxLoc.x,               maxLoc.y,                               maxLoc.z );
    gfx->Coord( maxLoc.x,               maxLoc.y+maxLineHeight + 20,            maxLoc.z );
    gfx->Coord( maxLoc.x-20,            maxLoc.y+1,                             maxLoc.z-20 );
    gfx->Coord( maxLoc.x+20,            maxLoc.y+1,                             maxLoc.z+20 );
    gfx->Coord( maxLoc.x-20,            maxLoc.y+1,                             maxLoc.z+20 );
    gfx->Coord( maxLoc.x+20,            maxLoc.y+1,                             maxLoc.z-20 );
    gfx->Coord( maxLoc.x-20,            maxLoc.y+maxLineHeight,                 maxLoc.z-20 );
    gfx->Coord( maxLoc.x+20,            maxLoc.y+maxLineHeight,                 maxLoc.z+20 );
    gfx->Coord( maxLoc.x-20,            maxLoc.y+maxLineHeight,                 maxLoc.z+20 );
    gfx->Coord( maxLoc.x+20,            maxLoc.y+maxLineHeight,                 maxLoc.z-20 );
    gfx->End();

}
//-----------------------------------------------------------------------------//
void nIM::calcPoints(){
    //n_printf("nIM(%s)::calcPoints\n", this->fn);
    vector3 min = worldMin.GetV();
    vector3 max = worldMax.GetV();
    const float height = 0.1f;
    this->squareSize = (worldMax.GetV() - worldMin.GetV()) / float(resolution.GetI());
    for( int x = 0; x <= res; x ++ )
        for( int z = 0; z <= res; z ++ )
            points[x][z] = vector3( min.x + squareSize.x * x, height, min.z + squareSize.z * z );

    ifvalid(map) {
        for( int x = 0; x <= res; x ++ ) {
            for( int z = 0; z <= res; z ++ ) {
                points[x][z].y = map->GetHeight( points[x][z].x, points[x][z].z ) + height;
                if ( points[x][z].y < 0 ) points[x][z].y = 0;
            }
        }
    }

    this->valsDirty = true;
}
/* -----------------------------------------------------------------------------//
 * approximaite a value by interpolating between its neighbors
 --------------------------------------------------------------------------------*/
float nIM::getVal ( vector3 v ) {
    int xi, yi;
    getCoord(v,xi,yi);
    float rx = v.x - points[xi][yi].x;
    float ry = v.z - points[xi][yi].z;
    float val;
    val += vals[xi+0][yi+0] * (1.0f-rx) * (1.0f-ry);
    val += vals[xi+1][yi+0] * (     rx) * (1.0f-ry);
    val += vals[xi+0][yi+1] * (1.0f-rx) * (     ry);
    val += vals[xi+1][yi+1] * (     rx) * (     ry);
    //printf("getValF(%s) xi:%i yi:%i val:%f\n", str(v), xi, yi, val);
    return val;
}
//-----------------------------------------------------------------------------//
void nIM::getCoord( vector3 point, int& x, int& y ){
    vector3 min = worldMin.GetV();
    vector3 max = worldMax.GetV();
    this->squareSize = (worldMax.GetV() - worldMin.GetV()) / float(resolution.GetI());
    x = (point.x - min.x) / squareSize.x;
    y = (point.z - min.z) / squareSize.z;
    n_assert( x >= 0 && x < res && y >= 0 && y < res && "got coord for some value which was invalid" );
}
//-----------------------------------------------------------------------------//
void nIM::calcVals(){
    //n_printf("nIM(%s)::calcVals()\n", this->fn);
    if( this->vals ) {
        //call my subclasses value calculation
        this->calculateValues();

        //post calc stuff
        this->calcNormVals();
        this->findMaxMinPoints();
        this->valsDirty = false;
    }
    this->valsDirty = false;
}
//-----------------------------------------------------------------------------//
void nIM::calculateValues(){
    //n_printf("nIM::calculateValues()\n");
    this->Tick(0,0);
    for( int x = 0; x < res; x ++ ) {
        for( int y = 0; y < res; y ++ ) {
            vals[x][y] = randF(0,.2);
            if( x > 0 )
                vals[x][y] += .4 * vals[x-1][y];
            if( y > 0 )
                vals[x][y] += .4 * vals[x][y-1];
        }
    }
}
//-----------------------------------------------------------------------------//
struct Info {
    int x;
};
//-----------------------------------------------------------------------------//
void nIM::calculateValues(void** v){
    //n_printf("nIM(%s)->calculateValues()\n", this->fn);
    Info* info = (Info*) (*v);
    if ( *v == NULL ) {
        //n_printf("%s calculateValues initially\n");
        info = new Info();
        info->x = 0;
        *v = info;
    } 
    //n_printf("calculateValues: x : %i\n", info->x);
    //for( int x = 0; x < res; x ++ ) {
    int& x = info->x;
    for( int y = 0; y < res; y ++ ) {
        vals[x][y] = randF(0,.2);
        if( x > 0 )
            vals[x][y] += .4 * vals[x-1][y];
        if( y > 0 )
            vals[x][y] += .4 * vals[x][y-1];
    }
    info->x++;
    if( info->x >= res ) {
        this->calcNormVals();
        this->findMaxMinPoints();
        delete info;
        *v = NULL;
    }
}
//-----------------------------------------------------------------------------//
void nIM::clearVals(){
    if( vals == 0 ) return;
    for( int x = 0; x < res; x ++ )
        for( int z = 0; z < res; z ++ )
            vals[x][z] = 0.0f;
}
//-----------------------------------------------------------------------------//
void nIM::calcNormVals(){
    float min, max;
    min = max = vals[0][0];
    //find min/max values
    //n_printf("%s->calcNormVals()\n", this->fn);
    for( int x = 0; x < res; x ++ ) {
        for( int z = 0; z < res; z++ ) {
            if (vals[x][z] * coef < min )      min = vals[x][z] * coef;
            if (vals[x][z] * coef > max )      max = vals[x][z] * coef;
        }
    }
    /*
       float range = max - min;
       if ( range == 0 )                   range = 1;  //divide by zero error if all vals are the same
    //n_printf("nIM(%s).calcNormVals() min:%f max:%f range:%f\n", this->fn, min, max, range );
    float offset;
    if (min >= 0 )
    offset = 0;
    else {
    if( max == 0 )
    offset = -1.0f;
    else if( max < 0 )
    offset = -1.1f;
    else  {
    offset = min / max;
    if( offset < -1.0f )
    offset = max / min;
    }
    }
    log("%s->min:%f max:%f offset:%f\n", this->fn, min, max, offset);
    */
    //n_printf("%s->min:%f max:%f\n", this->fn, min, max);
    for( int x = 0; x < res; x++ ) {
        for( int z = 0; z < res; z++ ) {
            //normVals[x][z] = (vals[x][z] - min) / range + offset;
            if ( vals[x][z] * coef== 0.0 )
                normVals[x][z] = 0.0;
            else if ( vals[x][z] * coef > 0 )
                normVals[x][z] = vals[x][z] * coef/ max;
            else
                normVals[x][z] = -1.0f * vals[x][z] * coef / min;
            //n_printf("vals[%i][%i] = %f\n", x, z, vals[x][z]);
            //normVals[x][z] = (vals[x][z] - min) / range + offset;
        }
    }
}
//-----------------------------------------------------------------------------//
void nIM::findMaxMinPoints(){
    //int minx, miny, maxx, maxy;
    minx = miny = maxx = maxy = 0;
    for( int x = 0; x < res; x ++ ) {
        for( int y = 0; y < res; y++ ) {
            if( vals[x][y] < vals[minx][miny] ) {
                minx = x;
                miny = y;
            }
            if( vals[x][y] > vals[maxx][maxy] ) {
                maxx = x;
                maxy = y;
            }
        }
    }
    minLocation.SetV( points[minx][miny] );
    maxLocation.SetV( points[maxx][maxy] );
}
//-----------------------------------------------------------------------------//
const char* nIM::GetBehavior(){
    if ( this->behavior != "none" ) //easy i have a behavior set
        return this->behavior.Get();
    else
        return "none";
    return "none";
}
