#define N_IMPLEMENTS nIMObjective
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "im/nimobjective.h"
#include "enkiduo/neventserver.h"
#include "im/nallocator.h"
#include "enkiduo/nprofserver.h"

nNebulaScriptClass(nIMObjective, "nimoperator");

//------------------------------------------------------------------------------
nIMObjective::nIMObjective(){
    this->imAssignEvent = 0;
    this->op = MAX;
    this->mode = NORMAL;
}
//------------------------------------------------------------------------------
void nIMObjective::Initialize(){
    nIMOperator::Initialize();
}
//------------------------------------------------------------------------------
void nIMObjective::InitVars(){
    nIMOperator::InitVars();
    objectivePointDistanceThreshold.Connect( ent, "objectivePointDistanceThreshold");
    objectivePointDistanceThreshold.DefaultF( (worldMax.GetV().x - worldMin.GetV().x )/ 50.0f );
}
//------------------------------------------------------------------------------
nIMObjective::~nIMObjective(){
}
//------------------------------------------------------------------------------
void nIMObjective::Tick( float time, float dtime ){
    nIMOperator::Tick( time, dtime );
}
enum States {
    LET_OPERATOR_RUN,
    FIND_CHILD,
    CALCULATE_OBJECTIVE_POINTS,
    ALIGN_OBJECTIVE_POINTS,
    TRIGGER_EVENT,
};
struct FuserInfo {
    States state;
    nIM* maxIM;
    nIM* minIM;
    float objectivePointValueThreshold;
    FuserInfo(){
        state = LET_OPERATOR_RUN;
        maxIM = 0;
        minIM = 0;
        operatorInfo = new void*();
        objectivePointValueThreshold = 0.0f;
    }
    ~FuserInfo(){
        delete operatorInfo;
    }
    void** operatorInfo;
    float adjustedObjectivePointDistanceThreshold;
};
//-----------------------------------------------------------------------------//
void nIMObjective::calculateValues(void** const v){
    prof->Begin("nIMObjective");
    FuserInfo* info = (FuserInfo*) (*v);
    //n_printf("calcVals:%p\n", v);
    if ( *v == NULL ) {
        info = new FuserInfo();
        *v = info;
    }
    //n_printf("nIMObjective::calculateValues() state=%i\n", info->state);
    if( info->state == LET_OPERATOR_RUN ){
        nIMOperator::calculateValues( info->operatorInfo );
        if( *info->operatorInfo == 0 )
            info->state = FIND_CHILD;
    }
    if ( info->state == FIND_CHILD ) {
        //CALCULATE POINT
        float minValFound, maxValFound;
        int minxFound, minyFound, maxxFound, maxyFound;

        bool first = true;
        foreachchild(nIM, im, this){
            int minx, miny, maxx, maxy;
            im->getMinMaxCoords(minx, miny, maxx, maxy);
            float minVal = im->getFinalVal(minx, miny);
            float maxVal = im->getFinalVal(maxx, maxy);
            //n_printf("child:%s\n", im->fn);
            //n_printf("    min:[%i,%i]->%f\n", minx, miny, minVal);
            //n_printf("    max:[%i,%i]->%f\n", maxx, maxy, maxVal);
            if ( info->minIM == 0 || minVal < minValFound ) {
                minValFound = minVal;
                minxFound = minx;
                minyFound = miny;
                info->minIM = im;
            }
            if ( info->maxIM == 0 || maxVal > maxValFound ) {
                maxValFound = maxVal;
                maxxFound = maxx;
                maxyFound = maxy;
                info->maxIM = im;
            }
        }
        //n_printf("min:[%i,%i]->%f\n", minxFound, minyFound, minValFound);
        //n_printf("max:[%i,%i]->%f\n", maxxFound, maxyFound, maxValFound);
        //this->clearVals();
        //this->vals[minxFound][minyFound] = minValFound;
        //this->vals[maxxFound][maxyFound] = maxValFound;
        //this->calcNormVals();
        //this->findMaxMinPoints();
        info->state = CALCULATE_OBJECTIVE_POINTS;
        info->adjustedObjectivePointDistanceThreshold = objectivePointDistanceThreshold.GetF();
        //n_printf("initial adjustedObjectivePointDistanceThreshold:%f\n", info->adjustedObjectivePointDistanceThreshold);
        this->objectivePoints.Clear();
        //n_printf("%s finding objective points\n", str(this));
    } else if ( info->state == CALCULATE_OBJECTIVE_POINTS ){
        /**
        okay so now we want to calcualte our objective points, which are high points on the map to which units could be allocated
        Well now how can we do this...well we could use populations of hill climbers =) lol...
        We could find the highest point, and then do a depression on the rest of the map
        Balancing points via a spring algorithm like technique is generally the best in my experience
        okay so how about this

        okay so lets try this..we search the map, find the highest point, add a point
        then we keep repeating this, so long as the points are atleast some distance away from each other
        then we do a repeat spring algorithm on each of the points, until they line up nicely
        */
        //float objectivePointDistanceThreshold = this->objectivePointDistanceThreshold.GetF();
        float minVal, maxVal;
        vector3 minCoord, maxCoord;
        bool first = true;
        bool farEnoughAway;
        for( int x = 0; x < res; x ++ ) {
            for( int y = 0; y < res; y ++ ){
                if( vals[x][y] <= info->objectivePointValueThreshold && this->objectivePoints.Size() > 0 )
                    continue;
                farEnoughAway = true;
                for( int i = 0; i < this->objectivePoints; i ++ ) {
                    //n_printf("points[%i][%i] = %s objectivePoints[%i] = %s dist:%f thresh:%f\n", 
                            //x, y, str(points[x][y]), i, str(objectivePoints[i]), (points[x][y] - objectivePoints[i]).len(), objectivePointThreshold);
                    if( (points[x][y] - this->objectivePoints[i]).len() < info->adjustedObjectivePointDistanceThreshold ){
                        farEnoughAway = false;
                    }
                }
                if( farEnoughAway ) {
                    //n_printf("x:%i y:%i val:%f point:%s minVal:%f maxVal:%f minCoord:%s maxCoord:%s\n", x, y, vals[x][y], str(points[x][y]), minVal, maxVal, str(minCoord), str(maxCoord));
                    if( first || vals[x][y] < minVal ) {
                        minVal = vals[x][y];
                        minCoord = points[x][y];
                    }
                    if( first || vals[x][y] > maxVal ) {
                        maxVal = vals[x][y];
                        maxCoord = points[x][y];
                    }
                    first = false;
                }
            }
        }
        if( first == false ) { //successfully found atleast 1 point
            //n_printf("    new objective point:%s\n", str( maxCoord ));
            //if its the first point set the threshold
            if( this->objectivePoints.Size() == 0 ) {
                info->objectivePointValueThreshold = maxVal * .75f;
                //n_printf("objectivePointValueThreshold: %f\n", info->objectivePointValueThreshold );
            }
            this->objectivePoints.Append( maxCoord );
            info->adjustedObjectivePointDistanceThreshold *= 1.2f;
            //n_printf("adjusted adjustedObjectivePointDistanceThreshold:%f\n", info->adjustedObjectivePointDistanceThreshold);
        } else {
            //n_printf("found all objective points\n");
            info->state = ALIGN_OBJECTIVE_POINTS;
        }
        //would be something along the lining the spring theorem or somethin to better line up the poits
    } else if (info->state == ALIGN_OBJECTIVE_POINTS ) {
        //skip this for now
        info->state = TRIGGER_EVENT;
    } else if (info->state == TRIGGER_EVENT ) {
        //now we have to actually assign the variables to our classes / determine the behavior
        bool takeMax = true;
        if ( takeMax ) {
            const char* behavior = info->maxIM->GetBehavior();
            //n_printf("taking behavior(%s) from maxIM(%s)\n", behavior, maxIM->fn);
            vector3 point = info->maxIM->getPoint(maxx, maxy);
            //n_printf("point:%s\n", str(point));
            ifvalid(ref_eventServer){
                nString event;
                event.PrintF("%s;%s;%s", ent->fn, behavior, str(point));
                ref_eventServer->pull(imAssignEvent, "imAssign", event.Get());
            }
            //trigger our allocator to reallocate things
            allocator->Allocate( this );
        }
        delete info;
        *v = NULL;
    }
    prof->End("nIMObjective");
}
void nIMObjective::Display( nGfxServer* gfx ){
    nIM::Display( gfx );
    for( int i = 0; i < objectivePoints; i ++ ){
        vector3 point = objectivePoints[i];
        gfx->Rgba( .8,.1,.1,1.0);
        gfx->Begin(N_PTYPE_LINE_STRIP);
        gfx->Coord( point + vector3(1.0,000.0,0.0) );
        gfx->Coord( point + vector3(1.0,500.0,0.0) );
        gfx->End();
    }
}
