#define N_IMPLEMENTS nBoatAStar
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/nboatastar.h"
#include "lagoon/nboathelmsman.h"
#include "lagoon/nboatengine.h"
#include "physics/nphysics.h"

nNebulaScriptClass(nBoatAStar, "nastar");

//------------------------------------------------------------------------------
nBoatAStar::nBoatAStar() {
    entDriving = 0;
}
//------------------------------------------------------------------------------
void nBoatAStar::Initialize(){
    nAStar::Initialize();
}
//------------------------------------------------------------------------------
void nBoatAStar::InitVars(){
    nAStar::InitVars();
}
//------------------------------------------------------------------------------
nBoatAStar::~nBoatAStar(){
}
//------------------------------------------------------------------------------
void nBoatAStar::Tick( float time, float dtime ){
    nAStar::Tick( time, dtime );
    this->time = time;
}

void nBoatAStar::FindPath( const char* path, vector3 pos, vector3 vel ){
    nEnt* e = (nEnt*) kernelServer->Lookup(path);
    this->entDriving = e;
    if ( e == 0 ) {
        n_printf("trying to find a path for a nonexistant ent:%s->%s,%s\n", path, str(pos), str(vel) );
        return;
    } else {
        this->FindPath( e, pos, vel );
    }
}
void nBoatAStar::FindPath( nEnt*e, vector3 pos, vector3 vel ) {
    n_printf("nBoatAStar::FindPath(%s)->%s,%s)\n", e->fn, str(pos), str(vel));
    for( int i = 0; i < nodes; i ++ )
        delete nodes[i];
    nodes.Clear();
    entsAvoiding.Clear();
    nRoot* root = kernelServer->Lookup("/game/ents");
    foreachchild( nRoot, entType, root ) {
        foreachchild( nEnt, ent, entType ) {
            if ( ent != e ) {
                entsAvoiding.Append( ent );
            }
        }
    }

    n_printf("ents avoiding\n");
    nEnt* ea;
    foreach( ea, entsAvoiding ) {
        n_printf("   %s\n", ea->fn);
    }

    BoatAStarNode* start = new BoatAStarNode( this );
    nodes.Append( start );
    start->Set( e->GetVarV("pos"), e->GetVarV("vel"), e->GetVarQ("rot"), e->GetVarQ("rvel"), time);

    BoatAStarNode* goal = new BoatAStarNode( this );
    nodes.Append( goal );
    quaternion goalRot;
    goalRot.set_rotate_y(3.1415);
    quaternion goalRVel;
    goalRVel.set(0,0,0,0);
    goal->Set( pos, vel, goalRot, goalRVel, time );

    AStarPath path = nAStar::FindPath( (AStarNode*) start, (AStarNode*) goal );
}

/**
 * check if to states are equal -> generally this versus the goal state
 * since we have a nice fuzzy world we need some kinda threshold here
 **/
bool BoatAStarNode::operator == ( const AStarNode& aneighbor ) {
    const BoatAStarNode* neighbor = dynamic_cast<const BoatAStarNode*>(&aneighbor);
    float distanceCost = (neighbor->pos - this->pos).len();
    //n_printf("distanceCost:%f\n", distanceCost);
    return distanceCost < 50.0f;
}
//-----------------------------------------------------------------------------//
//-----------------------------------------------------------------------------//
//-----------------------------------------------------------------------------//
/**
 * figure out howm uch it will cost to get from here to a neighbor
 * this cost should include actual travel time, fuel costs as well as undesireable behavior penalties
 * including: going near other boats, particularily in front of them, as well as going near land
 **/
float BoatAStarNode::CalculateCostTo( AStarNode* aneighbor ) {
    BoatAStarNode* neighbor = dynamic_cast<BoatAStarNode*>(aneighbor);

    float distanceCost = (neighbor->pos - this->pos).len();

    float nearOtherBoatsCost = 0.0f;
    EntArray& entsAvoiding = ((nBoatAStar*) (server))->entsAvoiding;
    nEnt* e;
    foreach( e, entsAvoiding ) {
        vector3 pos = e->GetVarV("pos");
        float d = (pos - this->pos).len();
        float nearCost = 0;
        float relavantDistance = 1000.0f;
        if ( d > relavantDistance ) 
            nearCost = 0;
        else { //pow 2 approcah to max cost of 10 per boat
            nearCost = (relavantDistance-d) / relavantDistance;
            //nearCost *= nearCost;
            nearCost = pow(nearCost, 10);
            nearCost = nearCost * 100.0f;
        }
        nearOtherBoatsCost += nearCost;
    }

    neighbor->distanceCost              = distanceCost;
    neighbor->nearOtherBoatsCost        = nearOtherBoatsCost;
    return distanceCost + nearOtherBoatsCost;
}
/**
 * give a rough estimate of how close this state is to some other state, usually the goal state
 * just for ageneral idea which way is towards the goal
 **/
float BoatAStarNode::EstimateCostTo( AStarNode* anode ) {
    BoatAStarNode* node = dynamic_cast<BoatAStarNode*>(anode);
    n_assert( node );

    vector3 pdiff                       = node->pos - this->pos;
    float distCost = pdiff.len();                                               //* (1.0 + fabs(otherHeading - myHeading));
    vector3 forward                     = this->rot.rotate(vector3(0,0,-1));
    pdiff.norm();
    vector3 pdiffCross                  = forward * pdiff;
    float pdiffDot                      = forward % pdiff;

    nBoatAStar* astarServer = dynamic_cast<nBoatAStar*>(this->server);
    n_assert( astarServer );
    //float similarityCost = astarServer->GetSimilarityCost( this->time );
    //n_printf("looking up for:%f\n", this->time);

    //n_printf("pdiff:%s forward:%s cross:%s dot:%f\n", str(pdiff), str(forward), str(pdiffCross), pdiffDot);
    //float myHeading                     = atan2(this->vel.x, this->vel.z);
    //float otherHeading                  = atan2(node->vel.x, node->vel.z);
    distCost *= .2f * (1.0f - pdiffDot) + 1.0f;                                       //increase the distance if its not dead ahead
    float timeCost = fabs((node->time - this->time - node->time));

    return 1.0f * (distCost + timeCost);
}

/**
 * draw me to the screen -> I only have the gfx server pointer so this will be a lil crude
 **/
void BoatAStarNode::Display( nGfxServer* gfx ){
    gfx->Begin( N_PTYPE_LINE_LIST );
    if ( this->expanded )
        gfx->Rgba( 1,1,1,1 );
    else
        gfx->Rgba( 1,1,0,1 );

    gfx->Coord( pos );
    gfx->Coord( pos + vel );

    gfx->Coord( pos );
    gfx->Coord( pos + rot.rotate(vector3(0,0,-10)));

    //line to my parent
    if( this->parent ) {
        gfx->Rgba( 0,0,1,1 );
        gfx->Coord( pos );
        gfx->Coord( ((BoatAStarNode*) (parent))->pos );
    }

    if ( this->expanded )               gfx->Rgba( 0,1,1,1 );
    else                                gfx->Rgba( 0,1,0,1 );
    gfx->Coord( pos + vector3(-.1,0,0) );
    gfx->Coord( pos + vector3(-.1,this->distanceCost / 10.0,0));

    gfx->Coord( pos + vector3(+.1,0,0) );
    gfx->Coord( pos + vector3(+.1,this->nearOtherBoatsCost / 10.0,0));

    gfx->End();
}

/**
 * return an array with all my neighbor nodes - for this case we are going to dynamically generate them, append them onto the list of coordinates and return them
 * so how do we do tihs? okay well we take our current physics situation, tweak our paramaeters and run the model
 **/
nArray<AStarNode*> BoatAStarNode::GetNeighbors(){
    nEnt*& entDriving = ((nBoatAStar*) (server))->entDriving;
    n_assert( entDriving );
    bool display = roll(0.0);

    nArray<AStarNode*> neighbors;
    BoatAStarNode* node;
    nBoatHelmsman* helmsman;
    nBoatEngine* engine;
    nPhysics* physics;

    helmsman                            = (nBoatHelmsman*) entDriving->Find("helmsman");
    engine                              = (nBoatEngine*)   entDriving->Find("engine");
    physics                             = (nPhysics*)      entDriving->Find("physics");
    n_assert( helmsman && engine && physics );
    if (display)
        n_printf("helmsman:%s engine:%s physics:%s\n", helmsman->fn, engine->fn, physics->fn);

    //now we want to adjust the input parameters and see what kind of output we get
    vector3 forward(0,0,-1);
    forward = this->rot.rotate(forward);
    float currentHeading = atan2(forward.x, forward.z);
    nArray<float> desiredHeadings;
    currentHeading = n_rad2deg(currentHeading);
    desiredHeadings.Append(currentHeading);
    desiredHeadings.Append(currentHeading + 90);
    desiredHeadings.Append(currentHeading - 90);
    nArray<float> desiredSpeeds; 
    //desiredSpeeds.Append(10);
    desiredSpeeds.Append(0);
    desiredSpeeds.Append(100);

    float desiredHeading;
    float desiredSpeed;
    vector3 currentPos                  = this->pos;
    vector3 currentVel                  = this->vel;
    quaternion currentRot               = this->rot;
    quaternion currentRVel              = this->rvel;

    float mass                          = entDriving->GetVarF("mass");
    float inertia                       = entDriving->GetVarF("inertia");
    float friction                      = entDriving->GetVarF("friction");
    float rfriction                     = entDriving->GetVarF("rfriction");
    if (display)
        n_printf("node:%s\n", str(this));

    const float timeStep = 30.0f;
    const int numSteps = 100;
    const float dtime = timeStep / numSteps;
    foreach( desiredSpeed, desiredSpeeds ) {
        foreach( desiredHeading, desiredHeadings){
            entDriving->                SetVarF("desiredHeading", desiredHeading);
            entDriving->                SetVarF("desiredSpeed", desiredSpeed);
            entDriving->                SetVarV("pos", currentPos);
            entDriving->                SetVarV("vel", currentVel);
            entDriving->                SetVarQ("rot", currentRot);
            entDriving->                SetVarQ("rvel", currentRVel);
            for ( int i = 0; i < numSteps; i ++ ) {
                helmsman->              Tick(0,dtime);
                engine->                Tick(0,dtime);
                physics->               Tick(0,dtime);
            }
            vector3 newPos              = entDriving->GetVarV("pos");
            vector3 newVel              = entDriving->GetVarV("vel");
            quaternion newRot           = entDriving->GetVarQ("rot");
            quaternion newRVel          = entDriving->GetVarQ("rvel");

            node = new BoatAStarNode(this->server);
            node->SetParent( this );
            node->Set( newPos, newVel, newRot, newRVel, this->time + dtime*numSteps );
            server->AppendNode( node );
            neighbors.Append(node);
        }
    }
    return neighbors;
}
//-----------------------------------------------------------------------------//
