#define N_IMPLEMENTS nBoatEngine
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/nboatengine.h"

nNebulaScriptClass(nBoatEngine, "nphysicalaspect");

//------------------------------------------------------------------------------
nBoatEngine::nBoatEngine() {
}
//------------------------------------------------------------------------------
void nBoatEngine::Initialize(){
    nPhysicalAspect::Initialize();
    //this->AddToDisplayList();
}
//------------------------------------------------------------------------------
void nBoatEngine::InitVars(){
    nPhysicalAspect::InitVars();
    rot.Connect                         ( ent, "rot" );
    vel.Connect                         ( ent, "vel" );
    throttle.Connect                    ( ent, "throttle" );
    rudder.Connect                      ( ent, "rudder" );
    enginePower.Connect                 ( ent, "enginePower" );
    rudderTorque.Connect                ( ent, "rudderTorque" );
    engineRudderDissipation.Connect     ( ent, "engineRudderDissipation" );
    keelForce.Connect                   ( ent, "keelforce" );

    rot.DefaultQ                        ( 0,0,0,1 );
    vel.DefaultV                        ( 0,0,0 );
    throttle.DefaultF                   ( 0.0f );
    rudder.DefaultF                     ( 0.0f );
    enginePower.DefaultF                ( 1.0f );
    rudderTorque.DefaultF               ( 1.0f );
    engineRudderDissipation.DefaultF    ( 0.0f );
    keelForce.DefaultF                  ( 10.0f );

    mass.Connect                        ( ent, "mass" );
    friction.Connect                    ( ent, "friction" );
    //    friction.SetF                       ( 0.0f );
    maxSpeed.Connect                    ( ent, "maxSpeed" );
    maxSpeed.DefaultF                   ( 1.0f );
}
//------------------------------------------------------------------------------
nBoatEngine::~nBoatEngine(){
}
//------------------------------------------------------------------------------
void nBoatEngine::Tick( float time, float dtime ){
    nPhysicalAspect::Tick( time, dtime );
    n_assert( physics.isvalid() );

    if( mass.isdirty() || enginePower.isdirty() || friction.isdirty() ) {
        //printf("%s calculating maxSpeed\n", str(this));
        maxSpeed.SetF( enginePower.GetF() / ( mass.GetF() * friction.GetF() ) );
    }

    vector3 v                           = vel.GetV();
    quaternion q                        = rot.GetQ();
    vector3 forward                     = q.rotate(vector3(0, 0,-1));
    vector3 right                       = q.rotate(vector3(1, 0, 0));
    //how much force we are getting from the engine
    
    float force                         = n_clamp(throttle.GetF(), -1.0f, 1.0f) * enginePower.GetF() * dtime;
    //calculate keel forces
    float velIntoKeel                   = right % v;
    //float keelF                         = -1.0f * velIntoKeel * keelForce.GetF();
    float keelF                         = -1.0f * velIntoKeel * keelForce.GetF() * dtime;
    //rudder effectiveness should be a summation of engine force and actual velocity
    float inlineVel                     = forward % v;
    float velOverRudder                 = force * (1.0f - engineRudderDissipation.GetF()) + inlineVel;
    //float torque                        = velOverRudder * rudder.GetF() * rudderTorque.GetF();
    float torque                        = velOverRudder * rudder.GetF() * rudderTorque.GetF() * dtime;

    //n_printf("speed: %f, inlineVel:%f\n", ent->GetVarF("speed"), inlineVel);
    //    if (inlineVel > .99f * maxSpeed.GetF() && throttle.GetF() > 0.0){
    //      force = 0.0;
    //    }
    if( this->displayed ) {
        vector3 p = ent->GetVarV("pos");
        this->DrawLine(p, p + 100.0f * forward, white, -1);
        this->DrawLine(p, p + 100.0f * forward * inlineVel, blue);
        this->DrawLine(p, p + 100.0f * right * torque, red);
        this->DrawLine(p, p + 100.0f * right * keelF, green, -30);
        this->DrawLine(p, p + 100.0f * forward * (-1 * velOverRudder), yellow);
    }

    physics->ApplyAlignedForce          (vector3(keelF, 0, -force));
    physics->ApplyAlignedTorque         (vector3(0, torque, 0));
}
