#define N_IMPLEMENTS nBoatHelmsman
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/nboathelmsman.h"
#include "node/nlinknode.h"
#include "node/nmeshnode.h"
#include "gfx/nvertexbuffer.h"

nNebulaScriptClass(nBoatHelmsman, "nai");

//------------------------------------------------------------------------------
nBoatHelmsman::nBoatHelmsman() {
}
//------------------------------------------------------------------------------
void nBoatHelmsman::Initialize(){
    nAI::Initialize();
    stateNames.Append("halt");
    stateNames.Append("forward");
    stateNames.Append("steering");
    stateNames.Append("reverseGlideSteering");
    stateNames.Append("reverseTurn");
    stateNames.Append("reverseGlideTurn");
    this->AddToDisplayList();
}
//------------------------------------------------------------------------------
void nBoatHelmsman::InitVars(){
    nAI::InitVars();
    desiredHeading.Connect              (ent, "desiredHeading");
    desiredSpeed.Connect                (ent, "desiredSpeed");
    throttle.Connect                    (ent, "throttle");
    rudder.Connect                      (ent, "rudder");
    reverseThreshold.Connect            (ent, "reverseThreshold");
    speed.Connect                       (ent, "speed");
    heading.Connect                     (ent, "heading");
    maxSpeed.Connect                    (ent, "maxSpeed");
    cameraDist.Connect                  (ent, "cameraDist");
    desiredHeading.DefaultF             (0.0f);
    desiredSpeed.DefaultF               (0.0f);
    throttle.DefaultF                   (0.0f);
    rudder.DefaultF                     (0.0f);
    reverseThreshold.DefaultF           (5.0f);                                 //disabled
    speed.DefaultF                      (0.0f);
    heading.DefaultF                    (0.0f);
    maxSpeed.DefaultF                   (1.0f);
    cameraDist.DefaultF                 (100.0f);
    ent->ExportChannel                  ("cameraDist");
    camera.Connect                      (ent, "camera");
    camera.DefaultRef                   ("/game/camera");
    cameraPos.Connect                   (camera, "pos");
    dimension.Connect                   (ent, "dimension");
    dimension.DefaultV                  (100,100,100);
    side.Connect                        (ent, "side");
    sideHandle.Connect                  (ent, "sideHandle");
    sideHandle.DefaultI                 (0);
    //inertia.Connect                     (ent, "inertia");
    //inertia.DefaultF                    (1.0f);

    flcoord.Connect(ent, "flcoord");
    frcoord.Connect(ent, "frcoord");
    rlcoord.Connect(ent, "rlcoord");
    rrcoord.Connect(ent, "rrcoord");
    flcoord.DefaultV(0,0,0);
    frcoord.DefaultV(0,0,0);
    rlcoord.DefaultV(0,0,0);
    rrcoord.DefaultV(0,0,0);


    bbmin.Connect                       (ent, "bbmin");
    bbmax.Connect                       (ent, "bbmax");
    vector3 min, max;

    //TODO - explore optomizations - in particular not locking / unlocking the buffer
    n3DNode* vis                        = ent->GetVis();
    if ( vis == 0 )                     { n_printf("no vis\n");                 return;}
    nLinkNode* link                     = (nLinkNode*) vis->Find("link");
    if ( link == 0 )                    { n_printf("no link\n");                return;}
    nVisNode* node                       = link->GetTargetPtr();
    if ( node == 0 )                    { n_printf("no node\n");                return;}
    nMeshNode* mesh                     = (nMeshNode*) node->Find("mesh");
    if ( mesh == 0 )                    { n_printf("no mesh\n");                return;}
    nVertexBuffer* buffer               = mesh->GetVertexBuffer();
    if ( buffer == 0 ) {                n_printf("no buffer\n");                return;}
    int numCoords                       = buffer->GetNumVertices();

    int coordIndex                      = randI( 0, numCoords-1);
    bool first = true;
    buffer->LockVertices();
    for( int i = 0; i < numCoords; i ++ ) {
        vector3 coord = buffer->GetCoord(i);
        if ( first || coord.x < min.x ) min.x = coord.x; 
        if ( first || coord.y < min.y ) min.y = coord.y; 
        if ( first || coord.z < min.z ) min.z = coord.z; 
        if ( first || coord.x > max.x ) max.x = coord.x; 
        if ( first || coord.y > max.y ) max.y = coord.y; 
        if ( first || coord.z > max.z ) max.z = coord.z; 
        first = false;
    }
    buffer->UnlockVertices();
    bbmin.SetV(min);
    bbmax.SetV(max);
}
//------------------------------------------------------------------------------
nBoatHelmsman::~nBoatHelmsman(){
}
//------------------------------------------------------------------------------
bool nBoatHelmsman::States( StateMachineEvent event, uint state, float time, float dtime ){
    if( side.isdirty() )
        sideHandle.SetI( (float) ent->GetSideHandle());
    quaternion q                        = rot.GetQ();
    vector3 forward                     = q.rotate(vector3(0,0,-1));
    float heading                       = atan2(forward.x, forward.z);
    float dheading                      = desiredHeading.GetF() - heading;
    vector3 v                           = vel.GetV();
    float inlineVel                     = forward % v;
    while( dheading > +N_PI ) dheading -= N_PI * 2.0f;
    while( dheading < -N_PI ) dheading += N_PI * 2.0f;
    float desiredThrottle               = desiredSpeed.GetF() / maxSpeed.GetF();
    float speed                         = v.len();
    //if ( speed > desiredSpeed.GetF())
        //desiredThrottle = -0.3f;
        
    //stuff ryan wants exported
    this->heading.SetF                  (heading);
    this->speed.SetF                    (speed);

    vector3 min, max;
    min = bbmin.GetV();
    max = bbmax.GetV();
    vector3 fl(min.x, 0, min.z);
    vector3 fr(max.x, 0, min.z);
    vector3 rl(min.x, 0, max.z);
    vector3 rr(max.x, 0, max.z);
    vector3 p = pos.GetV();
    fl = p + q.rotate(fl);
    fr = p + q.rotate(fr);
    rl = p + q.rotate(rl);
    rr = p + q.rotate(rr);

    ifvalid(camera)
        cameraDist.SetF( (cameraPos.GetV() - pos.GetV()).len());
    //physics->ApplyAlignedTorque(vector3(.00001 * sin(time/20.0f) * inertia.GetF(),0, .00005f * sin(time / 30.0f) * inertia.GetF()));
    
    if( this->displayed ) {
        //this->DrawPoint( fl, blue );
        //this->DrawPoint( fr, white );
        //this->DrawPoint( rl, red );
        //this->DrawPoint( rr, green );

        //this->DrawAngle( pos.GetV(), heading, white );
        this->DrawAngle( pos.GetV(), heading + dheading, blue, desiredSpeed.GetF() * 10.0f );
        //this->DrawAngle( pos.GetV(), heading + reverseThreshold.GetF(), green );
        //this->DrawAngle( pos.GetV(), heading - reverseThreshold.GetF(), green );
    }
    flcoord.SetV(fl);
    frcoord.SetV(fr);
    rlcoord.SetV(rl);
    rrcoord.SetV(rr);


    BeginStateMachine
        //-----------------------------------------------------------------------------//
        State(Halt)                     //hold position
            OnEnter
                log("nBoatHelmsman.Halt\n");
            OnTick
                if(desiredSpeed.GetF()  != 0.0f)                                this->SetState(Forward);
                throttle.SetF           (0.0f);
                rudder.SetF             (0.0f);
        //-----------------------------------------------------------------------------//
        State(Forward)                  //all ahead full
            OnEnter
                log("nBoatHelmsman.Forward\n");
            OnTick
                if(desiredSpeed.GetF() == 0.0f)                                 this->SetState(Halt);
                else if(fabs(dheading) > .05)                                   this->SetState(Steering);
                throttle.SetF           (desiredThrottle);
                rudder.SetF             (0.0f);
        //-----------------------------------------------------------------------------//
    State(Steering)                 //push the throttle aim at your destination
        OnEnter
            log("nBoatHelmsman.Steering\n");
        OnTick
            if(desiredSpeed.GetF() == 0.0f)                                     this->SetState(Halt);
            else if(fabs(dheading) < .03)                                       this->SetState(Forward);
            else if(fabs(dheading) > reverseThreshold.GetF())                   this->SetState(ReverseTurn);
            else if(inlineVel < 0.0f)                                           this->SetState(ReverseGlideSteering);
            throttle.SetF               (desiredThrottle);
            rudder.SetF                 (dheading);
    //-----------------------------------------------------------------------------//
    State(ReverseGlideSteering)     //coming out of a reverse turn - rudder goes the other way
        OnEnter
            log("nBoatHelmsman.ReverseGlideSteering\n");
        OnTick
            if(desiredSpeed.GetF() == 0.0f)                                     this->SetState(Halt);
            else if( inlineVel > 0.1f )                                         this->SetState(Steering);
            else if(fabs(dheading) > reverseThreshold.GetF())                   this->SetState(ReverseTurn);
            throttle.SetF               (1.0f);
            rudder.SetF                 (-dheading);
    //-----------------------------------------------------------------------------//
    State(ReverseTurn)              //full reverse turn towards
        OnEnter
            log("nBoatHelmsman.ReverseTurn\n");
        OnTick
            if(desiredSpeed.GetF() == 0.0f)                                     this->SetState(Halt);
            else if(inlineVel > 0.1f)                                           this->SetState(ReverseGlideTurn);
            else if(fabs(dheading) < reverseThreshold.GetF() * .9f)             this->SetState(Steering);
            throttle.SetF               (-1.0f);
            rudder.SetF                 (dheading > 0 ? -1.0f : +1.0f);
    //-----------------------------------------------------------------------------//
    State(ReverseGlideTurn)         //full reverse but still turn regular as we are gliding forwards
        OnEnter
            log("nBoatHelmsman.ReverseGlideTurn\n");
        OnTick
            if(desiredSpeed.GetF() == 0.0f)                                     this->SetState(Halt);
            else if (inlineVel < 0.0f)                                          this->SetState(ReverseTurn);
            else if (fabs(dheading) < reverseThreshold.GetF() * .9f)            this->SetState(Steering);
            throttle.SetF               (-1.0f);
            rudder.SetF                 (dheading > 0 ? +1.0f : -1.0f);
    //-----------------------------------------------------------------------------//
    EndStateMachine
}

bool nBoatHelmsman::Select(){
    this->AddToDisplayList();
}
bool nBoatHelmsman::DeSelect(){
    this->RemoveFromDisplayList();
}
