#define N_IMPLEMENTS nCarPhysics
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/ncarphysics.h"
#include "node/nlinknode.h"

nNebulaScriptClass(nCarPhysics, "nodephysics");

//------------------------------------------------------------------------------
/**
*/
nCarPhysics::nCarPhysics():
    refMap ( kernelServer, this ),
    gameServer (kernelServer, this )
{
    gameServer = "/sys/servers/game";
}

void nCarPhysics::Initialize(){
    nODEPhysics::Initialize();
    refMap = "/map/map";
    this->AddToDisplayList();
}

void nCarPhysics::InitVars(){
    nPhysics::InitVars();
    //state
    wheelDirection.Connect	( ent, "wheeldirection" );    	wheelDirection.DefaultF	( 0.0f );
    //controls
    steering.Connect( ent, "steering" );    		steering.DefaultF( 0.0f );
    throttle.Connect( ent, "throttle" );		throttle.DefaultF( 0.0f );

    //body dimensions
    width.Connect	( ent, "width"  );    	width.DefaultF	( 1.0f );
    height.Connect	( ent, "height" );	height.DefaultF	( 1.0f );
    depth.Connect	( ent, "depth"  );    	depth.DefaultF	( 1.0f );
    //engine params
    enginePower.Connect	( ent, "enginepower" );         enginePower.DefaultF	( 10.0f );
    engineResistance.Connect	( ent, "engineresistance" );    engineResistance.DefaultF	( 1.0f );
    maxSpeed.Connect	( ent, "maxspeed" );    	maxSpeed.DefaultF	( 10.0f );
    //brake params
    brakePower.Connect	( ent, "brakepower" );          brakePower.DefaultF	( 15.0f );
    //turning params
    turningPower.Connect	( ent, "turningpower" );    	turningPower.DefaultF	( 10.0f );
    turningSpeed.Connect	( ent, "turningspeed" );    	turningSpeed.DefaultF	( 2.0f );
    //wheel params
    wheelRadiusF.Connect	( ent, "wheelradiusf" );    	wheelRadiusF.DefaultF	( 0.5f );
    wheelWidthF.Connect	( ent, "wheelwidthf" );    	wheelWidthF.DefaultF	( 0.5f );
    wheelDensityF.Connect	( ent, "wheeldensityf" );    	wheelDensityF.DefaultF	( 0.1f );

    wheelRadiusR.Connect	( ent, "wheelradiusr" );    	wheelRadiusR.DefaultF	( 1.5f );
    wheelWidthR.Connect	( ent, "wheelwidthr" );    	wheelWidthR.DefaultF	( 0.5f );
    wheelDensityR.Connect	( ent, "wheeldensityr" );    	wheelDensityR.DefaultF	( 0.1f );
    // suspension parameters
    suspERP.Connect	( ent, "suspERP" );	suspERP.DefaultF	( 0.5f );
    suspCFM.Connect	( ent, "suspCFM" );	suspCFM.DefaultF	( 0.001f );
    ///drive trian

    driveTrain.Connect	( ent, "drivetrain");	driveTrain.DefaultI	( 1 );
    var suspCFM;
    
    wheels.SetFixedSize( 4 );
    for( int i = 0; i < 4; i ++ ) {
        wheels[i].ref = gameServer->GenVis( "wheel" );
        n_assert( wheels[i].ref.isvalid() );
    }

    dWorldID world = odeServer->GetWorld();
    //***********************************************************************
    // set up frame body
    //***********************************************************************
    frame = odeServer->CreateBody();
    bodies.Append( frame );
    dReal dq[] = {0,1,0,0};
    dBodySetQuaternion( frame, dq );

    //***********************************************************************
    // set up frame geom
    //***********************************************************************
    frameGeom = dCreateBox 	( odeServer->GetSpace(), 1, 1, 1 );
    geoms.Append( frameGeom );
    dGeomSetBody 	( frameGeom, frame );

    //***********************************************************************
    // set up wheels
    //***********************************************************************
    for( int i = 0; i < 4; i ++ ) {
        wheels[i].body = odeServer->CreateBody();
        bodies.Append( wheels[i].body );

        wheels[i].suspension = dJointCreateHinge2( world, 0 );
        joints.Append( wheels[i].suspension );
        dJointAttach( wheels[i].suspension, frame, wheels[i].body  );
        dJointSetHinge2Axis1( wheels[i].suspension, 0, 1, 0 );
        dJointSetHinge2Axis2( wheels[i].suspension, 1, 0, 0 );
        
        /* SLIDER - works
        joint = dJointCreateSlider( world, 0 ); 
        dJointAttach( joint, body, wheel );
        dJointSetSliderAxis( joint, 0, 0, 1 );
        */
        /*HINGE 1
        joint = dJointCreateHinge( world, 0 );
        dJointAttach( joint, body, wheel );
        dJointSetHingeAnchor( joint, offset.x, offset.y, offset.z );
        dJointSetHingeAxis( joint, 1, 0, 0 );
        */
        ///*HINGE 2
    }
    UpdateFrame();
    UpdateWheels();
    UpdateSuspension();
}

nCarPhysics::~nCarPhysics(){
}
//------------------------------------------------------------------------------
/**
*/

void nCarPhysics::UpdateFrame() {
    float w = width.GetF();
    float h = height.GetF();
    float d = depth.GetF();
    float rf = wheelWidthF.GetF();
    float rr = wheelWidthR.GetF();
    //vector3 p = pos.GetV();
    //quaternion q = rot.GetQ();

    //body mass
    dMass m;
    dMassSetBox 	( &m, 2.0, w, h, d );
    dBodySetMass 	( frame, &m );

    //body collision
    for( int i = 0; i < wheels; i ++ ) {
        vector3 offset;
        switch( i ) {
            case 0: offset.set( -w - rf, -h, -d ); 	break;
            case 1: offset.set(  w + rf, -h, -d ); 	break;
            case 2: offset.set( -w - rr, -h,  d ); 	break;
            case 3: offset.set(  w + rr, -h,  d ); 	break;
        }
        offset /= 2.0f;

        const dReal* dp = dBodyGetPosition	( bodies[0] );
        const dReal* dq = dBodyGetQuaternion	( bodies[0] );
        vector3 p(dp[0], dp[1], dp[2] );
        quaternion q(-dq[2], dq[1], -dq[0], dq[3] );

        this->DrawLine( p, p +  offset, green );
        this->DrawLine( p, p + q.rotate( offset ), blue );
        offset = q.rotate( offset );
        offset += p;

        dBodySetPosition( wheels[i].body, offset.x, offset.y, offset.z );
        dJointSetHinge2Anchor( wheels[i].suspension, offset.x, offset.y, offset.z );
    }
}
void nCarPhysics::UpdateWheels () { 
    float wheelrf = wheelRadiusF.GetF();
    float wheelwf = wheelWidthF.GetF();
    float wheeldf = wheelDensityF.GetF();
    float wheelrr = wheelRadiusR.GetF();
    float wheelwr = wheelWidthR.GetF();
    float wheeldr = wheelDensityR.GetF();
    dMass m;
    for( int i = 0; i < wheels.Size(); i ++ ) {
        if( i < 2 ) {
            dMassSetCylinder( &m, wheeldf, 1, wheelrf, wheelwf );
            dBodySetMass( wheels[i].body , &m );
            wheels[i].radius = wheelrf;
            wheels[i].width = wheelwf;
            wheels[i].ref->Sxyz( wheelwf, wheelrf, wheelrf );
        }
        else {
            dMassSetCylinder( &m, wheeldr, 1, wheelrr, wheelwr );
            dBodySetMass( wheels[i].body , &m );
            wheels[i].radius = wheelrr;
            wheels[i].width = wheelwr;
            wheels[i].ref->Sxyz( wheelwr, wheelrr, wheelrr );
        }
    }
}

void nCarPhysics::UpdateSuspension() {
    for( int i = 0; i < wheels; i ++ ) {
        dJointSetHinge2Param( wheels[i].suspension, dParamSuspensionERP, suspERP.GetF() );
        dJointSetHinge2Param( wheels[i].suspension, dParamSuspensionCFM, suspCFM.GetF() );
        if( i > 1 ) {
            dJointSetHinge2Param( wheels[i].suspension, dParamLoStop, 0 );
            dJointSetHinge2Param( wheels[i].suspension, dParamHiStop, 0 );
        } else {
            dJointSetHinge2Param( wheels[i].suspension, dParamLoStop, -.50 );
            dJointSetHinge2Param( wheels[i].suspension, dParamHiStop, .50 );
        }
    }
}
/**takes the car parameters and drives the car*/
void nCarPhysics::Tick( float time, float dtime ){
    nODEPhysics::Tick( time, dtime );
    if( width.isdirty() || height.isdirty() || depth.isdirty() )
        UpdateFrame();
    if( wheelRadiusF.isdirty() || wheelRadiusR.isdirty() || wheelDensityF.isdirty() ||wheelDensityR.isdirty() )
        UpdateWheels();
    if( wheelWidthF.isdirty() || wheelWidthR.isdirty() ) {
        UpdateFrame(); 
        UpdateWheels();
    }
    if( suspERP.isdirty() || suspCFM.isdirty() )
        UpdateSuspension();

    int dt = driveTrain.GetI();
    float s = steering.GetF();

    float t = throttle.GetF();
    float speed;
    float torque;
    if( t > 0 ) {
        speed = maxSpeed.GetF();
        torque = enginePower.GetF() * t;
    }
    else if( t == 0 ) {
        speed = 0.0;
        torque = engineResistance.GetF();
    }
    else {
        speed = -10.0;
        torque = brakePower.GetF() * t;
    }

    wheelDirection.SetF( -dJointGetHinge2Angle1( joints[0] ) );
    for( int i = 0; i < joints; i ++ ) {
        dJointID& joint = joints[i];
        if( dt == ALLWHEEL || ( dt == FRONTWHEEL && i < 2 ) || ( dt ==  REARWHEEL && i >= 2 ) ) {
            dJointSetHinge2Param (joint,dParamVel2,	-speed);
            dJointSetHinge2Param (joint,dParamFMax2,	torque );
        } else {
            dJointSetHinge2Param( joint, dParamVel2, 0.0 );
            dJointSetHinge2Param( joint, dParamFMax2, engineResistance.GetF() );
        }
        if( i < 2 ) {
            float angleOff = dJointGetHinge2Angle1( joints[i] ) - dJointGetHinge2Angle1( joints[1-i] );
            float goalSpeed = turningSpeed.GetF() * steering.GetF() - angleOff * 100;
            dJointSetHinge2Param (joint,dParamFMax,	turningPower.GetF() + fabs(angleOff) * 100 );
            dJointSetHinge2Param (joint,dParamFudgeFactor,	0.0);
            dJointSetHinge2Param (joint,dParamVel, 	goalSpeed );
        }
    }

    for( int i = 1; i < bodies; i ++ ) { //set the wheel positions
        const dReal* dp = dBodyGetPosition	( bodies[i] );
        const dReal* dq = dBodyGetQuaternion	( bodies[i] );
        vector3 p(dp[0], dp[1], dp[2] 	);
        quaternion q(-dq[2], dq[1], -dq[0], dq[3] 	);

        if( wheels[i-1].ref.isvalid() ) {
            wheels[i-1].pos = p;
            wheels[i-1].rot = q;
            wheels[i-1].ref->Txyz( p.x, p.y, p.z );
            wheels[i-1].ref->Qxyzw( q.x, q.y, q.z, q.w );
        }
    }
    //NEW NEW Way - search for the best pair of points to connect to the ground
    for( int i = 0; i < wheels; i ++ ) {
        Wheel& wheel = wheels[i];
        float r = wheel.radius / 2.0f;
        float t = 0.0f;

        quaternion q = wheels[i].rot;
        vector3 p = wheels[i].pos;
        vector3 v;

        //how do we do this..
        /**
          okay we search until we find one that touches the ground..then we search around it..until we find the deepest to the ground
         */
        struct D {
            float depth;
            vector3 pos;
            D(){ 
	depth = -10000000;
            }
        };
        D deepestLeft, deepestRight;
        for( int sign = -1; sign <= 1; sign += 2 ) {
            float * deepest;
            vector3* deepestPos;
            if ( sign == -1 ) {
	deepest = &deepestLeft.depth;
	deepestPos = &deepestLeft.pos;
            }
            else  {
	deepest = &deepestRight.depth;
	deepestPos = &deepestRight.pos;
            }

            float theta;
            vector3 side( wheel.width / 2.0 * sign, 0, 0 );
            side = q.rotate( side );
            for( theta = 0.0f; theta < 2 * PI; theta += .1 ) {
	v.x = 0.0f; 
	v.y = n_cos(theta) * r;
	v.z = n_sin(theta) * r;
	v = wheels[i].rot.rotate( v );
	v = v + p + side;
	float h;
	if( refMap.isvalid() )
	    h = refMap->GetHeight( v.x, v.z );
	else
	    h = 0.0f;
	float depth = h - v.y;
	if( depth > *deepest ) {
	    *deepest = depth;
	    *deepestPos = v;
	}
            }
        }
        bool print = roll( .01 );
        float otherTolerance = 0.00;
        int sign;
        if( !(i%2) )
            sign = -1;
        else
            sign = 1;
        if( deepestLeft.depth > 0 || deepestRight.depth > 0 ) { //one of the tires is really contacting
            if( deepestLeft.depth > -otherTolerance ) {
	vector3 v = deepestLeft.pos;
	vector3 side ( sign * wheel.width, 0, 0 );
	side = q.rotate( side );
	vector3 normal;
	if( refMap.isvalid() )
	    refMap->GetNormal( v.x, v.z, normal );
	else
	    normal.set( 0,1,0 );
	//this->DrawLine( v, v + normal * 10, blue );
	odeServer->AddContact( bodies[i+1], 0, v, normal, deepestLeft.depth, vector3(0,0,1) );
            }
            if( deepestRight.depth > -otherTolerance ) {
	vector3 v = deepestRight.pos;
	vector3 side (sign * wheel.width, 0, 0 );
	side = q.rotate( side );
	vector3 normal;
	if( refMap.isvalid() )
	    refMap->GetNormal( v.x, v.z, normal );
	else
	    normal.set( 0,1,0 );
	//this->DrawLine( v, v + normal * 10, blue );
	odeServer->AddContact( bodies[i+1], 0, v, normal, deepestRight.depth, vector3(0,0,1) );
            }
        }
    }
}

void nCarPhysics::Halt() {
    n_printf("nCarPhysics::Halt()\n");
    for( int i = 0; i < bodies; i ++ ) {
        dBodySetLinearVel(  bodies[i], 0,0,0 );
        dBodySetAngularVel( bodies[i], 0,0,0 );
    }
}
