#define N_IMPLEMENTS nODEPhysics
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/nodephysics.h"

nNebulaScriptClass(nODEPhysics, "nphysics");

//------------------------------------------------------------------------------
/**
*/
nODEPhysics::nODEPhysics():
    gameServer( kernelServer, this ),
    collideServer( kernelServer, this ),
    odeServer ( kernelServer, this )
{
}

void nODEPhysics::Initialize(){
    nPhysics::Initialize();
    gameServer 		= "/sys/servers/game";
    collideServer 	                                        = "/sys/servers/collide";
    odeServer 		= "/sys/servers/ode";
    classInitialized = false;
    shapeInitialized = false;
    collideObject = 0;
}

void nODEPhysics::InitVars(){
    nPhysics::InitVars();
    dBodyID & body = bodies.Push();
    body = odeServer->CreateBody();

    dMass m;
    dMassSetSphereTotal	(&m, 1.0, 1.0 ); //*mass, total_weight, r
    dBodySetMass 	(body,&m);
    mass.SetF( m.mass );
    //dGeomID& geom = geoms.Push();
    //geom = dCreateSphere( odeServer->GetSpace(), 1.0 );
    //dGeomSetBody ( geom, body );

    /*
    collideClass.Connect	( ent, "collideclass" );
    collideClass.DefaultS	("default");
    collideShape.Connect	( ent, "collideshape" );
    collideShape.DefaultS	("default");
    collideObject = gameServer->NewCollideObject(this);
    collideObject->SetClientData( ent );
    */
    //this->AddToDisplayList();
}

nODEPhysics::~nODEPhysics(){
    if( odeServer.isvalid() )
        for( int i = 0; i < bodies; i ++ )
            if( bodies[i] )
                odeServer->RemBody( bodies[i] );
    /*if( collideObject )
        gameServer->ReleaseCollideObject( collideObject );
        */
}
//------------------------------------------------------------------------------
/**
*/

/**
  Pulls our position out of ode
*/
void nODEPhysics::Tick( float time, float dtime ){
    //nPhysics::Tick( time, dtime ); dont call default tick
    nAspect::Tick( time, dtime );
    if( pos.isdirty() ){ //pos has changed externally
        vector3 np = pos.GetV();

        const dReal* r = dBodyGetPosition( bodies[0] );
        vector3 cp( r[0], r[1], r[2] );

        for( int i = 1; i < bodies.Size(); i++ ) {
            const dReal* r = dBodyGetPosition( bodies[i] );
            vector3 p0( r[0], r[1], r[2] );

            vector3 dp = p0 - cp;
            vector3 p1 = np + dp;
            dBodySetPosition( bodies[i], p1.x, p1.y, p1.z );
        }
        dBodySetPosition( bodies[0], np.x, np.y, np.z );
    }
    if( mass.isdirty() ){
        /*dMass* m;
        dBodyGetMass( bodies[0], m );
        dMassAdjust( m, (dReal) mass.GetF() );
        dBodySetMass( bodies[0], m );
        */
    }
    if( rot.isdirty() ){ //rot has changed externally
        //n_printf("rot.isdirty()!\n");
        quaternion nq = rot.GetQ();
        dReal q[4] = { -nq.z, nq.y, -nq.x, nq.w };
        //n_printf("setting q to :%f,%f,%f,%f\n", q[0],q[1],q[2],q[3] );
        for( int i = 0; i < bodies; i ++ )
        dBodySetQuaternion( bodies[i], q );
    }
    const dReal* dp = dBodyGetPosition	( bodies[0] );
    const dReal* dq = dBodyGetQuaternion	( bodies[0] );
    vector3 	p(dp[0], dp[1], dp[2] 	);
    quaternion 	q(-dq[2], dq[1], -dq[0], dq[3] 	);
    pos.SetV( p );
    pos.isdirty(); //silly trick to keep its dirtyness synched
    rot.SetQ( q );
    rot.isdirty();

    //dGeomSetPosition( geom, dp[0], dp[1], dp[2] );
    //dGeomSetQuaternion( geom, dq );

    vector3 f = force.GetV();
    dBodyAddForce( bodies[0], f.x, f.y, f.z );
    //this->DrawLine( p, p + f, green );
    force.SetV( 0,0,0 );

    vector3 t = torque.GetV();
    //this->DrawLine( p, p + t, red );
    dBodyAddTorque( bodies[0], t.z, t.y, t.x );
    torque.SetV( 0,0,0 );

    vector3 at = alignedTorque.GetV();
    //this->DrawLine( p, p + at, blue );
    dBodyAddRelTorque( bodies[0], at.x, at.y, at.z );
    alignedTorque.SetV( 0,0,0 );


    // FRICTION
    const dReal* df = dBodyGetLinearVel( bodies[0] );
    vector3 fric( df[0], df[1], df[2] );
    vel.SetV( fric );
    fric = fric * -friction.GetF();
    dBodyAddForce( bodies[0], fric.x, fric.y, fric.z );

    const dReal* drv = dBodyGetAngularVel( bodies[0]);
    vector3 afric( drv[0], drv[1], drv[2] );
    rvel.SetV( afric );
    afric = afric * -rfriction.GetF();
    dBodyAddTorque( bodies[0], afric.x, afric.y, afric.z );
    /*
       collidedLastTick = false;
       if( collideObject == 0 ){
       return;
       }
    //INIT collision objects
    if( classInitialized == false || collideClass.isdirty() ){
    classInitialized = true;
    const char* collClass = collideClass.GetS().Get();
    collideObject->SetCollClass( collideServer->QueryCollClass( collClass ) );
    }
    if( shapeInitialized == false || collideShape.isdirty() ){
    shapeInitialized = true;
    const char* collShape = collideShape.GetS().Get();
    nCollideShape* shape = gameServer->NewCollideShape( collShape, collShape );
    collideObject->SetShape(shape);
    }
    if( classInitialized && shapeInitialized ) {
    nCollideReport** report;
    int numColls = collideObject->GetCollissions( report );
    if ( numColls > 0 ) {
    for( int i = 0; i < numColls; i ++ ){
    nEnt* other = (nEnt*) report[i]->co2->GetClientData();
    if( other && other->IsAlive() ){
    vector3 contact = report[i]->contact;
    vector3 normal;
    if( ent == other ) {
    other = (nEnt*) report[i]->co1->GetClientData();
    normal = report[i]->co2_normal;
    } else {
    normal = report[i]->co1_normal;
    }
    dBodyID b2 = ((nODEPhysics*) (other->Find("physics")))->GetBody();
    //odeServer->AddCollision( body, b2, contact, normal );
    }
    }

    collidedLastTick = true;
    }

    matrix44 mx;
    mx.ident();
    mx.mult_simple(matrix44(rot.GetQ()));
    mx.translate(p);
    collideObject->Transform( time, mx );
    }
     */
}

bool nODEPhysics::BlockDeath(){
    if( collideObject )
        collideObject->SetCollClass( collideServer->QueryCollClass( "ignore" ) );
    if( collidedLastTick )
        return true;
    else 
        return false;
}
