#define N_IMPLEMENTS nBuildAI
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nbuildai.h"
#include "enkiduo/ngameserver.h"
#include "aspect/nhardpoint.h"
#include "aspect/nhardpointattach.h"
#include "rts/nfactorycontroller.h"

nNebulaScriptClass(nBuildAI, "nai");

//------------------------------------------------------------------------------
/**
*/
nBuildAI::nBuildAI():
gameServer( kernelServer, this )
{
}

void nBuildAI::Initialize(){
    nAI::Initialize();
    stateNames.Append( "idle" );
    stateNames.Append( "waitingForResources" );
    stateNames.Append( "waitingForHardpoint" );
    stateNames.Append( "building" );
    stateNames.Append( "done" );

    gameServer = "/sys/servers/game";
}

void nBuildAI::InitVars(){
    nAI::InitVars();
    //resources we have
    side.Connect	( ent, "side" );

    //whats being built / how much it costs
    building.Connect	( ent, "building");
    building.DefaultRef	("");
    buildtime.Connect	( building, "buildtime" );
    entBuilding.Connect	( ent, "entbuilding" );
    entBuilding.DefaultRef	("");

    timeSpent.Connect	( ent, "timespent");
    timeSpent.DefaultF	(0.0f);
    //exported for buildbar
    progress.Connect	( ent, "progress" );
    progress.DefaultF ( -1.0f );

    //resource available / required
    int numResources = 3;		//FIXMe
    for( int i = 0; i < numResources; i ++ ) {
        Resource* r = &resources.Push();
        nString name;
        name.PrintF("resource%i", i );
        r->available.Connect( side, name.Get() );
        name.PrintF("cost%i", i );
        r->cost.Connect( building, name.Get() );
    }

}

nBuildAI::~nBuildAI(){
}
//------------------------------------------------------------------------------
/**
*/

void nBuildAI::Tick( float time, float dtime ){
    nAI::Tick( time, dtime );
}


bool nBuildAI::States( StateMachineEvent event, uint state, float time, float dtime ){
    BeginStateMachine
        State(Idle)
            OnTick
	ifvalid( building )
	    this->SetState( WaitingForResources );
        State(WaitingForResources)
            OnTick
	ifvalid(building)
	    ifvalid(side)
	        if( HasResources() )
	            this->SetState( WaitingForHardpoint );
        State(WaitingForHardpoint)
            OnTick
	nRoot* hardpoints = ent->Find("hardpoints");
	bool attached = false;
	n_assert( building.isvalid() );
	foreachchild( nHardPoint, h, hardpoints) {
	    if( h->Open() ){
	        attached = true;
	        if( this->HasResources() == false ) {
	            this->SetState( WaitingForResources );
	        } else {
	            this->TakeResources();
	            nEnt* e = gameServer->GenEnt( building.GetRef()->GetName() );
	            e->SetVarRef("side", side.GetRef()->fn );
	            n_assert( e );
	            entBuilding.SetRef( e );
	            kernelServer->PushCwd( e );
	            entAttachment = (nHardPointAttach*) kernelServer->New("nhardpointattach", "buildinghp");
	            if( entAttachment )
		entAttachment->AttachTo( ent );
	            else
		n_printf("failed to create hardpoint attachment" );
	            kernelServer->PopCwd();
	            this->SetState( Building );
	            break;
	        }
	    }
	} else {
	    n_printf("failed to find /hardpoints\n");
	}
	if( !attached ){
	    n_printf("%s couldn't find a hardpoint\n", fn );
	}

        State(Building)
            OnTick
	float ts = timeSpent.GetF();
	//n_printf("ts:%f\n", ts );
	float bt = buildtime.GetF();
	ts += dtime;
	if( ts > bt )  { //done building
	    entAttachment->Die(); //detach them from the factory
	    classVar(nfactorycontroller, cl);
	    //n_printf("parent:%s\n", parent->GetName() );
	    n_assert( parent->IsA(cl));
	    nMessage* m = new nMessage();
	    *m = *((nFactoryController*) parent)->GetDefaultMessage();
	    m->to = entBuilding.GetRef()->fn;
	    this->Send( m );
	    //send them out default message
	    this->Die();
	    ts -= bt;
	}
	timeSpent.SetF( ts );
	progress.SetF( ts / bt );
    EndStateMachine
}

bool nBuildAI::HasResources() {
    bool haveResources = true;
    foreachp( Resource*, r, resources ) {
        if( r->available.GetF() < r->cost.GetF() ) {
            haveResources = false;
            break;
        }
    }
    return haveResources;
}

void nBuildAI::TakeResources(){
    n_assert( HasResources() );
    foreachp( Resource*, r, resources ) {
        r->available.SetF( r->available.GetF() - r->cost.GetF() );
    }
}
