#define N_IMPLEMENTS nLegionAttackLegion
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nlegionattacklegion.h"
#include "rts/nhoverlegionattacklegion.h"
#include "rts/nlegionaspect.h"
#include "util/nheap.h"

nNebulaScriptClass(nLegionAttackLegion, "nlegionbehavior");

//------------------------------------------------------------------------------
/**
*/
nLegionAttackLegion::nLegionAttackLegion()
{
    timeEnemiesLastBuilt = -1.0f;
    timeThreshold = 1.0f;
}

void nLegionAttackLegion::Initialize(){
    nLegionBehavior::Initialize();
}

void nLegionAttackLegion::InitVars(){
    n_printf("nLegionAttackLegion::InitVars()\n");
    nLegionBehavior::InitVars();
    targetLegion.Connect( ent, "targetlegion" );
    targetLegion.DefaultRef("");
    targetLegionPos.Connect( targetLegion, "pos" );
}

nLegionAttackLegion::~nLegionAttackLegion(){
}
//------------------------------------------------------------------------------
/**
*/

void nLegionAttackLegion::Tick( float time, float dtime ){
    nLegionBehavior::Tick( time, dtime );
    this->time = time;
}

///parses out my legion to attack
void nLegionAttackLegion::CreationMessage( nMessage* message ) {
    nLegionBehavior::CreationMessage( message );
    //n_printf("%s attacking legion: %s\n", this->fn, message->data.Get() );
    targetLegion.SetRef( message->data.Get() );
}

///okay so we have a unit we want to allocate
struct EnemyInfo2 {
    nEnt* ent;
    int index;
};
nEnt* nLegionAttackLegion::Allocate( nHoverLegionAttackLegion* proxy ) {
    //n_printf("Allocating for : %s enemies:%i\n", proxy->fn, enemies.Size() );
    this->BuildEnemiesList();
    if( enemies.Size() == 0 ) {
        //n_printf("no enemies to target\n");
        return 0;
    }
    if( enemies.Size() == 1 ) {
        return (nEnt*) kernelServer->Lookup( enemies[0].name.Get() );
    }
    nHeap<EnemyInfo2> heap;
    heap.initialize( enemies.Size() );
    //n_printf("%s allocating: %s\n", this->fn, proxy->fn );
    vector3 up = proxy->GetEnt()->GetVarV("pos");
    for( int i = 0; i < enemies; i ++ ) {
        nEnt* e = (nEnt*) kernelServer->Lookup( enemies[i].name.Get() );
        if( e ) {
            vector3 tp = e->GetVarV("pos");
            float d = (tp - up).len();
            EnemyInfo2 info;
            info.ent = e;
            info.index = i;
            n_assert( e->IsAlive() );
            heap.add( (1.0 + enemies[i].numAllocatedToo ) * d, info );
        } else
            n_error( "bad unit in my list\n");
    }
    if( heap.length() > 0 ) {
        EnemyInfo2 info = heap.pop();
        //n_printf("targeting: %s\n", info.ent->fn );
        enemies[info.index].numAllocatedToo++;
        return info.ent;
        /*
           if( enemies[i].numAllocatedToo == 0 ) {
           enemies[i].numAllocatedToo++;
           return (nEnt*) kernelServer->Lookup( enemies[i].name.Get() );
           }
         */
    }
    n_printf("failed to find any enemies to target\n");
    n_error(0);
    return 0;
}
///make a list of all the enemies my units can target
void nLegionAttackLegion::BuildEnemiesList(){
    /*
       if( timeEnemiesLastBuilt < 0 || time - timeEnemiesLastBuilt > timeThreshold ) {
       timeEnemiesLastBuilt = time;
       } else
       return;
     */
    //n_printf("%s->BuildEnemiesList(%s)\n", this->fn, targetLegion.GetRef()->GetName() );
    for( int i = 0; i < enemies; i ++ )
        enemies[i].found = false;
    ifvalid( targetLegion ) {
        nEnt* enemyLegion = (nEnt*) targetLegion.GetRef();
        nLegionAspect* la = (nLegionAspect*) enemyLegion->Find("legion");
        EntArray enemyUnits;
        ifnonzero( la ) {
            n_printf("la= %s\n", la->fn );
            la->BuildMyUnitsList( &enemyUnits );
        }
        //n_printf("%s has %i units\n", enemyLegion->fn, enemyUnits.Size() );
        for( int i = 0; i < enemyUnits; i ++ ) {
            bool found = false;
            for( int j = 0; j < enemies; j ++ ) {
	if( enemies[j].ent == enemyUnits[i]) {
	    enemies[j].found = true;
	    found = true;
	    //printf("found existing enemy: %s\n", enemies[j].ent->fn );
	    break;
	}
            }
            if( found == false ) {
	//printf("adding new enemy: %s\n", enemyUnits[i]->fn );
	EnemyInfo& ei = enemies.Push();
	ei.ent= enemyUnits[i];
	ei.name = enemyUnits[i]->fn;
	ei.numAllocatedToo = 0;
	ei.found = true;
            }
        }
        //n_printf("enemies: %i\n", enemies.Size() );
        for( int i = 0; i < enemies; i++ ) { ///take enemies out of the list that are no longer there
            if( enemies[i].found == false || enemies[i].ent->IsAlive() == false ) {
	//n_printf("enemies[%i](%s) being erased\n", i, enemies[i].name.Get() );
	enemies.Erase( i-- );
            }
        }
    }
    if( enemies.Size() == 0 )
        this->Die();
}

///every tick --- make our l0 match with ouself - so units trying to formate join with the rest - no default movement
///also modify the desired bearing to match a line path to our enemy - keep us pointed at them
bool nLegionAttackLegion::States( StateMachineEvent event, uint state, float time, float dtime ){ 
    nAspect::Tick(0,0);
    BeginStateMachine
        State(Kill)
        OnTick
        vector3 p = pos.GetV();
    if( targetLegion.isvalid() ) {
        vector3 tp = targetLegionPos.GetV();
        vector3 dx = tp - p;
        float bearing = atan2( dx.x, dx.z );
        quaternion q;
        q.set_rotate_y( bearing );
        rot.SetQ( q );
        l0pos.SetV( pos.GetV() );
        l0rot.SetQ( q );
    }
    EndStateMachine
}
