#define N_IMPLEMENTS nLegionWindow
#include "enkiduo/ngameserver.h"

#include "rts/nlegionaspect.h"
//------------------------------------------------------------------------------
//  (C) 2005	David Leistiko
//------------------------------------------------------------------------------
#include "rts/nlegionwindow.h"
nNebulaScriptClass(nLegionWindow, "nvisnode");

struct LegionWindowUnitInfo {
    vector3 formationPos;
    vector3 formationSize;
    int type;
};

nLegionWindow::nLegionWindow() {
    legionBox.SetFixedSize ( 2 );
    for ( int i = 0; i < legionBox; i ++ ) {
        legionBox[i].initialize( kernelServer, this );
        nString path;
        path.PrintF("/vis/vis/legionbox%i", i );
        legionBox[i] = path.Get();
        //n_printf("legionBox[%i] = %s\n;", i, path.Get() );
    }    
}
void nLegionWindow::Initialize() {
    nVisNode::Initialize();
}
nLegionWindow::~nLegionWindow() {
}
bool nLegionWindow::Attach( nSceneGraph2* sceneGraph) {
    //n_printf("nLegionWindow::Attach( %p )\n", sceneGraph );
    int i;
    static nGameServer* game = (nGameServer*) kernelServer->Lookup("/sys/servers/game");           //find the game server - it knows about the mouse
    MouseInfo mouseInfo = game->GetMouseInfo();                                                    //get the mouse information
    //n_printf("mouse pos: %f,%f buttons:%i,%i,%i\n", mouseInfo.pos.x, mouseInfo.pos.y, mouseInfo.pressed[0], mouseInfo.pressed[1], mouseInfo.pressed[2] );
    static nPlannerUI* plannerUI = (nPlannerUI*) kernelServer->Lookup("/ui/planner");              //find the planner - it knows whats selected
    const EntArray* selectedUnits = plannerUI->GetObjectsSelected();                               //horrible line of code - gets an array with everything selected
    if ( selectedUnits->Size() > 0 ) {                                                             //if somethings selected
        matrix44 m = sceneGraph->GetTransform();
        nEnt* selected = selectedUnits->Get(0);                                                    //just check the first on
        nLegionAspect* legionAspect = (nLegionAspect*) selected->Find("legion");                   //see if it has a legion aspect
        if ( legionAspect ) {                                                                      //if so
            if (sceneGraph->BeginNode()) {
                nArray<UnitInfo>* units;                                                           //array of units belonging to the legion
                units = legionAspect->GetUnits();                                                  //get the units
                nArray<LegionWindowUnitInfo> unitInfos(units->Size(),0);

                float maxx, minx;
                float maxz, minz;
                maxx = maxx = -10000.0f;
                minx = minz = 10000.0f;

                vector3 formationSize = selected->GetVarV("formationsize");
                vector2 scale( 0.7f / formationSize.x, 0.7f / formationSize.z);
                vector3 mscale = m.get_scale();
                scale.x *= mscale.x;
                scale.y *= mscale.z;
                for ( i = 0; i < units->Size(); i++ ) {                                            //go through the units	
                    // GET MAX X AND Z LOCATIONS FOR UNITS IN FORMATION
                    LegionWindowUnitInfo& info = unitInfos.Push();
                    info.formationPos  = units->Get(i).ent->GetVarV("formationpoint");
                    info.formationSize = units->Get(i).ent->GetVarV("formationsize");
                    if ( info.formationPos.x < minx )                        minx = info.formationPos.x;
                    if ( info.formationPos.x > maxx )                        maxx = info.formationPos.x;
                    if ( info.formationPos.z < minz )                        minz = info.formationPos.z;
                    if ( info.formationPos.z > maxz )                        maxz = info.formationPos.z;
                    info.type = 0;
                }
                // GET SCALE OF BLOCKS TO DRAW BASED ON NUMBER OF UNTIS
                float xRange = maxx - minx;
                float zRange = maxz - minz;

                if ( scale.x < scale.y )                    scale.y = scale.x;
                else                                        scale.x = scale.y;

                //go through each unit and draw it                

                for ( int i = 0; i < unitInfos; i ++ ) {
                    LegionWindowUnitInfo& unit = unitInfos[i];                    
                    float x, y;
                    if ( xRange > 0 )                        x = ( unit.formationPos.x - minx )/ xRange;
                    else                                     x = 0.5f;
                    if ( zRange > 0 )                        y = ( unit.formationPos.z - minz )/ zRange;
                    else                                     y = 0.5f;

                    nAutoRef<n3DNode>& legionbox = legionBox[unit.type];
                    if ( legionbox.isvalid() ) {                                                   //if your pointer to the square is good {                   
                        vector3 v(x ,y, 0.0f);
                        v.x = v.x * .7 + .15;                        
                        v.y = v.y * .7 + .15;
                        v = m * v;

                        vector2 myscale(scale);
                        myscale.x *= unit.formationSize.x;
                        myscale.y *= unit.formationSize.z;
                        // ("formationPos[%i]:%s,%s -> v:%s s:%s\n", i, str(formationPos[i]), str(units->Get(i).ent->GetVarV("formationsize")), str( v ), str(scale) );
                        legionbox->Txyz( v.x, v.y, v.z );                                          //put it at that point in the scren
                        legionbox->Sxyz( myscale.x, myscale.y, 1.0f );                             //scale the lil rectangles
                        legionbox->Attach( sceneGraph );                                           //draw
                    } else {
                        n_assert( legionBox[unit.type].isvalid() == false );
                        n_printf("legionbox[%i](%s) is invalid()\n", unit.type, legionBox[unit.type].getname());
                    }
                }
/*
                float scaleX = xRange/units->Size();
                float scaleZ = zRange/units->Size();
                vector3 tempV3;
                
                float scaleFactor = 0.06f;
                float xSpacing = 0.08f;
                if (units->Size() > 6 ) {
                    scaleFactor = 0.4f;
                    xSpacing = 0.55f;
                }
                float xOff = 1.0f - ((xSpacing * ((float)units->Size()-1))/2.0f) - (scaleFactor/2.0f);    // offset of where to begin drawing squares based on number of units
                float zOff = 0.0f;
                for ( int j = 0; j < units->Size(); j++ ) {                    // sorintg array of formationPos from least to greatest in terms of xPos
                    for ( i = 1; i < units->Size(); i++ ) {
                        if (formationPos[i-1].x > formationPos[i].x) {
                            tempV3 = formationPos[i];
                            formationPos[i] = formationPos[i-1];
                            formationPos[i-1] = tempV3;
                        }
                    }
                }
                for ( i=0; i < units->Size(); i++ ) {                          // drawing boxes
                    //n_printf("attaching at :%f,%f,%f\n", formationPos[i].x, formationPos[i].z, -10.0f );

                }*/
            }
            sceneGraph->EndNode();
            //printf("\n");	
        }
    }

    /*********************************************/
    // HANDLE MOUSE INPUT
    CheckMouse(mouseInfo);
    /*********************************************/
    return nVisNode::Attach( sceneGraph );
}

///handle mouse input
void nLegionWindow::CheckMouse(MouseInfo MI) {
    float left  = .65;                                                                             // BOUNDARIES OF WHHITE BOX THAT 
    float right = .95;                                                                             // CONTAINS LEGION FORMATION
    float top   = .20;
    float bottom= .50;
    float mx = MI.pos.x;
    float my = MI.pos.y;
    /*
    if (MI.pressed[0] == 1) {
        if (mx >= left && mx <= right && my >= top && my <= bottom)
            exit(0);
    }
    */
}
