#define N_IMPLEMENTS nTurretAI
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nturretai.h"
#include "enkiduo/nopenalserver.h"
#include "util/nheap.h"
#include "aspect/nside.h"
#include "aspect/nsoundemitter.h"
#include "enkiduo/nprofserver.h"
#include "aspect/nhardpointattach.h"
#include "turret/nturret.h"

nNebulaScriptClass(nTurretAI, "nai");

//------------------------------------------------------------------------------
nTurretAI::nTurretAI():
    weapon	( kernelServer, this ),
    turret                              ( kernelServer, this ),
    map	( kernelServer, this ),
    emitter	( kernelServer, this ),
    attach  	( kernelServer, this )
{
}

//-----------------------------------------------------------------------------//
void nTurretAI::Initialize(){
    nAI::Initialize();
    this->AddToDisplayList();
    this->clearLines = true;
    map = "/map/map";
}
//-----------------------------------------------------------------------------//

void nTurretAI::InitVars(){
    nAI::InitVars();
    mother.Connect                      ( ent, "mother" );
    mother.DefaultRef                   ("");
    side.Connect                        ( mother, "side" );
    motherTarget.Connect                ( mother, "target" );

    target.Connect                      ( ent, "target" );
    target.DefaultRef                   ( "" );
    targetPos.Connect                   ( target, "pos" );

    rotLocal.Connect                    ( ent, "rotlocal" );
    rotLocal.DefaultQ                   ( 0,0,0,1 );
    trackingSpeed.Connect               ( ent, "trackingspeed" );
    trackingSpeed.DefaultF              ( 5.0f );

    fires.Connect                       ( ent, "fires");
    nString f = fires.GetS();

    mySensors.Connect	( mother, "sensors" );
    targetECM.Connect	( target, "ecm" );
    weaponRange.Connect	( mother, (f + "range").Get() );
    targetRange.Connect	( ent, "targetrange" );
    sensorRC.Connect	( ent, "sensorrc" );
    sensorRC.DefaultInputRange	( 0, 10, 5);
    sensorRC.DefaultOutputRange	( 0, 1 );
    sensorRC.DefaultRC 	( RC_LINE, 1, 0 );
    rangeRC.Connect	( ent, "rangerc" );
    rangeRC.DefaultInputRange	( 0, 10, 5 );
    rangeRC.DefaultOutputRange	( 0, 2 );
    rangeRC.DefaultRC 	( RC_LINE, 1, 0 );

    weaponPower.Connect	( mother, (f + "power").Get() );
    weaponImpulse.Connect	( mother, (f + "impulse").Get() );
    weaponLevel.Connect	( mother, (f + "level").Get() );
    weaponEffectRadius.Connect	( mother, (f + "effectradius").Get() );

    weapon = ent->ChildPath("weapon");
    turret = ent->ChildPath("turret");
    emitter = ent->ChildPath("emitter");
    attach = ent->ChildPath("attach");

    autoFire.Connect                   ( mother, "autofire");
}

nTurretAI::~nTurretAI(){
}
//------------------------------------------------------------------------------
/**
*/

void nTurretAI::Tick( float time, float dtime ){
    nAI::Tick( time, dtime );
    //n_printf("nTurretAI::Tick(%f,%f)\n", time, dtime );
    //prof->Begin("nTurretAI::Tick()");
    if( mother.isvalid() == false ) {
        //n_printf("%s.mother invalid\n", this->fn );
        ent->Die();
        return;
    } else
        autoFire.DefaultI                  ( 0 );
    if( fires.isdirty()) { 
        const char* f = fires.GetS().Get();
        firingMissles = (strstr(f, "missle") > 0);
        firingMissles |= (strstr(f, "suicide") > 0);
    }
    if( motherTarget.isvalid() && motherTarget.isdirty() ){
        target.SetRef( motherTarget.GetRef()->fn );
    }
    if( target.isvalid() == false or roll(.01) && time > 1.0f) {
        //n_printf("time:%f\n", time );
        if ( autoFire.GetI() > 0 )
            if( roll( .005 ) )
                this->FindTarget();
    }
    if ( turret.isvalid() && turret->CanFire() ) {
        turret->Fire();
    } else 
    if( weapon.isvalid() && weaponPower.GetF() > 0 && weapon->CanFire() ) {
        if( target.isvalid() && target.GetRef()->IsAlive() && targetPos.isvalid() ) {
            vector3 p = weapon->GetOffsetPos();
            vector3 tp = targetPos.GetV();
            vector3 dx = tp - p;
            float tl = dx.len();

            float range = weaponRange.GetF();
            //if( roll(.01)) n_printf("tl:%f range:%f\n", tl, range );
            if( tl < range ) { 
                quaternion q = rot.GetQ();
                if( trackingSpeed.GetF() > 0 ) { //aim the turret at the enemy
                    vector3 forward	(0,0,-1);
                    forward = q.rotate( forward );

                    dx.norm();
                    targetRange.SetF( tl );

                    quaternion rotl = rotLocal.GetQ();
                    quaternion q2;
                    forward.norm();
                    q2.set_from_axes( forward, dx );
                    rotLocal.SetQ( q2 * rotl );
                    attach->Tick( 0,0);
                }

                //calculate hit %
                float hitChance = this->CalculateHitChance();
                bool connects = roll( hitChance );
                //calculate theta, phi;
                float range;
                vector3 v;
                this->CalculateAngles( connects, q, range, v );
                if( firingMissles ) {
                    ifvalid( emitter ) {
                        if( fires.GetS() == "smissle" )
                            emitter->Queue("smisslefire");
                        else
                            emitter->Queue("mmisslefire");
                    }
                    weapon->Fire( target.GetRef()->fn, q, 0, 
                            weaponPower.GetF(), 
                            weaponImpulse.GetF(), 
                            weaponEffectRadius.GetF(), 
                            weaponLevel.GetI() 
                            );
                } else { //direct fire weapon
                    if( weapon->Fire( target.GetRef()->fn, q, range, 
                                weaponPower.GetF(), 
                                weaponImpulse.GetF(), 
                                weaponEffectRadius.GetF(), 
                                weaponLevel.GetI() ) 
                      ) {                        
                        ifvalid( emitter ) {
                            nString soundName = fires.GetS() + "fire";
                            emitter->Queue(soundName.Get());
                        }
                        if( connects ) {
                            this->DoDamage( time, range );
                        } 
                    }
                }
            }
        }
    }
    //prof->End("nTurretAI::Tick()");
}

/////////////////////////////////////////////////////////////////////////////////
float nTurretAI::CalculateHitChance(){
    float ms = mySensors.GetF();
    float te = targetECM.GetF();
    float hitChanceA = sensorRC.Response( ms / te );
    //n_printf("ms: %f te:%f hcA: %f\n", ms, te, hitChanceA );

    float wr = weaponRange.GetF();
    float tr = targetRange.GetF();
    float hitChanceB = rangeRC.Response ( wr / tr );
    //n_printf("wr: %f tr: %f hcB: %f\n", wr, tr, hitChanceB );

    float hitChance = hitChanceA * hitChanceB;
    return 1.0f;
    //n_printf("hitChance: %f\n", hitChance );
    return hitChance;
}

////////////////////////////////////////////////////////////////////////////////
void nTurretAI::CalculateAngles( bool connects, quaternion& q, float& range, vector3& v ){
    //n_printf("%s\n", connects?"hit":"miss");
    float accuracy = .01f;
    if( connects )
        accuracy /= 10.0f;
    float theta                         = randF( -accuracy, accuracy );
    float phi 	= randF( -accuracy, accuracy ) / 100.0f;
    quaternion q0;
    q0.set_rotate_xyz( theta, phi, 0 );

    vector3 p = pos.GetV();
    quaternion q1 = rot.GetQ();
    quaternion q2 = q1 * q0;
    if( connects ) {
        vector3 p = pos.GetV();
        vector3 tp = targetPos.GetV();
        range = (tp - p).len();
    } else {
        vector3 forward(0,0,-range);
        vector3 offshot = q2.rotate( forward );
        line3 l(p,p + offshot);
        if( map->GetIntersect(l,v) ) 
            range = (v-p).len();
        else
            range = range;
    }
    //n_printf("range:%f\n", range );
    q = q2;
}
//-----------------------------------------------------------------------------//
void nTurretAI::DoDamage( float time, float range ){
    nMessage* m = new nMessage();
    m->time = time;
    m->from = ent->FullNameString();
    m->to = target.GetRef()->FullNameString();
    m->title = "damage";
    m->data.PrintF("%s%i", fires.GetS().Get(), weaponLevel.GetI() );
    m->f0 = weaponPower.GetF();
    m->f1 = weaponImpulse.GetF();
    m->v = pos.GetV();
    Send( m );
}
/**
  Scans for a targets within range - picking the closest one
  */
void nTurretAI::FindTarget(){
    //n_printf("%s.%s.FindTarget()", mother.GetRef()->fn, this->fn);
    static nHeap<nEnt*> heap(128);
    heap.clear();

    float dist;
    vector3 p = pos.GetV();
    //new way..pull the list of enemy factions
    //then pull their list of units, and scan them
    nEnt* sideEnt = (nEnt*) side.GetRef();
    n_assert( sideEnt );
    nSide* sidePtr = (nSide*) sideEnt->Find("side");
    n_assert( sidePtr );

    //n_printf("side:%s\n", sidePtr->GetName() );
    const SideArray* hostiles = sidePtr->GetHostiles();
    n_assert( hostiles );
    nSide* hs = 0;
    nEnt* e = 0;
    //n_printf("se:%s sp:%s hostiles:%i\n", sideEnt->fn, sidePtr->fn, hostiles->Size() );
    for( int i = 0; i < hostiles->Size(); i ++ ) {
        hs = hostiles->Get(i);
        const EntArray* units = hs->GetUnits();
        //n_printf("units:%i\n", units->Size() );
        for( int i = 0; i < units->Size(); i ++ ) {
            e = units->Get(i);
            static nRoot* legionRoot = kernelServer->Lookup("/game/ents/legion");
            if( e->GetParent() != legionRoot ) {
                if( e->IsAlive() ) {
                    vector3 tp = e->GetVarV("pos");
                    vector3 dx = tp - p;
                    dist = dx.len();
                    dist *= randF(.8f, 1.2f );
                    //n_printf("p:%s tp:%s dist:%f ", str(p), str(tp), dist );
                    //dist = 1.0f / dist;
                    //n_printf("1/d:%f\n", dist );
                    heap.add( dist, e );
                }
            }
        }
        e = 0;
    }
    if( heap.length() > 0 ) {
        heap.pop( dist, e );
        //n_printf("targetting:  %s\n", e->fn );
        target.SetRef( e );
    }
    //n_printf("/FindTarget()\n");
}
