#define N_IMPLEMENTS nTNL
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "tnl/ntnl.h"
#include "enkiduo/nent.h"
#include "enkiduo/ngameserver.h"

nKernelServer* nTNLConnection::kernelServer = 0;
nGameServer*   nTNLEnt::gameServer = 0;
nKernelServer*   nTNLEnt::kernelServer = 0;
nKernelServer*   nTNLPlayer::kernelServer = 0;
//------------------------------------------------------------------------------
nNebulaScriptClass(nTNL, "nfsm");
nTNL::nTNL()
{
    this->AddLogTag("ntnl");
    stateNames.Append( "void" );
    stateNames.Append( "server" );
    stateNames.Append( "client" );
    this->connection = 0;
    this->serverConnection = 0;
}

void nTNL::Initialize(){
    nFSM::Initialize();
    this->ticksPerFrame = TICKS_FOUR;
    this->AddToTriggerList();
    if( nTNLConnection::kernelServer == 0 )
        nTNLConnection::kernelServer = kernelServer;
    if( nTNLEnt::kernelServer == 0 )
        nTNLEnt::kernelServer = kernelServer;
    if( nTNLPlayer::kernelServer == 0 )
        nTNLPlayer::kernelServer = kernelServer;
    for( int i = 0; i < tnlents; i ++ )
        delete tnlents[i];
    connection = new nTNLConnection( );
}

nTNL::~nTNL(){
    delete connection;
}

void nTNL::Tick( float time, float dtime ){
    nFSM::Tick( time, dtime );
}

bool nTNL::States( StateMachineEvent event, uint state, float time, float dtime ){
    if( connection )
        connection->Tick( time, dtime ); //check messages / etc
    if( serverConnection )
        serverConnection->Tick( time, dtime );
    //n_printf("State: %s\n", stateNames[state]);
    BeginStateMachine
        State(Void)
            OnTick
        State(Server)
            OnTick
                for( int i = 0; i < tnlents; i++) {
                    tnlents[i]->tick(connection, TNLENTTICK_SERVER);
                    //n_printf("setting %s mask bits dirty()\n",tnlents[i]->GetEnt()->fn );
                }
        State(Client)                                                           //go through ents and do client tick
            OnTick
                for( int i = 0; i < tnlents; i ++ ) {
                    tnlents[i]->tick(connection, TNLENTTICK_CLIENT);
                }
    EndStateMachine
}
//-----------------------------------------------------------------------------//
void nTNL::RegisterEnt( nEnt* e ){
    //n_printf("Registering ent:%s\n", e->fn );
    registeredEnts.Append( e );
    nTNLEnt* te = new nTNLEnt;
    te->SetEnt( e );
    tnlents.Append( te );
}
//-----------------------------------------------------------------------------//
void nTNL::RemoveEnt( nEnt* e ){
    for( int i = 0; i < tnlents; i ++ ) {
        if( tnlents[i]->GetEnt() == e ) {
            tnlents.Erase( i-- );
        }
    }
}
//-----------------------------------------------------------------------------//
void nTNL::SetScopeAll( nTNLConnection* c ){
    for( int i = 0; i < tnlents; i ++ ) {
        //printf("ent in scope: %s\n", tnlents[i]->GetEnt()->fn );
        c->objectInScope( tnlents[i] );
    }
}
//-----------------------------------------------------------------------------//
/* -----------------------------------------------------------------------------//
 * open a server connection on this socket, and connect to it
 * nTNL.connection is always a client connection, if this is the server process it is just a local connection to another hidden server connection
--------------------------------------------------------------------------------*/
void nTNL::OpenServer( const char* ip ) { 
    serverConnection = new nTNLConnection();
    serverConnection->openServer( ip ); 
    this->connection->connectToLocalServer( ip, serverConnection );
    //this->connection->openServer(ip); //old way - just turn me into a server
}
//-----------------------------------------------------------------------------//
void nTNL::ConnectToServer( const char* ip )	{ 
    n_printf("nTNL::ConnectToServer(%s)\n", ip ); 
    this->connection->connectToServer( ip );
}
/* -----------------------------------------------------------------------------//
 * so my players have changed, pack up their information and send it over
--------------------------------------------------------------------------------*/
void nTNL::playersChanged(){
    n_printf("nTNL::playersChanged()\n");
    Vector<U8>                          ids;
    Vector<StringTableEntry>            playerNames;
    Vector<StringTableEntry>            playerSides;
    Vector<StringTableEntry>            playerFactions;

    nRoot* playersRoot = LOOKUP(nEnt*, "/game/players");
    foreachchild(nEnt, player, playersRoot ) {
        int id;
        int numScanned = sscanf(player->GetName(), "Player%i", &id);
        assert( numScanned == 1 );
        const char* playerName          = player->GetVarS("playername");
        const char* playerSide          = player->GetVarS("playerside");
        const char* playerFaction       = player->GetVarS("playerfaction");
        n_printf("  player:%i name:%s side:%s faction:%s\n", id, playerName, playerSide, playerFaction);
        ids.push_back(id);
        playerNames.push_back(playerName);
        playerSides.push_back(playerSide);
        playerFactions.push_back(playerFaction);
    }
    printf("pulling %p->rpcServerUpdatePlayers\n", connection);
    connection->rpcServerUpdatePlayers( ids, playerNames, playerSides, playerFactions );
    printf("done pulling rpcServerUpdatePlayers\n");
}
