#include "tnl/ntnlconnection.h"
#include "tnl/ntnl.h"
#include "enkiduo/neventserver.h"
using namespace TNL;
TNL_IMPLEMENT_NETCONNECTION    (nTNLConnection, NetClassGroupGame, true); 
//------------------------------------------------------------------------------
//        TNLCONNECTION STUFF    
//------------------------------------------------------------------------------
nTNLConnection::nTNLConnection():
    eventServer( kernelServer, 0 )
{
    n_assert( kernelServer );
    tnl = (nTNL*) kernelServer->Lookup("/net/tnl");
    n_assert( tnl );
    myNetInterface = 0;

    this->setIsAdaptive();
    //TNL::AsymmetricKey *theKey = new TNL::AsymmetricKey(32);
    //myNetInterface->setPrivateKey(theKey);
    //myNetInterface->setRequiresKeyExchange(true);
    eventServer = "/sys/servers/event";

    static int id = 0;
    this->id = id++;
}
nTNLConnection::~nTNLConnection(){
    disconnect("closing");
    if( myNetInterface )
        delete myNetInterface;
}

//------------------------------------------------------------------------------
/****NETWORKING RPCS*******************************************************************/
//------------------------------------------------------------------------------
TNL_IMPLEMENT_RPC    (nTNLConnection, rpcMessageClientToServer, (StringTableEntry s), (s), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirClientToServer, 0) { 
    printf("Got message from client: %s\n", s.getString()); 
    //tnl->AddServerSideConnection( this );
} 
TNL_IMPLEMENT_RPC    (nTNLConnection, rpcMessageServerToClient, (StringTableEntry s), (s), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirServerToClient, 0) { 
    printf("Got a message from server: %s\n", s.getString()); 
    //tnl->SetState("client");
}
//------------------------------------------------------------------------------
/****GENERAL RPCS*******************************************************************/
//------------------------------------------------------------------------------
TNL_IMPLEMENT_RPC    (nTNLConnection, rpcSetVarF, (StringTableEntry entname, StringTableEntry varname, F32 f), (entname, varname, f), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirClientToServer, 0) { 
    //printf("%s.%s = %f\n", entname.getString(), varname.getString(), f );
    nEnt* e = (nEnt*) kernelServer->Lookup( entname.getString() );
    if( e )
        e->SetVarF(varname.getString(), f);
    else
        printf("could not find entity: %s\n", entname.getString() );
}
TNL_IMPLEMENT_RPC    (nTNLConnection, rpcSetVarI, (StringTableEntry entname, StringTableEntry varname, S32 i), (entname, varname, i), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirClientToServer, 0) { 
    //printf("%s.%s = %i\n", entname.getString(), varname.getString(), i );
    nEnt* e = (nEnt*) kernelServer->Lookup( entname.getString() );
    if( e )
        e->SetVarI(varname.getString(), i);
    else
        printf("could not find entity: %s\n", entname.getString() );
}
TNL_IMPLEMENT_RPC    (nTNLConnection, rpcSetVarV, (StringTableEntry entname, StringTableEntry varname, F32 x, F32 y, F32 z), (entname, varname, x,y,z), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirClientToServer, 0) { 
    vector3 v(x,y,z);
    //printf("%s.%s = %s\n", entname.getString(), varname.getString(), str(v) );
    nEnt* e = (nEnt*) kernelServer->Lookup( entname.getString() );
    if( e )
        e->SetVarV(varname.getString(), v);
    else
        printf("could not find entity: %s\n", entname.getString() );
}

//------------------------------------------------------------------------------
/****EventServer Rpcs**********************************************************/
//------------------------------------------------------------------------------
TNL_IMPLEMENT_RPC (nTNLConnection, rpcBroadcastEvent, (StringTableEntry tag, StringTableEntry msg), (tag, msg), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirClientToServer, 0) { 
    printf("broadcast event: %s, %s\n", tag.getString(), msg.getString() );
    ///for each client connection -> call their receive event
    //rpcReceiveEvent( tag, msg );
    //printf("myNetInterface:%p\n", myNetInterface );
    if ( myNetInterface ) {
        Vector<NetConnection*>& connections = myNetInterface->getConnectionList();
        //printf("connections:%i\n", connections.size() );
    }
}
TNL_IMPLEMENT_RPC (nTNLConnection, rpcReceiveEvent, (StringTableEntry tag, StringTableEntry msg), (tag, msg), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirServerToClient, 0) { 
    printf("receive event: %s, %s\n", tag.getString(), msg.getString() );
    ///just call the local event server pull
    if( eventServer.isvalid() )
        eventServer->pullInefficient( tag.getString(), msg.getString() );
}
//-----------------------------------------------------------------------------//
/****Side Management RPCS******************************************************/
//------------------------------------------------------------------------------
TNL_IMPLEMENT_RPC (nTNLConnection, rpcServerUpdatePlayers, (Vector<U8> playerIds, Vector<StringTableEntry> playerNames, Vector<StringTableEntry> playerSides, Vector<StringTableEntry> playerFactions), 
        (playerIds, playerNames, playerSides, playerFactions), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirClientToServer, 0 ){
    printf("rpcServerUpdatePlayers()\n");
    if ( getInterface() ) {
        Vector<NetConnection*>& connections = getInterface()->getConnectionList();
        printf("connections:%i\n", connections.size() );
        for( int i = 0; i < connections.size(); i ++ ) {
            ((nTNLConnection*)connections[i])->rpcClientUpdatePlayers( playerIds, playerNames, playerSides, playerFactions);
        }
    }
}
//-----------------------------------------------------------------------------//
TNL_IMPLEMENT_RPC (nTNLConnection, rpcClientUpdatePlayers, (Vector<U8> playerIds, Vector<StringTableEntry> playerNames, Vector<StringTableEntry> playerSides, Vector<StringTableEntry> playerFactions), 
        (playerIds, playerNames, playerSides, playerFactions), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirServerToClient, 0 ){
    printf("rpcClientUpdatePlayers()\n");
    for( int i = 0; i < playerIds.size(); i ++ ) {
        int id = playerIds[i];
        const char* playerName      = playerNames[i].getString();
        const char* playerSide      = playerSides[i].getString();
        const char* playerFaction   = playerFactions[i].getString();
        printf("id:%i name:%s side:%s faction:%s\n", id, playerName, playerSide, playerFaction );
        static void* eventPtr = 0;
        nString updateInfo;
        updateInfo.PrintF("%i:%s:%s:%s", id, playerName, playerSide, playerFaction );
        eventServer->pull( eventPtr, "updatePlayer", updateInfo.Get());
    }
}
//------------------------------------------------------------------------------
void nTNLConnection::Tick( float time, float dtime ) {
    //n_printf("nTNLConnection::Tick(%f,%f)\n", time, dtime );
    if( myNetInterface ) {
        myNetInterface->checkIncomingPackets(); 
        myNetInterface->processConnections(); 
    } else
        printf("no net interface\n");
}
//------------------------------------------------------------------------------
void nTNLConnection::openServer( const char* ip){
    printf("%i.nTNLConnection::OpenServer(%s)\n", id, ip);
    Address cmdAddress(ip);
    myNetInterface = new NetInterface(cmdAddress); 
    myNetInterface->setAllowsConnections(true); 
    tnl->SetState("server");
}
//-----------------------------------------------------------------------------//
void nTNLConnection::connectToLocalServer( const char* ip, nTNLConnection* server ) {
    printf("%i.nTNLConnection::ConnectToLocalServer(%s)\n", id, ip);
    Address cmdAddress(ip);
    Address bindAddress(IPProtocol, Address::Any, 0); 
    myNetInterface = new NetInterface(bindAddress); 
    this->connectLocal( myNetInterface, server->myNetInterface );
}
//------------------------------------------------------------------------------
void nTNLConnection::connectToServer( const char* ip ){
    printf("%i.nTNLConnection::ConnectToServer(%s)\n", id, ip);
    Address cmdAddress(ip);
    Address bindAddress(IPProtocol, Address::Any, 0); 
    myNetInterface = new NetInterface(bindAddress); 
    this->connect(myNetInterface, cmdAddress); 
    this->rpcMessageClientToServer("hello");
    tnl->SetState("client");
}
//------------------------------------------------------------------------------
void nTNLConnection::packetDropped    ( PacketNotify* notify ){
    //printf("%i.nTNLConnection::packetDropped()\n", id);
    Parent::packetDropped( notify );
}
void nTNLConnection::packetReceived    ( PacketNotify* notify ){
    //printf("%i.nTNLConnection::packetReceived\n", id);
    Parent::packetReceived( notify );
}
void nTNLConnection::writePacket    ( BitStream* stream, PacketNotify* notify){
    //printf("nTNLConnection::writePacket\n");
    Parent::writePacket( stream, notify );
}
void nTNLConnection::readPacket    ( BitStream* stream ){
    //printf("nTNLConnection::readPacket\n");
    Parent::readPacket( stream );
}
void nTNLConnection::processEvent    ( NetEvent* event ){
    //printf("%i.nTNLConnection::processEvent\n", id);
    Parent::processEvent( event );
}
void nTNLConnection::writeConnectRequest    ( BitStream* stream ){
    //printf("%i.nTNLConnection::writeConnectRequest\n", id);
    Parent::writeConnectRequest( stream );
}
bool nTNLConnection::readConnectRequest    ( BitStream* stream, const char** errorString ){
    //printf("%i.nTNLConnection::readConnectRequest %s\n", id, errorString ? *errorString : "none" );
    return Parent::readConnectRequest( stream, errorString );
}
void nTNLConnection::writeConnectAccept    ( BitStream* stream ){
    //printf("%i.nTNLConnection::writeConnectAccept\n", id);
    Parent::writeConnectAccept( stream );
}
bool nTNLConnection::readConnectAccept    ( BitStream* stream, const char** errorString ){
    //printf("%i.nTNLConnection::readConnectAccept %s\n", id, errorString ? *errorString : "none" );
    return Parent::readConnectAccept( stream, errorString );
}
void nTNLConnection::onConnectionEstablished() {
    Parent::onConnectionEstablished();
    printf("%i.nTNLConnection::onConnectionEstablished()\n", id );
    if(isInitiator()) {
        if (!this->isLocalConnection()){
            setGhostFrom(false);
            setGhostTo(true);
        } else {
            setGhostFrom(false);
            setGhostTo(false);
        }
        printf("%s - connected to server\n", getNetAddressString());
    }
    else {
        printf("%s - client connected\n", getNetAddressString());
        nTNLPlayer *player = new nTNLPlayer;
        setScopeObject(player);
        if (!this->isLocalConnection()){
            setGhostFrom(true);
            setGhostTo(false);
            printf("Activating Ghosting\n");
            activateGhosting();
        } else {
            setGhostFrom(false);
            setGhostTo(false);
        }
    }
}
//-----------------------------------------------------------------------------//
