#include "enkiduo/ngameserver.h"
#include "tnl/ntnlent.h"
using namespace TNL;
//------------------------------------------------------------------------------
//        TNLENT STUFF    
//------------------------------------------------------------------------------
TNL_IMPLEMENT_NETOBJECT(nTNLEnt); 

nTNLEnt::nTNLEnt():
    ent(0),
    vars(30,0) //IMPORTANT 30 -> the number of possible variables (32 - the number ov other masks used
{
    mNetFlags.set(Ghostable);
    //printf("nTNLEnt::nTNLEnt()\n");
}
//-----------------------------------------------------------------------------//
void nTNLEnt::SetEnt( nEnt* e ) {
    //n_printf("nTNLEnt->SetEnt(%s)\n", e->fn);
    this->ent = e;
    varRoot = e->Find("vars");
    varTail = 0;
}
//-----------------------------------------------------------------------------//
void nTNLEnt::tick( nTNLConnection* connection, nTNLEntTicks tickMode){
    if( varTail != varRoot->GetTail() ){ //we have new vars
        foreachchild( nVar, v, varRoot ) {
            bool found = false;
            for( int i = 0; i < vars; i ++ ) {
                if( vars[i].GetVar() == v ) {
                    found = true;
                    break;
                }
            }
            if( !found && vars.Size() < 30 ) {
                int cat = v->GetCategory();
                if( cat == VARCAT_SERVER || cat == VARCAT_CLIENT ) { //yay send me
                    //printf("new var: %s withcat:%i\n", v->fn, cat );
                    var& nv = vars.Push();
                    nv.Connect( v );
                    varTail = v;
                }
                if( cat == VARCAT_CLIENT ) { //yay send me
                    var& nv = clientVars.Push();
                    nv.Connect( v );
                }
            }
        }
        //n_assert( varTail == varRoot->GetTail() );
        varTail = varRoot->GetTail();
        if (tickMode == TNLENTTICK_SERVER){
            this->setMaskBits( VarsChangedMask );
        }
    }
    if (tickMode == TNLENTTICK_SERVER){
        U32 mask = FirstVarMask;
        for( int i = 0; i < vars; i ++ ) {
            if( vars[i].isvalid() ) {
                if( vars[i].isdirty() ) {
                    //printf("var:%s is dirty()\n", vars[i].GetVarName());
                    this->setMaskBits( mask );
                }
            }
            else
                ;//FIXME printf("%s is invalid\n", vars[i].GetVarName() );
            mask <<= 1;
        }
    }
    if (tickMode == TNLENTTICK_CLIENT){
        for( int i = 0; i < clientVars; i ++ ) {
            if( clientVars[i].isdirty() ){ //this var has changed - call the rpc
                //n_printf("client var:%s has changed\n", clientVars[i].GetVar()->fn );
                int type = clientVars[i].GetType();
                vector3 v;
                quaternion q;
                //StringEntryTable s;
                switch (clientVars[i].GetType() ){
                    case VARTYPE_VOID:        
                        break;
                    case VARTYPE_INT:        
                        connection->rpcSetVarI( ent->fn, clientVars[i].GetVarName(), clientVars[i].GetI() );
                        break;
                    case VARTYPE_FLOAT:
                        connection->rpcSetVarF( ent->fn, clientVars[i].GetVarName(), clientVars[i].GetF() );
                        break;
                    case VARTYPE_VECTOR3:
                        v = clientVars[i].GetV();
                        connection->rpcSetVarV( ent->fn, clientVars[i].GetVarName(), v.x,v.y,v.z );
                        break;
                    case VARTYPE_QUATERNION:
                        q = clientVars[i].GetQ();
                        break;
                    case VARTYPE_STRING:
                        //s = clientVars[i].GetS().Get();
                        break;
                    default:
                        printf("client write -- ignoring var %s with type: %i\n", clientVars[i].GetVarName(), clientVars[i].GetType() );
                }
            }
        }
    }
    for( int i = 0; i < vars; i ++ ) {
        //printf("vars[%i].ApplyIpol()\n", i );
        vars[i].ApplyIpol();
    }
}
TNL::U32 nTNLEnt::packUpdate(TNL::GhostConnection *connection, TNL::U32 updateMask, TNL::BitStream *stream) {
    Parent::packUpdate( connection, updateMask, stream );
    //printf("nTNLEnt::packUpdate()\n");
    if( stream->writeFlag( updateMask & InitialMask ) ){
        StringTableEntry type = ent->GetParent()->GetName();
        StringTableEntry name = ent->GetName();
        stream->writeStringTableEntry( type );
        stream->writeStringTableEntry( name );
    }
    if( stream->writeFlag( updateMask & VarsChangedMask ) ) {
        stream->write( vars.Size() );
        for( int i = 0; i < vars; i ++ ) {
            StringTableEntry name = vars[i].GetVarName();
            int type = vars[i].GetType();
            stream->writeStringTableEntry( name );
            stream->write( type );
        }
    }
    U32 mask = FirstVarMask;
    for( int i = 0; i < vars; i ++ ) {
        if( stream->writeFlag( updateMask & mask ) ) { //should write this variable
            //printf("sending variable: %s\n", vars[i].GetVarName() );
            vector3 v;
            quaternion q;
            StringTableEntry s;
            //if ( vars[i].GetVar()->GetCategory() == VARCAT_CLIENT)
                //n_printf("ghosting client var:%s\n", vars[i].GetVar()->fn);
            switch( vars[i].GetType() ) {
                case VARTYPE_VOID:        
                    break;
                case VARTYPE_INT:        
                    stream->write( vars[i].GetI() );    
                    break;
                case VARTYPE_FLOAT:
                    stream->write( vars[i].GetF() );
                    break;
                case VARTYPE_VECTOR3:
                    v = vars[i].GetV();
                    stream->write( v.x );
                    stream->write( v.y );
                    stream->write( v.z );
                    break;
                case VARTYPE_QUATERNION:
                    q = vars[i].GetQ();
                    stream->write( q.x );
                    stream->write( q.y );
                    stream->write( q.z );
                    stream->write( q.w );
                    break;
                case VARTYPE_STRING:
                    s = vars[i].GetS().Get();
                    stream->writeStringTableEntry(s);
                    break;
                case VARTYPE_REF:
                    s = vars[i].GetRefPath();
                    stream->writeStringTableEntry(s);
                    break;
                default:
                    printf("ignoring var %s with type: %i\n", vars[i].GetVarName(), vars[i].GetType() );
            }

        }
        mask <<=1;
    }
    return 0;
}

/// unpackUpdate reads the data the server wrote in packUpdate from the packet.
void nTNLEnt::unpackUpdate(TNL::GhostConnection *connection, TNL::BitStream *stream){
    Parent::unpackUpdate( connection, stream );
    //printf("nTNLEnt::unpackUpdate()\n");
    if(stream->readFlag()) { //initial mask
        StringTableEntry type;
        StringTableEntry name;
        stream->readStringTableEntry( &type );
        stream->readStringTableEntry( &name );
        //printf("read type: %s\n", type.getString() );
        //printf("read name: %s\n", name.getString() );

        if( gameServer == 0 )
            gameServer = (nGameServer*) ((nTNLConnection*) connection)->kernelServer->Lookup("/sys/servers/game");

        n_assert( gameServer );
        n_assert( ent == 0 );
        this->SetEnt( gameServer->GenEnt( type.getString(), 0, name.getString() ) );
        printf("entity in scope: %s\n", ent->fn );
    }
    if( stream->readFlag() ) { //vars changed mask
        int num;
        stream->read( &num );
        StringTableEntry name;
        int type;
        //printf("num:%i\n", num );
        for( int i = 0; i < num; i ++ ) {
            stream->readStringTableEntry( &name );
            stream->read( &type );
            //printf("new var: %s type:%i\n", name.getString(), type );
            bool alreadyExists = false;
            for( int j = 0; j < vars; j ++ ) {
                if (nString(vars[j].GetVarName()) == name.getString()) {
                    alreadyExists = true;
                    n_printf("%s.prexisting var:%s\n", ent->fn, name.getString());
                }
            }
            if ( alreadyExists == false ) {
                //n_printf("%s.new var:%s\n", ent->fn, name.getString());
                var& v = vars.Push();
                v.Connect( ent, name.getString() );
                switch( type ) {
                    case VARTYPE_VOID:        
                        break;
                    case VARTYPE_INT:        
                        vars[i].DefaultI( 0 );
                        break;
                    case VARTYPE_FLOAT:
                        vars[i].DefaultF( 0.0f );
                        break;
                    case VARTYPE_VECTOR3:
                        vars[i].DefaultV( 0,0,0 );
                        break;
                    case VARTYPE_QUATERNION:
                        vars[i].DefaultQ( 0,0,0,1);
                        break;
                    case VARTYPE_STRING:
                        vars[i].DefaultS("");
                        break;
                    case VARTYPE_REF:
                        vars[i].DefaultRef("");
                        break;
                    default:
                        n_printf("unknown default type:%i\n", type);
                }
            }
        }
    }
    for( int i = 0; i < vars; i ++ ) {
        int in;
        float f;
        vector3 v;
        quaternion q;
        StringTableEntry s;
        if( stream->readFlag() ) {
            //printf("reading variable: %s, %i\n", vars[i].GetVarName(), vars[i].GetType() );
            switch( vars[i].GetType() ) {
                case VARTYPE_VOID:        
                    break;
                case VARTYPE_INT:        
                    stream->read( &in );
                    vars[i].SetI( in );
                    break;
                case VARTYPE_FLOAT:
                    stream->read( &f );
                    vars[i].SetF( f );
                    break;
                case VARTYPE_VECTOR3:
                    stream->read( &v.x );
                    stream->read( &v.y );
                    stream->read( &v.z );
                    //vars[i].GetVar()->SetIpolV( v );
                    vars[i].SetV(v);
                    break;
                case VARTYPE_QUATERNION:
                    stream->read( &q.x );
                    stream->read( &q.y );
                    stream->read( &q.z );
                    stream->read( &q.w );
                    vars[i].SetQ( q );
                    break;
                case VARTYPE_STRING:
                    stream->readStringTableEntry( &s );
                    vars[i].SetS( s.getString() );
                    break;
                case VARTYPE_REF:
                    stream->readStringTableEntry( &s );
                    //printf("set ref:%s->%s\n", vars[i].GetVar()->fn, s.getString());
                    vars[i].SetRef( s.getString() );
                    break;
                default:
                    printf("ignoring var %s with type: %s\n", vars[i].GetVar()->fn, vars[i].GetType() );
                    break;
            }
            if( vars[i].GetVar()->GetCategory() == VARCAT_CLIENT) {
                //n_printf("server setting client var %s\n", vars[i].GetVar()->fn);
                vars[i].isdirty();
                for( int i = 0; i < clientVars; i ++ ) {
                    if ( clientVars[i].GetVar() == vars[i].GetVar() ) { //its that one
                        //n_printf("setting local clientVar[%i] not dirty()\n", i);
                        clientVars[i].isdirty();
                    }
                }
            }
        }
    }
}
void nTNLEnt::onGhostAvailable(TNL::GhostConnection *theConnection){
    //printf("nTNLEnt::onGhostAvailable()\n");
    Parent::onGhostAvailable( theConnection );
}
bool nTNLEnt::onGhostAdd(TNL::GhostConnection *theConnection)
{
    //printf("nTNLEnt::onGhostAdd()\n");
    return Parent::onGhostAdd( theConnection );
}
void nTNLEnt::onGhostRemove(){
    if( ent )
        ent->Die();
}
