#define N_IMPLEMENTS nInterfaceServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ui/ninterfaceserver.h"
#include "kernel/ntimeserver.h"
#include "ui/nuielement.h"
#include "enkiduo/ngameserver.h"
#include "mathlib/plane.h"
#include "map/nmap.h"

nNebulaScriptClass(nInterfaceServer, "nfsm");

//------------------------------------------------------------------------------
/**
*/
nInterfaceServer::nInterfaceServer():
    //pathfinder                          ( kernelServer, this ),
    ref_map                             ( kernelServer, this ),
    ref_game                            ( kernelServer, this ),
    mouseReach                          ( 10000.0f )
{
    ref_game = "/sys/servers/game";
}

void nInterfaceServer::Initialize()
{
    nFSM::Initialize();
    //n_printf("controlling.Connect()\n");
    //AddToDisplayList();
    //
    camera.Connect( ent, "camera" );
    camera.DefaultRef("/game/camera");
    cameraHeading.Connect(camera, "heading");
    cameraHeading.DefaultF(0.0f);
    cameraPos.Connect( camera, "pos" );
    cameraRot.Connect( camera, "rot" );

    projectCameraCornerIntersections.Connect(ent,"projectCameraCornerIntersections");
    projectCameraCornerIntersections.DefaultI(1);
    cameraULIntersection.Connect( camera, "cameraULIntersection");
    cameraURIntersection.Connect( camera, "cameraURIntersection");
    cameraLLIntersection.Connect( camera, "cameraLLIntersection");
    cameraLRIntersection.Connect( camera, "cameraLRIntersection");

    projectMouse.Connect( ent, "projectmouse" );
    projectMouse.DefaultI(1);

    mousePos.Connect(ent, "mousepos");
    mousePos.DefaultV(0,0,0);
    mouseSensitivity.Connect(ent, "mousesensitivity");
    mouseSensitivity.DefaultF(-2.0);
    mouseScrollRate.Connect( ent, "mousescrollrate" );
    mouseScrollRate.DefaultF( 1.0f );

    //linearThrust.Connect( controlling, "thrust" );
    //linearThrust.DefaultF( 1.0f );
    //angularThrust.Connect( controlling, "rthrust" );
    //angularThrust.DefaultF( 1.0f );
    ref_map   = "/activemap";
    //pathfinder = "/sys/servers/pathfinder";

    windowSettings.Connect( ent, "windowSettings");
    windowSettings.DefaultRef("/sys/share/windowsettings");
    aspectRatio.Connect( windowSettings, "aspectRatio" );
}
//------------------------------------------------------------------------------
nInterfaceServer::~nInterfaceServer()
{
}
//-----------------------------------------------------------------------------//
vector3 nInterfaceServer::ScreenToGroundPlane( vector2 c ){
    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();
    vector3 a(0,0,0);
    vector3 b((1-c.x) * -2.0f + 1.0f,
            c.y  * -2.0f + 1.0f,
            -1.0f); //screen pos mapped from -1->1
    b.x *= aspect;
    //n_printf("a:%s b:%s\n", str(a), str(b));
    quaternion q = cameraRot.GetQ();

    vector3 p  = cameraPos.GetV();
    vector3 m  = q.rotate(a) + p;
    vector3 n  = q.rotate(b) + p;
    line3 line(m,n);
    plane pl;
    pl.set( vector3(0,0,0),
            vector3(1,0,0),
            vector3(0,0,1) );
    float t;
    if( pl.intersect( line, t ) ){
        if (t < 0.0f || t > mouseReach) 
            return vector3(0,0,0);
        vector3 d = line.b + line.m * t;
        return d;
    }
    return vector3(0,0,0);
}
//-----------------------------------------------------------------------------//
vector3 nInterfaceServer::ScreenToWorld( vector2 c ){
    if(ref_map.isvalid() == false ) 
        return ScreenToGroundPlane(c);
    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();
    vector3 p = cameraPos.GetV();
    quaternion r = cameraRot.GetQ();
    c.x = c.x * 2.0f - 1.0f;
    c.x *= aspect;
    c.y = c.y *-2.0f + 1.0f;
    vector3 a, b;
    a = p;
    b.set( c.x, c.y, -1.0f );
    b = r.rotate( b ) * mouseReach;
    b += a;
    vector3 contact;
    ref_map->GetIntersect(line3(a,b), contact);
    if( fabs(contact.x - a.x ) < 10.0f || fabs( contact.z - a.z )  < 10.0f ) {
        return vector3(0,0,0);
    }
    return contact;
}
//-----------------------------------------------------------------------------//
bool nInterfaceServer::States( StateMachineEvent event, uint state, float time, float dtime ){
    if( projectMouse.GetI() > 0 )
        this->ProjectMouse();
    if( projectCameraCornerIntersections.GetI() > 0 )
        this->ProjectCameraCornerIntersections();

    /*

    float rtime = (float) kernelServer->ts->GetFrameTime();
    if( rtime0 < 0 ) 
        rtime0 = rtime;

    bool physicsDirty = false;
    if( controlling.isdirty() ){
        nString s = controlling.GetS();
        if( !(s=="none")){
            entControlling = s.get();
            physicsDirty = true;
        }
    }
    if( physicsDirty || entPhysics.isvalid() == false ){
        if( entControlling.isvalid() ){
            nString p = entControlling->ChildPath("physics");
            entPhysics = p.Get();
            linearThrust.Connect( entControlling.get(), "thrust" );
            linearThrust.DefaultF( 1.0f );
            angularThrust.Connect( entControlling.get(), "rthrust" );
            angularThrust.DefaultF( 1.0f );
        }
        else
            once( n_printf("entControlling.isvalid() == false\n") );
    }
    //else
    //once( n_printf("physicsDirty == false && physicsEntSelected.isvalid() == true\n") );


    float lf = linearThrust.GetF();
    float af = angularThrust.GetF();

    MouseInfo mouseB = ref_game->GetMouseInfo();
    vector2 mouseDiff = mouseB.pos - mouseA.pos;
    mouseA = mouseB;
    mouseDiff *= mouseSensitivity.GetF();
    BeginStateMachine
        State(startup)
        OnEnter
        SetState(Control3D);
    State(Control2D)
        OnTick
        if( entPhysics.isvalid() ){
            if( ref_game->GetButton("pitchup") )      entPhysics->ApplyAlignedForce( vector3( 0, 0.000,-af) );
            if( ref_game->GetButton("pitchdown") )    entPhysics->ApplyAlignedForce( vector3( 0, 0.000, af) );

            if( ref_game->GetButton("yawleft") )      entPhysics->ApplyAlignedTorque( vector3( 0.000, af, 0.000) );
            if( ref_game->GetButton("yawright") )     entPhysics->ApplyAlignedTorque( vector3( 0.000,-af, 0.000) );
        }
        else
            once( n_printf("nInterfaceServer::entPhysics.isvalid() == false\n") );
    State(Control3D)
        OnTick
        if( entPhysics.isvalid() ){
            if( ref_game->GetButton("forward") )      entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000,-lf) );
            if( ref_game->GetButton("backward") )     entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000, lf) );
            if( ref_game->GetButton("up") )           entPhysics->ApplyAlignedForce( vector3( 0.000, lf, 0.000) );
            if( ref_game->GetButton("down") )         entPhysics->ApplyAlignedForce( vector3( 0.000,-lf, 0.000) );
            if( ref_game->GetButton("left") )         entPhysics->ApplyAlignedForce( vector3(-lf, 0.000, 0.000) );
            if( ref_game->GetButton("right") )        entPhysics->ApplyAlignedForce( vector3( lf, 0.000, 0.000) );

            if( ref_game->GetButton("rollleft") )     entPhysics->ApplyAlignedTorque( vector3( 0.000, 0.000, af) );
            if( ref_game->GetButton("rollright") )    entPhysics->ApplyAlignedTorque( vector3( 0.000, 0.000,-af) );
            if( ref_game->GetButton("yawleft") )      entPhysics->ApplyAlignedTorque( vector3( 0.000, af, 0.000) );
            if( ref_game->GetButton("yawright") )     entPhysics->ApplyAlignedTorque( vector3( 0.000,-af, 0.000) );
            if( ref_game->GetButton("pitchup") )      entPhysics->ApplyAlignedTorque( vector3(-af, 0.000, 0.000) );
            if( ref_game->GetButton("pitchdown") )    entPhysics->ApplyAlignedTorque( vector3( af, 0.000, 0.000) );
        }
        else
            n_printf("nInterfaceServer::entPhysics.isvalid() == false\n");
    State(ControlOffset)
        OnTick
            vector3 p = ref_camera->GetVarV("poffset");
            quaternion q = ref_camera->GetVarQ("roffset");
            vector3 r;
            float phi = atan2( p.x, p.z );
            float rho = atan2( p.y, vector2(p.x,p.z).len() );
            float dist = p.len();
            bool found = false;
            float f = 5.0;
            if( ref_game->GetButton("forward") ){          dist -= f / 10.0;        found = true; }
            if( ref_game->GetButton("backward") ){         dist += f / 10.0;        found = true; }
            if( ref_game->GetButton("left") ){          phi += f * dtime;    r.y += f * dtime;    found = true; }
            if( ref_game->GetButton("right") ){         phi -= f * dtime;    r.y -= f * dtime;    found = true; }
            if( ref_game->GetButton("up") ){          rho += f * dtime;    r.x -= f * dtime;    found = true; }
            if( ref_game->GetButton("down") ){         rho -= f * dtime;    r.x += f * dtime;    found = true; }

            if( ref_game->GetButton("pitchup") ){          r.x += f * dtime;    found = true; }
            if( ref_game->GetButton("pitchdown") ){         r.x -= f * dtime;    found = true; }
            if( ref_game->GetButton("yawleft") ){          r.y += f * dtime;    found = true; }
            if( ref_game->GetButton("yawright") ){         r.y -= f * dtime;    found = true; }
            if( ref_game->GetButton("rollleft") ){          r.z += f * dtime;    found = true; }
            if( ref_game->GetButton("rollright") ){         r.z -= f * dtime;    found = true; }
            if( dist < 0 )
                dist = 0;
            if ( found ) {
                vector3 pn( dist * sin(phi) * cos(rho),dist * sin(rho), dist * cos(phi) *cos(rho));
                ref_camera->SetVarV("poffset", pn );
                quaternion qr;
                qr.set_rotate_xyz( r.x, r.y, r.z );
                ref_camera->SetVarQ("roffset", q * qr );
            }
               if( ref_game->GetButton("backward") )     entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000, lf) );
               if( ref_game->GetButton("left") )         entPhysics->ApplyAlignedForce( vector3(-lf, 0.000, 0.000) );
               if( ref_game->GetButton("right") )        entPhysics->ApplyAlignedForce( vector3( lf, 0.000, 0.000) );

               if( ref_game->GetButton("rollleft") )     entPhysics->ApplyAlignedTorque( vector3( 0.000, 0.000, af) );
               if( ref_game->GetButton("rollright") )    entPhysics->ApplyAlignedTorque( vector3( 0.000, 0.000,-af) );
               if( ref_game->GetButton("yawleft") )      entPhysics->ApplyAlignedTorque( vector3( 0.000, af, 0.000) );
               if( ref_game->GetButton("yawright") )     entPhysics->ApplyAlignedTorque( vector3( 0.000,-af, 0.000) );
               if( ref_game->GetButton("pitchup") )      entPhysics->ApplyAlignedTorque( vector3(-af, 0.000, 0.000) );
               if( ref_game->GetButton("pitchdown") )    entPhysics->ApplyAlignedTorque( vector3( af, 0.000, 0.000) );
            State(ControlPlane)
                OnTick
                rdtime = dtime;
            if( entPhysics.isvalid() ){
                if( ref_game->GetButton("forward") )      entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000,-lf) );
                if( ref_game->GetButton("backward") )     entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000, lf) );
                //if( ref_game->GetButton("up") )           entPhysics->ApplyAlignedForce( vector3( 0.000, rdtime, 0.000) );
                //if( ref_game->GetButton("down") )         entPhysics->ApplyAlignedForce( vector3( 0.000,-rdtime, 0.000) );
                //if( ref_game->GetButton("left") )         entPhysics->ApplyAlignedForce( vector3(-rdtime, 0.000, 0.000) );
                //if( ref_game->GetButton("right") )        entPhysics->ApplyAlignedForce( vector3( rdtime, 0.000, 0.000) );

                if( ref_game->GetButton("rollleft") )     entPhysics->ApplyAlignedTorque( vector3( 0.000, 0.000, af) );
                if( ref_game->GetButton("rollright") )    entPhysics->ApplyAlignedTorque( vector3( 0.000, 0.000,-af) );
                if( ref_game->GetButton("yawleft") )      entPhysics->ApplyAlignedTorque( vector3( 0.000, af, 0.000) );
                if( ref_game->GetButton("yawright") )     entPhysics->ApplyAlignedTorque( vector3( 0.000,-af, 0.000) );
                if( ref_game->GetButton("pitchup") )      entPhysics->ApplyAlignedTorque( vector3(-af, 0.000, 0.000) );
                if( ref_game->GetButton("pitchdown") )    entPhysics->ApplyAlignedTorque( vector3( af, 0.000, 0.000) );

                   if( ref_game->GetButton("fireprimary" ) ) 
                   ifvalid( primaryWeapon ) 
                   primaryWeapon->Fire();
            }
            else
                n_printf("nInterfaceServer::entPhysics.isvalid() == false\n");
    State(ControlTurtle)
        OnTick
        if( entPhysics.isvalid() ){
            if( ref_game->GetButton("forward") ) {
                if( ref_game->GetButton("boost") ){
                    if(time - timeLastBoost > forwardBoostTime.GetF() ){
                        lf*=20;
                        timeLastBoost=time;
                    }
                }
                entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000,-lf) );

            } else {
                if( ref_game->GetButton("boost") ){
                    if(time - timeLastBoost > boostTime.GetF() ){
                        lf*=200;
                        timeLastBoost=time;
                    }
                }
            }
            if( ref_game->GetButton("backward") )     entPhysics->ApplyAlignedForce( vector3( 0.000, 0.000, lf) );
            if( ref_game->GetButton("up") )           entPhysics->ApplyAlignedForce( vector3( 0.000, lf, 0.000) );
            if( ref_game->GetButton("down") )         entPhysics->ApplyAlignedForce( vector3( 0.000,-lf, 0.000) );
            if( ref_game->GetButton("left") )         entPhysics->ApplyAlignedForce( vector3(-lf, 0.000, 0.000) );
            if( ref_game->GetButton("right") )        entPhysics->ApplyAlignedForce( vector3( lf, 0.000, 0.000) );

            if( ref_game->GetButton("yawleft") )      entPhysics->ApplyAlignedTorque( vector3( 0.000,-af, 0.000) );
            if( ref_game->GetButton("yawright") )     entPhysics->ApplyAlignedTorque( vector3( 0.000, af, 0.000) );
            if( ref_game->GetButton("pitchup") )      entPhysics->ApplyAlignedTorque( vector3( af, 0.000, 0.000) );
            if( ref_game->GetButton("pitchdown") )    entPhysics->ApplyAlignedTorque( vector3(-af, 0.000, 0.000) );

            entPhysics->ApplyAlignedTorque( vector3( -mouseDiff.y, -mouseDiff.x, 0.000 ) * af );
            if( mouseB.pressed[0]) {
                nWeapon * w = (nWeapon *) entControlling -> Find("primaryweapon");
                nString target = entControlling->GetVarRef("target");
                if( w )
                    if( target != "none" )
                        w->Fire( target.Get() );
                    else
                        w->Fire();
            }
        }
        else
            n_printf("nInterfaceServer::entPhysics.isvalid() == false\n");
    State(ControlCar)
        OnTick
        if( entControlling.isvalid() ) {
            entControlling->SetVarF("throttle", 0.0 );
            entControlling->SetVarF("steering", 0.0 );
            if( ref_game->GetButton("up") )     entControlling->SetVarF("throttle", +1.0 );
            if( ref_game->GetButton("down") )     entControlling->SetVarF("throttle", -1.0 );
            if( ref_game->GetButton("left") )     entControlling->SetVarF("steering", -1.0 );
            if( ref_game->GetButton("right") )     entControlling->SetVarF("steering", +1.0 );

            //n_printf("%f %f", entControlling->GetVarF("throttle"), entControlling->GetVarF("steering") );
        } else {
            n_printf("ent controlling invalid\n");
        }
    State(RTSCamera)
        OnTick
        if( entPhysics.isvalid() ){
            vector3 fr(0,0,-1);
            vector3 rt(-1,0,0);
            quaternion q = ref_camera->GetVarQ("rot");
            fr = q.rotate( fr );
            rt = q.rotate( rt );
            fr.y = 0;
            rt.y = 0;
            fr.norm();
            rt.norm();
            vector2 mp = ref_game->GetMouseInfo().pos;
            //n_printf("mp:%s\n", str(mp) );
            float msr = mouseScrollRate.GetF();
            if( mp.x == 0 && mp.y == 0 ) { } else {            
                if( mp.x < .01)         entPhysics->ApplyForce( vector3( rt.x * msr, 0, rt.z * msr ) );
                if( mp.x > .99)         entPhysics->ApplyForce( vector3(-rt.x * msr, 0,-rt.z * msr ) );
                if( mp.y < .01)         entPhysics->ApplyForce( vector3( fr.x * msr, 0, fr.z * msr ) );
                if( mp.y > .99)         entPhysics->ApplyForce( vector3( -fr.x * msr, 0, -fr.z * msr ) );
            }
            if( ref_game->GetButton("pitchup") )    entPhysics->ApplyForce( vector3(  fr.x, 0.000, fr.z) );
            if( ref_game->GetButton("pitchdown") )    entPhysics->ApplyForce( vector3( -fr.x, 0.000,-fr.z) );
            if( ref_game->GetButton("up") )    entPhysics->ApplyForce( vector3( 0.000, lf, 0.000) );
            if( ref_game->GetButton("down") )    entPhysics->ApplyForce( vector3( 0.000,-lf, 0.000) );
            if( ref_game->GetButton("yawleft") )    entPhysics->ApplyForce( vector3( rt.x, 0.000, rt.z) );
            if( ref_game->GetButton("yawright") )    entPhysics->ApplyForce( vector3( -rt.x, 0.000,-rt.z) );
        }
        else
            n_printf("nInterfaceServer::entPhysics.isvalid() == false\n");


    EndStateMachine
        */
    return true;
}

void nInterfaceServer::ProjectMouse(){
    vector3 pos;
    vector2 mp = ref_game->GetMouseInfo().pos;
    if( ref_map.isvalid() )
        pos = ScreenToWorld( mp );
    else
        pos = ScreenToGroundPlane( mp );
    //n_printf("pos:%s\n", str(pos));
    if( fabs( pos.x ) > 5.0f  && fabs(pos.z) > 5.0f )
        mousePos.SetV( pos );
}

void nInterfaceServer::ProjectCameraCornerIntersections(){

    quaternion q                        = cameraRot.GetQ();
    vector3 forward                     = q.rotate(vector3(0,0,-1));
    float heading                       = atan2(forward.x, forward.z);
    cameraHeading.SetF( heading );
    return;



    vector3 pos;
    vector2 mp;
    float x0 = 0.1f;
    float x1 = 0.9f;
    float xstep = .1f;
    float y0 = 0.1f;
    float y1 = 0.9f;
    float ystep = .001f;
    vector3 point;
    bool found;
    if( ref_map.isvalid() ) {
        found = false;
        for (     float x = x0; !found && x <= x1; x += xstep ) {
            for ( float y = y0; !found && y <= y1; y += ystep ) { 
                point = this->ScreenToWorld(vector2(x,y));
                if (point.x > 0.0f) found = true;
            }
        }
        cameraULIntersection.SetV( point );
        found = false;
        for (     float x = x1; !found && x >= x0; x -= xstep ) {
            for ( float y = y0; !found && y <= y1; y += ystep ) { 
                point = this->ScreenToWorld(vector2(x,y));
                if (point.x > 0.0f) found = true;
            }
        }
        cameraURIntersection.SetV( point );
        found = false;
        for (     float x = x0; !found && x <= x1; x += xstep ) {
            for ( float y = y1; !found && y >= y0; y -= ystep ) { 
                point = this->ScreenToWorld(vector2(x,y));
                if (point.x > 0.0f) found = true;
            }
        }
        cameraLLIntersection.SetV( point );
        found = false;
        for (     float x = x1; !found && x >= x0; x -= xstep ) {
            for ( float y = y1; !found && y >= y0; y -= ystep ) { 
                point = this->ScreenToWorld(vector2(x,y));
                if (point.x > 0.0f) found = true;
            }
        }
        cameraLRIntersection.SetV( point );

        //cameraURIntersection.SetV( this->ScreenToWorld(vector2( x[0],y[1])) );
        //cameraLLIntersection.SetV( this->ScreenToWorld(vector2( x[1],y[0])) );
        //cameraLRIntersection.SetV( this->ScreenToWorld(vector2( x[1],y[1])) );
        //this->DrawSphere(cameraULIntersection.GetV(), red, 50);
        //this->DrawSphere(cameraURIntersection.GetV(), blue, 50);
        //this->DrawSphere(cameraLLIntersection.GetV(), white, 50);
        //this->DrawSphere(cameraLRIntersection.GetV(), green, 50);
    } else {
        int xmisses = 0;
        int ymisses = 0;
        found = false;
        for (     float x = x0; !found && x <= x1; x += xstep ) {
            for ( float y = y0; !found && y <= y1; y += ystep ) { 
                point = this->ScreenToGroundPlane(vector2(x,y));
                if (point.x > 0.0f) found = true;
                else xmisses++;
            }
            if ( found == false ) ymisses++;
        }
        cameraULIntersection.SetV( point );
        found = false;
        for (     float x = x1; !found && x >= x0; x -= xstep ) {
            for ( float y = y0; !found && y <= y1; y += ystep ) { 
                point = this->ScreenToGroundPlane(vector2(x,y));
                if (point.x > 0.0f) found = true;
                else xmisses++;
            }
            if ( found == false ) ymisses++;
        }
        cameraURIntersection.SetV( point );
        found = false;
        for (     float x = x0; !found && x <= x1; x += xstep ) {
            for ( float y = y1; !found && y >= y0; y -= ystep ) { 
                point = this->ScreenToGroundPlane(vector2(x,y));
                if (point.x > 0.0f) found = true;
                else xmisses++;
            }
            if ( found == false ) ymisses++;
        }
        cameraLLIntersection.SetV( point );
        found = false;
        for (     float x = x1; !found && x >= x0; x -= xstep ) {
            for ( float y = y1; !found && y >= y0; y -= ystep ) { 
                point = this->ScreenToGroundPlane(vector2(x,y));
                if (point.x > 0.0f) found = true;
                else xmisses++;
            }
            if ( found == false ) ymisses++;
        }
        cameraLRIntersection.SetV( point );
        n_printf("xmisses:%i ymisses:%i\n", xmisses, ymisses);
    }
}
