#define N_IMPLEMENTS nPlannerUI
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ui/nplannerui.h"
#include "ui/nuielement.h"
#include "enkiduo/nfsmqueue.h"
#include "enkiduo/neventserver.h"
#include "ui/nplanner.h"
#include "ui/ninterfaceserver.h"
#include "util/nheap.h"

nNebulaScriptClass(nPlannerUI, "nfsm");

//------------------------------------------------------------------------------
/**
 */
nPlannerUI::nPlannerUI():
    gameServer( kernelServer, this ),
    octree( kernelServer, this ),
    eventServer( kernelServer, this ),
    interface( kernelServer, this ),
    overVis( kernelServer, this ),
    selectedVis( kernelServer, this ),
    overGroupVis( kernelServer, this ),
    selectedGroupVis( kernelServer, this ),
    catSelectedVis( kernelServer, this ),
    targettedVis( kernelServer, this )
{
}

void nPlannerUI::Initialize(){
    nFSM::Initialize();
    stateNames.Append("disabled");
    stateNames.Append("normal");
    stateNames.Append("leftmousebuttondown");
    stateNames.Append("middlemousebuttondown");
    stateNames.Append("rightmousebuttondown");
    stateNames.Append("entplacementpre");
    stateNames.Append("entplacement");
    stateNames.Append("multipleEntPlacementPre");
    stateNames.Append("multipleEntPlacement");
    stateNames.Append("boxSelection");
    stateNames.Append("RMenu");

    gameServer = "/sys/servers/game";
    octree = "/sys/servers/octree";
    eventServer = "/sys/servers/event";
    interface = "/ui/interface";
    AddToDisplayList();

    selectionEvent = 0;
    mouseOverEvent = 0;
    rightClickEvent = 0;
    rmenu                               = 0;

    this->createVisNode(this->fn);

    overVis                             = "/lib/node/overVis";
    selectedVis                         = "/lib/node/selectedVis";
    overGroupVis                        = "/lib/node/overGroupVis";
    selectedGroupVis                    = "/lib/node/selectedGroupVis";
    catSelectedVis                      = "/lib/node/catSelectedVis";
    targettedVis                        = "/lib/node/targettedVis";
    multiSelect                         = false;
}

void nPlannerUI::InitVars(){
    nFSM::InitVars();
    selected.Connect( ent, "selected" );
    selected.DefaultRef("");
    mousePos.Connect( ent, "mousepos" );
    selectionRadius.Connect( ent, "selectionradius");
    selectionRadius.DefaultF(10);
    selectionThreshold.Connect( ent, "selectionthreshold" );
    selectionThreshold.DefaultF( 0.995f );
    rotateYSensitivity.Connect( ent, "rotateysensitivity");
    rotateYSensitivity.DefaultF( 360.0f );

    camera.Connect( ent, "camera" );
    cameraPos.Connect( camera, "pos" );
    cameraRot.Connect( camera, "rot" );

    entbeingplaced.Connect( ent, "entbeingplaced");
    entbeingplaced.DefaultRef("");
    entbeingplacedpos.Connect( entbeingplaced, "pos");

    windowSettings.Connect( ent, "windowSettings");
    windowSettings.DefaultRef("/sys/share/windowsettings");
    aspectRatio.Connect( windowSettings, "aspectRatio" );

    rightClickTime.Connect(ent, "rightClickTime");
    rightClickTime.DefaultF(.15f);
}

nPlannerUI::~nPlannerUI(){
}
//------------------------------------------------------------------------------
/**
    watches the mouse for any selection / command events
 */
bool nPlannerUI::States( StateMachineEvent event, uint state, float time, float ){   
    int i;
    MouseInfo mouseInfo;
    ifvalid( gameServer )
        mouseInfo = gameServer->GetMouseInfo();
    vector2 mouseDiff = mouseInfo.pos - lastMouseInfo.pos;
    lastMouseInfo = mouseInfo;
    this->UpdateObjectsOver();
    float height = cameraPos.GetV().y;
    height /= 100.0f;
    /*
    for( i = 0; i < objectsOver.Size(); i ++ )
        this->DrawCircle( objectsOver[i]->GetVarV("pos") + vector3(0.0f, height, 0.0f), green, 
                objectsOver[i]->HasVar("selectionCircleRadius") ? objectsOver[i]->GetVarF("selectionCircleRadius") * .75: 5.0 );
    for( i = 0; i < objectsSelected.Size(); i ++ )
        this->DrawCircle( objectsSelected[i]->GetVarV("pos") + vector3(0.0, height, 0.0), yellow, 
                objectsSelected[i]->HasVar("selectionCircleRadius") ? objectsSelected[i]->GetVarF("selectionCircleRadius") : 10.0 );
    */
    if( state == Disabled ) 
        return false;

    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();
    //-----------------------------------------------------------------------------//
    BeginStateMachine
        //-----------------------------------------------------------------------------//
        State ( Disabled )
            n_error("cannot get here");
        //-----------------------------------------------------------------------------//
        State( Normal )
            OnTick
                if( mouseInfo.pressed[0] )
                    SetState( LeftMouseButtonDown );
                else if( mouseInfo.pressed[1] )
                    SetState( MiddleMouseButtonDown );
                else if( mouseInfo.pressed[2] )
                    SetState( RightMouseButtonDown );
        //-----------------------------------------------------------------------------//
        State( LeftMouseButtonDown )
            OnEnter
                this->MouseEvent( 0 );
                timedown[0] = time;
            OnTick
                if ( mouseInfo.pressed[0] == false )
                    SetState( Normal );
                else if (time - timedown[0] > .2f)
                    this->SetState(BoxSelection);
        //-----------------------------------------------------------------------------//
        State( MiddleMouseButtonDown )
            OnEnter
                //this->MouseEvent( 1 );
                timeRightMouseButtonDown = time;
            OnTick
                //if( mouseInfo.pressed[1] == false )
                    //SetState( Normal );
                /*
                float ry = mouseDiff.y * rotateYSensitivity.GetF();
                for( i = 0; i < objectsSelected; i ++ ) {
                    quaternion q = objectsSelected[i]->GetVarQ("rot");
                    quaternion q1;
                    q1.set_rotate_y( n_deg2rad(ry) );
                    q = q1 * q;
                    objectsSelected[i]->SetVarQ("rot", q );
                }
                */
                float timeDown = time - timeMiddleMouseButtonDown;
                if (timeDown > rightClickTime.GetF() ) {  //popupmenu
                    this->catMenu = true;
                    this->SetState(RMenu);
                }
                else if( mouseInfo.pressed[1] == false ) { //middle click
                    this->MouseEvent( 1 );
                    SetState( Normal );
                }

        //-----------------------------------------------------------------------------//
        State( RightMouseButtonDown )
            //the right mouse button is down, if they raise it quickly then pull a right click event, otherwise go into an open popup menu state
            OnEnter
                timeRightMouseButtonDown = time;
            OnTick
                float timeDown = time - timeRightMouseButtonDown;
                if (timeDown > rightClickTime.GetF() ) {  //popupmenu
                    this->catMenu = false;
                    this->SetState(RMenu);
                }
                else if( mouseInfo.pressed[2] == false ) { //right click
                    this->MouseEvent( 2 );
                    SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( EntPlacementPre )		//otherwise it will pull the same click that activated the button thats trying to place the ent
            OnEnter
            OnTick
                if( mouseInfo.pressed[0] == false ) {
                    this->SetState( EntPlacement );
                }
        //-----------------------------------------------------------------------------//
        State( MultipleEntPlacementPre )
            OnEnter
            OnTick
                if( mouseInfo.pressed[0] == false )
                    this->SetState( MultipleEntPlacement );
        //-----------------------------------------------------------------------------//
        State( EntPlacement )
            OnEnter
                n_assert( entbeingplaced.isvalid() );
            OnExit
                entbeingplaced.SetRef("");
            OnTick
                ifvalid( entbeingplaced )
                    entbeingplacedpos.SetV( mousePos.GetV() );
                if( mouseInfo.pressed[0] == true ) {                            //place it
                    this->SetState( Normal );
                    static void* v;
                    eventServer->pull(v, "entPlaced", str(mousePos.GetV()));
                }
                else if( mouseInfo.pressed[2] == true ) {                       //destroy it
                    entbeingplaced.GetRef()->Die();
                    this->SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( BoxSelection )
            OnTick
                if ( mouseInfo.pressed[0] == false ) {
                    this->BoxSelect();
                    SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( MultipleEntPlacement )
            OnExit
                entsBeingPlaced.Clear();
            OnTick
                for( int i = 0; i < entsBeingPlaced; i ++ )
                    entsBeingPlaced[i].pos.SetV(mousePos.GetV() + entsBeingPlaced[i].offset);
                if( mouseInfo.pressed[0] == true )
                    this->SetState( Normal );
                else if( mouseInfo.pressed[2] == true ) {
                    for( int i = 0; i < entsBeingPlaced; i ++ )
                        entsBeingPlaced[i].ent->Die();
                    this->SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( RMenu )
            ///so the user has open a right click menu -> what we need to do here is find the actual menu they want, and then open it
            ///okay so we need to check if the user has rightclicked on a unit, or in the open world or something
            OnEnter
                n_printf("Rmenu.enter\n");
                n_assert                (this->rmenu == 0);
                nString rmenuPath = "/ui/rmenus/command/";

                if ( this->catMenu ) {
                    if ( catSelectedEntities.Size() > 0 ) 
                        if ( objectsOver.Size() > 0  ) 
                            if ( catSelectedEntities[0] != objectsOver[0] )             
                                rmenuPath += "catUnitToUnit";              //commanding one entity to another entity
                            else                                                    rmenuPath += "catUnit";                    //right clicking on the selected entity
                        else                                                        rmenuPath += "catUnitToWorld";             //commanding one entity to the world
                    else if (objectsOver.Size() > 0 )                               rmenuPath += "catUnit";                    //right clicking on a unit
                    else                                                            rmenuPath += "catWorld";                   //right clicking on the world
                } else {
                    if ( objectsSelected.Size() > 0 ) 
                        if ( objectsOver.Size() > 0  ) 
                            if ( objectsSelected[0] != objectsOver[0] ) {
                                nString fromType = objectsSelected[0]->GetParent()->GetName();
                                nString toType   = objectsOver[0]->GetParent()->GetName();
                                n_printf("unitToUnit(%s->%s)\n", fromType.Get(), toType.Get() );
                                nString possibleRMenuPath = rmenuPath + fromType + "to" + toType;
                                n_printf("possibleRMenuPath:%s\n", possibleRMenuPath.Get() );
                                if ( LOOKUP(nEnt*, possibleRMenuPath.Get())){
                                    n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                                    rmenuPath = possibleRMenuPath;
                                } else {
                                    rmenuPath += "unitToUnit";              //commanding one entity to another entity
                                }
                            }
                            else                                                    rmenuPath += "unit";                    //right clicking on the selected entity
                        else                                                        rmenuPath += "unitToWorld";             //commanding one entity to the world
                    else if (objectsOver.Size() > 0 )                               rmenuPath += "unit";                    //right clicking on a unit
                    else                                                            rmenuPath += "world";                   //right clicking on the world
                }
                rmenu = (nEnt*) kernelServer->Lookup(rmenuPath.Get());
                if ( rmenu == 0 )
                    n_printf("failed to find appropriate rmenu to show: %i %i %s\n", objectsSelected.Size(), objectsOver.Size(), rmenuPath.Get());
                else {
                    //n_printf("rmenu:%p->%s\n", rmenu, rmenu->fn);
                    rmenu->SetVarI("menuActive", 1);

                    rmenu->CreateVar("mousePos")->SetV(mousePos.GetV());
                    rmenu->CreateVar("objectsOver")->SetS(this->getObjectsOverString());
                    rmenu->CreateVar("objectsSelected")->SetS(this->getObjectsSelectedString());
                    vector2 ringCenter;

                    if ( this->catMenu ) {
                        ringCenter.x            = mouseInfo.posPressed[1].x;
                        ringCenter.y            = mouseInfo.posPressed[1].y;
                    } else {
                        ringCenter.x            = mouseInfo.posPressed[2].x;
                        ringCenter.y            = mouseInfo.posPressed[2].y;
                    }
                    //n_printf("ringCenter;%s\n", str(ringCenter));
                    ringCenter.x            = (ringCenter.x - .5) * 2.0f * aspectRatio.GetF();
                    ringCenter.y            = (ringCenter.y - .5) * -2.0f;
                    //n_printf("ringCenter:%s\n", str(ringCenter));
                    rmenu->SetVarV          ("ringCenter", ringCenter.x, ringCenter.y, 0);
                }
            OnExit
                if ( rmenu ) {
                    rmenu->SetVarI          ("menuActive", 0);
                    rmenu = 0;
                } else {
                    n_warn("exiting out of non open rmenu\n");
                }
            OnTick
                //n_printf("catMenu:%i pressed:%i,%i,%i\n", this->catMenu, mouseInfo.pressed[0], mouseInfo.pressed[1], mouseInfo.pressed[2]);
                if ( this->catMenu ) {
                    if ( mouseInfo.pressed[1] == false ) {
                        SetState("normal");
                    }
                }
                else if ( mouseInfo.pressed[2] == false ) {
                    SetState("normal");
                }
        //-----------------------------------------------------------------------------//
        
    EndStateMachine

}
///makes a list of all ents that exist in the world.
void GatherEnts( nArray<nEnt*>& nodes, nRoot* r, nClass* entClass ){
    for( nRoot* node = r->GetHead(); node; node = node->GetSucc() ){
        if( node->IsA( entClass ) )
            nodes.Append( (nEnt*)node );
        GatherEnts( nodes, node, entClass );
    }
}
//-----------------------------------------------------------------------------//
void nPlannerUI::UpdateObjectsOver() {
    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();

    bool pullHook = false;
    if( this->currState.localState != BoxSelection ) {
        float bestVal = -1.0f;
        nEnt* bestEnt = 0;
        classVar( nent, entClass );

        ifvalid( camera ) {
            nRoot* r = kernelServer->Lookup("/game/ents");
            if( r ) {
                vector3 p                   = cameraPos.GetV();
                quaternion q                = cameraRot.GetQ();
                vector3 forward 	(0,0,-1);

                MouseInfo mi                = gameServer->GetMouseInfo();
                vector2 mp;
                mp.x = (1-mi.pos.x) * 2.0f - 1.0f;
                mp.x *= aspect;
                mp.y = mi.pos.y  * 2.0f - 1.0f;

                forward.set( -mp.x, -mp.y, -1.0);
                forward.norm();

                forward = q.rotate( forward );
                classVar( nent, entClass );
                nArray<nEnt*> nodes;
                GatherEnts( nodes, r, entClass );
                nEnt* node;
                foreach( node, nodes ) {
                    if( node->IsA( entClass ) ) {
                        nEnt* e = ( nEnt* ) node;
                        if( e->IsAlive() && e->GetSelectable() ) {
                            vector3 tp = e->GetVarV("pos");
                            //this->DrawPoint(tp, white);
                            vector3 dp = tp - p;
                            float dot = dp % forward;
                            dot /= dp.len();
                            float val = dot;
                            if( bestEnt == 0 || val > bestVal ) {
                                bestVal = val;
                                bestEnt = e;
                            }
                        }
                    }
                }
            }
        }
        if( bestEnt ) { //no entities at all in the world
            if( bestVal < selectionThreshold.GetF() ) {
                bestEnt = 0; //doesn't qualify
            }
        }
        nEnt* p0 = 0;
        if( objectsOver.Size() > 0 )
            p0 = objectsOver[0];
        ClearObjectsOver();
        if( bestEnt )
            objectsOver.Push() = bestEnt;
        if( objectsOver.Size() > 0 ) {
            if ( objectsOver[0] != p0 )
                pullHook = true;
        }
        else if (p0 != 0)
            pullHook = true;

    } else {
        //this->DrawPoint(this->boxSelectionPoints[0], white, .1);
        //this->DrawPoint(this->boxSelectionPoints[1],  blue, .1);
        ClearObjectsOver();
        nRoot* r = kernelServer->Lookup("/game/ents");
        if( r ) {
            vector3 p                   = cameraPos.GetV();
            quaternion q                = cameraRot.GetQ();
            vector3 forward 	(0,0,-1);
            forward                     = q.rotate(forward);
            float dot1                  = forward % (p - boxSelectionPoints[0]);
            float dot2                  = forward % (p - boxSelectionPoints[1]);
            matrix44 vrw                = camera.GetRef()->GetM();
            matrix44 vrw_inverted       = vrw;
            vrw_inverted.invert_simple();

            MouseInfo mi                = gameServer->GetMouseInfo();
            vector2& p0 = mi.posPressed[0];
            vector2& p1 = mi.pos;
            //n_printf("originalpos:%s,%s\n", str(p0), str(p1));
            p0.x = (1-p0.x) * 2.0f - 1.0f;
            p0.x *= aspect;
            p0.y = p0.y  * 2.0f - 1.0f;
            p1.x = (1-p1.x) * 2.0f - 1.0f;
            p1.x *= aspect;
            p1.y = p1.y  * 2.0f - 1.0f;
            p0 = p0 * -1.0f;
            p1 = p1 * -1.0f;
            //n_printf("manipulated:%s,%s\n", str(p0), str(p1));
            vector2 v[2];
            #define draw(p0,p1) this->DrawLine(p + q.rotate(p0), p+q.rotate(p1), white);
            draw(vector3(p0.x,p0.y,-1), vector3(p0.x,p1.y,-1));
            draw(vector3(p0.x,p0.y,-1), vector3(p1.x,p0.y,-1));
            draw(vector3(p0.x,p1.y,-1), vector3(p1.x,p1.y,-1));
            draw(vector3(p1.x,p0.y,-1), vector3(p1.x,p1.y,-1));
            #undef draw

            foreachchild(nThing, enttype, r){
                foreachchild(nEnt, ent, enttype){
                    if (ent->GetSelectable()) {
                        //n_printf("ent:%s  ", ent->fn);
                        vector3 ep = ent->GetVarV("pos");
                        ep = vrw_inverted * ep;
                        if (ep.z < 0 ){ 
                            ep.x /= -ep.z;
                            ep.y /= -ep.z;
                            ep.z /= -ep.z;
                            //n_printf("%s->%s %s %s\n", ent->fn, str(ep), str(p0), str(p1));
                            bool xbetween = ep.x >= p0.x && ep.x <= p1.x || ep.x >= p1.x && ep.x <= p0.x;
                            bool ybetween = ep.y >= p0.y && ep.y <= p1.y || ep.y >= p1.y && ep.y <= p0.y;
                            if ( xbetween && ybetween ) {
                                //this->DrawPoint( p + q.rotate(ep), white, .01);
                                objectsOver.Append(ent);
                            }
                            //else if( xbetween || ybetween )
                                //this->DrawPoint( p + q.rotate(ep), green, .01);
                            //else
                                //this->DrawPoint( p + q.rotate(ep), red, .01);
                        }
                    }
                }
            }
        }
    }
    ///Old bbox method
    /*
       float r = selectionRadius.GetF();
       vector3 p = mousePos.GetV();
       bbox3 bbox(p-vector3(r,r,r), p+vector3(r,r,r));
       nOctreeElement* elems[1024];
       int numElems = octree->CollectByBBox( bbox, (nOctElement**) elems, 1024 );
       */
    this->FindGroupMembers( objectsOver, overGroupEntities );
    if ( pullHook ) {
        //RUNSCRIPT ("Selection::updateOver");
        eventServer->pull( mouseOverEvent, "mouseOverChanged", getObjectsOverString().Get());
    }
}

/**
  called when a mouse event happens - ie a button is pressed
  */
void nPlannerUI::MouseEvent( int button ) {
    //n_printf("MouseEvent(%i)\n", button);
    bool blocked = blockedCounter > 0;
    //n_printf("blocked:%i\n", blocked);
    int i;
    int numSelected = objectsSelected.Size();
    bool* matched;
    EntArray newObjects;
    bool selectionChanged = false;
    for( i = 0; i < planners.Size(); i ++ ) {
        blocked = blocked || planners[i]->MouseEvent( button, &objectsSelected, &objectsOver, mousePos.GetV() );
        //if( blocked )
        //n_printf("%s blocked\n", planners[i]->GetName() );
    }
    int offset = 0;
    if( blocked )
        return;
    switch( button ) {
        case 0:		//left - new objects selected
            //ClearObjectsSelected();
            //n_printf("over:%s\n", this->getObjectsOverString().Get());
            //n_printf("selected:%s\n", this->getObjectsSelectedString().Get());

            matched = new bool [numSelected];
            memset(matched, 0, sizeof(bool)*numSelected);
            for( i = 0; i < objectsOver; i ++ ) {
                bool found = false;
                for( int j = 0; j < numSelected; j ++ )
                    if( objectsSelected[j] == objectsOver[i] ) {
                        found           = true;
                        matched[j]      = true;
                    }
                //n_printf("search for ent:%s == %i\n", objectsOver[i]->fn, found );
                if( !found )
                    newObjects.Append( objectsOver[i]);
            }
            for( i = 0; i < numSelected; i ++ ) {
                if( matched[i] == false ) {
                    //n_printf("unselecting: %s\n", objectsSelected[i+offset]->fn);
                    objectsSelected[i+offset]->Deselect();                             //remove it from objects selected
                    selectionChanged = true;
                    offset--;
                }
            }
            delete[] matched;
            this->multiSelect = true;
            for ( i = 0; i < newObjects; i ++ ) {
                //n_printf("selecting: %s\n", newObjects[i]->fn);
                newObjects[i]->Select();
                selectionChanged = true;
            }
            this->multiSelect = false;

            if ( selectionChanged )
                this->UpdateSelection();
            break;
        case 1:		//middle
            break;
        case 2:		//right
            //n_printf("right button event\n");
            if( objectsSelected.Size() == 0 ) {
                nMessage* m = new nMessage();    
                m->from     = ent->fn;
                m->to       = "unselected";
                m->title    = "command";
                m->v        = mousePos.GetV();
                if( game->GetButton("shift") )
                    m->string = "queue";
                else if( game->GetButton("ctrl"))
                    m->string = "push";
                printf("string:%s\n", m->string.Get() );
                Send( m );                
            }
            for( int i = 0; i < objectsSelected.Size(); i ++ ){
                if( objectsSelected[i]->Find("ai") ) {
                    nMessage* m = new nMessage();    
                    m->from     = ent->fn;
                    m->to       = objectsSelected[i]->fn;
                    m->title    = "command";
                    m->v        = mousePos.GetV();
                    if( game->GetButton("shift") )
                        m->string = "queue";
                    else if( game->GetButton("alt"))
                        m->string = "push";
                    //printf("string:%s\n", m->string.Get() );
                    if( objectsOver.Size() > 0 )
                        m->data = objectsOver[i]->fn;
                    Send( m );                
                } else {
                }
            }
            nString command;
            command.PrintF("%s", str(mousePos.GetV()));
            for( int i = 0; i < objectsOver; i ++ )
                command += " " + objectsOver[i]->FullNameString();
            //n_printf("eventServer->pull(rightClick\n");
            eventServer->pull( rightClickEvent, "rightClick", command.Get() );
            break;
    }
}
/**
  adds a particular planner to my list of planners
  those planners will be notified when something is selected / commanded
  if they return true - then they block me doing anything with that command
  */
void nPlannerUI::AddPlanner( nRoot* r ) {
    n_printf("AddPlanner( %s )\n", r->GetName() );
    classVar(nplanner, plannerclass);
    if( r->IsA( plannerclass )) {
        planners.Push() = (nPlanner*) r;
    } else {
        char fullname[1024];
        r->GetFullName( fullname, 1024 );
        n_printf("WARNING: nPlanner::AddPlanner( %s ) trying to add a non planner node\n", fullname );
    }
}

///clears the list of ents i am over
void nPlannerUI::ClearObjectsOver(){
    objectsOver.Clear();
}
void nPlannerUI::HideSelectedMenus() {
    //n_printf("HideSelectedMenus()\n");
    static nHeap<nUIElement*> 	elements(32);
    static nHeap<nRoot*> 	possib(32);
    nRoot* selectedMenusRoot = kernelServer->Lookup("/ui/menus/selected/");
    if( selectedMenusRoot ) {
        possib.add( 0, selectedMenusRoot );
        while( possib.length() > 0 ) {
            nRoot* r = possib.pop();
            classVar( nuielement, uielementClass );
            if( r->IsA( uielementClass ) )
                elements.add( 0, (nUIElement*) r );
            for( nRoot* r2 = r->GetHead(); r2; r2 = r2->GetSucc() )
                possib.add( 0, r2 );
        }
        while( elements.length() > 0 ) {
            //n_printf("%s->Hide()\n", elements.top()->fn );
            elements.pop()->Hide();
        }
    }
}
void nPlannerUI::ShowSelectedMenus(){
    nRoot* toDisplay = 0;
    nRoot* selectedMenusRoot = kernelServer->Lookup("/ui/menus/selected/");
    if( selectedMenusRoot ) {
        const char* s;
        if( objectsSelected.Size() == 0 ) {
            s = "default";
        }
        else {
            s = objectsSelected[0]->GetParent()->GetName();
        }
        toDisplay = selectedMenusRoot->Find(s);
        if( toDisplay ) {
            //n_printf("opening selected menu:%s\n", s );
            foreachchild( nUIElement, uie, toDisplay ){
                uie->Show();
            }
            ((nFSMQueue*)selectedMenusRoot)->SetState( s );
        }
    }
}

void nPlannerUI::PlaceEnt( nEnt* ent ) {
    ifnonzero (ent) {
        //n_printf("%s placing %s\n", this->fn, ent->fn );
        if( entbeingplaced.isvalid() ) {
            //printf("killing ent being placed to place new: %s\n", entbeingplaced.GetRef()->fn );
            entbeingplaced.GetRef()->Die();
        }
        entbeingplaced.SetRef( ent );
        this->SetState( EntPlacementPre );
    }
}
void nPlannerUI::AddSelectedEnt( nEnt* e ){
    objectsSelected.Append( e );
    if ( multiSelect == false ) {
        this->UpdateSelection();
    }
}
void nPlannerUI::RemoveSelectedEnt( nEnt* e ){
    for( int i = 0; i < objectsSelected; i ++ ) 
        if( objectsSelected[i] == e )
            objectsSelected.Erase( i-- );
}
void nPlannerUI::ClearSelection(){
    for( int i = 0; i < objectsSelected; i ++ )
        objectsSelected[i--]->Deselect();
}
nString nPlannerUI::getObjectsSelectedString(){
    nString answer;
    nString line;
    if (objectsSelected.Size() == 0 ) {
        answer = "";
    }
    else {
        answer.PrintF("%s", objectsSelected[0]->fn );
        for( int i = 1; i < objectsSelected; i ++ ) {
            line.PrintF(" %s", objectsSelected[i]->fn );
            answer += line;
        }
    }
    return answer;
}
nString nPlannerUI::getObjectsOverString(){
    nString answer;
    nString line;
    if (objectsOver.Size() == 0 ) {
        answer = "";
    }
    else {
        answer.PrintF("%s", objectsOver[0]->fn );
        for( int i = 1; i < objectsOver; i ++ ) {
            line.PrintF(" %s", objectsOver[i]->fn );
            answer += line;
        }
    }
    return answer;
}
//-----------------------------------------------------------------------------//
void nPlannerUI::AddSelectedEnt( const char* path ) {
    nEnt* e = (nEnt*) kernelServer->Lookup(path);
    this->AddSelectedEnt( e );
}
//-----------------------------------------------------------------------------//
void nPlannerUI::PlaceMultipleEnts( const char* paths ) {
    if( entsBeingPlaced.Size() != 0 ) {
        n_printf(" warning should finish multiple placements before starting another\n");
        entsBeingPlaced.Clear();
    }
    n_printf("nPlannerUI::PlaceMultipleEnts(%s)\n", paths);
    char* s = n_strdup(paths);
    char* path;
    while( (path = tok(s, " \n") )){
        //n_printf("path:%s\n", path);
        nEnt* e = dynamic_cast<nEnt*>(kernelServer->Lookup(path));
        n_assert(e && "trying to place a non ent");
        entsBeingPlacedInfo& i = entsBeingPlaced.Push();
        i.ent = e;
        //n_printf("i.ent:%s\n", i.ent->fn);
        i.pos.Connect( i.ent, "pos" );
        //n_printf("pos:%s\n", str(i.pos.GetV()) );
    }
    //n_printf("calculating centroid\n");
    vector3 centroid;
    for( int i = 0; i < entsBeingPlaced; i ++ )
        centroid += entsBeingPlaced[i].pos.GetV() / float(entsBeingPlaced.Size());
    //n_printf("centroid:%s\n", str(centroid) );
    for( int i = 0; i < entsBeingPlaced; i ++ )
        entsBeingPlaced[i].offset = entsBeingPlaced[i].pos.GetV() - centroid;

    this->SetState(MultipleEntPlacementPre);
}
/* -----------------------------------------------------------------------------//
 * user has trigger a box selection
 * what should we do?
 * just update our objects selected, and then trigger a box selection event
--------------------------------------------------------------------------------*/
void nPlannerUI::BoxSelect(){
    int i;
    int numSelected = objectsSelected.Size();
    this->multiSelect = true;

    bool* matched = new bool[numSelected];
    for( i = 0; i < numSelected; i ++ )
        matched[i] = false;
    for( i = 0; i < objectsOver; i ++ ) {
        bool found = false;
        for( int j = 0; j < objectsSelected; j ++ )
            if( objectsSelected[j] == objectsOver[i] ) {
                found = true;
                matched[j] = true;
            }
        //n_printf("search for ent:%s == %i\n", objectsOver[i]->fn, found );
        if( !found )
            objectsOver[i]->Select();
    }
    for( i = 0; i < numSelected; i ++ ) {
        if( matched[i] == false ) {
            //n_printf("unselecting: %s\n", objectsSelected[i]->fn);
            objectsSelected[i]->Deselect();                             //remove it from objects selected
            i--;                                                        //make sure we dont go past our list of selected objects into newly selected things
            numSelected--;
        }
    }
    delete[] matched;

    for( i = 0; i < planners; i ++ ) 
        planners[i]->SelectionEvent( &objectsSelected );
    this->HideSelectedMenus();
    this->ShowSelectedMenus();
    eventServer->pull( selectionBoxEvent, "selectionChangedBox", getObjectsSelectedString().Get() );
    this->multiSelect = false;
}
//-----------------------------------------------------------------------------//
bool nPlannerUI::Attach(nSceneGraph2* sceneGraph){
    //n_printf("nPlannerUI::Attach()\n");
    AttachRings(objectsOver,            overVis,                                sceneGraph);
    AttachRings(objectsSelected,        selectedVis,                            sceneGraph);
    AttachRings(overGroupEntities,      overGroupVis,                           sceneGraph);
    AttachRings(selectedGroupEntities,  selectedGroupVis,                       sceneGraph);
    AttachRings(catSelectedEntities,    catSelectedVis,                         sceneGraph);
    AttachRings(targettedEntities,      targettedVis,                           sceneGraph);
    return true;
}
//-----------------------------------------------------------------------------//
void nPlannerUI::AttachRings(const EntArray& ents, nAutoRef<n3DNode>& node, nSceneGraph2* sceneGraph ){
    nEnt* e;
    float r;
    if( node.isvalid() == false && roll(.1)) {
        once(n_printf("node:%s is invalid\n", node.getname()));
    }
    ifvalid(node)
    foreach( e, ents ) {
        vector3 p = e->GetVarV("pos");
        //n_printf("attaching %s to :%s @%s\n", node.getname(), e->fn, str(p));
        if ( e->HasVar("selectionCircleRadius"))
            r = e->GetVarF("selectionCircleRadius");
        else
            r = 5.0f;
        node->Txyz(p.x, p.y + 1, p.z);
        node->Sxyz(r, r, r);
        node->Attach( sceneGraph );
    }
}
//-----------------------------------------------------------------------------//
void nPlannerUI::FindGroupMembers(EntArray& ents, EntArray& groupMembers){
    groupMembers.Clear(); 
    nEnt* e;
    nEnt* group;
    //calculate which entities are in groups with entities i am over
    EntArray groups;
    foreach( e, ents){
        if (e->HasVar("groupable") && e->GetVarI("groupable") == 1) {
            if (e->HasVar("group") == false ){
                //n_printf("%s has no group\n", e->fn);
                static void* eventPtr = 0;
                eventServer->pull(eventPtr, "unitWithoutGroup", e->fn);
            } else {
                const char* groupPath;
                groupPath = e->GetVarRef("group");
                n_assert( groupPath );
                group = (nEnt*) kernelServer->Lookup(groupPath);
                n_assert( group );
                if ( groups.Find(group) == 0 ) {
                    groups.Append(group);
                }
            }
        }
    }
    foreach( group, groups) {
        foreachchild(nEnt, member, group){
            const char* path = member->GetVarS("path");
            e = (nEnt*) kernelServer->Lookup(path);
            if( ents.Find(e) == 0 ){
                groupMembers.Append(e);
            } 
        }
    }
}
//-----------------------------------------------------------------------------//
void nPlannerUI::AddCatSelectedEnt(const char* s ) {
    nRoot* r = kernelServer->Lookup(s);
    if ( r == 0 ) {
        n_printf("trying to catSelect non existent ent:%s\n", s);
        return;
    }
    classVar(nent, entClass);
    n_assert( r->IsA( entClass) );
    this->catSelectedEntities.Append( (nEnt*) r);
}
//-----------------------------------------------------------------------------//
void nPlannerUI::AddTarget(const char* s ) {
    nRoot* r = kernelServer->Lookup(s);
    if ( r == 0 ) {
        n_printf("trying to target non existent ent:%s\n", s);
        return;
    }
    classVar(nent, entClass);
    n_assert( r->IsA( entClass) );
    this->targettedEntities.Append( (nEnt*) r);
}
//-----------------------------------------------------------------------------//
void nPlannerUI::UpdateSelection(){
    for( int i = 0; i < planners; i ++ ) 
        planners[i]->SelectionEvent( &objectsSelected );
    this->HideSelectedMenus();
    this->ShowSelectedMenus();
    //RUNSCRIPT ("Selection::updateSelected");
    eventServer->pull( selectionEvent, "selectionChanged", getObjectsSelectedString().Get() );
    this->FindGroupMembers( objectsSelected, selectedGroupEntities );
}
