#define N_IMPLEMENTS nTextDisplayer
//------------------------------------------------------------------------------
//  (C) 2005	David Leistiko
//------------------------------------------------------------------------------
#include "ui/ntextdisplayer.h"
#include "node/n3dnode.h"
#include "node/nshadernode.h"
#include "gfx/nscenegraph2.h"

nNebulaScriptClass(nTextDisplayer, "n3dnode");

//------------------------------------------------------------------------------
/**
*/
nTextDisplayer::nTextDisplayer()
{
    text = "+-$%&{}()";
    alignment = ALIGN_LEFT;
    alpha = 1.0f;
}

//constructor 
void nTextDisplayer::Initialize(){
    n3DNode::Initialize();
    this->updateMatrix();
    offset.set( 0,0,-10 );
}

nTextDisplayer::~nTextDisplayer(){
}
//------------------------------------------------------------------------------
/**
*/

float* buildTable(){
    float* table = new float[256];
    for( int i = 0; i < 256; i ++ )
        table[i] = 0.55f;
    table['1'] 	= .375;
    table['i']  = .25;
    table['l']  = .375;
    table['m']  = .75;
    table['w']  = .75;
    table['.'] 	= .25;
    table[','] 	= .25;
    table[';'] 	= .25;
    table['!'] 	= .25;
    table['\"'] = .375;
    table['+'] 	= .625;
    table['-']  = .5;
    table['$']  = .5;
    table['%']  = .75;
    table['&']  = .625;
    table['{']  = .375;
    table['(']  = .375;
    table['[']  = .375;
    table['}']  = .375;
    table[')']  = .375;
    table[']']  = .375;
    return table;
}
bool nTextDisplayer::Attach(nSceneGraph2* sceneGraph) {
    kernelServer->PushCwd( kernelServer->Lookup("/vis/fonts/lagoonfont") );
    static float* offsetTable = buildTable();
    const char* s = text.Get();
    char c[2];
    c[1] = '\0';
    float x = 0;
    if (sceneGraph->BeginNode())
    {
        float width;
        if (alignment != ALIGN_LEFT ){
            for( int i = 0; i < text.Length(); i ++ ) {
                c[0] = s[i];
                c[0] = tolower(c[0]);
                width += offsetTable[c[0]];
            }
        }
        switch(alignment) {
            case ALIGN_LEFT:
                x = 0.0f;
                break;
            case ALIGN_CENTER:
                x = width * -.5f;
                break;
            case ALIGN_RIGHT:
                x = -width;
                break;
        }
        //sceneGraph->AttachTransformNode(this);
        n3DNode* v;
        for( int i = 0; i < text.Length(); i ++ ) {
            c[0] = s[i];
            c[0] = tolower(c[0]);
            switch( c[0] ) {
                case '!':
                    v = (n3DNode*) kernelServer->Lookup("exclamation");
                    break;
                case ' ':
                    v = (n3DNode*) kernelServer->Lookup("blank");
                    break;
                case '.':
                    v = (n3DNode*) kernelServer->Lookup("period");
                    break;
                case '(':
                case '{':
                case '[':
                    v = (n3DNode*) kernelServer->Lookup("leftbracket");
                    break;
                case ')':
                case '}':
                case ']':
                    v = (n3DNode*) kernelServer->Lookup("rightbracket");
                    break;
                case '&':
                    v = (n3DNode*) kernelServer->Lookup("ampersand");
                    break;
                case ',':
                    v = (n3DNode*) kernelServer->Lookup("comma");
                    break;
                case ':':
                    v = (n3DNode*) kernelServer->Lookup("colon");
                    break;
                case '$':
                    v = (n3DNode*) kernelServer->Lookup("dollar");
                    break;
                case '-':
                    v = (n3DNode*) kernelServer->Lookup("minus");
                    break;
                case '%':
                    v = (n3DNode*) kernelServer->Lookup("percent");
                    break;
                case '+':
                    v = (n3DNode*) kernelServer->Lookup("plus");
                    break;
                case '?':
                    v = (n3DNode*) kernelServer->Lookup("question");
                    break;
                case '`':
                case '\"':
                case '/':
                        v = (n3DNode*) kernelServer->Lookup("quote");
                    break;
                case ';':
                    v = (n3DNode*) kernelServer->Lookup("semicolon");
                    break;
                default:
                    v = (n3DNode*) kernelServer->Lookup(c);
                    break;
            }
            if(v) {
                v->Txyz( offset.x + x, offset.y-.5f, offset.z );
                //n_printf("offset:%s x: %f\n", str(offset), x);
                // attach self to scene
                // attach children to scene
                //nShaderNode* shader = (nShaderNode*) v->Find("shader");
                //shader->SetConst(0, vector4(0,0,0,this->alpha));
                //nChannelSet* channels = new nChannelSet;
                //v->AttachChannels(channels);
                v->Attach(sceneGraph);
            }
            else
                n_printf("cannot find letter:%s\n", c );
            n_assert( c[0] >= 0 && c[0] < 256 );
            x += offsetTable[c[0]];
        }
    }
    sceneGraph->EndNode();
    kernelServer->PopCwd();
    return n3DNode::Attach( sceneGraph );

}
