#holds stuff for the regular ga
#lets setup a nice gadisplayer
#it should display a table with all the population info
#right now it is just the table of individuals in the population

namespace eval GA {
    variable initialized "false"
    variable ga
    variable opened no
    variable min -1
    variable max -1
    variable avg -1
    variable maxOverAvg 0
    variable addchildcounter 0
    variable updating yes
    variable updateFreq
    variable var
#****************************************************************************************************************************************************************************************
    proc log { txt } {
        logto "all ga" $txt
    }
#****************************************************************************************************************************************************************************************
    proc init { } {
        global pathfinder
        variable initialized
        variable ga
        if { $initialized == true } { return } else { set initialized true }
        pushsel
        Pathfinder::init
        sn nent /ga
            set ga [new nga ga]
            GA::parseParams
        sel ..
        popsel

        after 1000 { GA::updateInfo}
    }
#****************************************************************************************************************************************************************************************
    proc parseParams { } {
        global xmldir
        XML::parse [file_to_string $xmldir/misc/ga.xml]
        while { [XML::next] } {
            global ::XML::type ::XML::tag ::XML::txt
            switch $type {
	open { 
	    switch $tag {
	        popsize 	{ ci popsize $txt }
	        numgens 	{ ci numgens $txt }
	        mutationprob  	{ cf mutationprob $txt }
	        crossoverprob 	{ cf crossoverprob $txt }
	        injectionamount	{ ci injectionamount $txt }
	        injectionfreq	{ ci injectionfreq $txt }
	        maintenancethreshold	{ ci maintenancethreshold $txt }
	        convergencethreshold	{ cf convergencethreshold $txt }
	    }
	}	
            }
        }
    }
#****************************************************************************************************************************************************************************************
    proc clear { } {
        delete /ga
    }
#****************************************************************************************************************************************************************************************
#plan against the current situation - locally
    proc run { } {
        variable ga
        set situation [Plan::from_gamestate]
        XMLWindow .situation "Situation" $situation
        Scenario::clear
        Scenario::load $situation
        Pathfinder::update
        $ga.setsituation $situation
        $ga.run     
    }
#****************************************************************************************************************************************************************************************
    proc replan { } {
        puts "GA::replan"
        global gamestate rank
        set gamestate [Plan::from_gamestate]
        if { $rank == 0 } {
            Plan::clear
        }
        Scenario::clear
        Scenario::load $gamestate
        /ga/ga.run $gamestate loadNewPlan
    }
#****************************************************************************************************************************************************************************************
    #called when the replan ga is done
    proc loadNewPlan { } {
        global gamestate
        Plan::clear
        Plan::load $gamestate
        Plan::load [/ga/ga.getbestplan]
    }
#****************************************************************************************************************************************************************************************
    #saves the best ga plan out to a file
    proc saveBest { } {
        global xmldir
        set filename $xmldir/scenario/bestplan.xml
        set best [/ga/ga.getbestplan]
        set file [open $filename "w"]
        close $file
    }
#****************************************************************************************************************************************************************************************
    proc TableEntry { p row col } {
#puts "tableentry $p $row $col"
        switch $col {
            rank	{ entry $p -textvariable GA::rank($row) -width 3			}
            evld 	{ checkbutton $p -variable GA::evaluated($row) -onvalue true -offvalue false 	}
            being 	{ checkbutton $p -variable GA::beingevaluated($row) -onvalue true -offvalue false }
            gen 	{ entry $p -textvariable GA::genmodified($row) -width 3		}
            fitness	{ entry $p -textvariable GA::fitness($row)	-width 10		}
            sv		{ entry $p -textvariable GA::sv($row)	-width 10		}
            bitstring	{ entry $p -textvariable GA::bitstring($row) -width 50		}
            rc		{ entry $p -textvariable GA::rc($row)	-width 8		}
            {injected amount}	{ entry $p -textvariable GA::injectedamount($row) -width 10		}
            default 	{ entry $p -textvariable GA::var($row.$col) -width 10		}
        }
    }
    proc UpdateStats { } {
#puts "GA::UpdateStats"
        set stats [/ga/ga.getstats]
        variable min [lindex $stats 0]
        variable max [lindex $stats 1]
        variable avg [lindex $stats 2]
        if { $avg == 0 } { set avg $max }
        variable maxOverAvg [expr $max / $avg]
    }
#/-----------------------------------------------------------------------------//
    proc clearMission { } {
        Plan::clear
        return
    }
    proc open { } {
        #okay so we want a table with population info for now
        #to do this we need a way to fetch pop info from the ga
        variable opened true
        variable genomes ""
        pushsel
        sel /ga/ga
        TL ga
        foreach genome [dir] {
            lappend genomes $genome
        }
        set rows $genomes
        set cols "rank evld being gen fitness sv bitstring rc {injected amount}"
        set cols "rank evld being gen fitness sv bitstring"
        sel genome0
        foreach child [dir] { lappend cols $child }
        pushf [frame $f.status]
            pack $f -expand yes -fill both -anchor w
            gc 0 0 glabel -text "Population Size:"
            gc 0 1 gentry -textvariable GA::popsize -width 5
            gc 0 2 glabel -text "Num Generations:"
            gc 0 3 gentry -textvariable GA::numgens -width 5
            gc 1 0 glabel -text "Current Generation:"
            gc 1 1 gentry -textvariable GA::generation -width 5
            gc 1 2 glabel -text "Num Evaluations:" 
            gc 1 3 gentry -textvariable GA::evals -width 5
            gc 2 0 glabel -text "Num Boots: "
            gc 2 1 gentry -textvariable GA::numboots -width 5
            gc 2 2 glabel -text "Max Boots: "
            gc 2 3 gentry -textvariable GA::maxboots -width 5
            gc 0 4 glabel -text "Cache Hits:"
            gc 0 5 gentry -textvariable GA::cachehits -width 5
            
        popf
        Table $f.table $rows $cols GA::TableEntry
        pushf [frame $f.stats]
            pack $f -expand yes -fill x -anchor w
            entry $f.min -textvariable GA::min
            entry $f.max -textvariable GA::max
            entry $f.avg -textvariable GA::avg
            label $f.lbl -text "max / avg: "
            entry $f.moa -textvariable GA::maxOverAvg
            pack $f.min $f.max $f.avg $f.lbl $f.moa -side left
        popf
        popsel
    }
    proc update { } {
        variable opened
        if { $opened } {
            open
        }
    }
    proc updateInfo { } {
        if {[catch {
            variable popsize [/ga.gi popsize]
            variable numgens [/ga.gi numgens]
            variable generation [/ga.gi generation]
            if { [/ga.hasvar numboots] } {
	variable numboots [/ga.gi numboots]
            }
            if { [/ga.hasvar maxboots] } {
	variable maxboots [/ga.gi maxboots]
            }

            variable evals [/ga.gi evals]
            variable cachehits [/ga.gi cachehits]
     
            variable rank
            variable evaluated
            variable beingevaluated
            variable genmodified
            variable fitness
            variable sv
            variable bitstring
            variable rc
            variable injectedamount
        }] } { puts "error:$err" }
        pushsel
        variable var
        if {[catch {
            sel /ga/ga
            foreach ga [dir] {
	sel $ga
	set rank($ga)		[.getrank]
	set evaluated($ga)		[.isevaluated]
	set beingevaluated($ga)	[.getevaluationstarted]
	set genmodified($ga)	[.getgenmodified]
	set fitness($ga)		[format %.4f [.getfitness]]
	set sv($ga)		[format %.4f [.getselectionvalue]]
	set bitstring($ga)		[.getbitstringstring]
#set rc($ga)		[format %.4f [rc.getf]]
	set injectedamount($ga)	[format %.4f [.gettotalinjectedamount]]
	foreach genome [dir] { set var($ga.$genome) [$genome.getval] }
	sel ..
            }
        } err ] } { puts "error:$err" }
        popsel
        variable updating
        if { $updating } { after 1000 GA::updateInfo }
    }

    proc load { sit } { 
        Scenario::clear
        Scenario::load "$sit"
        Plan::load "$sit" yes
        Pathfinder::update
    }
# ga has finished running - right now dump some log info..append it somewhee
    proc doneRunning { } {
        puts "DONE RUNNING"
        XML::clear
        XML::open GARUN
            XML::wrapDate
            XML::wrap EVALS 		[/ga.gi evals]
            XML::wrap GENERATION 	[/ga.gi generation]
            XML::wrap CACHEHITS		[/ga.gi cachehits]
            XML::open GAPARAMS
	XML::wrap TIMETAKEN	[/ga.gf timetaken]
	XML::wrap POPSIZE	[/ga.gi popsize]
	XML::wrap NUMGENS	[/ga.gi numgens]
	XML::wrap NUMINJECTED 	[/ga.gi numinjected]
	XML::wrap INJECTIONAMOUNT 	[/ga.gi injectionamount]
	XML::wrap INJECTIONFREQ	[/ga.gi injectionfreq]
	XML::wrap MAINTENANCETHRESHOLD	[/ga.gi maintenancethreshold]
	XML::wrap CONVERGENCETHRESHOLD	[/ga.gf convergencethreshold]
            XML::close GAPARAMS
            pushsel
            sel /ga/ga
            foreach ga [dir] {
	set rank [$ga.getrank]
	if { $rank == 0 } {
	    XML::wrap FITNESS	[$ga.getfitness]
	    XML::wrap SELECTIONVALUE	[$ga.getselectionvalue]
	    XML::wrap BITSTRING 	[$ga.getbitstringstring]
	    XML::wrap RANK	[$ga.getrank]
	    XML::wrap GENMODIFIED	[$ga.getgenmodified]
	    XML::wrap INJECTEDAMOUNT	[$ga.getinjectedamount]
            	    catch {XML::wrap CASEBASESIZE [llength [/casebase.getchildren]]}
	}
            }
            popsel
        XML::close GARUN
        global xmldir
        global socket
        $socket.send "garun" $::XML::lines
        appendToFile $xmldir/misc/gainfo.xml $::XML::lines
        pushsel
        sel /ga/ga
        puts "about to clear population"
        foreach genome [dir] { $genome.die }
        puts "done clearing population"
        popsel
        return
    }
}
