#***********************************************************************************************
#   HOLDS ALL THE DATA ON THE MAP
#   the size of the map - lat/lng info
#   current texture - heightmap stuff
#   any future information - such as scaling issues
#***********************************************************************************************
namespace eval Map {
    variable lat_top 0 
    variable lat_bottom 0 
    variable lng_left 0 
    variable lng_right 0 
    variable map_width 100 
    variable map_height 100
    
    variable heightmap  flat.bmp
    variable worldsize  1000
    variable maxheight  20
    variable terrain_texture  renotahoe_256.bmp
    variable terrain_detail   detail.bmp
    variable skybox	"none"
    variable detailsize 15.0
    variable error 5
    variable envmap false
    variable movedetail false

    proc update { } {
        variable heightmap
        variable worldsize
        variable maxheight
        variable terrain_texture
        variable terrain_detail

        catch { delete /game/vis/sky 	}
        catch { delete /game/vis/terrain 	}
        catch { delete /map 		}
        gen_terrain $heightmap $worldsize $maxheight $terrain_texture $terrain_detail false
        gen_skybox bluesky
    }

#***********************************************************************************************
#   converts a latitude into a z coordinate in world space 
#***********************************************************************************************
    proc lat_to_z { lat } {
        variable lat_top 
        variable lat_bottom
        variable map_height
        set lat_range [expr $lat_top - $lat_bottom]
        if { $lat_range == 0 } { puts "warning lat_range == 0" }
        set answer [expr (($lat - $lat_bottom) / $lat_range ) * $map_height ]
#puts [format "lat_to_z %8.2f -> %8.2f (%8.2f - %8.2f: %8.2f)" $lat $answer $lat_bottom $lat_top $lat_range]
        return $answer
    }
#***********************************************************************************************
#   converts a longitude into a x coordinate in world space
#***********************************************************************************************
    proc lng_to_x { lng } {
        variable lng_right 
        variable lng_left 
        variable map_width
        set lng_range [expr $lng_right - $lng_left]
        if { $lng_range == 0 } { puts "warning lng_range == 0" }
        set answer [expr (($lng - $lng_left) / $lng_range ) * $map_width ]
#puts [format "lng_to_x %8.2f -> %8.2f (%8.2f - %8.2f: %8.2f)" $lng $answer $lng_left $lng_right $lng_range]
        return $answer
    }
    proc gen_mapnode { } {
        variable heightmap
        variable gridinterval
        variable maxheight
        if { [catch { delete /map } err] } { puts $err }
        sn nmap /map
            .setheightmap lib/heightmaps/$heightmap
            setgridinterval $gridinterval
            setheightrange .1 $maxheight
        sel ..
    }
    proc gen_terrainnode { } {
        variable terrain_texture
        variable terrain_detail
        variable detailsize
        vars error envmap movedetail
        if { [catch { delete /game/vis/terrain } err] } { puts $err }
        sn n3dnode /game/vis/terrain
            txyz 0 0 0
            pushsel
            sel /game/vis/terrain
            set time [/sys/servers/channel.getchannel1f time]
#sn nipol sxyz
#	.connect "sxyz"
#	.addkey3f 0 0 0 0
#	.addkey3f [expr $time + 0] 1.0 0.1 1.0
#	.addkey3f [expr $time + .5] 1.0 0.1 1.0
#	.addkey3f [expr $time + 1] 1.0 1.0 1.0
#	.addkey3f 1000000000000 1.0 1.0 1.0
#            sel ..
            popsel

            sn nmapnode map
	.setmap "/map"
	.setblocksize 17
	.setdetailsize $detailsize
	.seterror $error
            sel ..
            sn ntexarraynode tex
	.settexture 0 lib/terrain/$terrain_texture none
	.settexture 1 lib/terrain/$terrain_detail none
	if { $envmap } {
	    .settexture 2 lib/tex/env.bmp none
	}
            sel ..
            sn nshadernode sn
	setrenderpri -100
	if { $envmap } {
	    setnumstages 3
	} else {
	    setnumstages 2
	}
	setcolorop 0 "mul tex prev"
	setcolorop 1 "mul tex prev"
	setconst0 0 0 0 .25
	.begintunit 0
	    .settexcoordsrc uv0
	.endtunit
	.begintunit 1
	    .setaddress wrap wrap
	    .settexcoordsrc uv1
	    .setenabletransform true
	.endtunit
	if { $envmap } {
    	    setcolorop 2 "ipol tex prev const.a"
    	    .begintunit 2
    	        .settexcoordsrc spheremap
    	    .endtunit
	}
	if { $movedetail } {
	    sn nipol txyz1
	        .connect txyz1
	        .addkey3f 0 0 0 0
	        .addkey3f 10 2 1 0
            	    sel ..
	}
	setlightenable true
	setdiffuse  1.0 1.0 1.0 1
	setambient  1.0 1.0 1.0 1
	setemissive 0.0 0.0 0.0 1.0
	setzfunc "less"
            sel ..
        sel ..
    }
    proc gen_basenode { } {
        if { [catch { delete /game/vis/base } err] } { puts $err }
        sn n3dnode /game/vis/base
            gen_mesh lib/meshes/plane.n3d
            gen_tex lib/terrain/$terrain
            gen_shader simple
            sxyz 10000 10000 10000
            shader.setrenderpri -20
            shader.setzwriteenable "false"
        sel ..
    }
    proc gen { } {
        variable worldsize
        variable gridinterval [expr $worldsize/369.0]
        variable genbase "false"
        pushsel
            set gridinterval   [expr $worldsize / 369.0]
            gen_mapnode 
            gen_terrainnode 
            if { $genbase } { gen_basenode }
        popsel
        vars skybox
        if { $skybox != "none" } {
            gen_skybox
        }
    }
    proc point { x y } {
        return "$x [/map.getheight $x $y] $y"
    }
}

proc Map::gen_skybox { } {
    variable skybox
    pushsel
    proc gen_side { name tx ty tz rx ry rz } { 
        upvar d d
        sn n3dnode $name
            txyz $tx $ty $tz
            rxyz $rx $ry $rz
            gen_mesh lib/meshes/plane.n3d
            gen_tex $d/$name.bmp
            sn nshadernode sn
	setrenderpri -500
	setzwriteenable false
	setzfunc always
	setnumstages 1
	setcolorop 0 "replace tex prev"
	setcullmode "cw"
            sel ..
        sel ..
    }
    set d "lib/tex/$skybox"
    catch { delete /game/vis/sky } err
#if { [/game/vis.find "sky"]!="null" } {
#           puts "erasing existing skybox"
#           delete /game/vis/sky
#       }
    sn n3dnode /game/vis/sky
        .sxyz 5 5 5
        setlockviewer true
        gen_side xp  0  0  1  90   0   0
        gen_side zp  1  0  0  90  90   0
        gen_side xn  0  0 -1  90 180   0
        gen_side zn -1  0  0  90 270   0
        gen_side yp  0  1  0   0   0   0
        gen_side yn  0 -1  0 180   0   0
        sn nipol sy
            .connect sy
            .setchannel camera_alt
            addkey1f -1000  10.0
            addkey1f -10    10.0
            addkey1f 1000   1.0
            addkey1f 100000 1.0
        sel ..
    sel ..
    popsel
}

#goes through all ents,  identifying any that are outside the world
proc Map::checkBounds { { path "/game/ents" } } { 
    variable worldsize
    pushsel
    sel $path
    if { [.isa nent] } {
        set p [.gv pos]
        set x [lindex $p 0]
        set y [lindex $p 1]
        set z [lindex $p 2]
        set inside false
        if { $x > 0 && $x < $worldsize } {
            if { $y > 0 && $y < $worldsize } {
        	if { $z > 0 && $z < $worldsize } {
	    set inside true
	}
            }
        }
        if { $inside == "false" } {
            puts [format "%-30s: %6.2f,%6.2f,%6.2f !! %6s" [.getfullname] $x $y $z $worldsize]
        }
    }
    foreach node [dir] {
        checkBounds $node
    }
    popsel
}
