#XML PARSER
namespace eval XML {
#PRIVATE
    variable xml "/sys/servers/xml"
    variable lines
#PUBLIC
    namespace export type tag next parse next 
    variable type ""
    variable tag ""
    variable txt ""
    variable indent
    variable parent
    variable mode "c"
    variable holdNext 0
    proc parse { str } {
        variable xml
        variable mode
        switch $mode {
            tcl {
	variable lines
	set lines ""
	set lines $str
            }
            c {
	$xml.parsexml $str
            }
        }
    }
    proc next { } {
        variable xml
        variable mode
        variable holdNext
        if { $holdNext > 0 } {
            incr holdNext -1
            return true
        }
        switch $mode {
            xml {
	variable lines 
	variable type 
	variable tag 
	variable txt
	if { $lines == "" } { return false }
	set lines [string trim $lines]
	set type ""
	set tag ""
	set txt ""
	if {       [regexp {^(</)([^>]+)([ >]+)(.*)} $lines sect openBrace tag closeBrace rest] } { 
	    set type "close"
	} elseif { [regexp {^(<)([^>]+)([ >]+)([^<]*)(.*)} $lines sect openBrace tag closeBrace txt rest] } { 
	    set type "open"
	} else {
	    return false
	}
	set lines $rest
	set tag [string tolower [string trim $tag]]
	set txt [string tolower [string trim $txt]]
	return true
            }
            c {
	if {[$xml.next]} {
	    variable type [$xml.gettype]
	    variable tag  [$xml.gettag]
	    variable txt  [$xml.gettxt]
		#puts "type:$type tag:$tag txt:$txt"
	    return true
	}
	return false
            }
        }
    }
    proc clear { } {
        variable lines "" 
        variable indent 0
    }
    proc open { tag { txt "" } } {
        variable lines
        variable indent
        set prefix ""
        for { set i 0 } { $i < $indent } { incr i 1 } { append prefix "  " }
        append lines "$prefix<$tag>$txt\n"
        incr indent 1
    }
    proc close { tag } {
        variable lines
        variable indent
        incr indent -1
        set prefix ""
        for { set i 0 } { $i < $indent } { incr i 1 } { append prefix "  " }
        append lines "$prefix</$tag>\n"
    }
    proc wrap { tag { txt "" } } {
        variable lines
        variable indent
        set prefix ""
        for { set i 0 } { $i < $indent } { incr i 1 } { append prefix "  " }
        append lines "$prefix<$tag>$txt</$tag>\n"
    }
    proc getlines { } { 
        variable lines
        return $lines
    }
#what we want
#match the first index easily
#match the second index - so long as it starts with a { }
    proc cleanup { stringName {delimeter "\n"} } {
        upvar $stringName str 
        set lines [split $str $delimeter]
        set str ""
        foreach line $lines {
            set line [string trim $line]
            if { $line != "" } {
	if { [llength $lines] > 1 } {
	    append str $line$delimeter
	} else {
	    append str $line
	}
            }
        }
        set str [string trim $str $delimeter]
    }
#construct a parser - the body of a function
    proc buildParser { str } {
        set cases ""
#set code "\{\n"
        set expBoth {
            ^			#start of string
            \W*			#empty space before tag
            (\w+)		#tag
            \W*			#space after tag
            {
            ([^\}]+)		#open
            }
            \W*
            {
            ([^\}]+)		#close
            }
            \W*
            (.*)
        }
        set expOpen {
            ^
            \W*
            (\w+)
            \W*
            {
            ([^\}]+ 
             | 
            {
	[^\{]+
	{
	    [^\}]+
	}
	[^\}]+
            })
            }
            (.*)
        }
        set expTag {
            ^
            \W*
            (\w+)
            \W*
            (.*)
        }
        while { $str != "" } {
            set all ""
            set tag ""
            set open ""
            set close ""

            if { [regexp -expanded $expBoth $str all tag open close rest] } {
#puts "found both |$tag| |$open| |$close|"
	set str $rest
            } elseif { [regexp -expanded $expOpen $str all tag open rest] } {
#	puts "found open |$tag| |$open|"
	set str $rest
            }  elseif { [regexp -expanded $expTag $str all tag rest] } {
#	puts "found tag: |$tag|"
	set str $rest
            } else {
#	puts "couldn'tmatch : $str"
	set str ""
            }
            set tag 			[string trim $tag]
            if { $tag == "" } { continue }

            sub open "\n" ";"
            cleanup open ";"

            sub close "\n" ";"
            cleanup close ";"

            if { $tag == "init" } {
	append code "    $open"
            } else {
	lappend cases "$tag {$open} {$close}"
            }
#puts [format "%-60s %-60s %-60s" $tag $open $close]
        }

        append code "\n"
        append code "    XML::parse \$plan\n"
        append code "    while \{ \[XML::next\] \} \{\n"
        append code "        global ::XML::type ::XML::tag ::XML::txt\n"
        append code "        switch \$type \{\n"
        append code "            open  \{\n"
        append code "                switch \$tag \{"
        foreach case $cases {
#puts "case: $case"
            set tag   [lindex $case 0]
            set open  [lindex $case 1]
            set close [lindex $case 2]
            set line [format "\n                       %-40s \{%-80s\}" $tag $open]
#puts $line
            append code $line
        }
        append code "\n               \}"
        append code "\n            \}\n            close \{"
        append code "\n                switch \$tag \{"
        foreach case $cases {
            set tag   [lindex $case 0]
            set open  [lindex $case 1]
            set close [lindex $case 2]
            append code [format "\n                      %-40s \{%-80s\}" $tag $close]
        }
        append code "\n               \}"
        append code "\n           \}"
        append code "\n        \}"
        append code "\n    \}"
        return $code
    }
    proc wrapDate { } {
        set date [clock format [clock seconds] -format "%Y %m %d %H %M %S"]
        open DATE
            wrap YEAR 	[lindex $date 0]
            wrap MONTH 	[lindex $date 1]
            wrap DAY 	[lindex $date 2]
            wrap HOUR 	[lindex $date 3]
            wrap MINUTE 	[lindex $date 4]
            wrap SECOND 	[lindex $date 5]
        close DATE
    }
    proc continueUntilClose { closeTag } {
        while { [XML::next] } {
            global ::XML::type ::XML::tag ::XML::txt
            if { $type == "close" && $tag == $closeTag } {
	variable holdNext
	incr holdNext
	return
            }
        }
    }
}
