namespace eval HullDisplay { 
    variable file "set1/d"
    variable creationNum 0
    variable dir "set1"
    variable numkeys 5
    variable slices
    variable maxSlices 34
    variable minSlice 1
    variable maxSlice 34
    variable animating true
    variable animationSpeed 500
    variable frameRange 1
#****************************************************************************************************************************************************************************************
    proc open { } {
        variable maxSlices
        set tl [toplevel .hd]
        wm title $tl "Hull Display"
        wm resizable $tl 1 1
        pushf [frame $tl.f -relief sunken -borderwidth 1]
            pack $f
            entry $f.file -textvariable ::HullDisplay::file
            button $f.load -text "Load Single Gesture" -command { HullDisplay::loadSingle }
            entry $f.dir -textvariable ::HullDisplay::dir
            button $f.loadDir -text "Load Directory of Gestures" -command { HullDisplay::loadDir }
            button $f.saveKeys -text "Save Keys" -command { HullDisplay::saveKeys }
            button $f.loadKeys -text "Load Keys" -command { HullDisplay::loadKeys }
            pack $f.file $f.load $f.dir $f.loadDir

            scale $f.minSlice\
            -length 200\
            -orient horizontal\
            -from 1 -to 34\
            -resolution 1\
            -showvalue 1\
            -label "Minimum Slice:"\
            -variable ::HullDisplay::minSlice\
            -command { HullDisplay::setMinSlice }

            scale $f.maxSlice\
            -length 200\
            -orient horizontal\
            -from 1 -to 34\
            -resolution 1\
            -showvalue 1\
            -label "Maximum Slice:"\
            -variable ::HullDisplay::maxSlice\
            -command { HullDisplay::setMaxSlice }

            scale $f.animationSpeed\
            -length 200\
            -orient horizontal\
            -from 1000 -to 10\
            -resolution 10\
            -showvalue 1\
            -label "Animation Speed - ms between frames:"\
            -variable ::HullDisplay::animationSpeed

            scale $f.frameRange\
            -length 200\
            -orient horizontal\
            -from 1 -to 34\
            -resolution 1\
            -showvalue 1\
            -label "Slice Range"\
            -variable ::HullDisplay::frameRange

            pack $f.minSlice $f.maxSlice $f.animationSpeed $f.frameRange
            pushf [frame $f.slices]
    	pack $f
	for { set i 0 } { $i < $maxSlices } { incr i } { 
    	    checkbutton $f.slice$i		\
    	        -onvalue    true		\
    	        -offvalue   false		\
    	        -variable   ::HullDisplay::slices($i)\
    	        -command    { HullDisplay::UpdateSlices }
    	    grid config $f.slice$i -column [expr int(fmod($i,17))] -row [expr $i / 17]
	}
            popf
            loadKeys
            initColors
            initSlices
            update
            ke_create $f
            pack $f.saveKeys $f.loadKeys
        popf
#tick
    }
#****************************************************************************************************************************************************************************************
    proc loadSingle { } {
        variable file
        variable creationNum
        global data
#idea, read in the hull file,  create cubes for each 1
        set pwd [pwd]
        pushsel
#cd ../../ttt/slices/
        cd ../../$file/slices
        set slices [glob *.n3d]
        cd $pwd
        puts "slices:$slices"
        sel /game/vis/
        set value 1
        set numSlices 0
        foreach slice $slices {
            if { $numSlices > 500 } { break }
            incr numSlices
            puts "loading slice : $slice"
            set cube [gen_ent cube "$creationNum[lindex [split $slice .] 0]"]
            $cube.svx pos [expr $creationNum * 30]
            set filename ../../$file/slices/$slice
            puts "loading file:$filename"
            [$cube.getvis]/mesh.setfilename $filename
            $cube.sf value [expr $value / 34.0]
            incr value
        }
        popsel
        incr creationNum
        update
    }
#****************************************************************************************************************************************************************************************
    proc loadDir { } {
        variable file
        variable dir
        set pwd [pwd]
        cd ../../$dir
        puts "loading dirs in directory : [pwd]"
        set files [glob *]
        puts "files: $files"
        cd $pwd

        foreach f $files {
            if { [file isdirectory ../../$dir/$f] } {
	set file $dir/$f
	puts "set file $f"
	catch { loadSingle } err
	puts "err:$err"
            }
        }
    }
#****************************************************************************************************************************************************************************************
    proc initColors { } {
        variable colors
        variable numkeys
        pushsel
        sel /vis/cube/shader/color
            .beginkeys 11 3
            for { set i 0 } { $i < $numkeys } { incr i } {
	set r $colors(0,$i)
	set g $colors(1,$i)
	set b $colors(2,$i)
	.setkey3f $i [expr $i / $numkeys] $r $g $b 
            }
            .setkey3f $i 2.0 $r $g $b 
        popsel

    }
#****************************************************************************************************************************************************************************************
    proc initSlices { } {
        variable slices
        variable maxSlices
        for { set i 0 } { $i < $maxSlices } { incr i } {
            set slices($i) true
        }
    }

#****************************************************************************************************************************************************************************************
    proc UpdateSlices { } {
        variable slices
        variable maxSlices
        pushsel
        if { [catch {
            sel /game/vis/cube
        } err] } { return }
        foreach slice [dir] {
            set pieces [split $slice "e"]
            set num [lindex $pieces end]
            if { $slices([expr $num-1]) } {
	$slice.setactive true
            } else {
	$slice.setactive false
            }
        }
    }
#****************************************************************************************************************************************************************************************
    proc update { } {
        variable colors
        variable numkeys
        global ke_num_channels ke_chn_array

        #update keys
        set ke_num_channels 3
        set ke_chn_array(0,name)        red
        set ke_chn_array(0,color)       red
        set ke_chn_array(0,minval)      0
        set ke_chn_array(0,maxval)      1.0
        set ke_chn_array(0,callback)    HullDisplay::callback
        set ke_chn_array(0,numkeys)     $numkeys

        set ke_chn_array(1,name)        green
        set ke_chn_array(1,color)       green
        set ke_chn_array(1,minval)      0
        set ke_chn_array(1,maxval)      1.0
        set ke_chn_array(1,callback)    HullDisplay::callback
        set ke_chn_array(1,numkeys)     $numkeys 

        set ke_chn_array(2,name)        blue
        set ke_chn_array(2,color)       blue
        set ke_chn_array(2,minval)      0
        set ke_chn_array(2,maxval)      1.0
        set ke_chn_array(2,callback)    HullDisplay::callback
        set ke_chn_array(2,numkeys)    $numkeys 

        pushsel
            if { [catch { sel /game/vis/cube } err] } { } else {
	set cubes [dir]
	foreach cube $cubes {
	    for { set i 0 } { $i < $numkeys } { incr i } {
	        set r $colors(0,$i)
	        set g $colors(1,$i)
	        set b $colors(2,$i)
	        $cube/shader/color.setkey3f $i [expr $i / 1.0 / $numkeys] $r $g $b 
	    }
	}
            }
        popsel

        for { set i 0 } { $i < $numkeys } { incr i } {
            for { set j 0 } { $j < 3 } { incr j } {
	set ke_chn_array($j,$i) $colors($j,$i)
            }
        }
        ke_refresh
    }
#****************************************************************************************************************************************************************************************
    proc callback { channel key val } {
        variable colors
            variable numkeys 
        set colors($channel,$key) $val
        update 
    }
#****************************************************************************************************************************************************************************************
    proc saveKeys { } {
        variable colors
        variable numkeys
        set file [::open "scripts/gui/hulldisplay.keys" "w"]
        for { set i 0 } { $i < $numkeys } { incr i } {
            set r $colors(0,$i)
            set g $colors(1,$i)
            set b $colors(2,$i)
            puts $file "$r $g $b"
        }
        close $file
    }
#****************************************************************************************************************************************************************************************
    proc loadKeys { } {
        variable colors
        variable numkeys
        for { set i 0 } { $i < $numkeys} { incr i } {
            for { set j 0 } { $j < 3 } { incr j } {
	set colors($j,$i) [expr $i / 1.0 / ($numkeys-1)]
            }
        }
        if { [catch {set file [::open "scripts/gui/hulldisplay.keys" "r"]} err]} { puts $err } else {
            for { set i 0 } { $i < $numkeys } { incr i } {
	set line [gets $file]
	set colors(0,$i) [lindex $line 0]
	set colors(1,$i) [lindex $line 1]
	set colors(2,$i) [lindex $line 2]
	puts "read colors : $colors(0,$i) $colors(1,$i) $colors(2,$i)"
            }
            close $file
        }
        update
    }
    proc setMinSlice { n } {
        variable slices
        variable minSlice
        variable maxSlice
        for { set i 0 } { $i < 34 } { incr i } {
            if { $i >= $minSlice && $i <= $maxSlice || $i <= $minSlice && $i >= $maxSlice} {
	set slices($i) 1
            } else {
	set slices($i) 0
            }
        }
        UpdateSlices
    }
    proc setMaxSlice { n } { 
        setMinSlice $n
    }
    proc animate { a } { 
        variable animating $a
    }
    proc tick { } {
        variable minSlice
        variable maxSlice
        variable animating
        variable animationSpeed
        variable frameRange
        if { $animating } {
            set minSlice [expr $minSlice+1]
            if { $minSlice >= 34 } { set minSlice 1 } 
            
            set maxSlice [expr $minSlice+$frameRange-1]
            if { $maxSlice >= 34 } { set maxSlice [expr $minSlice+$frameRange-33] }
            setMinSlice 0
        }
        after $animationSpeed HullDisplay::tick
    }
}
