# showCode --
# This procedure creates a toplevel window that displays the commands for
# a node and allows them to be edited and reinvoked.

proc edit_script { } {
    global tk_library
    if ![winfo exists .code] {
	toplevel .code
	frame .code.buttons
	pack .code.buttons -side bottom -fill x
	button .code.buttons.dismiss -text Dismiss \
            -default active -command "destroy .code"
	button .code.buttons.rerun -text "Rerun Demo" -command {
	    eval [.code.text get 1.0 end]
	}
	pack .code.buttons.dismiss .code.buttons.rerun -side left \
	    -expand 1 -pady 2
	frame .code.frame
	pack  .code.frame -expand yes -fill both -padx 1 -pady 1
	text .code.text -height 40 -wrap word\
	    -xscrollcommand ".code.xscroll set" \
	    -yscrollcommand ".code.yscroll set" \
	    -setgrid 1 -highlightthickness 0 -pady 2 -padx 3
	scrollbar .code.xscroll -command ".code.text xview" \
	    -highlightthickness 0 -orient horizontal
	scrollbar .code.yscroll -command ".code.text yview" \
	    -highlightthickness 0 -orient vertical

	grid .code.text -in .code.frame -padx 1 -pady 1 \
	    -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
	grid .code.yscroll -in .code.frame -padx 1 -pady 1 \
	    -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
#	grid .code.xscroll -in .code.frame -padx 1 -pady 1 \
#	    -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
	grid rowconfig    .code.frame 0 -weight 1 -minsize 0
	grid columnconfig .code.frame 0 -weight 1 -minsize 0
    } else {
	wm deiconify .code
	raise .code
    }

#--------------
# find commands:
    set commands [.getcmds]

    wm title .code "Edit Script"
    wm geometry .code 63x4+609+0
    .code.text delete 1.0 end
#    .code.text insert 1.0 
    .code.text mark set insert 1.0
}

#-----------------------------------------------------------------
#edit camera properties
proc edit_camera { } {
	global  target
	global	curmode
	global	viewtype

	set target [/camera.getcarrier]
	set curmode [/camera.getviewmode]
	set viewtype $curmode

	toplevel .cp
	wm title .cp "Camera Properties"
	wm geometry .cp 199x192+0+494
	
	frame	.cp.f2
	pack	.cp.f2 -side top -expand yes -fill both

	frame	.cp.f2.f
	pack	.cp.f2.f -expand yes -fill both -side top -padx 1 -pady 1

	label	.cp.f2.f.curviewmode -text "current mode: $curmode"
	button	.cp.f2.f.freeview	-text "free view" -command { camera_setfree }
	button	.cp.f2.f.chaseview -text "chase view" -command { camera_setchase }
	button	.cp.f2.f.stiffchase -text "stiff chase view" -command { camera_setstiffchase }
	button	.cp.f2.f.freefollow -text "freeview follower" -command { camera_setfreefollow }
	#pack 	.cp.f2.f.curviewmode .cp.f2.f.freeview .cp.f2.f.chaseview .cp.f2.f.stiffchase .cp.f2.f.freefollow -side top  -expand yes -fill x -side top
	pack 	.cp.f2.f.freeview .cp.f2.f.chaseview .cp.f2.f.stiffchase .cp.f2.f.freefollow -side top  -expand yes -fill x -side top

	frame	.cp.f2.f2
	pack	.cp.f2.f2 -expand yes -fill both -side top -padx 1 -pady 1
	label	.cp.f2.f.cardesc	-text "Carrier:"
	entry	.cp.f2.f.carrier	-textvariable target
	pack	.cp.f2.f.cardesc .cp.f2.f.carrier -side top -expand yes -fill x

	frame	.cp.k
	pack	.cp.k  -expand yes -fill both -side top -padx 1 -pady 1
	
	button	.cp.k.ok -text "save" -command { save_cam_config }
	button	.cp.k.cancel -text "close" -command { destroy .cp }
	pack	.cp.k.ok .cp.k.cancel  -expand yes -fill x -side left
}

#-----------------------------------------------------

proc new_level { } {
	global filename
	

	#delete all objects under /usr/world
	puts "\n\n******************\nreleasing old level!"
	puts "deleting objects"
	sel /usr/world/objects
	set objects [dir]
	foreach object $objects {
		delete $object
	}

	sel /usr/world/dynamic
	set objects [dir]
	foreach object $objects {
		delete $object
	}

	#unload all clans
	puts "\nunloading clans:"
	sel /usr/lib/clans
	set clans [dir]
	foreach clan $clans {
		if { [$clan.isloaded] == "true" } {
			puts "- unloading clan \"$clan\""
			$clan.unload
		}
	}
	
	sel /

	puts "\nLevel $filename has been released successfully!\n******************\n"
	
	set filename "Untitled"
	wm title . "Nebula Leveleditor - \"$filename\""
}

proc open_level { } {
	global filename
	set types {
		{"Nebula files"		{.n}	}
		{"Tcl Scripts"		{.tcl}		TEXT}
		{"All files"		*}
   	}
    	
	set file [tk_getOpenFile -filetypes $types]

	
    	if {$file == ""} {
		return
	}

	#release old level!
	new_level

	#load new one and update leveleditor title!
	sel /usr/world/objects
	source $file
	sel /

	#set title
	set filen [split $file .]
	set filename [lindex $filen 0]

	#make sure we do not have any directorys in the name
	set filesplit [split $filename /]
	set num_dirs [llength $filesplit]
	set filename [lindex $filesplit [expr $num_dirs - 1]]

	wm title . "Nebula Leveleditor - \"$filename\""
}

proc save_level { } {
	global filename
	if { $filename == "Untitled" } {
		save_level_as
	} else {
		sel /usr/world/objects
		.saveas levels/$filename
		sel /
	}
}

proc save_level_as { } {
	global filename
	set types {
		{"Nebula files"		{.n}	}
		{"Tcl Scripts"		{.tcl}		TEXT}
		{"All files"		*}
   	}
    	
	set file [tk_getSaveFile -filetypes $types]

	
    	if {$file == ""} {
		return
	}

	#remove the .n from the filename
	set filen [split $file .]
	set filename [lindex $filen 0]

	#make sure we do not have any directorys in the name
	set filesplit [split $filename /]
	set num_dirs [llength $filesplit]
	set filename [lindex $filesplit [expr $num_dirs - 1]]

	sel /usr/world/objects
	.saveas levels/$filename
	sel /
	wm title . "Nebula Leveleditor - \"$filename\""
}

#-----------------------------------------------------

proc  camera_setfree { } {
	global curmode
	/camera.setviewmode "FREE"
	set curmode "FREE"
	.cp.f.curviewmode config -text "current mode: $curmode"
}

proc  camera_setchase { } {
	global curmode
	/camera.setviewmode "CHASE"
	set curmode "CHASE"
	.cp.f.curviewmode config -text "current mode: $curmode"
}
	
proc  camera_setstiffchase { } {
	global curmode
	/camera.setviewmode "CHASE_STIFF"
	set curmode "CHASE_STIFF"
	.cp.f.curviewmode config -text "current mode: $curmode"
}

proc  camera_setfreefollow { } {
	global curmode
	/camera.setviewmode "FREE_FOLLOW"
	set curmode "FREE_FOLLOW"
	.cp.f.curviewmode config -text "current mode: $curmode"
}

proc  save_cam_config { } {
	global target

	/camera.setcarrier $target
	/camera.saveas data:scripts/camera	
}


#------------------------------------------------------------------
# create a new node
proc create_node { } {
	global classname
	global instancename

	set classname "nroot"
	set instancename ""
	
	toplevel .cn
	wm title .cn "Create Node"
	wm geometry .cn 280x103+320+494

	frame .cn.f
	pack .cn.f -side top -expand yes -fill both

	label .cn.f.cnd -text "classname"
	entry .cn.f.cn -textvariable classname
	label .cn.f.ind -text "instancename"
	entry .cn.f.in -textvariable instancename
	pack .cn.f.cnd .cn.f.cn .cn.f.ind .cn.f.in -side top -expand yes -fill x

	button .cn.f.k -text "OK" -command { new $classname $instancename }
	button .cn.f.c -text "Close" -command { destroy .cn }
	pack .cn.f.k .cn.f.c -side left -expand yes -fill x
}

#-------------------------------------------------------------------
# clone seleced
proc clone_sel { } {
	global path
	global act_node
	set name [$act_node.getname]
	set path "/usr/world/$name"

	toplevel .cls
	wm title .cls "Clone selected"
	wm geometry .cls 280x63+319+624

	frame .cls.f
	pack .cls.f -side top -expand yes -fill both

	label .cls.f.pd -text "new name"
	entry .cls.f.p -textvariable path
	pack .cls.f.pd .cls.f.p -side top -expand yes -fill x

	button .cls.f.k -text "Clone" -command { clone $path }
	button .cls.f.c -text "Close" -command { destroy .cls }
	pack .cls.f.k .cls.f.c -side left -expand yes -fill x
}