namespace eval Plan {
    namespace export clear load
    proc log { txt } {
        logto "all plan" $txt
    }
    proc clear { } {
        log "clear"
        pushsel
        sel /game/ents
        foreach type [dir] {
            if { [lsearch "platform asset target threat" $type] == -1 } { continue }
            sel /game/ents/$type
            foreach ent [dir] {
	log "/sys/servers/release.releasenow /game/ents/$type/$ent"
	/sys/servers/release.releasenow /game/ents/$type/$ent
            }
        }
        popsel
    }

    proc load { plan { canCreate no } } {
        log "load $canCreate"
        pushsel
        set side neutral
        set creator unknown
        set threat_radius_coef 1.0
        XML::parse $plan
        while { [XML::next] } {
            global ::XML::type ::XML::tag ::XML::txt
            switch $type {
                open { 
                    switch $tag {
                        bm              -
                        red             -
                        blue            -
                        PLAYERTYPES     	{   set creator $tag }

                        map  		{ }
                        heightmap           	{   set ::Map::heightmap $txt 		}
                        terrain             	{   set ::Map::terrain_texture $txt 	}
	        detail		{   set ::Map::terrain_detail $txt 	}
	        error		{   set ::Map::error $txt 		}
                        lattop              	{   set ::Map::lat_top $txt 		}
                        latbottom           	{   set ::Map::lat_bottom $txt 		}
                        lngleft             	{   set ::Map::lng_left $txt 		}
                        lngright            	{   set ::Map::lng_right $txt 		}
                        mapwidth            	{   set ::Map::map_width $txt 		}
                        mapheight           	{ 
			    set ::Map::map_height $txt 
			    set ::Map::worldsize $txt 
			}
	        camerapos	{ 
			    puts "eval /game/camera.sv pos $txt"
			    after 1000 "eval /game/camera.sv pos $txt"
			}
	        maxalt		{   set ::Map::maxheight $txt		} 
	        skybox		{   set ::Map::skybox $txt; Map::gen_skybox		} 
                        threatradiuscoef    	{   set threat_radius_coef $txt }
                        side            	{   set side $txt	}

	        mission 	-
	        description	-
                        DUMMYTAGS       	{ }

                        threatradiuscoef 	{   set threat_radius_coef $txt }
                        platform        	-
                        target          	-   
                        threat          	-
	        carrier		-
                        ENTTAGS         	{
			    if { $canCreate } {
			        sel [gen_ent $tag $txt]
			    } else {
			        if { [
			            catch { sel /game/ents/$tag/$txt }
			        ] } { 
			            XML::continueUntilClose $tag
			            continue
			        }
			    }
			    .cs class $tag
			    .cs name $txt
			    .cs creator $creator
			    if { $canCreate } {
			        .cref side /game/sides/$side
			    }
			}
                        asset           	{   
			    opt simple { } {
			        set asset 		[gen_ent asset]
			        $asset.cs type $txt
			        $asset/hpattach.attachto 	[.getfullname]
			    }
			}
	        lat             	{
			    .cf $tag $txt
			    .cvz pos 		[expr [Map::lat_to_z $txt] + [randf -10 10]]
			    .cvz startpos 	[Map::lat_to_z $txt]
			}
                        lng             	{
			    .cf $tag $txt
			    .cvx pos 		[expr [Map::lng_to_x $txt] + [randf -10 10]]
			    .cvx startpos 	[Map::lng_to_x $txt]
			}
                        alt             	{   .cvy pos $txt }   
                        INTEPRETEDVARS  	{ }

                        value           	-
                        lockontime      	-
                        hp              	-
	        active		-
                        FLOATVARS       	{   .cf $tag $txt }
                        detectionrange  	-
                        effectiverange  	-
                        RADIUSVARS      	{   .cf $tag [expr $txt * $threat_radius_coef] }
                                        
                        pos             	-
                        startpos        	-
                        VECTORVARS      	{   .cv $tag [expr [lindex $txt 0] + [randf -10 10]] [lindex $txt 1] [expr [lindex $txt 2] + [randf -10 10]] }
                        type            	{ 
			    .cs type $txt
			    .cs armorclass $txt
			}
                        armorclass      	-
                        fires           	-
                        STRINGVARS      	{   .cs $tag $txt }

	        name		{   .setname $txt  }


	        ai            	{   
			    catch {
			        vars/destination.resetcounter
			        vars/target.resetcounter
			        vars/withasset.resetcounter
			    }
			    sel ai
			    ::clear
	        }
                        takeoff         	-
                        land            	{ }
                        move            	{
			    #initvars allows you to update the var right away (since it is stacked)
			    opt simple {
			        [gen_unique nmoveai_simple move].initvars 
			    } {
			        [gen_unique nmoveai move].initvars 
			    }
			    ../.ssv destination [lindex $txt 0] [lindex $txt 1] [lindex $txt 2]
			}
	        movelatlng	{
			    #puts "MOVELATLNG : $txt"
			    [gen_unique nmoveai move].initvars
			    set lat 	[lindex $txt 0]
			    set lng 	[lindex $txt 1]
			    set x	[Map::lng_to_x $lng]
			    set y	[Map::lat_to_z $lat]
			    ../.ssv destination $x 0 $y
			}
                        attack          	{
			    opt simple {
			        [gen_unique nplatformattack_simple attack].initvars
			    } { 
			        [gen_unique nplatformattack attack].initvars
			    }
			    if { [/game/ents/target.find $txt] == "null" } {
			        ../.ssref target $txt 
			    } else {
			        ../.ssref target /game/ents/target/$txt 
			    }
			}
                        withasset       	{
#puts "../.sss withasset $txt"
			    ../.sss withasset $txt
			}
                        AISTUFF         	{ } 
                        default 	{ puts "pl unrecognized: <$tag>$txt" }
                    }
                }
                close { 
                    switch $tag {
                        terrain         	-
                        heightmap       	-
                        bm              	-
                        map             	-
                        lattop          	-
                        latbottom       	-
                        lngleft         	-
                        lngright        	-
                        mapwidth        	-
                        mapheight       	-
                        mapwidth        	-
                        mapheight       	-
                        side            	-
                        threatradiuscoef 	-
	        mission 	-
	        description	-
                        DUMMYTAGS       	{ }

                        platform        	-
                        target          	-   
                        threat          	-
                        ENTTAGS         	{ }
                        asset           	{ }

                        lat             	-
                        lng             	-
                        alt             	-
                        INTEPRETEDVARS  	-

                        type            	-
                        value           	-
                        armorclass      	-
                        lockontime      	-
                        detectionrange  	-
                        effectiverange  	-
                        fires           	-
                        hp              	-
	        active		-
                        pos             	-
                        startpos        	-
                        VARS            	{ }

                        ai              	{ sel .. }
                        takeoff         	-
                        land            	-
                        move            	-
                        movelatlng	-
	        attack          	- 
                        withasset       	-
                        AISTUFF         	{ }

	        name		-
	        MISC		{ }
 
                        default 	{ puts "pl unrecognized: </$tag>$txt" }
                    }
                }
            }
        }
        popsel
    }
    proc from_gamestate { {includeais no} } {
        log "from_gamestate"
        proc save_vars { } {
            sel vars
                set d [dir]
            sel ..
            foreach var $d {
                switch $var {
                    value               	-
                    lockontime          	-
                    detectionrange      	-
                    effectiverange      	-
                    hp                  	-
	    active		-
                    FLOATVARS           	{ XML::wrap [string toupper $var] [.gf $var] }

                    pos                 	-
                    startpos            	-
                    VECTORVARS          	{ XML::wrap [string toupper $var] [.gv $var] }

                    armorclass          	-
                    type                	-
                    STRINGVARS          	{ XML::wrap [string toupper $var] [.gs $var] }
                }
            }
        }
        proc save_tree { path type { customcall "none" } } {
            upvar includeais includeais
            log "save_tree"
            sel $path
                foreach ent [dir] {
                    sel $ent
	        set cont true
	        if { [.gs class] == "threat" } {
	            if {[.gf active] < .5 } {
		set cont false
	            }
	        }
	        if { $cont } {
	            XML::open $type [.getname]
	            save_vars
	            if { $customcall != "none" } {
		eval $customcall
	            }
	            XML::close $type
	        }
                    sel ..
                }
            sel ..
        }
        global ::XML::lines
        XML::clear
        pushsel
        XML::open SIDE blue
            save_tree /game/ents/platform PLATFORM { 
	upvar includeais includeais
	if { $includeais } {
	    set moves 0
	    set targets 0
	    sel ai
	    set ais [dir]
	    puts "ais:$ais num:[llength $ais]"
	    sel ..
	    foreach ai $ais {
	        if { [string match move* $ai] } {
	            set moves [string range $ai 4 end]
	            XML::wrap MOVE [vars/destination/destination$moves.getv]
	            incr moves
	        } elseif {[string match attack* $ai] } {
	            set targets [string range $ai 6 end]
	            XML::open ATTACK [vars/target/target$targets.getref]
		XML::wrap WITHASSET [vars/withasset/withasset$targets.gets]
	            XML::close MOVE
	            incr targets
	        }
	    }
	}
	sel hardpoints
    	    foreach hardpoint [dir] {
    	        if { [$hardpoint.open] == false } {
    	            set asset [$hardpoint.getattached]
    	            set assetType [$asset.gs type]
        	            XML::wrap ASSET $assetType
    	        }
    	    }
	sel ..
            }
        XML::close SIDE
        XML::open SIDE red
            save_tree /game/ents/target TARGET 
            save_tree /game/ents/threat THREAT 
        XML::close SIDE
        popsel
        return $lines
    }
#idea
#goes through the ent tree
#parses out all the right data from the objects belonging to the appropriate side and returns them in a big xml file
    proc buildPlan { side } {
        log "buildPlan"
        proc save_vars { } {
            sel vars
                set d [dir]
            sel ..
            foreach var $d {
                switch $var {
	    active		-
	    FLOATVARS		{ XML::wrap [string toupper $var] [.gf $var] }
                    pos                 	-
                    VECTORVARS          	{ XML::wrap [string toupper $var] [.gv $var] }
	    default		{ }
                }
            }
        }
        proc save_tree { path type { customcall "none" } } {
            upvar side side 
            log "save_tree $path $type $customcall"
            sel $path
                foreach ent [dir] {
                    sel $ent
	    set sideRef [.gref side]
	    if { $sideRef != "none" } {
	        if { [$sideRef.getname] == $side } {
	            XML::open $type [.getname]
	            save_vars
	            if { $customcall != "none" } {
		eval $customcall
	            }
	            XML::close $type
	        } else { }
	    }
                    sel ..
                }
            sel ..
        }
        global ::XML::lines
        XML::clear
        pushsel
        XML::open SIDE $side
        pushsel
        sel /game/ents
        set d [dir]
        foreach type $d {
            save_tree $type $type
        }
        XML::close SIDE
        popsel
#puts $lines
        return $lines
    }
    proc clearPlanSub { side path } {
#puts "clearPlanSub $side $path"
        set toerase "nmoveai nplatformattack"
        pushsel
        sel $path
        foreach node [dir] {
            set class [$node.getclass]
            if { [lsearch $toerase $class] != -1} { 
	delete $node 
            } else {
	if { [$node.isa nent] } {
	    if { [$node.hasvar side] } {
	        set side [[$node.gref side].getname]
	        if { $side != $side } { continue }
	    }
	}
	clearPlanSub $side $node
            }
        }
        popsel
    }
    proc clearPlan { side } {
        log "clearPlan $side"
#puts "clearPlan $side"
        clearPlanSub $side /game/ents 
    }
#load the wps associated with platforms into plans
#how do we do this?
#foreach platform
#foreach wp
    proc fromWPS { } {
        puts "fromWPS"
        global ::XML::lines
        XML::clear
        pushsel
        XML::open SIDE blue
        sel /game/ents/platform
        foreach platform [dir] {
            if { [$platform.find "wp"] == "null" } { continue }
            XML::open PLATFORM $platform
            XML::open AI
            sel $platform/wp
            foreach wp [dir] {
	if { $wp == "attachedwp" } { continue }
	set p [$wp.gv pos]
	if { [$wp.gf typef] > .05 } { set attacking true } else { set attacking false }
	puts "XML::wrap MOVE $p"
	XML::wrap MOVE $p
	if { $attacking } {
	    set attacking [$wp.gref "attack"]
	    set withasset [$wp.gs "withasset"]
	    if { $withasset == ":null:" } { continue }
	    if { $withasset == "null" } { continue }
	    foreach asset $withasset {
	        puts "XML::open ATTACK $attacking"
	            puts "XML::wrap WITHASSET $asset"
	        puts "XML::close ATTACK "
	        XML::open ATTACK $attacking
	            XML::wrap WITHASSET $asset
	        XML::close ATTACK 
	    }
	}
            }
            sel ../../
            XML::close AI
            XML::close PLATFORM
        }
        XML::close SIDE
        popsel
        return $lines
    }
}
#   <PLATFORM>platform1
#       <AI>
#           <TAKEOFF></TAKEOFF>
#           <MOVE>246.000000 5.100000 913.333313</MOVE>
#           <MOVE>246.000000 0.000000 913.333313</MOVE>
#           <MOVE>64.647171 0.100000 310.890381</MOVE>
#           <MOVE>60.223518 0.100000 290.969452</MOVE>
#           <MOVE>62.040878 0.100000 270.644379</MOVE>
#           <MOVE>69.928543 0.100000 251.824280</MOVE>
#           <MOVE>83.145630 0.100000 236.276917</MOVE>
#           <MOVE>100.450645 0.100000 225.462677</MOVE>
#           <MOVE>123.333336 0.100000 273.333344</MOVE>
#           <ATTACK>t1
#               <WITHASSET>gps1</WITHASSET>
#           </ATTACK>
#           <MOVE>123.333336 0.100000 273.333344</MOVE>
#           <MOVE>100.450645 0.100000 225.462677</MOVE>
#           <MOVE>83.145630 0.100000 236.276917</MOVE>
#           <MOVE>69.928543 0.100000 251.824280</MOVE>
#           <MOVE>62.040878 0.100000 270.644379</MOVE>
#           <MOVE>60.223518 0.100000 290.969452</MOVE>
#           <MOVE>64.647171 0.100000 310.890381</MOVE>
#           <MOVE>246.000000 0.000000 913.333313</MOVE>
#       </AI>
#   </PLATFORM>
